@@ -3,7 +3,7 @@
 # Setup the local working environment for the "configure" script
 #   Compiler:   GCC
 #
-# $Revision: 619260 $  // by Denis Vakatov, NCBI (vakatov@ncbi.nlm.nih.gov)
+# $Revision: 671130 $  // by Denis Vakatov, NCBI (vakatov@ncbi.nlm.nih.gov)
 #############################################################################
 
 
@@ -21,7 +21,7 @@ platform="`echo $platform | sed -e 's/SunOS5\./solaris/; s/\(sol.*\)-i386/\1-int
 oplatform=`echo $platform | sed -e 's/solaris11/solaris10/g'`
 
 case "$1" in
-  [1-9].* | [1-9][0-9].* )
+  [5-9] | [1-9].* | [12][0-9] | [1-9][0-9].* )
      # Look for the specified version in various reasonable places
      # (tuned for NCBI's installations).
      if /opt/ncbi/gcc/$1/bin/$CXX -dumpversion >/dev/null 2>&1; then
