@@ -1,4 +1,4 @@
-/*  $Id: blast_usage_report.hpp 628542 2021-04-01 19:34:46Z ivanov $
+/*  $Id: blast_usage_report.hpp 663458 2023-02-21 16:20:47Z boratyng $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -97,7 +97,8 @@ public:
 		eAWS,
 		eELBJobId,
 		eELBBatchNum,
-        eSRA
+        eSRA,
+        eELBVersion
 	};
 
 	CBlastUsageReport();
