@@ -1,4 +1,4 @@
-/*  $Id: blast_args.hpp 631554 2021-05-19 13:52:23Z ivanov $
+/*  $Id: blast_args.hpp 683928 2024-06-10 12:07:11Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -153,8 +153,8 @@ public:
 private:
     CNcbiIstream* m_InputStream;    ///< Application's input stream
     CNcbiOstream* m_OutputStream;   ///< Application's output stream
-    auto_ptr<CDecompressIStream> m_DecompressIStream;
-    auto_ptr<CCompressOStream> m_CompressOStream;
+    unique_ptr<CDecompressIStream> m_DecompressIStream;
+    unique_ptr<CCompressOStream> m_CompressOStream;
 
     /// ASN.1 specification of query sequences when read from a saved search
     /// strategy
@@ -305,7 +305,11 @@ public:
                        bool is_igblast = false, bool suppress_sum_stats = false)
         : m_QueryIsProtein(query_is_protein), m_IsRpsBlast(is_rpsblast),
           m_ShowPercentIdentity(show_perc_identity), m_IsTblastx(is_tblastx),
-          m_IsIgBlast(is_igblast), m_SuppressSumStats(suppress_sum_stats) {}
+          m_IsIgBlast(is_igblast), m_SuppressSumStats(suppress_sum_stats),
+          m_IsBlastn(false){}
+
+    // Only support and used by blastn for now
+    CGenericSearchArgs(EBlastProgramType program);
 
     /** Interface method, \sa IBlastCmdLineArgs::SetArgumentDescriptions */
     virtual void SetArgumentDescriptions(CArgDescriptions& arg_desc);
@@ -320,6 +324,8 @@ private:
     bool m_IsTblastx; /**< true if the search is tblastx */
     bool m_IsIgBlast; /**< true if the search is igblast */
     bool m_SuppressSumStats; /**< true if search is blastn or blastp */
+    bool m_IsBlastn;
+
 };
 
 /** Argument class for collecting filtering options */
@@ -872,7 +878,7 @@ private:
     vector<string> m_SraAccessions;
 
     CNcbiIstream* m_MateInputStream;
-    auto_ptr<CDecompressIStream> m_DecompressIStream;
+    unique_ptr<CDecompressIStream> m_DecompressIStream;
 
     bool m_EnableSraCache;
 };
@@ -1098,8 +1104,8 @@ public:
         if (m_IsIgBlast) {
             m_DfltNumAlignments = m_DfltNumDescriptions = 10;
         } else {
-            m_DfltNumAlignments = align_format::kDfltArgNumAlignments;
-            m_DfltNumDescriptions = align_format::kDfltArgNumDescriptions;
+            m_DfltNumAlignments = static_cast<ncbi::TSeqPos>(align_format::kDfltArgNumAlignments) ;
+            m_DfltNumDescriptions = static_cast<TSeqPos>(align_format::kDfltArgNumDescriptions);
         }        
     };
 
@@ -1258,6 +1264,9 @@ public:
     EOutputFormat GetUnalignedOutputFormat(void) const
     {return m_UnalignedOutputFormat;}
 
+    /// Get a user tag added to each alignment
+    const string& GetUserTag(void) const {return m_UserTag;}
+
 private:
     bool m_TrimReadIds;
     bool m_PrintUnaligned;
@@ -1269,6 +1278,7 @@ private:
     bool m_OnlyStrandSpecific;
     bool m_PrintMdTag;
     EOutputFormat m_UnalignedOutputFormat;
+    string m_UserTag;
 };
 
 /// Argument class to collect multi-threaded arguments
@@ -1277,8 +1287,9 @@ class NCBI_BLASTINPUT_EXPORT CMTArgs : public IBlastCmdLineArgs
 public:
 	enum EMTMode {
 		eNotSupported = -1,
-		eSplitByDB,
-	    eSplitByQueries
+		eSplitAuto,
+	    eSplitByQueries,
+		eSplitByDB
 	};
     /// Default Constructor
     CMTArgs(size_t default_num_threads = CThreadable::kMinNumThreads, EMTMode mt_mode = eNotSupported) :
@@ -1446,6 +1457,10 @@ public:
         return m_MTArgs->GetNumThreads();
     }
 
+    int GetMTMode() const {
+    	return m_MTArgs->GetMTMode();
+    }
+
     /// Get the input stream
     virtual CNcbiIstream& GetInputStream();
 
