@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.hpp 609644 2020-06-04 18:09:24Z fongah2 $
+/*  $Id: cmdline_flags.hpp 673656 2023-10-05 19:25:08Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -296,10 +296,18 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgSaveAllPssms;
 NCBI_BLASTINPUT_EXPORT extern const string kArgGLSubject;
 /// Argument to specify the germline database name for igblast
 NCBI_BLASTINPUT_EXPORT extern const string kArgGLDatabase;
+///Argument to specify the C region gene database
+NCBI_BLASTINPUT_EXPORT extern const string kArgCRegionDatabase;
 /// Argument to specify the germline database chaintype name for igblast
 NCBI_BLASTINPUT_EXPORT extern const string kArgGLChainType;
+/// Argument to specify custom internal data file
+NCBI_BLASTINPUT_EXPORT extern const string kArgCustomInternalData;
+/// Argument to specify d gene frame definition data file
+NCBI_BLASTINPUT_EXPORT extern const string kArgDFrameDefinitionFile;
 /// Argument to specify the number of alignments for germline database
 NCBI_BLASTINPUT_EXPORT extern const string kArgGLNumAlign;
+/// Argument to specify the number of alignments for c gene db
+NCBI_BLASTINPUT_EXPORT extern const string kArgCRegionNumAlign;
 /// Argument to specify the germline origin for igblast
 NCBI_BLASTINPUT_EXPORT extern const string kArgGLOrigin;
 /// Argument to specify the Ig domain system
@@ -423,7 +431,8 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgNegativeTaxIdList;
 NCBI_BLASTINPUT_EXPORT extern const string kArgTaxIdListFile;
 /// Argument to specify file with taxonomy ids for Negative filtering
 NCBI_BLASTINPUT_EXPORT extern const string kArgNegativeTaxIdListFile;
-
+/// Argument to not to resolve TaxId to descendant 
+NCBI_BLASTINPUT_EXPORT extern const string kArgNoTaxIdExpansion;
 
 // Magic-BLAST arguments
 /// Argument to specify whether mapped reads are paired
@@ -471,6 +480,8 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgUnalignedOutput;
 NCBI_BLASTINPUT_EXPORT extern const string kArgUnalignedFormat;
 /// Argument to specify mt mode (split by db or split by queries)
 NCBI_BLASTINPUT_EXPORT extern const string kArgMTMode;
+/// Argument to specify user tag for alignments (magicblast)
+NCBI_BLASTINPUT_EXPORT extern const string kArgUserTag;
 
 END_SCOPE(blast)
 END_NCBI_SCOPE
