@@ -1,4 +1,4 @@
-/* $Id: ncbiconf_xcode.h 606022 2020-04-17 19:10:23Z ucko $
+/* $Id: ncbiconf_xcode.h 667394 2023-05-15 17:19:52Z ucko $
  * By Vlad Lebedev, NCBI (lebedev@ncbi.nlm.nih.gov)
  *
  * Mac OS X - xCode Build
@@ -119,6 +119,9 @@
 /* Define to 1 if you have the <Accelerate/Accelerate.h> header file. */
 #define HAVE_ACCELERATE_ACCELERATE_H 1
 
+/* Define to 1 if you have the `accept4' function. */
+/* #undef HAVE_ACCEPT4 */
+
 /* Define to 1 if you have the `alarm' function. */
 #define HAVE_ALARM 1
 
@@ -222,6 +225,9 @@
 /* Define to 1 if you have the `eventfd' function. */
 /* #undef HAVE_EVENTFD */
 
+/* Define to 1 if you have the `execvpe' function. */
+/* #undef HAVE_EXECVPE */
+
 /* Define to 1 if you have the <fcntl.h> header file. */
 #define HAVE_FCNTL_H 1
 
@@ -252,14 +258,6 @@
 /* Define to 1 if you have the `getaddrinfo' function. */
 #define HAVE_GETADDRINFO 1
 
-/* If you have the `gethostbyaddr_r' function, define to the number of
-   arguments it takes (normally 7 or 8). */
-/* #undef HAVE_GETHOSTBYADDR_R */
-
-/* If you have the `gethostbyname_r' function, define to the number of
-   arguments it takes (normally 5 or 6). */
-/* #undef HAVE_GETHOSTBYNAME_R */
-
 /* Define to 1 if you have the `gethostent_r' function. */
 /* #undef HAVE_GETHOSTENT_R */
 
@@ -296,10 +294,6 @@
 /* Define to 1 if you have the `getrusage' function. */
 #define HAVE_GETRUSAGE 1
 
-/* If you have the `getservbyname_r' function, define to the number of
-   arguments it takes (normally 5 or 6). */
-/* #undef HAVE_GETSERVBYNAME_R */
-
 /* Define to 1 if you have the `gettimeofday' function. */
 #define HAVE_GETTIMEOFDAY 1
 
@@ -455,9 +449,6 @@
 /* Define to 1 if you have the `localtime_r' function. */
 #define HAVE_LOCALTIME_R 1
 
-/* Define to 1 if local LBSM support is available. */
-/* #undef HAVE_LOCAL_LBSM */
-
 /* Define to 1 if you have the `lutimes' function. */
 /* #undef HAVE_LUTIMES */
 
@@ -479,9 +470,6 @@
 /* Define to 1 if the NCBI C toolkit is available. */
 /* #undef HAVE_NCBI_C */
 
-/* Define to 1 if the real version of ncbi_crypt support is available. */
-/* #undef HAVE_NCBI_CRYPT */
-
 /* Define to 1 if you have the <netdb.h> header file. */
 #define HAVE_NETDB_H 1
 
@@ -503,6 +491,9 @@
 /* Define to 1 if you have the <paths.h> header file. */
 #define HAVE_PATHS_H 1
 
+/* Define to 1 if you have the `pipe2' function. */
+/* #undef HAVE_PIPE2 */
+
 /* Define to 1 if you have the `poll' function. */
 #define HAVE_POLL 1
 
@@ -540,6 +531,9 @@
 /* Define to 1 if you have the `readpassphrase' function. */
 #define HAVE_READPASSPHRASE 1
 
+/* Define to 1 if you have the `readv' function. */
+#define HAVE_READV 1
+
 /* Define to 1 if your C compiler supports some variant of the C99 `restrict'
    keyword. */
 #define HAVE_RESTRICT_C 1
@@ -797,10 +791,22 @@
 /* Rename DBLIB symbols in FTDS to avoid name clash with Sybase DBLIB. */
 #define NCBI_FTDS_RENAME_SYBDB 1
 
+/* If you have the `gethostbyaddr_r' function, define to the number of
+   arguments it takes (normally 7 or 8). */
+/* #undef NCBI_HAVE_GETHOSTBYADDR_R */
+
+/* If you have the `gethostbyname_r' function, define to the number of
+   arguments it takes (normally 5 or 6). */
+/* #undef NCBI_HAVE_GETHOSTBYNAME_R */
+
 /* If you have the `getpwuid_r' function, define to the number of arguments it
    takes (normally 4 or 5). */
 #define NCBI_HAVE_GETPWUID_R 5
 
+/* If you have the `getservbyname_r' function, define to the number of
+   arguments it takes (normally 5 or 6). */
+/* #undef NCBI_HAVE_GETSERVBYNAME_R */
+
 /* If you have the `readdir_r' function, define to the number of arguments it
    takes (normally 2 or 3). */
 #define NCBI_HAVE_READDIR_R 3
