@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_STREAM__HPP
 #define CONNECT___NCBI_CONN_STREAM__HPP
 
-/* $Id: ncbi_conn_stream.hpp 606402 2020-04-21 14:50:28Z lavr $
+/* $Id: ncbi_conn_stream.hpp 652076 2022-06-30 14:33:28Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,44 +34,52 @@
  *
  * Classes:
  *   CConn_IOStream 
- *      base class derived from "std::iostream" to perform I/O by means
- *      of underlying CConn_Streambuf (implemented privately in
- *      ncbi_conn_streambuf.[ch]pp).
+ *      Base class derived from "std::iostream" to perform I/O by means of the
+ *      underlying CConn_Streambuf, which is implemented privately in
+ *      ncbi_conn_streambuf.[ch]pp
  *
  *   CConn_SocketStream
- *      I/O stream based on socket connector.
+ *      I/O stream based on a socket connector.
  *
  *   CConn_HttpStream
- *      I/O stream based on HTTP connector (that is, the stream, which
- *      connects to HTTP server and exchanges information using HTTP(S)
+ *      I/O stream based on an HTTP connector (that is, a stream that can
+ *      connect to an HTTP server and exchange information using the HTTP(S)
  *      protocol).
  *
  *   CConn_ServiceStream
- *      I/O stream based on service connector, which is able to exchange
- *      data to/from a named service  that can be found via
- *      dispatcher/load-balancing  daemon and implemented as either
- *      HTTP GCI, standalone server, or NCBID service.
+ *      I/O stream based on a service connector, which is able to exchange data
+ *      to/from a named service that can be found via dispatcher/load-balancing
+ *      daemon, and implemented as either HTTP GCI, standalone server, or NCBID
+ *      service.
  *
  *   CConn_MemoryStream
- *      In-memory stream of data (analogous to strstream).
+ *      In-memory stream of data (analogous to std::str[ing]stream).
  *
  *   CConn_PipeStream
- *      I/O stream based on PIPE connector, which is able to exchange data
+ *      I/O stream based on a PIPE connector, which is able to exchange data
  *      with a spawned child process, specified via a command.
  *
  *   CConn_NamedPipeStream
- *      I/O stream based on NAMEDPIPE connector, which is able to exchange
- *      data with another process.
+ *      I/O stream based on a NAMEDPIPE connector, which is able to exchange
+ *      data with another process (or self) via a pipe.
  *
  *   CConn_FtpStream
- *      I/O stream based on FTP connector, which is able to retrieve files
- *      and file lists from remote FTP servers, and upload files as well.
+ *      I/O stream based on an FTP connector, which is able to retrieve files
+ *      and file lists from remote FTP servers, as well as to upload files.
+ *
+ *   CConn_FTPDownloadStream
+ *      Specialization of CConn_FtpStream for a (single) file download from an
+ *      FTP server.
+ *
+ *   CConn_FTPUploadStream
+ *      Specialization of CConn_FtpStream for a (single) file upload to an FTP
+ *      server
  *
  * API:
  *   NcbiOpenURL()
  *      Given a URL, return CConn_IOStream that is reading from the source.
- *      Supported schemes include: file://, http[s]://, ftp://, and finally
- *      a named NCBI service (null scheme).
+ *      Supported schemes include: file://, http[s]://, ftp://, and finally, a
+ *      named NCBI service (the null scheme).
  *
  */
 
@@ -106,9 +114,11 @@ const size_t kConn_DefaultBufSize = 16384;  ///< I/O buffer size per direction
 ///
 /// Base class, inherited from "std::iostream", does both input and output,
 /// using the specified CONNECTOR.
-/// The "buf_size" parameter designates the size of internal I/O buffers, which
-/// reside in between the stream and an underlying connector (which in turn may
-/// do further buffering, if needed).
+///
+/// The "buf_size" parameter designates the size of the internal I/O buffers,
+/// which reside in between the stream and an underlying connector (which in
+/// turn may do further buffering, if needed).
+///
 /// By default, all input operations are tied to the output ones, which means
 /// that any input attempt first flushes any pending output from the internal
 /// buffers.  The fConn_Untie flag can be used to untie the I/O directions.
@@ -124,7 +134,10 @@ class NCBI_XCONNECT_EXPORT CConn_IOStream : public            CNcbiIostream,
                                             virtual protected CConnIniter
 {
 public:
-    /// The values must be compatible with TCONN_Flags.
+    /// Polling timeout (non-NULL), with 0.0 time in it
+    static const STimeout* kZeroTimeout;
+
+    /// The values below must be compatible with TCONN_Flags.
     enum {
         fConn_Untie           = fCONN_Untie,///< do not flush before reading
         fConn_DelayOpen       = 2,          ///< do not force CONN open in ctor
@@ -133,7 +146,7 @@ public:
     } EConn_Flag;
     typedef unsigned int TConn_Flags;       ///< bitwise OR of EConn_Flag
 
-    /// Create a stream based on a CONN, which is to be closed upon stream dtor
+    /// Create a stream based on CONN, which is to be closed upon stream dtor
     /// but only if "close" parameter is passed as "true".
     ///
     /// @param conn
@@ -187,8 +200,8 @@ public:
         { }
     };
 protected:
-    /// Create a stream based on a CONNECTOR -- only for internal use in
-    /// derived classes.  The CONNECTOR always gets closed by stream dtor.
+    /// Create a stream based on CONNECTOR -- only for internal use in derived
+    /// classes.  The passed CONNECTOR always gets closed by stream dtor.
     ///
     /// @param connector
     ///   CONNECTOR coupled with an error code (if any)
@@ -198,7 +211,7 @@ protected:
     ///   Default size of underlying stream buffer's I/O arena (per direction)
     /// @param flags
     ///   Specifies whether to tie input and output (a tied stream flushes all
-    ///   pending output prior to doing any input) and what to buffer
+    ///   pending output prior to doing any input), and what to buffer
     /// @param ptr
     ///   Specifies data, which will be read from the stream prior to
     ///   extracting from the actual connection
@@ -254,16 +267,15 @@ public:
     /// Flush the stream and fetch the response (w/o extracting any user data).
     /// @return
     ///   eIO_Success if the operation was successful, and some input
-    ///   (including empty) will be available upon read.
-    /// @note
-    ///   Status eIO_Closed does not necessarily mean proper EOF here. 
+    ///   (including none in case of EOF) will be available upon read.
     EIO_Status         Fetch(const STimeout* timeout = kDefaultTimeout);
 
-    /// Return the specified data "data" of size "size" into the underlying
+    /// Push the specified data "data" of size "size" into back the underlying
     /// connection CONN.
     /// If there is any non-empty pending input sequence (internal read buffer)
-    /// it will first be attempted to return to CONN.  That excludes any
-    /// initial read area ("ptr") that might have been specified in the ctor.
+    /// it will first be attempted to return to CONN.  Note that it may include
+    /// any initial read area ("ptr"), which could have been specified in the
+    /// ctor yet still unread, so potentially unwanted data copying can result.
     /// Any status different from eIO_Success means that nothing from "data"
     /// has been pushed back to the connection.
     /// @note
@@ -271,7 +283,8 @@ public:
     ///   the CONN if used with a "size" of 0 ("data" is ignored then).
     /// @sa
     ///   CONN_Pushback
-    EIO_Status         Pushback(const CT_CHAR_TYPE* data, streamsize size);
+    virtual EIO_Status Pushback(const CT_CHAR_TYPE* data, streamsize size)
+    { return x_Pushback(data, size); }
 
     /// Get underlying SOCK, if available (e.g. after Fetch())
     SOCK               GetSOCK(void);
@@ -306,17 +319,39 @@ public:
     ///   CONN, ncbi_connection.h, CONN_GetFlags
     CONN               GetCONN(void) const;
 
+
+    /// Equivalent to CONN_Wait(GetCONN(), event, timeout)
+    /// @param event
+    ///   eIO_Read or eIO_Write
+    /// @param timeout
+    ///   Time to wait for the event (poll if zero time specified, and return
+    ///   immediately).
+    /// @return
+    ///   eIO_Success if the event is available, eIO_Timeout if the time has
+    ///   expired;  other code to signal other error condition.
+    /// @sa
+    ///   CONN_Wait
+    EIO_Status         Wait(EIO_Event event,
+                            const STimeout* timeout = kZeroTimeout);
+
 protected:
-    void x_Destroy(void);
+    EIO_Status x_Pushback(const CT_CHAR_TYPE* data, streamsize size,
+                          bool push = false);
+    void       x_Destroy(void);
+
+protected:
+    // Stream buffer
+    CConn_Streambuf*            m_CSb;
 
 private:
-    CConn_Streambuf*      m_CSb;
+    // Resource storage
+    unique_ptr<CConn_Streambuf> x_CSb;
+    CSocket                     m_Socket;
 
     // Cancellation
     SCONN_Callback        m_CB[4];
-    CSocket               m_Socket;
     CConstIRef<ICanceled> m_Canceled;
-    static EIO_Status x_IsCanceled(CONN conn, TCONN_Callback type, void* data);
+    static EIO_Status  sx_IsCanceled(CONN conn,TCONN_Callback type,void* data);
 
 private:
     // Disable copy constructor and assignment
@@ -431,6 +466,9 @@ public:
     ///   Default I/O timeout
     /// @param buf_size
     ///   Default buffer size
+    /// @param flags
+    ///   Specifies whether to tie input and output (a tied stream flushes all
+    ///   pending output prior to doing any input), and what to buffer
     /// @sa
     ///   CConn_IOStream
     CConn_SocketStream
@@ -438,7 +476,8 @@ public:
      unsigned short  port,                        ///< ... and port number
      unsigned short  max_try,                     ///< number of attempts
      const STimeout* timeout  = kDefaultTimeout,
-     size_t          buf_size = kConn_DefaultBufSize);
+     size_t          buf_size = kConn_DefaultBufSize,
+     TConn_Flags     flags    = 0);
 
     /// Create a direct connection with "host:port" and send an initial "data"
     /// block of the specified "size" first;  the remaining communication can
@@ -447,30 +486,36 @@ public:
     /// @param host
     ///   Host to connect to
     /// @param port
-    ///   ... and port number
+    ///   ... and port number (however, see a note below)
     /// @param data
     ///   Pointer to block of data to send once connection is ready
     /// @param size
     ///   Size of the data block to send (or 0 if to send nothing)
-    /// @param flags
-    ///   Socket flags
+    /// @param flgs
+    ///   Socket flgs (see <connect/ncbi_socket.h>)
     /// @param max_try
     ///   Number of attempts
     /// @param timeout
     ///   Default I/O timeout
     /// @param buf_size
     ///   Default buffer size
+    /// @param flags
+    ///   Specifies whether to tie input and output (a tied stream flushes all
+    ///   pending output prior to doing any input), and what to buffer
+    /// @note As a special case, if "port" is specified as 0, then the "host"
+    ///   parameter is expected to have a "host:port" string as its value.
     /// @sa
     ///   CConn_IOStream
     CConn_SocketStream
-    (const string&   host,                        ///< host to connect to
-     unsigned short  port,                        ///< ... and port number
+    (const string&   host,                        ///< host[:port] to connect
+     unsigned short  port     = 0,                ///< ... and port number
      const void*     data     = 0,                ///< initial data block
      size_t          size     = 0,                ///< size of the data block
-     TSOCK_Flags     flags    = fSOCK_LogDefault, ///< see ncbi_socket.h
+     TSOCK_Flags     flgs     = fSOCK_LogDefault, ///< see ncbi_socket.h
      unsigned short  max_try  = DEF_CONN_MAX_TRY, ///< number of attempts
      const STimeout* timeout  = kDefaultTimeout,
-     size_t          buf_size = kConn_DefaultBufSize);
+     size_t          buf_size = kConn_DefaultBufSize,
+     TConn_Flags     flags    = 0);
 
     /// This variant uses an existing socket "sock" to build a stream upon it.
     /// The caller may retain the ownership of "sock" by passing "if_to_own" as
@@ -484,13 +529,23 @@ public:
     ///
     /// @param sock
     ///   Socket to build the stream on
+    /// @param if_to_own
+    ///   Whether the sock object is owned (managed) by the stream at dtor
+    /// @param timeout
+    ///   Default I/O timeout
+    /// @param buf_size
+    ///   Default buffer size
+    /// @param flags
+    ///   Specifies whether to tie input and output (a tied stream flushes all
+    ///   pending output prior to doing any input), and what to buffer
     /// @sa
     ///   CConn_IOStream, ncbi_socket.h, SOCK_CreateConnectorOnTop
     CConn_SocketStream
     (SOCK            sock,         ///< socket
      EOwnership      if_to_own,    ///< whether stream to own "sock" param
      const STimeout* timeout  = kDefaultTimeout,
-     size_t          buf_size = kConn_DefaultBufSize);
+     size_t          buf_size = kConn_DefaultBufSize,
+     TConn_Flags     flags    = 0);
 
     /// This variant uses an existing CSocket to build a stream up on it.
     /// NOTE:  this always revokes all ownership of the "socket"'s internals
@@ -500,12 +555,20 @@ public:
     ///
     /// @param socket
     ///   Socket to build the stream up on
+    /// @param timeout
+    ///   Default I/O timeout
+    /// @param buf_size
+    ///   Default buffer size
+    /// @param flags
+    ///   Specifies whether to tie input and output (a tied stream flushes all
+    ///   pending output prior to doing any input), and what to buffer
     /// @sa
     ///   CConn_IOStream, ncbi_socket.hpp, SOCK_CreateConnectorOnTop
     CConn_SocketStream
     (CSocket&        socket,       ///< socket, underlying SOCK always grabbed
      const STimeout* timeout  = kDefaultTimeout,
-     size_t          buf_size = kConn_DefaultBufSize);
+     size_t          buf_size = kConn_DefaultBufSize,
+     TConn_Flags     flags    = 0);
 
     /// Create a tunneled socket stream connection.
     ///
@@ -528,38 +591,43 @@ public:
     ///   Pointer to block of data to send once connection is ready
     /// @param size
     ///   Size of the data block to send (or 0 if to send nothing)
-    /// @param flags
-    ///   Socket flags
+    /// @param flgs
+    ///   Socket flags (see <connect/ncbi_socket.h>)
     /// @param timeout
     ///   Default I/O timeout
     /// @param buf_size
     ///   Default buffer size
+    /// @param flags
+    ///   Specifies whether to tie input and output (a tied stream flushes all
+    ///   pending output prior to doing any input), and what to buffer
     /// @sa
     ///   CConn_IOStream, SConnNetInfo
     CConn_SocketStream
     (const SConnNetInfo& net_info,
      const void*         data     = 0,
      size_t              size     = 0,
-     TSOCK_Flags         flags    = fSOCK_LogDefault,
+     TSOCK_Flags         flgs     = fSOCK_LogDefault,
      const STimeout*     timeout  = kDefaultTimeout,
-     size_t              buf_size = kConn_DefaultBufSize);
+     size_t              buf_size = kConn_DefaultBufSize,
+     TConn_Flags         flags    = 0);
 };
 
 
 
 /// Helper class to fetch HTTP status code and text
 struct SHTTP_StatusData {
-    int         m_Code;
-    CTempString m_Text;
-    string      m_Header;
+    int               m_Code;
+    CTempString       m_Text;
+    string            m_Header;
 
-    SHTTP_StatusData(void) : m_Code(0)
+    SHTTP_StatusData(void)
+        : m_Code(0)
     { }
 
     EHTTP_HeaderParse Parse(const char* header);
 
     void              Clear(void)
-    { m_Code = 0; m_Text.clear(); m_Header = kEmptyStr; }
+    { m_Code = 0; m_Text.clear(); m_Header.clear(); }
 
 private:
     // Disable copy constructor and assignment
@@ -568,6 +636,35 @@ private:
 };
 
 
+/// Helper base class for HTTP-like streams
+class NCBI_XCONNECT_EXPORT CConn_HttpStream_Base : public CConn_IOStream
+{
+public:
+    /// Get the last seen HTTP status code, if available
+    int               GetStatusCode(void) const {return m_StatusData.m_Code;  }
+
+    /// Get the last seen HTTP status text, if available
+    const CTempString GetStatusText(void) const {return m_StatusData.m_Text;  }
+
+    /// Get the last seen HTTP header text, if available
+    const string&     GetHTTPHeader(void) const {return m_StatusData.m_Header;}
+
+protected:
+    /// @sa
+    ///   CConn_IOStream
+    CConn_HttpStream_Base
+    (const TConnector& connector,
+     const STimeout*   timeout,
+     size_t            buf_size,
+     TConn_Flags       flags = 0)
+        : CConn_IOStream(connector, timeout, buf_size, flags)
+    { }
+
+protected:
+    // HTTP status code & text seen last
+    SHTTP_StatusData  m_StatusData;
+};
+
 
 /////////////////////////////////////////////////////////////////////////////
 ///
@@ -599,7 +696,7 @@ private:
 ///   CConn_IOStream, HTTP_CreateConnector, ConnNetInfo_Create
 ///
 
-class NCBI_XCONNECT_EXPORT CConn_HttpStream : public CConn_IOStream
+class NCBI_XCONNECT_EXPORT CConn_HttpStream : public CConn_HttpStream_Base
 {
 public:
     CConn_HttpStream
@@ -654,42 +751,30 @@ public:
      size_t              buf_size     = kConn_DefaultBufSize
      );
 
-    ~CConn_HttpStream();
-
-    /// Get the last seen HTTP status code
-    int               GetStatusCode(void) const {return m_StatusData.m_Code;  }
-
-    /// Get the last seen HTTP status text
-    const CTempString GetStatusText(void) const {return m_StatusData.m_Text;  }
-
-    /// Get the entire HTTP header as received
-    const string&     GetHTTPHeader(void) const {return m_StatusData.m_Header;}
+    virtual ~CConn_HttpStream();
 
     /// Set new URL to hit next
-    void              SetURL(const string& url) { m_URL = url; }
+    void                     SetURL(const string& url) { m_URL = url; }
 
 protected:
-    // Chained callbacks
+    // Chained callbacks and data
     void*                    m_UserData;
     FHTTP_Adjust             m_UserAdjust;
     FHTTP_Cleanup            m_UserCleanup;
     FHTTP_ParseHeader        m_UserParseHeader;
 
-    // HTTP status & text seen last
-    SHTTP_StatusData         m_StatusData;
-
     // URL to hit next
     string                   m_URL;
 
 private:
-    // Interceptors
-    static int/*bool*/       x_Adjust     (SConnNetInfo* net_info,
-                                           void*         data,
-                                           unsigned int  count);
-    static void              x_Cleanup    (void*         data);
-    static EHTTP_HeaderParse x_ParseHeader(const char*   header,
-                                           void*         data,
-                                           int           code);
+    // Callback interceptors
+    static int/*bool*/       sx_Adjust     (SConnNetInfo* net_info,
+                                            void*         data,
+                                            unsigned int  count);
+    static void              sx_Cleanup    (void*         data);
+    static EHTTP_HeaderParse sx_ParseHeader(const char*   header,
+                                            void*         data,
+                                            int           code);
 };
 
 
@@ -704,6 +789,9 @@ private:
 /// otherwise created by using the service name as a registry section to obtain
 /// the information from (details: <connect/ncbi_connutil.h>).
 ///
+/// In case when the service is implemented over underlying HTTP (which is NOT
+/// generally guaranteed), the HTTP status data become available.
+///
 /// Provided "timeout" is set at the connection level, and if different from
 /// kDefaultTimeout, it overrides the value supplied by an underlying connector
 /// (the latter value is kept in SConnNetInfo::timeout).
@@ -712,7 +800,7 @@ private:
 ///   SERVICE_CreateConnector
 ///
 
-class NCBI_XCONNECT_EXPORT CConn_ServiceStream : public CConn_IOStream
+class NCBI_XCONNECT_EXPORT CConn_ServiceStream : public CConn_HttpStream_Base
 {
 public:
     CConn_ServiceStream
@@ -731,40 +819,24 @@ public:
      const STimeout*         timeout  = kDefaultTimeout,
      size_t                  buf_size = kConn_DefaultBufSize);
 
-    ~CConn_ServiceStream();
-
-    /// Get the last seen HTTP status code, if available
-    int               GetStatusCode(void) const {return m_CBD.status.m_Code;}
-
-    /// Get the last seen HTTP status text, if available
-    const CTempString GetStatusText(void) const {return m_CBD.status.m_Text;}
-
-    /// Get the last seen HTTP status text, if available
-    const string&     GetHTTPHeader(void) const {return m_CBD.status.m_Header;}
-
-public:
-    /// Helper class
-    struct SSERVICE_CBData {
-        SHTTP_StatusData     status;
-        SSERVICE_Extra       extra;
-    };
+    virtual ~CConn_ServiceStream();
 
 protected:
-    // Chained callbacks
-    SSERVICE_CBData          m_CBD;
+    // Chained callbacks and data
+    SSERVICE_Extra           m_Extra;
 
 private:
-    // Interceptors
-    static void              x_Reset      (void*         data);
-    static int/*bool*/       x_Adjust     (SConnNetInfo* net_info,
-                                           void*         data,
-                                           unsigned int  count);
-    static void              x_Cleanup    (void*         data);
-    static EHTTP_HeaderParse x_ParseHeader(const char*   header,
-                                           void*         data,
-                                           int           code);
-    static const SSERV_Info* x_GetNextInfo(void*         data,
-                                           SERV_ITER     iter);
+    // Callback interceptors
+    static void              sx_Reset      (void*         data);
+    static int/*bool*/       sx_Adjust     (SConnNetInfo* net_info,
+                                            void*         data,
+                                            unsigned int  count);
+    static void              sx_Cleanup    (void*         data);
+    static EHTTP_HeaderParse sx_ParseHeader(const char*   header,
+                                            void*         data,
+                                            int           code);
+    static const SSERV_Info* sx_GetNextInfo(void*         data,
+                                            SERV_ITER     iter);
 };
 
 
@@ -790,7 +862,7 @@ public:
                        size_t      buf_size = kConn_DefaultBufSize);
 
     /// Build a stream on top of an existing data area of a specified size.
-    /// The contents of the area is what will be read first from the stream.
+    /// The contents of the area are what will be read first from the stream.
     /// Writing to the stream will _not_ modify the contents of the area.
     /// When read from the stream, the written data will appear following the
     /// initial data block.
@@ -809,6 +881,9 @@ public:
 
     virtual ~CConn_MemoryStream();
 
+    virtual EIO_Status Pushback(const CT_CHAR_TYPE* data, streamsize size)
+    { return x_Pushback(data, size, true); }
+
     /// The CConnMemoryStream::To* methods allow to obtain unread portion of
     /// the stream into a single container (as a string or a vector) so that
     /// all data is kept in sequential memory locations.
@@ -817,6 +892,10 @@ public:
     void    ToString(string*);      ///< fill in the data, NULL is not accepted
     void    ToVector(vector<char>*);///< fill in the data, NULL is not accepted
 
+    /// Get the underlying BUF handle (it still remains managed by the stream).
+    /// @note Causes the stream to flush()
+    BUF     GetBUF(void);
+
 protected:
     const void* m_Ptr;              ///< pointer to read memory area (if owned)
 };
@@ -927,11 +1006,23 @@ public:
      size_t               buf_size = kConn_DefaultBufSize
      );
 
+    virtual ~CConn_FtpStream();
+
     /// Abort any command in progress, read and discard all input data, and
     /// clear stream error state when successful (eIO_Success returns).
     /// @note
-    ///   The call empties both the stream and the underlying CONN.
+    ///   The call empties out both the stream and the underlying CONN.
     virtual EIO_Status Drain(const STimeout* timeout = kDefaultTimeout);
+
+protected:
+    // Chained callback and data
+    SFTP_Callback m_Cmcb;
+
+private:
+    // Callback interceptor
+    static EIO_Status sx_FtpCallback(void*       data,
+                                     const char* cmd,
+                                     const char* arg);
 };
 
 
@@ -1007,10 +1098,15 @@ protected:
 /// See <connect/ncbi_connutil.h> for supported schemes.
 ///
 /// If "url" looks like an identifier (no scheme provided), then it will be
-/// opened as a named NCBI service (using CConn_ServiceStream).
+/// opened as a named NCBI service (using CConn_ServiceStream);  if the "url"
+/// looks like a string in the form "host:port", a socket to that host and port
+/// will be connected (using CConn_SocketStream);  otherwise, a stream will be
+/// created according to the URL scheme (recognized are: "ftp://", "http://",
+/// "https://", and "file://").
 ///
 /// @warning
-///   Writing to the resultant stream is undefined.
+///   Writing to the resultant stream is undefined, unless it's either a
+///   service or a socket stream.
 ///
 extern NCBI_XCONNECT_EXPORT
 CConn_IOStream* NcbiOpenURL(const string& url,
