@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNUTIL__H
 #define CONNECT___NCBI_CONNUTIL__H
 
-/* $Id: ncbi_connutil.h 629074 2021-04-09 14:31:41Z ivanov $
+/* $Id: ncbi_connutil.h 660675 2022-12-21 21:59:38Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,7 @@
  *       ConnNetInfo_Create()
  *       ConnNetInfo_Clone()
  *       ConnNetInfo_SetPath()
+ *       ConnNetInfo_AddPath()
  *       ConnNetInfo_SetArgs()
  *       ConnNetInfo_SetFrag()
  *       ConnNetInfo_GetArgs()
@@ -96,7 +97,7 @@
 #define CONN_USER_LEN    63
 #define CONN_PASS_LEN    63
 #define CONN_HOST_LEN    255
-#define CONN_PATH_LEN    4095
+#define CONN_PATH_LEN    4095 /* including arguments and a fragment */
 
 
 /** @addtogroup UtilityFunc
@@ -172,12 +173,12 @@ typedef unsigned EBDebugPrintout;
  * NOTE1:      Not every field may be fully utilized throughout the library.
  * NOTE2:      HTTP passwords can be either clear text or Base-64 encoded value
  *             enclosed in square brackets [] (which are not Base-64 charset).
- *             For encoding / decoding, one can use command-line OpenSSL:
+ *             For encoding / decoding, one can use the command-line OpenSSL:
  *             echo [-n] "password|base64value" | openssl enc {-e|-d} -base64
  *             or an online tool (search the Web for "base64 online").
- * NOTE3:      The NCBI_CRED credentials are stored by value and are not
- *             otherwise managed (reallocated / duplicated) by the API: they
- *             are passed "as-is" to the lower levels of the SSL.
+ * NOTE3:      The NCBI_CRED credentials are stored by the handle value and are
+ *             not otherwise managed (reallocated / duplicated) by the API:
+ *             they are passed strictly "as-is" to the lower levels of the SSL.
  */
 typedef struct {  /* NCBI_FAKE_WARNING: ICC */
     char            client_host[CONN_HOST_LEN+1]; /*client hostname('\0'=def)*/
@@ -190,8 +191,10 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
     unsigned        http_version:1;   /* HTTP/1.v (or selected by req_method)*/
     EBDebugPrintout debug_printout:2; /* switch to printout some debug info  */
     unsigned        http_push_auth:1; /* push authorize tags even w/o 401/407*/
-    unsigned        http_proxy_leak:1;/* non-zero when can fallback to direct*/
-    unsigned        reserved:14;      /* MBZ                                 */
+    unsigned        http_proxy_leak:1;/* TRUE when may fallback to direct    */
+    unsigned        http_proxy_skip:1;/* TRUE when *NOT* to read $http_proxy */
+    unsigned        http_proxy_only:1;/* TRUE when proxy is from $http_proxy */
+    unsigned        reserved:12;      /* MBZ                                 */
     char            user[CONN_USER_LEN+1];  /* username (if spec'd or req'd) */
     char            pass[CONN_PASS_LEN+1];  /* password (for non-empty user) */
     char            host[CONN_HOST_LEN+1];  /* host name to connect to       */
@@ -202,6 +205,7 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
     char            http_proxy_user[CONN_USER_LEN+1]; /* HTTP proxy username */
     char            http_proxy_pass[CONN_PASS_LEN+1]; /* HTTP proxy password */
     unsigned short  max_try;          /* max. # of attempts to connect (>= 1)*/
+    unsigned short  unused;           /* MBZ; 8-byte alignment               */
     const STimeout* timeout;          /* ptr to I/O timeout(infinite if NULL)*/
     const char*     http_user_header; /* user header to add to HTTP request  */
     const char*     http_referer;     /* request referrer (when spec'd)      */
@@ -254,6 +258,9 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
 #define REG_CONN_HTTP_PROXY_LEAK    "HTTP_PROXY_LEAK"
 #define DEF_CONN_HTTP_PROXY_LEAK    ""
 
+#define REG_CONN_HTTP_PROXY_SKIP    "HTTP_PROXY_SKIP"
+#define DEF_CONN_HTTP_PROXY_SKIP    ""
+
 #define REG_CONN_HTTP_PUSH_AUTH     "HTTP_PUSH_AUTH"
 #define DEF_CONN_HTTP_PUSH_AUTH     ""
 
@@ -288,35 +295,46 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
 #define DEF_CONN_HTTP_REFERER       0
 
 /* Environment/registry keys that are *not* kept in SConnNetInfo */
-#define REG_CONN_SERVICE_NAME               "SERVICE_NAME"
-#define REG_CONN_LOCAL_ENABLE               "LOCAL_ENABLE"
-#define REG_CONN_LBSMD_DISABLE              "LBSMD_DISABLE"
-#define REG_CONN_LBOS_ENABLE                "LBOS_ENABLE"
-#define REG_CONN_LINKERD_ENABLE             "LINKERD_ENABLE"
-#define REG_CONN_NAMERD_FOR_LINKERD_ENABLE  "NAMERD_FOR_LINKERD_ENABLE"
-#define REG_CONN_NAMERD_ENABLE              "NAMERD_ENABLE"
-#define REG_CONN_DISPD_DISABLE              "DISPD_DISABLE"
+#define REG_CONN_LOCAL_ENABLE       "LOCAL_ENABLE"
+#define REG_CONN_LBSMD_DISABLE      "LBSMD_DISABLE"
+#define REG_CONN_LBDNS_ENABLE       "LBDNS_ENABLE"
+#define REG_CONN_LINKERD_ENABLE     "LINKERD_ENABLE"
+#define REG_CONN_NAMERD_ENABLE      "NAMERD_ENABLE"
+#define REG_CONN_DISPD_DISABLE      "DISPD_DISABLE"
 
 /* Implicit server type (LINKERD/NAMERD) */
-#define REG_CONN_IMPLICIT_SERVER_TYPE       "IMPLICIT_SERVER_TYPE"
+#define REG_CONN_IMPLICIT_SERVER_TYPE  "IMPLICIT_SERVER_TYPE"
+
+/* Substitute (redirected) service name */
+#define REG_CONN_SERVICE_NAME       DEF_CONN_REG_SECTION "_" "SERVICE_NAME"
 
 /* Local service dispatcher */
-#define REG_CONN_LOCAL_SERVICES     "LOCAL_SERVICES"
+#define REG_CONN_LOCAL_SERVICES     DEF_CONN_REG_SECTION "_" "LOCAL_SERVICES"
 #define REG_CONN_LOCAL_SERVER       DEF_CONN_REG_SECTION "_" "LOCAL_SERVER"
 
+/* LBDNS settings */
+#define REG_CONN_LBDNS_DOMAIN       DEF_CONN_REG_SECTION "_" "LBDNS_DOMAIN"
+#define REG_CONN_LBDNS_DEBUG        DEF_CONN_REG_SECTION "_" "LBDNS_DEBUG"
+#define REG_CONN_LBDNS_HOST         DEF_CONN_REG_SECTION "_" "LBDNS_HOST"
+#define REG_CONN_LBDNS_PORT         DEF_CONN_REG_SECTION "_" "LBDNS_PORT"
+
+
 /* Local IP table */
-#define REG_CONN_LOCAL_IPS          "LOCAL_IPS"
-#define REG_CONN_LOCAL_IPS_DISABLE  "NONE"
+#define DEF_CONN_LOCAL_IPS          "LOCAL_IPS"
+#define REG_CONN_LOCAL_IPS          DEF_CONN_REG_SECTION "_" DEF_CONN_LOCAL_IPS
+#define DEF_CONN_LOCAL_IPS_DISABLE  "NONE"
 
 
-/* Lookup "param" in the registry / environment.
- * If "param" does not begin with "CONN_", then "CONN_" gets prepended
- * automatically in all lookups listed below, unless otherwise noted.
+/* Lookup "param" in the registry / environment.  "param" is usually all-caps.
+ * If "param" does not begin with "CONN_", then "CONN_" or "_CONN_" gets 
+ * automatically injected in all lookups listed below, unless otherwise noted.
  * The order of search is the following (the first match causes to return):
- * 1. Environment variable "service_param" (all upper-case;  and if failed,
- *    then "as-is");
- * 2. Registry key "param" in the section "[service]";
- * 3. Environment setting "param" (in all upper-case);
+ * 1. Environment variable "service_CONN_param", where "service" gets dashes
+ *    (if any) replaced with underscores: first looked up converted to all
+ *    uppercase, and if that failed, then looked up with the original case in
+ *    the "service" part;
+ * 2. Registry key "CONN_param" in the section '["service"]';
+ * 3. Environment setting "CONN_param" (in all upper-case);
  * 4. Registry key "param" (with the leading "CONN_" stripped) in the default
  *    section "[CONN]".
  * Steps 1 & 2 skipped for "service" passed as either NULL or empty ("").
@@ -325,19 +343,20 @@ typedef struct {  /* NCBI_FAKE_WARNING: ICC */
  * If the found match's value has enveloping quotes (either single '' or
  * double ""), then they are stripped from the result, which can then become
  * empty.
- * The first "value_size" bytes (including the terminating '\0') of the result
- * get copied to the "value" buffer (which may cause truncation!), and the
- * passed "value" address gets returned.
- * When no match is found, the "value" gets filled with "def_value" (or an
- * empty string), which then gets returned.  Return 0 on out of memory or
- * value truncation.
+ * Up to "value_size" bytes (including the terminating '\0') of the result get
+ * copied into the "value" buffer (which may cause truncation!).  When no match
+ * is found, the "value" gets filled up to "value_size" bytes with "def_value"
+ * (or an empty string).  Return 0 on out of memory (including "value_size"
+ * given as 0) or value truncation;  otherwise, return "value".
+ * @note
+ *  "value" is always '\0'-terminated if "value_size" is greater than 0.
  */
 extern NCBI_XCONNECT_EXPORT const char* ConnNetInfo_GetValue
 (const char* service,
  const char* param,
  char*       value,
  size_t      value_size,
- const char* def_value
+ const char* def_value    /* NB: both NULL and "" have the same effect */
  );
 
 
@@ -349,42 +368,44 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_Boolean
  );
 
 
-/* This function fills out the "*info" structure using
- * registry entries named (see above) in macros REG_CONN_<NAME>:
+/* This function fills out the "*info" structure using registry entries named
+ * (see above) in the REG_CONN_<NAME> macros (defaults are in DEF_CONN_<NAME>):
  *
- *  -- INFO FIELD --  ----- NAME -----  ---------- REMARKS/EXAMPLES ---------
- *  client_host       local host name   assigned automatically
- *  req_method        REQ_METHOD
+ *  -- INFO FIELD --  ----- NAME -----  ----------- REMARKS/EXAMPLES ----------
+ *  client_host       (auto-assigned)   local host name filled in automatically
+ *  req_method        REQ_METHOD        only "GET"/"POST"/"ANY" allowed
  *  user              USER
  *  pass              PASS
  *  host              HOST
  *  port              PORT
  *  path              PATH
- *                    ARGS              combined in "path" now
+ *                    ARGS              combined in "path" now -- use API calls
  *  http_version      HTTP_VERSION      HTTP version override
- *  http_proxy_host   HTTP_PROXY_HOST   if empty http_proxy_port is set 0
+ *  http_proxy_host   HTTP_PROXY_HOST   if empty http_proxy_port is set to 0
  *  http_proxy_port   HTTP_PROXY_PORT   no HTTP proxy if 0
  *  http_proxy_user   HTTP_PROXY_USER
  *  http_proxy_pass   HTTP_PROXY_PASS
  *  http_proxy_leak   HTTP_PROXY_LEAK   1 means to also re-try w/o the proxy
- *  http_push_auth    HTTP_PUSH_AUTH    Send credentials pre-emptively
+ *  http_proxy_skip   HTTP_PROXY_SKIP   1 means to skip $http_proxy environment
+ *  http_proxy_only   (auto-assigned)   1 means proxy loaded from $http_proxy
+ *  http_push_auth    HTTP_PUSH_AUTH    1 if to send credentials pre-emptively
  *  timeout           TIMEOUT           "<sec>.<usec>": "3.00005", "infinite"
- *  max_try           MAX_TRY  
- *  external          EXTERNAL
+ *  max_try           MAX_TRY
+ *  external          EXTERNAL          1 to mark requests as "external"
  *  firewall          FIREWALL
  *  stateless         STATELESS
- *  lb_disable        LB_DISABLE        obsolete
- *  debug_printout    DEBUG_PRINTOUT
+ *  lb_disable        LB_DISABLE        obsolete, disables the "LBSMD" mapper
+ *  debug_printout    DEBUG_PRINTOUT   "SOME"(eqv "TRUE") / "DATA"(eqv "ALL")
  *  http_user_header  HTTP_USER_HEADER  "\r\n" (if missing) is auto-appended
  *  http_referer      HTTP_REFERER      may be assigned automatically
  *  svc               SERVICE_NAME      no search/no value without service
  *
- * A value of the field NAME is first looked up in the environment variable
- * of the form <service>_CONN_<NAME>; then in the current corelib registry, in
- * the section 'service' by using the CONN_<NAME> key; then in the environment
- * variable again, but using the name CONN_<NAME>; and finally in the default
+ * A value of the field NAME is first looked up in the environment variable of
+ * the form <service>_CONN_<NAME>;  then in the current corelib registry, in
+ * the section 'service' by using the CONN_<NAME> key;  then in the environment
+ * variable again, but using the name CONN_<NAME>;  and finally, in the default
  * registry section (DEF_CONN_REG_SECTION), using just <NAME>.  If the service
- * is NULL or empty then the first 2 steps in the above lookup are skipped.
+ * is NULL or empty then the first two steps in the above lookup are skipped.
  *
  * For default values see right above, within macros DEF_CONN_<NAME>.
  *
@@ -403,42 +424,83 @@ extern NCBI_XCONNECT_EXPORT SConnNetInfo* ConnNetInfo_Clone
  );
 
 
-/* Convenience routines to manipulate URL path, arguments and fragment that are
- * now all combined within SConnNetInfo::path.
- * All routines below assume that "arg" is either a single arg name or an
- * "arg=val" pair (a fragment part, separated by '#', if any, is ignored).
- * In the former case, an additional "val" may be supplied separately (and
- * will be prepended by the '=').  In the latter case, also having a non-zero
- * string the in the "val" argument may result in an incorrect behavior.
- * The ampersand ('&') gets automatically managed to keep the arg list proper.
+/* Convenience routines to manipulate URL path, arguments, and fragment, all of
+ * which are now stored combined in SConnNetInfo::path.
+ *
  * @warning
- *   Arguments provided in the path element modification routines may not point
- *   to the path element currently stored in the SConnNetInfo being modified.
- * Return value (if any):  non-zero on success; 0 on error.
+ * Arguments provided to the SConnNetInfo::path manipulation routines below
+ * may not point to anywhere inside the path stored within the SConnNetInfo
+ * structure, which is being modified.
+ *
+ * All argument modification routines below assume that "arg" is either a
+ * single arg name or an "arg=val" pair (a fragment part, separated by '#', if
+ * any, is ignored).
+ * In the former case, an additional "val" may be supplied separately (and will
+ * be prepended with '=').  In the latter case, also having a non-zero string
+ * in the "val" argument may result in an incorrect behavior.  The ampersand
+ * ('&') gets automatically managed to keep the arg list consistent.
+ *
+ * Return values (if non-void):  non-zero(true) on success; 0(false) on error.
  */
 
 /* Set the path part in the path element.
- * New path can contain either or both the argument part (separated by '?') and
- * the fragment part (separated by '#'), in this order.  The new path will
- * replace the existing one up to the last specified part, preserving the
- * remainder.  Thus, "" causes only the path part to be removed (preserving any
- * existing argument(s) and fragment).  NULL clears the entire path element. */
+ * New path may also contain either or both the argument part (separated by
+ * '?'), and the fragment part (separated by '#'), in this order.  The new path
+ * will replace the existing one up to and including the last specified part,
+ * preserving the remainder.  Thus, "" causes only the path part to be removed
+ * (leaving any existing argument(s) and fragment alone).  However, "#" will
+ * cause to clear the entire path (single trailing '#' is never stored, but is
+ * always considered).  Also, NULL clears the entire path element. */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_SetPath
 (SConnNetInfo*       net_info,
  const char*         path);
 
+/* Add the path part to the path element.
+ * New path may also contain either or both the argument part (separated by
+ * '?'), and the fragment part (separated by '#'), in this order.  The new path
+ * will extend the existing one up to and including the last specified part,
+ * preserving the remainder.  If the new path begins with a slash ('/'), the
+ * extension starts at the end of the existing path, making sure the slash is
+ * not doubled;  if the new path does not begin with a slash, the extension
+ * starts after the last slash found in the existing path, or at the very
+ * beginning, if there was no slash in the old path.  Note that NULL or ""
+ * cause no effect.
+ *
+ * Corner cases:  if the new "path" begins with the arguments part ('?'), then
+ * the path part will be retained but the arguments part (and possibly the
+ * fragment, too, if specified) will be replaced.  If the new "path" begins
+ * with '#', then only the fragment part is replaced ("#" causes the fragment
+ * to be dropped, and the single trailing '#' will not be stored).
+ *
+ * Example: "/path1/path2?args1&args2#" extends the existing path part with
+ * appending two more levels "/path1/path2" to it, replaces all existing
+ * arguments with "args1&args2", and drops any exising fragment (not storing
+ * the trailing '#').  If the trailing '#' wasn't specified, the old fragment
+ * (if any) would have been kept intact.  "/path1/path2#frag" would work
+ * similarly with path, but drop all existing args, and replace the existing
+ * fragment.
+ *
+ * Note that incoming "path" is assumed to be minimally syntactically correct,
+ * so passing "/path1/path2?#frag" is allowed but will create the path element
+ * with a lone '?' character, just as given.
+ */
+extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_AddPath
+(SConnNetInfo*       net_info,
+ const char*         path);
+
 /* Set the args part (which may contain or just be a #frag part) in the path
  * element, adding the '?' separator automatically, if needed.  If the fragment
  * is missing from the new args, the existing one will be preserved;  and if
  * no arguments are provided before the new fragment, that part of the exising
  * arguments will not get modified.  Thus, "" causes all arguments but the
- * fragment to be removed.  NULL clears all existing path args and frag. */
+ * fragment to be removed.  NULL clears all existing args and frag. */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_SetArgs
 (SConnNetInfo*       net_info,
  const char*         args);
 
-/* Set fragment part only; delete if frag=="".  The passed string may start
- * with '#'; otherwise, the '#' separator will be added to the path element. */
+/* Set fragment part only;  delete if "frag" is NULL, "", or "#".  The passed
+ * string may start with '#';  otherwise, the '#' separator will be added to
+ * the path element. */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_SetFrag
 (SConnNetInfo*       net_info,
  const char*         frag);
@@ -448,14 +510,14 @@ extern NCBI_XCONNECT_EXPORT const char* ConnNetInfo_GetArgs
 (const SConnNetInfo* net_info
  );
 
-/* Append an argument to the end of the list, preserving any #frag. */
+/* Append an argument to the end of the list, preserving any #frag if there. */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_AppendArg
 (SConnNetInfo*       net_info,
  const char*         arg,
  const char*         val
  );
 
-/* Insert an argument at the front of the list. */
+/* Insert an argument at the front of the list, preserving any #frag part. */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PrependArg
 (SConnNetInfo*       net_info,
  const char*         arg,
@@ -465,14 +527,15 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PrependArg
 /* Delete an argument from the list of arguments in the path element.  In case
  * the passed arg is specified as "arg=val&arg2...", the value as well as any
  * successive arguments are ignored in the search for the matching argument.
- * Return zero if no arg was found;  non-zero if it was found and deleted. */
+ * Return zero if no arg was found;  non-zero if it was found and deleted.
+ * Any existing #frag part is not modified. */
 extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_DeleteArg
 (SConnNetInfo*       net_info,
  const char*         arg
  );
 
 /* Delete all arguments specified in "args" (regardless of their values) from
- * the path element. */
+ * the path element.  Any existing #frag part if not modified. */
 extern NCBI_XCONNECT_EXPORT void        ConnNetInfo_DeleteAllArgs
 (SConnNetInfo*       net_info,
  const char*         args
@@ -494,7 +557,7 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_PostOverrideArg
 
 
 /* Set up standard arguments:  service(as passed), address, and platform.
- * Also, set up the User-Agent: HTTP header (using CORE_GetAppName()) in
+ * Also, set up the "User-Agent:" HTTP header (using CORE_GetAppName()) in
  * SConnNetInfo::http_user_header.  Return non-zero on success; zero on error.
  * @note
  *  "service" passed as NULL does not add any service argument, yet "service"
@@ -511,8 +574,8 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_SetupStandardArgs
  );
 
 
-/* Set user header (discard previously set header, if any).  Remove the old
- * header (if any) only if "header" is NULL or empty ("" or all blanks).
+/* Set new user header (discard previously set header, if any).  Remove the old
+ * header (if any) only, if "header" is NULL or empty ("" or all blanks).
  * @warning
  *   New "header" may not be any part of the header kept in the structure.
  * Return non-zero if successful;  otherwise, return 0 to indicate an error.
@@ -523,7 +586,7 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_SetUserHeader
  );
 
 
-/* Append user header (same as ConnNetInfo_SetUserHeader() if no previous
+/* Append to user header (same as ConnNetInfo_SetUserHeader() if no previous
  * header was set);  do nothing if the provided "header" is NULL or empty.
  * @warning
  *   New "header" may not be any part of the header kept in the structure.
@@ -537,10 +600,11 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_AppendUserHeader
 
 /* Override user header.
  * Tags replaced (case-insensitively), and tags with empty values effectively
- * delete existing tags from the old user header, e.g. "My-Tag:\r\n" deletes
- * all appearences (if any) of "My-Tag: [<value>]" from the user header.
+ * delete all existing occurrences of the matching tags from the old user
+ * header, e.g. "My-Tag:\r\n" deletes all appearences of "My-Tag: [<value>]"
+ * from the user header, regardless of the "<value>" (if any).
  * Unmatched tags with non-empty values are simply added to the existing user
- * header (as with "Append" above).  Noop if "header" is an empty string ("").
+ * header (as with "Append" above).  No-op if "header" is an empty string ("").
  * @note
  *   New "header" may be part of the current header from the structure.
  * Return non-zero if successful, otherwise return 0 to indicate an error.
@@ -569,8 +633,8 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_ExtendUserHeader
  );
 
 
-/* Delete entries from current user header, if their tags match those
- * passed in "header" (regardless of the values, if any, in the latter).
+/* Delete entries from current user header, if their tags match those passed in
+ * "header" (regardless of the values, if any).
  * @note
  *   New "header" may be part of the current header from the structure.
  */
@@ -580,9 +644,8 @@ extern NCBI_XCONNECT_EXPORT void        ConnNetInfo_DeleteUserHeader
  );
 
 
-/* Set the timeout.  Accepted values can include a valid pointer
- * (to a finite timeout) or kInfiniteTimeout (or 0) to denote
- * the infinite timeout value.
+/* Set the timeout.  Accepted values can include a valid pointer (to a finite
+ * timeout) or kInfiniteTimeout (or 0) to denote the infinite timeout value.
  * Note that kDefaultTimeout as a pointer value is not accepted.
  * Return non-zero (TRUE) on success, or zero (FALSE) on error.
  */
@@ -603,10 +666,9 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_ParseURL
  );
 
 
-/* Reconstruct text URL out of the SConnNetInfo's components
- * (excluding username:password for safety reasons).
- * Returned string must be free()'d when no longer necessary.
- * Return NULL on error.
+/* Reconstruct text URL out of the SConnNetInfo's components (excluding
+ * username:password for safety reasons).  Return NULL on error.
+ * Returned non-NULL string must be free()'d when no longer needed.
  */
 extern NCBI_XCONNECT_EXPORT char*       ConnNetInfo_URL
 (const SConnNetInfo* net_info
@@ -622,7 +684,7 @@ extern NCBI_XCONNECT_EXPORT void        ConnNetInfo_Log
  );
 
 
-/* Destroy and deallocate "net_info" (if not NULL).
+/* Destroy and deallocate "net_info" (if not NULL;  otherwise, do nothing).
  */
 extern NCBI_XCONNECT_EXPORT void        ConnNetInfo_Destroy
 (SConnNetInfo*       net_info
@@ -636,7 +698,7 @@ extern NCBI_XCONNECT_EXPORT void        ConnNetInfo_Destroy
  * @sa
  *  HTTP_CreateConnector, CConn_HttpStream
  * 
- * Hit URL "http[s]://host[:port]/path[?args]" with the following request
+ * Hit a URL "http[s]://host[:port]/path[?args]" with the following request
  * (argument substitution is shown as enclosed in angle brackets (not present
  * in the actual request), and all optional parts shown in square brackets):
  *
@@ -648,28 +710,29 @@ extern NCBI_XCONNECT_EXPORT void        ConnNetInfo_Destroy
  * well-known standard value depending on the "fSOCK_Secure" bit in the "flags"
  * parameter, when connecting to an HTTP server.
  *
- * A protocol version "1.x" is selected by the "req_method"'s value, and
- * can be either 1.0 or 1.1.  METHOD can be any of those of EReqMethod.
+ * A protocol version "1.x" is selected by the "req_method"'s value, and can be
+ * either 1.0 or 1.1.  METHOD can be any of those of EReqMethod.
  *
- * @note that unlike the deprecated URL_Connect(), this call never encodes any
- * "args", and never auto-inserts any "Host:" tag into the headers (unless
+ * @note that unlike the now-deprecated URL_Connect(), this call never encodes
+ * any "args", and never auto-inserts any "Host:" tag into the headers (unless
  * provided by the "user_header" argument).
  *
  * Request method "eReqMethod_Any/11" selects an appropriate method depending
  * on the value of "content_length":  results in GET when no content is
  * expected ("content_length"==0), and POST when "content_length" is non-zero.
  *
- * The "content_length" parameter must specify the exact(!) amount of data
- * that is going to be sent (0 if none) to HTTP server.  The "Content-Length"
- * header with the specified value gets added to all legal requests
- * (but GET / HEAD / CONNECT).  Special case of (-1L) suppresses the header.
+ * The "content_length" parameter must specify the exact(!) amount of data that
+ * is going to be sent (0 if none) to HTTP server.  The "Content-Length" header
+ * with the specified value gets added to all legal requests (but GET / HEAD /
+ * CONNECT).  Special case of (-1L) suppresses the header.
  * Note that if "user_header" contains a "Transfer-Encoding:" header tag, then
- * "content_length" must be passed as (-1L) (RFC7230, 3.3.2).
+ * "content_length" *must* be passed as (-1L) (RFC7230, 3.3.2).
  *
  * Alternatively, "content_length" can specify the amount of initial data to be
  * sent with a CONNECT request into the established tunnel, and held by the
  * "args" parameter (although, no header tag will get added to the HTTP header
- * in that case, see below).
+ * in that case, see below).  Note, however, that RFC7231, 4.3.6 warns that the
+ * behavior of sending a payload with CONNECT is not well-defined.
  *
  * If the string "user_header" is not NULL/empty, it will be stripped off any
  * surrounding white space (including "\r\n"), then get added to the HTTP
@@ -824,6 +887,7 @@ extern NCBI_XCONNECT_EXPORT EIO_Status BUF_StripToPattern
  * bytes.
  * Assign "*src_read" to the # of bytes successfully encoded from "src_buf".
  * Assign "*dst_written" to the # of bytes written to buffer "dst_buf".
+ * @note "dst_buf" is not getting '\0'-terminated.
  */
 extern NCBI_XCONNECT_EXPORT void URL_Encode
 (const void* src_buf,      /* [in]     non-NULL  */
@@ -840,6 +904,7 @@ extern NCBI_XCONNECT_EXPORT void URL_Encode
  * The extra allowed symbols are passed in a '\0'-terminated string
  * "allow_symbols" (it can be NULL or empty -- then this will be an exact
  * equivalent of URL_Encode).
+ * @note "dst_buf" is not getting '\0'-terminated.
  */
 extern NCBI_XCONNECT_EXPORT void URL_EncodeEx
 (const void* src_buf,      /* [in]     non-NULL  */
