@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_HTTP_SESSION__HPP
 #define CONNECT___NCBI_HTTP_SESSION__HPP
 
-/* $Id: ncbi_http_session.hpp 612999 2020-07-30 19:22:55Z sadyrovr $
+/* $Id: ncbi_http_session.hpp 658917 2022-11-10 19:54:23Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -47,11 +47,6 @@
 BEGIN_NCBI_SCOPE
 
 
-// Forward declarations for shortcut functions.
-class CHttpResponse;
-class CHttpHeaders;
-
-
 // Default retries value.
 struct SGetHttpDefaultRetries
 {
@@ -62,68 +57,6 @@ struct SGetHttpDefaultRetries
 typedef CNullable<unsigned short, SGetHttpDefaultRetries> THttpRetries;
 
 
-/// Shortcut for GET request. Each request uses a separate session,
-/// no data like cookies is shared between multiple requests.
-/// @sa CHttpSession::Get()
-NCBI_XCONNECT_EXPORT
-CHttpResponse g_HttpGet(const CUrl&     url,
-                        const CTimeout& timeout = CTimeout(CTimeout::eDefault),
-                        THttpRetries    retries = null);
-
-/// Shortcut for GET request with custom headers. Each request uses a separate
-/// session, no data like cookies is shared between multiple requests.
-/// @sa CHttpSession::Get()
-NCBI_XCONNECT_EXPORT
-CHttpResponse g_HttpGet(const CUrl&         url,
-                        const CHttpHeaders& headers,
-                        const CTimeout&     timeout = CTimeout(CTimeout::eDefault),
-                        THttpRetries        retries = null);
-
-/// Shortcut for POST request. Each request uses a separate session,
-/// no data like cookies is shared between multiple requests.
-/// @sa CHttpSession::Post()
-NCBI_XCONNECT_EXPORT
-CHttpResponse g_HttpPost(const CUrl&     url,
-                         CTempString     data,
-                         CTempString     content_type = CTempString(),
-                         const CTimeout& timeout = CTimeout(CTimeout::eDefault),
-                         THttpRetries    retries = null);
-
-/// Shortcut for POST request with custom headers. Each request uses a separate
-/// session, no data like cookies is shared between multiple requests.
-/// @sa CHttpSession::Post()
-NCBI_XCONNECT_EXPORT
-CHttpResponse g_HttpPost(const CUrl&         url,
-                         const CHttpHeaders& headers,
-                         CTempString         data,
-                         CTempString         content_type = CTempString(),
-                         const CTimeout&     timeout = CTimeout(CTimeout::eDefault),
-                         THttpRetries        retries = null);
-
-/// Shortcut for PUT request. Each request uses a separate session,
-/// no data like cookies is shared between multiple requests.
-/// @sa CHttpSession::Put()
-NCBI_XCONNECT_EXPORT
-CHttpResponse g_HttpPut(const CUrl&     url,
-                        CTempString     data,
-                        CTempString     content_type = CTempString(),
-                        const CTimeout& timeout = CTimeout(CTimeout::eDefault),
-                        THttpRetries    retries = null);
-
-/// Shortcut for PUT request with custom headers. Each request uses a separate
-/// session, no data like cookies is shared between multiple requests.
-/// @sa CHttpSession::Put()
-NCBI_XCONNECT_EXPORT
-CHttpResponse g_HttpPut(const CUrl&         url,
-                        const CHttpHeaders& headers,
-                        CTempString         data,
-                        CTempString         content_type = CTempString(),
-                        const CTimeout&     timeout = CTimeout(CTimeout::eDefault),
-                        THttpRetries        retries = null);
-
-class CHttpSession_Base;
-
-
 class NCBI_XCONNECT_EXPORT CHttpHeaders : public CObject
 {
 public:
@@ -215,8 +148,8 @@ public:
     virtual ~CHttpHeaders(void) {}
 
 private:
-    friend class CHttpResponse;
     friend class CHttpRequest;
+    friend class CHttpResponse;
 
     // Prohibit copying headers.
     CHttpHeaders(const CHttpHeaders&);
@@ -229,7 +162,7 @@ private:
     // be set safely.
     bool x_IsReservedHeader(CTempString name) const;
 
-    THeaders            m_Headers;
+    THeaders m_Headers;
 
 public:
     /// Parse headers from the string (usually provided by a stream callback).
@@ -247,6 +180,159 @@ public:
 };
 
 
+/// Per-request proxy settings.
+class NCBI_XCONNECT_EXPORT CHttpProxy
+{
+public:
+    CHttpProxy(void) : m_Port(0) {}
+    CHttpProxy(string host, unsigned short port)
+        : m_Host(move(host)), m_Port(port) {}
+    CHttpProxy(string host, unsigned short port, string user, string password)
+        : m_Host(move(host)), m_Port(port), m_User(move(user)), m_Password(move(password)) {}
+
+    bool IsEmpty(void) const { return m_Host.empty(); }
+    const string& GetHost(void) const { return m_Host; }
+    unsigned short GetPort(void) const { return m_Port; }
+    const string& GetUser(void) const { return m_User; }
+    const string& GetPassword(void) const { return m_Password; }
+
+private:
+    string         m_Host;
+    unsigned short m_Port;
+    string         m_User;
+    string         m_Password;
+};
+
+
+class CHttpResponse;
+class CTlsCertCredentials;
+
+
+/// CHttpSession and CHttpRequest parameters.
+class NCBI_XCONNECT_EXPORT CHttpParam
+{
+public:
+    CHttpParam(void);
+
+    /// Add all HTTP headers to request.
+    CHttpParam& SetHeaders(const CHttpHeaders& headers);
+    /// Set or replace a single HTTP header, @sa CHttpHeaders::SetValue().
+    CHttpParam& SetHeader(CHttpHeaders::EHeaderName header, CTempString value);
+    /// Add a single HTTP header, @sa CHttpHeaders::AddValue().
+    CHttpParam& AddHeader(CHttpHeaders::EHeaderName header, CTempString value);
+    const CHttpHeaders& GetHeaders(void) const { return *m_Headers; }
+
+    CHttpParam& SetTimeout(const CTimeout& timeout);
+    const CTimeout& GetTimeout(void) const { return m_Timeout; }
+
+    CHttpParam& SetRetries(THttpRetries retries);
+    THttpRetries GetRetries(void) const { return m_Retries; }
+
+    CHttpParam& SetCredentials(shared_ptr<CTlsCertCredentials> credentials);
+    shared_ptr<CTlsCertCredentials> GetCredentials(void) const { return m_Credentials; }
+
+    CHttpParam& SetProxy(const CHttpProxy& proxy) { m_Proxy = proxy; return *this; }
+    const CHttpProxy& GetProxy(void) const { return m_Proxy; }
+
+    const CTimeout& GetDeadline() const { return m_Deadline; }
+    CHttpParam& SetDeadline(const CTimeout& deadline) { m_Deadline = deadline; return *this; }
+
+    ESwitch GetRetryProcessing() const { return m_RetryProcessing; }
+    CHttpParam& SetRetryProcessing(ESwitch on_off) { m_RetryProcessing = on_off; return *this; }
+
+private:
+    CRef<CHttpHeaders>              m_Headers;
+    CTimeout                        m_Timeout;
+    THttpRetries                    m_Retries;
+    shared_ptr<CTlsCertCredentials> m_Credentials;
+    CHttpProxy                      m_Proxy;
+    CTimeout                        m_Deadline;
+    ESwitch                         m_RetryProcessing;
+};
+
+
+/// Shortcut for GET request. Each request uses a separate session,
+/// no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Get()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpGet(const CUrl& url, const CHttpParam& param);
+
+/// Shortcut for GET request. Each request uses a separate session,
+/// no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Get()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpGet(const CUrl&     url,
+                        const CTimeout& timeout = CTimeout(CTimeout::eDefault),
+                        THttpRetries    retries = null);
+
+/// Shortcut for GET request with custom headers. Each request uses a separate
+/// session, no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Get()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpGet(const CUrl&         url,
+                        const CHttpHeaders& headers,
+                        const CTimeout&     timeout = CTimeout(CTimeout::eDefault),
+                        THttpRetries        retries = null);
+
+/// Shortcut for POST request. Each request uses a separate session,
+/// no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Post()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpPost(const CUrl&       url,
+                         CTempString       data,
+                         const CHttpParam& param = CHttpParam());
+
+/// Shortcut for POST request. Each request uses a separate session,
+/// no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Post()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpPost(const CUrl&     url,
+                         CTempString     data,
+                         CTempString     content_type,
+                         const CTimeout& timeout = CTimeout(CTimeout::eDefault),
+                         THttpRetries    retries = null);
+
+/// Shortcut for POST request with custom headers. Each request uses a separate
+/// session, no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Post()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpPost(const CUrl&         url,
+                         const CHttpHeaders& headers,
+                         CTempString         data,
+                         CTempString         content_type = CTempString(),
+                         const CTimeout&     timeout = CTimeout(CTimeout::eDefault),
+                         THttpRetries        retries = null);
+
+/// Shortcut for PUT request. Each request uses a separate session,
+/// no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Put()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpPut(const CUrl&       url,
+                        CTempString       data,
+                        const CHttpParam& param = CHttpParam());
+
+/// Shortcut for PUT request. Each request uses a separate session,
+/// no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Put()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpPut(const CUrl&     url,
+                        CTempString     data,
+                        CTempString     content_type,
+                        const CTimeout& timeout = CTimeout(CTimeout::eDefault),
+                        THttpRetries    retries = null);
+
+/// Shortcut for PUT request with custom headers. Each request uses a separate
+/// session, no data like cookies is shared between multiple requests.
+/// @sa CHttpSession::Put()
+NCBI_XCONNECT_EXPORT
+CHttpResponse g_HttpPut(const CUrl&         url,
+                        const CHttpHeaders& headers,
+                        CTempString         data,
+                        CTempString         content_type = CTempString(),
+                        const CTimeout&     timeout = CTimeout(CTimeout::eDefault),
+                        THttpRetries        retries = null);
+
+
 /// Interface for custom form data providers.
 class NCBI_XCONNECT_EXPORT CFormDataProvider_Base : public CObject
 {
@@ -380,6 +466,10 @@ public:
 };
 
 
+class CHttpRequest;
+class CHttpSession_Base;
+
+
 /// HTTP response
 class NCBI_XCONNECT_EXPORT CHttpResponse : public CObject
 {
@@ -435,6 +525,30 @@ private:
 };
 
 
+/// Wrapper class for NCBI_CRED
+class NCBI_XCONNECT_EXPORT CTlsCertCredentials : virtual protected CConnIniter
+{
+public:
+    /// Initialize credentials. The arguments are passed to NcbiCreateTlsCertCredentials.
+    /// @sa
+    ///  NcbiCreateTlsCertCredentials
+    CTlsCertCredentials(const CTempStringEx& cert, const CTempStringEx& pkey);
+    ~CTlsCertCredentials(void);
+
+    NCBI_CRED GetNcbiCred(void) const;
+    const string& GetCert(void) const { return m_Cert; }
+    const string& GetPKey(void) const { return m_PKey; }
+
+private:
+    CTlsCertCredentials(const CTlsCertCredentials&);
+    CTlsCertCredentials& operator=(const CTlsCertCredentials&);
+
+    string m_Cert;
+    string m_PKey;
+    mutable NCBI_CRED m_Cred = nullptr;
+};
+
+
 /// HTTP request
 class NCBI_XCONNECT_EXPORT CHttpRequest
 {
@@ -479,8 +593,8 @@ public:
     void SetRetries(THttpRetries retries) { m_Retries = retries; }
 
     /// Get current deadline for Execute().
-    /// For now, effective only for if waiting for actual response
-    /// (as opposed to a recognized 'retry later' response), infinite by default.
+    /// For now, effective only if waiting for actual response (as opposed to
+    /// a recognized 'retry later' response), infinite by default.
     /// @sa Execute() GetRetryProcessing()
     const CTimeout& GetDeadline() const { return m_Deadline; }
     /// Set new deadline for Execute().
@@ -506,12 +620,20 @@ public:
         m_AdjustUrl.Reset(new CAdjustUrlCallback<TCallback>(callback));
     }
 
+    /// Set proxy.
+    void SetProxy(const CHttpProxy& proxy) { m_Proxy = proxy; }
+    const CHttpProxy& GetProxy(void) const { return m_Proxy; }
+
+    /// Set request parameters.
+    /// @sa CHttpParam
+    void SetParam(const CHttpParam& param);
+
 private:
     friend class CHttpSession_Base;
-    friend class CHttpSessionImpl1x;
-    friend class CHttpSessionImpl2;
+    friend class CHttpSession;
+    friend class CHttp2Session;
 
-    CHttpRequest(CHttpSession_Base& session, const CUrl& url, EReqMethod method);
+    CHttpRequest(CHttpSession_Base& session, const CUrl& url, EReqMethod method, const CHttpParam& param = {});
 
     class CAdjustUrlCallback_Base : public CObject {
     public:
@@ -530,7 +652,7 @@ private:
 
     // Open connection, initialize response.
     void x_InitConnection(bool use_form_data);
-    void x_InitConnection2(shared_ptr<iostream> stream, bool is_service);
+    void x_InitConnection2(shared_ptr<iostream> stream);
 
     bool x_CanSendData(void) const;
 
@@ -540,10 +662,13 @@ private:
 
     void x_AdjustHeaders(bool use_form_data);
     void x_UpdateResponse(CHttpHeaders::THeaders headers, int status_code, string status_text);
+    void x_SetProxy(SConnNetInfo& net_info);
 
     // CConn_HttpStream callback for parsing headers.
     // 'user_data' must point to a CHttpRequest object.
-    static EHTTP_HeaderParse sx_ParseHeader(const char*, void*, int);
+    static EHTTP_HeaderParse sx_ParseHeader(const char* headers,
+                                            void*       user_data,
+                                            int         server_error);
     // CConn_HttpStream callback for handling retries and redirects.
     // 'user_data' must point to a CHttpRequest object.
     static int sx_Adjust(SConnNetInfo* net_info,
@@ -551,24 +676,26 @@ private:
                          unsigned int  failure_count);
 
     CRef<CHttpSession_Base> m_Session;
-    CUrl                m_Url;
-    bool                m_IsService;
-    EReqMethod          m_Method;
-    CRef<CHttpHeaders>  m_Headers;
-    CRef<CHttpFormData> m_FormData;
-    shared_ptr<iostream> m_Stream;
-    CRef<CHttpResponse> m_Response; // current response or null
-    CTimeout            m_Timeout;
-    THttpRetries        m_Retries;
-    CTimeout            m_Deadline;
-    ESwitch             m_RetryProcessing;
+    CUrl                    m_Url;
+    EReqMethod              m_Method;
+    CRef<CHttpHeaders>      m_Headers;
+    CRef<CHttpFormData>     m_FormData;
+    shared_ptr<iostream>    m_Stream;
+    CRef<CHttpResponse>     m_Response; // current response or null
+    CTimeout                m_Timeout;
+    THttpRetries            m_Retries;
+    CTimeout                m_Deadline;
+    ESwitch                 m_RetryProcessing;
     CRef<CAdjustUrlCallback_Base> m_AdjustUrl;
+    // Store credentials so that they are not destroyed while request is being executed.
+    shared_ptr<CTlsCertCredentials> m_Credentials;
+    CHttpProxy              m_Proxy;
 };
 
 
 /// HTTP session class, holding common data for multiple requests.
 class NCBI_XCONNECT_EXPORT CHttpSession_Base : public CObject,
-                                          virtual protected CConnIniter
+                                               virtual protected CConnIniter
 {
 public:
     /// Supported request methods, proxy for EReqMethod.
@@ -586,7 +713,7 @@ public:
     /// A user can set headers/form-data before opening the stream and
     /// sending the actual request.
     /// The default request method is GET.
-    CHttpRequest NewRequest(const CUrl& url, ERequestMethod method = eGet);
+    CHttpRequest NewRequest(const CUrl& url, ERequestMethod method = eGet, const CHttpParam& param = {});
 
     /// Shortcut for GET requests.
     /// @param url
@@ -637,7 +764,7 @@ public:
     enum EProtocol {
         eHTTP_10, ///< HTTP/1.0
         eHTTP_11, ///< HTTP/1.1
-        eHTTP_2,  ///< HTTP/2
+        eHTTP_2   ///< HTTP/2
     };
 
     /// Get protocol version.
@@ -652,9 +779,16 @@ public:
     /// @sa GetHttpFlags
     void SetHttpFlags(THTTP_Flags flags) { m_HttpFlags = flags; }
 
-    CHttpSession_Base(void);
+    /// Set TLS credentials.
+    /// @sa CTlsCertCredentials
+    void SetCredentials(shared_ptr<CTlsCertCredentials> cred);
+    shared_ptr<CTlsCertCredentials> GetCredentials(void) const { return m_Credentials; }
+
+    CHttpSession_Base(EProtocol protocol);
     virtual ~CHttpSession_Base(void) {}
 
+    void SetProxy(const CHttpProxy& proxy) { m_Proxy = proxy; }
+    const CHttpProxy& GetProxy(void) const { return m_Proxy; }
 private:
     friend class CHttpRequest;
     friend class CHttpResponse;
@@ -671,45 +805,31 @@ private:
     EProtocol    m_Protocol;
     THTTP_Flags  m_HttpFlags;
     CHttpCookies m_Cookies;
+    shared_ptr<CTlsCertCredentials> m_Credentials;
+    CHttpProxy   m_Proxy;
 };
 
 
-template <class TImpl>
-class CHttpSessionTmpl : public CHttpSession_Base
+/// @sa CHttpSession_Base
+class NCBI_XCONNECT_EXPORT CHttpSession : public CHttpSession_Base
 {
-    void x_StartRequest(EProtocol protocol, CHttpRequest& req, bool use_form_data) override
-    {
-        TImpl::StartRequest(protocol, req, use_form_data);
-    }
+public:
+    CHttpSession() : CHttpSession_Base(CHttpSession_Base::eHTTP_10) {}
 
-    bool x_Downgrade(CHttpResponse& resp, EProtocol& protocol) const override
-    {
-        return TImpl::Downgrade(resp, protocol);
-    }
-};
-
-
-class CHttpSessionImpl1x
-{
-    friend class CHttpSessionTmpl<CHttpSessionImpl1x>;
-
-    static void StartRequest(CHttpSession_Base::EProtocol _DEBUG_ARG(protocol), CHttpRequest& req, bool use_form_data)
+private:
+    void x_StartRequest(EProtocol _DEBUG_ARG(protocol), CHttpRequest& req, bool use_form_data) override
     {
         _ASSERT(protocol <= CHttpSession_Base::eHTTP_11);
         req.x_InitConnection(use_form_data);
     }
 
-    static bool Downgrade(CHttpResponse&, CHttpSession_Base::EProtocol&)
+    bool x_Downgrade(CHttpResponse&, EProtocol&) const override
     {
         return false;
     }
 };
 
 
-/// @sa CHttpSession_Base
-using CHttpSession = CHttpSessionTmpl<CHttpSessionImpl1x>;
-
-
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// CHttpSessionException --
@@ -721,6 +841,7 @@ class NCBI_XCONNECT_EXPORT CHttpSessionException : public CException
 {
 public:
     enum EErrCode {
+        eConnFailed,      ///< Failed to open connection.
         eBadRequest,      ///< Error initializing or sending a request.
         eBadContentType,  ///< Content-type conflicts with the data.
         eBadFormDataName, ///< Empty or bad name in form data.
