@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_UTIL__H
 #define CONNECT___NCBI_UTIL__H
 
-/* $Id: ncbi_util.h 620617 2020-11-25 05:25:50Z lavr $
+/* $Id: ncbi_util.h 668369 2023-06-03 01:48:35Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -59,7 +59,7 @@
  * 4. Miscellaneous:
  *       UTIL_MatchesMask[Ex]()
  *       UTIL_NcbiLocalHostName()
- *       UTIL_PrintableString[Size]()
+ *       UTIL_PrintableString[Ex|Size]()
  *
  * 5. Internal MSWIN support for Unicode (mostly in error messages)
  *
@@ -86,7 +86,7 @@ extern "C" {
 
 /** Set the MT critical section lock/unlock handler -- to be used by the core
  * internals for protection of internal static variables and other MT-sensitive
- * code from being accessed/changed by several threads simultaneously. It is
+ * code from being accessed/changed by several threads simultaneously.  It is
  * also to fully protect the core log handler and core regirsty, including
  * their setup, as well as callback and cleanup functions.
  * @warning This function itself is NOT MT-safe!
@@ -102,7 +102,7 @@ extern "C" {
  * a (write) lock), the lock must allow recursive locking (by the same thread)
  * to the depth of 2.
  * @param lk
- *  MT-Lock as created by MT_LOCK_Create
+ *  MT-Lock as created by MT_LOCK_Create (NULL "lk" drops the internal MT-Lock)
  * @sa
  *  MT_LOCK_Create, MT_LOCK_Delete, CORE_SetLOG, CORE_SetREG
  */
@@ -164,7 +164,7 @@ extern NCBI_XCONNECT_EXPORT MT_LOCK CORE_GetLOCK(void);
       (defined(__MWERKS__)        &&  (__MWERKS__ >= 0x3000))  ||     \
       (defined(__ICC)             &&  (__ICC >= 600))
 #  define CORE_CURRENT_FUNCTION  __PRETTY_FUNCTION__
-#elif defined(__FUNCSIG__)
+#elif  defined(__FUNCSIG__)
 #  define CORE_CURRENT_FUNCTION  __FUNCSIG__
 #elif (defined(__INTEL_COMPILER)  &&  (__INTEL_COMPILER >= 600))  ||  \
       (defined(__IBMCPP__)        &&  (__IBMCPP__ >= 500))
@@ -178,8 +178,19 @@ extern NCBI_XCONNECT_EXPORT MT_LOCK CORE_GetLOCK(void);
 #endif
 
 
+/** Default CORE LOG cut off log level.
+ * @sa
+ *  CORE_SetLOGFILE, CORE_SetLOGFILE_NAME
+ */
+#if !defined(NCBI_CXX_TOOLKIT)  &&  defined(_DEBUG)  &&  !defined(NDEBUG)
+#  define CORE_LOGFILE_CUTOFF_LEVEL  eLOG_Trace
+#else
+#  define CORE_LOGFILE_CUTOFF_LEVEL  eLOG_Note
+#endif /*!NCBI_CXX_TOOLKIT &&_DEBUG && !NDEBUG*/
+
+
 /** Set the log handle (no logging if "lg" is passed zero) -- to be used by
- * the core internals.
+ * the core internals (CORE LOG).
  * If there is an active log handler set already, and it is different from the
  * new one, then LOG_Delete is called internally for the old logger (that is,
  * the one being replaced).
@@ -191,7 +202,7 @@ extern NCBI_XCONNECT_EXPORT MT_LOCK CORE_GetLOCK(void);
 extern NCBI_XCONNECT_EXPORT void CORE_SetLOG(LOG lg);
 
 
-/** Get the log handle that is to be used by the core internals.
+/** Get the log handle that is to be used by the core internals (CORE LOG).
  * @return
  *  LOG handle as previously set by CORE_SetLOG() or NULL if no logging is
  *  currently active
@@ -204,7 +215,7 @@ extern NCBI_XCONNECT_EXPORT void CORE_SetLOG(LOG lg);
 extern NCBI_XCONNECT_EXPORT LOG  CORE_GetLOG(void);
 
 
-/** Standard logging to the specified file stream.
+/** Standard logging (CORE LOG) to the specified file stream.
  * @param fp
  *  The file stream to log to
  * @param cut_off
@@ -213,7 +224,7 @@ extern NCBI_XCONNECT_EXPORT LOG  CORE_GetLOG(void);
  *  Severity greater than or equal to the specified level always posts to log,
  *  and then aborts the application
  * @param auto_close
- *  Do "fclose(fp)" when the LOG is reset/destroyed
+ *  Whether to call "fclose(fp)" when the CORE LOG is reset/destroyed
  * @sa
  *  LOG_ToFILE_Ex, CORE_SetLOG
  */
@@ -225,7 +236,8 @@ extern NCBI_XCONNECT_EXPORT void CORE_SetLOGFILE_Ex
  );
 
 
-/** Same as CORE_SetLOGFILE_Ex(fp, eLOG_Trace, eLOG_Fatal, auto_close).
+/** Same as
+ *  CORE_SetLOGFILE_Ex(fp, CORE_LOGFILE_CUTOFF_LEVEL, eLOG_Fatal, auto_close).
  * @sa
  *  CORE_SetLOGFILE_Ex, CORE_SetLOG
  */
@@ -255,7 +267,8 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ CORE_SetLOGFILE_NAME_Ex
  );
 
 
-/** Same as CORE_SetLOGFILE_NAME_Ex(logfile, eLOG_Trace, eLOG_Fatal).
+/** Same as
+ *  CORE_SetLOGFILE_NAME_Ex(logfile, CORE_LOGFILE_CUTOFF_LEVEL, eLOG_Fatal).
  * @sa
  *  CORE_SetLOGFILE_NAME_Ex, CORE_SetLOG
  */
@@ -291,9 +304,9 @@ extern NCBI_XCONNECT_EXPORT TLOG_FormatFlags CORE_SetLOGFormatFlags
 /** Compose a message using the "call_data" info.
  * Full log record format:
  *     mm/dd/yy HH:MM:SS "<file>", line <line>: [<module>::<function>] <level>: <message>
- *     \n----- [BEGIN] Raw Data (<raw_size> bytes) -----\n
+ *     \n#################### [BEGIN] Raw Data (<raw_size> byte[s]):\n
  *     <raw_data>
- *     \n----- [END] Raw Data -----\n
+ *     \n#################### [_END_] Raw Data\n
  *
  * @note
  *  The returned string must be deallocated using "free()".
@@ -320,7 +333,7 @@ extern NCBI_XCONNECT_EXPORT char* LOG_ComposeMessage
  * @param fatal_err
  *  Severity greater or equal to "fatal_err" always logs and aborts the program
  * @param auto_close
- *  Whether to do "fclose(fp)" when the LOG is reset/destroyed
+ *  Whether to call "fclose(fp)" when the LOG is reset/destroyed
  * @sa
  *  LOG_Create, LOG_Reset, LOG_ComposeMessage, LOG_ToFILE
  */
@@ -686,10 +699,10 @@ extern NCBI_XCONNECT_EXPORT char* UTIL_NcbiLocalHostName
 
 
 /** Calculate size of buffer needed to store printable representation of the
- *  block of data of the specified size (or, if size is 0, strlen(data))
- *  (without the '\0' terminator).
- * @note
- *  The calculated size does not account for a terminating '\0'.
+ *  block of data of the specified size (or, if size is 0, strlen(data)) but
+ *  without the '\0' terminator.
+ * @warning
+ *  The calculated size does not account for a terminating '\0'!
  * @param data
  *  Block of data (NULL causes 0 to return regardless of "size")
  * @param size
@@ -697,7 +710,7 @@ extern NCBI_XCONNECT_EXPORT char* UTIL_NcbiLocalHostName
  * @return
  *  The buffer size needed (0 for NULL or empty data)
  * @sa
- *  UTIL_PrintableString
+ *  UTIL_PrintableStringEx, UTIL_PrintableString
  */
 extern NCBI_XCONNECT_EXPORT size_t UTIL_PrintableStringSize
 (const char* data,
@@ -705,21 +718,28 @@ extern NCBI_XCONNECT_EXPORT size_t UTIL_PrintableStringSize
  );
 
 
+#define UTIL_PRINTABLE_WIDTH  80   /** Default minimum printable width */
+
+
 typedef enum {
-    eUTIL_PrintableFullOctal = 1,  /**< No contactions in octals \ooo        */
-    eUTIL_PrintableNoNewLine = 2   /**< Do not include graphical newlines    */
+    fUTIL_PrintableFullOctal = 1,  /**< No contactions in octals \ooo        */
+    eUTIL_PrintableFullOctal = fUTIL_PrintableFullOctal,  /**< deprecated    */
+    fUTIL_PrintableNoNewLine = 2,  /**< Do not include graphical newlines    */
+    eUTIL_PrintableNoNewLine = fUTIL_PrintableNoNewLine   /**< deprecated    */
 } EUTIL_PrintableFlags;
 typedef int TUTIL_PrintableFlags;  /**< Bitwise "OR" of EUTIL_PrintableFlags */
 
 
-/** Create printable representation of the block of data of the specified size
+/** Create a printable representation of a block of data of the specified size
  *  (or, if size is 0, strlen(data)), and return the buffer pointer past the
  *  last stored character (non '\0'-terminated).  Non-printable characters can
  *  be represented in a reduced octal form as long as the result is unambiguous
  *  (unless "full" passed true (non-zero), in which case all non-printable
- *  characters get represented by full octal tetrads).  NB: Hexadecimal output
- *  is not used because it is ambiguous by the standard (can contain undefined
- *  number of hex digits).
+ *  characters get represented by full octal tetrads).
+ * @note
+ *  Hexadecimal output is not used because it is ambiguous by the standard
+ *  (can contain arbitrary number of hex digits, which may be indistiguishable
+ *  from the following text[a-z][A-Z] or numerals[0-9]).
  * @note
  *  The input buffer "buf" where to store the printable representation is
  *  assumed to be of adequate size to hold the resultant string (use
@@ -732,21 +752,41 @@ typedef int TUTIL_PrintableFlags;  /**< Bitwise "OR" of EUTIL_PrintableFlags */
  *  Buffer to store the result (NULL always causes NULL to return)
  * @param flags
  *  How to print representations of certain non-printable characters
+ * @param width
+ *  Insert escaped line breaks when output line exceeds "width"; no-op if 0
  * @return
  *  Next position in the buffer past the last stored character
  * @warning
  *  The call *does not* '\0'-terminate its output!
+ * @warning
+ *  Successive calls of UTIL_PrintableString() that append to the same buffer
+ *  one after another may not produce a well-formed C string literal, in all.
  * @sa
  *  UTIL_PrintableStringSize, EUTIL_PrintableFlags
  */
-extern NCBI_XCONNECT_EXPORT char* UTIL_PrintableString
+extern NCBI_XCONNECT_EXPORT char* UTIL_PrintableStringEx
 (const char*          data,
  size_t               size,
  char*                buf,
- TUTIL_PrintableFlags flags
+ TUTIL_PrintableFlags flags,
+ int                  width
  );
 
 
+/** Same as UTIL_PrintableStringEx(..., 0) -- i.e. w/o width restrictions.
+ * @sa
+ *  UTIL_PrintableStringEx
+ */
+#define UTIL_PrintableString(d,s,b,f) UTIL_PrintableStringEx((d),(s),(b),(f),0)
+
+
+/** Given the main()'s argc and argv return non-zero (true) if the arguments
+ *  specify that only a help option was requested. Return 0 (false), otherwise.
+ */
+extern NCBI_XCONNECT_EXPORT int/*bool*/ UTIL_HelpRequested
+(int argc, char** argv);
+
+
 /** Conversion from Unicode to UTF8, and back.  MSWIN-specific and internal.
  *
  * @note  UTIL_ReleaseBufferOnHeap() must be used to free the buffers returned
