@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_PARAM__HPP
 #define CORELIB___NCBI_PARAM__HPP
 
-/*  $Id: ncbi_param.hpp 598497 2019-12-10 14:23:27Z grichenk $
+/*  $Id: ncbi_param.hpp 606111 2020-04-20 13:34:35Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -84,105 +84,6 @@ BEGIN_NCBI_SCOPE
 ///
 
 
-/////////////////////////////////////////////////////////////////////////////
-///
-/// Helper functions for getting values from registry/environment
-///
-
-/// Get string configuration value.
-///
-/// @param section
-///   Check application configuration named section first if not null.
-/// @param variable
-///   Variable name within application section.
-///   If no value found in configuration file, environment variable with
-///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
-///   checked, depending on wether section is null.
-/// @param env_var_name
-///   If not empty, overrides the default NCBI_CONFIG__section__name
-///   name of the environment variable.
-/// @param default_value
-///   If no value found neither in configuration file nor in environment,
-///   this value will be returned, or empty string if this value is null.
-/// @return
-///   string configuration value.
-/// @sa g_GetConfigInt(), g_GetConfigFlag()
-string NCBI_XNCBI_EXPORT g_GetConfigString(const char* section,
-                                           const char* variable,
-                                           const char* env_var_name,
-                                           const char* default_value,
-                                           int* src = nullptr);
-
-/// Get integer configuration value.
-///
-/// @param section
-///   Check application configuration named section first if not null.
-/// @param variable
-///   Variable name within application section.
-///   If no value found in configuration file, environment variable with
-///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
-///   checked, depending on wether section is null.
-/// @param env_var_name
-///   If not empty, overrides the default NCBI_CONFIG__section__name
-///   name of the environment variable.
-/// @param default_value
-///   If no value found neither in configuration file nor in environment,
-///   this value will be returned.
-/// @return
-///   integer configuration value.
-/// @sa g_GetConfigString(), g_GetConfigFlag()
-int NCBI_XNCBI_EXPORT g_GetConfigInt(const char* section,
-                                     const char* variable,
-                                     const char* env_var_name,
-                                     int         default_value);
-
-/// Get boolean configuration value.
-///
-/// @param section
-///   Check application configuration named section first if not null.
-/// @param variable
-///   Variable name within application section.
-///   If no value found in configuration file, environment variable with
-///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
-///   checked, depending on wether section is null.
-/// @param env_var_name
-///   If not empty, overrides the default NCBI_CONFIG__section__name
-///   name of the environment variable.
-/// @param default_value
-///   If no value found neither in configuration file nor in environment,
-///   this value will be returned.
-/// @return
-///   boolean configuration value.
-/// @sa g_GetConfigString(), g_GetConfigInt()
-bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
-                                       const char* variable,
-                                       const char* env_var_name,
-                                       bool        default_value);
-
-
-/// Get double configuration value.
-///
-/// @param section
-///   Check application configuration named section first if not null.
-/// @param variable
-///   Variable name within application section.
-///   If no value found in configuration file, environment variable with
-///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
-///   checked, depending on wether section is null.
-/// @param env_var_name
-///   If not empty, overrides the default NCBI_CONFIG__section__name
-///   name of the environment variable.
-/// @param default_value
-///   If no value found neither in configuration file nor in environment,
-///   this value will be returned.
-/// @return
-///   double configuration value.
-/// @sa g_GetConfigString(), g_GetConfigInt()
-double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
-                                           const char* variable,
-                                           const char* env_var_name,
-                                           double  default_value);
-
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// Parameter declaration and definition macros
@@ -617,6 +518,106 @@ private:
 };
 
 
+/////////////////////////////////////////////////////////////////////////////
+///
+/// Helper functions for getting values from registry/environment
+///
+
+/// Get string configuration value.
+///
+/// @param section
+///   Check application configuration named section first if not null.
+/// @param variable
+///   Variable name within application section.
+///   If no value found in configuration file, environment variable with
+///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
+///   checked, depending on wether section is null.
+/// @param env_var_name
+///   If not empty, overrides the default NCBI_CONFIG__section__name
+///   name of the environment variable.
+/// @param default_value
+///   If no value found neither in configuration file nor in environment,
+///   this value will be returned, or empty string if this value is null.
+/// @return
+///   string configuration value.
+/// @sa g_GetConfigInt(), g_GetConfigFlag()
+string NCBI_XNCBI_EXPORT g_GetConfigString(const char* section,
+    const char* variable,
+    const char* env_var_name,
+    const char* default_value,
+    CParamBase::EParamSource* src = nullptr);
+
+/// Get integer configuration value.
+///
+/// @param section
+///   Check application configuration named section first if not null.
+/// @param variable
+///   Variable name within application section.
+///   If no value found in configuration file, environment variable with
+///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
+///   checked, depending on wether section is null.
+/// @param env_var_name
+///   If not empty, overrides the default NCBI_CONFIG__section__name
+///   name of the environment variable.
+/// @param default_value
+///   If no value found neither in configuration file nor in environment,
+///   this value will be returned.
+/// @return
+///   integer configuration value.
+/// @sa g_GetConfigString(), g_GetConfigFlag()
+int NCBI_XNCBI_EXPORT g_GetConfigInt(const char* section,
+    const char* variable,
+    const char* env_var_name,
+    int         default_value);
+
+/// Get boolean configuration value.
+///
+/// @param section
+///   Check application configuration named section first if not null.
+/// @param variable
+///   Variable name within application section.
+///   If no value found in configuration file, environment variable with
+///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
+///   checked, depending on wether section is null.
+/// @param env_var_name
+///   If not empty, overrides the default NCBI_CONFIG__section__name
+///   name of the environment variable.
+/// @param default_value
+///   If no value found neither in configuration file nor in environment,
+///   this value will be returned.
+/// @return
+///   boolean configuration value.
+/// @sa g_GetConfigString(), g_GetConfigInt()
+bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
+    const char* variable,
+    const char* env_var_name,
+    bool        default_value);
+
+
+/// Get double configuration value.
+///
+/// @param section
+///   Check application configuration named section first if not null.
+/// @param variable
+///   Variable name within application section.
+///   If no value found in configuration file, environment variable with
+///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
+///   checked, depending on wether section is null.
+/// @param env_var_name
+///   If not empty, overrides the default NCBI_CONFIG__section__name
+///   name of the environment variable.
+/// @param default_value
+///   If no value found neither in configuration file nor in environment,
+///   this value will be returned.
+/// @return
+///   double configuration value.
+/// @sa g_GetConfigString(), g_GetConfigInt()
+double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
+    const char* variable,
+    const char* env_var_name,
+    double  default_value);
+
+
 END_NCBI_SCOPE
 
 
