@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_URL__HPP
 #define CORELIB___NCBI_URL__HPP
 
-/*  $Id: ncbi_url.hpp 593659 2019-09-23 03:58:14Z lavr $
+/*  $Id: ncbi_url.hpp 671001 2023-08-07 12:48:18Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -169,7 +169,17 @@ private:
 class NCBI_XNCBI_EXPORT CUrlArgs_Parser
 {
 public:
-    CUrlArgs_Parser(void) : m_SemicolonIsNotArgDelimiter(false) {}
+    /// An inverted subset of CCgiRequest::TFlags
+    /// @sa CCgiRequest
+    typedef int TFlags;
+    enum Flags {
+        /// Treat semicolon as query string argument separator
+        fSemicolonIsArgDelimiter    = (1 << 11),
+        /// Enable parsing input as 'indexed' query (RFC3875) when no '=' is present
+        fEnableParsingAsIndex       = (1 << 14),
+    };
+
+    CUrlArgs_Parser(TFlags flags = 0) : m_Flags(flags) {}
     virtual ~CUrlArgs_Parser(void) {}
 
     /// Parse query string, call AddArgument() to store each value.
@@ -181,7 +191,11 @@ public:
     /// Treat semicolon as query string argument separator
     void SetSemicolonIsNotArgDelimiter(bool enable = true)
     {
-        m_SemicolonIsNotArgDelimiter = enable;
+        if (enable) {
+            m_Flags &= ~fSemicolonIsArgDelimiter;
+        } else {
+            m_Flags |= fSemicolonIsArgDelimiter;
+        }
     }
 
 protected:
@@ -206,11 +220,12 @@ protected:
                              const string& name,
                              const string& value,
                              EArgType      arg_type = eArg_Index) = 0;
+
+    TFlags m_Flags;
+
 private:
     void x_SetIndexString(const string& query,
                           const IUrlEncoder& encoder);
-
-    bool m_SemicolonIsNotArgDelimiter;
 };
 
 
@@ -225,11 +240,14 @@ class NCBI_XNCBI_EXPORT CUrlArgs : public CUrlArgs_Parser
 {
 public:
     /// Create an empty arguments set.
-    CUrlArgs(void);
+    CUrlArgs(TFlags flags = 0);
     /// Parse the query string, store the arguments.
-    CUrlArgs(const string& query, NStr::EUrlEncode decode);
+    CUrlArgs(const string& query, NStr::EUrlEncode decode, TFlags flags = 0);
     /// Parse the query string, store the arguments.
-    CUrlArgs(const string& query, const IUrlEncoder* encoder = 0);
+    CUrlArgs(const string& query, const IUrlEncoder* encoder = 0, TFlags flags = 0);
+
+    /// Clear the list.
+    void clear(void) { m_IsIndex = false; m_Args.clear(); }
 
     /// Ampersand encoding for composed URLs
     enum EAmpEncoding {
@@ -341,14 +359,20 @@ public:
     ///
     /// @param url
     ///   String to parse as URL:
-    ///   Generic: [scheme://[[user]:[password]@]]host[:port][/path][?args]
-    ///   Special: scheme:[path]
-    ///   The leading '/', if any, is included in path value.
+    ///   Generic:
+    ///      [scheme://[[user]:[password]@]]host[:port][/path][?args][#frag]
+    ///   Special:
+    ///      scheme:[path]
+    ///   The leading '/', if any, is included in the path value.
     /// @param encoder
     ///   URL encoder object. If not set, the default encoder will be used.
     ///   @sa CDefaultUrlEncoder
     CUrl(const string& url, const IUrlEncoder* encoder = 0);
 
+    CUrl(const char* url, const IUrlEncoder* encoder = 0)
+        : CUrl(string(url), encoder)
+    { }
+
     /// Parse the URL.
     ///
     /// @param url
@@ -371,38 +395,37 @@ public:
 
     // Access parts of the URL
 
-    string GetScheme(void) const            { return m_Scheme; }
-    void   SetScheme(const string& value);
+    const string& GetScheme(void) const             { return m_Scheme; }
+    void          SetScheme(const string& value);
 
     /// Generic schemes use '//' prefix (after optional scheme).
-    bool GetIsGeneric(void) const           { return m_IsGeneric; }
-    void SetIsGeneric(bool value)           { m_IsGeneric = value; }
+    bool          GetIsGeneric(void) const          { return m_IsGeneric; }
+    void          SetIsGeneric(bool value)          { m_IsGeneric = value; }
 
-    string GetUser(void) const              { return m_User; }
-    void   SetUser(const string& value)     { m_User = value; }
+    const string& GetUser(void) const               { return m_User; }
+    void          SetUser(const string& value)      { m_User = value; }
 
-    string GetPassword(void) const          { return m_Password; }
-    void   SetPassword(const string& value) { m_Password = value; }
+    const string& GetPassword(void) const           { return m_Password; }
+    void          SetPassword(const string& value)  { m_Password = value; }
 
-    string GetHost(void) const              { return m_Host; }
-    void   SetHost(const string& value);
+    const string& GetHost(void) const               { return m_Host; }
+    void          SetHost(const string& value);
 
-    bool IsService(void) const              { return !m_Service.empty(); }
-    string GetService(void) const           { return m_Service; }
-    void SetService(const string& value);
+    bool          IsService(void) const             {return!m_Service.empty();}
+    const string& GetService(void) const            { return m_Service; }
+    void          SetService(const string& value);
     
-    string GetPort(void) const              { return m_Port; }
-    void   SetPort(const string& value)     { m_Port = value; }
+    const string& GetPort(void) const               { return m_Port; }
+    void          SetPort(const string& value)      { m_Port = value; }
 
-    string GetPath(void) const              { return m_Path; }
-    void   SetPath(const string& value)     { m_Path = value; }
+    const string& GetPath(void) const               { return m_Path; }
+    void          SetPath(const string& value)      { m_Path = value; }
 
-    string GetFragment(void) const          { return m_Fragment; }
-    void   SetFragment(const string& value) { m_Fragment = value; }
+    const string& GetFragment(void) const           { return m_Fragment; }
+    void          SetFragment(const string& value)  { m_Fragment = value; }
 
     /// Get the original (unparsed and undecoded) query string
-    string GetOriginalArgsString(void) const
-        { return m_OrigArgs; }
+    const string& GetOriginalArgsString(void) const { return m_OrigArgs; }
 
     /// Check if the URL contains any arguments
     bool HaveArgs(void) const
@@ -444,7 +467,7 @@ public:
     typedef int TAdjustFlags;
 
     /// Adjust this URL using information from 'other' URL.
-    /// Scheme, host and port are never changed. Other parts can be replaced or merged
+    /// Host and port are never changed. Other parts can be replaced or merged
     /// depending on the flags.
     /// Throw CUrlException if the flags are inconsistent (e.g. both fPath_Replace and fPath_Append are set).
     void Adjust(const CUrl& other, TAdjustFlags flags);
@@ -461,6 +484,8 @@ private:
     void x_SetArgs(const string& args, const IUrlEncoder& encoder);
     void x_SetFragment(const string& fragment, const IUrlEncoder& encoder);
 
+    bool x_IsHostPort(const string& scheme, string& unparsed, const IUrlEncoder& encoder);
+
     string  m_Scheme;
     bool    m_IsGeneric;  // generic schemes include '//' delimiter
     string  m_User;
