@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMISC__HPP
 #define CORELIB___NCBIMISC__HPP
 
-/*  $Id: ncbimisc.hpp 627621 2021-03-16 14:36:05Z ivanov $
+/*  $Id: ncbimisc.hpp 668988 2023-06-16 13:42:24Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -635,9 +635,17 @@ private:
 
 
 /// Define "null" pointer value.
+#ifdef __cpp_using_enum
+enum class ENull {
+    null = 0
+};
+
+using enum ENull;
+#else
 enum ENull {
     null = 0
 };
+#endif
 
 
 // Default callback for null value - throws CCoreException.
@@ -659,8 +667,12 @@ struct SThrowOnNull
 /// is requested from a null object. By default CCoreException is thrown.
 /// To perform other actions (e.g. provide a default value) the functor
 /// must define 'TValue operator()(void) const' method.
+/// @deprecated: Use std::optional<> instead.
 template <class TValue, class TNullToValue = SThrowOnNull<TValue> >
 class CNullable
+//
+// DEPRECATED! - Use std::optional<> instead.
+//
 {
 public:
     /// Create an empty nullable.
@@ -921,8 +933,8 @@ private:
     typename enable_if<is_arithmetic<T>::value, TThis>::type& operator=(T id) { m_Id = id; return *this; }
     template<typename T>
     typename enable_if<is_arithmetic<T>::value, bool>::type operator==(T id) const { return m_Id == id; }
-    template<typename T>
-    operator typename enable_if<is_arithmetic<T>::value, T>::type(void) const { return static_cast<T>(m_Id); }
+    template<typename T, typename enable_if<is_arithmetic<T>::value, T>::type* = nullptr>
+    operator T(void) const { return static_cast<T>(m_Id); }
 #endif
 
 private:
@@ -966,9 +978,14 @@ CNcbiIstream& operator>>(CNcbiIstream& in, CStrictId<TKey, TStorage>& id)
 #  define NCBI_INT8_GI
 #endif
 
+#ifdef NCBI_TEST_STRICT_ENTREZ_ID
+// Allow strict TEntrezId test builds
+#define NCBI_STRICT_ENTREZ_ID
+#else
 // Temporary fix: disable strict TEntrezId
-#ifdef NCBI_STRICT_ENTREZ_ID
-#  undef NCBI_STRICT_ENTREZ_ID
+#  ifdef NCBI_STRICT_ENTREZ_ID
+#    undef NCBI_STRICT_ENTREZ_ID
+#  endif
 #endif
 
 #ifndef NCBI_STRICT_GI
@@ -1245,7 +1262,7 @@ public:
         {
         }
     CSafeFlags(enum_type flags)
-        : m_Flags(flags)
+        : m_Flags(static_cast<storage_type>(flags))
         {
         }
 
