@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBITHR__HPP
 #define CORELIB___NCBITHR__HPP
 
-/*  $Id: ncbithr.hpp 604363 2020-03-26 18:59:08Z ivanov $
+/*  $Id: ncbithr.hpp 639336 2021-10-18 16:06:35Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,7 +43,6 @@
 ///
 
 
-#include <corelib/ncbimtx.hpp>
 #include <corelib/ncbi_process.hpp>
 #include <corelib/ncbi_safe_static.hpp>
 #include <list>
@@ -139,9 +138,12 @@ private:
     /// Returns true if CTlsBase must be deregistered from current thread
     bool x_DeleteTlsData(ECleanupMode mode = eCleanup_Toolkit);
 
-public:
+    static void CleanupAndDeleteTlsData(void *data, ECleanupMode mode = eCleanup_Toolkit);
     static void CleanupTlsData(void *data, ECleanupMode mode = eCleanup_Toolkit);
+    
+    static void x_CleanupThreadCallback(void* ptr);
 
+public:
     template<class TValue>
     static void DefaultCleanup(TValue *value, void*) {
         if (value) {
