@@ -1,7 +1,7 @@
 #ifndef CORELIB___REQUEST_CTX__HPP
 #define CORELIB___REQUEST_CTX__HPP
 
-/*  $Id: request_ctx.hpp 632469 2021-06-02 11:12:10Z ivanov $
+/*  $Id: request_ctx.hpp 664060 2023-03-07 16:40:29Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -64,8 +64,13 @@ class CRequestContext_PassThrough;
 class NCBI_XNCBI_EXPORT CSharedHitId
 {
 public:
-    explicit CSharedHitId(const string& hit)
-        : m_HitId(hit), m_SubHitId(0), m_AppState(GetDiagContext().GetAppState()) {}
+    /// Set new hit id, checks its validity
+    explicit CSharedHitId(const string& hit_id)
+        : m_SubHitId(0), m_AppState(GetDiagContext().GetAppState())
+    { 
+        x_SetHitId(hit_id);
+    }
+
     CSharedHitId(void) : m_SubHitId(0) {}
     ~CSharedHitId(void) {}
 
@@ -90,7 +95,7 @@ public:
     {
         m_SharedSubHitId.Reset();
         m_SubHitId = 0;
-        m_HitId = hit_id;
+        x_SetHitId(hit_id);
         m_AppState = GetDiagContext().GetAppState();
     }
 
@@ -116,6 +121,10 @@ public:
             m_AppState == eDiagAppState_RequestEnd;
     }
 
+private:
+    /// Set new hit id, checks its validity
+    void x_SetHitId(const string& hit_id);
+
 private:
     typedef CObjectFor<CAtomicCounter> TSharedCounter;
 
@@ -129,6 +138,21 @@ private:
 class CMask;
 class CMaskFileName;
 
+class IRequestTracer
+{
+public:
+    virtual ~IRequestTracer(void) {}
+    virtual void OnRequestStart(CRequestContext& context) = 0;
+    virtual void OnRequestStop(CRequestContext& context) = 0;
+};
+
+
+class ITracerSpan
+{
+public:
+    virtual ~ITracerSpan(void) {}
+};
+
 
 class NCBI_XNCBI_EXPORT CRequestContext : public CObject
 {
@@ -165,9 +189,10 @@ public:
     void          SetAppState(EDiagAppState state);
 
     /// Client IP/hostname
-    const string& GetClientIP(void) const;
+    string        GetClientIP(void) const;
     void          SetClientIP(const string& client);
     bool          IsSetClientIP(void) const;
+    bool          IsSetExplicitClientIP(void) const;
     void          UnsetClientIP(void);
 
     /// Session ID
@@ -354,6 +379,13 @@ public:
     /// session id).
     TVersion GetVersion(void) const { return m_Version; }
 
+    /// Set request tracer to be called on context events (start/stop etc.).
+    /// @sa IRequestTracer
+    void SetRequestTracer(const shared_ptr<IRequestTracer>& tracer) { m_Tracer = tracer; }
+
+    void SetTracerSpan(const shared_ptr<ITracerSpan>& span) { m_TracerSpan = span; }
+    shared_ptr<ITracerSpan> GetTracerSpan(void) { return m_TracerSpan; }
+
 private:
     // Prohibit copying
     CRequestContext(const CRequestContext&);
@@ -463,6 +495,9 @@ private:
     friend class CRequestContext_PassThrough;
     mutable TPassThroughProperties m_PassThroughProperties;
 
+    shared_ptr<IRequestTracer> m_Tracer;
+    shared_ptr<ITracerSpan> m_TracerSpan;
+
     // Patterns from NCBI_CONTEXT_FIELDS variable.
     static unique_ptr<CMaskFileName> sm_ContextFields;
 
@@ -635,13 +670,21 @@ void CRequestContext::UnsetRequestID(void)
 
 
 inline
-const string& CRequestContext::GetClientIP(void) const
+string CRequestContext::GetClientIP(void) const
 {
-    return x_IsSetProp(eProp_ClientIP) ? m_ClientIP : kEmptyStr;
+    return x_IsSetProp(eProp_ClientIP) ?
+        m_ClientIP : GetDiagContext().GetDefaultClientIP();
 }
 
 inline
 bool CRequestContext::IsSetClientIP(void) const
+{
+    return IsSetExplicitClientIP() ||
+        !GetDiagContext().GetDefaultClientIP().empty();
+}
+
+inline
+bool CRequestContext::IsSetExplicitClientIP(void) const
 {
     return x_IsSetProp(eProp_ClientIP);
 }
