@@ -1,7 +1,7 @@
 #ifndef CORELIB___TEMPSTR__HPP
 #define CORELIB___TEMPSTR__HPP
 
-/*  $Id: tempstr.hpp 566948 2018-07-10 19:41:29Z lavr $
+/*  $Id: tempstr.hpp 663830 2023-03-01 16:19:54Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,6 +38,10 @@
 #include <corelib/ncbiexpt.hpp>
 #include <algorithm>
 
+#ifdef NCBI_HAVE_CXX17
+    #include <string_view>
+#endif
+
 
 BEGIN_NCBI_SCOPE
 
@@ -83,6 +87,10 @@ public:
     CTempString(const CTempString& str, size_type pos);
     CTempString(const CTempString& str, size_type pos, size_type len);
 
+    #ifdef NCBI_HAVE_CXX17
+        CTempString(const string_view&  str_view);
+    #endif
+
     /// Copy a substring into a string
     /// Somewhat similar to basic_string::assign()
     void Copy(string& dst, size_type pos, size_type len) const;
@@ -188,6 +196,10 @@ public:
 
     operator string(void) const;
 
+    #ifdef NCBI_HAVE_CXX17
+        operator string_view(void) const;
+    #endif
+
 private:
     const char* m_String;  ///< Stored pointer to string
     size_type   m_Length;  ///< Length of string
@@ -488,6 +500,15 @@ CTempString::CTempString(const CTempString& str, size_type pos, size_type len)
     NCBI_TEMPSTR_MAKE_COPY();
 }
 
+#ifdef NCBI_HAVE_CXX17
+inline
+CTempString::CTempString(const string_view&  str_view)
+    : m_String(str_view.data()), m_Length(str_view.size())
+{
+    NCBI_TEMPSTR_MAKE_COPY();
+}
+#endif
+
 
 inline
 void CTempString::erase(size_type pos)
@@ -772,6 +793,15 @@ CTempString::operator string(void) const
 }
 
 
+#ifdef NCBI_HAVE_CXX17
+inline
+CTempString::operator string_view(void) const
+{
+    return string_view(data(), length());
+}
+#endif
+
+
 inline
 int CTempString::compare(const CTempString str) const
 {
@@ -831,13 +861,13 @@ bool CTempString::operator>(const CTempString str) const
 inline
 bool operator==(const char* str1, const CTempString str2)
 {
-    return str2 == str1;
+    return str2 == CTempString(str1);
 }
 
 inline
 bool operator==(const string& str1, const CTempString str2)
 {
-    return str2 == str1;
+    return str2 == CTempString(str1);
 }
 
 inline
@@ -918,6 +948,13 @@ public:
           m_ZeroAtEnd(eNoZeroAtEnd)
         {
         }
+    #ifdef NCBI_HAVE_CXX17
+    CTempStringEx(const string_view&  str_view)
+        : CTempString(str_view),
+          m_ZeroAtEnd(eNoZeroAtEnd)
+        {
+        }
+    #endif
     CTempStringEx(const CTempString& str, size_type pos)
         : CTempString(str, pos),
           m_ZeroAtEnd(eNoZeroAtEnd)
