@@ -1,7 +1,7 @@
 #ifndef CORELIB___VERSION_API__HPP
 #define CORELIB___VERSION_API__HPP
 
-/*  $Id: version_api.hpp 619781 2020-11-10 18:40:38Z gouriano $
+/*  $Id: version_api.hpp 657700 2022-10-14 14:21:36Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,7 @@
 /// Define CVersionInfo, a version info storage class.
 
 
+#include <corelib/ncbitime.hpp>
 #include <corelib/ncbiobj.hpp>
 
 
@@ -67,11 +68,12 @@ struct NCBI_XNCBI_EXPORT SBuildInfo
         eTeamCityBuildConf,
         eTeamCityBuildNumber,
         eBuildID,
-        eSubversionRevision,
+        eSubversionRevision, ///< Numeric if present
         eStableComponentsVersion,
         eDevelopmentVersion,
         eProductionVersion,
-        eBuiltAs
+        eBuiltAs,
+        eRevision ///< Not necessarily numeric
     };
     string date;
     string tag;
@@ -91,6 +93,11 @@ struct NCBI_XNCBI_EXPORT SBuildInfo
     static string ExtraNameJson(EExtra key);
     static string ExtraNameAppLog(EExtra key);
 
+    /// Converts 'date' parameter to CTime.
+    /// Returns empty CTime object if unable to do an conversion or build date/time is unknown,
+    /// so check CTime::IsEmpty().
+    CTime GetBuildTime(void) const;
+
     string Print(size_t offset = 0) const;
     string PrintXml(void) const;
     string PrintJson(void) const;
