@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___EXCEPTION__HPP
 #define DBAPI_DRIVER___EXCEPTION__HPP
 
-/* $Id: exception.hpp 619521 2020-11-05 19:23:08Z grichenk $
+/* $Id: exception.hpp 623749 2021-01-19 13:43:22Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -68,7 +68,7 @@ END_SCOPE(impl)
         x_Assign(other); \
     } \
 public: \
-    virtual ~exception_class(void) throw() {} \
+    virtual ~exception_class(void) noexcept {} \
     const char* GetType(void) const override {return #exception_class;} \
     typedef int TErrCode; \
     TErrCode GetErrCode(void) const \
