@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___DBAPI_POOL_BALANCER__HPP
 #define DBAPI_DRIVER___DBAPI_POOL_BALANCER__HPP
 
-/*  $Id: dbapi_pool_balancer.hpp 628206 2021-03-26 13:07:00Z ivanov $
+/*  $Id: dbapi_pool_balancer.hpp 673279 2023-09-27 13:31:52Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,13 +42,30 @@
 
 BEGIN_NCBI_SCOPE
 
+class IDBServiceInfo : public CObject
+{
+public:
+    typedef IDBServiceMapper::TOptions             TOptions;
+    typedef SSimpleLock<IDBServiceInfo>            TLock;
+    typedef SSimpleUnlock<IDBServiceInfo>          TUnlock;
+    typedef CGuard<IDBServiceInfo>                 TGuard;
+    typedef CGuard<IDBServiceInfo, TUnlock, TLock> TAntiGuard;
+    
+    virtual const string& GetServiceName(void) const = 0;
+    virtual const TOptions& GetOptions(void) = 0;
+    virtual TSvrRef GetMappedServer(void) = 0;
+    virtual void Lock(void) = 0;
+    virtual void Unlock(void) = 0;
+};
+
+
 class CDBPoolBalancer : public CPoolBalancer
 {
 public:
-    CDBPoolBalancer(const string& service_name,
+    CDBPoolBalancer(IDBServiceInfo& service_info,
                     const string& pool_name,
-                    const IDBServiceMapper::TOptions& options,
-                    I_DriverContext* driver_ctx = nullptr);
+                    I_DriverContext* driver_ctx = nullptr,
+                    bool is_pooled = true);
 
     TSvrRef GetServer(CDB_Connection** conn, const CDBConnParams* params);
 
@@ -59,7 +76,13 @@ protected:
     void          x_Discard(const void* params, IBalanceable* conn) override;
 
 private:
-    I_DriverContext* m_DriverCtx;
+    static TFlags x_GetFlags(I_DriverContext* driver_ctx, bool is_pooled);
+    void x_ReinitFromCounts(void);
+    
+    CRef<IDBServiceInfo> m_ServiceInfo;
+    string               m_PoolName;
+    I_DriverContext*     m_DriverCtx;
+    bool                 m_IsPooled;
 };
 
 
@@ -70,7 +93,10 @@ TSvrRef CDBPoolBalancer::GetServer(CDB_Connection** conn,
     IBalanceable* balanceable = nullptr;
     auto svr_ref = x_GetServer(params, &balanceable);
     if (conn != nullptr) {
-        *conn = static_cast<CDB_Connection*>(balanceable);
+        *conn = dynamic_cast<CDB_Connection*>(balanceable);
+    }
+    if (conn == nullptr  ||  *conn == nullptr) {
+        delete balanceable;
     }
     return svr_ref;
 }
