@@ -1,7 +1,7 @@
 #ifndef MISC_JSONWRAPP___JSONWRAPP11__HPP
 #define MISC_JSONWRAPP___JSONWRAPP11__HPP
 
-/*  $Id: jsonwrapp11.hpp 619053 2020-10-29 19:09:26Z gouriano $
+/*  $Id: jsonwrapp11.hpp 675766 2023-12-01 14:57:17Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -68,19 +68,28 @@
 
 #include <corelib/ncbistr.hpp>
 #include <corelib/ncbidbg.hpp>
+#include <corelib/ncbi_system.hpp>
 #include <iterator>
 
 #define RAPIDJSON_NOMEMBERITERATORCLASS
+#if defined(_MSC_VER) && !defined(_SILENCE_CXX17_ITERATOR_BASE_CLASS_DEPRECATION_WARNING)
+#define _SILENCE_CXX17_ITERATOR_BASE_CLASS_DEPRECATION_WARNING
+#define UNDEFINE_SILENCE_CXX17_ITERATOR_BASE_CLASS_DEPRECATION_WARNING
+#endif
 #include "rapidjson11/rapidjson.h"
 #include "rapidjson11/document.h"
 #include "rapidjson11/prettywriter.h"
 #include "rapidjson11/filereadstream.h"
 #include "rapidjson11/istreamwrapper.h"
+#include "rapidjson11/istreamasync.h"
 #include "rapidjson11/filewritestream.h"
 #include "rapidjson11/ostreamwrapper.h"
 #include "rapidjson11/error/en.h"
 #include "rapidjson11/schema.h"
-
+#if defined(UNDEFINE_SILENCE_CXX17_ITERATOR_BASE_CLASS_DEPRECATION_WARNING)
+#undef _SILENCE_CXX17_ITERATOR_BASE_CLASS_DEPRECATION_WARNING
+#undef UNDEFINE_SILENCE_CXX17_ITERATOR_BASE_CLASS_DEPRECATION_WARNING
+#endif
 
 BEGIN_NCBI_SCOPE
 
@@ -1015,6 +1024,8 @@ private:
 ///
 /// Serializable, copyable container for JSON data.
 class CJson_Schema;
+using CJson_IStreamBuffer = rapidjson::IStreamWrapper;
+using CJson_IStreamAsyncBuffer = rapidjson::IStreamAsyncWrapper;
 
 class CJson_Document : public CJson_Node
 {
@@ -1039,9 +1050,17 @@ public:
     /// Read JSON data from a UTF8 string
     bool ParseString(const TStringType& v);
 
+    /// Read JSON data from a mutable char buffer
+    /// NOTE: as a result, the buffer may change
+    bool ParseMutableString(TCharType* v);
+
     /// Read JSON data from a stream
     bool Read(std::istream& in);
 
+    /// Read JSON data from a stream buffer
+    bool Read(CJson_IStreamBuffer& in);
+    bool Read(CJson_IStreamAsyncBuffer& in);
+
     /// Read JSON data from a file
     bool Read(const std::string& filename) {
         std::ifstream in(filename.c_str());
@@ -1054,11 +1073,19 @@ public:
     /// Read JSON data from a file, validating against schema
     bool Read(const std::string& filename, CJson_Schema& schema) {
         std::ifstream in(filename.c_str());
-	return Read(in, schema);
+        return Read(in, schema);
     }
 
+    /// Read JSON data from a file using character buffer
+    /// This appears to be faster than ifstream
+    bool ReadBuffered(const std::string& filename);
+
     /// Test if the most recent read was successful
-    bool ReadSucceeded(void);
+    bool ReadSucceeded(void) const;
+
+    /// Most recent read produced empty document
+    //  No more data in input stream
+    bool EndOfData(void) const;
 
     /// Get most recent read error
     std::string GetReadError(void) const;
@@ -1088,11 +1115,15 @@ public:
     }
 
     /// Traverse the document contents
-    void Walk(CJson_WalkHandler& walk) const;
+    /// returns TRUE if all the contents was processed
+    bool Walk(CJson_WalkHandler& walk) const;
 
     /// Traverse the JSON data stream contents
-    static void Walk(std::istream& in, CJson_WalkHandler& walk);
+    /// returns TRUE if all the contents was processed
+    static bool Walk(std::istream& in, CJson_WalkHandler& walk);
 
+    /// Check that the JSON document is a valid JSON schema
+    bool IsSchema(void) const;
 
 private:
     _DocImpl m_DocImpl;
@@ -1117,7 +1148,7 @@ public:
     /// Validate JSON data from a file
     bool Validate(const std::string& filename) {
         std::ifstream in(filename.c_str());
-	return Validate(in);
+        return Validate(in);
     }
 
     /// Return result of the most recent validation
@@ -1138,6 +1169,14 @@ private:
     friend class CJson_Document;
 };
 
+/// JSON meta-schema describes JSON schema itself
+/// So, other JSON schemas can be validated against it
+class CJson_MetaSchema : public CJson_Schema
+{
+public:
+    /// Construct JSON meta-schema
+    CJson_MetaSchema(void);
+};
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -1838,6 +1877,27 @@ CJson_Object::insert(const CJson_Node::TKeyType& name, const Uint8& v) {
     rapidjson::Value sv_name(name.c_str(), *a);
     m_Impl->AddMember( sv_name, rapidjson::Value().SetUint64(v).SetValueAllocator(a), *a);
 }
+#if NCBI_INT8_IS_LONG
+template<> inline void
+CJson_Object::insert(const CJson_Node::TKeyType& name, const long long& v) {
+    insert(name, static_cast<Int8>(v));
+}
+template<> inline void
+CJson_Object::insert(const CJson_Node::TKeyType& name,
+                     const unsigned long long& v) {
+    insert(name, static_cast<Uint8>(v));
+}
+#elif SIZEOF_LONG == 8
+template<> inline void
+CJson_Object::insert(const CJson_Node::TKeyType& name, const long& v) {
+    insert(name, static_cast<Int8>(v));
+}
+template<> inline void
+CJson_Object::insert(const CJson_Node::TKeyType& name, const unsigned long& v)
+{
+    insert(name, static_cast<Uint8>(v));
+}
+#endif
 template<> inline void
 CJson_Object::insert(const CJson_Node::TKeyType& name, const float& v) {
     rapidjson::Value::AllocatorType* a = m_Impl->GetValueAllocator();
@@ -2236,7 +2296,7 @@ CJson_WalkHandler::GetCurrentJPath(void) const {
     std::vector<CJson_Node::TKeyType>::const_iterator n = m_name.begin();
     CJson_Node::TKeyType path;
     for ( ++t, ++i, ++n; t != te; ++t, ++i, ++n) {
-        if (*t) {
+        if (*t && !n->empty()) {
             path += JSONWRAPP_TO_NCBIUTF8("/");
             path += JSONWRAPP_TO_NCBIUTF8(*n);
         } else  if (*i != size_t(-1)) {
@@ -2245,7 +2305,7 @@ CJson_WalkHandler::GetCurrentJPath(void) const {
             path += JSONWRAPP_TO_NCBIUTF8("]");
         }
     }
-    return path;
+    return path.empty() ? "/" : path;
 }
 
 inline CJson_Node::TKeyType CJson_WalkHandler::GetCurrentJPointer(void) const {
@@ -2255,14 +2315,15 @@ inline CJson_Node::TKeyType CJson_WalkHandler::GetCurrentJPointer(void) const {
     std::vector<CJson_Node::TKeyType>::const_iterator n = m_name.begin();
     CJson_Node::TKeyType path;
     for ( ++t, ++i, ++n; t != te; ++t, ++i, ++n) {
-        path += JSONWRAPP_TO_NCBIUTF8("/");
-        if (*t) {
+        if (*t && !n->empty()) {
+            path += JSONWRAPP_TO_NCBIUTF8("/");
             path += JSONWRAPP_TO_NCBIUTF8(*n);
         } else  if (*i != size_t(-1)) {
+            path += JSONWRAPP_TO_NCBIUTF8("/");
             path += JSONWRAPP_TO_NCBIUTF8(ncbi::NStr::NumericToString(*i));
         }
     }
-    return path;
+    return path.empty() ? "/" : path;
 }
 
 inline bool CJson_WalkHandler::Read(CJson_Document& doc) {
@@ -2318,12 +2379,26 @@ inline bool CJson_Document::ParseString(const CJson_ConstNode::TStringType& v) {
     return  !m_DocImpl.HasParseError();
 }
 
+inline bool CJson_Document::ParseMutableString(CJson_ConstNode::TCharType* v) {
+    m_DocImpl.ParseInsitu<rapidjson::kParseStopWhenDoneFlag >(v);
+    return  !m_DocImpl.HasParseError();
+}
+
 inline bool CJson_Document::Read(std::istream& in) {
     rapidjson::IStreamWrapper ifs(in);
     m_DocImpl.ParseStream<rapidjson::kParseStopWhenDoneFlag>(ifs);
     return  !m_DocImpl.HasParseError();
 }
 
+inline bool CJson_Document::Read(CJson_IStreamBuffer& in) {
+    m_DocImpl.ParseStream<rapidjson::kParseStopWhenDoneFlag>(in);
+    return  !m_DocImpl.HasParseError();
+}
+inline bool CJson_Document::Read(CJson_IStreamAsyncBuffer& in) {
+    m_DocImpl.ParseStream<rapidjson::kParseStopWhenDoneFlag>(in);
+    return  !m_DocImpl.HasParseError();
+}
+
 inline bool CJson_Document::Read(std::istream& in, CJson_Schema& schema) {
     rapidjson::IStreamWrapper ifs(in);
     rapidjson::SchemaValidatingReader<
@@ -2339,9 +2414,23 @@ inline bool CJson_Document::Read(std::istream& in, CJson_Schema& schema) {
     return !m_DocImpl.HasParseError();
 }
 
-inline bool CJson_Document::ReadSucceeded(void) {
+inline bool CJson_Document::ReadBuffered(const std::string& filename) {
+    FILE *fp = fopen(filename.c_str(), "rt");
+    unsigned long s = CSystemInfo::GetVirtualMemoryAllocationGranularity();
+    std::unique_ptr<char[]> buf(new char[s]);
+    rapidjson::FileReadStream fs(fp, buf.get(), s);
+    m_DocImpl.ParseStream<rapidjson::kParseStopWhenDoneFlag>(fs);
+    fclose(fp);
+    return  !m_DocImpl.HasParseError();
+}
+
+inline bool CJson_Document::ReadSucceeded(void) const {
     return !m_DocImpl.HasParseError();
 }
+inline bool CJson_Document::EndOfData(void) const {
+    return m_DocImpl.GetParseError() == rapidjson::kParseErrorDocumentEmpty;
+}
+
 inline std::string CJson_Document::GetReadError() const {
     return rapidjson::GetParseError_En(m_DocImpl.GetParseError());
 }
@@ -2381,23 +2470,183 @@ inline bool CJson_Document::Write(std::ostream& out, CJson_Schema& schema,
     return res;
 }
 
-inline void CJson_Document::Walk(CJson_WalkHandler& walk) const {
+inline bool CJson_Document::Walk(CJson_WalkHandler& walk) const {
     walk.x_SetSource(0);
-    m_DocImpl.Accept(walk);
+    return m_DocImpl.Accept(walk);
 }
 
-inline void CJson_Document::Walk(std::istream& in,
+inline bool CJson_Document::Walk(std::istream& in,
                                  CJson_WalkHandler& walk) {
     walk.x_SetSource(&in);
     rapidjson::IStreamWrapper ifs(in);
     rapidjson::Reader rdr;
-    rdr.Parse<rapidjson::kParseStopWhenDoneFlag>(ifs,walk);
+    return !rdr.Parse<rapidjson::kParseStopWhenDoneFlag>(ifs,walk).IsError();
+}
+
+inline bool CJson_Document::IsSchema(void) const {
+    return CJson_MetaSchema().Validate(*this);
+}
+
+// --------------------------------------------------------------------------
+// CJson_MetaSchema
+
+inline CJson_MetaSchema::CJson_MetaSchema(void)
+    : CJson_Schema( CJson_Document( R"META(
+{
+    "id": "http://json-schema.org/draft-04/schema#",
+    "$schema": "http://json-schema.org/draft-04/schema#",
+    "description": "Core schema meta-schema",
+    "definitions": {
+        "schemaArray": {
+            "type": "array",
+            "minItems": 1,
+            "items": { "$ref": "#" }
+        },
+        "positiveInteger": {
+            "type": "integer",
+            "minimum": 0
+        },
+        "positiveIntegerDefault0": {
+            "allOf": [ { "$ref": "#/definitions/positiveInteger" }, { "default": 0 } ]
+        },
+        "simpleTypes": {
+            "enum": [ "array", "boolean", "integer", "null", "number", "object", "string" ]
+        },
+        "stringArray": {
+            "type": "array",
+            "items": { "type": "string" },
+            "minItems": 1,
+            "uniqueItems": true
+        }
+    },
+    "type": "object",
+    "properties": {
+        "id": {
+            "type": "string"
+        },
+        "$schema": {
+            "type": "string"
+        },
+        "title": {
+            "type": "string"
+        },
+        "description": {
+            "type": "string"
+        },
+        "default": {},
+        "multipleOf": {
+            "type": "number",
+            "minimum": 0,
+            "exclusiveMinimum": true
+        },
+        "maximum": {
+            "type": "number"
+        },
+        "exclusiveMaximum": {
+            "type": "boolean",
+            "default": false
+        },
+        "minimum": {
+            "type": "number"
+        },
+        "exclusiveMinimum": {
+            "type": "boolean",
+            "default": false
+        },
+        "maxLength": { "$ref": "#/definitions/positiveInteger" },
+        "minLength": { "$ref": "#/definitions/positiveIntegerDefault0" },
+        "pattern": {
+            "type": "string",
+            "format": "regex"
+        },
+        "additionalItems": {
+            "anyOf": [
+                { "type": "boolean" },
+                { "$ref": "#" }
+            ],
+            "default": {}
+        },
+        "items": {
+            "anyOf": [
+                { "$ref": "#" },
+                { "$ref": "#/definitions/schemaArray" }
+            ],
+            "default": {}
+        },
+        "maxItems": { "$ref": "#/definitions/positiveInteger" },
+        "minItems": { "$ref": "#/definitions/positiveIntegerDefault0" },
+        "uniqueItems": {
+            "type": "boolean",
+            "default": false
+        },
+        "maxProperties": { "$ref": "#/definitions/positiveInteger" },
+        "minProperties": { "$ref": "#/definitions/positiveIntegerDefault0" },
+        "required": { "$ref": "#/definitions/stringArray" },
+        "additionalProperties": {
+            "anyOf": [
+                { "type": "boolean" },
+                { "$ref": "#" }
+            ],
+            "default": {}
+        },
+        "definitions": {
+            "type": "object",
+            "additionalProperties": { "$ref": "#" },
+            "default": {}
+        },
+        "properties": {
+            "type": "object",
+            "additionalProperties": { "$ref": "#" },
+            "default": {}
+        },
+        "patternProperties": {
+            "type": "object",
+            "additionalProperties": { "$ref": "#" },
+            "default": {}
+        },
+        "dependencies": {
+            "type": "object",
+            "additionalProperties": {
+                "anyOf": [
+                    { "$ref": "#" },
+                    { "$ref": "#/definitions/stringArray" }
+                ]
+            }
+        },
+        "enum": {
+            "type": "array",
+            "minItems": 1,
+            "uniqueItems": true
+        },
+        "type": {
+            "anyOf": [
+                { "$ref": "#/definitions/simpleTypes" },
+                {
+                    "type": "array",
+                    "items": { "$ref": "#/definitions/simpleTypes" },
+                    "minItems": 1,
+                    "uniqueItems": true
+                }
+            ]
+        },
+        "format": { "type": "string" },
+        "allOf": { "$ref": "#/definitions/schemaArray" },
+        "anyOf": { "$ref": "#/definitions/schemaArray" },
+        "oneOf": { "$ref": "#/definitions/schemaArray" },
+        "not": { "$ref": "#" }
+    },
+    "dependencies": {
+        "exclusiveMaximum": [ "maximum" ],
+        "exclusiveMinimum": [ "minimum" ]
+    },
+    "default": {}
+}
+    )META" )) {
 }
 
 // --------------------------------------------------------------------------
 // CJson_Schema methods
 
-
 inline CJson_Schema::CJson_Schema(const CJson_Document& schema)
     : m_SchemaDocument(schema.m_DocImpl),
       m_SchemaValidator( m_SchemaDocument)
