@@ -1,7 +1,7 @@
 #ifndef SEQ_LOC_MAPPER_BASE__HPP
 #define SEQ_LOC_MAPPER_BASE__HPP
 
-/*  $Id: seq_loc_mapper_base.hpp 572476 2018-10-15 15:17:03Z grichenk $
+/*  $Id: seq_loc_mapper_base.hpp 662934 2023-02-08 19:14:27Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -776,7 +776,7 @@ private:
     void x_InitSpliced(const CSpliced_seg& spliced,
                        const TSynonyms&    to_ids);
     void x_InitSpliced(const CSpliced_seg& spliced, ESplicedRow to_row);
-    void x_InitSparse(const CSparse_seg& sparse, int to_row);
+    void x_InitSparse(const CSparse_seg& sparse, size_t to_row);
 
     void x_IterateExonParts(const CSpliced_exon::TParts& parts,
                             ESplicedRow                  to_row,
