@@ -1,4 +1,4 @@
-/*  $Id: local_taxon.hpp 615586 2020-09-01 17:59:29Z ivanov $
+/*  $Id: local_taxon.hpp 663139 2023-02-13 20:02:01Z mozese2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -49,7 +49,7 @@ END_SCOPE(objects)
 class CLocalTaxon
 {
 public:
-    typedef int TTaxid;
+    typedef TTaxId TTaxid;
     typedef vector<TTaxid> TLineage;
 
     static void AddArguments(CArgDescriptions& arg_desc);
@@ -81,6 +81,7 @@ public:
     short int GetGeneticCode(TTaxid taxid);
     CConstRef<objects::COrg_ref> GetOrgRef(TTaxid taxid);
     void LookupMerge(objects::COrg_ref& org);
+    list<string> GetSynonyms(TTaxId taxid);
 
 public: // lookups other than taxid
 
@@ -88,7 +89,7 @@ public: // lookups other than taxid
 
 private: // data types
     struct STaxidNode;
-    typedef map<int, STaxidNode> TNodes;
+    typedef map<TTaxId, STaxidNode> TNodes;
     typedef map<string, STaxidNode> TScientificNameIndex;
     typedef TNodes::const_iterator TNodeRef;
     typedef TScientificNameIndex::const_iterator TScientificNameRef;
@@ -98,6 +99,7 @@ private: // data types
         TTaxid taxid;
         bool is_valid;
         string scientific_name;
+        list<string> synonyms;
         string rank;
         TNodeRef parent;
         short int genetic_code;
@@ -112,6 +114,7 @@ private: // static
     
 private: // data model
     bool m_db_supports_synonym;
+    bool m_fallback;
     unique_ptr<CSQLITE_Connection> m_SqliteConn;
     unique_ptr<objects::CTaxon1> m_TaxonConn;
     TNodes m_Nodes;
