@@ -1,7 +1,7 @@
 #ifndef NCBI_TAXON1_HPP
 #define NCBI_TAXON1_HPP
 
-/* $Id: taxon1.hpp 584206 2019-04-08 18:49:04Z domrach $
+/* $Id: taxon1.hpp 609816 2020-06-08 15:50:52Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -222,7 +222,7 @@ public:
     // Returns: number of organisms found (negative value on error), 
     // id list appended with found tax ids
     ///
-    TTaxId GetAllTaxIdByName(const string& orgname, TTaxIdList& lIds);
+    int GetAllTaxIdByName(const string& orgname, TTaxIdList& lIds);
 
     //----------------------------------------------
     // Get organism by tax_id
@@ -557,7 +557,7 @@ public:
     ///
     bool GetInheritedPropertyDefines( const string& prop_name,
 				      TInfoList& results_out,
-				      TTaxId subtree_root = 1 );
+				      TTaxId subtree_root = TAX_ID_CONST(1) );
 
     //---------------------------------------------------
     // This function returns the list of "type materials" for the node with taxid given.
