@@ -1,7 +1,7 @@
 #ifndef OBJECTS_MISC_SERIAL___GRID_RPC_CLIENT__HPP
 #define OBJECTS_MISC_SERIAL___GRID_RPC_CLIENT__HPP
 
-/* $Id: gridrpcclient.hpp 593123 2019-09-12 15:00:12Z grichenk $
+/* $Id: gridrpcclient.hpp 666819 2023-05-02 14:14:50Z evgeniev $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -245,6 +245,8 @@ protected:
             return "GZipFile";
         case CCompressStream::eConcatenatedGZipFile:
             return "GZipFile";
+        case CCompressStream::eZstd:
+            return "ZStandard";
         };
         NCBI_THROW(CException, eUnknown, "unexpected compression method");
     }
