@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR___DATA_LOADER__HPP
 #define OBJECTS_OBJMGR___DATA_LOADER__HPP
 
-/*  $Id: data_loader.hpp 610968 2020-06-26 12:55:17Z grichenk $
+/*  $Id: data_loader.hpp 660852 2022-12-28 15:46:44Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -391,6 +391,10 @@ public:
     typedef vector<int> TSequenceStates;
     typedef vector<int> TSequenceHashes;
     typedef vector<bool> THashKnown;
+    typedef map<CSeq_id_Handle, TTSE_LockSet> TTSE_LockSets;
+    typedef vector<vector<CSeq_id_Handle>> TSeqIdSets;
+    typedef vector<CTSE_Lock> TCDD_Locks;
+
     /// Bulk request for accession.version Seq-ids of a set of sequences.
     virtual void GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret);
     /// Bulk request for gis of a set of sequences.
@@ -411,11 +415,11 @@ public:
     /// Bulk request for hashes of a set of sequences.
     virtual void GetSequenceHashes(const TIds& ids, TLoaded& loaded,
                                    TSequenceHashes& ret, THashKnown& known);
+    virtual void GetCDDAnnots(const TSeqIdSets& id_sets, TLoaded& loaded, TCDD_Locks& ret);
 
     // Load multiple seq-ids. Same as GetRecords() for multiple ids
     // with choise set to eBlob. The map should be initialized with
     // the id handles to be loaded.
-    typedef map<CSeq_id_Handle, TTSE_LockSet> TTSE_LockSets;
     virtual void GetBlobs(TTSE_LockSets& tse_sets);
 
     // blob operations
@@ -460,6 +464,7 @@ public:
     virtual double EstimateLoadSeconds(const CTSE_Chunk_Info& chunk, Uint4 bytes) const;
 
     virtual unsigned GetDefaultBlobCacheSizeLimit() const;
+    virtual bool GetTrackSplitSeq() const;
 
 protected:
     /// Register the loader only if the name is not yet
@@ -491,7 +496,7 @@ private:
 
 END_SCOPE(objects)
 
-NCBI_DECLARE_INTERFACE_VERSION(objects::CDataLoader, "xloader", 6, 0, 0);
+NCBI_DECLARE_INTERFACE_VERSION(objects::CDataLoader, "xloader", 8, 0, 0);
 
 template<>
 class CDllResolver_Getter<objects::CDataLoader>
