@@ -1,7 +1,7 @@
 #ifndef OBJMGR_IMPL_SCOPE_IMPL__HPP
 #define OBJMGR_IMPL_SCOPE_IMPL__HPP
 
-/*  $Id: scope_impl.hpp 610968 2020-06-26 12:55:17Z grichenk $
+/*  $Id: scope_impl.hpp 667314 2023-05-12 14:57:38Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -346,6 +346,10 @@ public:
     typedef vector<CBioseq_Handle> TBioseqHandles;
     TBioseqHandles GetBioseqHandles(const TIds& ids);
 
+    typedef vector<CTSE_Handle> TCDD_Entries;
+    TCDD_Entries GetCDDAnnots(const TIds& idhs);
+    TCDD_Entries GetCDDAnnots(const TBioseqHandles& bhs);
+
     // Get a set of accession/version pairs
     void GetAccVers(TIds& ret, const TIds& idhs, TGetFlags flags);
 
@@ -396,7 +400,6 @@ public:
     void GetSequenceHashes(TSequenceHashes& ret,
                            const TIds& idhs, TGetFlags flags);
 
-
 private:
     // constructor/destructor visible from CScope
     CScope_Impl(CObjectManager& objmgr);
@@ -601,7 +604,8 @@ private:
     void x_AddTSESetWithAnnots(TTSE_LockMatchSet& lock,
                                TTSE_MatchSet* save_match,
                                const TTSE_LockMatchSet_DS& add,
-                               CDataSource_ScopeInfo& ds_info);
+                               CDataSource_ScopeInfo& ds_info,
+                               CDataLoader::TProcessedNAs* filter_nas = 0);
     void x_GetTSESetWithOrphanAnnots(TTSE_LockMatchSet& lock,
                                      TTSE_MatchSet* save_match,
                                      const TSeq_idSet& ids,
@@ -614,6 +618,11 @@ private:
     void x_GetTSESetWithBioseqAnnots(TTSE_LockMatchSet& lock,
                                      CBioseq_ScopeInfo& binfo,
                                      const SAnnotSelector* sel);
+    
+    void x_UpdateProcessedNAs(const SAnnotSelector*& sel,
+                              unique_ptr<SAnnotSelector>& sel_copy,
+                              CDataLoader::TProcessedNAs& filter_nas,
+                              CDataLoader::TProcessedNAs& processed_nas);
 
     void x_LockMatchSet(TTSE_LockMatchSet& lock,
                         const TTSE_MatchSet& match);
@@ -640,8 +649,8 @@ private:
 
     IScopeTransaction_Impl* m_Transaction;
 
-    volatile int m_BioseqChangeCounter;
-    volatile int m_AnnotChangeCounter;
+    atomic<int> m_BioseqChangeCounter;
+    atomic<int> m_AnnotChangeCounter;
     bool m_KeepExternalAnnotsForEdit;
 
     friend class CScope;
