@@ -1,7 +1,7 @@
 #ifndef SCOPE_INFO__HPP
 #define SCOPE_INFO__HPP
 
-/*  $Id: scope_info.hpp 598334 2019-12-06 18:14:42Z vasilche $
+/*  $Id: scope_info.hpp 669326 2023-06-27 12:49:05Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -497,8 +497,9 @@ private: // members
     TBioseqById                 m_BioseqById;
     // TSE locking support
     mutable CMutex              m_TSE_LockMutex;
-    mutable CAtomicCounter_WithAutoInit m_TSE_LockCounter;
-    mutable CAtomicCounter_WithAutoInit m_UserLockCounter;
+    mutable atomic<Int8>        m_TSE_LockCounter;
+    mutable atomic<Int8>        m_UserLockCounter;
+    mutable atomic<bool>        m_TSE_LockAssigned;
     mutable CTSE_Lock           m_TSE_Lock;
     // Used by TSE support
     mutable const CTSE_ScopeInfo*   m_UsedByTSE;
@@ -522,10 +523,18 @@ public:
     typedef set<CSeq_id_Handle>                         TSeq_idSet;
     typedef vector< pair<TTSE_ScopeInfo, CSeq_id_Handle> > TTSE_MatchSet;
     struct SAnnotSetCache : public CObject {
-        volatile int m_SearchTimestamp;
+        atomic<int> m_SearchTimestamp;
         TTSE_MatchSet match;
     };
-    typedef SAnnotSelector::TNamedAnnotAccessions       TNASetKey;
+    struct SNASetKey {
+        SAnnotSelector::TNamedAnnotAccessions accessions;
+        int adjust = 0;
+        bool operator<(const SNASetKey& other) const {
+            if (adjust != other.adjust) return adjust < other.adjust;
+            return accessions < other.accessions;
+        }
+    };
+    typedef SNASetKey                                   TNASetKey;
     typedef CInitMutex<SAnnotSetCache>                  TAnnotRefInfo;
     typedef map<TNASetKey, TAnnotRefInfo>               TNAAnnotRefInfo;
     typedef TIndexIds                                   TIds;
@@ -598,7 +607,7 @@ private: // members
     TBlobStateFlags                 m_BlobState;
 
     // Cached information.
-    volatile int m_UnresolvedTimestamp;
+    atomic<int> m_UnresolvedTimestamp;
     // Cache synonyms of bioseq if any.
     // All synonyms share the same CBioseq_ScopeInfo object.
     CInitMutex<CSynonymsSet>        m_SynCache;
@@ -753,7 +762,7 @@ const CTSE_Lock& CTSE_ScopeInfo::GetTSE_Lock(void) const
 inline
 bool CTSE_ScopeInfo::IsUserLocked(void) const
 {
-    return int(m_UserLockCounter.Get()) > 0;
+    return m_UserLockCounter > 0;
 }
 
 
