@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR_IMPL___TSE_INFO__HPP
 #define OBJECTS_OBJMGR_IMPL___TSE_INFO__HPP
 
-/*  $Id: tse_info.hpp 606922 2020-04-28 18:58:25Z ivanov $
+/*  $Id: tse_info.hpp 667643 2023-05-18 18:05:50Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -709,7 +709,7 @@ private:
         eNotInCache,
         eInCache
     };
-    ELoadState              m_LoadState;
+    atomic<ELoadState>      m_LoadState;
     mutable ECacheState     m_CacheState;
     
     typedef list< CRef<CTSE_Info> > TTSE_Cache;
@@ -768,7 +768,7 @@ private:
         CTSE_Lock      m_BaseTSE;
         TObjectCopyMap m_ObjectCopyMap;
     };
-    auto_ptr<SBaseTSE> m_BaseTSE;
+    unique_ptr<SBaseTSE> m_BaseTSE;
 
     CRef<IEditSaver>              m_EditSaver;
     int                           m_InternalBioObjNumber;
