@@ -1,7 +1,7 @@
 #ifndef OBJMGR_TSE_HANDLE__HPP
 #define OBJMGR_TSE_HANDLE__HPP
 
-/*  $Id: tse_handle.hpp 603807 2020-03-17 14:55:09Z ivanov $
+/*  $Id: tse_handle.hpp 669326 2023-06-27 12:49:05Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -410,7 +410,11 @@ public:
         }
     bool HasObject(void) const
         {
-            return m_ObjectInfo.NotNull();
+            return m_ObjectInfoAssigned;
+        }
+    bool HasObject(const CTSE_Info_Object& info) const
+        {
+            return HasObject() && &GetObjectInfo_Base() == &info;
         }
 
     typedef CSeq_id_Handle TIndexId;
@@ -447,6 +451,7 @@ public:
 
     const CTSE_Info_Object& GetObjectInfo_Base(void) const
         {
+            _ASSERT(HasObject());
             return reinterpret_cast<const CTSE_Info_Object&>(*m_ObjectInfo);
         }
 
@@ -563,11 +568,11 @@ protected:
 
     void AddInfoLock(void)
         {
-            m_LockCounter.Add(1);
+            ++m_LockCounter;
         }
     void RemoveInfoLock(void)
         {
-            if ( m_LockCounter.Add(-1) <= 0 ) {
+            if ( --m_LockCounter <= 0 ) {
                 x_ResetTSE_Lock();
             }
         }
@@ -575,11 +580,13 @@ protected:
 private: // data members
 
     CTSE_ScopeInfo*         m_TSE_ScopeInfo; // null if object is removed.
-    CAtomicCounter_WithAutoInit m_LockCounter; // counts all referencing handles.
+    atomic<Uint8>           m_LockCounter; // counts all referencing handles.
     // The following members are not null when handle is locked (counter > 0)
     // and not removed.
     CTSE_Handle             m_TSE_Handle; // locks TSE from releasing.
     CConstRef<CObject>      m_ObjectInfo; // current object info.
+    atomic<bool>            m_TSE_HandleAssigned;
+    atomic<bool>            m_ObjectInfoAssigned;
     CRef<CObject>           m_DetachedInfo;
 
 private: // to prevent copying
