@@ -91,7 +91,8 @@ public:
         fDevMode           = 1 <<  5, ///< Development mode for testing new features.
         fShowModifiers     = 1 <<  6, ///< Show key-value pair modifiers (e.g. "[organism=Homo sapiens]")
         fUseAutoDef        = 1 <<  7, ///< Run auto-def for nucleotides if user object is present
-        fFastaFormat       = 1 <<  8  ///< Generate FASTA defline
+        fFastaFormat       = 1 <<  8, ///< Generate FASTA defline
+        fDoNotUseAutoDef   = 1 <<  9  ///< Disable internal call to  auto-def
     };
     typedef int TUserFlags; ///< Binary "OR" of EUserFlags
 
@@ -211,7 +212,8 @@ private:
     void x_SetTitleFromMap (void);
 
     void x_SetPrefix (
-        string& prefix
+        string& prefix,
+        const CBioseq_Handle& bsh
     );
     void x_SetSuffix (
         string& suffix,
@@ -335,6 +337,7 @@ private:
     /// orgmod fields
     CTempString m_Breed;
     CTempString m_Cultivar;
+    CTempString m_SpecimenVoucher;
     CTempString m_Isolate;
     CTempString m_Strain;
     CTempString m_Substrain;
@@ -343,6 +346,8 @@ private:
     /// user object fields
     bool m_IsUnverified;
     CTempString m_UnverifiedPrefix;
+    bool m_IsUnreviewed;
+    CTempString m_UnreviewedPrefix;
     CTempString m_TargetedLocus;
 
     /// comment fields
