@@ -1,7 +1,7 @@
 #ifndef SEQUENCE__HPP
 #define SEQUENCE__HPP
 
-/*  $Id: sequence.hpp 626747 2021-03-03 18:46:39Z ivanov $
+/*  $Id: sequence.hpp 668023 2023-05-24 19:16:13Z foleyjp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -71,6 +71,7 @@ class CSeq_entry_handle;
 class CGenetic_code;
 class CMolInfo;
 class CSeq_gap;
+class COrgMod;
 
 BEGIN_SCOPE(sequence)
 
@@ -361,7 +362,7 @@ public:
 
     virtual void setUpFeatureIterator ( 
         CBioseq_Handle &bioseq_handle,
-        auto_ptr<CFeat_CI> &feat_ci,
+        unique_ptr<CFeat_CI> &feat_ci,
         TSeqPos circular_length ,
         CRange<TSeqPos> &range,
         const CSeq_loc& loc,
@@ -715,6 +716,30 @@ CBioseq_Handle GetBioseqFromSeqLoc(const CSeq_loc& loc, CScope& scope,
 NCBI_XOBJUTIL_EXPORT
 string GetProteinName(const CBioseq_Handle& seq);
 
+NCBI_XOBJUTIL_EXPORT
+CConstRef<CSeq_feat> GetSourceFeatForProduct(const CBioseq_Handle& bsh);
+
+NCBI_XOBJUTIL_EXPORT
+TTaxId GetTaxIdForProduct(const CBioseq_Handle& bsh);
+
+NCBI_XOBJUTIL_EXPORT
+void GetOrg_refForProduct(const CBioseq_Handle& bsh, const COrg_ref*);
+
+/// Find an Org-ref for the given Bioseq:
+///  If it's a protein then look on the source feature of the product.
+///  Otherwise find a source descriptor for the sequence.
+///  Otherwise, try to find a source feature for the sequence.
+///  Return nullptr if we still turn up empty.
+NCBI_XOBJUTIL_EXPORT
+const COrg_ref* GetOrg_refForBioseq(const CBioseq_Handle& bsh);
+
+/// Find a BioSource for the given Bioseq:
+///  If it's a protein then look for the source feature of the product.
+///  Otherwise find a source descriptor for the sequence.
+///  Otherwise, try to find a source feature for the sequence.
+///  Return nullptr if we still turn up empty.
+NCBI_XOBJUTIL_EXPORT
+const CBioSource* GetBioSourceForBioseq(const CBioseq_Handle& bsh);
 
 /* @} */
 
@@ -767,7 +792,8 @@ public:
         eInstantiateGaps = fInstantiateGaps,
         fUseAutoDef         = 1 << 17,  ///< Disregard original ID when constructing defline
         fBaseFirstUnused    = 1 << 18, ///< first avalailabe for derived classes
-        fDoNotUseAutoDef    = 1 << 19
+        fDoNotUseAutoDef    = 1 << 19,
+        fShowGnlAndAcc      = 1 << 20, ///< Show general id and accession in the defline
     };
     typedef long TFlags; ///< binary OR of EFlags
 
@@ -866,7 +892,7 @@ public:
 
 protected:
     CNcbiOstream&       m_Out;
-    auto_ptr<sequence::CDeflineGenerator> m_Gen;
+    unique_ptr<sequence::CDeflineGenerator> m_Gen;
 
     virtual void x_WriteSeqIds    ( const CBioseq& bioseq,
                                     const CSeq_loc* location);
