@@ -1,4 +1,4 @@
-/*  $Id: align_format_util.hpp 620577 2020-11-24 14:41:21Z zaretska $
+/*  $Id: align_format_util.hpp 683948 2024-06-10 12:11:11Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -48,7 +48,9 @@
 #ifdef _MSC_VER
 #define strcasecmp _stricmp
 #define strdup _strdup
+#  if _MSC_VER < 1900
 #define snprintf _snprintf
+#  endif
 #endif
 
 /**setting up scope*/
@@ -103,6 +105,7 @@ static const string kUnigeneDispl =  "<div><@lnk@>-<span class=\"rlLink\">cluste
 ///structure
 // .ncbirc alias: STRUCTURE_URLhttps://www.ncbi.nlm.nih.gov/Structure/pdb/1AJ9 
 static const char kStructureUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/Structure/icn3d/full.html?from=blast&blast_rep_id=<@label@>&query_id=<@queryID@>&command=view+annotations;set+annotation+cdd;set+annotation+site;set+view+detailed+view;select+chain+<@label@>;show+selection&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+static const char kStructureAlphaFoldUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/Structure/icn3d/full.html?from=blast&blast_rep_id=<@label@>&query_id=<@queryID@>&command=view+annotations;set+annotation+cdd;set+annotation+site;set+view+detailed+view;select+chain+!A;show+selection&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
 
 
 //substitues <@lnk_displ@>
@@ -808,7 +811,7 @@ public:
     ///@param out: ostream to add white space
     ///@param number: the number of white spaces desired
     ///
-    static void AddSpace(CNcbiOstream& out, int number);
+    static void AddSpace(CNcbiOstream& out, size_t number);
 
     ///Return ID for GNL label
     ///@param dtg: dbtag to build label from
@@ -848,7 +851,7 @@ public:
     ///@return actual number of subject sequences
     static void PruneSeqalign(const objects::CSeq_align_set& source_aln, 
                               objects::CSeq_align_set& new_aln,
-                              unsigned int num = kDfltArgNumAlignments);
+                              unsigned int num = static_cast<unsigned int>(kDfltArgNumAlignments));
 
     ///Calculate number of subject sequnces in alignment limitted by num
     ///@param source_aln: the original alnset    
@@ -1372,11 +1375,11 @@ public:
 
     ///Calculate the number of spaces and add them to paramVal
     ///@param string: input parameter value
-    ///@param string: max length for the string that holds parameter
+    ///@param size_t: max length for the string that holds parameter
     ///@param int: additional fomatting after adding spaces
     ///@param  string: the position of spaces and additional formatting
     ///@return:string containing paramVal and spaces place appropriately
-    static string AddSpaces(string paramVal, unsigned int maxParamLength, int spacesFormatFlag = eSpacePosToCenter);
+    static string AddSpaces(string paramVal, size_t maxParamLength, int spacesFormatFlag = eSpacePosToCenter);
 
 
     static string GetProtocol(void);
@@ -1542,16 +1545,10 @@ public:
     ///@return: bool indicating if accession is WGS
     static bool IsWGSPattern(string &wgsAccession);
 
-    ///Get Gene symobol for gi
-    ///@param  giForGeneLookup: gi
-    ///@return: string gene symbol
-    static string  GetGeneInfo(TGi giForGeneLookup);
-    
-
     static unique_ptr<CNcbiRegistry> m_Reg;
     static string m_Protocol;
     static bool   m_geturl_debug_flag;
-    static auto_ptr<CGeneInfoFileReader> m_GeneInfoReader;
+    
 
     /// Calculate the uniq subject query coverage range (blastn only)
     static int GetUniqSeqCoverage(objects::CSeq_align_set & alnset);
