@@ -1,4 +1,4 @@
-/*  $Id: showalign.hpp 610973 2020-06-26 12:58:47Z grichenk $
+/*  $Id: showalign.hpp 683946 2024-06-10 12:10:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -492,6 +492,7 @@ protected:
         string feature_string;
         list<TSeqPos> feature_start;
         CRange < TSignedSeqPos > aln_range;
+        int genetic_code;                //genetice code used to this feature if applicable   
     };
     typedef list< CRef<SAlnFeatureInfo> > TSAlnFeatureInfoList;
     
@@ -651,7 +652,7 @@ protected:
     CCgiContext* m_Ctx;
     SAlignTemplates *m_AlignTemplates;    
     /// Gene info reader object, reads Gene info entries from files.
-    auto_ptr<CGeneInfoFileReader> m_GeneInfoReader;
+    unique_ptr<CGeneInfoFileReader> m_GeneInfoReader;
 
     /// Current alignment index (added to the linkout and entrez URL's)
     mutable int m_cur_align;
@@ -744,6 +745,7 @@ protected:
     
     ///get feature info
     ///@param feature: where feature info to be filled
+    ///@param custom_genetic_code: use this code (set to -1 if you don't want to use this code)
     ///@param scope: scope to fectch sequence
     ///@param choice: which feature to get
     ///@param row: current row number
@@ -752,7 +754,9 @@ protected:
     ///@param feat_seq_strand: strand to be filled corresponding to feat_seq_range
     ///@param fill_feat_range: to fill feat_seq_range?
     ///
-    void x_GetFeatureInfo(TSAlnFeatureInfoList& feature, objects::CScope & scope,
+    void x_GetFeatureInfo(TSAlnFeatureInfoList& feature, 
+                          int custom_genetic_code,
+                          objects::CScope & scope,
                           objects::CSeqFeatData::E_Choice choice, int row,
                           string& sequence,
                           list<list<CRange<TSeqPos> > >& feat_seq_range,
@@ -814,7 +818,7 @@ protected:
     void x_SetFeatureInfo(CRef<SAlnFeatureInfo> feat_info, const objects::CSeq_loc& seqloc,
                           int aln_from, int aln_to, int aln_stop,
                           char pattern_char,  string pattern_id,
-                          string& alternative_feat_str) const;
+                          string& alternative_feat_str, int genetic_code) const;
 
     ///get insert information
     ///@param insert_list: list to be filled
@@ -829,11 +833,7 @@ protected:
                       objects::CAlnMap::TSeqPosList& insert_length, 
                       int line_aln_stop);
 
-    ///check if Gene info is enabled and a Gene link is present for a hit
-    ///@param aln_vec_info: alnvec list
-    ///
-    bool x_IsGeneInfoAvailable(SAlnInfo* aln_vec_info);
-
+    
     ///get the URL of the Gene info link.
     ///@param gene_id: gene id to link to.
     ///@return: fully formatted URL of the Gene info link.
@@ -892,9 +892,6 @@ protected:
                          CNcbiOstream& out);
 
     CRef<objects::CAlnVec> x_GetAlnVecForSeqalign(const objects::CSeq_align& align); 
-    ///Display Gene Info
-    ///
-    string x_DisplayGeneInfo(const objects::CBioseq_Handle& bsp_handle,SAlnInfo* aln_vec_info);
     
     ///Dipslay Bl2seq TBLASTX link
     ///
