@@ -1,4 +1,4 @@
-/* $Id: tabular.hpp 616915 2020-09-22 19:19:18Z jianye $ 
+/* $Id: tabular.hpp 673788 2023-10-10 17:45:20Z ivanov $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -567,7 +567,8 @@ public:
         x_ResetIgFields();
     };
 
-    void PrintHeader(const string& program, 
+    void PrintHeader(const CConstRef<blast::CIgBlastOptions>& ig_opts,
+                     const string& program, 
                      const objects::CBioseq& bioseq, 
                      const string& dbname, 
                      const string& domain_sys,
@@ -596,7 +597,7 @@ public:
     virtual void Print(void);
 
     /// Print domain information
-    void PrintMasterAlign(const string& header = "# ") const;
+    void PrintMasterAlign(const CConstRef<blast::CIgBlastOptions>& ig_opts, const string& header = "# ") const;
     
     void SetAirrFormatData(CScope& scope,
                            const CRef<blast::CIgAnnotation> &annot,
@@ -619,7 +620,7 @@ public:
                                 const CConstRef<blast::CIgBlastOptions>& ig_opts);
 
     /// Print Html style summary
-    void PrintHtmlSummary() const;
+    void PrintHtmlSummary(const CConstRef<blast::CIgBlastOptions>& ig_opts) const;
 
     /// Set out-of-frame information                                        
     void SetFrame(const string &frame = "N/A") { 
@@ -661,16 +662,24 @@ public:
         m_JGene.Set(id, s,e);
     }
 
+    /// Set gene info
+    void SetCGene(const string &id, int s, int e) {
+        m_CGene.Set(id, s,e);
+    }
+
     /// One method to set all annotation information
     void SetIgAnnotation(const CRef<blast::CIgAnnotation> &annot, 
                          const CConstRef<blast::CIgBlastOptions> &ig_opts,
                          CConstRef<CSeq_align_set>& align_result,
                          CScope& scope);
+    /// method to set cdr3 and fwr4 annotation information
+    void SetIgCDR3FWR4Annotation(const CRef<blast::CIgAnnotation> &annot);
 
     ///Getter
     void GetIgInfo (string& v, 
                     string& d, 
                     string& j,
+                    string& c,
                     string& master_chain_to_show,
                     string& cdr3_nuc,
                     string& cdr3_aa,
@@ -678,6 +687,7 @@ public:
         v = m_VGene.sid;
         d = m_DGene.sid;  
         j = m_JGene.sid;
+        c = m_CGene.sid;
         master_chain_to_show = m_MasterChainTypeToShow;
         cdr3_nuc = m_Cdr3Seq;
         cdr3_aa = m_Cdr3SeqTrans;
@@ -711,6 +721,7 @@ private:
     SIgGene m_VGene;
     SIgGene m_DGene;
     SIgGene m_JGene;
+    SIgGene m_CGene;
     vector<SIgDomain *> m_IgDomains; 
 
     //index 0-2, not currently being used
@@ -749,6 +760,7 @@ private:
     int m_VAlignStart;
     int m_QueryVAlignEnd;
     CRef<CSeq_align> m_TopAlign_J; 
+    CRef<CSeq_align> m_TopAlign_C;
     map<string, string> m_AirrData;
     int m_QueryAlignSeqEnd;
 };
