@@ -1,7 +1,7 @@
 #ifndef CORELIB__SEQDB__SEQDBGENERAL_HPP
 #define CORELIB__SEQDB__SEQDBGENERAL_HPP
 
-/*  $Id: seqdbgeneral.hpp 631511 2021-05-19 13:47:54Z ivanov $
+/*  $Id: seqdbgeneral.hpp 683929 2024-06-10 12:07:52Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1279,7 +1279,7 @@ int SeqDB_VectorAssign(const T & data, vector<U> & v)
         i++;
     }
     
-    return i;
+    return static_cast<int>(i);
 }
 
 END_NCBI_SCOPE
