@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBVOL_HPP
 
-/*  $Id: seqdbvol.hpp 631537 2021-05-19 13:50:49Z ivanov $
+/*  $Id: seqdbvol.hpp 679614 2024-02-28 18:46:23Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -884,6 +884,14 @@ public:
         m_MemBit = mbit;
     }
 
+    void SetOidMaskType(int oid_masks) const {
+        m_OidMaskType = oid_masks;
+    }
+
+    int GetOidMaskType() const {
+        return m_OidMaskType;
+    }
+
 private:
     void x_StringToOids(const string         & acc,
                         ESeqDBIdType           id_type,
@@ -937,9 +945,9 @@ private:
                        bool          & have_vol) const
     {
         if (! have_user) {
-            if (m_UserGiList.NotEmpty() && m_UserGiList->GetNumTaxIds() == 0 && m_UserGiList->GetNumPigs() == 0 ) {
+            if (m_UserGiList.NotEmpty() && m_UserGiList->HasIdFilters()) {
                 have_user |= x_ListIncludesId(*m_UserGiList, id);
-            } else if (m_NegativeList.NotEmpty() && m_NegativeList->GetNumTaxIds() == 0 && m_NegativeList->GetNumPigs() == 0 ) {
+            } else if (m_NegativeList.NotEmpty() && m_NegativeList->HasIdFilters() ) {
                 have_user |= x_ListIncludesId(*m_NegativeList, id);
             } else {
                 have_user = true;
@@ -1399,6 +1407,8 @@ private:
     /// The filtering MEMB_BIT
     mutable int m_MemBit;
 
+    mutable int m_OidMaskType;
+
     /// Cached/ranged sequence info type.
     typedef map<int, CRef<CSeqDBRangeList> > TRangeCache;
 
