@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 
-/*  $Id: seqdb.hpp 610974 2020-06-26 12:59:33Z grichenk $
+/*  $Id: seqdb.hpp 654652 2022-08-18 16:44:30Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,6 +50,8 @@
 #include <objects/seq/Bioseq.hpp>
 #include <objects/seq/Seq_data.hpp>
 #include <objects/seqloc/Seq_id.hpp>
+#include <objects/blastdb/Blast_db_metadata.hpp>
+#include <objects/blastdb/Blast_db_mask_info.hpp>
 #include <util/sequtil/sequtil.hpp>
 #include <util/range.hpp>
 #include <set>
@@ -1506,6 +1508,22 @@ public:
     /// @param tax_ids	return taxonomy ids in db
     void GetDBTaxIds(set<TTaxId> & tax_ids) const;
 
+    void GetTaxIdsForOids(const vector<blastdb::TOid> & oids, set<TTaxId> & tax_ids) const;
+
+    CRef<CBlast_db_metadata> GetDBMetaData(string user_path = kEmptyStr);
+
+
+    /// Get all tax ids for an accessions
+    /// @param accs input accession
+    /// @param taxids  taxids for accession
+    void GetTaxIdsForAccession(const string & accs, vector<TTaxId> & taxids);
+
+
+    /// Get all tax ids for a seq id
+    /// @param seq_id input seq id
+    /// @param taxids  taxids for accession
+    void GetTaxIdsForSeqId(const CSeq_id & seq_id, vector<TTaxId> & taxids);
+
 protected:
     /// Implementation details are hidden.  (See seqdbimpl.hpp).
     class CSeqDBImpl * m_Impl;
@@ -1515,6 +1533,8 @@ protected:
     /// This version of the constructor is used as an extension by the
     /// 'expert' interface in seqdbexpert.hpp.
     CSeqDB();
+
+    void x_GetDBFilesMetaData(Int8 & disk_bytes, Int8 & cached_bytes, vector<string> & db_files, const string & user_path) const;
 };
 
 /// Structure to define basic information to initialize a BLAST DB
