@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
 
-/*  $Id: seqdbcommon.hpp 610974 2020-06-26 12:59:33Z grichenk $
+/*  $Id: seqdbcommon.hpp 683533 2024-05-30 12:54:51Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -87,7 +87,10 @@ public:
         eVersionErr,
 
         /// No Tax Id Found
-        eTaxidErr
+        eTaxidErr,
+
+		/// Too many open files
+		eOpenFileErr
     };
 
     /// Get a message describing the situation leading to the throw.
@@ -96,7 +99,10 @@ public:
         switch ( GetErrCode() ) {
         case eArgErr:  return "eArgErr";
         case eFileErr: return "eFileErr";
+        case eMemErr: return "eMemErr";
         case eVersionErr: return "eVersionErr";
+        case eTaxidErr: return "eTaxidErr";
+        case eOpenFileErr: return "eOpenFileErr";
         default:       return CException::GetErrCodeString();
         }
     }
@@ -150,6 +156,11 @@ struct NCBI_XOBJREAD_EXPORT SBlastSeqIdListInfo {
 		string db_vol_names;
 };
 
+enum EOidMaskType{
+	fNone = 0x0,
+ 	fExcludeModel = 0x01
+ };
+
 /// CSeqDBGiList
 ///
 /// This class defines an interface to a list of GI,OID pairs.  It is
@@ -373,10 +384,14 @@ public:
         return (int) m_PigsOids.size();
     }
 
+    Uint8 GetMaskOpts() const {
+    	return m_MaskOpts;
+    }
+
     /// Return false if there are elements present.
     bool Empty() const
     {
-        return ! (GetNumGis() || GetNumSis() || GetNumTis() || GetNumTaxIds() || GetNumPigs());
+        return ! (GetNumGis() || GetNumSis() || GetNumTis() || GetNumTaxIds() || GetNumPigs() || GetMaskOpts());
     }
 
     /// Return true if there are elements present.
@@ -385,6 +400,10 @@ public:
         return ! Empty();
     }
 
+    bool HasIdFilters() const
+    {
+        return (GetNumGis() || GetNumSis() || GetNumTis());
+    }
     /// Specify the correct OID for a GI.
     ///
     /// When SeqDB translates a GI into an OID, this method is called
@@ -568,6 +587,11 @@ public:
         	m_SisOids.push_back(*itr);
         }
     }
+
+    void SetMaskOpts(EOidMaskType mask_opts) {
+    	m_MaskOpts|=mask_opts;
+    }
+
 protected:
     /// Indicates the current sort order, if any, of this container.
     ESortOrder m_CurrentOrder;
@@ -587,6 +611,8 @@ protected:
 
     SBlastSeqIdListInfo m_ListInfo;
 
+    Uint8 m_MaskOpts;
+
 private:
     // The following disabled methods are reasonable things to do in
     // some cases.  But I suspect they are more likely to happen
@@ -942,6 +968,11 @@ public:
         return size;
     }
 
+    bool HasIdFilters() const
+    {
+        return (GetNumGis() || GetNumSis() || GetNumTis());
+    }
+
     /// Return false if there are elements present.
     bool Empty() const
     {
@@ -2020,6 +2051,9 @@ NCBI_XOBJREAD_EXPORT
 void SeqDB_GetLMDBFileExtensions(bool db_is_protein,
 		                         vector<string>& extn);
 
+NCBI_XOBJREAD_EXPORT
+void SeqDB_GetMetadataFileExtension(bool db_is_protein, string & extn);
+
 /// Determine if id is srting id
 /// @param id input id for check
 /// @return
@@ -2027,10 +2061,19 @@ void SeqDB_GetLMDBFileExtensions(bool db_is_protein,
 NCBI_XOBJREAD_EXPORT
 bool IsStringId(const CSeq_id & id);
 
+
 /// Return ID string as stored in lmdb
 NCBI_XOBJREAD_EXPORT
 string GetBlastSeqIdString(const CSeq_id & seqid, bool version);
 
+
+NCBI_XOBJREAD_EXPORT
+const string SeqDB_GetOidMaskFileExt(bool db_is_protein, EOidMaskType t);
+
+
+
+
+
 END_NCBI_SCOPE
 
 #endif // OBJTOOLS_BLAST_SEQDB_READER___SEQDBCOMMON__HPP
