@@ -1,4 +1,4 @@
-/*  $Id: build_db.hpp 631508 2021-05-19 13:47:28Z ivanov $
+/*  $Id: build_db.hpp 675078 2023-11-14 13:36:47Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -153,6 +153,9 @@ public:
     /// @param long_seqids if true, requires long sequence ids
     /// (database|accession) when parsing fasta sequences [in]
     /// @param dbver version of BLAST database to generate [in]
+    /// @param scan_bioseq_4_cfastareader_usrobj [in]
+    ///   If true, scan the Bioseq objects for a CFastaReader-created User-object
+    ///   containing a defline
     CBuildDatabase(const string         & dbname,
                    const string         & title,
                    bool                   is_protein,
@@ -161,7 +164,9 @@ public:
                    ostream              * logfile,
                    bool                   long_seqids = false,
                    EBlastDbVersion        dbver = eBDB_Version4,
-                   bool                   limit_defline = false);
+                   bool                   limit_defline = false,
+                   Uint8                  oid_masks = EOidMaskType::fNone,
+                   bool scan_bioseq_4_cfastareader_usrobj = true);
 
     // Note -- should deprecate (or just remove) the following one:
     // - sparse does nothing
@@ -184,6 +189,9 @@ public:
     /// @param indexing index fields to add to database. [in]
     /// @param long_seqids if true, requires long sequence ids
     /// (database|accession) when parsing fasta sequences [in]
+    /// @param scan_bioseq_4_cfastareader_usrobj [in]
+    ///   If true, scan the Bioseq objects for a CFastaReader-created User-object
+    ///   containing a defline
     CBuildDatabase(const string         & dbname,
                    const string         & title,
                    bool                   is_protein,
@@ -193,7 +201,9 @@ public:
                    ostream              * logfil,
                    bool                   long_seqids = false,
                    EBlastDbVersion dbver = eBDB_Version4,
-                   bool                   limit_defline = false);
+                   bool                   limit_defline = false,
+                   Uint8                  oid_masks = EOidMaskType::fNone,
+                   bool scan_bioseq_4_cfastareader_usrobj = true);
 
     ~CBuildDatabase();
 
@@ -452,6 +462,7 @@ public:
     /// @param dbname output blast db name (with path)
     static void CreateDirectories(const string& dbname);
 
+    string GetOutputDbName() const { return m_OutputDbName; }
 private:
     /// Get a scope for remote loading of objects.
     objects::CScope & x_GetScope();
@@ -654,6 +665,10 @@ private:
 
     /// If set to true, skip GIs with value > 0x7FFFFFFF
     bool m_SkipLargeGis;
+
+    string m_OutputDbName;
+
+    bool m_ScanBioseq4CFastaReaderUsrObjct;
 };
 
 END_NCBI_SCOPE
