@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_WRITER___WRITEDB__HPP
 #define OBJTOOLS_BLAST_SEQDB_WRITER___WRITEDB__HPP
 
-/*  $Id: writedb.hpp 588812 2019-07-01 12:29:10Z fongah2 $
+/*  $Id: writedb.hpp 675078 2023-11-14 13:36:47Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -152,6 +152,9 @@ public:
     ///   If true, generate GI-based mask files [in]
     /// @param dbver
     ///   version of BLAST database to generate [in]
+    /// @param scan_bioseq_4_cfastareader_usrobj [in]
+    ///   If true, scan the Bioseq objects for a CFastaReader-created User-object
+    ///   containing a defline
     CWriteDB(const string & dbname,
              ESeqType       seqtype,
              const string & title,
@@ -160,7 +163,9 @@ public:
              bool           long_ids = false,
              bool           use_gi_mask = false,
              EBlastDbVersion    dbver = eBDB_Version4,
-             bool			limit_defline = false);
+             bool			limit_defline = false,
+             Uint8          oid_masks = EOidMaskType::fNone,
+             bool           scan_bioseq_4_cfastareader_usrobj = false);
 
     /// Destructor.
     ///
@@ -355,10 +360,14 @@ public:
     /// @param bs The bioseq from which to extract a defline set. [in]
     /// @param parse_ids If seqid should be parsed [in]
     /// @param long_ids It true, use long sequence ids (database|accession) [in]
+    /// @param scan_bioseq_4_cfastareader_usrobj [in]
+    ///   If true, scan the Bioseq objects for a CFastaReader-created User-object
+    ///   containing a defline
     /// @return A set of deflines for this CBioseq.
     static CRef<CBlast_def_line_set>
     ExtractBioseqDeflines(const CBioseq & bs, bool parse_ids=true,
-                          bool long_ids=false);
+                          bool long_ids=false,
+                          bool scan_bioseq_4_cfastareader_usrobj=false);
 
     /// Set letters that should not be used in sequences.
     ///
@@ -696,6 +705,14 @@ void CWriteDB_ConsolidateAliasFiles(const list<string>& alias_files,
 NCBI_XOBJWRITE_EXPORT
 void CWriteDB_ConsolidateAliasFiles(bool delete_source_alias_files = false);
 
+
+NCBI_XOBJWRITE_EXPORT
+void CWriteDB_CreateOidMaskDB(const string& input_db,
+							  const string & output_db,
+                              CWriteDB::ESeqType seq_type,
+                              int oid_mask_type,
+                              const string & title = string());
+
 END_NCBI_SCOPE
 
 #endif // OBJTOOLS_BLAST_SEQDB_WRITER___WRITEDB__HPP
