@@ -1,7 +1,7 @@
 #ifndef CLEANUP___CLEANUP__HPP
 #define CLEANUP___CLEANUP__HPP
 
-/*  $Id: cleanup.hpp 632626 2021-06-03 17:38:42Z ivanov $
+/*  $Id: cleanup.hpp 681442 2024-04-09 14:53:56Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -75,7 +75,9 @@ public:
         eClean_NoNcbiUserObjects = 0x4,
         eClean_SyncGenCodes      = 0x8,
         eClean_NoProteinTitles   = 0x10,
-        eClean_KeepTopSet        = 0x20
+        eClean_KeepTopSet        = 0x20,
+        eClean_KeepSingleSeqSet  = 0x40,
+        eClean_InHugeSeqSet      = 0x80,
     };
 
     enum EScopeOptions {
@@ -84,49 +86,52 @@ public:
     };
 
     // Construtor / Destructor
-    CCleanup(CScope* scope = NULL, EScopeOptions scope_handling = eScope_Copy);
+    CCleanup(CScope* scope = nullptr, EScopeOptions scope_handling = eScope_Copy);
+    CCleanup(const CCleanup&) = delete;
+    CCleanup& operator=(const CCleanup&) = delete;
+
     ~CCleanup();
 
     void SetScope(CScope* scope);
 
+    using TChanges = CConstRef<CCleanupChange>;
+
     // BASIC CLEANUP
 
-    CConstRef<CCleanupChange> BasicCleanup(CSeq_entry& se,  Uint4 options = 0);
+    TChanges BasicCleanup(CSeq_entry& se,  Uint4 options = 0);
     /// Cleanup a Seq-submit.
-    CConstRef<CCleanupChange> BasicCleanup(CSeq_submit& ss,  Uint4 options = 0);
-    /// Cleanup a Bioseq.
-    CConstRef<CCleanupChange> BasicCleanup(CBioseq& bs,     Uint4 ooptions = 0);
+    TChanges BasicCleanup(CSeq_submit& ss,  Uint4 options = 0);
     /// Cleanup a Bioseq_set.
-    CConstRef<CCleanupChange> BasicCleanup(CBioseq_set& bss, Uint4 options = 0);
+    TChanges BasicCleanup(CBioseq_set& bss, Uint4 options = 0);
     /// Cleanup a Seq-Annot.
-    CConstRef<CCleanupChange> BasicCleanup(CSeq_annot& sa,  Uint4 options = 0);
+    TChanges BasicCleanup(CSeq_annot& sa,  Uint4 options = 0);
     /// Cleanup a Seq-feat.
-    CConstRef<CCleanupChange> BasicCleanup(CSeq_feat& sf,   Uint4 options = 0);
+    TChanges BasicCleanup(CSeq_feat& sf,   Uint4 options = 0);
     /// Cleanup a BioSource.
-    CConstRef<CCleanupChange> BasicCleanup(CBioSource& src,   Uint4 options = 0);
+    TChanges BasicCleanup(CBioSource& src,   Uint4 options = 0);
     // Cleanup a Submit-block
-    CConstRef<CCleanupChange> BasicCleanup(CSubmit_block& block, Uint4 options = 0);
+    TChanges BasicCleanup(CSubmit_block& block, Uint4 options = 0);
     // Cleanup descriptors
-    CConstRef<CCleanupChange> BasicCleanup(CSeqdesc& desc, Uint4 options = 0);
-    CConstRef<CCleanupChange> BasicCleanup(CSeq_descr & desc, Uint4 options = 0);
+    TChanges BasicCleanup(CSeqdesc& desc, Uint4 options = 0);
+    TChanges BasicCleanup(CSeq_descr & desc, Uint4 options = 0);
 
     // Handle versions.
-    CConstRef<CCleanupChange> BasicCleanup(CSeq_entry_Handle& seh, Uint4 options = 0);
-    CConstRef<CCleanupChange> BasicCleanup(CBioseq_Handle& bsh,    Uint4 options = 0);
-    CConstRef<CCleanupChange> BasicCleanup(CBioseq_set_Handle& bssh, Uint4 options = 0);
-    CConstRef<CCleanupChange> BasicCleanup(CSeq_annot_Handle& sak, Uint4 options = 0);
-    CConstRef<CCleanupChange> BasicCleanup(CSeq_feat_Handle& sfh,  Uint4 options = 0);
+    TChanges BasicCleanup(CSeq_entry_Handle& seh, Uint4 options = 0);
+    TChanges BasicCleanup(CBioseq_Handle& bsh,    Uint4 options = 0);
+    TChanges BasicCleanup(CBioseq_set_Handle& bssh, Uint4 options = 0);
+    TChanges BasicCleanup(CSeq_annot_Handle& sak, Uint4 options = 0);
+    TChanges BasicCleanup(CSeq_feat_Handle& sfh,  Uint4 options = 0);
 
     // Extended Cleanup
         /// Cleanup a Seq-entry.
-    CConstRef<CCleanupChange> ExtendedCleanup(CSeq_entry& se,  Uint4 options = 0);
+    TChanges ExtendedCleanup(CSeq_entry& se,  Uint4 options = 0);
     /// Cleanup a Seq-submit.
-    CConstRef<CCleanupChange> ExtendedCleanup(CSeq_submit& ss, Uint4 options = 0);
+    TChanges ExtendedCleanup(CSeq_submit& ss, Uint4 options = 0);
     /// Cleanup a Seq-Annot.
-    CConstRef<CCleanupChange> ExtendedCleanup(CSeq_annot& sa,  Uint4 options = 0);
+    TChanges ExtendedCleanup(CSeq_annot& sa,  Uint4 options = 0);
 
     // Handle versions
-    static CConstRef<CCleanupChange> ExtendedCleanup(CSeq_entry_Handle& seh, Uint4 options = 0);
+    static TChanges ExtendedCleanup(CSeq_entry_Handle& seh, Uint4 options = 0);
 
     // Useful cleanup functions
 
@@ -312,7 +317,7 @@ public:
     static void SetProteinName(CSeq_feat& cds, const string& protein_name, bool append, CScope& scope);
     static void SetMrnaName(CSeq_feat& mrna, const string& protein_name);
     static const string& GetProteinName(const CProt_ref& prot);
-    static const string& GetProteinName(const CSeq_feat& cds, CScope& scope);
+    static const string& GetProteinName(const CSeq_feat& cds, CSeq_entry_Handle seh);
 
 /// Sets MolInfo::tech for a sequence
 /// @param seq Bioseq to edit
@@ -342,6 +347,9 @@ public:
 /// @return Boolean return value indicates whether object was removed
     static bool RemoveNcbiCleanupObject(CSeq_entry &seq_entry);
 
+/// Adds NcbiCleanup User Object to Seq-descr
+    static void AddNcbiCleanupObject(int ncbi_cleanup_version, CSeq_descr& descr);
+
 /// Looks up Org-refs in the Seq-entry
 /// @param seh Seq-entry to edit
 /// @return Boolean return value indicates whether object was updated
@@ -598,11 +606,10 @@ public:
 
     static void AutodefId(CSeq_entry_Handle seh);
 
-private:
-    // Prohibit copy constructor & assignment operator
-    CCleanup(const CCleanup&);
-    CCleanup& operator= (const CCleanup&);
+    // for finding the correct amino acid letter given an abbreviation
+    static char ValidAminoAcid(string_view abbrev);
 
+private:
     CRef<CScope>            m_Scope;
 
     static bool x_CleanupUserField(CUser_field& field);
