@@ -1,7 +1,7 @@
 #ifndef GBLOADER__HPP_INCLUDED
 #define GBLOADER__HPP_INCLUDED
 
-/*  $Id: gbloader.hpp 613335 2020-08-05 16:17:36Z sadyrovr $
+/*  $Id: gbloader.hpp 675419 2023-11-21 19:45:39Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -109,6 +109,16 @@ public:
             return m_LoaderName;
         }
 
+    void SetLoaderMethod(const string& loader_method)
+    {
+        m_LoaderMethod = loader_method;
+    }
+
+    const string& GetLoaderMethod(void) const
+    {
+        return m_LoaderMethod;
+    }
+
     void SetReaderName(const string& reader_name)
         {
             m_ReaderName = reader_name;
@@ -118,6 +128,15 @@ public:
             return m_ReaderName;
         }
 
+    void SetWriterName(const string& writer_name)
+        {
+            m_WriterName = writer_name;
+        }
+    const string& GetWriterName(void) const
+        {
+            return m_WriterName;
+        }
+
     void SetReaderPtr(CReader* reader_ptr);
     CReader* GetReaderPtr(void) const;
 
@@ -139,8 +158,7 @@ public:
                         const string& web_cookie = NcbiEmptyString)
         {
             m_HasHUPIncluded = include_hup;
-            if (include_hup && !web_cookie.empty())
-                m_WebCookie = web_cookie;
+            m_WebCookie = web_cookie;
         }
     bool HasHUPIncluded(void) const
         {
@@ -162,6 +180,13 @@ public:
         m_PSGServiceName = service_name;
     }
 
+    bool GetUsePSG() const;
+    void SetUsePSG(bool use_psg = true)
+    {
+        m_UsePSG = use_psg;
+        m_UsePSGInitialized = true;
+    }
+
     bool GetPSGNoSplit(void) const
     {
         return m_PSGNoSplit;
@@ -172,16 +197,37 @@ public:
         m_PSGNoSplit = no_split;
     }
 
+    bool IsSetEnableSNP(void) const;
+    bool GetEnableSNP(void) const;
+    void SetEnableSNP(bool enable);
+
+    bool IsSetEnableWGS(void) const;
+    bool GetEnableWGS(void) const;
+    void SetEnableWGS(bool enable);
+
+    bool IsSetEnableCDD(void) const;
+    bool GetEnableCDD(void) const;
+    void SetEnableCDD(bool enable);
+
 private:
     string m_ReaderName;
+    string m_WriterName;
+    string m_LoaderMethod;
     CRef<CReader> m_ReaderPtr;
     const TParamTree* m_ParamTree;
     EPreopenConnection m_Preopen;
+    mutable bool m_UsePSGInitialized; // can be changed in GetUsePSG()
+    mutable bool m_UsePSG;
+    bool m_PSGNoSplit;
     bool m_HasHUPIncluded;
     string m_WebCookie;
     string m_LoaderName;
     string m_PSGServiceName;
-    bool m_PSGNoSplit;
+    CNullable<bool> m_EnableSNP;
+    CNullable<bool> m_EnableWGS;
+    CNullable<bool> m_EnableCDD;
+
+    friend class CGBDataLoader_Native;
 };
 
 class NCBI_XLOADER_GENBANK_EXPORT CGBDataLoader : public CDataLoader
@@ -234,13 +280,15 @@ public:
         EIncludeHUP     include_hup,
         CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
         CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
+    static string GetLoaderNameFromArgs(EIncludeHUP     include_hup);
     static TRegisterLoaderInfo RegisterInObjectManager(
         CObjectManager& om,
         EIncludeHUP     include_hup,
         const string& web_cookie,
         CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
         CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
-    static string GetLoaderNameFromArgs(EIncludeHUP     include_hup);
+    static string GetLoaderNameFromArgs(EIncludeHUP     include_hup,
+                                        const string& web_cookie);
 
     // GBLoader with HUP data included.
     // The reader can be either pubseqos or pubseqos2.
@@ -251,6 +299,8 @@ public:
         EIncludeHUP     include_hup,
         CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
         CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
+    static string GetLoaderNameFromArgs(const string&   reader_name, // pubseqos or pubseqos2
+                                        EIncludeHUP     include_hup);
     static TRegisterLoaderInfo RegisterInObjectManager(
         CObjectManager& om,
         const string&   reader_name, // pubseqos or pubseqos2
@@ -259,7 +309,8 @@ public:
         CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
         CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
     static string GetLoaderNameFromArgs(const string&   reader_name, // pubseqos or pubseqos2
-                                        EIncludeHUP     include_hup);
+                                        EIncludeHUP     include_hup,
+                                        const string&   web_cookie);
 
     // Setup loader using param tree. If tree is null or failed to find params,
     // use environment or select default reader.
@@ -280,15 +331,15 @@ public:
         CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
     static string GetLoaderNameFromArgs(const CGBLoaderParams& params);
 
-    CConstRef<CSeqref> GetSatSatkey(const CSeq_id_Handle& idh);
-    CConstRef<CSeqref> GetSatSatkey(const CSeq_id& id);
-
     virtual TNamedAnnotNames GetNamedAnnotAccessions(const CSeq_id_Handle& idh) = 0;
     virtual TNamedAnnotNames GetNamedAnnotAccessions(const CSeq_id_Handle& idh,
         const string& named_acc) = 0;
 
-    const TRealBlobId& GetRealBlobId(const TBlobId& blob_id) const;
-    const TRealBlobId& GetRealBlobId(const CTSE_Info& tse_info) const;
+    CConstRef<CSeqref> GetSatSatkey(const CSeq_id_Handle& idh);
+    CConstRef<CSeqref> GetSatSatkey(const CSeq_id& id);
+
+    TRealBlobId GetRealBlobId(const TBlobId& blob_id) const;
+    TRealBlobId GetRealBlobId(const CTSE_Info& tse_info) const;
 
     // params modifying access methods
     // argument params should be not-null
@@ -379,14 +430,40 @@ public:
             m_PTISErrorAction = action;
         }
 
+    virtual CObjectManager::TPriority GetDefaultPriority(void) const override;
+    virtual bool GetTrackSplitSeq() const override;
+
 protected:
+    template <class TDataLoader>
+        class CGBLoaderMaker : public CLoaderMaker_Base
+    {
+    public:
+        CGBLoaderMaker(const CGBLoaderParams& params)
+            : m_Params(params)
+            {
+                m_Name = CGBDataLoader::GetLoaderNameFromArgs(params);
+            }
+        
+    virtual ~CGBLoaderMaker(void) {}
+
+    virtual CDataLoader* CreateLoader(void) const
+        {
+            return new TDataLoader(m_Name, m_Params);
+        }
+    typedef SRegisterLoaderInfo<CGBDataLoader> TRegisterInfo;
+    TRegisterInfo GetRegisterInfo(void)
+        {
+            TRegisterInfo info;
+            info.Set(m_RegisterInfo.GetLoader(), m_RegisterInfo.IsCreated());
+            return info;
+        }
+    protected:
+        CGBLoaderParams m_Params;
+    };
     CGBDataLoader(const string&     loader_name,
                   const CGBLoaderParams& params);
 
-    // Find GB loader params in the tree or return the original tree.
-    const TParamTree* x_GetLoaderParams(const TParamTree* params) const;
-    // Get reader name from the GB loader params.
-    string x_GetReaderName(const TParamTree* params) const;
+    static string x_GetLoaderMethod(const TParamTree* params);
 
     TExpirationTimeout      m_IdExpirationTimeout;
 
@@ -401,6 +478,7 @@ private:
     CGBDataLoader(const CGBDataLoader&);
     CGBDataLoader& operator=(const CGBDataLoader&);
 
+    virtual TRealBlobId x_GetRealBlobId(const TBlobId& blob_id) const = 0;
 public:
     static bool IsUsingPSGLoader(void);
 };
