@@ -1,7 +1,7 @@
 #ifndef GBNATIVE__HPP_INCLUDED
 #define GBNATIVE__HPP_INCLUDED
 
-/*  $Id: gbnative.hpp 613335 2020-08-05 16:17:36Z sadyrovr $
+/*  $Id: gbnative.hpp 675419 2023-11-21 19:45:39Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -171,66 +171,6 @@ public:
     virtual bool CanGetBlobById(void) const override;
     virtual TTSE_Lock GetBlobById(const TBlobId& id) override;
 
-    // Create GB loader and register in the object manager if
-    // no loader with the same name is registered yet.
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        CReader* reader = 0,
-        CObjectManager::EIsDefault is_default = CObjectManager::eDefault,
-        CObjectManager::TPriority priority = CObjectManager::kPriority_NotSet);
-
-    // Select reader by name. If failed, select default reader.
-    // Reader name may be the same as in environment: PUBSEQOS, ID1 etc.
-    // Several names may be separated with ":". Empty name or "*"
-    // included as one of the names allows to include reader names
-    // from environment and registry.
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        const string&   reader_name,
-        CObjectManager::EIsDefault is_default = CObjectManager::eDefault,
-        CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
-
-    // GBLoader with HUP data included.
-    // The reader will be chosen from default configuration,
-    // either pubseqos or pubseqos2.
-    // The default loader priority will be slightly lower than for main data.
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        EIncludeHUP     include_hup,
-        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
-        CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        EIncludeHUP     include_hup,
-        const string& web_cookie,
-        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
-        CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
-
-    // GBLoader with HUP data included.
-    // The reader can be either pubseqos or pubseqos2.
-    // The default loader priority will be slightly lower than for main data.
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        const string&   reader_name, // pubseqos or pubseqos2
-        EIncludeHUP     include_hup,
-        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
-        CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        const string&   reader_name, // pubseqos or pubseqos2
-        EIncludeHUP     include_hup,
-        const string& web_cookie,
-        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
-        CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
-
-    // Setup loader using param tree. If tree is null or failed to find params,
-    // use environment or select default reader.
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        const TParamTree& params,
-        CObjectManager::EIsDefault is_default = CObjectManager::eDefault,
-        CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
-
     // Setup loader using param tree. If tree is null or failed to find params,
     // use environment or select default reader.
     static TRegisterLoaderInfo RegisterInObjectManager(
@@ -272,8 +212,6 @@ public:
                     time_t                access_timeout = 0) override;
     void CloseCache(void) override;
 
-    virtual CObjectManager::TPriority GetDefaultPriority(void) const override;
-
 protected:
     friend class CGBReaderRequestResult;
 
@@ -287,19 +225,13 @@ protected:
 
 private:
     friend class CGBDataLoader;
-    typedef CParamLoaderMaker<CGBDataLoader_Native, const CGBLoaderParams&> TGBMaker;
-    friend class CParamLoaderMaker<CGBDataLoader_Native, const CGBLoaderParams&>;
+    friend class CGBLoaderMaker<CGBDataLoader_Native>;
 
-    static TRegisterLoaderInfo ConvertRegInfo(const TGBMaker::TRegisterInfo& info);
+    TRealBlobId x_GetRealBlobId(const TBlobId& blob_id) const override;
 
     CGBDataLoader_Native(const string&     loader_name,
                          const CGBLoaderParams& params);
 
-    // Find GB loader params in the tree or return the original tree.
-    const TParamTree* x_GetLoaderParams(const TParamTree* params) const;
-    // Get reader name from the GB loader params.
-    string x_GetReaderName(const TParamTree* params) const;
-
     CInitMutexPool          m_MutexPool;
 
     CRef<CReadDispatcher>   m_Dispatcher;
@@ -314,12 +246,13 @@ private:
 
     void x_CreateDriver(const CGBLoaderParams& params);
 
-    pair<string, string> GetReaderWriterName(const TParamTree* params) const;
+    pair<string, string> GetReaderWriterName(const TParamTree* params, const CGBLoaderParams& loader_params) const;
     bool x_CreateReaders(const string& str,
                          const TParamTree* params,
+                         const CGBLoaderParams& gb_params,
                          CGBLoaderParams::EPreopenConnection preopen);
     void x_CreateWriters(const string& str, const TParamTree* params);
-    CReader* x_CreateReader(const string& names, const TParamTree* params = 0);
+    CReader* x_CreateReader(const string& names, const CGBLoaderParams& gb_params, const TParamTree* params = 0);
     CWriter* x_CreateWriter(const string& names, const TParamTree* params = 0);
 
     typedef CPluginManager<CReader> TReaderManager;
