@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_DATA_LOADERS_PSG___PSG_LOADER_IMPL__HPP
 #define OBJTOOLS_DATA_LOADERS_PSG___PSG_LOADER_IMPL__HPP
 
-/*  $Id: psg_loader_impl.hpp 630073 2021-04-26 17:10:31Z ivanov $
+/*  $Id: psg_loader_impl.hpp 675200 2023-11-16 13:48:27Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,7 @@
 #include <objtools/data_loaders/genbank/psg_loader.hpp>
 #include <objtools/pubseq_gateway/client/psg_client.hpp>
 #include <objtools/data_loaders/genbank/blob_id.hpp>
+#include <objtools/data_loaders/genbank/impl/incr_time.hpp>
 #include <memory>
 #include <vector>
 
@@ -45,48 +46,18 @@
 BEGIN_NCBI_SCOPE
 
 class CThreadPool;
+class CThreadPool_Task;
 
 BEGIN_SCOPE(objects)
 
-class CBlobId;
-
-class CPsgBlobId : public CBlobId
-{
-public:
-    explicit CPsgBlobId(const string& id);
-    CPsgBlobId(const string& id, const string& id2_info);
-    virtual ~CPsgBlobId();
-    
-    const string& ToPsgId() const
-        {
-            return m_Id;
-        }
-    
-    virtual string ToString(void) const override;
-    virtual bool operator<(const CBlobId& id) const override;
-    virtual bool operator==(const CBlobId& id) const override;
-
-    const string& GetId2Info() const
-        {
-            return m_Id2Info;
-        }
-    void SetId2Info(const string& id2_info)
-        {
-            m_Id2Info = id2_info;
-        }
-    
-private:
-    string m_Id;
-    string m_Id2Info;
-};
-
 struct SPsgBioseqInfo
 {
-    SPsgBioseqInfo(const CPSG_BioseqInfo& bioseq_info);
+    SPsgBioseqInfo(const CPSG_BioseqInfo& bioseq_info, int lifespan);
 
-    typedef CPSG_Request_Resolve::TIncludeInfo TIncludedInfo;
-    typedef vector<CSeq_id_Handle> TIds;
-    TIncludedInfo included_info;
+    typedef underlying_type_t<CPSG_Request_Resolve::EIncludeInfo> TIncludedInfo;
+    typedef CDataLoader::TIds TIds;
+
+    atomic<TIncludedInfo> included_info;
     CSeq_inst::TMol molecule_type;
     Uint8 length;
     CPSG_BioseqInfo::TState state;
@@ -100,6 +71,7 @@ struct SPsgBioseqInfo
     CDeadline deadline;
 
     TIncludedInfo Update(const CPSG_BioseqInfo& bioseq_info);
+    CBioseq_Handle::TBioseqStateFlags GetBioseqStateFlags() const;
 
 private:
     SPsgBioseqInfo(const SPsgBioseqInfo&);
@@ -109,14 +81,15 @@ private:
 
 struct SPsgBlobInfo
 {
-    SPsgBlobInfo(const CPSG_BlobInfo& blob_info);
+    explicit SPsgBlobInfo(const CPSG_BlobInfo& blob_info);
+    explicit SPsgBlobInfo(const CTSE_Info& tse);
 
     string blob_id_main;
     string id2_info;
-    int blob_state;
+    CBioseq_Handle::TBioseqStateFlags blob_state_flags;
     Int8 last_modified;
 
-    int GetBlobVersion() const { return last_modified/60000; /* ms to minutes */ }
+    int GetBlobVersion() const { return int(last_modified/60000); /* ms to minutes */ }
 
     bool IsSplit() const { return !id2_info.empty(); }
 
@@ -126,55 +99,97 @@ private:
 };
 
 
-class CPsgClientThread;
 class CPSGBioseqCache;
+class CPSGAnnotCache;
+class CPSGCDDInfoCache;
 class CPSGBlobMap;
-class CPsgClientContext_Bulk;
+class CPSGIpgTaxIdMap;
 class CPSG_Blob_Task;
+class CPSG_PrefetchCDD_Task;
 
 
 class CPSGDataLoader_Impl : public CObject
 {
 public:
     explicit CPSGDataLoader_Impl(const CGBLoaderParams& params);
-    ~CPSGDataLoader_Impl(void);
+    ~CPSGDataLoader_Impl(void) override;
 
     typedef CDataLoader::TIds TIds;
     typedef CDataLoader::TGis TGis;
+    typedef CDataLoader::TTaxIds TTaxIds;
+    typedef CDataLoader::TSequenceLengths TSequenceLengths;
+    typedef CDataLoader::TSequenceTypes TSequenceTypes;
     typedef CDataLoader::TLoaded TLoaded;
     typedef CDataLoader::TTSE_LockSets TTSE_LockSets;
 
     void GetIds(const CSeq_id_Handle& idh, TIds& ids);
+    void GetIdsOnce(const CSeq_id_Handle& idh, TIds& ids);
     CDataLoader::SGiFound GetGi(const CSeq_id_Handle& idh);
+    CDataLoader::SGiFound GetGiOnce(const CSeq_id_Handle& idh);
     CDataLoader::SAccVerFound GetAccVer(const CSeq_id_Handle& idh);
+    CDataLoader::SAccVerFound GetAccVerOnce(const CSeq_id_Handle& idh);
     TTaxId GetTaxId(const CSeq_id_Handle& idh);
+    TTaxId GetTaxIdOnce(const CSeq_id_Handle& idh);
+    void GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret);
+    void GetTaxIdsOnce(const TIds& ids, TLoaded& loaded, TTaxIds& ret);
     TSeqPos GetSequenceLength(const CSeq_id_Handle& idh);
+    TSeqPos GetSequenceLengthOnce(const CSeq_id_Handle& idh);
     CDataLoader::SHashFound GetSequenceHash(const CSeq_id_Handle& idh);
+    CDataLoader::SHashFound GetSequenceHashOnce(const CSeq_id_Handle& idh);
     CDataLoader::STypeFound GetSequenceType(const CSeq_id_Handle& idh);
-    int GetSequenceState(const CSeq_id_Handle& idh);
+    CDataLoader::STypeFound GetSequenceTypeOnce(const CSeq_id_Handle& idh);
+    int GetSequenceState(CDataSource* data_source, const CSeq_id_Handle& idh);
+    int GetSequenceStateOnce(CDataSource* data_source, const CSeq_id_Handle& idh);
 
     CDataLoader::TTSE_LockSet GetRecords(CDataSource* data_source,
                                          const CSeq_id_Handle& idh,
                                          CDataLoader::EChoice choice);
+    CDataLoader::TTSE_LockSet GetRecordsOnce(CDataSource* data_source,
+                                         const CSeq_id_Handle& idh,
+                                         CDataLoader::EChoice choice);
     CRef<CPsgBlobId> GetBlobId(const CSeq_id_Handle& idh);
+    CRef<CPsgBlobId> GetBlobIdOnce(const CSeq_id_Handle& idh);
     CTSE_Lock GetBlobById(CDataSource* data_source,
                               const CPsgBlobId& blob_id);
+    CTSE_Lock GetBlobByIdOnce(CDataSource* data_source,
+                              const CPsgBlobId& blob_id);
     void LoadChunk(CDataSource* data_source,
                    CTSE_Chunk_Info& chunk_info);
     void LoadChunks(CDataSource* data_source,
                     const CDataLoader::TChunkSet& chunks);
+    void LoadChunksOnce(CDataSource* data_source,
+                    const CDataLoader::TChunkSet& chunks);
 
     void GetBlobs(CDataSource* data_source, TTSE_LockSets& tse_sets);
+    typedef set<CSeq_id_Handle> TLoadedSeqIds;
+    void GetBlobsOnce(CDataSource* data_source, TLoadedSeqIds& loaded, TTSE_LockSets& tse_sets);
+
+    typedef CDataLoader::TSeqIdSets TSeqIdSets;
+    typedef CDataLoader::TCDD_Locks TCDD_Locks;
+    void GetCDDAnnots(CDataSource* data_source,
+        const TSeqIdSets& id_sets, TLoaded& loaded, TCDD_Locks& ret);
+    void GetCDDAnnotsOnce(CDataSource* data_source,
+        const TSeqIdSets& id_sets, TLoaded& loaded, TCDD_Locks& ret);
 
     CDataLoader::TTSE_LockSet GetAnnotRecordsNA(CDataSource* data_source,
-                                                const CSeq_id_Handle& idh,
+                                                const TIds& ids,
                                                 const SAnnotSelector* sel,
                                                 CDataLoader::TProcessedNAs* processed_nas);
+    CDataLoader::TTSE_LockSet GetAnnotRecordsNAOnce(CDataSource* data_source,
+                                                    const TIds& ids,
+                                                    const SAnnotSelector* sel,
+                                                    CDataLoader::TProcessedNAs* processed_nas);
 
     void DropTSE(const CPsgBlobId& blob_id);
 
     void GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret);
+    void GetAccVersOnce(const TIds& ids, TLoaded& loaded, TIds& ret);
     void GetGis(const TIds& ids, TLoaded& loaded, TGis& ret);
+    void GetGisOnce(const TIds& ids, TLoaded& loaded, TGis& ret);
+    void GetSequenceLengths(const TIds& ids, TLoaded& loaded, TSequenceLengths& ret);
+    void GetSequenceLengthsOnce(const TIds& ids, TLoaded& loaded, TSequenceLengths& ret);
+    void GetSequenceTypes(const TIds& ids, TLoaded& loaded, TSequenceTypes& ret);
+    void GetSequenceTypesOnce(const TIds& ids, TLoaded& loaded, TSequenceTypes& ret);
 
     static CObjectIStream* GetBlobDataStream(const CPSG_BlobInfo& blob_info, const CPSG_BlobData& blob_data);
 
@@ -183,54 +198,86 @@ public:
     struct SReplyResult {
         CTSE_Lock lock;
         string blob_id;
+        shared_ptr<SPsgBlobInfo> blob_info;
     };
 
     static void NCBI_XLOADER_GENBANK_EXPORT SetGetBlobByIdShouldFail(bool value);
     static bool NCBI_XLOADER_GENBANK_EXPORT GetGetBlobByIdShouldFail();
 
+    template<class Call>
+    typename std::invoke_result<Call>::type
+    CallWithRetry(Call&& call,
+                  const char* name,
+                  int retry_count = 0);
+    
+    void PrefetchCDD(const TIds& ids);
+
 private:
     friend class CPSG_Blob_Task;
 
-    void x_SendRequest(shared_ptr<CPSG_Request> request);
-    CPSG_BioId x_GetBioId(const CSeq_id_Handle& idh);
-    shared_ptr<CPSG_Reply> x_ProcessRequest(shared_ptr<CPSG_Request> request);
-    SReplyResult x_ProcessBlobReply(shared_ptr<CPSG_Reply> reply, CDataSource* data_source, CSeq_id_Handle req_idh, bool retry);
+    shared_ptr<CPSG_Reply> x_SendRequest(shared_ptr<CPSG_Request> request);
+    SReplyResult x_ProcessBlobReply(shared_ptr<CPSG_Reply> reply, CDataSource* data_source, CSeq_id_Handle req_idh, bool retry, bool lock_asap = false, CTSE_LoadLock* load_lock = nullptr);
     SReplyResult x_RetryBlobRequest(const string& blob_id, CDataSource* data_source, CSeq_id_Handle req_idh);
+    string x_GetCachedBlobId(const CSeq_id_Handle& idh);
     shared_ptr<SPsgBioseqInfo> x_GetBioseqInfo(const CSeq_id_Handle& idh);
-    CTSE_Lock x_LoadBlob(const SPsgBlobInfo& psg_blob_info, CDataSource& data_source);
+    shared_ptr<SPsgBlobInfo> x_GetBlobInfo(CDataSource* data_source, const string& blob_id);
+    typedef pair<shared_ptr<SPsgBioseqInfo>, shared_ptr<SPsgBlobInfo>> TBioseqAndBlobInfo;
+    TBioseqAndBlobInfo x_GetBioseqAndBlobInfo(CDataSource* data_source, const CSeq_id_Handle& idh);
+
+    enum EMainChunkType {
+        eNoDelayedMainChunk,
+        eDelayedMainChunk
+    };
+    enum ESplitInfoType {
+        eNoSplitInfo,
+        eIsSplitInfo
+    };
+    // caller of x_ReadBlobData() should call SetLoaded();
     void x_ReadBlobData(
         const SPsgBlobInfo& psg_blob_info,
         const CPSG_BlobInfo& blob_info,
         const CPSG_BlobData& blob_data,
         CTSE_LoadLock& load_lock,
-        bool is_split_info);
+        ESplitInfoType split_info_type);
+    void x_SetLoaded(CTSE_LoadLock& load_lock, EMainChunkType main_chunk_type);
 
-    typedef map<void*, size_t> TIdxMap;
-
-    void x_GetBulkBioseqInfo(CPSG_Request_Resolve::EIncludeInfo info,
+    // returns pair(number of loaded infos, number of failed infos)
+    pair<size_t, size_t> x_GetBulkBioseqInfo(CPSG_Request_Resolve::EIncludeInfo info,
         const TIds& ids,
-        TLoaded& loaded,
+        const TLoaded& loaded,
         TBioseqInfos& ret);
 
     shared_ptr<CPSG_Request_Blob>
     x_MakeLoadLocalCDDEntryRequest(CDataSource* data_source,
-                                   CDataLoader::TChunk chunk,
-                                   shared_ptr<CPsgClientContext_Bulk> context);
+                                   CDataLoader::TChunk chunk);
     bool x_ReadCDDChunk(CDataSource* data_source,
                         CDataLoader::TChunk chunk,
                         const CPSG_BlobInfo& blob_info,
                         const CPSG_BlobData& blob_data);
-    
-    // Map seq-id to bioseq info.
-    typedef map<CSeq_id_Handle, shared_ptr<SPsgBioseqInfo> > TBioseqCache;
+    bool x_CheckAnnotCache(const string& name,
+                           const TIds& ids,
+                           CDataSource* data_source,
+                           CDataLoader::TProcessedNAs* processed_nas,
+                           CDataLoader::TTSE_LockSet& locks);
+    TTaxId x_GetIpgTaxId(const CSeq_id_Handle& idh);
+    void x_GetIpgTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret);
 
-    bool m_NoSplit = false;
+    CPSG_Request_Biodata::EIncludeData m_TSERequestMode = CPSG_Request_Biodata::eSmartTSE;
+    CPSG_Request_Biodata::EIncludeData m_TSERequestModeBulk = CPSG_Request_Biodata::eWholeTSE;
     bool m_AddWGSMasterDescr = true;
     shared_ptr<CPSG_Queue> m_Queue;
-    CRef<CPsgClientThread> m_Thread;
+    CRef<CRequestContext> m_RequestContext;
     unique_ptr<CPSGBlobMap> m_BlobMap;
+    unique_ptr<CPSGIpgTaxIdMap> m_IpgTaxIdMap;
     unique_ptr<CPSGBioseqCache> m_BioseqCache;
+    unique_ptr<CPSGAnnotCache> m_AnnotCache;
+    unique_ptr<CPSGCDDInfoCache> m_CDDInfoCache;
     unique_ptr<CThreadPool> m_ThreadPool;
+    CRef<CPSG_PrefetchCDD_Task> m_CDDPrefetchTask;
+    int m_CacheLifespan;
+    unsigned int m_RetryCount;
+    unsigned int m_BulkRetryCount;
+    CIncreasingTime m_WaitTime;
 };
 
 END_SCOPE(objects)
