@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_DATA_LOADERS_PSG___PSG_LOADER__HPP
 #define OBJTOOLS_DATA_LOADERS_PSG___PSG_LOADER__HPP
 
-/*  $Id: psg_loader.hpp 620489 2020-11-23 18:09:39Z sadyrovr $
+/*  $Id: psg_loader.hpp 675419 2023-11-21 19:45:39Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,7 +35,7 @@
 
 #include <corelib/ncbistd.hpp>
 #include <objtools/data_loaders/genbank/gbloader.hpp>
-#include <objtools/pubseq_gateway/client/psg_client.hpp>
+#include <objtools/pubseq_gateway/client/psg_client.hpp> // for HAVE_PSG_CLIENT
 
 
 #if defined(HAVE_PSG_LOADER) && !defined(HAVE_PSG_CLIENT)
@@ -49,23 +49,53 @@ BEGIN_SCOPE(objects)
 
 class CPSGDataLoader_Impl;
 
+class NCBI_XLOADER_GENBANK_EXPORT CPsgBlobId : public CBlobId
+{
+public:
+    explicit CPsgBlobId(const string& id);
+    CPsgBlobId(const string& id, const string& id2_info);
+    virtual ~CPsgBlobId();
+    
+    const string& ToPsgId() const
+        {
+            return m_Id;
+        }
+    
+    virtual string ToString(void) const override;
+    virtual bool operator<(const CBlobId& id) const override;
+    virtual bool operator==(const CBlobId& id) const override;
+
+    const string& GetId2Info() const
+        {
+            return m_Id2Info;
+        }
+    void SetId2Info(const string& id2_info)
+        {
+            m_Id2Info = id2_info;
+        }
+    
+    bool GetSatSatkey(int& sat, int& satkey) const;
+
+    static CConstRef<CPsgBlobId> GetPsgBlobId(const CBlobId& blob_id);
+
+private:
+    string m_Id;
+    string m_Id2Info;
+};
+
+
 class NCBI_XLOADER_GENBANK_EXPORT CPSGDataLoader : public CGBDataLoader
 {
 public:
+    static TRegisterLoaderInfo RegisterInObjectManager(
+        CObjectManager& om,
+        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
+        CObjectManager::TPriority priority = CObjectManager::kPriority_NotSet);
     static TRegisterLoaderInfo RegisterInObjectManager(
         CObjectManager& om,
         const CGBLoaderParams& params,
         CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
         CObjectManager::TPriority priority = CObjectManager::kPriority_NotSet);
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
-        CObjectManager::TPriority priority = CObjectManager::kPriority_NotSet);
-    static TRegisterLoaderInfo RegisterInObjectManager(
-        CObjectManager& om,
-        const TParamTree& param_tree,
-        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
-        CObjectManager::TPriority priority = CObjectManager::kPriority_NotSet);
 
     virtual TBlobId GetBlobId(const CSeq_id_Handle& idh) override;
     virtual TBlobId GetBlobIdFromString(const string& str) const override;
@@ -78,9 +108,6 @@ public:
     virtual TTSE_LockSet GetOrphanAnnotRecordsNA(const CSeq_id_Handle& idh,
         const SAnnotSelector* sel,
         TProcessedNAs* processed_nas) override;
-    virtual TTSE_LockSet GetExternalAnnotRecordsNA(const CSeq_id_Handle& idh,
-        const SAnnotSelector* sel,
-        TProcessedNAs* processed_nas) override;
     virtual TTSE_LockSet GetExternalAnnotRecordsNA(const CBioseq_Info& bioseq,
         const SAnnotSelector* sel,
         TProcessedNAs* processed_nas) override;
@@ -90,6 +117,8 @@ public:
 
     virtual void GetBlobs(TTSE_LockSets& tse_sets) override;
 
+    virtual void GetCDDAnnots(const TSeqIdSets& id_sets, TLoaded& loaded, TCDD_Locks& ret) override;
+
     virtual void GetIds(const CSeq_id_Handle& idh, TIds& ids) override;
     virtual SAccVerFound GetAccVerFound(const CSeq_id_Handle& idh) override;
     virtual SGiFound GetGiFound(const CSeq_id_Handle& idh) override;
@@ -114,20 +143,23 @@ public:
 
     virtual void GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret) override;
     virtual void GetGis(const TIds& ids, TLoaded& loaded, TGis& ret) override;
+    void GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret) override;
     /*
     void GetLabels(const TIds& ids, TLoaded& loaded, TLabels& ret) override;
-    void GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret) override;
     void GetSequenceStates(const TIds& ids, TLoaded& loaded, TSequenceStates& ret) override;
     void GetSequenceHashes(const TIds& ids, TLoaded& loaded, TSequenceHashes& ret, THashKnown& known) override;
+    */
     void GetSequenceLengths(const TIds& ids, TLoaded& loaded, TSequenceLengths& ret) override;
     void GetSequenceTypes(const TIds& ids, TLoaded& loaded, TSequenceTypes& ret) override;
-    */
+
+    // Global default SNP scale limit.
+    static CSeq_id::ESNPScaleLimit GetSNP_Scale_Limit(void);
+    static void SetSNP_Scale_Limit(CSeq_id::ESNPScaleLimit value);
 
 private:
-    typedef CParamLoaderMaker<CPSGDataLoader, CGBLoaderParams> TMaker;
-    friend class CParamLoaderMaker<CPSGDataLoader, CGBLoaderParams>;
+    friend class CGBLoaderMaker<CPSGDataLoader>;
 
-    static TRegisterLoaderInfo ConvertRegInfo(const TMaker::TRegisterInfo& info);
+    TRealBlobId x_GetRealBlobId(const TBlobId& blob_id) const override;
 
     // default constructor
     CPSGDataLoader(void);
