@@ -1,6 +1,6 @@
 #ifndef READER__HPP_INCLUDED
 #define READER__HPP_INCLUDED
-/*  $Id: reader.hpp 610970 2020-06-26 12:56:46Z grichenk $
+/*  $Id: reader.hpp 675198 2023-11-16 13:48:10Z ivanov $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -49,6 +49,32 @@ class CReaderCacheManager;
 class CReaderAllocatedConnection;
 struct SAnnotSelector;
 
+
+class CReaderParams
+{
+public:
+    bool IsSetEnableSNP(void) const { return !m_EnableSNP.IsNull(); }
+    bool GetEnableSNP(void) const { return m_EnableSNP.GetValue(); }
+    void SetEnableSNP(CNullable<bool> enable) { m_EnableSNP = enable; }
+    void SetEnableSNP(bool enable) { m_EnableSNP = enable; }
+
+    bool IsSetEnableWGS(void) const { return !m_EnableWGS.IsNull(); }
+    bool GetEnableWGS(void) const { return m_EnableWGS.GetValue(); }
+    void SetEnableWGS(CNullable<bool> enable) { m_EnableWGS = enable; }
+    void SetEnableWGS(bool enable) { m_EnableWGS = enable; }
+
+    bool IsSetEnableCDD(void) const { return !m_EnableCDD.IsNull(); }
+    bool GetEnableCDD(void) const { return m_EnableCDD.GetValue(); }
+    void SetEnableCDD(CNullable<bool> enable) { m_EnableCDD = enable; }
+    void SetEnableCDD(bool enable) { m_EnableCDD = enable; }
+
+private:
+    CNullable<bool> m_EnableSNP;
+    CNullable<bool> m_EnableWGS;
+    CNullable<bool> m_EnableCDD;
+};
+
+
 class NCBI_XREADER_EXPORT CReader : public CObject
 {
 public:
@@ -274,7 +300,18 @@ public:
     
     virtual void SetIncludeHUP(bool include_hup = true,
                                const string& web_cookie = NcbiEmptyString);
+    bool HasHUPIncluded() const
+    {
+        return m_IncludeHUP;
+    }
 
+    virtual void SetParams(const CReaderParams& params);
+
+#if 0
+/*
+*   On Windows this works with ostrstream,
+    but does not work with ostringstream
+*/
     class NCBI_XREADER_EXPORT CDebugPrinter : public CNcbiOstrstream
     {
     public:
@@ -282,6 +319,26 @@ public:
         CDebugPrinter(const char* name);
         ~CDebugPrinter();
     };
+#else
+    class NCBI_XREADER_EXPORT CDebugPrinter
+    {
+    public:
+        CDebugPrinter(TConn conn, const char* name);
+        CDebugPrinter(const char* name);
+        ~CDebugPrinter();
+
+        operator CNcbiOstrstream&(void) {
+            return m_os;
+        }
+        template<typename T>
+        friend CDebugPrinter& operator<<( CDebugPrinter& pr, const T& obj) {
+            pr.m_os << obj;
+            return pr;
+        }
+    private:
+        CNcbiOstrstream m_os;
+    };
+#endif
     
 protected:
     CReadDispatcher* m_Dispatcher;
@@ -299,6 +356,10 @@ protected:
     // report failed or stale connection
     void x_ReportDisconnect(const char* reader, const char* server,
                             TConn conn, bool failed) const;
+    void x_SetIncludeHUP(bool include_hup)
+    {
+        m_IncludeHUP = include_hup;
+    }
 
 private:
     friend class CReaderAllocatedConnection;
@@ -314,6 +375,7 @@ private:
     // parameters
     TConn            m_MaxConnections;
     bool             m_PreopenConnection;
+    bool             m_IncludeHUP;
 
     // current state
     TConn            m_NextNewConnection;
@@ -327,7 +389,7 @@ private:
     CMutex           m_ConnectionsMutex;
     CSemaphore       m_NumFreeConnections;
     int              m_MaximumRetryCount;
-    int              m_ConnectFailCount;
+    atomic<int>      m_ConnectFailCount;
     CTime            m_LastTimeFailed;
     CTime            m_NextConnectTime;
     int              m_WaitTimeErrors;
