@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_EDIT___SEQ_ENTRY_EDIT__HPP
 #define OBJTOOLS_EDIT___SEQ_ENTRY_EDIT__HPP
 
-/*  $Id: seq_entry_edit.hpp 632623 2021-06-03 17:38:11Z ivanov $
+/*  $Id: seq_entry_edit.hpp 664960 2023-03-24 14:16:03Z stakhovv $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -107,7 +107,7 @@ CRef<CSeq_entry> SeqEntryFromSeqSubmit(const CSeq_submit& submit);
 ///   The Seq-entry to split
 /// @param bioseq_handles
 ///   The array of bioseqs that should end up in the second part of the target, with the rest in the first part.
-NCBI_XOBJEDIT_EXPORT
+NCBI_DEPRECATED NCBI_XOBJEDIT_EXPORT
 void SegregateSetsByBioseqList(const CSeq_entry_Handle & target,
     const CScope::TBioseqHandles & bioseq_handles );
 
@@ -182,11 +182,11 @@ void ConvertRawToDeltaByNs(CBioseq_Handle bsh,
                            size_t min_known, int max_known,
                            bool is_assembly_gap = false, int gap_type = CSeq_gap::eType_unknown, int linkage = -1, int linkage_evidence = -1 );
 
-typedef pair<size_t, int> TLocAdjustment;
+typedef pair<TSeqPos, int> TLocAdjustment;
 typedef vector<TLocAdjustment> TLocAdjustmentVector;
 
 NCBI_XOBJEDIT_EXPORT
-TLocAdjustmentVector NormalizeUnknownLengthGaps(CSeq_inst& inst, size_t unknown_length = 100);
+TLocAdjustmentVector NormalizeUnknownLengthGaps(CSeq_inst& inst, TSeqPos unknown_length = 100);
 
 NCBI_XOBJEDIT_EXPORT
 void SetLinkageType(CSeq_ext& ext, CSeq_gap::TType linkage_type);
@@ -294,7 +294,7 @@ void DeleteProteinAndRenormalizeNucProtSet(const CSeq_feat_Handle& feat_h);
 
 /// Secondary function needed after trimming Seq-feat.
 /// If TrimSeqFeat()'s bFeatureTrimmed returns true, then adjust cdregion frame.
-NCBI_XOBJEDIT_EXPORT
+NCBI_STD_DEPRECATED("AdjustCdregionFrame() doesn't work and needs to be removed.") NCBI_XOBJEDIT_EXPORT
 void AdjustCdregionFrame(TSeqPos original_nuc_len,
                          CRef<CSeq_feat> cds,
                          const TCuts& sorted_cuts);
