@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 #define OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 
-/*  $Id: flat_file_config.hpp 621839 2020-12-17 13:42:42Z ivanov $
+/*  $Id: flat_file_config.hpp 676886 2023-12-27 15:10:53Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -77,14 +77,11 @@ struct SModelEvidance;
 
 // --- Flat File configuration class
 
-// NEW_HTML_FMT define enables new interceptor interface for HTML links generation
-#define NEW_HTML_FMT
-
 class NCBI_FORMAT_EXPORT IHTMLFormatter: public CObject
 {
 public:
-    virtual ~IHTMLFormatter() {};
-    
+    virtual ~IHTMLFormatter() {}
+
     virtual void FormatProteinId(string& str, const CSeq_id& seq_id, const string& prot_id) const = 0;
     virtual void FormatTranscriptId(string& str, const CSeq_id& seq_id, const string& nuc_id) const = 0;
     virtual void FormatNucSearch(CNcbiOstream& os, const string& id) const = 0;
@@ -101,27 +98,27 @@ public:
 class NCBI_FORMAT_EXPORT CHTMLEmptyFormatter : public IHTMLFormatter
 {
 public:
-    virtual ~CHTMLEmptyFormatter() {};
+    ~CHTMLEmptyFormatter() override {}
 
-    void FormatProteinId(string& str, const CSeq_id& seq_id, const string& prot_id) const;
-    void FormatTranscriptId(string& str, const CSeq_id& seq_id, const string& nuc_id) const;
-    void FormatNucSearch(CNcbiOstream& os, const string& id) const;
-    void FormatNucId(string& str, const CSeq_id& seq_id, TIntId gi, const string& acc_id) const;
-    void FormatTaxid(string& str, const TTaxId taxid, const string& taxname) const;
-    void FormatLocation(string& str, const CSeq_loc& loc, TIntId gi, const string& visible_text) const;
-    void FormatModelEvidence(string& str, const SModelEvidance& me) const;
-    void FormatTranscript(string& str, const string& name) const;
-    void FormatGeneralId(CNcbiOstream& os, const string& id) const;
-    void FormatGapLink(CNcbiOstream& os, TSeqPos gap_size, const string& id, bool is_prot) const;
-    void FormatUniProtId(string& str, const string& prot_id) const;
+    void FormatProteinId(string& str, const CSeq_id& seq_id, const string& prot_id) const override;
+    void FormatTranscriptId(string& str, const CSeq_id& seq_id, const string& nuc_id) const override;
+    void FormatNucSearch(CNcbiOstream& os, const string& id) const override;
+    void FormatNucId(string& str, const CSeq_id& seq_id, TIntId gi, const string& acc_id) const override;
+    void FormatTaxid(string& str, const TTaxId taxid, const string& taxname) const override;
+    void FormatLocation(string& str, const CSeq_loc& loc, TIntId gi, const string& visible_text) const override;
+    void FormatModelEvidence(string& str, const SModelEvidance& me) const override;
+    void FormatTranscript(string& str, const string& name) const override;
+    void FormatGeneralId(CNcbiOstream& os, const string& id) const override;
+    void FormatGapLink(CNcbiOstream& os, TSeqPos gap_size, const string& id, bool is_prot) const override;
+    void FormatUniProtId(string& str, const string& prot_id) const override;
 };
 
 class NCBI_FORMAT_EXPORT CFlatFileConfig
 {
 public:
-    
+
     enum EFormat {
-        // formatting styles    
+        // formatting styles
         eFormat_GenBank,
         eFormat_EMBL,
         eFormat_DDBJ,
@@ -194,6 +191,8 @@ public:
         fExpandGaps            = 1 << 3,
         fShowSNPFeatures       = 1 << 6,
         fShowCDDFeatures       = 1 << 7,
+        fShowDebugTiming       = 1 << 8,
+        fFasterReleaseSets     = 1 << 9,
         fDisableAnnotRefs      = 1 << 10,
         fUseSeqEntryIndexer    = 1 << 11,
         fUseAutoDef            = 1 << 12,
@@ -202,7 +201,10 @@ public:
         fShowFtablePeptides    = 1 << 15,
         fDisableReferenceCache = 1 << 16,
         fShowDeflineModifiers  = 1 << 17,
-        fDoNotUseAutoDef       = 1 << 18
+        fDoNotUseAutoDef       = 1 << 18,
+        fOldTpaDisplay         = 1 << 19,
+        fDisableDefaultIndex   = 1 << 20,
+        fGeoLocNameCountry     = 1 << 21
     };
 
     enum EView {
@@ -216,11 +218,12 @@ public:
     enum EPolicy {
         // far feature fetch policy
         ePolicy_Adaptive = 0,
-        ePolicy_Internal,
-        ePolicy_External,
-        ePolicy_Exhaustive,
-        ePolicy_Ftp,
-        ePolicy_Web
+        ePolicy_Internal = 1,
+        ePolicy_External = 2,
+        ePolicy_Exhaustive = 3,
+        ePolicy_Ftp = 4,
+        ePolicy_Web = 5,
+        ePolicy_Genomes = 6
     };
 
     // These flags are used to select the GenBank sections to print or skip.
@@ -248,7 +251,7 @@ public:
         fGenbankBlocks_Source     = (1u <<  9),
         fGenbankBlocks_Reference  = (1u << 10),
         fGenbankBlocks_Comment    = (1u << 11),
-        fGenbankBlocks_Primary    = (1u << 12), 
+        fGenbankBlocks_Primary    = (1u << 12),
         fGenbankBlocks_Featheader = (1u << 13),
         fGenbankBlocks_Sourcefeat = (1u << 14),
         fGenbankBlocks_FeatAndGap = (1u << 15),
@@ -271,7 +274,7 @@ public:
     typedef unsigned int    TView;
     typedef unsigned int    TGenbankBlocks;
     typedef unsigned int    TCustom; // binary OR of "ECustom"
-    
+
     class NCBI_FORMAT_EXPORT CGenbankBlockCallback : public CObject {
     public:
         enum EAction {
@@ -279,7 +282,7 @@ public:
             eAction_Default,
             /// skip this block (i.e. don't print it)
             eAction_Skip,
-            /// If for some reason you don't want the rest of the flatfile generated, 
+            /// If for some reason you don't want the rest of the flatfile generated,
             /// this will trigger a CFlatException of type eHaltRequested
             eAction_HaltFlatfileGeneration
         };
@@ -291,7 +294,7 @@ public:
 
         virtual ~CGenbankBlockCallback(void) { }
 
-        // It is intentional that CBioseqContext is non-const here, 
+        // It is intentional that CBioseqContext is non-const here,
         // but const in the other notify functions.
         virtual EBioseqSkip notify_bioseq( CBioseqContext& ctx );
 
@@ -302,7 +305,7 @@ public:
         virtual EAction notify( string & block_text,
                                 const CBioseqContext& ctx,
                                 const CHtmlAnchorItem & anchor_item );
-        virtual EAction notify( string & block_text, 
+        virtual EAction notify( string & block_text,
                                 const CBioseqContext& ctx,
                                 const CLocusItem &locus_item );
         virtual EAction notify( string & block_text,
@@ -381,10 +384,10 @@ public:
         // ...or override
         // this if you want only a single entry-point for
         // notifications.
-        virtual EAction unified_notify( 
+        virtual EAction unified_notify(
             string & /*block_text*/,
-            const CBioseqContext& /*ctx*/, 
-            const IFlatItem & /*flat_item*/, 
+            const CBioseqContext& /*ctx*/,
+            const IFlatItem & /*flat_item*/,
             FGenbankBlocks /*which_block*/ ) { return eAction_Default; }
     };
 
@@ -400,7 +403,6 @@ public:
     // destructor
     ~CFlatFileConfig(void);
 
-#ifdef NEW_HTML_FMT
     void SetHTMLFormatter(CRef<IHTMLFormatter> html_fmt)
     {
         m_html_formatter = html_fmt;
@@ -409,7 +411,6 @@ public:
     {
         return *m_html_formatter;
     }
-#endif
 
     // -- Format
     // getters
@@ -501,6 +502,7 @@ public:
     bool IsPolicyExhaustive (void) const { return m_Policy == ePolicy_Exhaustive;  }
     bool IsPolicyFtp (void) const { return m_Policy == ePolicy_Ftp;  }
     bool IsPolicyWeb (void) const { return m_Policy == ePolicy_Web;  }
+    bool IsPolicyGenomes (void) const { return m_Policy == ePolicy_Genomes;  }
     // setters
     void SetPolicy(const TPolicy& Policy) { m_Policy = Policy;  }
     void SetPolicyAdaptive (void) { m_Policy = ePolicy_Adaptive;  }
@@ -509,6 +511,7 @@ public:
     void SetPolicyExhaustive (void) { m_Policy = ePolicy_Exhaustive;  }
     void SetPolicyFtp (void) { m_Policy = ePolicy_Ftp;  }
     void SetPolicyWeb (void) { m_Policy = ePolicy_Web;  }
+    void SetPolicyGenomes (void) { m_Policy = ePolicy_Genomes;  }
 
     // -- Flags
     // getters
@@ -575,7 +578,7 @@ public:
     bool GoQualsEachMerge    (void) const;
     bool ShowOutOfBoundsFeats(void) const;
     bool HideSpecificGeneMaps(void) const;
-    
+
     // setters (for customization flags)
     void SetFlags(const TFlags& flags) { m_Flags = flags; }
     CFlatFileConfig& SetDoHTML               (bool val = true);
@@ -619,6 +622,8 @@ public:
     bool ExpandGaps            (void) const;
     bool ShowSNPFeatures       (void) const;
     bool ShowCDDFeatures       (void) const;
+    bool ShowDebugTiming       (void) const;
+    bool FasterReleaseSets     (void) const;
     bool DisableAnnotRefs      (void) const;
     bool UseSeqEntryIndexer    (void) const;
     bool UseAutoDef            (void) const;
@@ -628,6 +633,9 @@ public:
     bool DisableReferenceCache (void) const;
     bool ShowDeflineModifiers  (void) const;
     bool DoNotUseAutoDef       (void) const;
+    bool OldTpaDisplay         (void) const;
+    bool DisableDefaultIndex   (void) const;
+    bool GeoLocNameCountry  (void) const;
 
     // setters
     void SetCustom(const TCustom& custom) { m_Custom = custom; }
@@ -637,6 +645,8 @@ public:
     CFlatFileConfig& SetExpandGaps            (bool val = true);
     CFlatFileConfig& SetShowSNPFeatures       (bool val = true);
     CFlatFileConfig& SetShowCDDFeatures       (bool val = true);
+    CFlatFileConfig& SetShowDebugTiming       (bool val = true);
+    CFlatFileConfig& SetFasterReleaseSets     (bool val = true);
     CFlatFileConfig& SetDisableAnnotRefs      (bool val = true);
     CFlatFileConfig& SetUseSeqEntryIndexer    (bool val = true);
     CFlatFileConfig& SetUseAutoDef            (bool val = true);
@@ -646,18 +656,25 @@ public:
     CFlatFileConfig& SetDisableReferenceCache (bool val = true);
     CFlatFileConfig& SetShowDeflineModifiers  (bool val = true);
     CFlatFileConfig& SetDoNotUseAutoDef       (bool val = true);
+    CFlatFileConfig& SetOldTpaDisplay         (bool val = true);
+    CFlatFileConfig& SetDisableDefaultIndex   (bool val = true);
+    CFlatFileConfig& SetGeoLocNameCountry  (bool val = true);
 
     // adjust mode dependant flags for RefSeq
     void SetRefSeqConventions(void);
 
-  int GetFeatDepth(void) const { return m_FeatDepth; }
-  void SetFeatDepth(const int featDepth) { m_FeatDepth = featDepth; }
+    // -- Single Accession filter
+    void SetSingleAccession(const string& accn) { m_SingleAccession = accn; }
+    const string& GetSingleAccession(void) const { return m_SingleAccession; }
 
-  int GetGapDepth(void) const { return m_GapDepth; }
-  void SetGapDepth(const int gapDepth) { m_GapDepth = gapDepth; }
+    int GetFeatDepth(void) const { return m_FeatDepth; }
+    void SetFeatDepth(const int featDepth) { m_FeatDepth = featDepth; }
+
+    int GetGapDepth(void) const { return m_GapDepth; }
+    void SetGapDepth(const int gapDepth) { m_GapDepth = gapDepth; }
 
 
-    void SetGenbankBlocks(const TGenbankBlocks& genbank_blocks) 
+    void SetGenbankBlocks(const TGenbankBlocks& genbank_blocks)
     {
         m_fGenbankBlocks = genbank_blocks;
     };
@@ -665,7 +682,7 @@ public:
     // check if the given section is shown
     bool IsShownGenbankBlock(FGenbankBlocks fTGenbankBlocksMask) const
     {
-        return (m_fGenbankBlocks & fTGenbankBlocksMask) == fTGenbankBlocksMask;
+        return m_fGenbankBlocks & fTGenbankBlocksMask;
     }
 
     // set the given section to be shown
@@ -688,7 +705,7 @@ public:
     static const vector<string> & GetAllGenbankStrings(void);
 
     // return non-const callback even if the CFlatFileConfig is const
-    CRef<CGenbankBlockCallback> GetGenbankBlockCallback(void) const { 
+    CRef<CGenbankBlockCallback> GetGenbankBlockCallback(void) const {
         return m_GenbankBlockCallback; }
     void SetGenbankBlockCallback(CGenbankBlockCallback* pGenbankBlockCallback) {
         m_GenbankBlockCallback = pGenbankBlockCallback;
@@ -707,7 +724,7 @@ public:
     // setters
     void BasicCleanup(bool clean) { m_BasicCleanup = clean; }
 
-    /// This throws a CFlatException if 
+    /// This throws a CFlatException if
     /// flatfile generation cancellation has been requested via ICanceled.
     void ThrowIfCanceled(void) const {
         if( m_pCanceledCallback && m_pCanceledCallback->IsCanceled() ) {
@@ -744,9 +761,8 @@ private:
     TCustom     m_Custom;
     int         m_FeatDepth;
     int         m_GapDepth;
-#ifdef NEW_HTML_FMT
+    string      m_SingleAccession;
     CRef<IHTMLFormatter> m_html_formatter;
-#endif
 };
 
 
@@ -837,6 +853,8 @@ CUSTOM_ARG_IMP(LongLocusNames)
 CUSTOM_ARG_IMP(ExpandGaps)
 CUSTOM_ARG_IMP(ShowSNPFeatures)
 CUSTOM_ARG_IMP(ShowCDDFeatures)
+CUSTOM_ARG_IMP(ShowDebugTiming)
+CUSTOM_ARG_IMP(FasterReleaseSets)
 CUSTOM_ARG_IMP(DisableAnnotRefs)
 CUSTOM_ARG_IMP(UseSeqEntryIndexer)
 CUSTOM_ARG_IMP(UseAutoDef)
@@ -846,6 +864,9 @@ CUSTOM_ARG_IMP(ShowFtablePeptides)
 CUSTOM_ARG_IMP(DisableReferenceCache)
 CUSTOM_ARG_IMP(ShowDeflineModifiers)
 CUSTOM_ARG_IMP(DoNotUseAutoDef)
+CUSTOM_ARG_IMP(OldTpaDisplay)
+CUSTOM_ARG_IMP(DisableDefaultIndex)
+CUSTOM_ARG_IMP(GeoLocNameCountry)
 
 #undef FLAG_ARG_IMP
 #undef FLAG_ARG_GET
