@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___GATHER_ITEMS__HPP
 #define OBJTOOLS_FORMAT___GATHER_ITEMS__HPP
 
-/*  $Id: gather_items.hpp 632193 2021-05-27 13:43:02Z ivanov $
+/*  $Id: gather_items.hpp 665420 2023-04-04 13:16:28Z stakhovv $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -58,7 +58,7 @@ class CFlatItemOStream;
 class NCBI_FORMAT_EXPORT CFlatGatherer : public CObject
 {
 public:
-    
+
     // virtual constructor
     static CFlatGatherer* New(CFlatFileConfig::TFormat format);
 
@@ -66,7 +66,7 @@ public:
                         bool doNuc = true, bool doProt = true) const;
     virtual void Gather(CFlatFileContext& ctx, CFlatItemOStream& os,
                         const CSeq_entry_Handle& entry, CBioseq_Handle bsh, bool useSeqEntryIndexing,
-                        bool doNuc = true, bool doProt = true) const;
+                        bool doNuc = true, bool doProt = true, bool fasterSets = false) const;
 
     virtual void SetCanceledCallback(const ICanceled* pCanceledCallback) {
         m_pCanceledCallback = pCanceledCallback;
@@ -77,7 +77,7 @@ public:
 protected:
     typedef CRange<TSeqPos> TRange;
 
-    CFlatGatherer(void) : m_pCanceledCallback(NULL) {}
+    CFlatGatherer() : m_pCanceledCallback(nullptr) {}
 
     CFlatItemOStream& ItemOS     (void) const { return *m_ItemOS;  }
     CBioseqContext& Context      (void) const { return *m_Current; }
@@ -94,7 +94,7 @@ protected:
         bool doNuc = true, bool doProt = true ) const;
 
     virtual void x_GatherBioseq(
-        const CBioseq_Handle& prev_seq, const CBioseq_Handle& this_seq, const CBioseq_Handle& next_seq, 
+        const CBioseq_Handle& prev_seq, const CBioseq_Handle& this_seq, const CBioseq_Handle& next_seq,
         CRef<CTopLevelSeqEntryContext> topLevelSeqEntryContext = CRef<CTopLevelSeqEntryContext>() ) const;
 
     virtual void x_DoMultipleSections(const CBioseq_Handle& seq) const;
@@ -177,12 +177,13 @@ protected:
     void x_RemoveExcessNewlines(void) const;
     void x_FlushComments(void) const;
     void x_UnverifiedComment(CBioseqContext& ctx) const;
+    void x_UnreviewedComment(CBioseqContext& ctx) const;
     void x_AuthorizedAccessComment(CBioseqContext& ctx) const;
     void x_MapComment(CBioseqContext& ctx) const;
     void x_BasemodComment(CBioseqContext& ctx) const;
     void x_IdComments(CBioseqContext& ctx,
         EGenomeAnnotComment eGenomeAnnotComment) const;
-    void x_RefSeqComments(CBioseqContext& ctx, 
+    void x_RefSeqComments(CBioseqContext& ctx,
         EGenomeAnnotComment eGenomeAnnotComment) const;
     void x_HistoryComments(CBioseqContext& ctx) const;
     void x_RefSeqGenomeComments(CBioseqContext& ctx) const;
@@ -203,9 +204,9 @@ protected:
     void x_StructuredComments(CBioseqContext& ctx) const;
     void x_UnorderedComments(CBioseqContext& ctx) const;
 
-    // sequence 
+    // sequence
     void x_GatherSequence  (void) const;
-    
+
     // types
     typedef vector< CRef<CCommentItem> > TCommentVec;
 
