@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS__PUBSEQ_GATEWAY__CLIENT__IMPL__MISC__HPP
 #define OBJTOOLS__PUBSEQ_GATEWAY__CLIENT__IMPL__MISC__HPP
 
-/*  $Id: misc.hpp 628920 2021-04-07 18:46:32Z ivanov $
+/*  $Id: misc.hpp 675367 2023-11-20 16:22:48Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,265 +35,146 @@
 #include <condition_variable>
 #include <memory>
 #include <mutex>
+#include <deque>
 #include <thread>
 
 #include <corelib/ncbitime.hpp>
 #include <corelib/ncbi_param.hpp>
 
+#include <connect/impl/connect_misc.hpp>
+
 BEGIN_NCBI_SCOPE
 
-template <int ITERS, class TBase>
-struct SPSG_CV_Base : TBase
-{
-    mutex& GetMutex() { return TBase::m_Mutex; }
-};
-
-template <int ITERS>
-struct SPSG_CV_Base<ITERS, void>
-{
-};
-
 template <>
-struct SPSG_CV_Base<0, void>
+struct SThreadSafe<void>
 {
-    mutex& GetMutex() { return m_Mutex; }
-
-private:
+protected:
     mutex m_Mutex;
 };
 
-template <int ITERS, typename TBase = void>
-struct SPSG_CV : SPSG_CV_Base<ITERS, TBase>
+template <typename TType = void>
+struct SPSG_CV : SThreadSafe<TType>
 {
-private:
-    template <int I, typename NA = void>
-    struct SImpl
-    {
-        SImpl(SPSG_CV_Base<I, TBase>&) {}
-
-        void NotifyOne() {}
-        void NotifyAll() {}
-
-        bool WaitUntil(const CDeadline& deadline)
-        {
-            return x_WaitUntil() || !deadline.IsExpired();
-        }
-
-        template <typename T = bool>
-        bool WaitUntil(const volatile atomic<T>& a, const CDeadline& deadline, T v = false, bool rv = false)
-        {
-            return x_WaitUntil() || (a != v ? rv : !deadline.IsExpired());
-        }
-
-    private:
-        bool x_WaitUntil()
-        {
-            constexpr auto kWait = chrono::microseconds(10);
-            this_thread::sleep_for(kWait);
-
-            if (m_I-- > 0) {
-                return true;
-            }
-
-            m_I += I;
-            return false;
-        }
-
-        int m_I = I;
-    };
-
-    template <typename NA>
-    struct SImpl<0, NA>
-    {
-        using clock = chrono::system_clock;
-
-        SImpl(SPSG_CV_Base<0, TBase>& base) : m_Base(base), m_Signal(0) {}
-
-        void NotifyOne() { x_Signal(); m_CV.notify_one(); }
-        void NotifyAll() { x_Signal(); m_CV.notify_all(); }
-
-        bool WaitUntil(const CDeadline& deadline)
-        {
-            return deadline.IsInfinite() ? x_Wait() : x_Wait(x_GetTP(deadline));
-        }
-
-        template <typename T = bool>
-        bool WaitUntil(const volatile atomic<T>& a, const CDeadline& deadline, T v = false, bool rv = false)
-        {
-            constexpr auto kWait = chrono::milliseconds(100);
-            const auto until = deadline.IsInfinite() ? clock::time_point::max() : x_GetTP(deadline);
-            const auto max = clock::now() + kWait;
-
-            do {
-                if (until < max) {
-                    return x_Wait(until);
-                }
-
-                if (x_Wait(max)) {
-                    return true;
-                }
-            }
-            while (a == v);
-
-            return rv;
-        }
-
-    private:
-        static clock::time_point x_GetTP(const CDeadline& d)
-        {
-            time_t seconds;
-            unsigned int nanoseconds;
-
-            d.GetExpirationTime(&seconds, &nanoseconds);
-            const auto ns = chrono::duration_cast<clock::duration>(chrono::nanoseconds(nanoseconds));
-            return clock::from_time_t(seconds) + ns;
-        }
-
-        template <class... TArgs>
-        bool x_Wait(TArgs&&... args)
-        {
-            unique_lock<mutex> lock(m_Base.GetMutex());
-            auto p = [&](){ return m_Signal > 0; };
-
-            if (!x_CvWait(lock, p, forward<TArgs>(args)...)) return false;
-
-            m_Signal--;
-            return true;
-        }
-
-        template <class TL, class TP, class TT>
-        bool x_CvWait(TL& l, TP p, const TT& t)
-        {
-            return m_CV.wait_until(l, t, p);
-        }
-
-        template <class TL, class TP>
-        bool x_CvWait(TL& l, TP p)
-        {
-            m_CV.wait(l, p); return true;
-        }
-
-        void x_Signal()
-        {
-            lock_guard<mutex> lock(m_Base.GetMutex());
-            m_Signal++;
-        }
-
-        SPSG_CV_Base<ITERS, TBase>& m_Base;
-        condition_variable m_CV;
-        int m_Signal;
-    };
-
 public:
-    SPSG_CV() : m_Impl(*this) {}
-
-    void NotifyOne() volatile { GetImpl().NotifyOne(); }
-    void NotifyAll() volatile { GetImpl().NotifyAll(); }
+    void NotifyOne() volatile { GetThis().x_NotifyOne(); }
+    void NotifyAll() volatile { GetThis().x_NotifyAll(); }
 
     template <class... TArgs>
     bool WaitUntil(TArgs&&... args) volatile
     {
-        return GetImpl().WaitUntil(forward<TArgs>(args)...);
+        return GetThis().x_WaitUntil(forward<TArgs>(args)...);
     }
 
+    bool Reset() volatile { return GetThis().x_Reset(); }
+
 private:
-    using TImpl = SImpl<ITERS>;
+    using clock = chrono::system_clock;
 
-    TImpl& GetImpl() volatile { return const_cast<TImpl&>(m_Impl); }
+    void x_NotifyOne() { x_Signal(); m_CV.notify_one(); }
+    void x_NotifyAll() { x_Signal(); m_CV.notify_all(); }
 
-    TImpl m_Impl;
-};
-
-template <class TValue>
-struct CPSG_Stack
-{
-private:
-    struct TElement
+    bool x_WaitUntil(const CDeadline& deadline)
     {
-        shared_ptr<TElement> next;
-        TValue value;
-
-        template <class... TArgs>
-        TElement(shared_ptr<TElement> n, TArgs&&... args) : next(n), value(forward<TArgs>(args)...) {}
-    };
-
-public:
-    ~CPSG_Stack() { Clear(); }
-
-    template <class... TArgs>
-    void Emplace(TArgs&&... args)
-    {
-        auto head = make_shared<TElement>(atomic_load(&m_Head), forward<TArgs>(args)...);
-
-        while (!atomic_compare_exchange_weak(&m_Head, &head->next, head));
+        return deadline.IsInfinite() ? x_Wait() : x_Wait(x_GetTP(deadline));
     }
 
-    void Push(TValue value)
+    template <typename T = bool>
+    bool x_WaitUntil(const volatile atomic<T>& a, const CDeadline& deadline, T v = false, bool rv = false)
     {
-        auto head = make_shared<TElement>(atomic_load(&m_Head), move(value));
+        constexpr auto kWait = chrono::milliseconds(100);
+        const auto until = deadline.IsInfinite() ? clock::time_point::max() : x_GetTP(deadline);
 
-        while (!atomic_compare_exchange_weak(&m_Head, &head->next, head));
-    }
+        do {
+            const auto max = clock::now() + kWait;
 
-    bool Pop(TValue& value)
-    {
-        while (auto head = atomic_load(&m_Head)) {
-            if (atomic_compare_exchange_weak(&m_Head, &head, head->next)) {
-                value = move(head->value);
+            if (until < max) {
+                return x_Wait(until);
+            }
+
+            if (x_Wait(max)) {
                 return true;
             }
         }
+        while (a == v);
 
-        return false;
+        return rv;
     }
 
-    void Clear()
+    static clock::time_point x_GetTP(const CDeadline& d)
     {
-        while (auto head = atomic_load(&m_Head)) {
-            if (atomic_compare_exchange_weak(&m_Head, &head, {})) {
-                while (auto old_head = head) {
-                    head = head->next;
-                }
-            }
-        }
+        time_t seconds;
+        unsigned int nanoseconds;
+
+        d.GetExpirationTime(&seconds, &nanoseconds);
+        const auto ns = chrono::duration_cast<clock::duration>(chrono::nanoseconds(nanoseconds));
+        return clock::from_time_t(seconds) + ns;
     }
 
-    bool Empty() const { return !atomic_load(&m_Head); }
+    template <class... TArgs>
+    bool x_Wait(TArgs&&... args)
+    {
+        unique_lock<mutex> lock(SThreadSafe<TType>::m_Mutex);
 
-private:
-    shared_ptr<TElement> m_Head;
+        if (!x_CvWait(lock, forward<TArgs>(args)...)) return false;
+
+        m_Signal--;
+        return true;
+    }
+
+    bool x_CvWait(unique_lock<mutex>& l, const clock::time_point& t)
+    {
+        return m_CV.wait_until(l, t, [&](){ return m_Signal > 0; });
+    }
+
+    bool x_CvWait(unique_lock<mutex>& l)
+    {
+        m_CV.wait(l, [&](){ return m_Signal > 0; });
+        return true;
+    }
+
+    void x_Signal()
+    {
+        lock_guard<mutex> lock(SThreadSafe<TType>::m_Mutex);
+        m_Signal++;
+    }
+
+    bool x_Reset()
+    {
+        lock_guard<mutex> lock(SThreadSafe<TType>::m_Mutex);
+        return exchange(m_Signal, 0);
+    }
+
+    SPSG_CV& GetThis() volatile { return const_cast<SPSG_CV&>(*this); }
+
+    condition_variable m_CV;
+    int m_Signal = 0;
 };
 
 template <class TValue>
-struct CPSG_WaitingStack : private CPSG_Stack<TValue>
+struct CPSG_WaitingQueue : SPSG_CV<deque<TValue>>
 {
-    CPSG_WaitingStack() : m_Stopped(false) {}
-
-    template <class... TArgs>
-    void Emplace(TArgs&&... args)
-    {
-        if (m_Stopped) return;
-
-        CPSG_Stack<TValue>::Emplace(forward<TArgs>(args)...);
-        m_CV.NotifyOne();
-    }
+    CPSG_WaitingQueue() : m_Stopped(false) {}
 
     void Push(TValue value)
     {
         if (m_Stopped) return;
 
-        CPSG_Stack<TValue>::Push(move(value));
-        m_CV.NotifyOne();
+        this->GetLock()->push_back(move(value));
+        this->NotifyOne();
     }
 
     bool Pop(TValue& value, const CDeadline& deadline = CDeadline::eInfinite)
     {
         do {
-            if (CPSG_Stack<TValue>::Pop(value)) {
-                return true;
+            if (auto locked = this->GetLock()) {
+                if (!locked->empty()) {
+                    value = move(locked->front());
+                    locked->pop_front();
+                    return true;
+                }
             }
         }
-        while (m_CV.WaitUntil(m_Stopped, deadline));
+        while (this->WaitUntil(m_Stopped, deadline));
 
         return false;
     }
@@ -302,15 +183,14 @@ struct CPSG_WaitingStack : private CPSG_Stack<TValue>
     void Stop(EStop stop)
     {
         m_Stopped.store(true);
-        if (stop == eClear) CPSG_Stack<TValue>::Clear();
-        m_CV.NotifyAll();
+        if (stop == eClear) this->GetLock()->clear();
+        this->NotifyAll();
     }
 
     const atomic_bool& Stopped() const { return m_Stopped; }
-    bool Empty() const { return m_Stopped && CPSG_Stack<TValue>::Empty(); }
+    bool Empty() const { return m_Stopped && this->GetLock()->empty(); }
 
 private:
-    SPSG_CV<0> m_CV;
     atomic_bool m_Stopped;
 };
 
@@ -321,9 +201,11 @@ struct SPSG_ParamValue
 
     // Getting default incurs some performance penalty, so this ctor is explicit
     enum EGetDefault { eGetDefault };
-    explicit SPSG_ParamValue(EGetDefault) : m_Value(TParam::GetDefault()) { _DEBUG_ARG(sm_Used = true); }
+    explicit SPSG_ParamValue(EGetDefault)                     : SPSG_ParamValue(sm_Adjust(TParam::GetDefault())) {}
+    explicit SPSG_ParamValue(function<TValue(TValue)> adjust) : SPSG_ParamValue(   adjust(TParam::GetDefault())) {}
 
     operator TValue() const { return m_Value; }
+    TValue Get() const { return m_Value; }
 
     static TValue GetDefault() { return TParam::GetDefault(); }
 
@@ -356,6 +238,8 @@ struct SPSG_ParamValue
     }
 
 private:
+    SPSG_ParamValue(TValue value) : m_Value(value) { _DEBUG_ARG(sm_Used = true); }
+
     // TDescription is not publicly available in CParam, but it's needed for string to enum conversion.
     // This templated method circumvents that shortcoming.
     template <class TDescription>
@@ -364,6 +248,8 @@ private:
         SetDefault(CParam<TDescription>::TParamParser::StringToValue(value, TDescription::sm_ParamDescription));
     }
 
+    static TValue sm_Adjust(TValue value) { return value; }
+
     TValue m_Value;
     _DEBUG_ARG(static bool sm_Used);
 };
@@ -372,6 +258,22 @@ _DEBUG_ARG(template <class TParam> bool SPSG_ParamValue<TParam>::sm_Used = false
 
 #define PSG_PARAM_VALUE_TYPE(section, name) SPSG_ParamValue<NCBI_PARAM_TYPE(section, name)>
 
+#define PSG_PARAM_VALUE_DEF_ADJUST(type, section, name, default_value)                                                          \
+    NCBI_PARAM_DEF(type, section, name, default_value);                                                                         \
+    template <>                                                                                                                 \
+    typename SPSG_ParamValue<NCBI_PARAM_TYPE(section, name)>::TValue                                                            \
+    SPSG_ParamValue<NCBI_PARAM_TYPE(section, name)>::sm_Adjust(SPSG_ParamValue<NCBI_PARAM_TYPE(section, name)>::TValue value)
+
+#define PSG_PARAM_VALUE_DEF_MIN(type, section, name, default_value, min_value)                                                  \
+    PSG_PARAM_VALUE_DEF_ADJUST(type, section, name, default_value)                                                              \
+    {                                                                                                                           \
+        if (value >= min_value) return value;                                                                                   \
+                                                                                                                                \
+        ERR_POST(Warning << "[" #section "] " #name " ('" << value << "')"                                                      \
+                " was increased to the minimum allowed value ('" #min_value "')");                                              \
+        return min_value;                                                                                                       \
+    }
+
 NCBI_PARAM_DECL(unsigned, PSG, rd_buf_size);
 typedef NCBI_PARAM_TYPE(PSG, rd_buf_size) TPSG_RdBufSize;
 
@@ -381,6 +283,15 @@ typedef NCBI_PARAM_TYPE(PSG, wr_buf_size) TPSG_WrBufSize;
 NCBI_PARAM_DECL(unsigned, PSG, max_concurrent_streams);
 typedef NCBI_PARAM_TYPE(PSG, max_concurrent_streams) TPSG_MaxConcurrentStreams;
 
+NCBI_PARAM_DECL(unsigned, PSG, max_concurrent_submits);
+using TPSG_MaxConcurrentSubmits = PSG_PARAM_VALUE_TYPE(PSG, max_concurrent_submits);
+
+NCBI_PARAM_DECL(unsigned, PSG, max_sessions);
+typedef NCBI_PARAM_TYPE(PSG, max_sessions) TPSG_MaxSessions;
+
+NCBI_PARAM_DECL(unsigned, PSG, max_concurrent_requests_per_server);
+using TPSG_MaxConcurrentRequestsPerServer = PSG_PARAM_VALUE_TYPE(PSG, max_concurrent_requests_per_server);
+
 NCBI_PARAM_DECL(unsigned, PSG, num_io);
 typedef NCBI_PARAM_TYPE(PSG, num_io) TPSG_NumIo;
 
@@ -390,8 +301,14 @@ typedef NCBI_PARAM_TYPE(PSG, reader_timeout) TPSG_ReaderTimeout;
 NCBI_PARAM_DECL(double, PSG, rebalance_time);
 typedef NCBI_PARAM_TYPE(PSG, rebalance_time) TPSG_RebalanceTime;
 
-NCBI_PARAM_DECL(unsigned, PSG, request_timeout);
-using TPSG_RequestTimeout = PSG_PARAM_VALUE_TYPE(PSG, request_timeout);
+NCBI_PARAM_DECL(double, PSG, io_timer_period);
+using TPSG_IoTimerPeriod = PSG_PARAM_VALUE_TYPE(PSG, io_timer_period);
+
+NCBI_PARAM_DECL(double, PSG, request_timeout);
+typedef NCBI_PARAM_TYPE(PSG, request_timeout) TPSG_RequestTimeout;
+
+NCBI_PARAM_DECL(double, PSG, competitive_after);
+typedef NCBI_PARAM_TYPE(PSG, competitive_after) TPSG_CompetitiveAfter;
 
 NCBI_PARAM_DECL(size_t, PSG, requests_per_io);
 using TPSG_RequestsPerIo = PSG_PARAM_VALUE_TYPE(PSG, requests_per_io);
@@ -399,21 +316,45 @@ using TPSG_RequestsPerIo = PSG_PARAM_VALUE_TYPE(PSG, requests_per_io);
 NCBI_PARAM_DECL(unsigned, PSG, request_retries);
 using TPSG_RequestRetries = PSG_PARAM_VALUE_TYPE(PSG, request_retries);
 
+NCBI_PARAM_DECL(unsigned, PSG, refused_stream_retries);
+using TPSG_RefusedStreamRetries = PSG_PARAM_VALUE_TYPE(PSG, refused_stream_retries);
+
 NCBI_PARAM_DECL(string, PSG, request_user_args);
 typedef NCBI_PARAM_TYPE(PSG, request_user_args) TPSG_RequestUserArgs;
 
+NCBI_PARAM_DECL(bool, PSG, user_request_ids);
+using TPSG_UserRequestIds = PSG_PARAM_VALUE_TYPE(PSG, user_request_ids);
+
 NCBI_PARAM_DECL(unsigned, PSG, localhost_preference);
 typedef NCBI_PARAM_TYPE(PSG, localhost_preference) TPSG_LocalhostPreference;
 
 NCBI_PARAM_DECL(bool, PSG, fail_on_unknown_items);
 typedef NCBI_PARAM_TYPE(PSG, fail_on_unknown_items) TPSG_FailOnUnknownItems;
 
+NCBI_PARAM_DECL(bool, PSG, fail_on_unknown_chunks);
+typedef NCBI_PARAM_TYPE(PSG, fail_on_unknown_chunks) TPSG_FailOnUnknownChunks;
+
 NCBI_PARAM_DECL(bool, PSG, https);
 typedef NCBI_PARAM_TYPE(PSG, https) TPSG_Https;
 
 NCBI_PARAM_DECL(double, PSG, no_servers_retry_delay);
 typedef NCBI_PARAM_TYPE(PSG, no_servers_retry_delay) TPSG_NoServersRetryDelay;
 
+NCBI_PARAM_DECL(string, PSG, service);
+using TPSG_Service = NCBI_PARAM_TYPE(PSG, service);
+
+NCBI_PARAM_DECL(string, PSG, auth_token_name);
+using TPSG_AuthTokenName = PSG_PARAM_VALUE_TYPE(PSG, auth_token_name);
+
+NCBI_PARAM_DECL(string, PSG, auth_token);
+using TPSG_AuthToken = PSG_PARAM_VALUE_TYPE(PSG, auth_token);
+
+NCBI_PARAM_DECL(bool, PSG, stats);
+typedef NCBI_PARAM_TYPE(PSG, stats) TPSG_Stats;
+
+NCBI_PARAM_DECL(double, PSG, stats_period);
+typedef NCBI_PARAM_TYPE(PSG, stats_period) TPSG_StatsPeriod;
+
 NCBI_PARAM_DECL(double, PSG, throttle_relaxation_period);
 using TPSG_ThrottlePeriod = NCBI_PARAM_TYPE(PSG, throttle_relaxation_period);
 
@@ -435,7 +376,7 @@ NCBI_PARAM_ENUM_DECL(EPSG_UseCache, PSG, use_cache);
 using TPSG_UseCache = PSG_PARAM_VALUE_TYPE(PSG, use_cache);
 
 // Performance reporting/request IDs for psg_client app
-enum class EPSG_PsgClientMode { eOff, eInteractive, ePerformance, eIo };
+enum class EPSG_PsgClientMode { eOff, ePerformance };
 NCBI_PARAM_ENUM_DECL(EPSG_PsgClientMode, PSG, internal_psg_client_mode);
 using TPSG_PsgClientMode = PSG_PARAM_VALUE_TYPE(PSG, internal_psg_client_mode);
 
