@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___FASTA_EXCEPTION__HPP
 #define OBJTOOLS_READERS___FASTA_EXCEPTION__HPP
 
-/*  $Id: fasta_exception.hpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: fasta_exception.hpp 680775 2024-03-27 13:51:53Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -32,19 +32,29 @@
 *   Exceptions for CFastaReader.
 */
 
-#include <objtools/readers/reader_exception.hpp>
-#include <objects/seqloc/Seq_id.hpp>
+#include <corelib/ncbiexpt.hpp>
+#include <corelib/ncbiobj.hpp>
 
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 
-class CBadResiduesException : public CObjReaderException
+class CSeq_id;
+
+class CBadResiduesException : public CException
 {
 public:
     enum EErrCode {
-        eBadResidues
+        eBadResidues,
+        eInvalid = -1
     };
 
+    EErrCode GetErrCode(void) const
+    {
+        return typeid(*this) == typeid(CBadResiduesException) ?
+            (EErrCode) this->x_GetErrCode() :
+            (EErrCode) CException::eInvalid;
+    }
+
     virtual const char* GetErrCodeString(void) const override
     {
         switch (GetErrCode()) {
@@ -57,12 +67,8 @@ public:
         // map lines to the index of bad residues on that line
         typedef map<int, vector<TSeqPos> > TBadIndexMap;
 
-        SBadResiduePositions(void) { }
-
-        SBadResiduePositions(
-            CConstRef<CSeq_id> seqId,
-            const TBadIndexMap & badIndexMap )
-            : m_SeqId(seqId), m_BadIndexMap(badIndexMap) { }
+        SBadResiduePositions() = default;
+        SBadResiduePositions(const SBadResiduePositions&) = default;
 
         // convenience ctor for when all bad indexes are on the same line
         SBadResiduePositions(
@@ -89,19 +95,21 @@ public:
         TBadIndexMap m_BadIndexMap;
     };
 
-    virtual void ReportExtra(ostream& out) const override;
-
-    CBadResiduesException(const CDiagCompileInfo& info,
+    void ReportExtra(ostream& out) const override;
+    CBadResiduesException(
+        const CDiagCompileInfo &info,
         const CException* prev_exception,
-        EErrCode err_code, const string& message,
+        EErrCode err_code,
+        const string& message,
         const SBadResiduePositions& badResiduePositions,
-        EDiagSev severity = eDiag_Error) THROWS_NONE
-        : CObjReaderException(info, prev_exception,
-        (CObjReaderException::EErrCode) CException::eInvalid,
-        message), m_BadResiduePositions(badResiduePositions)
-        NCBI_EXCEPTION_DEFAULT_IMPLEMENTATION(CBadResiduesException, CObjReaderException);
+        EDiagSev severity = eDiag_Error)
+        : CException(info, prev_exception, (CException::EErrCode) err_code, message, severity, 0),
+            m_BadResiduePositions(badResiduePositions)
+    {}
+
+    CBadResiduesException() = delete;
+    CBadResiduesException(const CBadResiduesException&) = default;
 
-public:
     // Returns the bad residues found, which might not be complete
     // if we bailed out early.
     const SBadResiduePositions& GetBadResiduePositions(void) const THROWS_NONE
@@ -114,6 +122,12 @@ public:
         return m_BadResiduePositions.m_BadIndexMap.empty();
     }
 
+protected:
+    const CException* x_Clone(void) const override
+    {
+        return new CBadResiduesException(*this);
+    }
+
 private:
     SBadResiduePositions m_BadResiduePositions;
 };
