@@ -1,4 +1,4 @@
- /*  $Id: gff3_reader.hpp 632526 2021-06-02 17:25:01Z ivanov $
+ /*  $Id: gff3_reader.hpp 663694 2023-02-27 16:15:10Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,8 +50,8 @@ public:
     CGff3ReadRecord() {};
     ~CGff3ReadRecord() {};
 
-    virtual bool AssignFromGff(
-        const string& );
+    bool AssignFromGff(
+        const string& ) override;
 
 protected:
     string x_NormalizedAttributeKey(
@@ -87,10 +87,10 @@ public:
         //range 12..23
         fGeneXrefs = (0x1 << 12),
     };
-    typedef unsigned int TReaderFlags;
+    //typedef unsigned int TReaderFlags;
 
     CGff3Reader(
-        unsigned int uFlags,
+        TReaderFlags uFlags,
         const string& name = "",
         const string& title = "",
         SeqIdResolver resolver = CReadUtil::AsSeqId,
@@ -102,32 +102,39 @@ public:
 
     virtual ~CGff3Reader();
 
-    virtual CRef<CSeq_annot>
+    CRef<CSeq_annot>
     ReadSeqAnnot(
         ILineReader& lr,
-        ILineErrorListener* pErrors=nullptr);
+        ILineErrorListener* pErrors=nullptr) override;
 
     TSeqPos SequenceSize() const;
 
-protected:
-    virtual void xProcessData(
-        const TReaderData&,
-        CSeq_annot&);
+    TSeqPos GetSequenceSize(
+        const string&) const;
 
-    virtual CGff3ReadRecord* x_CreateRecord() { return new CGff3ReadRecord(); };
+    shared_ptr<CGff3LocationMerger> GetLocationMerger() {
+        return mpLocations;
+    };
+
+protected:
+    void xProcessData(
+        const TReaderData&,
+        CSeq_annot&) override;
+
+    CGff3ReadRecord* x_CreateRecord() override { return new CGff3ReadRecord(); };
 
     virtual bool xInitializeFeature(
         const CGff2Record&,
         CRef<CSeq_feat> );
 
-    virtual bool xUpdateAnnotFeature(
+    bool xUpdateAnnotFeature(
         const CGff2Record&,
         CSeq_annot&,
-        ILineErrorListener*);
+        ILineErrorListener*) override;
 
-    virtual bool xAddFeatureToAnnot(
+    bool xAddFeatureToAnnot(
         CRef< CSeq_feat >,
-        CSeq_annot& );
+        CSeq_annot& ) override;
 
     virtual bool xUpdateAnnotExon(
         const CGff2Record&,
@@ -163,7 +170,7 @@ protected:
         CSeq_annot&,
         ILineErrorListener*);
 
-    virtual bool xUpdateAnnotMrna(
+    virtual bool xUpdateAnnotRna(
         const CGff2Record&,
         CRef<CSeq_feat>,
         CSeq_annot&,
@@ -176,6 +183,9 @@ protected:
     void xVerifyCdsParents(
         const CGff2Record&);
 
+    virtual void xValidateAnnot(
+        const CSeq_annot&) override;
+
     virtual bool xFeatureSetXrefGrandParent(
         const string&,
         CRef<CSeq_feat>);
@@ -184,7 +194,7 @@ protected:
         const string&,
         CRef<CSeq_feat>);
 
-    virtual bool xReadInit();
+    bool xReadInit() override;
 
     static string xNextGenericId();
 
@@ -195,8 +205,8 @@ protected:
         const string&,
         const CGff2Record&);
 
-    virtual bool xIsIgnoredFeatureType(
-        const string&);
+    bool xIsIgnoredFeatureType(
+        const string&) override;
 
     virtual void xAddPendingExon(
         const string&,
@@ -206,22 +216,22 @@ protected:
         const string&,
         list<CGff2Record>&);
 
-    virtual void xPostProcessAnnot(
-        CSeq_annot&);
+    void xPostProcessAnnot(
+        CSeq_annot&) override;
 
     void xProcessAlignmentData(
         CSeq_annot& pAnnot);
 
-    virtual bool xParseFeature(
+    bool xParseFeature(
         const string&,
         CSeq_annot&,
-        ILineErrorListener*);
+        ILineErrorListener*) override;
 
     virtual bool xParseAlignment(
         const string& strLine);
 
-    virtual void xProcessSequenceRegionPragma(
-        const string& pragma);
+    void xProcessSequenceRegionPragma(
+        const string& pragma) override;
 
     // Data:
     map<string, string> mCdsParentMap;
@@ -232,7 +242,7 @@ protected:
     using PENDING_EXONS = map<string, list<CGff2Record> >;
     PENDING_EXONS mPendingExons;
 
-    unique_ptr<CGff3LocationMerger> mpLocations;
+    shared_ptr<CGff3LocationMerger> mpLocations;
     static unsigned int msGenericIdCounter;
 };
 
