@@ -1,4 +1,4 @@
-/*  $Id: gff_base_columns.hpp 632531 2021-06-02 17:25:37Z ivanov $
+/*  $Id: gff_base_columns.hpp 665420 2023-04-04 13:16:28Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,7 +44,8 @@ class CGffBaseColumns
 {
 public:
     using TFrame = CCdregion::EFrame;
-    using SeqIdResolver = CRef<CSeq_id> (*)(const string&, unsigned int, bool);
+    using TReaderFlags = long; //technical debt: duplicated in reader_base
+    using SeqIdResolver = CRef<CSeq_id> (*)(const string&, TReaderFlags, bool);
 
     CGffBaseColumns();
 
@@ -93,28 +94,28 @@ public:
     };
 
     bool IsSetScore() const {
-        return m_pdScore != 0;
-    };
+        return m_pdScore != nullptr;
+    }
 
     bool IsSetStrand() const {
-        return m_peStrand != 0;
-    };
+        return m_peStrand != nullptr;
+    }
 
     bool IsSetPhase() const {
-        return m_pePhase != 0;
-    };
+        return m_pePhase != nullptr;
+    }
 
     CRef<CSeq_id> GetSeqId(
-        int,
+        TReaderFlags,
         SeqIdResolver = nullptr ) const;
 
     CRef<CSeq_loc> GetSeqLoc(
-        int,
+        TReaderFlags,
         SeqIdResolver seqidresolve = nullptr) const;
 
     // feature initialization:
     virtual bool InitializeFeature(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>,
         SeqIdResolver = nullptr ) const;
 
@@ -132,16 +133,16 @@ public:
     }
 
     virtual bool xInitFeatureId(
-        int, //flags
+        TReaderFlags,
         CRef<CSeq_feat> ) const;
 
     virtual bool xInitFeatureLocation(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>,
         SeqIdResolver = nullptr ) const;
 
     virtual bool xInitFeatureData(
-        int,
+        TReaderFlags,
         CRef<CSeq_feat>) const;
 
     // utility:
