@@ -1,4 +1,4 @@
- /*  $Id: gvf_reader.hpp 632526 2021-06-02 17:25:01Z ivanov $
+ /*  $Id: gvf_reader.hpp 663694 2023-02-27 16:15:10Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -53,21 +53,21 @@ class CGvfReadRecord
 public:
     CGvfReadRecord(
         unsigned int lineNumber,
-        ILineErrorListener* pEC = 0):
+        ILineErrorListener* pEC = nullptr):
         mLineNumber(lineNumber),
         mpMessageListener(pEC)
     {};
     ~CGvfReadRecord() {};
 
-    virtual bool AssignFromGff(
-        const string& );
+    bool AssignFromGff(
+        const string& ) override;
 
     bool SanityCheck() const;
 
 protected:
-    virtual bool xAssignAttributesFromGff(
+    bool xAssignAttributesFromGff(
         const string&,
-        const string& );
+        const string& ) override;
 
     unsigned int mLineNumber;
     ILineErrorListener* mpMessageListener;
@@ -80,41 +80,37 @@ class NCBI_XOBJREAD_EXPORT CGvfReader
 {
 public:
     CGvfReader(
-        unsigned int uFlags,
+        TReaderFlags uFlags,
         const string& name = "",
         const string& title = "",
         CReaderListener* = nullptr);
 
     virtual ~CGvfReader();
 
-    virtual CRef<CSeq_annot>
+    CRef<CSeq_annot>
     ReadSeqAnnot(
         ILineReader&,
-        ILineErrorListener* = nullptr );
+        ILineErrorListener* = nullptr ) override;
 
 protected:
-    virtual void xGetData(
+    void xGetData(
         ILineReader&,
-        TReaderData&);
+        TReaderData&) override;
 
-    virtual void xProcessData(
+    void xProcessData(
         const TReaderData&,
-        CSeq_annot&);
+        CSeq_annot&) override;
 
-    virtual bool xParseStructuredComment(
-        const string&);
+    bool xParseStructuredComment(
+        const string&) override;
 
     bool xParseFeature(
         const string&,
         CSeq_annot&,
-        ILineErrorListener*);
+        ILineErrorListener*) override;
 
-    virtual void xPostProcessAnnot(
-        CSeq_annot&);
-
-    CRef<CSeq_annot> x_GetAnnotById(
-        TAnnots& annots,
-        const string& strId );
+    void xPostProcessAnnot(
+        CSeq_annot&) override;
 
     virtual bool xMergeRecord(
         const CGvfReadRecord&,
@@ -133,18 +129,6 @@ protected:
         const CGff2Record&,
         CSeq_feat&);
 
-    bool xSetLocation(
-        const CGff2Record&,
-        CSeq_loc&);
-
-    bool xSetLocationInterval(
-        const CGff2Record&,
-        CSeq_loc&);
-
-    bool xSetLocationPoint(
-        const CGff2Record&,
-        CSeq_loc&);
-
     bool xFeatureSetVariation(
         const CGvfReadRecord&,
         CSeq_feat&);
@@ -226,7 +210,7 @@ protected:
         const CGvfReadRecord&,
         CVariation_ref&);
 
-    virtual CGff3ReadRecord* x_CreateRecord() { return new CGvfReadRecord(m_uLineNumber); };
+    CGff3ReadRecord* x_CreateRecord() override { return new CGvfReadRecord(m_uLineNumber); };
 
     bool xIsDbvarCall(
         const string& nameAttr) const;
