@@ -1,4 +1,4 @@
-/*  $Id: message_listener.hpp 608381 2020-05-15 12:43:35Z ivanov $
+/*  $Id: message_listener.hpp 684363 2024-06-20 19:34:09Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -45,58 +45,40 @@ BEGIN_SCOPE(objects) // namespace ncbi::objects::
 
 //  ============================================================================
 class ILineErrorListener : public CObject, public IObjtoolsListener
-//  ============================================================================
 {
 public:
-    virtual ~ILineErrorListener() {}
+    ~ILineErrorListener() override {}
 
-    // IListener::Post() implementation
-    NCBI_STD_DEPRECATED("This method is no longer functional and will be removed in SC-25.") 
-    virtual void Post(const IMessage& /*message*/)
-    {
-        // Remove in SC-25
-        return;
-    }
-
-    NCBI_STD_DEPRECATED("This method is redundant and will be removed in SC-25.") 
-    virtual void Push(const IObjtoolsMessage& message) 
-    {
-        // Remove in SC-25
-        PutMessage(message);
-    }
-    /// Store error in the container, and 
+    /// Store error in the container, and
     /// return true if error was stored fine, and
     /// return false if the caller should terminate all further processing.
     ///
-    virtual bool
-    PutError(
-        const ILineError& ) = 0;
+    virtual bool PutError(const ILineError&) = 0;
 
-    virtual bool 
-    PutMessage(const IObjtoolsMessage& message) {
+    bool PutMessage(const IObjtoolsMessage& message) override
+    {
         const ILineError* le = dynamic_cast<const ILineError*>(&message);
-        if (!le) return true;
+        if (! le)
+            return true;
         return PutError(*le);
     }
-    
+
     // IListener::Get() implementation
     virtual const ILineError& Get(size_t index) const
-    { return this->GetError(index); }
+    {
+        return this->GetError(index);
+    }
 
     /// 0-based error retrieval.
-    virtual const ILineError&
-    GetError(size_t ) const =0;
+    virtual const ILineError& GetError(size_t) const = 0;
 
-    virtual size_t Count(void) const = 0;
+    virtual size_t Count() const = 0;
 
     /// Returns the number of errors seen so far at the given severity.
-    virtual size_t
-    LevelCount(
-        EDiagSev )=0;
+    virtual size_t LevelCount(EDiagSev) = 0;
 
     /// Clear all accumulated messages.
-    virtual void
-    ClearAll() =0;
+    virtual void ClearAll() = 0;
 
     // IListener::Progress() implementation
     virtual void Progress(const string& message,
@@ -104,102 +86,87 @@ public:
                           Uint8         total) { PutProgress(message, current, total); }
 
     /// This is used for processing progress messages.
-    virtual void
-    PutProgress(
-        const string & sMessage,
-        const Uint8 iNumDone = 0,
-        const Uint8 iNumTotal = 0 ) = 0;
+    virtual void PutProgress(
+        const string& sMessage,
+        const Uint8   iNumDone  = 0,
+        const Uint8   iNumTotal = 0) = 0;
 
     virtual const ILineError& GetMessage(size_t index) const
-    { return Get(index); }
+    {
+        return Get(index);
+    }
 
-    virtual void Clear(void)
-    { ClearAll(); }
-};
-
-
-// Compatibility declaration, avoid using objects::IMessageListener -
-// use ncbi::IMessageListener or objects::ILineErrorListener instead.
-class IMessageListener : public ILineErrorListener
-{
-public:
-    virtual ~IMessageListener(void) {}
+    virtual void Clear()
+    {
+        ClearAll();
+    }
 };
 
 
 //  ============================================================================
-class NCBI_XOBJREAD_EXPORT CMessageListenerBase : public objects::IMessageListener
-//  ============================================================================
+class NCBI_XOBJREAD_EXPORT CMessageListenerBase : public ILineErrorListener
 {
 public:
-    CMessageListenerBase() : m_pProgressOstrm(0) {};
-    virtual ~CMessageListenerBase() {};
-    
+    CMessageListenerBase() :
+        m_pProgressOstrm(nullptr) {}
+    ~CMessageListenerBase() override {}
+
 public:
-    size_t
-    Count() const { return m_Errors.size(); };
-    
-    virtual size_t
-    LevelCount(
-        EDiagSev eSev ) {
-        
-        size_t uCount( 0 );
-        for ( size_t u=0; u < Count(); ++u ) {
-            if ( m_Errors[u]->GetSeverity() == eSev ) ++uCount;
+    size_t Count() const override { return m_Errors.size(); }
+
+    size_t LevelCount(EDiagSev eSev) override
+    {
+        size_t uCount(0);
+        for (size_t u = 0; u < Count(); ++u) {
+            if (m_Errors[u]->GetSeverity() == eSev)
+                ++uCount;
         }
         return uCount;
-    };
-    
-    void
-    ClearAll() { m_Errors.clear(); };
-    
-    const ILineError&
-    GetError(
-        size_t uPos ) const { 
-            return *dynamic_cast<ILineError*>(m_Errors[ uPos ].get()); }
-    
+    }
+
+    void ClearAll() override { m_Errors.clear(); }
+
+    const ILineError& GetError(size_t uPos) const override
+    {
+        return *dynamic_cast<ILineError*>(m_Errors[uPos].get());
+    }
+
     virtual void Dump()
     {
         if (m_pProgressOstrm)
             Dump(*m_pProgressOstrm);
     }
 
-    virtual void Dump(
-        std::ostream& out )
+    virtual void Dump(std::ostream& out)
     {
-        if ( m_Errors.size() ) {
+        if (m_Errors.size()) {
             TLineErrVec::iterator it;
-            for ( it = m_Errors.begin(); it != m_Errors.end(); ++it ) {
-                (*it)->Dump( out );
+            for (it = m_Errors.begin(); it != m_Errors.end(); ++it) {
+                (*it)->Dump(out);
                 out << endl;
             }
-        }
-        else {
+        } else {
             out << "(( no errors ))" << endl;
         }
-    };
+    }
 
-    virtual void DumpAsXML(
-        std::ostream& out )
+    virtual void DumpAsXML(std::ostream& out)
     {
-        if ( m_Errors.size() ) {
+        if (m_Errors.size()) {
             TLineErrVec::iterator it;
-            for ( it = m_Errors.begin(); it != m_Errors.end(); ++it ) {
-                (*it)->DumpAsXML( out );
+            for (it = m_Errors.begin(); it != m_Errors.end(); ++it) {
+                (*it)->DumpAsXML(out);
                 out << endl;
             }
-        }
-        else {
+        } else {
             out << "(( no errors ))" << endl;
         }
-    };
+    }
 
-
-    virtual void
-    PutProgress(
-        const string & sMessage,
-        const Uint8 iNumDone,
-        const Uint8 iNumTotal );
+    void PutProgress(
+        const string& sMessage,
+        const Uint8   iNumDone,
+        const Uint8   iNumTotal) override;
 
     /// This sets the stream to which progress messages are written.
     ///
@@ -209,14 +176,13 @@ public:
     /// @param eNcbiOwnership
     ///   Indicates whether this CMessageListenerBase should own
     ///   pProgressOstrm.
-    virtual void 
-    SetProgressOstream(
-        CNcbiOstream * pProgressOstrm,
-        ENcbiOwnership eNcbiOwnership = eNoOwnership )
+    virtual void SetProgressOstream(
+        CNcbiOstream*  pProgressOstrm,
+        ENcbiOwnership eNcbiOwnership = eNoOwnership)
     {
         m_pProgressOstrm = pProgressOstrm;
-        if( eNcbiOwnership == eTakeOwnership && pProgressOstrm ) {
-            m_progressOstrmDestroyer.reset( pProgressOstrm );
+        if (eNcbiOwnership == eTakeOwnership && pProgressOstrm) {
+            m_progressOstrmDestroyer.reset(pProgressOstrm);
         } else {
             m_progressOstrmDestroyer.reset();
         }
@@ -226,22 +192,21 @@ private:
     // private so later we can change the structure if
     // necessary (e.g. to have indexing and such to speed up
     // level-counting)
-    // typedef std::vector< AutoPtr<ILineError> > TLineErrVec;
+    // typedef std::vector< unique_ptr<ILineError> > TLineErrVec;
 
-    using TLineErrVec = vector<AutoPtr<IObjtoolsMessage>>;
+    using TLineErrVec = vector<unique_ptr<IObjtoolsMessage>>;
     TLineErrVec m_Errors;
 
 
     // The stream to which progress messages are written.
     // If NULL, progress messages are not written.
-    CNcbiOstream * m_pProgressOstrm;
+    CNcbiOstream* m_pProgressOstrm;
 
     // do not read this pointer.  It's just used to make
     // sure m_pProgressOstrm is destroyed if we own it.
-    AutoPtr<CNcbiOstream> m_progressOstrmDestroyer;
+    unique_ptr<CNcbiOstream> m_progressOstrmDestroyer;
 
 protected:
-
     // Child classes should use this to store errors
     // into m_Errors
     void StoreError(const ILineError& err)
@@ -251,147 +216,116 @@ protected:
 
     void StoreMessage(const IObjtoolsMessage& message)
     {
-        m_Errors.emplace_back(dynamic_cast<IObjtoolsMessage*>(message.Clone()));
+        m_Errors.emplace_back(message.Clone());
     }
 };
 
 //  ============================================================================
-class CMessageListenerLenient:
-//
 //  Accept everything.
-//  ============================================================================
-    public CMessageListenerBase
+class CMessageListenerLenient : public CMessageListenerBase
 {
 public:
-    CMessageListenerLenient() {};
-    ~CMessageListenerLenient() {};
+    CMessageListenerLenient() {}
+    ~CMessageListenerLenient() override {}
 
-    bool
-    PutMessage(
-        const IObjtoolsMessage& message)
+    bool PutMessage(const IObjtoolsMessage& message) override
     {
         StoreMessage(message);
         return true;
     }
-    
-    bool
-    PutError(
-        const ILineError& err ) 
+
+    bool PutError(const ILineError& err) override
     {
         return PutMessage(err);
-    };
-};        
+    }
+};
 
 //  ============================================================================
-class CMessageListenerStrict:
-//
 //  Don't accept any errors, at all.
-//  ============================================================================
-    public CMessageListenerBase
+class CMessageListenerStrict : public CMessageListenerBase
 {
 public:
-    CMessageListenerStrict() {};
-    ~CMessageListenerStrict() {};
-    
-    bool 
-    PutMessage(
-        const IObjtoolsMessage& message)
+    CMessageListenerStrict() {}
+    ~CMessageListenerStrict() override {}
+
+    bool PutMessage(const IObjtoolsMessage& message) override
     {
         StoreMessage(message);
         return false;
     }
 
-    bool
-    PutError(
-        const ILineError& err ) 
+    bool PutError(const ILineError& err) override
     {
         return PutMessage(err);
-    };
-};        
+    }
+};
 
 //  ===========================================================================
-class CMessageListenerCount:
-//
 //  Accept up to <<count>> errors, any level.
-//  ===========================================================================
-    public CMessageListenerBase
+class CMessageListenerCount : public CMessageListenerBase
 {
 public:
     CMessageListenerCount(
-        size_t uMaxCount ): m_uMaxCount( uMaxCount ) {};
-    ~CMessageListenerCount() {};
-   
-    bool PutMessage(
-        const IObjtoolsMessage& message)
+        size_t uMaxCount) :
+        m_uMaxCount(uMaxCount) {}
+    ~CMessageListenerCount() override {}
+
+    bool PutMessage(const IObjtoolsMessage& message) override
     {
         StoreMessage(message);
         return (Count() < m_uMaxCount);
     }
 
-
-    bool
-    PutError(
-        const ILineError& err ) 
+    bool PutError(const ILineError& err) override
     {
         return PutMessage(err);
-    };    
+    }
+
 protected:
     size_t m_uMaxCount;
 };
 
 //  ===========================================================================
-class CMessageListenerLevel:
-//
 //  Accept evrything up to a certain level.
-//  ===========================================================================
-    public CMessageListenerBase
+class CMessageListenerLevel : public CMessageListenerBase
 {
 public:
-    CMessageListenerLevel(
-        int iLevel ): m_iAcceptLevel( iLevel ) {};
-    ~CMessageListenerLevel() {};
-   
-    bool 
-    PutMessage(
-        const IObjtoolsMessage& message) 
+    CMessageListenerLevel(int iLevel) :
+        m_iAcceptLevel(iLevel) {}
+    ~CMessageListenerLevel() override {}
+
+    bool PutMessage(const IObjtoolsMessage& message) override
     {
         StoreMessage(message);
         return (message.GetSeverity() <= m_iAcceptLevel);
     }
 
-    bool
-    PutError(
-        const ILineError& err ) 
+    bool PutError(const ILineError& err) override
     {
         return PutMessage(err);
-    };    
-    protected:
-        int m_iAcceptLevel;
-    };
+    }
+
+protected:
+    int m_iAcceptLevel;
+};
 
-    //  ===========================================================================
-    class CMessageListenerWithLog:
-    //
-    //  Accept everything, and besides storing all errors, post them.
 //  ===========================================================================
-    public CMessageListenerBase
+//  Accept everything, and besides storing all errors, post them.
+class CMessageListenerWithLog : public CMessageListenerBase
 {
 public:
-    CMessageListenerWithLog(const CDiagCompileInfo& info)
-        : m_Info(info) {};
-    ~CMessageListenerWithLog() {};
+    CMessageListenerWithLog(const CDiagCompileInfo& info) :
+        m_Info(info) {}
+    ~CMessageListenerWithLog() override {}
 
-    bool
-    PutError(
-        const ILineError& err )
+    bool PutError(const ILineError& err) override
     {
-        CNcbiDiag(m_Info, err.Severity(),
-                  eDPF_Log | eDPF_IsMessage).GetRef()
-           << err.Message() << Endm;
+        CNcbiDiag(m_Info, err.Severity(), eDPF_Log | eDPF_IsMessage).GetRef()
+            << err.Message() << Endm;
 
         StoreError(err);
         return true;
-    };
+    }
 
 private:
     const CDiagCompileInfo m_Info;
@@ -399,14 +333,13 @@ private:
 
 
 //  ===========================================================================
-class NCBI_XOBJREAD_EXPORT CGPipeMessageListener :
-//  ===========================================================================
-    public CMessageListenerBase 
+class NCBI_XOBJREAD_EXPORT CGPipeMessageListener : public CMessageListenerBase
 {
 public:
-    CGPipeMessageListener(bool ignoreBadModValue=false);
+    CGPipeMessageListener(bool ignoreBadModValue = false);
 
     bool PutError(const ILineError& err) override final;
+
 private:
     bool m_IgnoreBadModValue;
 };
@@ -416,4 +349,3 @@ END_SCOPE(objects)
 END_NCBI_SCOPE
 
 #endif // OBJTOOLS_READERS___MESSAGELISTENER__HPP
-
