@@ -1,4 +1,4 @@
-/*  $Id: reader_base.hpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: reader_base.hpp 665420 2023-04-04 13:16:28Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -81,7 +81,7 @@ public:
 
         fAsRaw = 1<<3,
     };
-    typedef unsigned int TReaderFlags;
+    using TReaderFlags = long;
     enum ObjectType {
         OT_UNKNOWN,
         OT_SEQANNOT,
@@ -92,7 +92,7 @@ public:
     typedef TAnnots::iterator TAnnotIt;
     typedef TAnnots::const_iterator TAnnotCit;
 
-    using SeqIdResolver = CRef<CSeq_id> (*)(const string&, unsigned int, bool);
+    using SeqIdResolver = CRef<CSeq_id> (*)(const string&, long, bool);
 
 protected:
     /// Protected constructor. Use GetReader() to get an actual reader object.
@@ -122,7 +122,7 @@ public:
     static CReaderBase* GetReader(
         CFormatGuess::EFormat format,
         TReaderFlags flags = 0,
-        CReaderListener* = nullptr );
+        CReaderListener* = nullptr);
 
     /// Read an object from a given input stream, render it as the most
     /// appropriate Genbank object.
@@ -134,7 +134,7 @@ public:
     virtual CRef< CSerialObject >
     ReadObject(
         CNcbiIstream& istr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors = nullptr);
 
     /// Read an object from a given line reader, render it as the most
     /// appropriate Genbank object. This will be Seq-annot by default
@@ -151,7 +151,7 @@ public:
     virtual CRef< CSerialObject >
     ReadObject(
         ILineReader& lr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors = nullptr);
 
     /// Read an object from a given input stream, render it as a single
     /// Seq-annot. Return empty Seq-annot otherwise.
@@ -163,7 +163,7 @@ public:
     virtual CRef< CSeq_annot >
     ReadSeqAnnot(
         CNcbiIstream& istr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors = nullptr);
 
     /// Read an object from a given line reader, render it as a single
     /// Seq-annot, if possible. Return empty Seq-annot otherwise.
@@ -175,7 +175,7 @@ public:
     virtual CRef< CSeq_annot >
     ReadSeqAnnot(
         ILineReader& lr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors = nullptr);
 
     /// Read all objects from given insput stream, returning them as a vector of
     /// Seq-annots.
@@ -190,7 +190,7 @@ public:
     ReadSeqAnnots(
         TAnnots& annots,
         CNcbiIstream& istr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors = nullptr);
 
     /// Read all objects from given insput stream, returning them as a vector of
     /// Seq-annots.
@@ -205,7 +205,7 @@ public:
     ReadSeqAnnots(
         TAnnots& annots,
         ILineReader& lr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors = nullptr);
 
     /// Read an object from a given input stream, render it as a single
     /// Seq-entry, if possible. Return empty Seq-entry otherwise.
@@ -217,7 +217,7 @@ public:
     virtual CRef< CSeq_entry >
     ReadSeqEntry(
         CNcbiIstream& istr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors = nullptr);
 
     /// Read an object from a given line reader, render it as a single
     /// Seq-entry, if possible. Return empty Seq-entry otherwise.
@@ -229,7 +229,7 @@ public:
     virtual CRef< CSeq_entry >
     ReadSeqEntry(
         ILineReader& lr,
-        ILineErrorListener* pErrors=0 );
+        ILineErrorListener* pErrors = nullptr);
 
     void
     SetProgressReportInterval(
@@ -237,7 +237,7 @@ public:
 
     void
     SetCanceler(
-        ICanceled* =0);
+        ICanceled* = nullptr);
 
     bool
     IsCanceled() const { return m_pCanceler && m_pCanceler->IsCanceled(); };
@@ -263,6 +263,9 @@ protected:
         const TReaderData&,
         CSeq_annot&);
 
+    virtual void xValidateAnnot(
+        const CSeq_annot&) {};
+
     virtual bool xGetLine(
         ILineReader&,
         string&);
@@ -292,9 +295,6 @@ protected:
     virtual bool xParseTrackLine(
         const string&);
 
-    virtual bool xParseBrowserLine(
-        const string&);
-
     virtual void xSetBrowserRegion(
         const string&,
         CAnnot_descr&);
@@ -355,6 +355,13 @@ protected:
     xProcessUnknownException(
         const CException&);
 
+    static void
+    xAddStringFlagsWithMap(
+        const list<string>& stringFlags,
+        const  map<string, TReaderFlags> flagMap,
+        TReaderFlags& baseFlags);
+        
+        
     //
     //  Data:
     //
