@@ -62,23 +62,30 @@ public:
     /// Note that "bad key" could mean many different conditions, from
     /// unrecognized feature name to qualifier without a feature, and so on.
     enum EFlags {
-        fReportBadKey           = (1<<0), ///< = 0x01 (Report bad keys into the error container)
+        fReportBadKey NCBI_STD_DEPRECATED("This flag is redundant. Bad keys are now reported by default"  
+                                          " and fSuppressBadKeyWarning must be specified to silence these warnings.")         
+                                = (1<<0), ///< = 0x01 (Report bad keys into the error container)
         fKeepBadKey             = (1<<1), ///< = 0x02 (As much as possible, try to use bad keys as if they were acceptable)
         fTranslateBadKey        = (1<<2), ///< = 0x04 (yields misc_feature /standard_name="...")
         fIgnoreWebComments      = (1<<3), ///< = 0x08 (ignore web comment lines such as lines that start with " INFO:", or consist of many equals signs)
-        fCreateGenesFromCDSs    = (1<<4), ///< = 0x10 (If a CDS has a gene xref, create a gene with the same intervals if one doesn't already exist.)
-        fCDSsMustBeInTheirGenes = (1<<5), ///< = 0x20 (If a CDS has a gene xref, it *must* be inside of that gene)
-        fReportDiscouragedKey   = (1<<6), ///< = 0x40 (Report discouraged keys into the error container)
-        fLeaveProteinIds        = (1<<7), ///< = 0x80 (Leave all protein_id as a qualifiers)
-        fAllIdsAsLocal          = (1<<8), ///< = 0xA0 (Do not attempt to parse accessions)
-        fPreferGenbankId        = (1<<9), ///< = 0xB0 (Prefer Genbank accession ids)
+        fCreateGenesFromCDSs    = (1<<4),  ///< = 0x10 (If a CDS has a gene xref, create a gene with the same intervals if one doesn't already exist.)
+        fCDSsMustBeInTheirGenes = (1<<5),  ///< = 0x20 (If a CDS has a gene xref, it *must* be inside of that gene)
+        fReportDiscouragedKey   = (1<<6),  ///< = 0x40 (Report discouraged keys into the error container)
+        fLeaveProteinIds        = (1<<7),  ///< = 0x80 (Leave all protein_id as a qualifiers)
+        fAllIdsAsLocal          = (1<<8),  ///< = 0x100 (Do not attempt to parse accessions)
+        fPreferGenbankId        = (1<<9),  ///< = 0x200 (Prefer Genbank accession ids)
+        fSuppressBadKeyWarning  = (1<<10), ///< = 0x400 (Suppress 'bad key' errors; Not recommended.)
+        fIncludeObjectInMsg     = (1<<11), ///< = 0x800 (Include reference to feature object in message). 
     };
-    typedef int TFlags;
+    using TFlags = long; ///< binary OR of EFlags
+    static void AddStringFlags(
+        const list<string>& stringFlags,
+        TFlags& baseFlags);
 
     CFeature_table_reader(TReaderFlags fReaderFlags = 0);
 
     CFeature_table_reader(ILineReader& lr,
-                          ILineErrorListener* pErrors=0);
+                          ILineErrorListener* pErrors = nullptr);
 
     ~CFeature_table_reader();
 
@@ -90,67 +97,67 @@ public:
     CRef<CSeq_annot>    ReadSeqAnnot (ILineReader &lr, ILineErrorListener *pErrors) override;
 
     CRef<CSeq_annot> ReadSequinFeatureTable(const TFlags flags=0,
-                                            ITableFilter* filter=0,
+                                            ITableFilter* filter = nullptr,
                                             const string& seqid_prefix = kEmptyStr);
 
     // read 5-column feature table and return Seq-annot
     static CRef<CSeq_annot> ReadSequinFeatureTable(ILineReader& reader,
                                                    const TFlags flags = 0,
-                                                   ILineErrorListener* pMessageListener=0,
-                                                   ITableFilter *filter = 0,
+                                                   ILineErrorListener* pMessageListener = nullptr,
+                                                   ITableFilter* filter = nullptr,
                                                    const string& seqid_prefix = kEmptyStr);
 
-    static CRef<CSeq_annot> ReadSequinFeatureTable (CNcbiIstream& ifs,
-                                                    const TFlags flags = 0,
-                                                    ILineErrorListener* pMessageListener=0,
-                                                   ITableFilter *filter = 0);
+    static CRef<CSeq_annot> ReadSequinFeatureTable(CNcbiIstream& ifs,
+                                                   const TFlags flags = 0,
+                                                   ILineErrorListener* pMessageListener = nullptr,
+                                                   ITableFilter* filter = nullptr);
 
-    static CRef<CSeq_annot> ReadSequinFeatureTable (ILineReader& reader,
-                                                    const string& seqid,
-                                                    const string& annotname,
-                                                    const TFlags flags = 0,
-                                                    ILineErrorListener* pMessageListener=0,
-                                                   ITableFilter *filter = 0);
+    static CRef<CSeq_annot> ReadSequinFeatureTable(ILineReader& reader,
+                                                   const string& seqid,
+                                                   const string& annotname,
+                                                   const TFlags flags = 0,
+                                                   ILineErrorListener* pMessageListener = nullptr,
+                                                   ITableFilter* filter = nullptr);
 
-    static CRef<CSeq_annot> ReadSequinFeatureTable (CNcbiIstream& ifs,
-                                                    const string& seqid,
-                                                    const string& annotname,
-                                                    const TFlags flags = 0,
-                                                    ILineErrorListener* pMessageListener=0,
-                                                   ITableFilter *filter = 0);
+    static CRef<CSeq_annot> ReadSequinFeatureTable(CNcbiIstream& ifs,
+                                                   const string& seqid,
+                                                   const string& annotname,
+                                                   const TFlags flags = 0,
+                                                   ILineErrorListener* pMessageListener = nullptr,
+                                                   ITableFilter* filter = nullptr);
 
     // read all feature tables available from the input, attaching each
     // at an appropriate position within the Seq-entry object
     static void ReadSequinFeatureTables(ILineReader& reader,
                                         CSeq_entry& entry,
                                         const TFlags flags = 0,
-                                        ILineErrorListener* pMessageListener=0,
-                                        ITableFilter *filter = 0);
+                                        ILineErrorListener* pMessageListener = nullptr,
+                                        ITableFilter* filter = nullptr);
 
     static void ReadSequinFeatureTables(CNcbiIstream& ifs,
                                         CSeq_entry& entry,
                                         const TFlags flags = 0,
-                                        ILineErrorListener* pMessageListener=0,
-                                        ITableFilter *filter = 0);
+                                        ILineErrorListener* pMessageListener = nullptr,
+                                        ITableFilter* filter = nullptr);
 
     // create single feature from key
-    static CRef<CSeq_feat> CreateSeqFeat (const string& feat,
-                                          CSeq_loc& location,
-                                          const TFlags flags = 0,
-                                          ILineErrorListener* pMessageListener = 0,
-                                          unsigned int line = 0,
-                                          std::string *seq_id = 0,
-                                          ITableFilter *filter = 0);
+    static CRef<CSeq_feat> CreateSeqFeat(const string& feat,
+                                         CSeq_loc& location,
+                                         const TFlags flags = 0,
+                                         ILineErrorListener* pMessageListener = nullptr,
+                                         unsigned int line = 0,
+                                         std::string* seq_id = nullptr,
+                                         ITableFilter* filter = nullptr);
 
     // add single qualifier to feature
-    static void AddFeatQual (CRef<CSeq_feat> sfp,
-                             const string& feat_name,
-                             const string& qual,
-                             const string& val,
-                             const TFlags flags = 0,
-                             ILineErrorListener* pMessageListener=0,
-                             int line = 0,
-                             const string &seq_id = std::string() );
+    static void AddFeatQual(CRef<CSeq_feat> sfp,
+                            const string& feat_name,
+                            const string& qual,
+                            const string& val,
+                            const TFlags flags = 0,
+                            ILineErrorListener* pMessageListener = nullptr,
+                            int line = 0,
+                            const string& seq_id = std::string());
 
     /// If line_arg is a feature line (e.g. ">Feature lcl|seq1"), then
     /// it will parse it and return true.
