@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___SOURCE_MOD_PARSER__HPP
 #define OBJTOOLS_READERS___SOURCE_MOD_PARSER__HPP
 
-/*  $Id: source_mod_parser.hpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: source_mod_parser.hpp 673184 2023-09-25 14:08:03Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,7 +116,7 @@ public:
     string ParseTitle(const CTempString& title, CConstRef<CSeq_id> seqid,
         size_t iMaxModsToParse = std::numeric_limits<size_t>::max() );
 
-    void AddMods(const CTempString& name, const CTempString& value);
+    bool AddMods(const CTempString& name, const CTempString& value);
 
     /// Apply previously extracted modifiers to the given object, marking all
     /// relevant ones as used.
