@@ -1,7 +1,7 @@
 #ifndef OBJOSTR__HPP
 #define OBJOSTR__HPP
 
-/*  $Id: objostr.hpp 574303 2018-11-08 19:59:04Z gouriano $
+/*  $Id: objostr.hpp 623252 2021-01-11 16:26:27Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -639,7 +639,7 @@ public:
     };
 
 #if HAVE_NCBI_C
-    class NCBI_XSERIAL_EXPORT AsnIo
+    class AsnIo
     {
     public:
         AsnIo(CObjectOStream& out, const string& rootTypeName);
