@@ -70,7 +70,7 @@ enum distance_metric
     \ingroup  distance
 */
 inline
-distance_metric operation2metric(set_operation op)
+distance_metric operation2metric(set_operation op) BMNOEXCEPT
 {
     BM_ASSERT(is_const_set_operation(op));
     if (op == set_COUNT) op = set_COUNT_B;
@@ -83,7 +83,6 @@ distance_metric operation2metric(set_operation op)
     \brief Distance metric descriptor, holds metric code and result.
     \sa distance_operation
 */
-
 struct distance_metric_descriptor
 {
 #ifdef BM64ADDR
@@ -95,11 +94,11 @@ struct distance_metric_descriptor
      distance_metric   metric;
      size_type          result;
      
-     distance_metric_descriptor(distance_metric m)
+     distance_metric_descriptor(distance_metric m) BMNOEXCEPT
      : metric(m),
        result(0)
     {}
-    distance_metric_descriptor()
+    distance_metric_descriptor() BMNOEXCEPT
     : metric(bm::COUNT_XOR),
       result(0)
     {}
@@ -107,12 +106,77 @@ struct distance_metric_descriptor
     /*! 
         \brief Sets metric result to 0
     */
-    void reset()
+    void reset() BMNOEXCEPT
     {
         result = 0;
     }
 };
 
+/// functor-adaptor for C-style callbacks
+///
+/// @internal
+///
+template <class VCBT, class size_type>
+struct bit_visitor_callback_adaptor
+{
+    typedef VCBT bit_visitor_callback_type;
+
+    bit_visitor_callback_adaptor(void* h, bit_visitor_callback_type cb_func)
+        : handle_(h), func_(cb_func)
+    {}
+
+    int add_bits(size_type offset, const unsigned char* bits, unsigned size)
+    {
+        for (unsigned i = 0; i < size; ++i)
+        {
+            int ret = func_(handle_, offset + bits[i]);
+            if (ret < 0)
+                return ret;
+        }
+        return 0;
+    }
+    int add_range(size_type offset, size_type size)
+    {
+        for (size_type i = 0; i < size; ++i)
+        {
+            int ret = func_(handle_, offset + i);
+            if (ret < 0)
+                return ret;
+        }
+        return 0;
+    }
+
+    void*                     handle_;
+    bit_visitor_callback_type func_;
+};
+
+/// functor-adaptor for back-inserter
+///
+/// @internal
+///
+template <class BII, class size_type>
+struct bit_visitor_back_inserter_adaptor
+{
+
+    bit_visitor_back_inserter_adaptor(BII bi)
+        : bi_(bi)
+    {}
+
+    int add_bits(size_type offset, const unsigned char* bits, unsigned size)
+    {
+        for (unsigned i = 0; i < size; ++i)
+            *bi_ = offset + bits[i];
+        return 0;
+    }
+    int add_range(size_type offset, size_type size)
+    {
+        for (size_type i = 0; i < size; ++i)
+            *bi_ = offset + i;
+        return 0;
+    }
+
+    BII bi_;
+};
 
 
 /*!
@@ -125,7 +189,7 @@ inline
 void combine_count_operation_with_block(const bm::word_t*           blk,
                                         const bm::word_t*           arg_blk,
                                         distance_metric_descriptor* dmit,
-                                        distance_metric_descriptor* dmit_end)
+                                        distance_metric_descriptor* dmit_end) BMNOEXCEPT
                                             
 {     
      gap_word_t* g1 = BMGAP_PTR(blk);
@@ -340,7 +404,7 @@ void combine_count_operation_with_block(const bm::word_t*           blk,
 */
 inline
 unsigned combine_count_and_operation_with_block(const bm::word_t* blk,
-                                                const bm::word_t* arg_blk)
+                                                const bm::word_t* arg_blk) BMNOEXCEPT
 {
     unsigned gap = BM_IS_GAP(blk);
     unsigned arg_gap = BM_IS_GAP(arg_blk);
@@ -381,7 +445,7 @@ void combine_any_operation_with_block(const bm::word_t* blk,
                                       const bm::word_t* arg_blk,
                                       unsigned arg_gap,
                                       distance_metric_descriptor* dmit,
-                                      distance_metric_descriptor* dmit_end)
+                                      distance_metric_descriptor* dmit_end) BMNOEXCEPT
                                             
 {
      gap_word_t* res=0;
@@ -628,7 +692,7 @@ inline
 unsigned
 combine_count_operation_with_block(const bm::word_t* blk,
                                    const bm::word_t* arg_blk,
-                                   distance_metric metric)
+                                   distance_metric metric) BMNOEXCEPT
 {
     distance_metric_descriptor dmd(metric);
     combine_count_operation_with_block(blk, //gap, 
@@ -649,7 +713,7 @@ combine_any_operation_with_block(const bm::word_t* blk,
                                           unsigned gap,
                                           const bm::word_t* arg_blk,
                                           unsigned arg_gap,
-                                          distance_metric metric)
+                                          distance_metric metric) BMNOEXCEPT
 {
     distance_metric_descriptor dmd(metric);
     combine_any_operation_with_block(blk, gap, 
@@ -668,7 +732,7 @@ combine_any_operation_with_block(const bm::word_t* blk,
 inline
 void distance_stage(const distance_metric_descriptor* dmit,
                     const distance_metric_descriptor* dmit_end,
-                    bool*                             is_all_and)
+                    bool*                             is_all_and) BMNOEXCEPT
 {
     for (const distance_metric_descriptor* it = dmit; it < dmit_end; ++it)
     {
@@ -702,7 +766,7 @@ template<class BV>
 void distance_operation(const BV& bv1, 
                         const BV& bv2, 
                         distance_metric_descriptor* dmit,
-                        distance_metric_descriptor* dmit_end)
+                        distance_metric_descriptor* dmit_end) BMNOEXCEPT
 {
     const typename BV::blocks_manager_type& bman1 = bv1.get_blocks_manager();
     const typename BV::blocks_manager_type& bman2 = bv2.get_blocks_manager();
@@ -764,7 +828,7 @@ void distance_operation(const BV& bv1,
 
             arg_blk = bman2.get_block(i, j);
 
-            if (!blk & !arg_blk)
+            if (!blk && !arg_blk)
                 continue;
                 
             combine_count_operation_with_block(blk, 
@@ -787,7 +851,7 @@ void distance_operation(const BV& bv1,
 */
 template<class BV>
 typename BV::size_type distance_and_operation(const BV& bv1,
-                                              const BV& bv2)
+                                              const BV& bv2) BMNOEXCEPT
 {
     const typename BV::blocks_manager_type& bman1 = bv1.get_blocks_manager();
     const typename BV::blocks_manager_type& bman2 = bv2.get_blocks_manager();
@@ -858,7 +922,7 @@ template<class BV>
 void distance_operation_any(const BV& bv1, 
                             const BV& bv2, 
                             distance_metric_descriptor* dmit,
-                            distance_metric_descriptor* dmit_end)
+                            distance_metric_descriptor* dmit_end) BMNOEXCEPT
 {
     const typename BV::blocks_manager_type& bman1 = bv1.get_blocks_manager();
     const typename BV::blocks_manager_type& bman2 = bv2.get_blocks_manager();
@@ -980,7 +1044,8 @@ void distance_operation_any(const BV& bv1,
     \internal
 */
 template<typename It, typename SIZE_TYPE>
-It block_range_scan(It  first, It last, SIZE_TYPE nblock, SIZE_TYPE* max_id)
+It block_range_scan(It  first, It last,
+                    SIZE_TYPE nblock, SIZE_TYPE* max_id) BMNOEXCEPT
 {
     SIZE_TYPE m = *max_id;
     It right;
@@ -1333,7 +1398,11 @@ typename BV::size_type count_intervals(const BV& bv)
     typename BV::blocks_manager_type::block_idx_type st = 0;
     bm::for_each_block(blk_root, bman.top_block_size(), func, st);
 
-    return func.count();        
+    typename BV::size_type intervals = func.count();
+    bool last_bit_set = bv.test(bm::id_max-1);
+
+    intervals -= last_bit_set; // correct last (out of range) interval
+    return intervals;
 }
 
 /*!
@@ -1514,39 +1583,167 @@ void export_array(BV& bv, It first, It last)
 
 
 /*!
-   \brief for-each visitor, calls a special visitor functor for each 1 bit group
+   \brief for-each visitor, calls a visitor functor for each 1 bit group
  
    \param block - bit block buffer pointer
    \param offset - global block offset (number of bits)
    \param bit_functor - functor must support .add_bits(offset, bits_ptr, size)
+   \return - functor return code (< 0 - interrupt the processing)
  
    @ingroup bitfunc
    @internal
 */
 template<typename Func, typename SIZE_TYPE>
-void for_each_bit_blk(const bm::word_t* block, SIZE_TYPE offset,
+int for_each_bit_blk(const bm::word_t* block, SIZE_TYPE offset,
                       Func&  bit_functor)
 {
+    BM_ASSERT(block);
+    int ret;
     if (IS_FULL_BLOCK(block))
     {
-        bit_functor.add_range(offset, bm::gap_max_bits);
-        return;
+        ret = bit_functor.add_range(offset, bm::gap_max_bits);
+        return ret;
     }
     unsigned char bits[bm::set_bitscan_wave_size*32];
-
     SIZE_TYPE offs = offset;
-    unsigned cnt;
     const word_t* block_end = block + bm::set_block_size;
     do
     {
-        cnt = bm::bitscan_wave(block, bits);
-        if (cnt)
-            bit_functor.add_bits(offs, bits, cnt);
+        if (unsigned cnt = bm::bitscan_wave(block, bits))
+        {
+            ret = bit_functor.add_bits(offs, bits, cnt);
+            if (ret < 0)
+                return ret;
+        }
         offs += bm::set_bitscan_wave_size * 32;
         block += bm::set_bitscan_wave_size;
     } while (block < block_end);
+    return 0;
 }
 
+/*!
+   \brief for-each range visitor, calls a visitor functor for each 1 bit group
+
+   \param block - bit block buffer pointer
+   \param offset - global block offset (number of bits)
+   \param left - bit addredd in block from [from..to]
+   \param right - bit addredd in block to [from..to]
+   \param bit_functor - functor must support .add_bits(offset, bits_ptr, size)
+   \return - functor return code (< 0 - interrupt the processing)
+   @ingroup bitfunc
+   @internal
+*/
+template<typename Func, typename SIZE_TYPE>
+int for_each_bit_blk(const bm::word_t* block, SIZE_TYPE offset,
+                      unsigned left, unsigned right,
+                      Func&  bit_functor)
+{
+    BM_ASSERT(block);
+    BM_ASSERT(left <= right);
+    BM_ASSERT(right < bm::bits_in_block);
+
+    int ret = 0;
+    if (IS_FULL_BLOCK(block))
+    {
+        unsigned sz = right - left + 1;
+        ret = bit_functor.add_range(offset + left, sz);
+        return ret;
+    }
+    unsigned char bits[bm::set_bitscan_wave_size*32];
+
+    unsigned cnt, nword, nbit, bitcount, temp;
+    nbit = left & bm::set_word_mask;
+    const bm::word_t* word =
+        block + (nword = unsigned(left >> bm::set_word_shift));
+    if (left == right)  // special case (only 1 bit to check)
+    {
+        if ((*word >> nbit) & 1u)
+        {
+            bits[0] = (unsigned char)nbit;
+            ret = bit_functor.add_bits(offset + (nword * 32), bits, 1);
+        }
+        return ret;
+    }
+
+    bitcount = right - left + 1u;
+    if (nbit) // starting position is not aligned
+    {
+        unsigned right_margin = nbit + right - left;
+        if (right_margin < 32)
+        {
+            unsigned mask_r = bm::mask_r_u32(nbit);
+            unsigned mask_l = bm::mask_l_u32(right_margin);
+            unsigned mask = mask_r & mask_l;
+
+            temp = (*word & mask);
+            cnt = bm::bitscan(temp, bits);
+            if (cnt)
+                ret = bit_functor.add_bits(offset + (nword * 32), bits, cnt);
+            return ret;
+        }
+        unsigned mask_r = bm::mask_r_u32(nbit);
+        temp = *word & mask_r;
+        cnt = bm::bitscan(temp, bits);
+        if (cnt)
+        {
+            ret = bit_functor.add_bits(offset + (nword * 32), bits, cnt);
+            if (ret < 0)
+                return ret;
+        }
+        bitcount -= 32 - nbit;
+        ++word; ++nword;
+    }
+    else
+    {
+        bitcount = right - left + 1u;
+    }
+
+    // word aligned now - scan the bit-stream loop unrolled 4x words(wave)
+    BM_ASSERT(bm::set_bitscan_wave_size == 4);
+    for ( ;bitcount >= 128;
+           bitcount-=128, word+=bm::set_bitscan_wave_size,
+           nword += bm::set_bitscan_wave_size)
+    {
+        cnt = bm::bitscan_wave(word, bits);
+        if (cnt)
+        {
+            ret = bit_functor.add_bits(offset + (nword * 32), bits, cnt);
+            if (ret < 0)
+                return ret;
+        }
+    } // for
+
+    for ( ;bitcount >= 32; bitcount-=32, ++word)
+    {
+        temp = *word;
+        cnt = bm::bitscan(temp, bits);
+        if (cnt)
+        {
+            ret = bit_functor.add_bits(offset + (nword * 32), bits, cnt);
+            if (ret < 0)
+                return ret;
+        }
+        ++nword;
+    } // for
+
+    BM_ASSERT(bitcount < 32);
+
+    if (bitcount)  // we have a tail to count
+    {
+        unsigned mask_l = bm::mask_l_u32(bitcount-1);
+        temp = *word & mask_l;
+        cnt = bm::bitscan(temp, bits);
+        if (cnt)
+        {
+            ret = bit_functor.add_bits(offset + (nword * 32), bits, cnt);
+            if (ret < 0)
+                return ret;
+        }
+    }
+    return 0;
+}
+
+
 
 /*!
    \brief for-each visitor, calls a special visitor functor for each 1 bit range
@@ -1554,27 +1751,288 @@ void for_each_bit_blk(const bm::word_t* block, SIZE_TYPE offset,
    \param buf - bit block buffer pointer
    \param offset - global block offset (number of bits)
    \param bit_functor - functor must support .add_range(offset, bits_ptr, size)
- 
+   \return - functor return code (< 0 - interrupt the processing)
+
    @ingroup gapfunc
    @internal
 */
 template<typename T, typename Func, typename SIZE_TYPE>
-void for_each_gap_blk(const T* buf, SIZE_TYPE offset,
+int for_each_gap_blk(const T* buf, SIZE_TYPE offset,
                       Func&  bit_functor)
 {
     const T* pcurr = buf + 1;
     const T* pend = buf + (*buf >> 3);
-
+    int ret = 0;
     if (*buf & 1)
     {
-        bit_functor.add_range(offset, *pcurr + 1);
+        ret = bit_functor.add_range(offset, *pcurr + 1);
+        if (ret < 0)
+            return ret;
         ++pcurr;
     }
+    for (++pcurr; (pcurr <= pend) && (ret >= 0); pcurr += 2)
+    {
+        T prev = *(pcurr-1);
+        ret = bit_functor.add_range(offset + prev + 1, *pcurr - prev);
+    }
+    return ret;
+}
+
+/*!
+   \brief for-each visitor, calls a special visitor functor for each 1 bit range
+
+   \param buf - bit block buffer pointer
+   \param offset - global block offset (number of bits)
+   \param left - interval start [left..right]
+   \param right - intreval end [left..right]
+   \param bit_functor - functor must support .add_range(offset, bits_ptr, size)
+   \return - functor return code (< 0 - interrupt the processing)
+
+   @ingroup gapfunc
+   @internal
+*/
+template<typename T, typename Func, typename SIZE_TYPE>
+int for_each_gap_blk_range(const T* BMRESTRICT buf,
+                            SIZE_TYPE offset,
+                            unsigned left, unsigned right,
+                            Func&  bit_functor)
+{
+    BM_ASSERT(left <= right);
+    BM_ASSERT(right < bm::bits_in_block);
+
+    unsigned is_set;
+    unsigned start_pos = bm::gap_bfind(buf, left, &is_set);
+    const T* BMRESTRICT pcurr = buf + start_pos;
+    int ret = 0;
+    if (is_set)
+    {
+        if (right <= *pcurr)
+        {
+            ret = bit_functor.add_range(offset + left, (right + 1)-left);
+            return ret;
+        }
+        ret = bit_functor.add_range(offset + left, (*pcurr + 1)-left);
+        if (ret < 0)
+            return ret;
+        ++pcurr;
+    }
+
+    const T* BMRESTRICT pend = buf + (*buf >> 3);
     for (++pcurr; pcurr <= pend; pcurr += 2)
     {
         T prev = *(pcurr-1);
-        bit_functor.add_range(offset + prev + 1, *pcurr - prev);
+        if (right <= *pcurr)
+        {
+            int sz = int(right) - int(prev);
+            if (sz > 0)
+                ret = bit_functor.add_range(offset + prev + 1, unsigned(sz));
+            return ret;
+        }
+        ret = bit_functor.add_range(offset + prev + 1, *pcurr - prev);
+        if (ret < 0)
+            return ret;
+    } // for
+    return 0;
+}
+
+
+
+/*! For each non-zero block in [from, to] executes supplied functor
+    \internal
+*/
+template<typename T, typename N, typename F>
+int for_each_bit_block_range(T*** root,
+                              N top_size, N nb_from, N nb_to, F& f)
+{
+    BM_ASSERT(top_size);
+    if (nb_from > nb_to)
+        return 0;
+    unsigned i_from = unsigned(nb_from >> bm::set_array_shift);
+    unsigned j_from = unsigned(nb_from &  bm::set_array_mask);
+    unsigned i_to = unsigned(nb_to >> bm::set_array_shift);
+    unsigned j_to = unsigned(nb_to &  bm::set_array_mask);
+
+    if (i_from >= top_size)
+        return 0;
+    if (i_to >= top_size)
+    {
+        i_to = unsigned(top_size-1);
+        j_to = bm::set_sub_array_size-1;
     }
+    int ret;
+    for (unsigned i = i_from; i <= i_to; ++i)
+    {
+        T** blk_blk = root[i];
+        if (!blk_blk)
+            continue;
+        if ((bm::word_t*)blk_blk == FULL_BLOCK_FAKE_ADDR)
+        {
+            unsigned j = (i == i_from) ? j_from : 0;
+            if (!j && (i != i_to)) // full sub-block
+            {
+                N base_idx = bm::get_super_block_start<N>(i);
+                ret = f.add_range(base_idx, bm::set_sub_total_bits);
+                if (ret < 0)
+                    return ret;
+            }
+            else
+            {
+                do
+                {
+                    N base_idx = bm::get_block_start<N>(i, j);
+                    ret = f.add_range(base_idx, bm::gap_max_bits);
+                    if (ret < 0)
+                        return ret;
+                    if ((i == i_to) && (j == j_to))
+                        return 0;
+                } while (++j < bm::set_sub_array_size);
+            }
+        }
+        else
+        {
+            unsigned j = (i == i_from) ? j_from : 0;
+            do
+            {
+                const T* block;
+                if (blk_blk[j])
+                {
+                    N base_idx = bm::get_block_start<N>(i, j);
+                    if (0 != (block = blk_blk[j]))
+                    {
+                        if (BM_IS_GAP(block))
+                            ret = bm::for_each_gap_blk(BMGAP_PTR(block), base_idx, f);
+                        else
+                            ret = bm::for_each_bit_blk(block, base_idx, f);
+                        if (ret < 0)
+                            return ret;
+                    }
+                }
+
+                if ((i == i_to) && (j == j_to))
+                    return 0;
+            } while (++j < bm::set_sub_array_size);
+        }
+    } // for i
+    return 0;
+}
+
+
+/**
+    Implementation of for_each_bit_range without boilerplave checks
+    @internal
+*/
+template<class BV, class Func>
+int for_each_bit_range_no_check(const BV&             bv,
+                       typename BV::size_type left,
+                       typename BV::size_type right,
+                       Func&                  bit_functor)
+{
+    typedef typename BV::size_type      size_type;
+    typedef typename BV::block_idx_type block_idx_type;
+
+    const typename BV::blocks_manager_type& bman = bv.get_blocks_manager();
+    bm::word_t*** blk_root = bman.top_blocks_root();
+    if (!blk_root)
+        return 0;
+        
+    block_idx_type nblock_left  = (left  >> bm::set_block_shift);
+    block_idx_type nblock_right = (right >> bm::set_block_shift);
+
+    unsigned i0, j0;
+    bm::get_block_coord(nblock_left, i0, j0);
+    const bm::word_t* block = bman.get_block_ptr(i0, j0);
+    unsigned nbit_left  = unsigned(left  & bm::set_block_mask);
+    size_type offset = nblock_left * bm::bits_in_block;
+    int ret = 0;
+    if (nblock_left == nblock_right) // hit in the same block
+    {
+        if (!block)
+            return ret;
+        unsigned nbit_right = unsigned(right & bm::set_block_mask);
+        if (BM_IS_GAP(block))
+        {
+            ret = bm::for_each_gap_blk_range(BMGAP_PTR(block), offset,
+                                             nbit_left, nbit_right, bit_functor);
+        }
+        else
+        {
+            ret = bm::for_each_bit_blk(block, offset, nbit_left, nbit_right,
+                                 bit_functor);
+        }
+        return ret;
+    }
+    // process left block
+    if (nbit_left && block)
+    {
+        if (BM_IS_GAP(block))
+        {
+            ret = bm::for_each_gap_blk_range(BMGAP_PTR(block), offset,
+                                nbit_left, bm::bits_in_block-1, bit_functor);
+        }
+        else
+        {
+            ret = bm::for_each_bit_blk(block, offset, nbit_left, bm::bits_in_block-1,
+                                 bit_functor);
+        }
+        if (ret < 0)
+            return ret;
+        ++nblock_left;
+    }
+
+    // process all complete blocks in the middle
+    {
+        block_idx_type top_blocks_size = bman.top_block_size();
+        ret = bm::for_each_bit_block_range(blk_root, top_blocks_size,
+                                nblock_left, nblock_right-1, bit_functor);
+        if (ret < 0)
+            return ret;
+    }
+
+    unsigned nbit_right = unsigned(right & bm::set_block_mask);
+    bm::get_block_coord(nblock_right, i0, j0);
+    block = bman.get_block_ptr(i0, j0);
+
+    if (block)
+    {
+        offset = nblock_right * bm::bits_in_block;
+        if (BM_IS_GAP(block))
+        {
+            ret = bm::for_each_gap_blk_range(BMGAP_PTR(block), offset,
+                                       0, nbit_right, bit_functor);
+        }
+        else
+        {
+            ret = bm::for_each_bit_blk(block, offset, 0, nbit_right, bit_functor);
+        }
+    }
+    return ret;
+}
+
+/**
+    convert sub-blocks to an array of set 1s (32-bit)
+    @internal
+ */
+template<typename BV, typename VECT>
+void convert_sub_to_arr(const BV& bv, unsigned sb, VECT& vect)
+{
+    vect.resize(0);
+
+    typename BV::size_type from, to, idx;
+    typename BV::size_type sb_max_bc = bm::set_sub_array_size * bm::gap_max_bits;
+    from = sb * sb_max_bc;
+    to = (sb+1) * sb_max_bc;
+    if (!to || to > bm::id_max) // overflow check
+        to = bm::id_max;
+
+    typename BV::enumerator en = bv.get_enumerator(from);
+    for (; en.valid(); ++en)
+    {
+        idx = *en;
+        if (idx >= to)
+            break;
+        idx -= from;
+        vect.push_back((typename VECT::value_type)idx);
+    } // for en
 }
 
 
