@@ -67,25 +67,25 @@ public:
     // -----------------------------------------------------------------
 
     /// set empty (no bits set super-block)
-    void set_null_super_block(unsigned i);
+    void set_null_super_block(unsigned i) BMNOEXCEPT;
     
     /// set FULL (all bits set super-block)
-    void set_full_super_block(unsigned i);
+    void set_full_super_block(unsigned i) BMNOEXCEPT;
     
     /// set super block count
-    void set_super_block(unsigned i, size_type bcount);
+    void set_super_block(unsigned i, size_type bcount) BMNOEXCEPT;
     
     /// return bit-count in super-block
-    unsigned get_super_block_count(unsigned i) const;
+    unsigned get_super_block_count(unsigned i) const BMNOEXCEPT;
 
     /// return running bit-count in super-block
-    size_type get_super_block_rcount(unsigned i) const;
+    size_type get_super_block_rcount(unsigned i) const BMNOEXCEPT;
     
     /// return number of super-blocks
-    size_type super_block_size() const;
+    size_type super_block_size() const BMNOEXCEPT;
     
     /// Find super-block with specified rank
-    unsigned find_super_block(size_type rank) const;
+    unsigned find_super_block(size_type rank) const BMNOEXCEPT;
     
     
     /// set size of true super-blocks (not NULL and not FFFFF)
@@ -94,14 +94,14 @@ public:
     /// Add block list belonging to one super block
     void register_super_block(unsigned i,
                               const unsigned* bcount,
-                              const unsigned* sub_count);
+                              const bm::id64_t* sub_count);
 
     /// find block position and sub-range for the specified rank
     bool find(size_type* rank,
               block_idx_type* nb, bm::gap_word_t* sub_range) const;
     
-    /// return sub-clock counts
-    unsigned sub_count(block_idx_type nb) const;
+    /// return sub-clock counts (64-bit for additional information)
+    bm::id64_t sub_count(block_idx_type nb) const BMNOEXCEPT;
 
 
     // -----------------------------------------------------------------
@@ -126,13 +126,14 @@ public:
     
     /// determine the sub-range within a bit-block
     static
-    unsigned find_sub_range(unsigned block_bit_pos);
+    unsigned find_sub_range(unsigned block_bit_pos) BMNOEXCEPT;
     
     /// determine block sub-range for rank search
-    bm::gap_word_t select_sub_range(block_idx_type nb, size_type& rank) const;
+    bm::gap_word_t select_sub_range(block_idx_type nb,
+                                    size_type& rank) const BMNOEXCEPT;
     
     /// find block position for the specified rank
-    block_idx_type find(size_type rank) const;
+    block_idx_type find(size_type rank) const BMNOEXCEPT;
     
 private:
     typedef bm::heap_vector<sblock_count_type, bv_allocator_type, false>
@@ -141,16 +142,130 @@ private:
                                                     sblock_row_vector_type;
     typedef bm::dynamic_heap_matrix<unsigned, bv_allocator_type>  blocks_matrix_type;
 
+    typedef bm::dynamic_heap_matrix<bm::id64_t, bv_allocator_type>  blocks_matrix64_type;
+
 private:
     unsigned                  sblock_rows_;
     sblock_count_vector_type  sblock_count_;   ///< super-block accumulated counts
     sblock_row_vector_type    sblock_row_idx_; ///< super-block row numbers
     blocks_matrix_type        block_matr_;     ///< blocks within super-blocks (matrix)
-    blocks_matrix_type        sub_block_matr_; ///< sub-block counts
+    blocks_matrix64_type      sub_block_matr_; ///< sub-block counts
     size_type                 total_blocks_;   ///< total bit-blocks in the index
     unsigned                  max_sblock_;     ///< max. superblock index
 };
 
+/**
+    Precalculated decision table fdr interval selection
+    @internal
+ */
+template<bool T> struct rs_intervals
+{
+    struct codes
+    {
+        const unsigned char _lut[65536] = {0, }; // nibble lookup table
+
+        codes()
+        {
+            #if defined(BM64_SSE4) || defined(BM64_AVX2) || defined(BM64_AVX512)
+            const unsigned cutoff_bias = rs3_half_span/8;
+            #else
+            const unsigned cutoff_bias = 0;
+            #endif
+
+            for (unsigned nbit_right = 0; nbit_right < 65536; ++nbit_right)
+            {
+                unsigned sub = (nbit_right <= rs3_border0) ? 0 :
+                               (nbit_right > rs3_border1) ? 2 : 1;
+                switch(sub)  // sub-range rank calc
+                {
+                case 0:
+                    // |--x-----[0]-----------[1]----------|
+                    if (nbit_right <= rs3_border0/2 + cutoff_bias)
+                        sub = 0;
+                    else // |--------[x]-----------[1]----------|
+                        if (nbit_right == rs3_border0)
+                            sub = 1;
+                        else // |------x-[0]-----------[1]----------|
+                            sub = 2;
+                break;
+
+                case 1:
+                    // |--------[0]-x---------[1]----------|
+                    if (nbit_right <= rs3_border0_1 + cutoff_bias)
+                        sub = 3;
+                    else
+                    {
+                        // |--------[0]-----------[x]----------|
+                        if (nbit_right == rs3_border1)
+                            sub = 4;
+                        else
+                        {
+                            // |--------[0]--------x--[1]----------|
+                            unsigned d1 = nbit_right - bm::rs3_border0_1;
+                            unsigned d2 = rs3_border1 - nbit_right;
+                            if (d2 < d1) // |--------[0]----|----x-[1]--------|
+                                sub = 5;
+                            else // |--------[0]----|-x----[1]----------|
+                                sub = 6;
+                        }
+                    }
+                break;
+
+                case 2:
+                // |--------[0]-----------[1]-x---*----|
+                if (nbit_right <= rs3_border1_1)
+                {
+                    unsigned d1 = nbit_right - bm::rs3_border1;
+                    unsigned d2 = (rs3_border1_1 + cutoff_bias) - nbit_right;
+                    if (d1 < d2) // |--------[0]-----------[1]-x---*----|
+                        sub = 7;
+                    else // |--------[0]-----------[1]---x-*----|
+                    {
+                        if (nbit_right == rs3_border1_1)
+                            sub = 8;
+                        else
+                            sub = 9;
+                    }
+                }
+                else
+                {
+                    // |--------[0]-----------[1]----------x
+                    if (nbit_right == bm::gap_max_bits-1)
+                        sub = 10;
+                    else
+                    {
+                        // |--------[0]-----------[1]-------x--|
+                        BM_ASSERT(nbit_right > bm::rs3_border1_1);
+                        unsigned d1 = nbit_right - bm::rs3_border1_1;
+                        unsigned d2 = bm::gap_max_bits - nbit_right;
+                        if (d2 < d1) // |--------[0]----------[1]----*---x-|
+                            sub = 11;
+                        else // |--------[0]----------[1]----*-x---|
+                            sub = 12;
+                    }
+                }
+                break;
+
+                default:
+                    BM_ASSERT(0);
+                    sub = 0;
+                } // switch
+
+                bm::set_nibble(
+                    const_cast<unsigned char*>(_lut),
+                    nbit_right, (unsigned char)sub);
+
+                BM_ASSERT((unsigned char)sub ==
+                 bm::get_nibble(const_cast<unsigned char*>(_lut), nbit_right));
+
+            } // for
+        }
+    };
+    static codes _c;
+};
+template<bool T> typename rs_intervals<T>::codes rs_intervals<T>::_c;
+
+
 //---------------------------------------------------------------------
 //
 //---------------------------------------------------------------------
@@ -169,8 +284,8 @@ void rs_index<BVAlloc>::init() BMNOEXCEPT
 {
     sblock_count_.resize(0);
     sblock_row_idx_.resize(0);
-    block_matr_.resize(0, 0);
-    sub_block_matr_.resize(0, 0);
+    block_matr_.resize(0, 0, false);
+    sub_block_matr_.resize(0, 0, false);
     
     total_blocks_ = sblock_rows_ = max_sblock_ = 0;
 }
@@ -271,7 +386,7 @@ rs_index<BVAlloc>::rcount(block_idx_type nb) const
 //---------------------------------------------------------------------
 
 template<typename BVAlloc>
-unsigned rs_index<BVAlloc>::find_sub_range(unsigned block_bit_pos)
+unsigned rs_index<BVAlloc>::find_sub_range(unsigned block_bit_pos)  BMNOEXCEPT
 {
     return (block_bit_pos <= rs3_border0) ? 0 :
             (block_bit_pos > rs3_border1) ? 2 : 1;
@@ -280,10 +395,12 @@ unsigned rs_index<BVAlloc>::find_sub_range(unsigned block_bit_pos)
 //---------------------------------------------------------------------
 
 template<typename BVAlloc>
-bm::gap_word_t rs_index<BVAlloc>::select_sub_range(block_idx_type nb,
-                                                   size_type& rank) const
+bm::gap_word_t rs_index<BVAlloc>::select_sub_range(
+                                            block_idx_type nb,
+                                            size_type& rank) const BMNOEXCEPT
 {
-    unsigned sub_cnt = sub_count(nb);
+    bm::id64_t sub = sub_count(nb);
+    unsigned sub_cnt = unsigned(sub);
     unsigned first = sub_cnt & 0xFFFF;
     unsigned second = sub_cnt >> 16;
 
@@ -305,7 +422,7 @@ bm::gap_word_t rs_index<BVAlloc>::select_sub_range(block_idx_type nb,
 
 template<typename BVAlloc>
 typename rs_index<BVAlloc>::block_idx_type
-rs_index<BVAlloc>::find(size_type rank) const
+rs_index<BVAlloc>::find(size_type rank) const BMNOEXCEPT
 {
     BM_ASSERT(rank);
 
@@ -344,28 +461,28 @@ rs_index<BVAlloc>::find(size_type rank) const
 
 //---------------------------------------------------------------------
 template<typename BVAlloc>
-unsigned rs_index<BVAlloc>::sub_count(block_idx_type nb) const
+bm::id64_t rs_index<BVAlloc>::sub_count(block_idx_type nb) const BMNOEXCEPT
 {
     if (nb >= total_blocks_)
         return 0;
     
     unsigned i = unsigned(nb >> bm::set_array_shift);
     size_type sb_count = get_super_block_count(i);
-    
     if (!sb_count)
         return 0;
+
     if (sb_count == (bm::set_sub_array_size * bm::gap_max_bits)) // FULL BLOCK
     {
         unsigned first = rs3_border0 + 1;
         unsigned second = rs3_border1 - first + 1;
-        return (second << 16) | first;
+        bm::id64_t aux0 = bm::rs3_border0_1 + 1;
+        bm::id64_t aux1 = bm::rs3_border1_1 + 1;
+        return (aux0 << 32) | (aux1 << 48) | (second << 16) | first;
     }
     
-    unsigned j = unsigned(nb &  bm::set_array_mask);
-
     unsigned row_idx = sblock_row_idx_[i+1];
-    const unsigned* sub_row = sub_block_matr_.row(row_idx);
-    return sub_row[j];
+    const bm::id64_t* sub_row = sub_block_matr_.row(row_idx);
+    return sub_row[nb &  bm::set_array_mask]; // j - coordinate element
 }
 
 
@@ -435,9 +552,9 @@ bool rs_index<BVAlloc>::find(size_type* rank,
         if (j)
             *rank -= row[j-1];
         
-        const unsigned* sub_row = sub_block_matr_.row(row_idx);
+        const bm::id64_t* sub_row = sub_block_matr_.row(row_idx);
         unsigned first = sub_row[j] & 0xFFFF;
-        unsigned second = sub_row[j] >> 16;
+        unsigned second = (sub_row[j] >> 16) & 0xFFFF;
         if (*rank > first)
         {
             *rank -= first;
@@ -466,7 +583,7 @@ bool rs_index<BVAlloc>::find(size_type* rank,
 //---------------------------------------------------------------------
 
 template<typename BVAlloc>
-void rs_index<BVAlloc>::set_null_super_block(unsigned i)
+void rs_index<BVAlloc>::set_null_super_block(unsigned i) BMNOEXCEPT
 {
     if (i > max_sblock_)
         max_sblock_ = i;
@@ -477,7 +594,7 @@ void rs_index<BVAlloc>::set_null_super_block(unsigned i)
 //---------------------------------------------------------------------
 
 template<typename BVAlloc>
-void rs_index<BVAlloc>::set_full_super_block(unsigned i)
+void rs_index<BVAlloc>::set_full_super_block(unsigned i) BMNOEXCEPT
 {
     set_super_block(i, bm::set_sub_array_size * bm::gap_max_bits);
 }
@@ -485,7 +602,7 @@ void rs_index<BVAlloc>::set_full_super_block(unsigned i)
 //---------------------------------------------------------------------
 
 template<typename BVAlloc>
-void rs_index<BVAlloc>::set_super_block(unsigned i, size_type bcount)
+void rs_index<BVAlloc>::set_super_block(unsigned i, size_type bcount) BMNOEXCEPT
 {
     if (i > max_sblock_)
         max_sblock_ = i;
@@ -505,7 +622,7 @@ void rs_index<BVAlloc>::set_super_block(unsigned i, size_type bcount)
 //---------------------------------------------------------------------
 
 template<typename BVAlloc>
-unsigned rs_index<BVAlloc>::get_super_block_count(unsigned i) const
+unsigned rs_index<BVAlloc>::get_super_block_count(unsigned i) const BMNOEXCEPT
 {
     if (i > max_sblock_)
         return 0;
@@ -520,7 +637,7 @@ unsigned rs_index<BVAlloc>::get_super_block_count(unsigned i) const
 
 template<typename BVAlloc>
 typename rs_index<BVAlloc>::size_type
-rs_index<BVAlloc>::get_super_block_rcount(unsigned i) const
+rs_index<BVAlloc>::get_super_block_rcount(unsigned i) const BMNOEXCEPT
 {
     if (i > max_sblock_)
         i = max_sblock_;
@@ -530,7 +647,7 @@ rs_index<BVAlloc>::get_super_block_rcount(unsigned i) const
 //---------------------------------------------------------------------
 
 template<typename BVAlloc>
-unsigned rs_index<BVAlloc>::find_super_block(size_type rank) const
+unsigned rs_index<BVAlloc>::find_super_block(size_type rank) const BMNOEXCEPT
 {
     const sblock_count_type* bcount_arr = sblock_count_.begin();
     unsigned i;
@@ -547,7 +664,7 @@ unsigned rs_index<BVAlloc>::find_super_block(size_type rank) const
 
 template<typename BVAlloc>
 typename rs_index<BVAlloc>::size_type
-rs_index<BVAlloc>::super_block_size() const
+rs_index<BVAlloc>::super_block_size() const BMNOEXCEPT
 {
 
     size_type total_sblocks = (size_type)sblock_count_.size();
@@ -570,7 +687,7 @@ void rs_index<BVAlloc>::resize_effective_super_blocks(size_type sb_eff)
 template<typename BVAlloc>
 void rs_index<BVAlloc>::register_super_block(unsigned i,
                                              const unsigned* bcount,
-                                             const unsigned* sub_count_in)
+                                             const bm::id64_t* sub_count_in)
 {
     BM_ASSERT(bcount);
     BM_ASSERT(sub_count_in);
@@ -581,7 +698,7 @@ void rs_index<BVAlloc>::register_super_block(unsigned i,
     
     sblock_row_idx_[i+1] = sblock_rows_;
     unsigned* row = block_matr_.row(sblock_rows_);
-    unsigned* sub_row = sub_block_matr_.row(sblock_rows_);
+    bm::id64_t* sub_row = sub_block_matr_.row(sblock_rows_);
     ++sblock_rows_;
 
     unsigned bc = 0;
@@ -590,7 +707,9 @@ void rs_index<BVAlloc>::register_super_block(unsigned i,
         bc += bcount[j];
         row[j] = bc;
         sub_row[j] = sub_count_in[j];
-        BM_ASSERT(bcount[j] >= (sub_count_in[j] >> 16) + (sub_count_in[j] & bm::set_block_mask));
+        BM_ASSERT(bcount[j] >=
+                     ((sub_count_in[j] >> 16) & bm::set_block_mask) +
+                     (sub_count_in[j] & bm::set_block_mask));
     } // for j
     sblock_count_[i+1] = sblock_count_[i] + bc;
 }
