@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__LZO__HPP
 #define UTIL_COMPRESS__LZO__HPP
 
-/*  $Id: lzo.hpp 612744 2020-07-27 14:26:09Z ivanov $
+/*  $Id: lzo.hpp 667989 2023-05-24 13:59:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,6 +31,7 @@
  */
 
 /// @file lzo.hpp
+///
 /// LZO Compression API.
 ///
 /// LZO is a data compression library which is suitable for data
@@ -43,10 +44,15 @@
 /// compression on yours own data before making decision to use LZO.
 /// Use zlib if you need a more universal and robust solution, that is also
 /// slower and needs more memory.
-
+///
 /// We don't support all possible algorithms, implemented in LZO.
 /// Only LZO1X is used in this API. Author of LZO says that it is 
 /// often the best choice of all.
+/// 
+/// NOTE:
+///   LZO is a just an compression algorithm to compress buffers,
+///   it doesnt support streaming or file compression out from the box.
+///   We use our own stream/file format to add such support.
 ///
 /// CLZOCompression        - base methods for compression/decompression
 ///                          memory buffers and files.
@@ -64,6 +70,18 @@
 ///
 /// For more details see LZO documentation:
 ///    http://www.oberhumer.com/opensource/lzo/
+/// 
+/// @warning
+///   LZO ia an optional compression component and can be missed on a current
+///   platform. It is recommended to guard its usage:
+/// 
+///   #if defined(HAVE_LIBLZO)
+///       // use LZO related code here
+///   #else
+///      // some backup code, or error reporting
+///   #endif
+/// 
+/// See also comments at the beginning of "compress.hpp".
 
 
 #include <util/compress/stream.hpp>
@@ -80,26 +98,8 @@
 BEGIN_NCBI_SCOPE
 
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// Special compression parameters (description partialy copied from LZO docs):
-//        
-// <blocksize>
-//    LZO is a block compression algorithm - it compresses and decompresses
-//    a block of data. Block size must be the same for compression
-//    and decompression. This parameter define a block size used only
-//    for file/stream based compression/decompression. The methods operated
-//    with all data located in memory (like CompressBuffer/DecompressBuffer)
-//    works by default with one big block - the whole data buffer.
-//
-
 /// Default LZO block size.
-///
-/// We use such block size to reduce overhead with a stream processor's
-/// methods calls, because compression/decompression output streams use
-/// by default (16Kb - 1) as output buffer size. But you can use any value
-/// if you think that it works better for you.
-/// @sa CCompressionStreambuf::CCompressionStreambuf
+/// @deprecated Please use CLZOCompression::GetBlockSizeDefault() instead
 const size_t kLZODefaultBlockSize = 24*1024;
 
 // Forward declaration of structure to define parameters for some level of compression.
@@ -116,54 +116,6 @@ struct SCompressionParam;
 class NCBI_XUTIL_EXPORT CLZOCompression : public CCompression 
 {
 public:
-    /// Compression/decompression flags.
-    enum EFlags {
-        ///< Allow transparent reading data from buffer/file/stream
-        ///< regardless is it compressed or not. But be aware,
-        ///< if data source contains broken data and API cannot detect that
-        ///< it is compressed data, that you can get binary instead of
-        ///< decompressed data. By default this flag is OFF.
-        fAllowTransparentRead = (1<<0),
-        ///< Allow to "compress/decompress" empty data. Buffer compression
-        ///< functions starts to return TRUE instead of FALSE for zero-length
-        ///< input. And, if this flag is used together with fStreamFormat
-        ///< than the output will have header and footer only.
-        fAllowEmptyData       = (1<<1),
-        ///< Add/check (accordingly to compression or decompression)
-        ///< the compressed data checksum. A checksum is a form of
-        ///< redundancy check. We use the safe decompressor, but this can be
-        ///< not enough, because many data errors will not result in
-        ///< a compressed data violation.
-        fChecksum             = (1<<2),
-        ///< Use stream compatible format for data compression.
-        ///< This flag have an effect only for CompressBuffer/DecompressBuffer.
-        ///< File and stream based compressors always use it by default.
-        ///< Use this flag with DecompressBuffer() to decompress data,
-        ///< compressed using streams, or compress data with CompressBuffer(),
-        ///< that can be decompressed using decompression stream.
-        ///< Also, this flag is reguired to compress data > 4GB.
-        ///< LZO default single block compression cannot handle such large
-        ///< amount of data on some platforms.
-        fStreamFormat         = (1<<3),
-        ///< Store file information like file name and file modification date
-        ///< of the compressed file into the file/stream.
-        ///< Works only with fStreamFormat flag.
-        fStoreFileInfo        = (1<<4) | fStreamFormat
-    }; 
-    typedef CLZOCompression::TFlags TLZOFlags; ///< Bitwise OR of EFlags
-
-    /// Constructor.
-    CLZOCompression(
-        ELevel level      = eLevel_Default,
-        size_t blocksize  = kLZODefaultBlockSize
-    );
-
-    /// Destructor.
-    virtual ~CLZOCompression(void);
-
-    /// Return name and version of the compression library.
-    virtual CVersionInfo GetVersion(void) const;
-
     /// Initialize LZO library.
     ///
     /// You should call this method only once, before any real
@@ -175,21 +127,79 @@ public:
     ///   before you run any other threads.
     static bool Initialize(void);
 
+    /// Compression/decompression flags.
+    enum EFlags {
+        /// Allow transparent reading data from buffer/file/stream
+        /// regardless is it compressed or not. But be aware,
+        /// if data source contains broken data and API cannot detect that
+        /// it is compressed data, that you can get binary instead of
+        /// decompressed data. By default this flag is OFF.
+        fAllowTransparentRead = (1<<0),
+        /// Allow to "compress/decompress" empty data. Buffer compression
+        /// functions starts to return TRUE instead of FALSE for zero-length
+        /// input. And, if this flag is used together with fStreamFormat
+        /// than the output will have header and footer only.
+        fAllowEmptyData       = (1<<1),
+        /// Add/check (accordingly to compression or decompression)
+        /// the compressed data checksum. A checksum is a form of
+        /// redundancy check. We use the safe decompressor, but this can be
+        /// not enough, because many data errors will not result in
+        /// a compressed data violation.
+        fChecksum             = (1<<2),
+        /// Use stream compatible format for data compression.
+        /// This flag have an effect for CompressBuffer/DecompressBuffer only.
+        /// File and stream based compressors always use it by default.
+        /// Use this flag with DecompressBuffer() to decompress data,
+        /// compressed using streams, or compress data with CompressBuffer(),
+        /// that can be decompressed using decompression stream.
+        /// @note 
+        ///   This flag is reguired to compress data > 4GB.
+        ///   LZO default single block compression cannot handle such large
+        ///   amount of data on some platforms.
+        fStreamFormat         = (1<<3),
+        /// Store file information like file name and file modification date
+        /// of the compressed file into the file/stream.
+        /// Works only with fStreamFormat flag.
+        fStoreFileInfo        = (1<<4) | fStreamFormat
+    }; 
+    typedef CLZOCompression::TFlags TLZOFlags; ///< Bitwise OR of EFlags
+
+    /// Constructor.
+    CLZOCompression(ELevel level = eLevel_Default);
+
+    /// @deprecated 
+    ///   Use CLZOCompression(ELevel) constructor without block size, that can be set separately
+    NCBI_DEPRECATED_CTOR(CLZOCompression(
+        ELevel level,
+        size_t blocksize
+    ));
+
+    /// Destructor.
+    virtual ~CLZOCompression(void);
+
+    /// Return name and version of the compression library.
+    virtual CVersionInfo GetVersion(void) const;
+
     /// Get compression level.
     ///
-    /// NOTE: This API use only two compression levels for LZO method.
-    ///       So, all compression levels will be translated only into 2
-    ///       real values. We use LZO1X-999 for "eLevel_Best", and 
-    ///       LZO1X-1 for all other levels of compression.
+    /// @note
+    ///   This API use only two compression levels for LZO method.
+    ///   All compression levels will be translated only into 2 real values. 
+    ///   We use LZO1X-999 compression for eLevel_Best, and LZO1X-1 for all
+    ///   other levels of compression.
     virtual ELevel GetLevel(void) const;
 
     /// Returns default compression level for a compression algorithm.
     virtual ELevel GetDefaultLevel(void) const
         { return eLevel_Lowest; };
 
-    //
+    /// Check if compression have support for a specified feature
+    virtual bool HaveSupport(ESupportFeature feature);
+
+
+    //=======================================================================
     // Utility functions 
-    //
+    //=======================================================================
 
     /// Compress data in the buffer.
     ///
@@ -250,47 +260,78 @@ public:
     /// Estimate buffer size for data compression.
     ///
     /// Simplified method for estimation of the size of buffer required
-    /// to compress specified number of bytes of data.
+    /// to compress specified number of bytes of data, uses current block size and flags.
     /// @sa
     ///   EstimateCompressionBufferSize, CompressBuffer
-    size_t EstimateCompressionBufferSize(size_t src_len, 
-                                         size_t blocksize = 0);
+    virtual size_t EstimateCompressionBufferSize(size_t src_len);
 
-    /// Estimate buffer size for data compression.
+    /// Estimate buffer size for data compression (advanced version).
     ///
     /// The function shall estimate the size of buffer required to compress
     /// specified number of bytes of data. This function return a conservative
     /// value that larger than 'src_len'. 
     /// @param src_len
-    ///   Size of data in source buffer.
+    ///   Size of data in the source buffer.
     /// @blocksize
-    ///   Size of blocks used by compressor to compress source data.
-    ///   Value 0 means that will be used block size specified in constructor. 
+    ///   Size of blocks that will be used for compression.
+    ///   Value 0 means default block size, same as GetBlockSizeDefault().
     /// @flags
     ///   Flags that will be used for compression.
     /// @return
     ///   Estimated buffer size.
     /// @sa
-    ///   CompressBuffer
-    size_t EstimateCompressionBufferSize(size_t    src_len, 
-                                         size_t    blocksize, 
-                                         TLZOFlags flags);
+    ///   CompressBuffer, GetBlockSizeDefault
+    static size_t EstimateCompressionBufferSize(size_t src_len, size_t blocksize, TLZOFlags flags);
+
+    /// Get recommended buffer sizes for stream/file I/O.
+    ///
+    /// These buffer sizes are softly recommended. They are not required, (de)compression
+    /// streams accepts any reasonable buffer size, for both input and output.
+    /// Respecting the recommended size just makes it a bit easier for (de)compressor,
+    /// reducing the amount of memory shuffling and buffering, resulting in minor 
+    /// performance savings. If compression library doesn't have preferences about 
+    /// I/O buffer sizes, kCompressionDefaultBufSize will be used.
+    /// @param round_up_by
+    ///   If specified, round up a returned value by specified amount. 
+    ///   Useful for better memory management. For example you can round up to virtual
+    ///   memory page size.
+    /// @return
+    ///   Structure with recommended buffer sizes.
+    /// @note
+    ///   Applicable for streaming/file operations.
+    /// @sa
+    ///   kCompressionDefaultBufSize, CSystemInfo::GetVirtualMemoryPageSize()
+    /// 
+    static SRecommendedBufferSizes GetRecommendedBufferSizes(size_t round_up = 0);
+
     /// Compress file.
     ///
     /// @param src_file
     ///   File name of source file.
     /// @param dst_file
     ///   File name of result file.
-    /// @param buf_size
-    ///   Buffer size used to read/write files.
+    /// @param file_io_bufsize
+    ///   Size of the buffer used to read from a source file. 
+    ///   Writing happens immediately on receiving some data from a compressor.
+    /// @param compression_in_bufsize
+    ///   Size of the internal buffer holding input data to be compressed.
+    ///   It can be different from 'file_io_bufsize' depending on a using 
+    ///   compression method, OS and file system.
+    /// @param compression_out_bufsize
+    ///   Size of the internal buffer to receive data from a compressor.
     /// @return
     ///   Return TRUE on success, FALSE on error.
+    /// @note
+    ///   This method don't store any file meta information like name, date/time, owner or attributes.
     /// @sa
-    ///   DecompressFile
+    ///   DecompressFile, GetRecommendedBufferSizes, CLZOCompressionFile
+    /// 
     virtual bool CompressFile(
         const string& src_file,
         const string& dst_file,
-        size_t        buf_size = kCompressionDefaultBufSize
+        size_t        file_io_bufsize         = kCompressionDefaultBufSize,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
     );
 
     /// Decompress file.
@@ -299,16 +340,26 @@ public:
     ///   File name of source file.
     /// @param dst_file
     ///   File name of result file.
-    /// @param buf_size
-    ///   Buffer size used to read/write files.
+    /// @param file_io_bufsize
+    ///   Size of the buffer used to read from a source file. 
+    ///   Writing happens immediately on receiving some data from a decompressor.
+    /// @param decompression_in_bufsize
+    ///   Size of the internal buffer holding input data to be decompressed.
+    ///   It can be different from 'file_io_bufsize' depending on a using 
+    ///   compression method, OS and file system.
+    /// @param decompression_out_bufsize
+    ///   Size of the internal buffer to receive data from a decompressor.
     /// @return
     ///   Return TRUE on success, FALSE on error.
     /// @sa
-    ///   CompressFile
+    ///   CompressFile, GetRecommendedBufferSizes, CLZOCompressionFile
+    /// 
     virtual bool DecompressFile(
         const string& src_file,
         const string& dst_file, 
-        size_t        buf_size = kCompressionDefaultBufSize
+        size_t        file_io_bufsize           = kCompressionDefaultBufSize,
+        size_t        decompression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        decompression_out_bufsize = kCompressionDefaultBufSize
     );
 
     /// Structure to keep compressed file information.
@@ -319,6 +370,50 @@ public:
         SFileInfo(void) : mtime(0) {};
     };
 
+    /// @warning No dictionary support for LZO. Always return FALSE.
+    /// @sa HaveSupport
+    virtual bool SetDictionary(
+        CCompressionDictionary& dict, 
+        ENcbiOwnership          own = eNoOwnership
+    );
+
+    //=======================================================================
+    // Advanced compression-specific parameters
+    //=======================================================================
+    // Allow to tune up (de)compression for a specific needs.
+    //
+    // - Pin down compression parameters to some specific values, so these
+    //   values are no longer dynamically selected by the compressor.
+    // - All setting parameters should be in the range [min,max], 
+    //   or equal to default.
+    // - All parameters should be set before starting (de)compression, 
+    //   or it will be ignored for current operation.
+    //=======================================================================
+
+    /// Block size 
+    /// 
+    /// LZO is a block compression algorithm - it compresses and decompresses
+    /// blocks of data. Block size must be the same for compression and 
+    /// decompression. This parameter define a block size used
+    /// for file/stream based compression/decompression to divide big 
+    /// data to chunks and compress them separately.
+    /// 
+    /// Block size set a memory budget for streaming (de)compression,
+    /// with larger values of block size requiring more memory and typically
+    /// better compression.
+    ///
+    /// Methods operated with all data located in memory, like CompressBuffer()
+    /// or DecompressBuffer() works by default with one big block, except
+    /// 'fStreamFormat' flag has specified. Stream format stores used block
+    /// size, so it is used automatically for decompression instead of provided
+    /// value.
+    /// 
+    void SetBlockSize(size_t block_size);
+    size_t GetBlockSize(void) const { return m_BlockSize; }
+    static size_t GetBlockSizeDefault(void);
+    static size_t GetBlockSizeMin(void);
+    static size_t GetBlockSizeMax(void);
+
 protected:
     /// Initialize compression parameters.
     void InitCompression(ELevel level);
@@ -385,21 +480,39 @@ class NCBI_XUTIL_EXPORT CLZOCompressionFile : public CLZOCompression,
                                               public CCompressionFile
 {
 public:
+    // TODO: add compression_in_bufsize / compression_out_bufsize parameters after 
+    // removing deprecated constructors only, to avoid conflicts. See zst as example.
+    // JIRA: CXX-12640
+
     /// Constructor.
-    /// For a special parameters description see CLZOCompression.
+    ///
+    /// Automatically calls Open() with given file name, mode and compression level.
+    /// @note
+    ///   This constructor don't allow to use any advanced compression parameters
+    ///   or a dictionary. If you need to set any of them, please use simplified
+    ///   conventional constructor, set advanced parameters and use Open().
+    /// 
     CLZOCompressionFile(
         const string& file_name,
         EMode         mode,
-        ELevel        level     = eLevel_Default,
-        size_t        blocksize = kLZODefaultBlockSize
+        ELevel        level = eLevel_Default
     );
-
     /// Conventional constructor.
-    /// For a special parameters description see CLZOCompression.
     CLZOCompressionFile(
-        ELevel        level     = eLevel_Default,
-        size_t        blocksize = kLZODefaultBlockSize
+        ELevel        level = eLevel_Default
     );
+    /// @deprecated Use CLZOCompressionFile(const string&, EMode, ELevel) construtor instead.
+    NCBI_DEPRECATED_CTOR(CLZOCompressionFile(
+        const string& file_name,
+        EMode         mode,
+        ELevel        level,
+        size_t        blocksize
+    ));
+    /// @deprecated Use CLZOCompressionFile(ELevel) construtor instead.
+    NCBI_DEPRECATED_CTOR(CLZOCompressionFile(
+        ELevel        level,
+        size_t        blocksize
+    ));
 
     /// Destructor
     ~CLZOCompressionFile(void);
@@ -410,11 +523,24 @@ public:
     ///   File name of the file to open.
     /// @param mode
     ///   File open mode.
+    /// @param compression_in_bufsize
+    ///   Size of the internal buffer holding input data to be (de)compressed.
+    /// @param compression_out_bufsize
+    ///   Size of the internal buffer to receive data from a (de)compressor.
     /// @return
     ///   TRUE if file was opened successfully or FALSE otherwise.
     /// @sa
     ///   CLZOCompression, Read, Write, Close
-    virtual bool Open(const string& file_name, EMode mode);
+    /// @note
+    ///   All advanced compression parameters or a dictionary should be set before
+    ///   Open() method, otherwise they will not have any effect.
+    /// 
+    virtual bool Open(
+        const string& file_name, 
+        EMode         mode,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
+    );
 
     /// Opens a compressed file for reading or writing.
     ///
@@ -424,15 +550,25 @@ public:
     /// @param mode
     ///   File open mode.
     /// @param info
-    ///   Pointer to file information structure. If it is not NULL,
-    ///   that it will be used to get information about compressed file
-    ///   in the read mode, and set it in the write mode for compressed 
-    ///   files.
+    ///   Pointer to file information structure. If not NULL, that it will
+    ///   be used to get information about compressed file in the read mode,
+    ///   and set it in the write mode for compressed files.
+    /// @param compression_in_bufsize
+    ///   Size of the internal buffer holding input data to be (de)compressed.
+    /// @param compression_out_bufsize
+    ///   Size of the internal buffer to receive data from a (de)compressor.
     /// @return
     ///   TRUE if file was opened successfully or FALSE otherwise.
     /// @sa
     ///   CLZOCompression, Read, Write, Close
-    virtual bool Open(const string& file_name, EMode mode, SFileInfo* info);
+    ///
+    virtual bool Open(
+        const string& file_name, 
+        EMode         mode, 
+        SFileInfo*    info,
+        size_t        compression_in_bufsize  = kCompressionDefaultBufSize,
+        size_t        compression_out_bufsize = kCompressionDefaultBufSize
+    );
 
     /// Read data from compressed file.
     /// 
@@ -447,6 +583,7 @@ public:
     ///   The number of really read bytes can be less than requested.
     /// @sa
     ///   Open, Write, Close
+    ///
     virtual long Read(void* buf, size_t len);
 
     /// Write data to compressed file.
@@ -462,6 +599,7 @@ public:
     ///   Returned value can be less than "len".
     /// @sa
     ///   Open, Read, Close
+    ///
     virtual long Write(const void* buf, size_t len);
 
     /// Close compressed file.
@@ -471,6 +609,7 @@ public:
     ///   TRUE on success, FALSE on error.
     /// @sa
     ///   Open, Read, Write
+    ///
     virtual bool Close(void);
 
 protected:
@@ -547,10 +686,18 @@ class NCBI_XUTIL_EXPORT CLZOCompressor : public CLZOCompression,
 public:
     /// Constructor.
     CLZOCompressor(
-        ELevel    level       = eLevel_Default,
-        size_t    blocksize   = kLZODefaultBlockSize,
-        TLZOFlags flags       = 0
+        ELevel    level = eLevel_Default,
+        TLZOFlags flags = 0
     );
+    /// @deprecated 
+    ///   Use CLZOCompressor(ELevel = eLevel_Default, TLZOFlags = 0) constructor
+    ///   without block size parameter, that can be set separately if necessary.
+    NCBI_DEPRECATED_CTOR(CLZOCompressor(
+        ELevel    level,
+        size_t    blocksize,
+        TLZOFlags flags
+    ));
+
     /// Destructor.
     virtual ~CLZOCompressor(void);
 
@@ -599,10 +746,15 @@ class NCBI_XUTIL_EXPORT CLZODecompressor : public CLZOCompression,
 {
 public:
     /// Constructor.
-    CLZODecompressor(
-        size_t    blocksize = kLZODefaultBlockSize,
-        TLZOFlags flags     = 0
-    );
+    CLZODecompressor(TLZOFlags flags);
+
+    /// @deprecated 
+    ///   Use CLZODecompressor(TLZOFlags = 0) constructor
+    ///   without block size parameter, that can be set separately if necessary.
+    NCBI_DEPRECATED_CTOR(CLZODecompressor(
+        size_t    blocksize,
+        TLZOFlags flags
+    ));
 
     /// Destructor.
     virtual ~CLZODecompressor(void);
@@ -661,30 +813,45 @@ public:
         CLZOCompression::ELevel    level,
         streamsize                 in_bufsize,
         streamsize                 out_bufsize,
-        size_t                     blocksize = kLZODefaultBlockSize,
         CLZOCompression::TLZOFlags flags = 0
         )
         : CCompressionStreamProcessor(
-              new CLZOCompressor(level, blocksize, flags),
-              eDelete, in_bufsize, out_bufsize)
+              new CLZOCompressor(level, flags), eDelete, in_bufsize, out_bufsize)
     {}
 
+    /// @deprecated 
+    ///   Use CLZOStreamCompressor() constructor
+    ///   without block size parameter, that can be set separately if necessary.
+    NCBI_DEPRECATED_CTOR(CLZOStreamCompressor(
+        CLZOCompression::ELevel    level,
+        streamsize                 in_bufsize,
+        streamsize                 out_bufsize,
+        size_t                     blocksize,
+        CLZOCompression::TLZOFlags flags
+    ));
+
     /// Conventional constructor
     CLZOStreamCompressor(
         CLZOCompression::ELevel    level,
         CLZOCompression::TLZOFlags flags = 0
         )
         : CCompressionStreamProcessor(
-              new CLZOCompressor(level, kLZODefaultBlockSize, flags),
+              new CLZOCompressor(level, flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
 
     /// Conventional constructor
     CLZOStreamCompressor(CLZOCompression::TLZOFlags flags = 0)
         : CCompressionStreamProcessor(
-              new CLZOCompressor(CLZOCompression::eLevel_Default, kLZODefaultBlockSize, flags),
+              new CLZOCompressor(CLZOCompression::eLevel_Default, flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
+
+    /// Return a pointer to compressor.
+    /// Can be used mostly for setting an advanced compression-specific parameters.
+    CLZOCompressor* GetCompressor(void) const {
+        return dynamic_cast<CLZOCompressor*>(GetProcessor());
+    }
 };
 
 
@@ -706,20 +873,34 @@ public:
     CLZOStreamDecompressor(
         streamsize                 in_bufsize,
         streamsize                 out_bufsize,
-        size_t                     blocksize,
         CLZOCompression::TLZOFlags flags   = 0
         )
         : CCompressionStreamProcessor(
-             new CLZODecompressor(blocksize, flags),
-             eDelete, in_bufsize, out_bufsize)
+             new CLZODecompressor(flags), eDelete, in_bufsize, out_bufsize)
     {}
 
+    /// @deprecated 
+    ///   Use CLZOStreamDecompressor() constructor
+    ///   without block size parameter, that can be set separately if necessary.
+    NCBI_DEPRECATED_CTOR(CLZOStreamDecompressor(
+        streamsize                 in_bufsize,
+        streamsize                 out_bufsize,
+        size_t                     blocksize,
+        CLZOCompression::TLZOFlags flags
+    ));
+
     /// Conventional constructor
     CLZOStreamDecompressor(CLZOCompression::TLZOFlags flags = 0)
         : CCompressionStreamProcessor( 
-              new CLZODecompressor(kLZODefaultBlockSize, flags),
+              new CLZODecompressor(flags),
               eDelete, kCompressionDefaultBufSize, kCompressionDefaultBufSize)
     {}
+
+    /// Return a pointer to decompressor.
+    /// Can be used mostly for setting an advanced compression-specific parameters.
+    CLZODecompressor* GetDecompressor(void) const {
+        return dynamic_cast<CLZODecompressor*>(GetProcessor());
+    }
 };
 
 
