@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__STREAM__HPP
 #define UTIL_COMPRESS__STREAM__HPP
 
-/*  $Id: stream.hpp 612744 2020-07-27 14:26:09Z ivanov $
+/*  $Id: stream.hpp 656534 2022-09-22 17:57:22Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -234,11 +234,18 @@ public:
     /// (Re)Initialize stream processor.
     void Init(void);
 
-    // Get stream processor's status
+    /// Get stream processor's status
     bool IsOkay(void) const {
         return m_Processor  &&  m_Processor->IsBusy()  &&  m_State != eDone;
     }
 
+    /// Return a pointer to currently used stream processor.
+    /// Can be used mostly for setting advanced compression-specific parameters
+    /// for stream compressors/decompressors from the upper level code.
+    CCompressionProcessor* GetProcessor(void) const {
+        return m_Processor;
+    }
+
 private:
     CCompressionProcessor* m_Processor;   ///< (De)compression processor.
     CT_CHAR_TYPE*          m_InBuf;       ///< Buffer of unprocessed data.
@@ -316,14 +323,12 @@ protected:
     /// Default constructor.
     ///
     /// Default constructor allow to create stream with specific
-    /// characteristics later, not necessary in the constructor.
-    /// Can be used in derived classes.
+    /// characteristics later. Can be used in derived classes.
     /// @sa CCompressionStream, Create()
     CCompressionIStream(void) : CNcbiIstream(0) { }
 
-    /// Create stream with specific characteristics later,
-    /// not necessary in the constructor. 
-    /// Do nothing, if stream is already initialized.
+    /// Create stream with specific characteristics. 
+    /// Do nothing, if stream is already created.
     /// @sa CCompressionStream
     void Create(CNcbiIos&                    stream,
                 CCompressionStreamProcessor* stream_processor,
