@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_cfg.sh 630684 2021-05-06 15:49:59Z ivanov $
+# $Id: check_make_cfg.sh 677891 2024-01-24 19:09:24Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -139,6 +139,7 @@ if test -n "$NCBI_AUTOMATED_BUILD"; then
          vs2013 ) signature="VS_2013"   ;;
          vs2015 ) signature="VS_2015"   ;;
          vs2017 ) signature="VS_2017"   ;;
+         vs2019 ) signature="VS_2019"   ;;
       esac
       signature="$signature-${x_cfg}"
       case "$x_cfg" in
@@ -240,7 +241,7 @@ case "\$method" in
       ( 
 #      cat \$res_log
       test -f \$res_journal  ||  exit 0
-      x_files=\`cat \$res_journal | sed -e 's/ /%gj_s4%/g'\`
+      x_files=\`cat \$res_journal | tr -d '\r' | sed -e 's/ /%gj_s4%/g'\`
       for x_file in \$x_files; do
          x_file=\`echo "\$x_file" | sed -e 's/%gj_s4%/ /g'\`
          echo 
@@ -256,7 +257,7 @@ case "\$method" in
       ( 
 #      egrep 'ERR \[|TO  -' \$res_log
       test -f \$res_journal  ||  exit 0
-      x_files=\`cat \$res_journal | sed -e 's/ /%gj_s4%/g'\`
+      x_files=\`cat \$res_journal | tr -d '\r' | sed -e 's/ /%gj_s4%/g'\`
       for x_file in \$x_files; do
          x_file=\`echo "\$x_file" | sed -e 's/%gj_s4%/ /g'\`
          x_code=\`grep -c '@@@ EXIT CODE:' \$x_file\`
@@ -320,6 +321,13 @@ fi
 if test -z "\$NCBI_TEST_DATA"; then
     case `uname -s` in
        CYGWIN* ) NCBI_TEST_DATA=//snowman/win-coremake/Scripts/test_data ;;
+       Darwin )
+           if test -d /am/ncbiapdata/test_data; then
+               NCBI_TEST_DATA=/am/ncbiapdata/test_data
+           else
+               NCBI_TEST_DATA=/Volumes/ncbiapdata/test_data
+           fi
+           ;;
        *)        NCBI_TEST_DATA=/am/ncbiapdata/test_data ;;
     esac
     export NCBI_TEST_DATA
@@ -359,11 +367,16 @@ if test -n "\$NCBI_AUTOMATED_BUILD"; then
    fi
 fi
 
-# Check for ncbi_applog
+# Check for some executables
 have_ncbi_applog=false
 if (ncbi_applog generate) >/dev/null 2>&1; then
    have_ncbi_applog=true
 fi
+have_uptime=false
+if (which uptime) >/dev/null 2>&1; then
+   have_uptime=true
+fi
+
 
 #//////////////////////////////////////////////////////////////////////////
 
@@ -400,6 +413,13 @@ RunTest() {
     x_ext="\$6"
     x_timeout="\$7"
     x_authors="\$8"
+    x_unmet_req="\$9"
+
+    # Check if test don't met requires: so don't run, but mark as "disabled"
+    is_not_skipped=true
+    if test -n "\$x_unmet_req"; then
+        is_not_skipped=false
+    fi
  
     x_work_dir="\$check_dir/\$x_wdir"
     mkdir -p "\$x_work_dir" > /dev/null 2>&1 
@@ -513,6 +533,8 @@ RunTest() {
            start_time="\`date +'$x_date_format'\`"
            check_exec="\$root_dir/scripts/common/check/check_exec.sh"
            
+           if \$is_not_skipped; then
+
            # Generate PHID and SID for a new test
            logfile=\$NCBI_CONFIG__LOG__FILE
            NCBI_CONFIG__LOG__FILE=
@@ -557,13 +579,25 @@ RunTest() {
            exec_time=\`echo "\$exec_time" | tr '\n\r' '%%'\`
            echo \$exec_time | grep 'real [0-9]\|Maximum execution .* is exceeded' > /dev/null 2>&1 
            if test \$? -eq 0;  then
-              exec_time=\`echo \$exec_time | sed -e 's/^%*//' -e 's/%*$//' -e 's/%%/%/g' -e 's/%/, /g' -e 's/[ ] */ /g'\`
+              exec_time=\`echo \$exec_time | tr -d '\n\r' | sed -e 's/^%*//' -e 's/%*$//' -e 's/%%/%/g' -e 's/%/, /g' -e 's/[ ] */ /g'\`
            else
               exec_time='unparsable timing stats'
            fi
            rm -f $x_tmp/\$\$.out
            rm -f \$x_test_out.\$\$
 
+           else # skipped
+               echo "Test disabled due unmet check requires: \$x_unmet_req" >> \$x_test_out
+               echo NCBI_UNITTEST_DISABLED >> \$x_test_out
+               echo >> \$x_test_out
+               # Define necessary variables to write into the test footer
+               stop_time="\$start_time"
+               load_avg=""
+               result=0
+               exec_time="real 0.0, user 0.0, sys 0.0"
+               runid=""
+           fi
+
            # Write result of the test into the his output file
            echo "Start time   : \$start_time"   >> \$x_test_out
            echo "Stop time    : \$stop_time"    >> \$x_test_out
@@ -636,20 +670,43 @@ RunTest() {
         # Always load test results for automated builds on a 'run' command.
         
         if \$is_run && \$is_db_load; then
-           echo "\$x_path_app:" >> "\$build_dir/test_stat_load.log" 2>&1
+           stat_out="\$build_dir/test_stat_load.tmp.\$x_app"
+           retry="${x_check_scripts_dir}/retry_db_load.sh"
+           echo "======================================================================" >> \$stat_out 2>&1
+           echo "[\$build_tree/\$build_cfg] \$x_name"                                    >> \$stat_out 2>&1
+           echo "======================================================================" >> \$stat_out 2>&1
+           echo                                                                          >> \$stat_out 2>&1
            if test -n "\$saved_phid";  then
               NCBI_LOG_HIT_ID=\$saved_phid
               export NCBI_LOG_HIT_ID
            fi
            case "$x_compiler" in
              MSVC )
-                test_stat_load "\$(cygpath -w "\$x_test_rep")" "\$(cygpath -w "\$x_test_out")" "\$(cygpath -w "\$x_boost_rep")" "\$(cygpath -w "\$top_srcdir/build_info")" >> "\$build_dir/test_stat_load.log" 2>&1
+                # To avoid problems with retry script:
+                # - convert Cygwin paths to Windows paths
+                # - convert back slashes to forward slashes
+                args=""
+                for arg in "\$(cygpath -w "\$x_test_rep")" "\$(cygpath -w "\$x_test_out")" "\$(cygpath -w "\$x_boost_rep")" "\$(cygpath -w "\$top_srcdir/build_info")"; do
+                    args="\$args \$arg"
+                done
+                args=\`echo \$args | tr '\\\\\' '/'\`
+                \$retry \$stat_out test_stat_load \$args
                 ;;        
              XCODE ) 
-                $NCBI/bin/_production/CPPCORE/test_stat_load "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" >> "\$build_dir/test_stat_load.log" 2>&1
+                \$retry \$stat_out $NCBI/bin/_production/CPPCORE/test_stat_load "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info"
                 ;;
            esac
-           echo >> "\$build_dir/test_stat_load.log" 2>&1
+           case \$? in
+             0 )
+               echo "OK: [\$build_tree/\$build_cfg] \$x_name" >> \$stat_out 2>&1 ;;
+             4 )
+               echo "TO: Error loading results for [\$build_tree/\$build_cfg] \$x_name" >> \$stat_out 2>&1 ;;
+             * )
+               echo "ERR: Error loading results for [\$build_tree/\$build_cfg] \$x_name" >> \$stat_out 2>&1 ;;
+           esac
+           echo >> \$stat_out 2>&1
+           cat \$stat_out >> "\$build_dir/test_stat_load.log" 2>&1        
+           rm -f \$stat_out > /dev/null 2>&1
         fi
         if test \$is_run  -a  -n "\$saved_phid"; then
             rm -f \$saved_phid* > /dev/null 2>&1
@@ -691,13 +748,13 @@ EOF
 
 
 # Read list with tests
-x_tests=`cat "$x_list" | sed -e 's/ /%gj_s4%/g'`
+x_tests=`cat "$x_list" | tr -d '\r' | sed -e 's/ /%gj_s4%/g'`
 x_test_prev=""
    
 # For all tests
 for x_row in $x_tests; do
    # Get one row from list
-   x_row=`echo "$x_row" | sed -e 's/%gj_s4%/ /g' -e 's/^ *//' -e 's/ ____ /~/g'`
+   x_row=`echo "$x_row" | sed -e 's/%gj_s4%/ /g' -e 's/^ *//' -e 's/ ____ /~/g' | tr -d '\r'`
 
    # Split it to parts
    x_rel_dir=`echo \$x_row | sed -e 's/~.*$//'`
@@ -708,22 +765,26 @@ for x_row in $x_tests; do
    x_name=`echo "$x_row" | sed -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/~.*$//'`
    x_files=`echo "$x_row" | sed -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/~.*$//'`
    x_timeout=`echo "$x_row" | sed -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/~.*$//'`
-   ###x_requires=`echo "$x_row" | sed -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/~.*$//'`
+   x_requires=`echo "$x_row" | sed -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/^[^~]*~//' -e 's/~.*$//'`
    x_authors=`echo "$x_row" | sed -e 's/.*~//'`
 
    # Default timeout
    test -z "$x_timeout"  &&  x_timeout=$NCBI_CHECK_TIMEOUT_DEFAULT
 
-   # Check application requirements
-   # TODO:
-   #    This check can be removed later, when project_tree_builder starts to check requiments
-   #    on check list generation step.
-   ###for x_req in $x_requires; do
-   ###   (echo "$x_features" | grep " $x_req " > /dev/null)  ||  continue 2
-   ###done
+   # Check requirements and save unmet requires for later reporting as disabled test
+   x_unmet_requires=""
+   for x_req in $x_requires; do
+       t_minus=${x_req::1}
+       if test $t_minus = "-"; then
+          t_req=${x_req:1}
+          (echo "$x_features" | grep " $t_req " > /dev/null)  &&  x_unmet_requires="${x_unmet_requires}${x_req} "
+       else
+          (echo "$x_features" | grep " $x_req " > /dev/null)  ||  x_unmet_requires="${x_unmet_requires}${x_req} "
+       fi
+   done
    
    # Copy specified files into the check tree
-   if test ! -z "$x_files" ; then
+   if test ! -z "$x_files"  -a  -z "$x_unmet_requires"; then
       x_path="$x_check_dir/$x_rel_dir"
       mkdir -p "$x_path"
       # Automatically copy .ini file if exists
@@ -756,6 +817,7 @@ for x_row in $x_tests; do
    fi
    x_test_prev="$x_test"
 
+
    # Write current test commands into a script file
    cat >> $x_out <<EOF
 ######################################################################
@@ -766,7 +828,8 @@ RunTest "$x_rel_dir"  \\
         "$x_name"     \\
         "$x_test_ext" \\
         "$x_timeout"  \\
-        "$x_authors"
+        "$x_authors"  \\
+        "$x_unmet_requires"
 EOF
 
 #//////////////////////////////////////////////////////////////////////////
