@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_unix_cmake.sh 630684 2021-05-06 15:49:59Z ivanov $
+# $Id: check_make_unix_cmake.sh 680027 2024-03-08 18:23:04Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -342,6 +342,8 @@ export NCBI_CHECK_TIMEOUT_DEFAULT=$NCBI_CHECK_TIMEOUT_DEFAULT
 if test -z "\$NCBI_TEST_DATA"; then
     if [ \$cygwin = true ]; then
        NCBI_TEST_DATA=//snowman/win-coremake/Scripts/test_data
+    elif test -d /Volumes/ncbiapdata/test_data; then
+       NCBI_TEST_DATA=/Volumes/ncbiapdata/test_data
     else
        NCBI_TEST_DATA=/am/ncbiapdata/test_data
     fi
@@ -355,8 +357,8 @@ fi
 
 # Valgrind/Helgrind configurations
 VALGRIND_SUP="\${script_dir}/common/check/valgrind.supp"
-VALGRIND_CMD="--tool=memcheck --suppressions=\$VALGRIND_SUP"
-HELGRIND_CMD="--tool=helgrind --suppressions=\$VALGRIND_SUP"
+VALGRIND_CMD="--tool=memcheck --gen-suppressions=all --suppressions=\$VALGRIND_SUP"
+HELGRIND_CMD="--tool=helgrind --gen-suppressions=all --suppressions=\$VALGRIND_SUP"
 if (valgrind --ncbi --help) >/dev/null 2>&1; then
     VALGRIND_CMD="--ncbi \$VALGRIND_CMD" # --ncbi must be the first option!
     HELGRIND_CMD="--ncbi \$HELGRIND_CMD" # --ncbi must be the first option!
@@ -389,8 +391,15 @@ if test -z "\$NCBI_EXPORT_PROJECT"; then
     export NCBI_CONFIG__LOG__FILE
 fi
 
-# Add additional necessary directories to PATH: current, build, scripts, utility and $HOME/bin (for Ubuntu).
 PATH="\${script_dir}/common/impl:\$NCBI/bin/_production/CPPCORE:\$HOME/bin:.:\${build_dir}:\${bin_dir}:\${PATH}"
+
+export PATH
+# Add additional necessary directories to PATH: current, build, scripts, utility and \$HOME/bin (for Ubuntu, automated build only)
+if test -n "\$NCBI_AUTOMATED_BUILD"; then
+    PATH="\${script_dir}/common/impl:\$NCBI/bin/_production/CPPCORE:\$HOME/bin:.:\${build_dir}:\${bin_dir}:\${PATH}"
+else
+    PATH="\${script_dir}/common/impl:\$NCBI/bin/_production/CPPCORE:.:\${build_dir}:\${bin_dir}:\${PATH}"
+fi
 export PATH
 
 # Export bin and lib pathes
@@ -520,15 +529,15 @@ is_max_debug=false
 if test -f "\${conf_dir}/status/MaxDebug.enabled"; then
    is_max_debug=true
 fi
-if test "\$NCBI_CHECK_SETLIMITS" != "0"  -a  ! \$is_max_debug; then
+if test "\$NCBI_CHECK_SETLIMITS" != "0"  -a  \$is_max_debug = false; then
    ulimit -c 1000000
    ulimit -n 8192
    if [ \$cygwin = false ]; then
        if test "\$NCBI_CHECK_TOOLS" = "regular"; then
-          ulimit -v 48000000
+          ulimit -v 110000000
        else
           # Increase memory limits if run under check tool
-          ulimit -v 64000000
+          ulimit -v 150000000
        fi
    fi
 fi
@@ -561,6 +570,10 @@ RunTest()
     x_log="$x_tmp/\$\$.~\$x_name.out"
     x_info="$x_tmp/\$\$.~\$x_name.info"
 
+    if test -e "\$checkdir/~ABORT"; then
+        touch "\$checkdir/~RUN_CHECKS.next"
+        return 0
+    fi
     if test -f "/etc/nologin"; then
         echo "Nologin detected, probably host going to reboot. Skipping test:" \$x_name
         touch "\$checkdir/~RUN_CHECKS.next"
@@ -575,31 +588,27 @@ RunTest()
     fi
     test -d \${x_work_dir} || mkdir -p \${x_work_dir}
 
+
+    # Check requirements
+    
+    x_unmet_requires=""
     for x_req in \$x_requires; do
-      t_sub=\${x_req::1}
-      t_res="yes"
-      if test \$t_sub = "-"; then
-          t_sub=\${x_req:1}
-          if test -f "\$conf_dir/status/\$t_sub.enabled" ; then
-            t_res="no"
-          fi
-      else
-          t_sub=\${x_req}
-          if test ! -f "\$conf_dir/status/\$t_sub.enabled" ; then
-            t_res="no"
-          fi
-      fi
-      if test "\$t_res" = "no"; then
-           x_test_out="\$x_wlog_dir/\$x_name.test_out\$x_ext"
-           echo NCBI_UNITTEST_SKIPPED > \$x_test_out
-           echo "t_cmd=\\"[\$r_id/\$x_TestsTotal \$x_work_dir_tail] \$x_name (unmet CHECK_REQUIRES=\$x_req)\\"" > \$x_info
-           echo "t_test_out=\\"\$x_test_out\\"" >> \$x_info
-           mv \$x_info "\${x_done}.done"
-           touch "\$checkdir/~RUN_CHECKS.next"
-           rm -rf "\${x_work_dir}"
-           return 0
-      fi
+        t_minus=\${x_req::1}
+        if test \$t_minus = "-"; then
+           t_req=\${x_req:1}
+           if test -f "\$conf_dir/status/\${t_req}.enabled" ; then
+              x_unmet_requires="\${x_unmet_requires}\${x_req} "
+           fi
+        else
+           if test ! -f "\$conf_dir/status/\${x_req}.enabled" ; then
+              x_unmet_requires="\${x_unmet_requires}\${x_req} "
+           fi
+        fi
     done
+    is_not_skipped=true
+    if test -n "\$x_unmet_requires"; then
+        is_not_skipped=false
+    fi
 
     if test -n "\$x_resources"; then
         for r in \$x_resources ; do
@@ -793,6 +802,8 @@ RunTest()
 
                 # Run check
                 start_time="\`date +'$x_date_format'\`"
+
+                if \$is_not_skipped; then
                         
                 # Use separate shell to run test.
                 # This will allow to know execution time for applications with timeout.
@@ -829,7 +840,7 @@ EOF_launch
                 exec_time=\`\${checkroot}/sysdep.sh tl 7 \$x_log | tr '\n\r' '%%' | tr -d '\000-\037' | tr  -d '\176-\377'\`
                 echo \$exec_time | egrep 'real [0-9]|Maximum execution .* is exceeded' > /dev/null 2>&1 
                 if test \$? -eq 0;  then
-                    exec_time=\`echo \$exec_time |   \\
+                    exec_time=\`echo \$exec_time | tr -d '\n\r' | \\
                                 sed -e 's/%%/%/g'    \\
                                     -e 's/%$//'      \\
                                     -e 's/%/, /g'    \\
@@ -860,11 +871,27 @@ EOF_launch
                                    grep '==ERROR: AddressSanitizer:' \$x_test_out > /dev/null 2>&1 
                                    if test \$? -eq 0;  then
                                       result=253
+                                   else
+                                       grep 'FATAL: ThreadSanitizer CHECK failed:' \$x_test_out > /dev/null 2>&1 
+                                       if test \$? -eq 0;  then
+                                          result=252
+                                       fi
                                    fi
                                fi
-                    
                 esac
 
+                else # skipped
+                    echo "Test disabled due unmet check requires: \$x_unmet_requires" >> \$x_test_out
+                    echo NCBI_UNITTEST_DISABLED >> \$x_test_out
+                    echo >> \$x_test_out
+                    # Define necessary variables to write into the test footer
+                    result=0
+                    stop_time="\$start_time"
+                    load_avg=""
+                    exec_time="real 0.0, user 0.0, sys 0.0"
+                    runid=""
+                fi
+
                 # Write result of the test into the his output file
                 echo "Start time   : \$start_time"   >> \$x_test_out
                 echo "Stop time    : \$stop_time"    >> \$x_test_out
@@ -891,19 +918,14 @@ EOF_launch
                 if \$is_automated; then
                     if grep NCBI_UNITTEST_DISABLED \$x_test_out >/dev/null; then
                         echo "DIS" >> "\$x_test_rep"
-
                     elif grep NCBI_UNITTEST_SKIPPED \$x_test_out >/dev/null; then
                         echo "SKP" >> "\$x_test_rep"
-
                     elif grep NCBI_UNITTEST_TIMEOUTS_BUT_NO_ERRORS \$x_test_out >/dev/null; then
                         echo "TO" >> "\$x_test_rep"
-
                     elif echo "\$exec_time" | egrep 'Maximum execution .* is exceeded' >/dev/null || egrep "Maximum execution .* is exceeded" \$x_test_out >/dev/null; then
                         echo "TO" >> "\$x_test_rep"
-
                     elif test \$result -eq 0; then
                         echo "OK" >> "\$x_test_rep"
-
                     else
                         echo "ERR" >> "\$x_test_rep"
                     fi
@@ -946,21 +968,46 @@ EOF_launch
         if \$is_run && \$is_db_load; then
             while ! mkdir "\$checkdir/~test_stat_load.lock" 2>/dev/null
             do
-	            sleep 1
+               sleep 1
             done
-           if test -n "\$saved_phid";  then
-              NCBI_LOG_HIT_ID=\$saved_phid
-              export NCBI_LOG_HIT_ID
-           fi
-           case \`uname -s\` in
-              CYGWIN* )
-                test_stat_load "\$(cygpath -w "\$x_test_rep")" "\$(cygpath -w "\$x_test_out")" "\$(cygpath -w "\$x_boost_rep")" "\$(cygpath -w "\$top_srcdir/build_info")" >> "\${checkdir}/test_stat_load.log" 2>&1 ;;
-              IRIX* )
-                test_stat_load.sh "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" >> "\${checkdir}/test_stat_load.log" 2>&1 ;;
-              * )
-                test_stat_load "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" >> "\${checkdir}/test_stat_load.log" 2>&1 ;;
+            stat_out="\${checkdir}/test_stat_load.tmp.\$x_name"
+            retry="\${script_dir}/common/check/retry_db_load.sh"
+            echo "======================================================================" >> \$stat_out 2>&1
+            echo "[\$x_work_dir_tail] \$x_name"                                           >> \$stat_out 2>&1
+            echo "======================================================================" >> \$stat_out 2>&1
+            echo                                                                          >> \$stat_out 2>&1
+            if test -n "\$saved_phid";  then
+               NCBI_LOG_HIT_ID=\$saved_phid
+               export NCBI_LOG_HIT_ID
+            fi
+            case \`uname -s\` in
+               CYGWIN* )
+                 # To avoid problems with retry script:
+                 # - convert Cygwin paths to Windows paths
+                 # - convert back slashes to forward slashes
+                 args=""
+                 for arg in "\$(cygpath -w "\$x_test_rep")" "\$(cygpath -w "\$x_test_out")" "\$(cygpath -w "\$x_boost_rep")" "\$(cygpath -w "\$top_srcdir/build_info")"; do
+                     args="\$args \$arg"
+                 done
+                 args=\`echo \$args | tr '\\\\\' '/'\`
+                 \$retry \$stat_out test_stat_load \$args
+                 ;;
+               IRIX* )
+                 \$retry \$stat_out test_stat_load.sh "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" ;;
+               * )
+                 \$retry \$stat_out test_stat_load "\$x_test_rep" "\$x_test_out" "\$x_boost_rep" "\$top_srcdir/build_info" ;;
             esac
-            echo >> "\${checkdir}/test_stat_load.log" 2>&1
+            case \$? in
+             0 )
+               echo "OK: [\$x_work_dir_tail] \$x_name" >> \$stat_out 2>&1 ;;
+             4 )
+               echo "TO: Error loading results for [\$x_work_dir_tail] \$x_name" >> \$stat_out 2>&1 ;;
+             * )
+               echo "ERR: Error loading results for [\$x_work_dir_tail] \$x_name" >> \$stat_out 2>&1 ;;
+            esac
+            echo >> \$stat_out 2>&1
+            cat \$stat_out >> "\${checkdir}/test_stat_load.log" 2>&1
+            rm -f \$stat_out > /dev/null 2>&1
             rm -rf "\$checkdir/~test_stat_load.lock"
         fi
         if test \$is_run  -a  -n "\$saved_phid"; then
@@ -1093,6 +1140,7 @@ ProcessDone()
             fi
         fi
     done
+    rm "\$checkdir/~ABORT" 2>/dev/null
     rm "\$checkdir/~DONE"
 }
 
@@ -1103,7 +1151,11 @@ AddJob()
     a_id="\$3"
 
     if test "\${a_pid}" -gt 0; then
-        echo "        Start \$a_id: \${a_name} (\$a_pid)" > "\$checkdir/~\$a_name.started"
+        if test ! -e "\$checkdir/~ABORT"; then
+            echo "        Start \$a_id: \${a_name} (\$a_pid)" > "\$checkdir/~\$a_name.started"
+        else
+            echo "        Drop: \${a_name}"
+        fi
         while test ! -e "\$checkdir/~RUN_CHECKS.next"; do
             if test -e "\$checkdir/~SERIAL.lock"; then
                 sleep 2
@@ -1134,7 +1186,6 @@ AddJob()
     a_run=\`ls \${checkdir}/*.in_progress 2>/dev/null\`
     a_run=\`echo \$a_run | wc -w | sed -e 's/ //g'\`
     if test "\${a_run}" -ne "\${a_run}"; then
-echo "error:  1 a_run = \$a_run"
         a_run=0
     fi
 
@@ -1143,7 +1194,6 @@ echo "error:  1 a_run = \$a_run"
         a_run=\`ls \${checkdir}/*.in_progress 2>/dev/null\`
         a_run=\`echo \$a_run | wc -w | sed -e 's/ //g'\`
         if test "\${a_run}" -ne "\${a_run}"; then
-echo "error:  2 a_run = \$a_run"
             break
         fi
         if test "\${a_run}" -le 0; then
@@ -1183,6 +1233,7 @@ fi
 x_test=""
 x_TestsTotal=\`cat "\$res_list" | wc -l | sed -e 's/ //g'\`
 x_START=\$SECONDS
+trap "echo Stopping tests, please wait...; touch \${checkdir}/~ABORT"  INT
 RunJobs "\$res_list" &
 ProcessDone
 x_DURATION=\`expr \$SECONDS - \$x_START\`
