@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: generate_all_objects.sh 619037 2020-10-29 17:49:26Z ucko $
+# $Id: generate_all_objects.sh 668296 2023-06-01 18:09:45Z ivanov $
 
 # Generate classes for all public ASN.1/XML specs and selected
 # internal ones (if present).
@@ -7,7 +7,7 @@
 LC_ALL=C
 export LC_ALL
 
-new_module=$NCBI/c++.metastable/Release/build/new_module.sh
+new_module=$NCBI/c++.metastable/ReleaseMT/build/new_module.sh
 clients=false
 force=false
 
@@ -15,14 +15,15 @@ for arg in "$@"; do
     case "$arg" in
         --clients) clients=true ;;
         --force ) force=true ;;
-        *       ) echo "Usage: $1 [--clients] [--force]" ; exit 1 ;;
+        *       ) echo "Usage: $0 [--clients] [--force]" ; exit 1 ;;
     esac
 done
 
 failed=
 
 for spec in src/serial/test/we_cpp.asn src/objects/*/*.asn \
-  src/objtools/eutils/*/*.dtd src/gui/objects/*.asn src/gui/framework/*.dtd \
+  src/objtools/eutils/*/*.dtd src/objtools/eutils/*/*.xsd \
+  src/gui/objects/*.asn src/gui/framework/*.dtd \
   src/algo/gnomon/gnomon.asn src/algo/ms/formats/*/*.??d \
   src/build-system/project_tree_builder/msvc71_project.dtd \
   src/build-system/project_tree_builder/property_list.xsd \
@@ -75,7 +76,10 @@ for spec in src/serial/test/we_cpp.asn src/objects/*/*.asn \
         if $clients; then
             grep '^clients[     ]*=' $dir/$base.def >/dev/null 2>&1 || continue
         fi
-        if $force || [ ! -f $dir/$base.files ]; then
+        if $force; then
+            rm -f $dir/$base.files
+        fi
+        if [ ! -f $dir/$base.files ]; then
             echo $spec
             if (cd $dir && $new_module $flag $base >/dev/null 2>&1); then
                 : # all good
@@ -97,8 +101,12 @@ done
 splitdb_dir=src/internal/blast/SplitDB/asn
 if [ -f $splitdb_dir/Makefile.asntool ]; then
     top_srcdir=`pwd`
-    builddir=`ls -dt $top_srcdir/*/build $top_srcdir/.[A-Z]??*/build | head -1`
-    [ -d "$builddir" ] || builddir=$NCBI/c++.metastable/Release/build
+    builds=`ls -dt $top_srcdir/*/build $top_srcdir/.[A-Z]??*/build 2>/dev/null`
+    for builddir in $builds $NCBI/c++.metastable/ReleaseMT/build; do
+        if [ -f $builddir/Makefile.mk ]; then
+            break
+        fi
+    done
     make_asntool="${MAKE-make} -f Makefile.asntool sources top_srcdir=$top_srcdir builddir=$builddir"
     if $force || [ ! -f ${splitdb_dir}gendefs/objGendefs.c ]; then
         (cd ${splitdb_dir}gendefs && $make_asntool) || failed="$failed asngendefs-C"
