@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: install.sh 429376 2014-03-13 19:37:01Z ucko $
+# $Id: install.sh 631931 2021-05-24 21:39:35Z fukanchi $
 # Author:  Denis Vakatov, NCBI 
 #
 ###########################################################################
@@ -16,8 +16,8 @@
 
 echo "[`date`]"
 
-svn_location=`echo '$URL: https://svn.ncbi.nlm.nih.gov/repos/toolkit/release/blast/2.12.0/c++/scripts/common/impl/install.sh $' | sed "s%\\$[U]RL: *\\([^$][^$]*\\) \\$.*%\\1%"`
-svn_revision=`echo '$Revision: 429376 $' | sed "s%\\$[R]evision: *\\([^$][^$]*\\) \\$.*%\\1%"`
+svn_location=`echo '$URL: https://svn.ncbi.nlm.nih.gov/repos/toolkit/release/blast/2.16.0/c++/scripts/common/impl/install.sh $' | sed "s%\\$[U]RL: *\\([^$][^$]*\\) \\$.*%\\1%"`
+svn_revision=`echo '$Revision: 631931 $' | sed "s%\\$[R]evision: *\\([^$][^$]*\\) \\$.*%\\1%"`
 
 script_name=`basename $0`
 script_dir=`dirname $0`
@@ -106,7 +106,7 @@ fi
 if test "$with_src" = "no"; then
     install_dirs="$install_dirs src/build-system"
 else
-    install_dirs="$install_dirs src configure"
+    install_dirs="$install_dirs src configure cmake-configure cmake-configure.bat"
 fi
 if test "$with_doc" != "no" -a -d "$top_srcdir/doc"; then
     install_dirs="$install_dirs doc"
