@@ -1,4 +1,4 @@
-# $Id: ncbicxx_build_info.py 595471 2019-10-22 15:54:41Z ucko $
+# $Id: ncbicxx_build_info.py 675074 2023-11-14 13:33:27Z ivanov $
 import ast
 from datetime import date, datetime, timedelta, timezone
 from distutils.sysconfig import parse_makefile, expand_makefile_vars
@@ -176,7 +176,7 @@ class Collector(object):
         if command[0].endswith('.sh'):
             raise IrrelevantCommandError
 
-        irrelevant_re = re.compile(r'(?:check|clean|export-headers'
+        irrelevant_re = re.compile(r'(?:check|clean|export-headers|flag-stamps'
                                    + r'|mark-as-disabled|purge|requirements'
                                    + r'|sources)(?:[._].+)?$')
         template_re   = re.compile(r'/Makefile\.(app|lib).tmpl$')
@@ -206,7 +206,7 @@ class Collector(object):
                 info['target_name'] = x[5:]
             elif x.startswith('srcdir='):
                 info['srcdir'] = x[7:]
-            elif x[0] != '-' and not '=' in x:
+            elif x[0] != '-' and not '=' in x and not x.isnumeric():
                 relevant_targets.append(x)
         if len(info) < 3 \
            or (len(irrelevant_targets) > 0 and len(relevant_targets) == 0):
@@ -268,7 +268,7 @@ class Collector(object):
     def get_vcs_info(self, srcdir, rest = (), fallback = None):
         if os.path.isdir(os.path.join(srcdir, '.svn')):
             return self.get_svn_info(srcdir, rest)
-        elif os.path.isdir(os.path.join(srcdir, '.git')):
+        elif os.path.exists(os.path.join(srcdir, '.git')):
             return self.get_git_info(srcdir, rest)
         elif len(rest) == 0 and os.path.isdir(os.path.join(srcdir, 'CVS')):
             return self.get_cvs_info(srcdir)
@@ -306,11 +306,11 @@ class Collector(object):
                     break
         if 'vcs_path' not in info:
             # Maybe controlled by git after all, in a hybrid layout?
-            if os.path.isdir(os.path.join(srcdir, '.git')):
+            if os.path.exists(os.path.join(srcdir, '.git')):
                 return self.get_git_info(srcdir, rest)
             while srcdir != '/':
                 (srcdir, child) = os.path.split(srcdir)
-                if os.path.isdir(os.path.join(srcdir, '.git')):
+                if os.path.exists(os.path.join(srcdir, '.git')):
                     return self.get_git_info(srcdir, (child,) + rest)
             return None
         return info
@@ -333,10 +333,8 @@ class Collector(object):
                                       universal_newlines = True,
                                       cwd = srcdir) as remote:
                     for l in remote.stdout:
-                        (k, v) = l.strip().split(': ', 1)
-                        if k == 'Fetch URL':
-                            url = v
-                            break
+                        if 'Fetch URL: ' in l:
+                            url = l.strip()[11:]
             except subprocess.CalledProcessError:
                 pass
             if url is None:
