@@ -1,6 +1,6 @@
 #!/bin/sh
 #############################################################################
-# $Id: new_cmake_project.sh 611183 2020-06-30 16:28:14Z gouriano $
+# $Id: new_cmake_project.sh 674846 2023-11-06 19:12:27Z ivanov $
 #   Create new CMake project which uses prebuilt NCBI C++ toolkit from a sample
 #   Author: Andrei Gourianov, gouriano@ncbi
 #############################################################################
@@ -12,7 +12,9 @@ cd "$initial_dir"
 tree_root=`pwd`
 
 build_root=$NCBI
-build_dir=c++.cmake.stable
+build_pfx="c++.cmake."
+build_def="stable"
+build_dir="${build_pfx}${build_def}"
 
 repository="https://svn.ncbi.nlm.nih.gov/repos/toolkit/trunk/c++"
 rep_inc="include"
@@ -25,7 +27,7 @@ Usage()
 {
     cat <<EOF
 USAGE:
-  $script_name <name> <type> [builddir] 
+  $script_name <name> <type> [builddir] [OPTIONS] 
 SYNOPSIS:
   Create new CMake project which uses prebuilt NCBI C++ toolkit from a sample.
 ARGUMENTS:
@@ -33,15 +35,17 @@ ARGUMENTS:
   <name>       -- project name (destination directory)
   <type>       -- project type
   builddir     -- root directory of the pre-built NCBI C++ toolkit
-                  default: $build_dir
+                  default: $build_def
+OPTIONS:
+  --noconfig   -- skip configuring build
 EOF
 if test -z "$1"; then
   echo
   echo The following project types are available:
   svn list -R $repository/$rep_src/$rep_sample | while IFS= read -r line; do
-    if [[ $line =~ .*/$ ]]; then
+    case "$line" in */ )
       count=`echo $line | tr -cd '/' | wc -c`
-      if [[ $count -eq 2 ]]; then
+      if [ $count -eq 2 ]; then
         prj=${line%?}
         case "$prj" in 
         app/alnmgr)         echo "  app/alnmgr          an application that uses Alignment Manager";;
@@ -73,7 +77,8 @@ if test -z "$1"; then
         *)                  echo "  $prj";;
         esac 
       fi
-    fi
+      ;;
+    esac
   done
 fi
 echo " "
@@ -90,6 +95,27 @@ Error()
   exit 1
 }
 
+Get_build_dir()
+{
+  all=`ls $NCBI | grep ${build_pfx} | sed "s/${build_pfx}//g"`
+  while true; do
+    cat <<EOF
+
+Please pick a stability level
+Available:
+
+$all
+
+EOF
+    read -p "Desired stability (default = ${build_def}): " stability
+    test -n "$stability"  ||  stability=${build_def}
+    builddir=${build_root}/${build_pfx}${stability}
+    if [ -d ${builddir} ]; then
+      break
+    fi
+  done
+}
+
 #----------------------------------------------------------------------------
 if [ $# -eq 0 ]; then
   do_help="yes"
@@ -101,6 +127,9 @@ while [ $# -ne 0 ]; do
     --help|-help|help|-h)
       do_help="yes"
     ;; 
+    --noconfig)
+      noconfig="yes"
+    ;;
     *) 
       if [ $pos -eq 0 ]; then
         prj_name=$1
@@ -131,15 +160,18 @@ fi
 if [ -z "$prj_type" ]; then
   Error "Mandatory argument 'branch' is missing"
 fi
+if [ -e $prj_name ]; then
+  Error File or directory $prj_name already exists
+fi
 if [ -z "$toolkit" ]; then
-  builddir=${build_root}/${build_dir}
+  Get_build_dir
 else
   if [ -d "$toolkit" ]; then
     builddir="$toolkit"
   elif [ -d "${build_root}/$toolkit" ]; then
     builddir="${build_root}/$toolkit"
-  elif [ -d "${build_root}/c++.cmake.$toolkit" ]; then
-    builddir="${build_root}/c++.cmake.$toolkit"
+  elif [ -d "${build_root}/${build_pfx}$toolkit" ]; then
+    builddir="${build_root}/${build_pfx}$toolkit"
   fi
 fi
 if [ ! -d "$builddir" ]; then
@@ -150,13 +182,44 @@ if [ ! -d "$builddir" ]; then
     echo "ERROR:  Directory not found: $toolkit" 1>&2
   fi
   echo "Try one of these:"
-  ls $NCBI | grep c++.cmake
+  ls $NCBI | grep ${build_pfx} | sed "s/${build_pfx}//g"
   exit 1
 fi
 
-if [ -e $prj_name ]; then
-  Error File or directory $prj_name already exists
+prebuilt_path="?"
+prebuilds=""
+if [ ! "$noconfig" = "yes" ]; then
+    tmp=`ls ${builddir}`
+    if [ -n "$tmp" ]; then
+        for d in $tmp; do
+            if [ -f ${builddir}/$d/cmake/buildinfo ]; then
+                prebuilds="${prebuilds} $d"
+            fi
+	    done
+        for d in $prebuilds; do
+            build_def=$d
+            break
+        done
+        all=`echo $prebuilds | tr ' ' '\n'`
+        while true; do
+            cat <<EOF
+
+Please pick a build configuration
+Available:
+
+$all
+
+EOF
+            read -p "Desired configuration (default = ${build_def}): " build_config
+            test -n "$build_config"  ||  build_config=${build_def}
+            prebuilt_path=${build_config}
+            if [ -d ${builddir}/${prebuilt_path} ]; then
+                break
+            fi
+        done
+    fi
 fi
+
 mkdir -p $prj_name
 if [ ! -d $prj_name ]; then
   Error Failed to create directory $prj_name
@@ -201,4 +264,8 @@ chmod a+x $cfg_cfg
 echo "Created project $prj_name"
 echo "To configure:  cd $prj_name; ./$cfg_cfg <arguments>"
 echo "For help:      cd $prj_name; ./$cfg_cfg --help"
+if [ ! "$noconfig" = "yes" ]; then
+    echo ----------------------------------------------------------------------
+    ./$cfg_cfg --with-prebuilt=${prebuilt_path}
+fi
 cd "$initial_dir"
