@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 """Script to create the Windows installer for BLAST command line applications"""
-# $Id: make_win.py 620125 2020-11-16 22:02:18Z jianye $
+# $Id: make_win.py 667929 2023-05-23 18:26:13Z camacho $
 #
 # Author: Christiam camacho
 
@@ -43,6 +43,8 @@ def main():
     
     apps = [ "igblastn.exe", "igblastp.exe",
             "edit_imgt_file.pl",
+             "makeogrannote.py",
+             "makeogrdb.py",
              "ncbi-vdb-md.dll",
              "makeblastdb.exe",
              "blastdbcmd.exe"
@@ -55,7 +57,7 @@ def main():
             print("Copying", app, "to", cwd)
         shutil.copy(app, cwd)
     
-    dll = os.path.join('\\\\', 'snowman', 'win-coremake', 'Lib', 'ThirdParty', 'nghttp2', 'vs2017.64', '1.33.0', 'bin', 'ReleaseDLL', 'nghttp2.dll')
+    dll = '\\\\' + os.path.join('snowman', 'win-coremake', 'Lib', 'ThirdParty', 'nghttp2', 'vs2017.64', '1.33.0', 'bin', 'ReleaseDLL', 'nghttp2.dll')
     shutil.copy(dll, cwd)
     
     
