@@ -1,4 +1,4 @@
-/*  $Id: effsearchspace_calc.cpp 631494 2021-05-19 13:45:51Z ivanov $
+/*  $Id: effsearchspace_calc.cpp 683928 2024-06-10 12:07:11Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -133,7 +133,7 @@ Int8
 CEffectiveSearchSpaceCalculator::GetEffSearchSpace(size_t query_index) const
 {
     _ASSERT((Int4)query_index < m_QueryInfo->num_queries);
-    return BlastQueryInfoGetEffSearchSpace(m_QueryInfo, m_Program, query_index);
+    return BlastQueryInfoGetEffSearchSpace(m_QueryInfo, m_Program, static_cast<Int4>(query_index));
 }
 
 Int8
