@@ -1,4 +1,4 @@
-/*  $Id: remote_blast.cpp 631493 2021-05-19 13:45:43Z ivanov $ 
+/*  $Id: remote_blast.cpp 683928 2024-06-10 12:07:11Z ivanov $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -2373,12 +2373,12 @@ string CRemoteBlast::GetTitle(void)
 	    // Build the request
 	    CRef<CBlast4_request_body> request_body =
 	        s_BuildSearchInfoRequest(m_RID,
-                                     CTempString(kBlast4SearchInfoReqName_Search),
-	                                 CTempString(kBlast4SearchInfoReqValue_Title));
+                                     kBlast4SearchInfoReqName_Search,
+	                                 kBlast4SearchInfoReqValue_Title);
 	    CRef<CBlast4_reply> reply = x_SendRequest(request_body);
 	    return x_GetStringFromSearchInfoReply(reply,
-	                                          CTempString(kBlast4SearchInfoReqName_Search),
-	                                          CTempString(kBlast4SearchInfoReqValue_Title));
+	                                          kBlast4SearchInfoReqName_Search,
+	                                          kBlast4SearchInfoReqValue_Title);
 
 }
 // Disk Cache version: x_CheckResults
@@ -2507,7 +2507,7 @@ CRemoteBlast::x_GetSearchResultsHTTP(void)
 
     unique_ptr<fstream> tmp_stream( CDirEntry::CreateTmpFile() );
     do{
-		n_read = CStreamUtils::Readsome(ios,incoming_buffer, read_max);
+		n_read = static_cast<int>( CStreamUtils::Readsome(ios,incoming_buffer, read_max) );
 		if( n_read > 0 ){
 			l_total_bytes += n_read;
 			try{
@@ -2568,14 +2568,7 @@ void CRemoteBlast::x_GetSubjects(void)
         return; // already got data
 
     // Build the request
-    CRef<CBlast4_get_search_info_request> info_request( new CBlast4_get_search_info_request );
-    info_request->SetRequest_id( m_RID );
-    info_request->SetInfo().Add(CTempString(kBlast4SearchInfoReqName_Search), 
-                                CTempString(kBlast4SearchInfoReqValue_Subjects));
-
-    CRef<CBlast4_request_body> body(new CBlast4_request_body);
-    body->SetGet_search_info( *info_request );
-
+    CRef<CBlast4_request_body> body = s_BuildSearchInfoRequest(m_RID, kBlast4SearchInfoReqName_Search, kBlast4SearchInfoReqValue_Subjects);
     CRef<CBlast4_request> request(new CBlast4_request);
     request->SetBody(*body);
     
@@ -2616,8 +2609,8 @@ void CRemoteBlast::x_GetSubjects(void)
                 if( info_reply.CanGetInfo() ){
                     const CBlast4_parameters &params = info_reply.GetInfo();
                     string reply_name =
-                          Blast4SearchInfo_BuildReplyName(CTempString(kBlast4SearchInfoReqName_Search),
-                                                          CTempString(kBlast4SearchInfoReqValue_Subjects));
+                          Blast4SearchInfo_BuildReplyName(kBlast4SearchInfoReqName_Search,
+                                                          kBlast4SearchInfoReqValue_Subjects);
                     CRef< CBlast4_parameter > search_param = params.GetParamByName (reply_name);
                     // reply could have string, seq-loc-list or 
                     // bioseq-list, but we don't care about string result for bl2seq
@@ -2668,12 +2661,12 @@ unsigned int CRemoteBlast::x_GetPsiIterationsFromServer()
 
      CRef<CBlast4_request_body> request_body =
          s_BuildSearchInfoRequest(m_RID,
-                                  CTempString(kBlast4SearchInfoReqName_Search),
-                                  CTempString(kBlast4SearchInfoReqValue_PsiIterationNum));
+                                  kBlast4SearchInfoReqName_Search,
+                                  kBlast4SearchInfoReqValue_PsiIterationNum);
      CRef<CBlast4_reply> reply = x_SendRequest(request_body);
      string num = x_GetStringFromSearchInfoReply(reply,
-                                                 CTempString(kBlast4SearchInfoReqName_Search),
-                                                 CTempString(kBlast4SearchInfoReqValue_PsiIterationNum));
+                                                 kBlast4SearchInfoReqName_Search,
+                                                 kBlast4SearchInfoReqValue_PsiIterationNum);
       if ( !num.empty() ) {
          try { retval = NStr::StringToUInt(num); }
          catch (...) {}  // ignore errors and leave as unset
