@@ -1,4 +1,4 @@
-/*  $Id: blast_fasta_input.cpp 602525 2020-02-25 17:57:10Z jianye $
+/*  $Id: blast_fasta_input.cpp 683928 2024-06-10 12:07:11Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -588,18 +588,31 @@ CShortReadFastaInputSource::GetNextSequence(CBioseq_set& bioseq_set)
 }
 
 
+// Return a reference to the UserObject in SeqDescr labeled Mapping. Create one
+// if it does not exist.
+static CUser_object& s_SetSeqdescUser(CSeq_entry& entry)
+{
+    CRef<CSeqdesc> seqdesc;
+    // find user object labeled "Mapping" in Seq_entry
+    for (auto& it: entry.SetSeq().SetDescr().Set()) {
+        if (it->IsUser() && it->GetUser().GetType().GetStr() == "Mapping") {
+            seqdesc.Reset(it);
+            break;
+        }
+    }
+    // if not present create a new one
+    if (seqdesc.Empty()) {
+        seqdesc.Reset(new CSeqdesc());
+        seqdesc->SetUser().SetType().SetStr("Mapping");
+        entry.SetSeq().SetDescr().Set().push_back(seqdesc);
+    }
+    return seqdesc->SetUser();
+}
+
+
 void
 CShortReadFastaInputSource::x_ReadFastaOrFastq(CBioseq_set& bioseq_set)
 {
-    // tags to indicate paired sequences
-    CRef<CSeqdesc> seqdesc_first(new CSeqdesc);
-    seqdesc_first->SetUser().SetType().SetStr("Mapping");
-    seqdesc_first->SetUser().AddField("has_pair", eFirstSegment);
-
-    CRef<CSeqdesc> seqdesc_last(new CSeqdesc);
-    seqdesc_last->SetUser().SetType().SetStr("Mapping");
-    seqdesc_last->SetUser().AddField("has_pair", eLastSegment);
-
     CRef<CSeq_entry> first;
     CRef<CSeq_entry> second;
     switch (m_Format) {
@@ -636,14 +649,16 @@ CShortReadFastaInputSource::x_ReadFastaOrFastq(CBioseq_set& bioseq_set)
 
         if (first.NotEmpty()) {
             if (second.NotEmpty()) {
-                first->SetSeq().SetDescr().Set().push_back(seqdesc_first);
+                // tag to indicate paired sequences
+                s_SetSeqdescUser(*first).AddField("has_pair", eFirstSegment);
             }
             bioseq_set.SetSeq_set().push_back(first);
         }
 
         if (second.NotEmpty()) {
             if (first.NotEmpty()) {
-                second->SetSeq().SetDescr().Set().push_back(seqdesc_last);
+                // tag to indicate paired sequences
+                s_SetSeqdescUser(*second).AddField("has_pair", eLastSegment);
             }
             bioseq_set.SetSeq_set().push_back(second);
         }
@@ -692,7 +707,7 @@ CShortReadFastaInputSource::x_ReadFastc(CBioseq_set& bioseq_set)
     if (line[0] != '>') {
         NCBI_THROW(CInputException, eInvalidInput,
                    (string)"Missing defline before line: " +
-                   NStr::IntToString(m_LineReader->GetLineNumber()));
+                   NStr::NumericToString(m_LineReader->GetLineNumber()));
     }
 
     id = x_ParseDefline(line);
@@ -723,7 +738,7 @@ CShortReadFastaInputSource::x_ReadFastc(CBioseq_set& bioseq_set)
         NCBI_THROW(CInputException, eInvalidInput,
                    (string)"FASTC parse error: Sequence separator '><'"
                    " was not found in line: " +
-                   NStr::IntToString(m_LineReader->GetLineNumber()));
+                   NStr::NumericToString(m_LineReader->GetLineNumber()));
     }
 
     // set up reads, there are two sequences in the same line separated
@@ -749,7 +764,7 @@ CShortReadFastaInputSource::x_ReadFastc(CBioseq_set& bioseq_set)
             }
             bioseq.SetInst().SetMol(CSeq_inst::eMol_na);
             bioseq.SetInst().SetRepr(CSeq_inst::eRepr_raw);
-            bioseq.SetInst().SetLength(first_len);
+            bioseq.SetInst().SetLength(static_cast<CSeq_inst_Base::TLength>(first_len));
             first[first_len] = 0;
             bioseq.SetInst().SetSeq_data().SetIupacna(CIUPACna(first));
             bioseq.SetDescr().Set().push_back(seqdesc_first);
@@ -775,7 +790,7 @@ CShortReadFastaInputSource::x_ReadFastc(CBioseq_set& bioseq_set)
             }
             bioseq.SetInst().SetMol(CSeq_inst::eMol_na);
             bioseq.SetInst().SetRepr(CSeq_inst::eRepr_raw);
-            bioseq.SetInst().SetLength(second_len);
+            bioseq.SetInst().SetLength(static_cast<CSeq_inst_Base::TLength>(second_len));
             second[second_len] = 0;
             bioseq.SetInst().SetSeq_data().SetIupacna(CIUPACna(second));
             bioseq.SetDescr().Set().push_back(seqdesc_last);
@@ -810,7 +825,7 @@ CShortReadFastaInputSource::x_ReadFastaOneSeq(CRef<ILineReader> line_reader)
         // increase the sequence buffer if necessary
         if (start + line.length() + 1 > m_SeqBuffLen) {
             string tmp;
-            m_SeqBuffLen = 2 * (start + line.length() + 1);
+            m_SeqBuffLen = static_cast<TSeqPos>(2 * (start + line.length() + 1));
             tmp.resize(m_SeqBuffLen);
             memcpy(&tmp[0], &m_Sequence[0], start);
             m_Sequence.swap(tmp);
@@ -879,7 +894,7 @@ CShortReadFastaInputSource::x_ReadFastqOneSeq(CRef<ILineReader> line_reader)
     if (line[0] != '@') {
         NCBI_THROW(CInputException, eInvalidInput, (string)"FASTQ parse error:"
                    " defline expected at line: " +
-                   NStr::IntToString(line_reader->GetLineNumber()));
+                   NStr::NumericToString(line_reader->GetLineNumber()));
     }
 
     defline_id = x_ParseDefline(line);
@@ -920,7 +935,7 @@ CShortReadFastaInputSource::x_ReadFastqOneSeq(CRef<ILineReader> line_reader)
             empty_sequence = true;
         }
         else {
-            bioseq.SetInst().SetLength(line.length());
+            bioseq.SetInst().SetLength(static_cast<CSeq_inst_Base::TLength>(line.length()));
             bioseq.SetInst().SetSeq_data().SetIupacna(CIUPACna(line.data()));
             m_BasesAdded += line.length();
         }
@@ -942,13 +957,19 @@ CShortReadFastaInputSource::x_ReadFastqOneSeq(CRef<ILineReader> line_reader)
     if (line[0] != '+') {
         NCBI_THROW(CInputException, eInvalidInput, (string)"FASTQ parse error:"
                    " defline expected at line: " +
-                   NStr::IntToString(line_reader->GetLineNumber()));
+                   NStr::NumericToString(line_reader->GetLineNumber()));
     }
 
     if (!empty_sequence) {
-        // read and skip quality scores
+        // read and quality scores
         ++(*line_reader);
         line = **line_reader;
+
+        if (!line.empty()) {
+            // store quality string
+            s_SetSeqdescUser(*retval).AddField("quality", line.data());
+        }
+
         // skip empty lines
         while (!line_reader->AtEOF() && line.empty()) {
             ++(*line_reader);
@@ -969,15 +990,6 @@ CShortReadFastaInputSource::x_ReadFromTwoFiles(CBioseq_set& bioseq_set,
                    "used with two files");
     }
 
-    // tags to indicate paired sequences
-    CRef<CSeqdesc> seqdesc_first(new CSeqdesc);
-    seqdesc_first->SetUser().SetType().SetStr("Mapping");
-    seqdesc_first->SetUser().AddField("has_pair", eFirstSegment);
-
-    CRef<CSeqdesc> seqdesc_last(new CSeqdesc);
-    seqdesc_last->SetUser().SetType().SetStr("Mapping");
-    seqdesc_last->SetUser().AddField("has_pair", eLastSegment);
-
     CRef<CSeq_entry> first;
     CRef<CSeq_entry> second;
 
@@ -992,14 +1004,15 @@ CShortReadFastaInputSource::x_ReadFromTwoFiles(CBioseq_set& bioseq_set,
 
     if (first.NotEmpty()) {
         if (second.NotEmpty()) {
-            first->SetSeq().SetDescr().Set().push_back(seqdesc_first);
+            s_SetSeqdescUser(*first).AddField("has_pair", eFirstSegment);
         }
         bioseq_set.SetSeq_set().push_back(first);
     }
 
     if (second.NotEmpty()) {
         if (first.NotEmpty()) {
-            second->SetSeq().SetDescr().Set().push_back(seqdesc_last);
+            s_SetSeqdescUser(*second).AddField("has_pair", eLastSegment);
+
         }
         bioseq_set.SetSeq_set().push_back(second);
     }
