@@ -1,4 +1,4 @@
-/* $Id: blast_aascan.c 500404 2016-05-04 14:59:01Z camacho $
+/* $Id: blast_aascan.c 683930 2024-06-10 12:07:59Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -107,7 +107,7 @@ static Int4 s_BlastAaScanSubject(const LookupTableWrap * lookup_wrap,
                 /* copy the hits. */
                 {
                     Int4 i;
-                    Int4 s_off = s - subject->sequence;
+                    Int4 s_off = (Int4)(s - subject->sequence);
                     for (i = 0; i < numhits; i++) {
                         offset_pairs[i + totalhits].qs_offsets.q_off = src[i];
                         offset_pairs[i + totalhits].qs_offsets.s_off = s_off;
@@ -118,12 +118,12 @@ static Int4 s_BlastAaScanSubject(const LookupTableWrap * lookup_wrap,
             } else
                 /* not enough space in the destination array; return early */
             {
-                s_range[1] = s - subject->sequence;
+                s_range[1] = (Int4)(s - subject->sequence);
                 return totalhits;
             }
         }
     } /* end for */
-    s_range[1] = s - subject->sequence;
+    s_range[1] = (Int4)(s - subject->sequence);
     } /* end while */
 
     /* if we get here, we fell off the end of the sequence */
@@ -193,7 +193,7 @@ static Int4 s_BlastSmallAaScanSubject(const LookupTableWrap * lookup_wrap,
                 /* copy the hits. */
                 {
                     Int4 i;
-                    Int4 s_off = s - subject->sequence;
+                    Int4 s_off = (Int4)(s - subject->sequence);
                     for (i = 0; i < numhits; i++) {
                         offset_pairs[i + totalhits].qs_offsets.q_off = src[i];
                         offset_pairs[i + totalhits].qs_offsets.s_off = s_off;
@@ -204,12 +204,12 @@ static Int4 s_BlastSmallAaScanSubject(const LookupTableWrap * lookup_wrap,
             } else
                 /* not enough space in the destination array; return early */
             {
-                s_range[1] = s - subject->sequence;
+                s_range[1] = (Int4)(s - subject->sequence);
                 return totalhits;
             }
         }
     } /* end for */
-    s_range[1] = s - subject->sequence;
+    s_range[1] = (Int4)(s - subject->sequence);
 
     } /* end while */
     /* if we get here, we fell off the end of the sequence */
@@ -329,7 +329,7 @@ static Int4 s_BlastCompressedAaScanSubject(
 
        /* if there are hits */
        if (PV_TEST(pv, index, pv_array_bts)) {
-          Int4 s_off = s - subject->sequence;
+          Int4 s_off = (Int4)(s - subject->sequence);
 
           CompressedLookupBackboneCell* backbone_cell = 
                                         lookup->backbone + index;
@@ -337,20 +337,22 @@ static Int4 s_BlastCompressedAaScanSubject(
           numhits = backbone_cell->num_used;
          
           /* and there is enough space in the destination array */
+          if (numhits) {
           if (numhits <= (array_size - totalhits)) {
            
              /* copy the hits to the destination */
            
              Int4 i;
-             Int4 *query_offsets;
              BlastOffsetPair *dest = offset_pairs + totalhits;
-           
-             if (numhits <= COMPRESSED_HITS_PER_BACKBONE_CELL) {
+
+             dest[0].qs_offsets.q_off = backbone_cell->query_offset;
+             dest[0].qs_offsets.s_off = s_off;
+
+             if (numhits <= COMPRESSED_HITS_PER_BACKBONE_CELL+1) {
+            	 dest ++;
                 /* hits all live in the backbone */
-             
-                query_offsets = backbone_cell->payload.query_offsets;
-                for (i = 0; i < numhits; i++) {
-                   dest[i].qs_offsets.q_off = query_offsets[i];
+                for (i = 0; i < numhits-1; i++) {
+                   dest[i].qs_offsets.q_off = backbone_cell->payload.query_offsets[i];
                    dest[i].qs_offsets.s_off = s_off;
                 }
              } 
@@ -358,50 +360,39 @@ static Int4 s_BlastCompressedAaScanSubject(
                 /* hits are in the backbone cell and in the overflow list */
                 CompressedOverflowCell* curr_cell = 
                                     backbone_cell->payload.overflow_list.head;
-                /* we know the overflow list has at least one cell,
-                   so it's safe to speculatively fetch the pointer
-                   to further cells */
-                CompressedOverflowCell* next_cell = curr_cell->next;
 
                 /* the number of hits in the linked list of cells has
                    1 added to it; the extra hit was spilled from the
                    backbone when the list was first created */
-                Int4 first_cell_entries = (numhits -
-                                     COMPRESSED_HITS_PER_BACKBONE_CELL) %
-                                     COMPRESSED_HITS_PER_OVERFLOW_CELL + 1;
+                Int4 first_cell_entries = (numhits - 3) & COMPRESSED_HITS_CELL_MASK;
 
                 /* copy hits from backbone */
-                query_offsets = 
-                         backbone_cell->payload.overflow_list.query_offsets;
-                for(i = 0; i < COMPRESSED_HITS_PER_BACKBONE_CELL - 1; i++) {
-                   dest[i].qs_offsets.q_off = query_offsets[i];
-                   dest[i].qs_offsets.s_off = s_off;
-                }
+                dest[1].qs_offsets.q_off = backbone_cell->payload.overflow_list.query_offsets[0];
+                dest[1].qs_offsets.s_off = s_off;
+                dest[2].qs_offsets.q_off = backbone_cell->payload.overflow_list.query_offsets[1];
+                dest[2].qs_offsets.s_off = s_off;
               
                 /* handle the overflow list */
               
                 /* first cell can be partially filled */
-                query_offsets = curr_cell->query_offsets;
-                dest += i;
+                dest += 3;
                 for (i = 0; i < first_cell_entries; i++) {
-                   dest[i].qs_offsets.q_off = query_offsets[i];
+                   dest[i].qs_offsets.q_off =  curr_cell->query_offsets[i];
                    dest[i].qs_offsets.s_off = s_off;
                 }
 
                 /* handle the rest of the list */
-
-                if (next_cell != NULL) {
-                   curr_cell = next_cell;
+                if (first_cell_entries) {
+                   curr_cell = curr_cell->next;
+                }
                    while (curr_cell != NULL) {
-                      query_offsets = curr_cell->query_offsets;
-                      curr_cell = curr_cell->next;    /* prefetch */
                       dest += i;
                       for (i = 0; i < COMPRESSED_HITS_PER_OVERFLOW_CELL; i++) {
-                         dest[i].qs_offsets.q_off = query_offsets[i];
+                         dest[i].qs_offsets.q_off =  curr_cell->query_offsets[i];
                          dest[i].qs_offsets.s_off = s_off;
                       }
+                      curr_cell = curr_cell->next;    /* prefetch */
                    }
-                }
              }
 
              totalhits += numhits;
@@ -409,12 +400,13 @@ static Int4 s_BlastCompressedAaScanSubject(
           else
               /* not enough space in the destination array */
           {
-              s_range[1] = s - subject->sequence;
+              s_range[1] = (Int4)(s - subject->sequence);
               return totalhits;
           }
+          }
        }
     } /* end for */
-    s_range[1] = s - subject->sequence;
+    s_range[1] = (Int4)(s - subject->sequence);
     } /* end while */
 
     /* if we get here, we fell off the end of the sequence */
@@ -520,7 +512,7 @@ Int4 BlastRPSScanSubject(const LookupTableWrap * lookup_wrap,
                 Int4 *src;
                 Int4 i;
                 Uint4 q_off;
-                Uint4 s_off = s - abs_start;
+                Uint4 s_off = (Uint4)(s - abs_start);
                 if (numhits <= RPS_HITS_PER_CELL) {
                     for (i = 0; i < numhits; i++) {
                         q_off = cell->entries[i] - table_correction;
@@ -553,7 +545,7 @@ Int4 BlastRPSScanSubject(const LookupTableWrap * lookup_wrap,
     }
 
     /* if we get here, we fell off the end of the sequence */
-    *offset = s - abs_start;
+    *offset = (Int4)(s - abs_start);
 
     return totalhits;
 }
