@@ -1,4 +1,4 @@
-/* $Id: blast_engine.c 617226 2020-09-28 18:25:19Z ivanov $
+/* $Id: blast_engine.c 683952 2024-06-10 12:29:05Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -79,9 +79,8 @@
 #define CONV_NUCL2PROT_COORDINATES(length) (length) / CODON_LENGTH
 
 NCBI_XBLAST_EXPORT const int   kBlastMajorVersion = 2;
-NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 11;
+NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 16;
 NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 0;
-NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "Oct-15-2020";
 
 /** Structure to be passed to s_BlastSearchEngineCore, containing pointers
     to various preallocated structures and arrays. */
@@ -615,7 +614,7 @@ s_BlastSearchEngineCoreCleanUp(EBlastProgramType program_number,
         BlastQueryInfo* query_info,
         const BlastQueryInfo* query_info_in,
         Uint1* translation_buffer,
-        Int4* frame_offsets_a)
+        Uint4* frame_offsets_a)
 {
     /* Free the local query info structure when needed (in RPS BLAST). */
     if (query_info != query_info_in)
@@ -719,8 +718,8 @@ s_BlastSearchEngineCore(EBlastProgramType program_number,
 {
     BlastHSPList* hsp_list_out=NULL;
     Uint1* translation_buffer = NULL;
-    Int4* frame_offsets   = NULL;
-    Int4* frame_offsets_a = NULL; /* Will be freed if non-null */
+    Uint4* frame_offsets   = NULL;
+    Uint4* frame_offsets_a = NULL; /* Will be freed if non-null */
     BlastHitSavingOptions* hit_options = hit_params->options;
     BlastScoringOptions* score_options = score_params->options;
     Int2 status = 0;
@@ -768,7 +767,7 @@ s_BlastSearchEngineCore(EBlastProgramType program_number,
             /* For RPS tblastn, subject is actually query, which has already
                been translated during the setup stage. */
             translation_buffer = backup.sequence - 1;
-            frame_offsets_a = frame_offsets = ContextOffsetsToOffsetArray(query_info_in);
+            frame_offsets_a = frame_offsets = (Uint4*)ContextOffsetsToOffsetArray(query_info_in);
         } else {
             BLAST_GetAllTranslations(backup.sequence, eBlastEncodingNcbi2na,
                                      backup.full_range.right,
