@@ -1,4 +1,4 @@
-/* $Id: blast_traceback.c 604796 2020-04-02 12:32:43Z fongah2 $
+/* $Id: blast_traceback.c 683930 2024-06-10 12:07:59Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1516,7 +1516,8 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
         }
 
 #pragma omp parallel for default(none) num_threads(actual_num_threads) schedule(guided) if (actual_num_threads > 1) \
-        shared(retval, thread_data, batches, score_params, program_number, sbp, hit_params, pattern_blk, query, ext_params, query_info, default_db_genetic_code, has_been_interrupted, interrupt_search, progress_info)
+        shared(retval, thread_data, batches, score_params, program_number, sbp, hit_params, pattern_blk, query, \
+        	   ext_params, query_info, default_db_genetic_code, has_been_interrupted, interrupt_search, progress_info, actual_num_threads)
         for (i = 0; i < batches->num_batches; i++) {
             BlastSeqSrcGetSeqArg seq_arg = {0,0,0,0,NULL,NULL};
             Int4 hsplist_itr = 0;
@@ -1537,7 +1538,8 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
             perform_partial_fetch = BlastSeqSrcGetSupportsPartialFetching(seqsrc);
 
             /* check for interrupt */
-            if (interrupt_search && (*interrupt_search)(progress_info) == TRUE) {
+            if ((interrupt_search && (*interrupt_search)(progress_info) == TRUE)  &&
+            	(actual_num_threads > 1)){
                 batches->array_of_batches[i] = Blast_HSPStreamResultBatchReset(batch);
 #pragma omp critical(retval)
                 {
@@ -1570,6 +1572,7 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
 
                 if (BlastSeqSrcGetSequence(seqsrc, &seq_arg) < 0) {
                     batches->array_of_batches[i] = Blast_HSPStreamResultBatchReset(batch);
+                    seq_arg.ranges = BlastSeqSrcSetRangesArgFree(ranges);
                     continue;
                 }
 
@@ -1578,14 +1581,18 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
                 * code for all subjects (as in the C toolkit) */
                 if (Blast_SubjectIsTranslated(program_number) &&
                     seq_arg.seq->gen_code_string == NULL) {
+                	if(actual_num_threads > 1) {
 #pragma omp critical(tback_gen_code)
-                    {
                         seq_arg.seq->gen_code_string =
                             GenCodeSingletonFind(default_db_genetic_code);
 #ifndef _OPENMP
                         ASSERT(seq_arg.seq->gen_code_string);
 #endif
                     }
+                	else {
+                        seq_arg.seq->gen_code_string =
+                            GenCodeSingletonFind(default_db_genetic_code);
+                	}
                 }
 
                 if (BlastSeqSrcGetTotLen(seqsrc) == 0) {
@@ -1608,8 +1615,12 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
                                 seq_arg.seq->length, score_params->options,
                                 qi, sbp, hp, NULL, elp)) != 0) {
                         batches->array_of_batches[i] = Blast_HSPStreamResultBatchReset(batch);
+                        if (actual_num_threads >1) {
 #pragma omp critical(retval)
-                        {
+                            retval = status;
+                            has_been_interrupted = TRUE;
+                        }
+                        else {
                             retval = status;
                             has_been_interrupted = TRUE;
                         }
@@ -1648,14 +1659,18 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
                             /* The C toolkit will erase genetic_code, so do it again */
                             if (Blast_SubjectIsTranslated(program_number) &&
                                 seq_arg.seq->gen_code_string == NULL) {
+                            	if (actual_num_threads > 1) {
 #pragma omp critical(tback_gen_code)
-                                {
                                     seq_arg.seq->gen_code_string =
                                         GenCodeSingletonFind(default_db_genetic_code);
 #ifndef _OPENMP
                                     ASSERT(seq_arg.seq->gen_code_string);
 #endif
                                 }
+                            	else {
+                                    seq_arg.seq->gen_code_string =
+                                        GenCodeSingletonFind(default_db_genetic_code);
+                            	}
                             }
 
                             /* Retry the alignment with fence_hit set*/
@@ -1745,16 +1760,17 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
     }
     // -RMH-: end of change
 
+    if(hit_params->options->query_cov_hsp_perc > 0 || hit_params->options->max_hsps_per_subject > 0 ||
+       (hit_params->options->hsp_filt_opt != NULL && hit_params->options->hsp_filt_opt->subject_besthit_opts != NULL)) {
+    	s_FilterBlastResults(results, hit_params->options, query_info, program_number);
+    }
+
     /* Re-sort the hit lists according to their best e-values, because they
        could have changed. Only do this for a database search. */
     if (BlastSeqSrcGetTotLen(seq_src) > 0) {
         Blast_HSPResultsSortByEvalue(results);
     }
 
-    if(hit_params->options->query_cov_hsp_perc > 0 || hit_params->options->max_hsps_per_subject > 0 ||
-       (hit_params->options->hsp_filt_opt != NULL && hit_params->options->hsp_filt_opt->subject_besthit_opts != NULL)) {
-    	s_FilterBlastResults(results, hit_params->options, query_info, program_number);
-    }
 
     /* Eliminate extra hits from results, if preliminary hit list size is
        larger than the final hit list size */
@@ -1803,7 +1819,7 @@ Blast_RunTracebackSearchWithInterrupt(EBlastProgramType program,
                                       TInterruptFnPtr interrupt_search,  SBlastProgress* progress_info,
                                       size_t num_threads)
 {
-    const int N_T = ((num_threads == 0) ? 1 : num_threads);
+    const int N_T = ((num_threads == 0) ? 1 : (int)num_threads);
     Int2 status = 0;
     SThreadLocalDataArray* thread_data = SThreadLocalDataArrayNew(N_T);
     char * ADAPTIVE_CBS_ENV = getenv("ADAPTIVE_CBS");
