@@ -133,7 +133,7 @@ CBlastFormat::CBlastFormat(const blast::CBlastOptions& options,
         {
                 BlastSeqSrc* seqsrc = db_adapter.MakeSeqSrc();
                 num_seqs=BlastSeqSrcGetNumSeqs(seqsrc);
-                total_length=BlastSeqSrcGetTotLen(seqsrc);
+                total_length=static_cast<int>(BlastSeqSrcGetTotLen(seqsrc));
         }
 	CBlastFormatUtil::FillScanModeBlastDbInfo(m_DbInfo, m_DbIsAA,
                 num_seqs, total_length, m_SubjectTag);
@@ -837,7 +837,8 @@ static void s_SetCloneInfo(const CIgBlastTabularInfo& tabinfo,
         wid->GetLabel(&seqid, CSeq_id::eContent);
         clone_info.seqid = seqid.substr(0, 45);
     }
-    tabinfo.GetIgInfo (clone_info.v_gene, clone_info.d_gene, clone_info.j_gene,
+    tabinfo.GetIgInfo (clone_info.v_gene, clone_info.d_gene, clone_info.j_gene, 
+                       clone_info.c_gene,
                        clone_info.chain_type, clone_info.na, clone_info.aa, clone_info.productive);
     clone_info.identity = 0;
     const vector<CIgBlastTabularInfo::SIgDomain*>& domains = tabinfo.GetIgDomains();
@@ -931,7 +932,7 @@ CBlastFormat::x_PrintIgTabularReport(const blast::CIgBlastResults& results,
         if (fill_clone_info) {
             s_SetCloneInfo(tabinfo, bhandle, clone_info);
         }
-        tabinfo.PrintHeader(strProgVersion, *(bhandle.GetBioseqCore()),
+        tabinfo.PrintHeader(m_IgOptions, strProgVersion, *(bhandle.GetBioseqCore()),
                             m_DbName, 
                             m_IgOptions->m_DomainSystem,
                             results.GetRID(), 
@@ -947,7 +948,7 @@ CBlastFormat::x_PrintIgTabularReport(const blast::CIgBlastResults& results,
             tabinfo.Print();
         }
     } else {
-        tabinfo.PrintHeader(strProgVersion, *(bhandle.GetBioseqCore()),
+        tabinfo.PrintHeader(m_IgOptions, strProgVersion, *(bhandle.GetBioseqCore()),
                                 m_DbName, 
                                 m_IgOptions->m_DomainSystem,
                                 results.GetRID(), 
@@ -974,10 +975,8 @@ void CBlastFormat::x_PrintAirrRearrangement(const blast::CIgBlastResults& result
     // tabular formatting just prints each alignment in turn
     // (plus a header)
 
-    const CBlastTabularInfo::EFieldDelimiter kDelim =
-            (m_FormatType == CFormattingArgs::eCommaSeparatedValues
-             ? CBlastTabularInfo::eComma : CBlastTabularInfo::eTab);
-
+    const CBlastTabularInfo::EFieldDelimiter kDelim = CBlastTabularInfo::eTab;
+    
     CIgBlastTabularInfo tabinfo(m_Outfile, m_CustomOutputFormatSpec, kDelim);
     tabinfo.SetParseLocalIds(m_BelieveQuery);
 
@@ -1075,7 +1074,7 @@ CBlastFormat::WriteArchive(blast::IQueryFactory& queries,
         archive = BlastBuildArchive(queries, options_handle, results, subjects);
         
     }
-    else
+    else if (!m_SearchDb.Empty())
     {
     	// Use only by psi blast
     	if(num_iters != 0) {
@@ -1085,6 +1084,19 @@ CBlastFormat::WriteArchive(blast::IQueryFactory& queries,
     		archive = BlastBuildArchive(queries, options_handle, results, m_SearchDb );
     	}
     }
+    else
+    {
+    	if(m_DbInfo.empty()) {
+    		NCBI_THROW(CException, eUnknown, "Subject or DB info not available");
+    	}
+    	string db_list = kEmptyStr;
+    	CSearchDatabase::EMoleculeType mol_type = m_DbInfo[0].is_protein ? CSearchDatabase::eBlastDbIsProtein : CSearchDatabase::eBlastDbIsNucleotide;
+    	for (unsigned int i=0; i < m_DbInfo.size(); i++) {
+    		db_list += m_DbInfo[i].name;
+    	}
+    	CRef<CSearchDatabase>  sdb (new CSearchDatabase(db_list, mol_type));
+    	archive = BlastBuildArchive(queries, options_handle, results, sdb);
+    }
 
     if(msg.size() > 0) {
     	archive->SetMessages() = msg;
@@ -1686,7 +1698,8 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
     // print 1-line summaries
     if ( !m_IsBl2Seq ) {
         CPsiBlastIterationState::TSeqIds prev_ids = CPsiBlastIterationState::TSeqIds();
-        int additional =  results.m_NumActualV +results.m_NumActualD + results.m_NumActualJ;
+        int additional =  results.m_NumActualV +results.m_NumActualD + results.m_NumActualJ +
+            results.m_NumActualC;
         x_DisplayDeflines(aln_set, numeric_limits<unsigned int>::max(), prev_ids, additional, index, 100);
     }
 
@@ -1715,9 +1728,9 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
         }
         m_Outfile << "Domain classification requested: " << m_IgOptions->m_DomainSystem << endl << endl;
         if (m_IsHTML) {
-            tabinfo.PrintHtmlSummary();
+            tabinfo.PrintHtmlSummary(m_IgOptions);
         } else {
-            tabinfo.PrintMasterAlign("");
+            tabinfo.PrintMasterAlign(m_IgOptions, "");
         }
     }
 
@@ -1741,9 +1754,9 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
 
     list < CRef<CDisplaySeqalign::DomainInfo> >  domain;
    
-    string kabat_domain_name[] = {"FR1", "CDR1", "FR2", "CDR2", "FR3", "CDR3", "FR4"};
-    string imgt_domain_name[] = {"FR1-IMGT", "CDR1-IMGT", "FR2-IMGT", "CDR2-IMGT", "FR3-IMGT", "CDR3-IMGT", "FR4-IMGT"};
-    int domain_name_length = 7;
+    string kabat_domain_name[] = {"FR1", "CDR1", "FR2", "CDR2", "FR3", "CDR3", "FR4", "C region"};
+    string imgt_domain_name[] = {"FR1-IMGT", "CDR1-IMGT", "FR2-IMGT", "CDR2-IMGT", "FR3-IMGT", "CDR3-IMGT", "FR4-IMGT", "C region"};
+    int domain_name_length = 8;
     vector<string> domain_name;
     if (m_IgOptions->m_DomainSystem == "kabat") {
         for (int i = 0; i < domain_name_length; i ++) {
@@ -1821,9 +1834,33 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
         domain.push_back(temp); 
     }
 
+    //C region
+
+    if (annots->m_CDomain[0] > 0 && annots->m_CDomain[1] > 0 && 
+        annots->m_JDomain[2] > 0 && annots->m_JDomain[3] > 0){      
+        CRef<CDisplaySeqalign::DomainInfo> temp(new CDisplaySeqalign::DomainInfo);
+        int start = annots->m_CDomain[0];
+        int subject_start = -1;
+        int stop = annots->m_CDomain[1];
+        int subject_stop = -1;
+
+        temp->seqloc = new CSeq_loc((CSeq_loc::TId &) aln_set->Get().front()->GetSeq_id(0),
+                                    (CSeq_loc::TPoint) start,
+                                    (CSeq_loc::TPoint) stop);
+        CRef<CSeq_id> id_holder (new CSeq_id);
+        temp->subject_seqloc = new CSeq_loc(*id_holder,
+                                            (CSeq_loc::TPoint) subject_start,
+                                            (CSeq_loc::TPoint) subject_stop);
+        temp->is_subject_start_valid = subject_start > 0 ? true:false;
+        temp->is_subject_stop_valid = subject_stop > 0 ? true:false;
+        temp->domain_name = domain_name[7];
+        domain.push_back(temp); 
+    }
+
+
     CDisplaySeqalign display(*aln_set, *m_Scope, &masklocs, NULL,  m_MatrixName);
     int num_align_to_show = results.m_NumActualV + results.m_NumActualD + 
-        results.m_NumActualJ;
+        results.m_NumActualJ + results.m_NumActualC;
     if (m_DbName != m_IgOptions->m_Db[0]->GetDatabaseName()){
         num_align_to_show += m_NumAlignments;
     }
@@ -2244,11 +2281,7 @@ void CBlastFormat::ResetScopeHistory()
 {
     // Do not reset the scope for BLAST2Sequences or else we'll loose the
     // sequence data! (see x_CreateSubjectBioseq)
-    if ((m_IsBl2Seq)
-    	|| (m_FormatType == CFormattingArgs::eXml2)
-    	|| (m_FormatType == CFormattingArgs::eJson)
-    	|| (m_FormatType == CFormattingArgs::eXml2_S)
-    	|| (m_FormatType == CFormattingArgs::eJson_S)){
+    if (m_IsBl2Seq){
         return;
     }
 
@@ -2433,8 +2466,6 @@ void CBlastFormat::LogBlastSearchInfo(CBlastUsageReport & report)
 	if (report.IsEnabled()) {
 		report.AddParam(CBlastUsageReport::eProgram, m_Program);
 		EProgram task = m_Options->GetProgram();
-		string task_str =  EProgramToTaskName(task);
-		report.AddParam(CBlastUsageReport::eTask, task_str);
 		report.AddParam(CBlastUsageReport::eEvalueThreshold, m_Options->GetEvalueThreshold());
 		report.AddParam(CBlastUsageReport::eHitListSize, m_Options->GetHitlistSize());
 		report.AddParam(CBlastUsageReport::eOutputFmt, m_FormatType);
@@ -2457,7 +2488,7 @@ void CBlastFormat::LogBlastSearchInfo(CBlastUsageReport & report)
 				report.AddParam(CBlastUsageReport::eNumSubjects, (int) m_SeqInfoSrc->Size());
 				int total_subj_length = 0;
 				for (size_t i = 0; i < m_SeqInfoSrc->Size(); i++) {
-				       total_subj_length += m_SeqInfoSrc->GetLength(i);
+				       total_subj_length += (int)m_SeqInfoSrc->GetLength(static_cast<Uint4>(i));
 				}
 				report.AddParam(CBlastUsageReport::eSubjectsLength, total_subj_length);
 			}
@@ -2469,6 +2500,11 @@ void CBlastFormat::LogBlastSearchInfo(CBlastUsageReport & report)
 			if (dir != kEmptyStr) {
 				db_name = m_DbName.substr(dir.length());
 			}
+
+			if (db_name.size() > 500) {
+				db_name.resize(500);
+				NStr::TruncateSpacesInPlace(db_name, NStr::eTrunc_End);
+			}
 			report.AddParam(CBlastUsageReport::eDBName, db_name);
 			report.AddParam(CBlastUsageReport::eDBLength, GetDbTotalLength());
 			report.AddParam(CBlastUsageReport::eDBNumSeqs, num_seqs);
