@@ -1,4 +1,4 @@
-/*  $Id: ntlookup_unit_test.cpp 561271 2018-04-04 21:19:19Z boratyng $
+/*  $Id: ntlookup_unit_test.cpp 621886 2020-12-17 15:27:31Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -96,7 +96,7 @@ struct NtlookupTestFixture {
         sprintf(buf, "gi|%d", query_gi);
         CSeq_id id(buf);
         
-        auto_ptr<SSeqLoc> ssl(CTestObjMgr::Instance().CreateSSeqLoc(
+        unique_ptr<SSeqLoc> ssl(CTestObjMgr::Instance().CreateSSeqLoc(
                                                    id, eNa_strand_both));
     
         SBlastSequence sequence(
