@@ -1,4 +1,4 @@
-/*  $Id: querydata_unit_test.cpp 631031 2021-05-12 14:09:49Z ivanov $
+/*  $Id: querydata_unit_test.cpp 641170 2021-11-26 14:52:37Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -517,7 +517,7 @@ struct CQueryDataTestFixture
         TSeqLocVector queries;
         ITERATE(vector<TGi>, itr, gis) {
             CSeq_id qid(CSeq_id::e_Gi, *itr);
-            auto_ptr<SSeqLoc> sl(CTestObjMgr::Instance().CreateSSeqLoc(qid));
+            unique_ptr<SSeqLoc> sl(CTestObjMgr::Instance().CreateSSeqLoc(qid));
             queries.push_back(*sl);
         }
 
@@ -628,17 +628,17 @@ BOOST_AUTO_TEST_CASE(ObjMgr_QueryFactory_LocalDataFromBlastQueryVector_Nucleotid
 }
 
 BOOST_AUTO_TEST_CASE(ObjMgr_QueryFactory_RemoteData_SingleBioseqFromTSeqLocVector) {
-    const int kGi = 129295;
+    TGi kGi = GI_CONST(129295);
     TSeqLocVector queries;
     CSeq_id qid(CSeq_id::e_Gi, kGi);
-    auto_ptr<SSeqLoc> sl(CTestObjMgr::Instance().CreateSSeqLoc(qid));
+    unique_ptr<SSeqLoc> sl(CTestObjMgr::Instance().CreateSSeqLoc(qid));
     queries.push_back(*sl);
     CRef<IQueryFactory> query_factory(new CObjMgr_QueryFactory(queries));
-    CSequenceDataTester(query_factory, GI_FROM(int, kGi))();
+    CSequenceDataTester(query_factory, kGi)();
 }
 
 BOOST_AUTO_TEST_CASE(ObjMgr_QueryFactory_RemoteData_SingleBioseqFromBlastQueryVector) {
-    const TIntId kGi = 129295;
+    TGi kGi = GI_CONST(129295);
 
     CRef<CBlastQueryVector> queries(new CBlastQueryVector);
     CSeq_id qid(CSeq_id::e_Gi, kGi);
@@ -649,7 +649,7 @@ BOOST_AUTO_TEST_CASE(ObjMgr_QueryFactory_RemoteData_SingleBioseqFromBlastQueryVe
     queries->AddQuery(sq);
     
     CRef<IQueryFactory> query_factory(new CObjMgr_QueryFactory(*queries));
-    CSequenceDataTester(query_factory, GI_FROM(int, kGi))();
+    CSequenceDataTester(query_factory, kGi)();
 }
 
 BOOST_AUTO_TEST_CASE(ObjMgr_QueryFactory_EmptyTSeqLocVector) {
