@@ -1,4 +1,4 @@
-/*  $Id: remote_blast_unit_test.cpp 600218 2020-01-14 17:23:41Z fongah2 $
+/*  $Id: remote_blast_unit_test.cpp 621871 2020-12-17 15:27:09Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -105,7 +105,7 @@ string x_Stringify(TOBJ & obj)
 {
     CNcbiOstrstream oss;
     
-    auto_ptr<CObjectOStream>
+    unique_ptr<CObjectOStream>
         outpstr(CObjectOStream::Open(eSerial_AsnText, oss));
     
     *outpstr << obj;
@@ -1024,7 +1024,7 @@ BOOST_AUTO_TEST_CASE(CheckDuplicateOptions)
 // RequiredEnd and RequiredStart fields are sent over the network
 BOOST_AUTO_TEST_CASE(GetSearchStrategy_FullQuery) {
     CRef<CSeq_id> id(new CSeq_id(CSeq_id::e_Gi, 555));
-    auto_ptr<blast::SSeqLoc> sl(CTestObjMgr::Instance().CreateSSeqLoc(*id));
+    unique_ptr<blast::SSeqLoc> sl(CTestObjMgr::Instance().CreateSSeqLoc(*id));
     TSeqLocVector queries(1, *sl.get());
     CRef<IQueryFactory> qf(new CObjMgr_QueryFactory(queries));
     const string kDbName("nt");
@@ -1085,7 +1085,7 @@ BOOST_AUTO_TEST_CASE(GetSearchStrategy_FullQuery) {
 BOOST_AUTO_TEST_CASE(GetSearchStrategy_QueryWithRange) {
     CRef<CSeq_id> id(new CSeq_id(CSeq_id::e_Gi, 555));
     TSeqRange query_range(1,200);
-    auto_ptr<blast::SSeqLoc> sl(CTestObjMgr::Instance().CreateSSeqLoc(*id,
+    unique_ptr<blast::SSeqLoc> sl(CTestObjMgr::Instance().CreateSSeqLoc(*id,
                                 query_range));
     TSeqLocVector queries(1, *sl.get());
     CRef<IQueryFactory> qf(new CObjMgr_QueryFactory(queries));
