@@ -1,4 +1,4 @@
-/*  $Id: blast_app_util.cpp 632181 2021-05-27 13:23:25Z ivanov $
+/*  $Id: blast_app_util.cpp 683535 2024-05-30 12:54:58Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -124,22 +124,6 @@ InitializeRemoteBlast(CRef<blast::IQueryFactory> queries,
     return retval;
 }
 
-static string s_FindBlastDbDataLoaderName(const string& dbname, bool is_protein)
-{
-    // This string is built based on the knowledge of how the BLAST DB data
-    // loader names are created, see 
-    // {CBlastDbDataLoader,CRemoteBlastDbDataLoader}::GetLoaderNameFromArgs
-    const string str2find(string("_") + dbname + string(is_protein ? "P" : "N"));
-    CObjectManager::TRegisteredNames loader_names;
-    CObjectManager::GetInstance()->GetRegisteredNames(loader_names);
-    ITERATE(CObjectManager::TRegisteredNames, loader_name, loader_names) {
-        if (NStr::Find(*loader_name, str2find) != NPOS) {
-            return *loader_name;
-        }
-    }
-    return kEmptyStr;
-}
-
 blast::SDataLoaderConfig 
 InitializeQueryDataLoaderConfiguration(bool query_is_protein, 
                                        CRef<blast::CLocalDbAdapter> db_adapter)
@@ -193,13 +177,20 @@ InitializeSubject(CRef<blast::CBlastDatabaseArgs> db_args,
         const bool is_protein = 
             Blast_SubjectIsProtein(opts_hndl->GetOptions().GetProgramType())
 			? true : false;
-        SDataLoaderConfig config(is_protein);
-        CBlastScopeSource scope_src(config);
+        CRef<CBlastScopeSource> scope_src;
+        if (search_db.Empty()) {
+        	SDataLoaderConfig config(is_protein);
+        	scope_src.Reset(new CBlastScopeSource(config));
+        }
+        else {
+        	SDataLoaderConfig config(search_db->GetDatabaseName(), is_protein);
+        	scope_src.Reset(new CBlastScopeSource(config));
+        }
         // configure scope to fetch sequences remotely for formatting
         if (scope.NotEmpty()) {
-            scope_src.AddDataLoaders(scope);
+            scope_src->AddDataLoaders(scope);
         } else {
-            scope = scope_src.NewScope();
+            scope = scope_src->NewScope();
         }
     } else {
         if (scope.Empty()) {
@@ -779,14 +770,17 @@ void BlastFormatter_PreFetchSequenceData(const blast::CSearchResultSet& results,
        s_ExtractSeqidsAndRanges(results, ids, ranges);
        _TRACE("Prefetching " << ids.size() << " sequence lengths");
        LoadSequencesToScope(ids, ranges, scope);
-	} catch (CException& e) {
+	}
+    catch(CSeqDBException & ) {
+    	throw;
+    }
+    catch (CException& e) {
 	   if(s_IsUsingRemoteBlastDbDataLoader()) {
-          NCBI_THROW(CBlastSystemException, eNetworkError,
-                     "Error fetching sequence data from BLAST databases at NCBI, "
+          ERR_POST(Warning << "Error fetching sequence data from BLAST databases at NCBI, "
                      "please try again later");
 	   }
 	   else {
-          NCBI_RETHROW_SAME(e, "Error pre-fetching sequence data ");
+          ERR_POST(Warning << "Error pre-fetching sequence data");
 	   }
    }
 
@@ -949,15 +943,18 @@ void LogQueryInfo(CBlastUsageReport & report, const CBlastInput & q_info)
 }
 
 
-void LogRPSBlastOptions(blast::CBlastUsageReport & report, const CBlastOptions & opt)
+void LogBlastOptions(blast::CBlastUsageReport & report, const CBlastOptions & opt)
 {
-	report.AddParam(CBlastUsageReport::eProgram, Blast_ProgramNameFromType(opt.GetProgramType()));
+	EBlastProgramType prog_type = opt.GetProgramType();
+	report.AddParam(CBlastUsageReport::eProgram, Blast_ProgramNameFromType(prog_type));
 	report.AddParam(CBlastUsageReport::eEvalueThreshold, opt.GetEvalueThreshold());
 	report.AddParam(CBlastUsageReport::eHitListSize, opt.GetHitlistSize());
-    report.AddParam(CBlastUsageReport::eCompBasedStats, opt.GetCompositionBasedStats());
+	if (!Blast_ProgramIsNucleotide(prog_type)) {
+		report.AddParam(CBlastUsageReport::eCompBasedStats, opt.GetCompositionBasedStats());
+	}
 }
 
-void LogRPSCmdOptions(blast::CBlastUsageReport & report, const CBlastAppArgs & args)
+void LogCmdOptions(blast::CBlastUsageReport & report, const CBlastAppArgs & args)
 {
 	if (args.GetBlastDatabaseArgs().NotEmpty() &&
 		args.GetBlastDatabaseArgs()->GetSearchDatabase().NotEmpty() &&
@@ -980,7 +977,7 @@ void LogRPSCmdOptions(blast::CBlastUsageReport & report, const CBlastAppArgs & a
 	}
 }
 
-int GetMTByQueriesBatchSize(EProgram p, int num_threads)
+int GetMTByQueriesBatchSize(EProgram p, int num_threads, const string & task)
 {
 	    int batch_size = 0;
 
@@ -989,65 +986,43 @@ int GetMTByQueriesBatchSize(EProgram p, int num_threads)
 			batch_size = NStr::StringToInt(mt_query_batch_env);
 		}
 		else {
-			int factor = 1;
-			if ( EProgramToEBlastProgramType(p) == eBlastTypeBlastn ) {
-				factor = 2;
-			}
-			batch_size = GetQueryBatchSize(p)/factor;
+			batch_size = GetQueryBatchSize(p);
 		}
+                if (task == "blastx-fast")
+                {  // Set batch_size to 20004
+                        batch_size *= 2;
+                }
 		return batch_size;
 }
 
-void CheckMTByQueries_DBSize(CRef<CLocalDbAdapter> & db_adapter, const CBlastOptions & opt)
+void MTByQueries_DBSize_Warning(const Int8 length_limit, bool is_db_protein)
 {
-	CRef<CSearchDatabase> sdb = db_adapter->GetSearchDatabase();
-	const CRef<CSeqDBGiList>&  gi = sdb->GetGiList();
-	const CRef<CSeqDBGiList>& n_gi = sdb->GetNegativeGiList();
-
-	if (gi.NotEmpty() || n_gi.NotEmpty()) {
-		return;
-	}
-
-	EProgram prog = opt.GetProgram();
-	CRef<CSeqDB> seqdb = sdb->GetSeqDb();
-	Uint8 total_length = seqdb->GetTotalLength();
-	Uint8 length_limit = 0;
-
-	if (db_adapter->IsProtein()) {
-		static const Uint8 kMaxProtTotalLength = 2000000000;
-		length_limit = kMaxProtTotalLength;
+	string warn = "This database is probably too large to benefit from -mt_mode=1. " \
+				  "We suggest using -mt_mode=1 only if the database is less than " \
+				  + NStr::Int8ToString(length_limit, NStr::fWithCommas);
+	if (is_db_protein) {
+		warn +=  + " residues ";
 	}
 	else {
-		static const Uint8 kMaxNuclTotalLength = 14000000000;
-		length_limit = kMaxNuclTotalLength;
-	}
-
-	if(total_length > length_limit) {
-		string warn = "This database is probably too large to benefit from -mt_mode=1. " \
-					  "We suggest using -mt_mode=1 only if the database is less than";
-		if (db_adapter->IsProtein()) {
-			warn += " 2 billion residues ";
-		}
-		else {
-			warn += " 14 billion bases ";
-		}
-       	ERR_POST(Warning <<   warn + "or if the search is limited by an option such as -taxids, -taxidlist or -gilist.");
+		warn += " bases ";
 	}
+   	ERR_POST(Warning <<   warn + "or if the search is limited by an option such as -taxids, -taxidlist or -gilist.");
 	return;
 }
 
 void CheckMTByQueries_QuerySize(EProgram prog, int batch_size)
 {
-	string warning = "This set of queries is too small to fully benefit from the -mt_mode=1 option. "\
+	string warning = "This set of queries is too small to fully benefit from the -mt_mode=1 option. " \
 			         "The total number of letters should be at least ";
 	warning += NStr::IntToString(batch_size);
 	EBlastProgramType p = EProgramToEBlastProgramType(prog);
 	if (Blast_QueryIsProtein(p)) {
-		warning += " residues per thread.";
+		warning += " residues";
 	}
 	else {
-		warning += " bases per thread.";
+		warning += " bases";
 	}
+	warning += " per thread, and there should be at least one query of this length per thread.";
     ERR_POST(Warning << warning);
 }
 
