@@ -1,4 +1,4 @@
-/*  $Id: blastx_app.cpp 632181 2021-05-27 13:23:25Z ivanov $
+/*  $Id: blastx_app.cpp 683938 2024-06-10 12:09:24Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -39,6 +39,7 @@
 #include <algo/blast/blastinput/blastx_args.hpp>
 #include <algo/blast/api/objmgr_query_data.hpp>
 #include <algo/blast/format/blast_format.hpp>
+#include <util/profile/rtprofile.hpp>
 #include "blast_app_util.hpp"
 #include "blastx_node.hpp"
 
@@ -76,8 +77,10 @@ private:
 
     /// This application's command line args
     CRef<CBlastxAppArgs> m_CmdLineArgs;
+    CBlastAppDiagHandler m_Bah;
     CBlastUsageReport m_UsageReport;
     CStopWatch m_StopWatch;
+    CRef<CBlastOptionsHandle> m_OptsHndl;
 };
 
 void CBlastxApp::Init()
@@ -92,49 +95,108 @@ void CBlastxApp::Init()
     SetupArgDescriptions(m_CmdLineArgs->SetCommandLine());
 }
 
+// To Do : Need to fill in numbers
+static void s_GetMT1Cutoffs(const int word_size, Int8 & max_db_size, Int8 & min_q_size)
+{
+	const Int8 kMT1MaxDbSz_Blastx = 900000000;
+	const Int8 kMT1MinQSz_Blastx = 1250000;
+
+	const Int8 kMT1MaxDbSz_BlastxFast = 900000000;
+	const Int8 kMT1MinQSz_BlastxFast = 1250000;
+
+	if (word_size > 4) {
+		// CompressedAa
+		max_db_size = kMT1MaxDbSz_BlastxFast;
+		min_q_size = kMT1MinQSz_BlastxFast;
+	}
+	else {
+		max_db_size = kMT1MaxDbSz_Blastx;
+		min_q_size = kMT1MinQSz_Blastx;
+	}
+}
+
 int CBlastxApp::Run(void)
 {
+    int status = BLAST_EXIT_SUCCESS;
+    BLAST_PROF_ADD2( PROGRAM, blastx ) ;
+    BLAST_PROF_ADD( THREADS , (int)m_CmdLineArgs->GetNumThreads() );
+    BLAST_PROF_ADD( MT_MODE , (int)m_CmdLineArgs->GetMTMode() );
 	const CArgs& args = GetArgs();
-	CMTArgs mt_args(args);
-	if ((mt_args.GetMTMode() == CMTArgs::eSplitByQueries) &&
-		(mt_args.GetNumThreads() > 1)){
+	//Check to see if search should split by queries
+	try {
+	SetDiagPostLevel(eDiag_Warning);
+	SetDiagPostPrefix("blastx");
+	SetDiagHandler(&m_Bah, false);
+    if(RecoverSearchStrategy(args, m_CmdLineArgs)) {
+    	m_OptsHndl.Reset(&*m_CmdLineArgs->SetOptionsForSavedStrategy(args));
+    }
+    else {
+    	m_OptsHndl.Reset(&*m_CmdLineArgs->SetOptions(args));
+    }
+    int num_threads = m_CmdLineArgs->GetNumThreads();
+    int mt_mode = m_CmdLineArgs->GetMTMode();
+	if (!m_CmdLineArgs->ExecuteRemotely() && (num_threads > 1) &&
+	   (mt_mode != CMTArgs::eSplitByDB)){
+    	CRef<CBlastDatabaseArgs> db_args(m_CmdLineArgs->GetBlastDatabaseArgs());
+		CRef<CSearchDatabase> sdb=db_args->GetSearchDatabase();
+		if (db_args->GetDatabaseName() != kEmptyStr) {
+			int word_size = m_OptsHndl->GetOptions().GetWordSize();
+			Int8 max_db_size = 0;
+			Int8 min_q_size = 0;
+			s_GetMT1Cutoffs(word_size, max_db_size, min_q_size);
+			CRef<CSeqDB> seqdb = sdb->GetSeqDb();
+			Uint8 total_length = seqdb->GetTotalLength();
+			if (mt_mode == CMTArgs::eSplitAuto){
+				if (total_length < max_db_size) {
+					if (args.Exist(kArgQuery) && args[kArgQuery].HasValue()) {
+						CFile file( args[kArgQuery].AsString());
+						if (file.GetLength() > min_q_size) {
+							m_UsageReport.AddParam(CBlastUsageReport::eMTMode, CMTArgs::eSplitByQueries);
+							return x_RunMTBySplitQuery();
+						}
+					}
+				}
+			}
+			else {
+				if (total_length > max_db_size) {
+					MTByQueries_DBSize_Warning(max_db_size, true);
+				}
+			}
+		}
+	}
+
+	if ((mt_mode == CMTArgs::eSplitByQueries) && (num_threads > 1)){
 		m_UsageReport.AddParam(CBlastUsageReport::eMTMode, CMTArgs::eSplitByQueries);
 		return x_RunMTBySplitQuery();
 	}
 	else {
 		return x_RunMTBySplitDB();
 	}
+	} CATCH_ALL(status)
+
+    if(!m_Bah.GetMessages().empty()) {
+    	PrintErrorArchive(args, m_Bah.GetMessages());
+    }
+	return status;
 }
 
 int CBlastxApp::x_RunMTBySplitDB()
 {
-
+    BLAST_PROF_START( APP.MAIN );
+    BLAST_PROF_START( APP.PRE );
     int status = BLAST_EXIT_SUCCESS;
-    CBlastAppDiagHandler bah;
+    int batch_num = 0;
 
     try {
 
-        // Allow the fasta reader to complain on invalid sequence input
-        SetDiagPostLevel(eDiag_Warning);
-        SetDiagPostPrefix("blastx");
-        SetDiagHandler(&bah, false);
-
-        /*** Get the BLAST options ***/
         const CArgs& args = GetArgs();
-        CRef<CBlastOptionsHandle> opts_hndl;
-        if(RecoverSearchStrategy(args, m_CmdLineArgs)) {
-           	opts_hndl.Reset(&*m_CmdLineArgs->SetOptionsForSavedStrategy(args));
-        }
-        else {
-           	opts_hndl.Reset(&*m_CmdLineArgs->SetOptions(args));
-        }
-        const CBlastOptions& opt = opts_hndl->GetOptions();
+		const CBlastOptions& opt = m_OptsHndl->GetOptions();
 
         /*** Initialize the database/subject ***/
         CRef<CBlastDatabaseArgs> db_args(m_CmdLineArgs->GetBlastDatabaseArgs());
         CRef<CLocalDbAdapter> db_adapter;
         CRef<CScope> scope;
-        InitializeSubject(db_args, opts_hndl, m_CmdLineArgs->ExecuteRemotely(),
+        InitializeSubject(db_args, m_OptsHndl, m_CmdLineArgs->ExecuteRemotely(),
                          db_adapter, scope);
         _ASSERT(db_adapter && scope);
 
@@ -159,7 +221,7 @@ int CBlastxApp::x_RunMTBySplitDB()
         CRef<CFormattingArgs> fmt_args(m_CmdLineArgs->GetFormattingArgs());
         bool isArchiveFormat = fmt_args->ArchiveFormatRequested(args);
         if(!isArchiveFormat) {
-           	bah.DoNotSaveMessages();
+           	m_Bah.DoNotSaveMessages();
         }
         CBlastFormat formatter(opt, *db_adapter,
                                fmt_args->GetFormattedOutputChoice(),
@@ -191,31 +253,36 @@ int CBlastxApp::x_RunMTBySplitDB()
         }
         formatter.PrintProlog();
 
+	BLAST_PROF_STOP( APP.PRE );
+	BLAST_PROF_ADD( BATCH_SIZE, (int)input.GetBatchSize() );
         /*** Process the input ***/
         for (; !input.End(); formatter.ResetScopeHistory(), QueryBatchCleanup()) {
-
+	    BLAST_PROF_START( APP.LOOP.PRE );
             CRef<CBlastQueryVector> query_batch(input.GetNextSeqBatch(*scope));
             CRef<IQueryFactory> queries(new CObjMgr_QueryFactory(*query_batch));
 
-            SaveSearchStrategy(args, m_CmdLineArgs, queries, opts_hndl);
+            SaveSearchStrategy(args, m_CmdLineArgs, queries, m_OptsHndl);
 
             CRef<CSearchResultSet> results;
+	    BLAST_PROF_STOP( APP.LOOP.PRE );
 
             if (m_CmdLineArgs->ExecuteRemotely()) {
                 CRef<CRemoteBlast> rmt_blast = 
-                    InitializeRemoteBlast(queries, db_args, opts_hndl,
+                    InitializeRemoteBlast(queries, db_args, m_OptsHndl,
                           m_CmdLineArgs->ProduceDebugRemoteOutput(),
                           m_CmdLineArgs->GetClientId());
                 results = rmt_blast->GetResultSet();
             } else {
-                CLocalBlast lcl_blast(queries, opts_hndl, db_adapter);
+	        BLAST_PROF_START( APP.LOOP.BLAST );
+                CLocalBlast lcl_blast(queries, m_OptsHndl, db_adapter);
                 lcl_blast.SetNumberOfThreads(m_CmdLineArgs->GetNumThreads());
                 results = lcl_blast.Run();
+	        BLAST_PROF_STOP( APP.LOOP.BLAST );
             }
-
+	    BLAST_PROF_START( APP.LOOP.FMT );
             if (fmt_args->ArchiveFormatRequested(args)) {
-                formatter.WriteArchive(*queries, *opts_hndl, *results, 0, bah.GetMessages());
-                bah.ResetMessages();
+                formatter.WriteArchive(*queries, *m_OptsHndl, *results, 0, m_Bah.GetMessages());
+                m_Bah.ResetMessages();
             } else {
                 BlastFormatter_PreFetchSequenceData(*results, scope,
                 		                            fmt_args->GetFormattedOutputChoice());
@@ -223,45 +290,43 @@ int CBlastxApp::x_RunMTBySplitDB()
                	    formatter.PrintOneResultSet(**result, query_batch);
             	}
             }
+	    BLAST_PROF_STOP( APP.LOOP.FMT );
+	    batch_num++;
         }
-
+        BLAST_PROF_START( APP.POST );
         formatter.PrintEpilog(opt);
 
         if (m_CmdLineArgs->ProduceDebugOutput()) {
-            opts_hndl->GetOptions().DebugDumpText(NcbiCerr, "BLAST options", 1);
+            m_OptsHndl->GetOptions().DebugDumpText(NcbiCerr, "BLAST options", 1);
         }
 
         LogQueryInfo(m_UsageReport, input);
         formatter.LogBlastSearchInfo(m_UsageReport);
+        BLAST_PROF_STOP( APP.POST );
     } CATCH_ALL(status)
-    if(!bah.GetMessages().empty()) {
+    if(!m_Bah.GetMessages().empty()) {
        	const CArgs & a = GetArgs();
-       	PrintErrorArchive(a, bah.GetMessages());
+       	PrintErrorArchive(a, m_Bah.GetMessages());
     }
+
+    m_UsageReport.AddParam(CBlastUsageReport::eTask, m_CmdLineArgs->GetTask());
 	m_UsageReport.AddParam(CBlastUsageReport::eNumThreads, (int) m_CmdLineArgs->GetNumThreads());
     m_UsageReport.AddParam(CBlastUsageReport::eExitStatus, status);
+    BLAST_PROF_STOP( APP.MAIN );
+    BLAST_PROF_ADD( BATCHES , (int)batch_num );
+    BLAST_PROF_ADD( EXIT_STATUS , (int)status );
+    BLAST_PROF_REPORT ;
     return status;
 }
 
 int CBlastxApp::x_RunMTBySplitQuery()
 {
     int status = BLAST_EXIT_SUCCESS;
-    CBlastAppDiagHandler bah;
-
-    // Allow the fasta reader to complain on invalid sequence input
-    SetDiagPostLevel(eDiag_Warning);
-    SetDiagPostPrefix("blastx");
-    SetDiagHandler(&bah, false);
+    int chunk_num = 0; // logically this is also batch number
+    BLAST_PROF_START( APP.MAIN );
 
 	try {
     	const CArgs& args = GetArgs();
-    	CRef<CBlastOptionsHandle> opts_hndl;
-        if(RecoverSearchStrategy(args, m_CmdLineArgs)) {
-        	opts_hndl.Reset(&*m_CmdLineArgs->SetOptionsForSavedStrategy(args));
-        }
-        else {
-        	opts_hndl.Reset(&*m_CmdLineArgs->SetOptions(args));
-        }
     	if(IsIStreamEmpty(m_CmdLineArgs->GetInputStream())){
        		ERR_POST(Warning << "Query is Empty!");
        		return BLAST_EXIT_SUCCESS;
@@ -269,9 +334,13 @@ int CBlastxApp::x_RunMTBySplitQuery()
     	CNcbiOstream & out_stream = m_CmdLineArgs->GetOutputStream();
     	const int kMaxNumOfThreads =  m_CmdLineArgs->GetNumThreads();
 		CBlastMasterNode master_node(out_stream, kMaxNumOfThreads);
-   		int chunk_num = 0;
-   	    int batch_size = GetMTByQueriesBatchSize(opts_hndl->GetOptions().GetProgram(), kMaxNumOfThreads);
+
+   		LogBlastOptions(m_UsageReport, m_OptsHndl->GetOptions());
+   		LogCmdOptions(m_UsageReport, *m_CmdLineArgs);
+
+   	    int batch_size = GetMTByQueriesBatchSize(m_OptsHndl->GetOptions().GetProgram(), kMaxNumOfThreads, m_CmdLineArgs->GetTask());
    		INFO_POST("Batch Size: " << batch_size);
+	        BLAST_PROF_ADD( BATCH_SIZE, (int)batch_size );
    		CBlastNodeInputReader input(m_CmdLineArgs->GetInputStream(), batch_size, 2000);
 		while (master_node.Processing()) {
 			if (!input.AtEOF()) {
@@ -281,7 +350,7 @@ int CBlastxApp::x_RunMTBySplitQuery()
 					int num_q = input.GetQueryBatch(qb, q_index);
 					if (num_q > 0) {
 						CBlastNodeMailbox * mb(new CBlastNodeMailbox(chunk_num, master_node.GetBuzzer()));
-						CBlastxNode * t(new CBlastxNode(chunk_num, GetArguments(), args, bah, qb, q_index, num_q, mb));
+						CBlastxNode * t(new CBlastxNode(chunk_num, GetArguments(), args, m_Bah, qb, q_index, num_q, mb));
 						master_node.RegisterNode(t, mb);
 						chunk_num ++;
 					}
@@ -292,16 +361,23 @@ int CBlastxApp::x_RunMTBySplitQuery()
 			}
     	}
 
-		if(chunk_num < kMaxNumOfThreads){
-			CheckMTByQueries_QuerySize(opts_hndl->GetOptions().GetProgram(), batch_size);
-		}
+		m_UsageReport.AddParam(CBlastUsageReport::eNumQueryBatches, chunk_num);
+		m_UsageReport.AddParam(CBlastUsageReport::eNumQueries, master_node.GetNumOfQueries());
+		m_UsageReport.AddParam(CBlastUsageReport::eTotalQueryLength, master_node.GetQueriesLength());
+		m_UsageReport.AddParam(CBlastUsageReport::eNumErrStatus, master_node.GetNumErrStatus());
+
 	} CATCH_ALL (status)
-
-    if(!bah.GetMessages().empty()) {
+    if(!m_Bah.GetMessages().empty()) {
     	const CArgs & a = GetArgs();
-    	PrintErrorArchive(a, bah.GetMessages());
+    	PrintErrorArchive(a, m_Bah.GetMessages());
     }
-
+    m_UsageReport.AddParam(CBlastUsageReport::eTask, m_CmdLineArgs->GetTask());
+    m_UsageReport.AddParam(CBlastUsageReport::eNumThreads, (int) m_CmdLineArgs->GetNumThreads());
+    m_UsageReport.AddParam(CBlastUsageReport::eExitStatus, status);
+    BLAST_PROF_STOP( APP.MAIN );
+    BLAST_PROF_ADD( BATCHES , (int)chunk_num );
+    BLAST_PROF_ADD( EXIT_STATUS , (int)status );
+    BLAST_PROF_REPORT ;
     return status;
 }
 
