@@ -1,6 +1,6 @@
 #! /bin/sh
 #############################################################################
-# $Id: cmICC.sh 612333 2020-07-20 14:49:13Z gouriano $
+# $Id: cmICC.sh 666592 2023-04-27 17:30:21Z gouriano $
 #   Compiler:   Intel C++
 #   Find ICC compiler of a specific version and
 #   call CMake configuration script
@@ -18,7 +18,7 @@ CXX="icpc"
 
 Usage() {
     echo "USAGE:   $script_name [version] [configure-flags] | -h]"
-    echo "example: $script_name 19.0"
+    echo "example: $script_name 21.0"
 }
 
 if test $# -eq 0 -o "$1" = "-h"; then
@@ -36,11 +36,14 @@ fi
 
 cxx_version=""
 if [ -z "$CMAKECFGRECURSIONGUARD" ]; then
-  cxx_version=$1
+  cxx_version=${1#20}
 else
   for val in ${script_args}
   do
   case "$val" in 
+    20[0-9][0-9] | 20[0-9][0-9].* )
+      cxx_version=${val#20}
+      ;;
     [1-9]*)
       cxx_version=$val
       ;; 
@@ -56,18 +59,37 @@ if test -z "$cxx_version"; then
   exit 1
 fi
 
+case "$cxx_version" in
+  *.* )
+    ;;
+  * )
+    cxx_version=$cxx_version.0
+    ;;
+esac
+
+case "$cxx_version" in
+  [2-9][0-9] | [2-9][0-9].* )
+    CC=icx
+    CXX=icpx
+    cxx_path=$cxx_version/compiler/latest/linux
+    ;;
+  * )
+    cxx_path=$cxx_version
+    ;;      
+esac
+
 # Look for the specified version in various reasonable places
 # (tuned for NCBI's installations).
 case "$cxx_version" in
   [1-9]*)
-     if $intel_root/Compiler/$cxx_version/bin/$CXX -dumpversion >/dev/null 2>&1; then
-       CXX=$intel_root/Compiler/$cxx_version/bin/$CXX
-       CC=$intel_root/Compiler/$cxx_version/bin/$CC
-       libICC=$intel_root/Compiler/$cxx_version/lib/intel64 
+     if $intel_root/Compiler/$cxx_path/bin/$CXX -dumpversion >/dev/null 2>&1; then
+       CXX=$intel_root/Compiler/$cxx_path/bin/$CXX
+       CC=$intel_root/Compiler/$cxx_path/bin/$CC
+       libICC=$intel_root/Compiler/$cxx_path/lib/intel64 
        gccver=7.3.0
      else
-       if [ -x $intel_root/Compiler/$cxx_version/bin/$CXX ]; then
-         $intel_root/Compiler/$cxx_version/bin/$CXX -dumpversion >/dev/null
+       if [ -x $intel_root/Compiler/$cxx_path/bin/$CXX ]; then
+         $intel_root/Compiler/$cxx_path/bin/$CXX -dumpversion >/dev/null
        fi
        cat <<EOF 1>&2
 ERROR:  cannot find ICC version $cxx_version; you may need to adjust PATH explicitly.
@@ -93,6 +115,18 @@ EOF
     exit 1
 fi
 
+case $cxx_version in
+  [2-9][0-9] | [2-9][0-9].* )
+    NCBI_CLANG_CC=$CC
+    NCBI_CLANG_CXX=$CXX
+    NCBI_CLANG_NAME="ICC"
+    libICC=$intel_root/Compiler/$cxx_path/compiler/lib/intel64 
+    export NCBI_CLANG_CC NCBI_CLANG_CXX NCBI_CLANG_NAME libICC
+    exec "`dirname $0`"/cmClang.sh "$@"
+    ;;
+esac
+  
+
 # Intel C++ Compiler: GCC* Compatibility and Interoperability
 # https://software.intel.com/en-us/cpp-compiler-developer-guide-and-reference-gcc-compatibility-and-interoperability
 
