@@ -1,4 +1,4 @@
-/*  $Id: cgictx.cpp 595395 2019-10-21 14:46:03Z grichenk $
+/*  $Id: cgictx.cpp 673232 2023-09-26 15:06:09Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #include <corelib/ncbidiag.hpp>
 #include <corelib/request_ctx.hpp>
 #include <corelib/ncbi_safe_static.hpp>
+#include <corelib/ncbi_url.hpp>
 #include <cgi/ncbires.hpp>
 #include <cgi/cgictx.hpp>
 #include <cgi/cgi_util.hpp>
@@ -45,6 +46,9 @@
 
 
 #ifdef NCBI_OS_UNIX
+#  ifdef HAVE_POLL_H
+#    include <poll.h>
+#  endif
 #  ifdef _AIX32 // version 3.2 *or higher*
 #    include <strings.h> // needed for bzero()
 #  endif
@@ -118,7 +122,7 @@ CCgiContext::CCgiContext(CCgiApplication&        app,
                                 env  ? env  : &app.GetEnvironment(),
                                 inp, flags, ifd, errbuf_size)),
       m_Response(out, ofd),
-      m_SecureMode(eSecure_NotSet),
+      m_IsSecure(false),
       m_StatusCode(CCgiException::eStatusNotSet)
 {
     m_Response.SetRequestMethod(m_Request->GetRequestMethod());
@@ -139,7 +143,7 @@ CCgiContext::CCgiContext(CCgiApplication&        app,
     : m_App(&app),
       m_Request(new CCgiRequest()),
       m_Response(os, -1),
-      m_SecureMode(eSecure_NotSet),
+      m_IsSecure(false),
       m_StatusCode(CCgiException::eStatusNotSet)
 {
     m_Request->Deserialize(*is,flags);
@@ -160,7 +164,7 @@ CCgiContext::CCgiContext(ICgiSessionStorage*     session_storage,
     : m_App(nullptr),
       m_Request(new CCgiRequest(args, env, inp, flags, ifd, errbuf_size)),
       m_Response(out, ofd),
-      m_SecureMode(eSecure_NotSet),
+      m_IsSecure(false),
       m_StatusCode(CCgiException::eStatusNotSet)
 {
     m_Response.SetRequestMethod(m_Request->GetRequestMethod());
@@ -337,124 +341,179 @@ const string& CCgiContext::GetSelfURL(void) const
     if ( !m_SelfURL.empty() )
         return m_SelfURL;
 
-    // First check forwarded URLs
-    const string& caf_url = GetRequest().GetRandomProperty("CAF_URL");
-    if ( !caf_url.empty() ) {
-        m_SelfURL = caf_url;
-        return m_SelfURL;
-    }
-
-    bool secure
-        = AStrEquiv
-        (GetRequest().GetRandomProperty("HTTPS", false), "on",
-            PNocase())
-        || AStrEquiv
-        (GetRequest().GetRandomProperty("X_FORWARDED_PROTO"), "https",
-            PNocase());
-
-    string host_port;
-    // Check HTTP_X_FORWARDED_HOST for host:port
-    const string& x_fwd_host = GetRequest().GetRandomProperty("X_FORWARDED_HOST");
-    if (!x_fwd_host.empty()) {
-        host_port = x_fwd_host;
-    }
-    else {
-        string server(GetRequest().GetProperty(eCgi_ServerName));
-        if (server.empty()) {
-            return kEmptyStr;
-        }
-
-        host_port = server;
-        string port = GetRequest().GetProperty(eCgi_ServerPort);
-        // Skip port if it's default for the selected scheme
-        if ((secure  &&  port == "443") || (!secure  &&  port == "80")
-            || (server.size() >= port.size() + 2 && NStr::EndsWith(server, port)
-                && server[server.size() - port.size() - 1] == ':')) {
-            port.erase();
-        }
-        if (!port.empty()) {
-            host_port += ':';
-            host_port += port;
-        }
-    }
+    ESecureMode secure = x_IsSecure();
 
     // Compose self URL
-    m_SecureMode = secure ? eSecure_On : eSecure_Off;
-    m_SelfURL    = secure ? "https://" : "http://";
-    m_SelfURL += host_port;
+    CUrl url;
 
-    string script_uri;
-    script_uri = GetRequest().GetRandomProperty("SCRIPT_URL", false);
-    if ( script_uri.empty() ) {
-        script_uri = GetRequest().GetProperty(eCgi_ScriptName);
+    // Forwarded URL
+    const string& caf_url = GetRequest().GetRandomProperty("CAF_URL");
+    if ( !caf_url.empty() ) { 
+        url.SetUrl(caf_url);
+        url.GetArgs().clear();
+        url.SetFragment(kEmptyStr);
+        if (secure != eSecure_On) {
+            secure  = NStr::EqualNocase(caf_url, 0, 8, "https://") 
+                ? eSecure_On : eSecure_Off;
+        }
     }
-    // Remove args if any
-    size_t arg_pos = script_uri.find('?');
-    if (arg_pos != NPOS) {
-        script_uri.resize(arg_pos);
+    m_IsSecure = secure == eSecure_On;
+
+    // Check HTTP_X_FORWARDED_HOST for host:port
+    const string& x_fwd_host =
+        GetRequest().GetRandomProperty("X_FORWARDED_HOST");
+    // Fallback to HTTP_HOST if no HTTP_X_FORWARDED_HOST
+    const string& host_port = !x_fwd_host.empty() ? x_fwd_host
+        : caf_url.empty() ? GetRequest().GetRandomProperty("HOST") : kEmptyStr;
+
+    CTempString host, port;
+    bool no_host = false, no_port = false;
+    if ( !host_port.empty() ) {
+        size_t pos = host_port.find(':');
+        host = CTempString(host_port, 0, pos != NPOS ? pos : host_port.size());
+        port = CTempString(host_port, pos != NPOS? pos + 1 : host_port.size(),
+                           host_port.size());
+    } else if ( !caf_url.empty() ) {
+        no_host = no_port = true;
+        port = url.GetPort();
+    } else {
+        host = GetRequest().GetProperty(eCgi_ServerName);
+        port = GetRequest().GetProperty(eCgi_ServerPort);
+    }
+    // Skip port if it's default for the selected scheme
+    if ((secure == eSecure_Off  &&  port == "80")  ||
+        (secure == eSecure_On   &&  port == "443")) {
+        no_port = false;
+        port.clear();
+    }
+    if ( !no_host ) {
+        url.SetHost(host);
+    }
+    if ( !no_port ) {
+        url.SetPort(port);
+    }
+
+    string path;
+    if ( !caf_url.empty() ) {
+        path = url.GetPath();
+    } else {
+        path = GetRequest().GetRandomProperty("SCRIPT_URL", false);
+        if ( path.empty() ) {
+            path = GetRequest().GetProperty(eCgi_ScriptName);
+        }
+        // Remove args if any
+        size_t pos = path.find_first_of("?#");
+        if (pos != NPOS) {
+            path.resize(pos);
+        }
     }
     // (replace adjacent '//' to work around a bug in the "www.ncbi" proxy;
     //  it should not hurt, and may help with similar proxies outside NCBI)
-    m_SelfURL += NStr::ReplaceInPlace(script_uri, "//", "/");
+    url.SetPath(NStr::ReplaceInPlace(path, "//", "/"));
 
+    if (!url.IsEmpty()) {
+        url.SetIsGeneric(true);
+        if (secure != eSecure_NotSet) {
+            url.SetScheme(m_IsSecure ? "https" : "http");
+        } else {
+            _ASSERT(url.GetScheme().empty());
+        }
+        url.ComposeUrl(CUrlArgs::eAmp_Char/*no args anyways*/).swap(m_SelfURL);
+    }
     return m_SelfURL;
 }
 
 
-bool CCgiContext::IsSecure(void) const
+CCgiContext::ESecureMode CCgiContext::x_IsSecure(void) const
 {
-    if (m_SecureMode == eSecure_NotSet) {
-        m_SecureMode
-            =   NStr::EqualNocase
-            (CTempStringEx(GetSelfURL(), 0, 8), "https://")
-            ||  NStr::EqualNocase
-            (GetRequest().GetRandomProperty("HTTPS", false), "on")
-            ||  NStr::EqualNocase
-            (GetRequest().GetRandomProperty("X_FORWARDED_PROTO"), "https")
-            ? eSecure_On : eSecure_Off;
+    ESecureMode secure;
+    const string& x_fwd_proto
+        = GetRequest().GetRandomProperty("X_FORWARDED_PROTO");
+    if ( !x_fwd_proto.empty() ) {
+        if ( NStr::EqualNocase(x_fwd_proto, "https") )
+            return eSecure_On;
+        if ( NStr::EqualNocase(x_fwd_proto, "http") )
+            return eSecure_Off;
+        secure = eSecure_Off;
+    } else {
+        secure = eSecure_NotSet;
     }
-    return m_SecureMode == eSecure_On;
+    return NStr::EqualNocase
+        (GetRequest().GetRandomProperty("HTTPS", false), "on")
+        ? eSecure_On : secure;
 }
 
 
 CCgiContext::TStreamStatus
-CCgiContext::GetStreamStatus(STimeout* timeout) const
+CCgiContext::GetStreamStatus(const CTimeout& timeout) const
 {
-#if defined(NCBI_OS_UNIX)
-    int ifd  = m_Request->GetInputFD();
-    int ofd  = m_Response.GetOutputFD();
-    int nfds = max(ifd, ofd) + 1;
-    if (nfds == 0) {
-        return 0;
-    }
-
-    fd_set readfds, writefds;
-    FD_ZERO(&readfds);
-    if (ifd >= 0) {
-        FD_SET(ifd, &readfds);
-    }
-    FD_ZERO(&writefds);
-    if (ofd >= 0) {
-        FD_SET(ofd, &writefds);
-    }
-    struct timeval tv;
-    tv.tv_sec  = timeout->sec;
-    tv.tv_usec = timeout->usec;
-    ::select(nfds, &readfds, &writefds, NULL, &tv);
-
     TStreamStatus result = 0;
-    if (ifd >= 0  &&  FD_ISSET(ifd, &readfds)) {
-        result |= fInputReady;
+
+#if defined(NCBI_OS_UNIX)
+    int ifd = m_Request->GetInputFD();
+    int ofd = m_Response.GetOutputFD();
+
+#  if  defined(HAVE_POLL_H)     &&  \
+      !defined(NCBI_OS_DARWIN)  &&  !defined(NCBI_OS_CYGWIN)
+    if (max(ifd, ofd) >= 0) {
+        struct pollfd pfd[2] = { {ifd, POLLIN,  0},
+                                 {ofd, POLLOUT, 0} };
+        int tmo = timeout.IsInfinite()                  ? -1
+            : timeout.IsZero()  ||  timeout.IsDefault() ?  0
+            : (int) timeout.GetAsMilliSeconds();
+        if (::poll(pfd, 2, tmo) > 0) {
+            if (pfd[0].revents)
+                result |= fInputReady;
+            if (pfd[1].revents)
+                result |= fOutputReady;
+        }
     }
-    if (ofd >= 0  &&  FD_ISSET(ofd, &writefds)) {
-        result |= fOutputReady;
+#  else
+    fd_set readfds, *rfds = 0, writefds, *wfds = 0;
+    int nfds = -1;
+
+    if (ifd >= 0  &&  ifd < FD_SETSIZE) {
+        rfds = &readfds;
+        FD_ZERO(rfds);
+        FD_SET(ifd, rfds);
+        nfds = ifd;
     }
-    return result;
+    if (ofd >= 0  &&  ofd < FD_SETSIZE) {
+        wfds = &writefds;
+        FD_ZERO(wfds);
+        FD_SET(ofd, wfds);
+        if (nfds < ofd)
+            nfds = ofd;
+    }
+    if (++nfds) {
+        struct timeval tv, *tvp = 0;
+        if (timeout.IsFinite()) {
+            tvp = &tv;
+            ::memset(tvp, 0, sizeof(*tvp));
+            if (!timeout.IsZero()  &&  !timeout.IsDefault()) {
+                unsigned int sec,  usec;
+                timeout.Get(&sec, &usec);
+                tv.tv_sec  = sec;
+                tv.tv_usec = usec;
+            }
+        }
+        if (::select(nfds, rfds, wfds, NULL, tvp) > 0) {
+            if (rfds  &&  FD_ISSET(ifd, rfds))
+                result |= fInputReady;
+            if (wfds  &&  FD_ISSET(ofd, wfds))
+                result |= fOutputReady;
+        }
+    }
+#  endif /*HAVE_POLL_H && !NCBI_OS_DARWIN && !NCBI_OS_CYGWIN*/
+
 #else
-    return 0;
+    (void) timeout;
 #endif
+
+    return result;
 }
 
+
 static inline bool s_CheckValueForTID(const string& value, string& tid)
 {
     string part1, part2;
@@ -466,6 +525,7 @@ static inline bool s_CheckValueForTID(const string& value, string& tid)
     return false;
 }
 
+
 static inline bool s_CheckCookieForTID(const CCgiCookies& cookies,
     const string& cookie_name, string& tid)
 {
@@ -474,6 +534,7 @@ static inline bool s_CheckCookieForTID(const CCgiCookies& cookies,
     return cookie != NULL && s_CheckValueForTID(cookie->GetValue(), tid);
 }
 
+
 static inline bool s_CheckRequestEntryForTID(const CCgiRequest* request,
     const string& entry_name, string& tid)
 {
@@ -501,11 +562,6 @@ static inline bool s_IsTID(const string& tid)
 
 string CCgiContext::RetrieveTrackingId() const
 {
-    if ( !m_TrackingId.empty() ) {
-        // Use cached value
-        return m_TrackingId;
-    }
-
     static const char* cookie_or_entry_name_1 = "WebCubbyUser";
     static const char* cookie_or_entry_name_2 = "WebEnv";
 
