@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_streambuf.cpp 624532 2021-02-01 13:23:19Z ivanov $
+/* $Id: ncbi_conn_streambuf.cpp 652143 2022-07-01 14:35:45Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -49,9 +49,11 @@ static inline bool x_IsThrowable(EIO_Status status)
 {
     _ASSERT(status != eIO_Success);
 #if (defined(NCBI_COMPILER_GCC)  &&  NCBI_COMPILER_VERSION < 700)  \
-    ||  defined(NCBI_COMPILER_LLVM_CLANG)
+    ||  defined(NCBI_COMPILER_ANY_CLANG)
     // For C++ STLs that have a bug that sentry ctor does not include try/catch
     // so exceptions leak instead of setting badbit as the standard requires.
+    // https://bugs.llvm.org/show_bug.cgi?id=48912
+    // https://github.com/llvm/llvm-project/issues/48256
     return false;
 #else
     return status != eIO_Timeout ? true : false;
@@ -75,8 +77,10 @@ string CConn_Streambuf::x_Message(const char*     method,
                                   EIO_Status      status,
                                   const STimeout* timeout)
 {
-    const char* type = m_Conn ? CONN_GetType    (m_Conn) : 0;
-    char*       text = m_Conn ? CONN_Description(m_Conn) : 0;
+    const char*
+        type = m_Conn ? CONN_GetType(m_Conn)     : 0;
+    unique_ptr<char, void (*)(void*)>
+        text  (m_Conn ? CONN_Description(m_Conn) : 0, free);
     string result("[CConn_Streambuf::");
     result += method;
     result += '(';
@@ -86,20 +90,25 @@ string CConn_Streambuf::x_Message(const char*     method,
             result += "; ";
     }
     if (text) {
-        _ASSERT(*text);
-        result += text;
-        free(text);
+        _ASSERT(*text.get());
+        result += text.get();
     }
     result += ")]  ";
     result += message;
     result += ": ";
     if (status == eIO_Success)
         status  = m_Status;
+    _ASSERT(status != eIO_Success);
     result += IO_StatusStr(status);
-    if (status == eIO_Timeout  &&  timeout  &&  timeout != kDefaultTimeout) {
-        char tmo[40];
-        ::sprintf(tmo, "[%u.%06us]", timeout->sec, timeout->usec);
-        result += tmo;
+    if (status == eIO_Timeout  &&  timeout/*!=kInfiniteTimeout*/) {
+        if (timeout != kDefaultTimeout) {
+            char x_timeout[40];
+            ::sprintf(x_timeout, "[%u.%06us]",
+                      timeout->usec / 1000000 + timeout->sec,
+                      timeout->usec % 1000000);
+            result += x_timeout;
+        } else
+            result += "(default)";
     }
     return result;
 }
@@ -112,7 +121,7 @@ CConn_Streambuf::CConn_Streambuf(CONNECTOR                   connector,
                                  CConn_IOStream::TConn_Flags flgs,
                                  CT_CHAR_TYPE*               ptr,
                                  size_t                      size)
-    : m_Conn(0),
+    : m_Conn(0), x_Connector(connector),
       m_WriteBuf(0), m_ReadBuf(&x_Buf), m_BufSize(1), m_Status(status),
       m_Tie(false), m_Close(true), m_CbValid(false), m_Initial(false),
       x_Buf(), x_GPos((CT_OFF_TYPE)(ptr ? size : 0)), x_PPos((CT_OFF_TYPE)size)
@@ -121,8 +130,24 @@ CConn_Streambuf::CConn_Streambuf(CONNECTOR                   connector,
         if (m_Status == eIO_Success)
             m_Status  = eIO_InvalidArg;
         ERR_POST_X(2, x_Message("CConn_Streambuf", "NULL connector"));
+        //NCBI_IO_CHECK(m_Status); /*NB: should be able to throw here*/
         return;
     }
+    //
+    // CAUTION!  May not throw from this point on in this ctor!
+    //
+    // This ctor must not throw from here on because otherwise it would either:
+    // a. leak memory allocated to the passed CONNECTOR (see the construction
+    //    of CConn_IOStream-based classes); or
+    // b. if CONNECTOR's destroy method was called here to cleanup, that would
+    //    have chained some of the CONNECTOR's cleanup callback(s) as well, yet
+    //    with the callback data (user_data), which might still be unreleased
+    //    by guard objects (e.g. smart pointers) that hold the ownership until
+    //    after the construction's completion;  thus, exception throwing would
+    //    make those guards attempt to destroy user_data for the second time!
+    //  So we just hold the error status and rely on CConn_IOStream to return a
+    //  bad stream;  and later to cleanup everything by a proper destructor.
+    //
     if (!(flgs & (CConn_IOStream::fConn_Untie |
                   CConn_IOStream::fConn_WriteUnbuffered))  &&  buf_size) {
         m_Tie = true;
@@ -133,8 +158,6 @@ CConn_Streambuf::CConn_Streambuf(CONNECTOR                   connector,
         != eIO_Success) {
         ERR_POST_X(3, x_Message("CConn_Streambuf", "CONN_Create() failed"));
         _ASSERT(!m_Conn  &&  !connector->meta  &&  !connector->next);
-        if (connector->destroy) // NB: cleanup callback (if any) must be valid!
-            connector->destroy(connector);
         return;
     }
     _ASSERT(m_Conn);
@@ -149,7 +172,7 @@ CConn_Streambuf::CConn_Streambuf(CONN                        conn,
                                  CConn_IOStream::TConn_Flags flgs,
                                  CT_CHAR_TYPE*               ptr,
                                  size_t                      size)
-    : m_Conn(conn),
+    : m_Conn(conn), x_Connector(0),
       m_WriteBuf(0), m_ReadBuf(&x_Buf), m_BufSize(1), m_Status(eIO_Success),
       m_Tie(false), m_Close(close), m_CbValid(false), m_Initial(false),
       x_Buf(), x_GPos((CT_OFF_TYPE)(ptr ? size : 0)), x_PPos((CT_OFF_TYPE)size)
@@ -157,6 +180,7 @@ CConn_Streambuf::CConn_Streambuf(CONN                        conn,
     if (!m_Conn) {
         m_Status = eIO_InvalidArg;
         ERR_POST_X(1, x_Message("CConn_Streambuf", "NULL connection"));
+        NCBI_IO_CHECK(m_Status);
         return;
     }
     if (!(flgs & (CConn_IOStream::fConn_Untie |
@@ -167,6 +191,19 @@ CConn_Streambuf::CConn_Streambuf(CONN                        conn,
 }
 
 
+CConn_Streambuf::~CConn_Streambuf()
+{
+    Close();
+
+    _ASSERT(!m_Conn  &&  !m_CbValid);
+
+    if (x_Connector  &&  x_Connector->destroy)
+        x_Connector->destroy(x_Connector);
+
+    delete[] m_WriteBuf;
+}
+
+
 EIO_Status CConn_Streambuf::Status(EIO_Event direction) const
 {
     return direction == eIO_Close ? m_Status
@@ -178,7 +215,7 @@ void CConn_Streambuf::x_Init(const STimeout* timeout, size_t buf_size,
                              CConn_IOStream::TConn_Flags flgs,
                              CT_CHAR_TYPE* ptr, size_t size)
 {
-    _ASSERT(m_Status == eIO_Success);
+    _ASSERT(m_Conn  &&  m_Status == eIO_Success);
 
     if (timeout != kDefaultTimeout) {
         _VERIFY(CONN_SetTimeout(m_Conn, eIO_Open,      timeout) ==eIO_Success);
@@ -192,19 +229,26 @@ void CConn_Streambuf::x_Init(const STimeout* timeout, size_t buf_size,
             CConn_IOStream::fConn_WriteUnbuffered)) {
         buf_size = 0;
     }
+    unique_ptr<CT_CHAR_TYPE[]> wbp;
     if (buf_size) {
-        m_WriteBuf = new
-            CT_CHAR_TYPE[buf_size
-                         << ((flgs & (CConn_IOStream::fConn_ReadUnbuffered |
-                                      CConn_IOStream::fConn_WriteUnbuffered))
-                             ? 0 : 1)];
+        wbp.reset
+            (new
+             CT_CHAR_TYPE[buf_size
+                          << ((flgs & (CConn_IOStream::fConn_ReadUnbuffered |
+                                       CConn_IOStream::fConn_WriteUnbuffered))
+                              ? 0 : 1)]);
+        CT_CHAR_TYPE* write_buf = wbp.get();
         if (!(flgs & CConn_IOStream::fConn_ReadUnbuffered))
-            m_BufSize =  buf_size;
+            m_BufSize = buf_size;
         if (  flgs & CConn_IOStream::fConn_WriteUnbuffered)
-            buf_size  =  0;
+            buf_size  = 0;
         if (!(flgs & CConn_IOStream::fConn_ReadUnbuffered))
-            m_ReadBuf =  m_WriteBuf + buf_size;
-        setp(m_WriteBuf, m_WriteBuf + buf_size);
+            m_ReadBuf = write_buf + buf_size;
+#if 0/*unnecessary*/
+        if (  flgs & CConn_IOStream::fConn_WriteUnbuffered)
+            write_buf = 0;
+#endif
+        setp(write_buf, write_buf + buf_size);
     }/* else
         setp(0, 0) */
 
@@ -222,18 +266,23 @@ void CConn_Streambuf::x_Init(const STimeout* timeout, size_t buf_size,
     m_CbValid = true;
 
     if (!(flgs & CConn_IOStream::fConn_DelayOpen)) {
-        SOCK s/*dummy*/;
+        SOCK unused;
         // NB: CONN_Write(0 bytes) could have caused the same effect
-        (void) CONN_GetSOCK(m_Conn, &s);  // Prompt CONN to actually open
+        (void) CONN_GetSOCK(m_Conn, &unused);  // Prompt CONN to actually open
         if ((m_Status = CONN_Status(m_Conn, eIO_Open)) != eIO_Success) {
-            ERR_POST_X(17, x_Message("CConn_Streambuf", "Failed to open",
-                                     m_Status, timeout));
+            ERR_POST_X(17, x_Message("CConn_Streambuf",
+                                     "Failed to open", m_Status, timeout));
+            if (!x_Connector)
+                NCBI_IO_CHECK(m_Status);
         }
     }
+
+    if (m_Status == eIO_Success)
+        m_WriteBuf = wbp.release();
 }
 
 
-EIO_Status CConn_Streambuf::x_Pushback(void)
+EIO_Status CConn_Streambuf::x_Pushback(void) THROWS_NONE
 {
     _ASSERT(m_Conn);
 
@@ -250,10 +299,11 @@ EIO_Status CConn_Streambuf::x_Pushback(void)
 
 EIO_Status CConn_Streambuf::x_Close(bool close)
 {
-    if (!m_Conn)
-        return close ? eIO_Closed : eIO_Success;
- 
+    _ASSERT(m_Conn);
+
     EIO_Status status = eIO_Success;
+    bool cb_valid = m_CbValid;
+    m_CbValid = false;
 
     // push any still unread data from the buffer back to the device
     if (!m_Close  &&  close  &&  !m_Initial) {
@@ -272,9 +322,9 @@ EIO_Status CConn_Streambuf::x_Close(bool close)
         if (x_status != eIO_Success) {
             status = m_Status = x_status;
             if (CONN_Status(m_Conn, eIO_Open) == eIO_Success) {
-                _TRACE(x_Message("Close",
-                                 "Cannot finalize implicitly"
-                                 ", data loss may result"));
+                _ALWAYS_TRACE(x_Message("Close",
+                                        "Cannot finalize implicitly"
+                                        ", data loss may result"));
             }
         } else {
             bool synced = false;
@@ -291,40 +341,56 @@ EIO_Status CConn_Streambuf::x_Close(bool close)
     }
     setp(0, 0);
 
-    CONN c = m_Conn;
+    CONN conn = m_Conn;
+    x_Connector = 0;
     m_Conn = 0;  // NB: no re-entry
 
     if (close) {
-        // here: not called from the close callback x_OnClose
-        if (m_CbValid) {
+        // Here when not called from the close callback x_OnClose
+        if (cb_valid) {
             SCONN_Callback cb;
-            CONN_SetCallback(c, eCONN_OnClose, &m_Cb, &cb);
-            if ((void*) cb.func != (void*) x_OnClose  ||  cb.data != this)
-                CONN_SetCallback(c, eCONN_OnClose, &cb, 0);
+            // Restore the original callback
+            CONN_SetCallback(conn, eCONN_OnClose, &m_Cb, &cb);
+            if (cb.func != x_OnClose  ||  cb.data != this) {
+                // Restore again if our callback was replaced
+                CONN_SetCallback(conn, eCONN_OnClose, &cb, 0);
+            }
         }
-        if (m_Close  &&  (m_Status = CONN_Close(c)) != eIO_Success) {
-            _TRACE(x_Message("Close",
-                             "CONN_Close() failed"));
-            if (status == eIO_Success)
-                status  = m_Status;
+        if (m_Close) {
+            STimeout        xtmo;
+            const STimeout* ctmo = CONN_GetTimeout(conn, eIO_Close);
+            if (ctmo != kInfiniteTimeout  &&  ctmo != kDefaultTimeout) {
+                xtmo  = *ctmo;
+                ctmo  = &xtmo;
+            }
+            if ((m_Status = CONN_Close(conn)) != eIO_Success) {
+                _ALWAYS_TRACE(x_Message("Close",
+                                        "CONN_Close() failed",
+                                        m_Status, ctmo));
+                if (status == eIO_Success)
+                    status  = m_Status;
+            }
         }
-    } else if (m_CbValid  &&  m_Cb.func) {
-        EIO_Status cbstat = m_Cb.func(c, eCONN_OnClose, m_Cb.data);
+    } else if (cb_valid  &&  m_Cb.func) {
+        // In x_OnClose callback here: upcall the original callback
+        EIO_Status cbstat = m_Cb.func(conn, eCONN_OnClose, m_Cb.data);
         if (cbstat != eIO_Success)
             status  = cbstat;
     }
+
     return status;
 }
 
 
+// x_OnClose() is called when CONN gets closed from outside this class
 EIO_Status CConn_Streambuf::x_OnClose(CONN           _DEBUG_ARG(conn),
                                       TCONN_Callback _DEBUG_ARG(type),
                                       void*          data)
 {
-    CConn_Streambuf* sb = static_cast<CConn_Streambuf*>(data);
+    CConn_Streambuf* sb = reinterpret_cast<CConn_Streambuf*>(data);
     _ASSERT(type == eCONN_OnClose  &&  sb  &&  conn);
     _ASSERT(!sb->m_Conn  ||  sb->m_Conn == conn);
-    return sb->x_Close(false);
+    return sb->m_Conn ? sb->x_Close(false) : eIO_Success;
 }
 
 
@@ -392,7 +458,9 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
         if (n_towrite) {
             _ASSERT(m_Status != eIO_Success);
             ERR_POST_X(4, x_Message("overflow",
-                                    "CONN_Write() failed"));
+                                    "CONN_Write() failed", m_Status,
+                                    m_Status != eIO_Timeout ? 0 :
+                                    CONN_GetTimeout(m_Conn, eIO_Write)));
             if (x_IsThrowable(m_Status))
                 NCBI_IO_CHECK(m_Status);
             return CT_EOF;
@@ -405,7 +473,9 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
         if (!n_written) {
             _ASSERT(m_Status != eIO_Success);
             ERR_POST_X(5, x_Message("overflow",
-                                    "CONN_Write(1) failed"));
+                                    "CONN_Write(1) failed", m_Status,
+                                    m_Status != eIO_Timeout ? 0 :
+                                    CONN_GetTimeout(m_Conn, eIO_Write)));
             if (x_IsThrowable(m_Status))
                 NCBI_IO_CHECK(m_Status);
             return CT_EOF;
@@ -417,7 +487,9 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
     _ASSERT(CT_EQ_INT_TYPE(c, CT_EOF));
     if ((m_Status = CONN_Flush(m_Conn)) != eIO_Success) {
         ERR_POST_X(9, x_Message("overflow",
-                                "CONN_Flush() failed"));
+                                "CONN_Flush() failed", m_Status,
+                                m_Status != eIO_Timeout ? 0 :
+                                CONN_GetTimeout(m_Conn, eIO_Write)));
         if (x_IsThrowable(m_Status))
             NCBI_IO_CHECK(m_Status);
         return CT_EOF;
@@ -463,7 +535,9 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
                 if (!x_written) {
                     _ASSERT(m_Status != eIO_Success);
                     ERR_POST_X(6, x_Message("xsputn",
-                                            "CONN_Write() failed"));
+                                            "CONN_Write() failed", m_Status,
+                                            m_Status != eIO_Timeout ? 0 :
+                                            CONN_GetTimeout(m_Conn, eIO_Write)));
                     break;
                 }
                 memmove(pbase(), pbase() + x_written, x_towrite - x_written);
@@ -479,7 +553,9 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
         if (!x_written  &&  n) {
             _ASSERT(m_Status != eIO_Success);
             ERR_POST_X(7, x_Message("xsputn",
-                                    "CONN_Write(direct) failed"));
+                                    "CONN_Write(direct) failed", m_Status,
+                                    m_Status != eIO_Timeout ? 0 :
+                                    CONN_GetTimeout(m_Conn, eIO_Write)));
             break;
         }
         x_PPos    += (CT_OFF_TYPE) x_written;
@@ -535,7 +611,9 @@ CT_INT_TYPE CConn_Streambuf::underflow(void)
         _ASSERT(m_Status != eIO_Success);
         if (m_Status != eIO_Closed) {
             ERR_POST_X(8, x_Message("underflow",
-                                    "CONN_Read() failed"));
+                                    "CONN_Read() failed", m_Status,
+                                    m_Status != eIO_Timeout ? 0 :
+                                    CONN_GetTimeout(m_Conn, eIO_Read)));
             if (x_IsThrowable(m_Status))
                 NCBI_IO_CHECK(m_Status);
         }
@@ -595,7 +673,9 @@ streamsize CConn_Streambuf::x_Read(CT_CHAR_TYPE* buf, streamsize m)
             _ASSERT(!x_toread  ||  m_Status != eIO_Success);
             if (m_Status != eIO_Success  &&  m_Status != eIO_Closed) {
                 ERR_POST_X(10, x_Message("xsgetn",
-                                         "CONN_Read() failed"));
+                                         "CONN_Read() failed", m_Status,
+                                         m_Status != eIO_Timeout ? 0 :
+                                         CONN_GetTimeout(m_Conn, eIO_Read)));
             }
             break;
         }
@@ -639,7 +719,7 @@ streamsize CConn_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 
 streamsize CConn_Streambuf::showmanyc(void)
 {
-    static const STimeout kZeroTmo = {0, 0};
+#define POLLING  CConn_IOStream::kZeroTimeout
 
     _ASSERT(gptr() >= egptr());
 
@@ -660,11 +740,11 @@ streamsize CConn_Streambuf::showmanyc(void)
         tmo = timeout;
 
     if (!tmo)
-        _VERIFY(CONN_SetTimeout(m_Conn, eIO_Read, &kZeroTmo) == eIO_Success);
+        _VERIFY(CONN_SetTimeout(m_Conn, eIO_Read, POLLING) == eIO_Success);
     size_t x_read;
     m_Status = CONN_Read(m_Conn, m_ReadBuf, m_BufSize, &x_read, eIO_ReadPlain);
     if (!tmo)
-        _VERIFY(CONN_SetTimeout(m_Conn, eIO_Read, timeout)   == eIO_Success);
+        _VERIFY(CONN_SetTimeout(m_Conn, eIO_Read, timeout) == eIO_Success);
     _ASSERT(x_read > 0  ||  m_Status != eIO_Success);
 
     if (!x_read) {
@@ -690,6 +770,8 @@ streamsize CConn_Streambuf::showmanyc(void)
     setg(m_ReadBuf, m_ReadBuf, m_ReadBuf + x_read);
     x_GPos += x_read;
     return x_read;
+
+#undef POLLING
 }
 
 
@@ -726,16 +808,22 @@ CT_POS_TYPE CConn_Streambuf::seekoff(CT_OFF_TYPE        off,
 }
 
 
-EIO_Status CConn_Streambuf::Pushback(const CT_CHAR_TYPE* data, streamsize size)
+EIO_Status CConn_Streambuf::Pushback(const CT_CHAR_TYPE* data,
+                                     streamsize          size,
+                                     bool                push)
 {
     if (!m_Conn)
         return eIO_Closed;
 
-    if ((!m_Initial  &&  (m_Status = x_Pushback()) != eIO_Success)
-        ||  (m_Status = CONN_Pushback(m_Conn, data, size)) != eIO_Success) {
+    m_Status = x_Pushback();
+    if (m_Status == eIO_Success  &&  size)
+        m_Status  = CONN_Pushback(m_Conn, data, size);
+    if (m_Status != eIO_Success) {
         ERR_POST_X(14, x_Message("Pushback",
                                  "CONN_Pushback() failed"));
-    }
+    } else if (push)
+        x_PPos += (CT_OFF_TYPE) size;
+        
     return m_Status;
 }
 
@@ -743,7 +831,7 @@ EIO_Status CConn_Streambuf::Pushback(const CT_CHAR_TYPE* data, streamsize size)
 EIO_Status CConn_Streambuf::Fetch(const STimeout* timeout)
 {
     if (!m_Conn)
-        return eIO_Closed;
+        return eIO_InvalidArg;
 
     if (timeout == kDefaultTimeout) {
         // HACK * HACK * HACK
@@ -773,9 +861,11 @@ EIO_Status CConn_Streambuf::Fetch(const STimeout* timeout)
 
         if (!synced) {
             _ASSERT(m_Status != eIO_Success);
-            ERR_POST_X(15, x_Message("Fetch",
-                                     "Failed to flush",
-                                     m_Status, timeout));
+            ERR_POST_X(15, (m_Status != eIO_Timeout  ||  !timeout  ||
+                            (timeout->sec | timeout->usec) ? Error : Trace)
+                       << x_Message("Fetch",
+                                    "Failed to flush",
+                                    m_Status, timeout));
         }
     }
 
@@ -786,7 +876,8 @@ EIO_Status CConn_Streambuf::Fetch(const STimeout* timeout)
     // now wait for some input
     EIO_Status status = CONN_Wait(m_Conn, eIO_Read, timeout);
     if (status != eIO_Success) {
-        ERR_POST_X(16, (status == eIO_Timeout ? Warning : Error)
+        ERR_POST_X(16, (status != eIO_Timeout  ||  !timeout ? Error :
+                        (timeout->sec | timeout->usec) ? Warning : Trace)
                    << x_Message("Fetch",
                                 "CONN_Wait() failed",
                                 status, timeout));
