@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_test.cpp 606555 2020-04-23 14:38:32Z lavr $
+/* $Id: ncbi_conn_test.cpp 654831 2022-08-22 21:20:43Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,13 +41,13 @@
 #include <iterator>
 #include <stdlib.h>
 
-#define _STR(x)           #x
-#define STRINGIFY(x)  _STR(x)
-
 #define HELP_EMAIL    (m_Email.empty()                                    \
                        ? string("NCBI Help Desk <info@ncbi.nlm.nih.gov>") \
                        : m_Email)
 
+#define NCBI_WWW       DEF_CONN_HOST
+#define NCBI_WWW_BEMD  "www.be-md.ncbi.nlm.nih.gov"
+#define NCBI_WWW_STVA  "www.st-va.ncbi.nlm.nih.gov"
 #define NCBI_FWD_BEMD  "130.14.29.112"
 #define NCBI_FWD_STVA  "165.112.7.12"
 
@@ -104,6 +104,7 @@ EIO_Status CConnTest::Execute(EStage& stage, string* reason)
     typedef EIO_Status (CConnTest::*FCheck)(string* reason);
     FCheck check[] = {
         NULL,
+        &CConnTest::DnsOkay,
         &CConnTest::HttpOkay,
         &CConnTest::DispatcherOkay,
         &CConnTest::ServiceOkay,
@@ -120,7 +121,7 @@ EIO_Status CConnTest::Execute(EStage& stage, string* reason)
         reason->clear();
     m_CheckPoint.clear();
 
-    int s = eHttp;
+    int s = eNone + 1;
     EIO_Status status;
     do {
         if ((status = (this->*check[s])(reason)) != eIO_Success) {
@@ -129,7 +130,7 @@ EIO_Status CConnTest::Execute(EStage& stage, string* reason)
         }
     } while (EStage(s++) < stage);
 
-    if (status != eIO_Success  &&  status != eIO_Interrupt)
+    if (stage != eDns  &&  status != eIO_Success  &&  status != eIO_Interrupt)
         ExtraCheckOnFailure();
     return status;
 }
@@ -193,11 +194,13 @@ static inline bool x_IsFatalError(int error)
 
 struct SAuxData {
     const ICanceled* m_Canceled;
+    bool             m_Redirect;
     bool             m_Failed;
     void*            m_Data;
 
     SAuxData(const ICanceled* canceled, void* data)
-        : m_Canceled(canceled), m_Failed(false), m_Data(data)
+        : m_Canceled(canceled), m_Redirect(false),
+          m_Failed(false), m_Data(data)
     { }
 };
 
@@ -221,7 +224,7 @@ static EHTTP_HeaderParse s_GoodHeader(const char* /*header*/,
     SAuxData* auxdata = reinterpret_cast<SAuxData*>(data);
     _ASSERT(auxdata);
     if (server_error)
-        auxdata->m_Failed = true;
+        auxdata->m_Failed = server_error != 301  &&  server_error != 302;
     return eHTTP_HeaderSuccess;
 }
 
@@ -241,10 +244,12 @@ static EHTTP_HeaderParse s_SvcHeader(const char* header,
 }
 
 
-static int/*bool*/ s_Adjust(SConnNetInfo*, void* data, unsigned int /*count*/)
+static int/*bool*/ s_Adjust(SConnNetInfo*, void* data, unsigned int count)
 {
     SAuxData* auxdata = reinterpret_cast<SAuxData*>(data);
     _ASSERT(auxdata);
+    if (!count/*redirect*/)
+        auxdata->m_Redirect = true;
     return auxdata->m_Failed
         ||  (auxdata->m_Canceled  &&  auxdata->m_Canceled->IsCanceled())
         ? 0/*false*/ : 1/*true*/;
@@ -280,22 +285,22 @@ EIO_Status CConnTest::ExtraCheckOnFailure(void)
     } kTests[] = {
         // 0. NCBI default
         { eURL_Http,
-          0,                            0                      }, // NCBI
+          0,                0        }, // NCBI
         // 1. External server(s)
         { eURL_Https,
-          "www.google.com",             0                      },
+          "www.google.com", 0        },
         // 2. NCBI servers, explicitly
         { eURL_Https,
-          "www.be-md.ncbi.nlm.nih.gov", "www.ncbi.nlm.nih.gov" }, // NCBI main
+          NCBI_WWW_BEMD,    NCBI_WWW }, // NCBI main
         { eURL_Https,
-          "www.st-va.ncbi.nlm.nih.gov", "www.ncbi.nlm.nih.gov" }  // NCBI colo
+          NCBI_WWW_STVA,    NCBI_WWW }  // NCBI colo
     };
 
     m_CheckPoint.clear();
     PreCheck(eNone, 0/*main*/, "Failback HTTP access check");
 
     AutoPtr<SConnNetInfo> net_info(ConnNetInfo_Create(0, eDebugPrintout_Data));
-    if (!net_info.get()) {
+    if (!net_info) {
         PostCheck(eNone, 0/*main*/,
                   eIO_Unknown, "Unable to create network info structure");
         return eIO_Unknown;
@@ -374,19 +379,40 @@ EIO_Status CConnTest::ExtraCheckOnFailure(void)
 }
 
 
+EIO_Status CConnTest::DnsOkay(string* reason)
+{
+    EIO_Status status;
+    string result;
+
+    PreCheck(eDns, 0/*main*/,
+             "Checking whether NCBI is known to DNS");
+
+    if (CSocketAPI::gethostbyname(NCBI_WWW) == 0) {
+        result = "Unable to resolve " NCBI_WWW;
+        status = eIO_Unknown;
+    } else {
+        result = "OK";
+        status = eIO_Success;
+    }
+
+    PostCheck(eDns, 0/*main*/, status, result);
+    return status;
+}
+
+
 EIO_Status CConnTest::HttpOkay(string* reason)
 {
     PreCheck(eHttp, 0/*main*/,
              "Checking whether NCBI is HTTP(S) accessible");
 
     AutoPtr<SConnNetInfo> net_info(ConnNetInfo_Create(0, m_DebugPrintout));
-    if (!net_info.get()) {
-        PostCheck(eNone, 0/*main*/,
+    if (!net_info) {
+        PostCheck(eHttp, 0/*main*/,
                   eIO_Unknown, "Unable to create network info structure");
         return eIO_Unknown;
     }
 
-    net_info->scheme = eURL_Https;
+    net_info->scheme = eURL_Http;
     if (net_info->http_proxy_host[0]  &&  net_info->http_proxy_port)
         m_HttpProxy = true;
     // Make sure there are no extras
@@ -396,7 +422,8 @@ EIO_Status CConnTest::HttpOkay(string* reason)
     SAuxData* auxdata = new SAuxData(m_Canceled, 0);
     CConn_HttpStream http("/Service/index.html",
                           net_info.get(), kEmptyStr/*user_hdr*/, s_GoodHeader,
-                          auxdata, s_Adjust, s_Cleanup, 0/*flags*/, m_Timeout);
+                          auxdata, s_Adjust, s_Cleanup, fHTTP_AdjustOnRedirect,
+                          m_Timeout);
     http.SetCanceledCallback(m_Canceled);
     string temp;
     http >> temp;
@@ -411,33 +438,47 @@ EIO_Status CConnTest::HttpOkay(string* reason)
             temp = x_TimeoutMsg();
         else
             temp.clear();
-        bool host = NStr::CompareNocase(net_info->host, DEF_CONN_HOST) != 0;
-        bool port = net_info->port != 0;
-        if (host  ||  port) {
-            int n = 0;
-            temp += "Make sure that ";
-            if (host) {
-                n++;
-                temp += "[CONN]HOST=\"";
-                temp += net_info->host;
-                temp += port ? "\"" : "\" and ";
+        if (auxdata->m_Redirect) {
+            // Redirect to https:// is expected
+            temp += "Your HTTP connection appears to work but it does not seem"
+                " to be able to upgrade to secure HTTP (HTTPS), most likely"
+                " because of a malfunctioning HTTP proxy server in the way\n";
+        } else {
+            bool host = NStr::CompareNocase(net_info->host, DEF_CONN_HOST) !=0;
+            bool port = net_info->port != 0;
+            if (host  ||  port) {
+                int n = 0;
+                temp += "Make sure that ";
+                if (host) {
+                    n++;
+                    temp += "[" DEF_CONN_REG_SECTION "]" REG_CONN_HOST "=\"";
+                    temp += net_info->host;
+                    temp += port ? "\"" : "\" and ";
+                }
+                if (port) {
+                    n++;
+                    temp += "[" DEF_CONN_REG_SECTION "]" REG_CONN_PORT "=\"";
+                    temp += NStr::UIntToString(net_info->port);
+                    temp += '"';
+                }
+                _ASSERT(n);
+                temp += n > 1 ? " are" : " is";
+                temp += " redefined correctly\n";
             }
-            if (port) {
-                n++;
-                temp += "[CONN]PORT=\"";
-                temp += NStr::UIntToString(net_info->port);
-                temp += '"';
-            }
-            _ASSERT(n);
-            temp += n > 1 ? " are" : " is";
-            temp += " redefined correctly\n";
         }
+        bool auth = true;
         if (m_HttpProxy) {
             temp += "Make sure that the HTTP proxy server \'";
             temp += net_info->http_proxy_host;
             temp += ':';
             temp += NStr::UIntToString(net_info->http_proxy_port);
-            temp += "' specified with [CONN]HTTP_PROXY_{HOST|PORT} is correct";
+            temp += "' specified with ";
+            if (net_info->http_proxy_only) {
+                temp += "environment variable $http_proxy or $HTTP_PROXY";
+                auth = false;
+            } else
+                temp += "[" DEF_CONN_REG_SECTION "]HTTP_PROXY_{HOST|PORT}";
+            temp += " is correct";
         } else {
             if (net_info->http_proxy_host[0]  ||  net_info->http_proxy_port) {
                 temp += "Note that your HTTP proxy seems to have been"
@@ -452,22 +493,25 @@ EIO_Status CConnTest::HttpOkay(string* reason)
                 temp += ")\n";
             }
             temp += "If your network access requires the use of an HTTP proxy"
-                " server, please contact your network administrator and set"
-                " [CONN]HTTP_PROXY_{HOST|PORT} (both must be set) in your"
-                " configuration accordingly";
+                " server, please contact your network administrator, and set"
+                " [" DEF_CONN_REG_SECTION "]{HTTP_PROXY_{HOST|PORT} (both must"
+                " be set) in your configuration accordingly";
         }
-        temp += "; and if your proxy server requires authorization, please"
-            " check that appropriate [CONN]HTTP_PROXY_{USER|PASS} have been"
-            " set\n";
+        if (auth) {
+            temp += "; and if your proxy server requires authorization, please"
+                " check that appropriate ["
+                DEF_CONN_REG_SECTION "]HTTP_PROXY_{USER|PASS} have been set\n";
+        } else
+            temp += '\n';
         if (*net_info->user  ||  *net_info->pass) {
-            temp += "Make sure there are no stray [CONN]{USER|PASS} that"
-                " appear in your configuration -- NCBI services neither"
-                " require nor use them\n";
+            temp += "Make sure there are no stray values for ["
+                DEF_CONN_REG_SECTION "]{" REG_CONN_USER "|" REG_CONN_PASS
+                "} in your configuration -- NCBI services neither require"
+                " nor use them\n";
         }
         if (status == eIO_NotSupported)
             temp += "Your application may also need to have SSL support on\n";
     }
-    net_info.reset();
 
     PostCheck(eHttp, 0/*main*/, status, temp);
 
@@ -507,8 +551,10 @@ EIO_Status CConnTest::DispatcherOkay(string* reason)
     else {
         if (status != eIO_Timeout) {
             if (okay) {
-                temp = "Make sure there are no stray [CONN]{HOST|PORT|PATH}"
-                    " settings in the way in your configuration\n";
+                temp = "Make sure there are no stray values for ["
+                    DEF_CONN_REG_SECTION "]{" REG_CONN_HOST "|"
+                    REG_CONN_PORT "|" REG_CONN_PATH "} in your"
+                    " configuration\n";
             }
             if (okay == 1) {
                 temp += "Service response was not recognized; please contact "
@@ -520,10 +566,9 @@ EIO_Status CConnTest::DispatcherOkay(string* reason)
             temp += "Check with your network administrator that your network"
                 " neither filters out nor blocks non-standard HTTP headers\n";
         }
-        if (net_info.get()  &&  status == eIO_NotSupported)
+        if (net_info  &&  status == eIO_NotSupported)
             temp += "NCBI network dispatcher must be accessed via HTTPS\n";
     }
-    net_info.reset();
 
     PostCheck(eDispatcher, 0/*main*/, status, temp);
 
@@ -542,7 +587,7 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
 
     AutoPtr<SConnNetInfo> net_info(ConnNetInfo_Create(kService,
                                                       m_DebugPrintout));
-    if (net_info.get())
+    if (net_info)
         net_info->lb_disable = 1/*no local LB to use even if available*/;
 
     CConn_ServiceStream svc(kService, fSERV_Stateless, net_info.get(),
@@ -560,12 +605,16 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
     else if (status == eIO_Success)
         temp = "OK";
     else {
-        char* str = net_info.get() ? SERV_ServiceName(kService) : 0;
+        char* str = net_info ? SERV_ServiceName(kService) : 0;
         if (str  &&  NStr::CompareNocase(str, kService) == 0) {
             free(str);
             str = 0;
         }
+        string mapper;
         SERV_ITER iter = SERV_OpenSimple(kService);
+        const char* x_mapper = SERV_MapperName(iter);
+        if (x_mapper)
+            mapper = x_mapper;
         if (!iter  ||  !SERV_GetNextInfo(iter)) {
             // Service not found
             SERV_Close(iter);
@@ -591,18 +640,23 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
                 temp += "; please remove [";
                 string upper(kService);
                 temp += NStr::ToUpper(upper);
-                temp += "]CONN_SERVICE_NAME=\"";
+                temp += "]" REG_CONN_SERVICE_NAME "=\"";
                 temp += str;
                 temp += "\" from your configuration\n";
             } else if (status != eIO_Timeout  ||  x_Large(m_Timeout))
                 temp += "; please contact " + HELP_EMAIL + '\n';
         }
         if (status != eIO_Timeout) {
-            const char* mapper = SERV_MapperName(iter);
-            if (!mapper  ||  NStr::CompareNocase(mapper, "DISPD") != 0) {
+            x_mapper = SERV_MapperName(iter);
+            if (!x_mapper  ||  NStr::CompareNocase(x_mapper, "DISPD") != 0) {
                 temp += "Network dispatcher is not enabled as a service"
                     " locator;  please review your configuration to purge any"
-                    " occurrences of [CONN]DISPD_DISABLE off your settings\n";
+                    " occurrences of [" DEF_CONN_REG_SECTION "]"
+                    REG_CONN_DISPD_DISABLE " from your settings\n";
+                if (!mapper.empty()  &&  mapper != (x_mapper ? x_mapper : "")){
+                    temp += "It appears that \"" + mapper + "\" is currently"
+                        " set as your default service locator\n";
+                }
             }
         } else
             temp += x_TimeoutMsg();
@@ -610,7 +664,6 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
         if (str)
             free(str);
     }
-    net_info.reset();
 
     PostCheck(eStatelessService, 0/*main*/, status, temp);
 
@@ -701,7 +754,8 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
     if (m_Firewall) {
         temp += "This mode requires your firewall to be configured in such a"
             " way that it allows outbound connections to the port range ["
-            STRINGIFY(CONN_FWD_PORT_MIN) ".." STRINGIFY(CONN_FWD_PORT_MAX)
+            NCBI_AS_STRING(CONN_FWD_PORT_MIN) ".."
+            NCBI_AS_STRING(CONN_FWD_PORT_MAX)
             "] (inclusive) at the two fixed NCBI addresses, "
             NCBI_FWD_BEMD " and " NCBI_FWD_STVA ".\n"
             "To set that up correctly, please have your network administrator"
@@ -715,11 +769,12 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             " blocking had not been an issue\n";
     }
     if (m_Firewall) {
-        _ASSERT(net_info.get());
+        _ASSERT(net_info);
         switch (net_info->firewall) {
         case eFWMode_Adaptive:
             temp += "Also, there are usually a few additional ports such as "
-                STRINGIFY(CONN_PORT_SSH) " and " STRINGIFY(CONN_PORT_HTTPS)
+                NCBI_AS_STRING(CONN_PORT_SSH) " and "
+                NCBI_AS_STRING(CONN_PORT_HTTPS)
                 " at " NCBI_FWD_BEMD ", which can be used if connections to"
                 " the ports in the range described above, have failed\n";
             break;
@@ -730,7 +785,8 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             break;
         case eFWMode_Fallback:
             temp += "There are usually a few backup connection ports such as "
-                STRINGIFY(CONN_PORT_SSH) " and " STRINGIFY(CONN_PORT_HTTPS)
+                NCBI_AS_STRING(CONN_PORT_SSH) " and "
+                NCBI_AS_STRING(CONN_PORT_HTTPS)
                 " at " NCBI_FWD_BEMD ", which can be used as a failover if"
                 " connections to the port range above fail.  However, your "
                 " configuration explicitly requests that only those fallback"
@@ -748,7 +804,7 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             " firewall imposing a fine-grained control over which hosts and"
             " ports the outbound connections are allowed to use\n";
     }
-    if (m_HttpProxy  &&  net_info.get()) {
+    if (m_HttpProxy  &&  net_info  &&  !net_info->http_proxy_only) {
         temp += "Connections to the aforementioned ports will be made via an"
             " HTTP proxy at '";
         temp += net_info->http_proxy_host;
@@ -820,13 +876,11 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
 
     PostCheck(eFirewallConnPoints, 1/*sub*/, status, temp);
 
-    net_info.reset();
-
     if (status == eIO_Success) {
         PreCheck(eFirewallConnPoints, 2/*sub*/,
                  "Verifying configuration for consistency");
 
-        temp.resize(2);
+        temp.resize(2);//"OK"
         bool firewall = true;
         // Only check primary ports here
         ITERATE(vector<CConnTest::CFWConnPoint>, cp, m_Fwd) {
@@ -875,7 +929,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
     AutoPtr<SConnNetInfo> net_info(ConnNetInfo_Create(0, m_DebugPrintout));
 
     string temp("Checking individual connection points..\n");
-    if (net_info.get()) {
+    if (net_info) {
         if (net_info->firewall != eFWMode_Fallback) {
             temp += "NOTE that even though that not the entire port range may"
                 " currently be utilized and checked, in order for NCBI"
@@ -902,8 +956,8 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
     size_t n;
     bool url = false;
     unsigned int m = 0;
-    EIO_Status status = net_info.get() ? eIO_Unknown : eIO_InvalidArg;
-    for (n = 0;  net_info.get()  &&  n < sizeof(fwd) / sizeof(fwd[0]);  ++n) {
+    EIO_Status status = net_info ? eIO_Unknown : eIO_InvalidArg;
+    for (n = 0;  net_info  &&  n < sizeof(fwd) / sizeof(fwd[0]);  ++n) {
         if (fwd[n]->empty())
             continue;
         status = eIO_Success;
@@ -980,7 +1034,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
             }
             if (status != eIO_Success)
                 break;
-            vector<CSocketAPI::SPoll>  poll;
+            vector<CSocketAPI::SPoll> poll;
             ITERATE(vector<CFWCheck>, ck, fwck) {
                 CConn_IOStream* fw = ck->first.get();
                 if (!fw  ||  ck->second->status != eIO_Success)
@@ -1034,7 +1088,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
 #else
                 k = distance(fwd[n]->begin(), cp);
 #endif // NCBI_COMPILER_WORKSHOP
-                if (!fwck[k].first.get())
+                if (!fwck[k].first)
                     m_CheckPoint = "Connection closed";
                 else
                     m_CheckPoint = "Connection refused";
@@ -1064,7 +1118,8 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                     temp = x_TimeoutMsg();
                 else
                     temp.clear();
-                if (m_HttpProxy) {
+                if (m_HttpProxy  &&  !(net_info->http_proxy_only |
+                                       net_info->http_proxy_leak)) {
                     temp += "Your HTTP proxy '";
                     temp += net_info->http_proxy_host;
                     temp += ':';
@@ -1090,9 +1145,9 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                     }
                     temp += '\n';
                 } else {
-                    temp += " try setting [CONN]FIREWALL=1 in your"
-                        " configuration to use a more narrow"
-                        " port range";
+                    temp += " try setting [" DEF_CONN_REG_SECTION "]"
+                        REG_CONN_FIREWALL "=1 in your configuration to use a"
+                        " more narrow port range";
                     if (!url) {
                         temp += " per: " CONN_FWD_URL;
                         url = true;
@@ -1104,7 +1159,8 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                         " over dedicated (persistent) connections; some can"
                         " still work (at the cost of degraded performance)"
                         " over a stateless carrier such as HTTP;  try setting"
-                        " [CONN]STATELESS=1 in your configuration\n";
+                        " [" DEF_CONN_REG_SECTION "]" REG_CONN_STATELESS "=1"
+                        " in your configuration\n";
                 }
             } else
                 temp = "Now checking fallback port(s)...";
@@ -1150,7 +1206,6 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
 
     PostCheck(eFirewallConnections, 0/*main*/, status, temp);
 
-    net_info.reset();
     if (reason)
         reason->swap(temp);
     return status;
@@ -1236,7 +1291,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                 " please remove [";
             string upper(kEcho);
             temp += NStr::ToUpper(upper);
-            temp += "]CONN_SERVICE_NAME=\"";
+            temp += "]" REG_CONN_SERVICE_NAME "=\"";
             temp += str;
             temp += "\" from your configuration\n";
             free(str); // NB: still, str != NULL
@@ -1252,16 +1307,17 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                 }
                 temp += x_TimeoutMsg();
             }
-            if (m_Stateless  ||  (net_info.get()  &&  net_info->stateless)) {
+            if (m_Stateless  ||  (net_info  &&  net_info->stateless)) {
                 temp += "STATELESS mode forced by your configuration may be"
                     " preventing this stateful service from operating"
                     " properly; try to remove [";
                 if (!m_Stateless) {
                     string upper(kEcho);
                     temp += NStr::ToUpper(upper);
-                    temp += "]CONN_STATELESS\n";
+                    temp += "]"
+                        DEF_CONN_REG_SECTION "_" REG_CONN_STATELESS "\n";
                 } else
-                    temp += "CONN]STATELESS\n";
+                    temp += DEF_CONN_REG_SECTION "]" REG_CONN_STATELESS "\n";
             } else if (!str) {
                 SERV_ITER iter = 0;
                 if (status != eIO_Timeout
@@ -1269,7 +1325,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                          ||  !SERV_GetNextInfo(iter))) {
                     temp += "The service is currently unavailable;"
                         " you may want to contact " + HELP_EMAIL + '\n';
-                } else if (m_Fwd.empty()  &&  net_info.get()
+                } else if (m_Fwd.empty()  &&  net_info
                            &&  net_info->firewall != eFWMode_Fallback) {
                     temp += "The most likely reason for the failure is that"
                         " your firewall is still blocking ports as reported"
@@ -1279,18 +1335,23 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                 SERV_Close(iter);
             }
         } else if (!str) {
-            if (n  &&  net_info.get()  &&  net_info->http_proxy_port
-                &&  NStr::strncasecmp(recv, kFWSign, (size_t) n) == 0) {
+            if (n  &&  NStr::strncasecmp(recv, kFWSign, (size_t) n) == 0) {
                 temp += "NCBI Firewall";
                 if (!net_info->firewall)
                     temp += " (Connection Relay)";
-                temp += " Daemon reports negotitation error, which usually"
-                    " means that an intermediate HTTP proxy '";
-                temp += net_info->http_proxy_host;
-                temp += ':';
-                temp += NStr::UIntToString(net_info->http_proxy_port);
-                temp += "' may be buggy."
-                    " Please contact your network administrator\n";
+                temp += " Daemon reports negotitation error";
+                if (net_info  &&  m_HttpProxy
+                    &&  !(net_info->http_proxy_only |
+                          net_info->http_proxy_leak)) {
+                    temp += ", which usually means that an intermediate HTTP"
+                        " proxy '";
+                    temp += net_info->http_proxy_host;
+                    temp += ':';
+                    temp += NStr::UIntToString(net_info->http_proxy_port);
+                    temp += "' may be buggy.";
+                } else
+                    temp += '.';
+                temp += " Please contact your network administrator\n";
             } else {
                 temp += n ? "Unrecognized" : "No";
                 temp += " response from service;"
@@ -1298,7 +1359,6 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
             }
         }
     }
-    net_info.reset();
 
     PostCheck(eStatefulService, 0/*main*/, status, temp);
 
