@@ -1,4 +1,4 @@
-/* $Id: ncbi_core_cxx.cpp 624723 2021-02-03 18:51:43Z ivanov $
+/* $Id: ncbi_core_cxx.cpp 668369 2023-06-03 01:48:35Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,7 +35,6 @@
 
 #include <ncbi_pch.hpp>
 #include "ncbi_ansi_ext.h"
-#include "ncbi_priv.h"
 #include "ncbi_socketp.h"
 #include <corelib/ncbiapp.hpp>
 #include <corelib/ncbidbg.hpp>
@@ -44,8 +43,13 @@
 #include <connect/error_codes.hpp>
 #include <connect/ncbi_core_cxx.hpp>
 #include <connect/ncbi_monkey.hpp>
+#include <connect/ncbi_util.h>
 #include <common/ncbi_sanitizers.h>
 
+#ifdef NCBI_POSIX_THREADS
+#include <pthread.h>
+#endif // NCBI_POSIX_THREADS
+
 #define NCBI_USE_ERRCODE_X   Connect_Core
 
 
@@ -289,7 +293,7 @@ static void s_LOG_Handler(void*             /*data*/,
             level = eDiag_Fatal;
             break;
         }
-        if (!IsVisibleDiagPostLevel(level))
+        if (level != eDiag_Fatal  &&  !IsVisibleDiagPostLevel(level))
             return;
 
         CDiagCompileInfo info(mess->file,
@@ -300,16 +304,30 @@ static void s_LOG_Handler(void*             /*data*/,
         diag.SetErrorCode(mess->err_code, mess->err_subcode);
         diag << mess->message;
         if (mess->raw_size) {
-            diag <<
-                "\n#################### [BEGIN] Raw Data (" <<
-                mess->raw_size <<
-                " byte" << (mess->raw_size != 1 ? "s" : "") << ")\n" <<
-                NStr::PrintableString
-                (CTempString(static_cast<const char*>(mess->raw_data),
-                             mess->raw_size),
-                 NStr::fNewLine_Passthru | NStr::fNonAscii_Quote) <<
-                "\n#################### [END] Raw Data";
+            typedef AutoPtr< char, ArrayDeleter<char> >  TTempCharPtr;
+            TTempCharPtr buf;
+            CTempString data;
+            if (mess->raw_data) {
+                buf.reset(new char[UTIL_PrintableStringSize
+                                   ((const char*)
+                                    mess->raw_data,
+                                    mess->raw_size)]);
+                char* end = UTIL_PrintableStringEx((const char*)
+                                                   mess->raw_data,
+                                                   mess->raw_size,
+                                                   buf.get(), 0/*flags*/,
+                                                   UTIL_PRINTABLE_WIDTH);
+                data.assign(buf.get(), (size_t)(end - buf.get()));
+            }
+            diag << "\n#################### [BEGIN] Raw Data ("
+                 << mess->raw_size
+                 << " byte" << &"s"[!(mess->raw_size != 1)] << "):"
+                 << CTempString(data.empty() ? " <NULL>" : "\n") << data
+                 << "\n#################### [_END_] Raw Data";
         }
+        diag << Endm;
+        if (level == eDiag_Fatal)
+            Abort();
     }
     NCBI_CATCH_ALL_X(4, "s_LOG_Handler(" + x_Log(mess->level) + ") failed");
 }
@@ -434,6 +452,21 @@ static const char* s_GetAppName(void)
 }
 
 
+/***********************************************************************
+ *                               Referer                               *
+ ***********************************************************************/
+
+
+extern "C" {
+static const char* s_GetReferer(void)
+{
+    const string& referer
+        = CDiagContext::GetRequestContext().GetProperty("SELF_URL");
+    return referer.empty() ? 0 : strdup(referer.c_str());
+}
+}
+
+
 /***********************************************************************
  *                           NCBI Request ID                           *
  ***********************************************************************/
@@ -578,12 +611,27 @@ static void s_SetMonkeyHooks(EMonkeyHookSwitch hook_switch)
 #endif //NCBI_MONKEY
 
 
-static volatile enum EConnectInit {
+extern "C" {
+static void x_PreFork(void)
+{
+    CORE_LOCK_WRITE;
+}
+
+static void x_PostFork(void)
+{
+    CORE_UNLOCK;
+}
+} // extern "C"
+
+
+enum EConnectInit {
     eConnectInit_Weak     = -1,  ///< CConn_Initer
     eConnectInit_Intact   =  0,  ///< Not yet visited
     eConnectInit_Strong   =  1,  ///< User init detected
     eConnectInit_Explicit =  2   ///< CONNECT_Init() called
-} s_ConnectInit = eConnectInit_Intact;
+};
+
+static atomic<EConnectInit> s_ConnectInit = eConnectInit_Intact;
 
 
 /***********************************************************************
@@ -603,7 +651,7 @@ static void s_Fini(void) THROWS_NONE
     if (s_CORE_Set & eCORE_SetLOG)
         CORE_SetLOG(0);
     if (s_CORE_Set & eCORE_SetLOCK)
-        CORE_SetLOCK(&g_CORE_MT_Lock_default);
+        CORE_SetLOCK(0);
     g_CORE_Set &= ~s_CORE_Set;
     s_CORE_Set  =  0;
 }
@@ -624,43 +672,65 @@ static void s_Init(const IRWRegistry* reg  = 0,
                    TConnectInitFlags  flag = 0,
                    EConnectInit       how  = eConnectInit_Weak)
 {
-    _TRACE("CONNECT::s_Init("
-           + NStr::PtrToString(reg)                         + ", "
-           + NStr::PtrToString((void*) ssl)                 + "(), "
-           + NStr::PtrToString(lock)                        + ", 0x"
-           + NStr::UIntToString((unsigned int) flag, 0, 16) + ", "
+    _TRACE("CONNECT::s_Init(reg="
+           + NStr::PtrToString(reg)                         + ", ssl="
+           + NStr::PtrToString((void*) ssl)                 + "(), lock="
+           + NStr::PtrToString(lock)                        + ", flag=0x"
+           + NStr::UIntToString((unsigned int) flag, 0, 16) + ", how="
            + NStr::IntToString(int(how))                    + ')');
     _ASSERT(how != eConnectInit_Intact);
 
-    TCORE_Set set = 0;
+    auto connect_init = s_ConnectInit.load(memory_order_acquire);
+    if (connect_init == how  &&  how == eConnectInit_Explicit)
+        ERR_POST_X(11, "CONNECT_Init() called more than once");
+
+    TCORE_Set x_set = 0;
     if (!(g_CORE_Set & eCORE_SetLOCK)) {
         NCBI_LSAN_DISABLE_GUARD;  
         CORE_SetLOCK(MT_LOCK_cxx2c(lock, !!(flag & eConnectInit_OwnLock)));
-        set |= eCORE_SetLOCK;
+        x_set |= eCORE_SetLOCK;
     }
     if (!(g_CORE_Set & eCORE_SetLOG)) {
         CORE_SetLOG(LOG_cxx2c());
-        set |= eCORE_SetLOG;
+        x_set |= eCORE_SetLOG;
     }
     if (!(g_CORE_Set & eCORE_SetREG)) {
         CORE_SetREG(REG_cxx2c(reg, !!(flag & eConnectInit_OwnRegistry)));
-        set |= eCORE_SetREG;
+        x_set |= eCORE_SetREG;
     }
     if (!(g_CORE_Set & eCORE_SetSSL)) {
-        SOCK_SetupSSLInternal(ssl, 1/*init*/);
-        set |= ssl ? eCORE_SetSSL : 0;
+        EIO_Status status = SOCK_SetupSSLInternalEx(ssl, 1/*init*/);
+        if (status != eIO_Success) {
+            ERR_POST_X(10, Critical << "Failed to initialize SSL: "
+                       << IO_StatusStr(status));
+        }
+        if (ssl)
+            x_set |= eCORE_SetSSL;
     }
-    g_CORE_Set &= ~set;
-    s_CORE_Set |=  set;
+    g_CORE_Set &= ~x_set;
+    s_CORE_Set |=  x_set;
 
-    if (s_ConnectInit == eConnectInit_Intact) {
+    if (connect_init == eConnectInit_Intact) {
+        int err = 0;
         g_NCBI_ConnectRandomSeed
             = (unsigned int) time(0) ^ NCBI_CONNECT_SRAND_ADDEND;
         srand(g_NCBI_ConnectRandomSeed);
-        atexit(s_Fini);
+        if (x_set  &&  atexit(s_Fini) != 0)
+            err |= 1;
+#ifdef NCBI_POSIX_THREADS
+        if (pthread_atfork(x_PreFork, x_PostFork, x_PostFork) != 0)
+            err |= 2;
+#endif // NCBI_POSIX_THREADS
+        if (err) {
+            static const char* what[] = {0, "exit", "fork", "exit/fork"};
+            ERR_POST_X(9, Critical
+                       << "Failed to register "
+                       << what[err] << " handler" << &"s"[!(err == 3)]);
+        }
     }
 
     g_CORE_GetAppName     = s_GetAppName;
+    g_CORE_GetReferer     = s_GetReferer;
     g_CORE_GetRequestID   = s_GetRequestID;
     g_CORE_GetRequestDtab = s_GetRequestDTab;
 
@@ -671,7 +741,10 @@ static void s_Init(const IRWRegistry* reg  = 0,
     CMonkey::Instance();
 #endif //NCBI_MONKEY
 
-    s_ConnectInit = g_CORE_Set ? eConnectInit_Strong : how;
+    if (how < eConnectInit_Strong  &&  g_CORE_Set)
+        how = eConnectInit_Strong;
+    if (connect_init < how  ||  connect_init == eConnectInit_Intact)
+        s_ConnectInit.store(how, memory_order_release);
 }
 
 
@@ -682,10 +755,10 @@ extern void CONNECT_Init(const IRWRegistry* reg,
                          FSSLSetup          ssl)
 {
     CFastMutexGuard guard(s_ConnectInitMutex);
-    _TRACE("CONNECT_Init("
-           + NStr::PtrToString(reg)                         + ", "
-           + NStr::PtrToString(lock)                        + ", 0x"
-           + NStr::UIntToString((unsigned int) flag, 0, 16) + ", "
+    _TRACE("CONNECT_Init(reg="
+           + NStr::PtrToString(reg)                         + ", lock="
+           + NStr::PtrToString(lock)                        + ", flag=0x"
+           + NStr::UIntToString((unsigned int) flag, 0, 16) + ", ssl="
            + NStr::PtrToString((void*) ssl)                 + "())");
     try {
         g_CORE_Set = 0;
@@ -699,13 +772,16 @@ extern void CONNECT_Init(const IRWRegistry* reg,
 
 CConnIniter::CConnIniter(void)
 {
-    if (s_ConnectInit != eConnectInit_Intact)
+    auto connect_init = s_ConnectInit.load(memory_order_relaxed);
+    if (connect_init != eConnectInit_Intact)
         return;
     CFastMutexGuard guard(s_ConnectInitMutex);
     try {
-        if (s_ConnectInit == eConnectInit_Intact) {
+        connect_init = s_ConnectInit.load(memory_order_acquire);
+        if (connect_init == eConnectInit_Intact) {
             CNcbiApplicationGuard app = CNcbiApplication::InstanceGuard();
-            s_Init(app ? &app->GetConfig() : 0, NcbiSetupTls);
+            s_Init(app ? &app->GetConfig() : 0, NcbiSetupTls,
+                   0, eConnectInit_OwnRegistry);
         }
     }
     NCBI_CATCH_ALL_X(7, "CConn_Initer::CConn_Initer() failed");
