@@ -1,4 +1,4 @@
-/* $Id: ncbi_ftp_connector.c 610782 2020-06-23 20:07:07Z lavr $
+/* $Id: ncbi_ftp_connector.c 662326 2023-01-27 16:15:31Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -89,7 +89,7 @@ typedef struct {
     const SConnNetInfo*   info;    /* connection parameters                  */
     unsigned              sync:1;  /* true when last cmd acked (cntl synced) */
     unsigned              send:1;  /* true when in send mode (STOR/APPE)     */
-    unsigned              open:1;  /* true when data open ok in send mode    */
+    unsigned              open:1;  /* true when data opened ok in send mode  */
     unsigned              rclr:1;  /* true when "rest" to clear by next cmd  */
     unsigned              abor:1;  /* last cmd was ABOR (proftpd bug w/450)  */
     unsigned              free:1;  /* unused                                 */
@@ -112,7 +112,7 @@ typedef struct {
 
 static const STimeout kFailsafeTimeout = { 10, 0 };
 static const STimeout kZeroTimeout     = {  0, 0 };
-static const char     kDigits[] = "0123456789";
+static const char     kDigits[]        = "0123456789";
 
 
 typedef EIO_Status (*FFTPReplyCB)(SFTPConnector* xxx,  int   code,
@@ -184,10 +184,10 @@ static EIO_Status x_FTPCloseData(SFTPConnector* xxx,
     } else {
         if (!xxx->cntl) {
             how = eIO_Open;
-        } else if (xxx->what  &&  how != eIO_Close) {
+        } else if (how != eIO_Close) {
             CORE_LOGF_X(1, xxx->send ? eLOG_Error : eLOG_Warning,
-                        ("[FTP; %s]  Data connection transfer aborted",
-                         xxx->what));
+                        ("[FTP%s%s]  Data connection transfer aborted",
+                         xxx->what ? ": " : "", xxx->what ? xxx->what : ""));
         }
         if (how != eIO_Close) {
             status = SOCK_Abort(xxx->data);
@@ -204,6 +204,26 @@ static EIO_Status x_FTPCloseData(SFTPConnector* xxx,
 }
 
 
+static void x_FTPCloseCntl(SFTPConnector* xxx, const char* abort)
+{
+    SOCK cntl = xxx->cntl;
+    xxx->cntl = 0;
+    if (abort) {
+        CORE_LOGF_X(10, eLOG_Error,
+                    ("[FTP%s%s]  Lost connection to %s:%hu (%s)",
+                     xxx->what ? "; " : "", xxx->what ? xxx->what : "",
+                     xxx->info->host, xxx->info->port, abort));
+    }
+    if (xxx->data)
+        x_FTPCloseData(xxx, eIO_Close/*silent close*/, 0);
+    if (abort)
+        SOCK_Abort(cntl);
+    else
+        SOCK_SetTimeout(cntl, eIO_Close, &kZeroTimeout);
+    SOCK_Close(cntl);
+}
+
+
 static EIO_Status x_FTPParseReply(SFTPConnector* xxx, int* code,
                                   char* line, size_t maxlinelen,
                                   FFTPReplyCB replycb)
@@ -223,15 +243,16 @@ static EIO_Status x_FTPParseReply(SFTPConnector* xxx, int* code,
         /* all FTP replies are at least '\n'-terminated, not ending with EOF */
         rdstat = SOCK_ReadLine(xxx->cntl, buf, sizeof(buf), &len);
         if (rdstat != eIO_Success) {
-            status  = rdstat;
+            status = SOCK_Status(xxx->cntl, eIO_Read) == eIO_Closed
+                ? eIO_Closed : rdstat;
             break;
         }
         if (len == sizeof(buf)) {
-            status = eIO_Unknown/*line too long*/;
+            status = eIO_NotSupported/*line too long*/;
             break;
         }
         msg = buf;
-        if (!lineno  ||  isdigit((unsigned char)(*buf))) {
+        if (!lineno  ||  strchr(kDigits, (unsigned char)(*buf))) {
             if (sscanf(buf, "%d%n", &c, &m) < 1  ||  m != 3  ||  !c
                 ||  (buf[m]  &&  buf[m] != ' '  &&  buf[m] != '-')
                 ||  (lineno  &&  c != *code)) {
@@ -289,23 +310,8 @@ static EIO_Status s_FTPReply(SFTPConnector* xxx, int* code,
             sprintf(reason, "code %d", c);
         } else
             strncpy0(reason, IO_StatusStr(status), sizeof(reason) - 1);
-        if (status == eIO_Closed   ||  c == 221) {
-            SOCK cntl = xxx->cntl;
-            xxx->cntl = 0;
-            if (status == eIO_Closed) {
-                CORE_LOGF_X(10, eLOG_Error,
-                            ("[FTP%s%s]  Lost connection to %s:%hu (%s)",
-                             xxx->what ? "; " : "", xxx->what ? xxx->what : "",
-                             xxx->info->host, xxx->info->port, reason));
-            }
-            if (xxx->data)
-                x_FTPCloseData(xxx, eIO_Close/*silent close*/, 0);
-            if (status == eIO_Closed)
-                SOCK_Abort(cntl);
-            else
-                SOCK_SetTimeout(cntl, eIO_Close, &kZeroTimeout);
-            SOCK_Close(cntl);
-        }
+        if (status == eIO_Closed   ||  c == 221)
+            x_FTPCloseCntl(xxx, status == eIO_Closed ? reason : 0);
         if (status == eIO_Success  &&  c == 530/*not logged in*/)
             status  = eIO_Closed;
     } else
@@ -316,7 +322,9 @@ static EIO_Status s_FTPReply(SFTPConnector* xxx, int* code,
 }
 
 
-static EIO_Status s_FTPDrainReply(SFTPConnector* xxx, int* code, int cXX)
+/* Read replies while successful until the received response is: ether == *code
+ * or its hundred's place == cXX;  return the response code via the pointer. */
+static EIO_Status x_FTPDrainReply(SFTPConnector* xxx, int* code, int cXX)
 {
     int        c;
     EIO_Status status;
@@ -394,8 +402,8 @@ static EIO_Status s_FTPCommandEx(SFTPConnector* xxx,
 static const char* x_4Word(const char* line, const char word[4+1])
 {
     const char* s = strstr(line, word);
-    return !s  ||  ((s == line  ||  isspace((unsigned char) s[-1]))
-                    &&  !isalpha((unsigned char) s[4])) ? s : 0;
+    return s  &&  ((s == line  ||  isspace((unsigned char) s[-1]))
+                   &&  !isalpha((unsigned char) s[4])) ? s : 0;
 }
 
 
@@ -486,11 +494,14 @@ static EIO_Status x_FTPHelp(SFTPConnector* xxx)
     if (status != eIO_Success)
         return status;
     feat = xxx->feat;
+    xxx->feat = 0;
     status = s_FTPReply(xxx, &code, 0, 0, x_FTPHelpCB);
     if (status != eIO_Success  ||  (code != 211  &&  code != 214)) {
         xxx->feat = feat;
         return status != eIO_Success ? status : eIO_NotSupported;
     }
+    CORE_TRACEF(("FTP features parsed from HELP: 0x%04hX", xxx->feat));
+    xxx->feat |= feat;
     return eIO_Success;
 }
 
@@ -537,11 +548,14 @@ static EIO_Status x_FTPFeat(SFTPConnector* xxx)
     if (status != eIO_Success)
         return status;
     feat = xxx->feat;
+    xxx->feat = 0;
     status = s_FTPReply(xxx, &code, 0, 0, x_FTPFeatCB);
     if (status != eIO_Success  ||  code != 211) {
         xxx->feat = feat;
         return status != eIO_Success ? status : eIO_NotSupported;
     }
+    CORE_TRACEF(("FTP features parsed from FEAT: 0x%04hX", xxx->feat));
+    xxx->feat |= feat;
     return eIO_Success;
 }
 
@@ -606,10 +620,8 @@ static EIO_Status x_FTPLogin(SFTPConnector* xxx)
         status = s_FTPReply(xxx, &code, 0, 0, 0);
         if (status != eIO_Success)
             return status;
-        if (code == 503)
-            return eIO_Closed;
         if (code != 230  &&  code != 202)
-            return code == 332 ? eIO_NotSupported : eIO_Unknown;
+            return code == 332 ? eIO_NotSupported : eIO_Unknown/*incl 503*/;
     }
     status = x_FTPFeatures(xxx);
     if (status != eIO_Success)
@@ -732,7 +744,7 @@ static EIO_Status x_FTPDir(SFTPConnector* xxx,
     } else if (toupper((unsigned char)(*cmd)) == 'C') { /* [X]CWD, CDUP/XCUP */
         if (code != 200  &&  code != 250)
             return eIO_Unknown;
-        /* fixup codes w/accordance to RFC959 */
+        /* fix up codes in accordance with RFC959 */
         if (toupper((unsigned char) cmd[1]) != 'W') {
             /* CDUP, XCUP */
             if (code != 200)
@@ -764,6 +776,14 @@ static EIO_Status x_FTPDir(SFTPConnector* xxx,
 }
 
 
+/* Worthwhile reading:
+ * https://datatracker.ietf.org/doc/html/rfc793#section-3.1
+ * https://datatracker.ietf.org/doc/html/rfc1122#page-84
+ * https://datatracker.ietf.org/doc/html/draft-ietf-tcpm-urgent-data-05
+ * https://datatracker.ietf.org/doc/html/rfc854
+ * https://www.freesoft.org/CIE/RFC/854/5a.htm
+ * http://alas.matf.bg.ac.rs/manuals/lspe/snode=126.html
+ */
 static EIO_Status x_FTPTelnetSynch(SFTPConnector* xxx)
 {
     EIO_Status status;
@@ -782,6 +802,55 @@ static EIO_Status x_FTPTelnetSynch(SFTPConnector* xxx)
 }
 
 
+static void x_StoreTimeoutNormalized(STimeout* dst, const STimeout* src)
+{
+    dst->sec  = src->sec;
+    dst->sec += src->usec / 1000000;
+    dst->usec = src->usec % 1000000;
+}
+
+
+static int/*bool*/ x_IsLongerTimeout(const STimeout* p1,
+                                     const STimeout* p2)
+{
+    STimeout t1, t2;
+    assert(p2);
+    if (!p1)
+        return 1/*T*/;
+    x_StoreTimeoutNormalized(&t1, p1);
+    x_StoreTimeoutNormalized(&t2, p2);
+    return   t1.sec  > t2.sec
+        ||  (t1.sec == t2.sec  &&  t1.usec > t2.usec) ? 1/*T*/ : 0/*F*/;
+}
+
+
+static EIO_Status x_DrainData(SFTPConnector* xxx, const STimeout* timeout)
+{
+    EIO_Status status;
+    time_t deadline = time(0);
+    /* this is not "data" per se, so go silent */
+    ESwitch log = xxx->flag & fFTP_LogData
+        ? SOCK_SetDataLogging(xxx->data, eDefault) : eDefault;
+    const STimeout* r_timeout = SOCK_GetTimeout(xxx->data, eIO_Read);
+    if (x_IsLongerTimeout(r_timeout, timeout))
+        SOCK_SetTimeout(xxx->data, eIO_Read, timeout);
+    else
+        timeout = r_timeout;
+    deadline += (time_t)(timeout->sec + (timeout->usec + 500000) / 1000000);
+
+    /* drain up data connection by discarding 1MB blocks repeatedly */
+    while ((status = SOCK_Read(xxx->data, 0, 1 << 20, 0, eIO_ReadPlain))
+           == eIO_Success) {
+        if (time(0) >= deadline)
+            break;
+    }
+
+    if (log != eDefault)
+        SOCK_SetDataLogging(xxx->data, log);
+    return status;
+}
+
+
 /*
  * how = 0 -- ABOR sequence only if data connection is open;
  * how = 1 -- just abort data connection, if any open;
@@ -800,25 +869,20 @@ static EIO_Status x_FTPAbort(SFTPConnector*  xxx,
         return eIO_Success;
     if (!xxx->cntl  ||  how == 1)
         return x_FTPCloseData(xxx, eIO_Close/*silent close*/, 0);
-    if (!timeout)
+    if (x_IsLongerTimeout(timeout, &kFailsafeTimeout))
         timeout = &kFailsafeTimeout;
     SOCK_SetTimeout(xxx->cntl, eIO_ReadWrite, timeout);
     status = x_FTPTelnetSynch(xxx);
     if (status == eIO_Success)
         status  = s_FTPCommand(xxx, "ABOR", 0);
     if (xxx->data) {
-        if (status == eIO_Success  &&  !xxx->send) {
-            /* this is not "data" per se, so go silent */
-            if (xxx->flag & fFTP_LogData)
-                SOCK_SetDataLogging(xxx->data, eDefault);
-            SOCK_SetTimeout(xxx->data, eIO_ReadWrite, timeout);
-            /* drain up data connection by discarding 1MB blocks repeatedly */
-            while (SOCK_Read(xxx->data, 0, 1<<20, 0, eIO_ReadPlain)
-                   == eIO_Success) {
-                continue;
-            }
+        EIO_Status abort = status;
+        if (abort == eIO_Success  &&  !xxx->send) {
+            if (x_DrainData(xxx, timeout) == eIO_Success)
+                abort = eIO_Timeout;
         }
         x_FTPCloseData(xxx, how == 3
+                       ||  abort != eIO_Success
                        ||  SOCK_Status(xxx->data, eIO_Read) != eIO_Closed
                        ? eIO_Open/*warning*/ : eIO_Close/*silent*/, 0);
     }
@@ -827,7 +891,7 @@ static EIO_Status x_FTPAbort(SFTPConnector*  xxx,
     if (status == eIO_Success) {
         int         code = 426;
         int/*bool*/ sync = xxx->sync;
-        status = s_FTPDrainReply(xxx, &code, 2/*2xx*/);
+        status = x_FTPDrainReply(xxx, &code, 2/*2xx*/);
         if (status == eIO_Success) {
             /* Microsoft FTP is known to return 225 instead of 226 */
             if (code != 225  &&  code != 226  &&  code != 426)
@@ -875,13 +939,14 @@ static EIO_Status x_FTPEpsv(SFTPConnector*  xxx,
 
 
 /* all implementations MUST support PASV */
+/* NB: RFC1639 extends PASV for various protocol families with LPSV */
 static EIO_Status x_FTPPasv(SFTPConnector*  xxx,
                             unsigned int*   host,
                             unsigned short* port)
 {
     EIO_Status status;
-    int  code, o[6];
     unsigned int i;
+    int code, o[6];
     char buf[128];
 
     status = s_FTPCommand(xxx, "PASV", 0);
@@ -891,38 +956,38 @@ static EIO_Status x_FTPPasv(SFTPConnector*  xxx,
     if (status != eIO_Success  ||  code != 227)
         return eIO_Unknown;
     for (;;) {
-        char* c;
-        size_t len;
+        char* c, q;
         /* RFC 1123 4.1.2.6 says that ()'s in PASV reply MUST NOT be assumed */
-        for (c = buf;  *c;  ++c) {
-            if (isdigit((unsigned char)(*c)))
-                break;
-        }
-        if (!*c)
+        if (!*(c = buf + strcspn(buf, kDigits)))
             return eIO_Unknown;
-        len = 0;
+        q = c > buf ? c[-1] : '\0';
         for (i = 0;  i < (unsigned int)(sizeof(o) / sizeof(o[0]));  ++i) {
-            if (sscanf(c + len, &",%d%n"[!i], &o[i], &code) < 1)
+            if (sscanf(c, &",%d%n"[!i], &o[i], &code) < 1)
                 break;
-            len += (size_t) code;
+            assert(code > 0);
+            c += code;
         }
-        if (i >= (unsigned int)(sizeof(o) / sizeof(o[0])))
-            break;
-        if (!len) {
-            len = strspn(c, kDigits);
-            assert(len > 0);
-        }
-        memmove(buf, c + len, strlen(c + len) + 1);
+        if (i >= (unsigned int)(sizeof(o) / sizeof(o[0]))) {
+            if (!q  ||  q != '('  ||  *c == ')')
+                break;
+        } else if (!i)
+            return eIO_Unknown;
+        assert(c > buf);
+        memmove(buf, c, strlen(c) + 1);
     }
-    for (i = 0;  i < (unsigned int)(sizeof(o) / sizeof(o[0]));  ++i) {
+    assert(i == (unsigned int)(sizeof(o) / sizeof(o[0])));
+    while (i--) {
         if (o[i] < 0  ||  o[i] > 255)
             return eIO_Unknown;
     }
     if (!(i = (unsigned int)((((((o[0]<<8) | o[1])<<8) | o[2])<<8) | o[3])))
         return eIO_Unknown;
+    if (i == (unsigned int)(-1))
+        return eIO_Unknown;
     *host = SOCK_HostToNetLong(i);
     if (!(i = (unsigned int)((o[4]<<8) | o[5])))
         return eIO_Unknown;
+    assert(i <= 0xFFFF);
     *port = (unsigned short) i;
     return eIO_Success;
 }
@@ -996,7 +1061,7 @@ static EIO_Status x_FTPEprt(SFTPConnector* xxx,
         return eIO_NotSupported;
     memcpy(buf, "|1|", 3); /*IPv4*/
     SOCK_ntoa(host, buf + 3, sizeof(buf) - 3);
-    sprintf(buf + 3 + strlen(buf + 3), "|%hu|", port);
+    sprintf((buf + 3) + strlen(buf + 3), "|%hu|", port);
     status = s_FTPCommand(xxx, "EPRT", buf);
     if (status != eIO_Success)
         return status;
@@ -1012,6 +1077,7 @@ static EIO_Status x_FTPEprt(SFTPConnector* xxx,
 
 
 /* all implementations MUST support PORT */
+/* NB: RFC1639 extends PORT for various protocol families with LPRT */
 static EIO_Status x_FTPPort(SFTPConnector* xxx,
                             unsigned int   host,
                             unsigned short port)
@@ -1046,9 +1112,9 @@ static EIO_Status x_FTPActive(SFTPConnector*  xxx,
     unsigned int   host;
     unsigned short port;
 
-    /* NB: Apache FTP proxy re-uses SOCK_LocalPort(xxx->cntl);
-     * which is the default port for user-end data port per RFC959,
-     * other implementations don't do that leaving OS to decide,
+    /* NB: Apache FTP proxy re-uses SOCK_GetLocalPort(xxx->cntl);
+     * which is the default port for user-end data port per RFC959 3.2,
+     * other implementations don't do that leaving for OS to decide,
      * since the PORT command will be issued, anyways... */
     status = LSOCK_CreateEx(0, 1, lsock, xxx->flag & fFTP_LogControl
                             ? fSOCK_LogOn : fSOCK_LogDefault);
@@ -1160,6 +1226,7 @@ static EIO_Status x_FTPXfer(SFTPConnector*  xxx,
                 xxx->sync = 0/*false*/;
                 return eIO_Success;
             }
+            assert(code == 2/*full abort*/);
         } else if (code == 450  ||  code == 550) {
             /* file processing errors: not a file, not a dir, etc */
             status = eIO_Closed;
@@ -1645,14 +1712,22 @@ static EIO_Status s_FTPNegotiate(SFTPConnector* xxx,
 
 
 /* NB: data connection (upload only) may end up closed */
-static EIO_Status s_FTPPollCntl(SFTPConnector* xxx, const STimeout* timeout)
+static EIO_Status x_FTPPollCntl(SFTPConnector* xxx, const STimeout* timeout)
 {
     EIO_Status status = eIO_Success;
     int/*bool*/ abor = xxx->abor;
     xxx->abor = 0/*false*/;
-    while (SOCK_Wait(xxx->cntl, eIO_Read, &kZeroTimeout) == eIO_Success) {
-        char buf[80];
-        int  code;
+    for (;;) {
+        char       buf[80];
+        int        code;
+        EIO_Status wait = SOCK_Wait(xxx->cntl, eIO_Read, &kZeroTimeout);
+        if (wait != eIO_Success) {
+            if (wait  == eIO_Closed) {
+                status = eIO_Closed;
+                x_FTPCloseCntl(xxx, IO_StatusStr(status));
+            }
+            break;
+        }
         if (timeout != &kZeroTimeout) {
             SOCK_SetTimeout(xxx->cntl, eIO_Read,
                             timeout ? timeout : &kFailsafeTimeout);
@@ -1685,7 +1760,7 @@ static EIO_Status s_FTPPollCntl(SFTPConnector* xxx, const STimeout* timeout)
 }
 
 
-static EIO_Status s_FTPSyncCntl(SFTPConnector* xxx, const STimeout* timeout)
+static EIO_Status x_FTPSyncCntl(SFTPConnector* xxx, const STimeout* timeout)
 {
     if (!xxx->sync) {
         EIO_Status status;
@@ -1696,7 +1771,7 @@ static EIO_Status s_FTPSyncCntl(SFTPConnector* xxx, const STimeout* timeout)
         timeout = &kZeroTimeout;
         assert(xxx->sync);
     }
-    return s_FTPPollCntl(xxx, timeout);
+    return x_FTPPollCntl(xxx, timeout);
 }
 
 
@@ -1714,7 +1789,7 @@ static EIO_Status s_FTPExecute(SFTPConnector* xxx, const STimeout* timeout)
         xxx->what = 0;
     }
     if (status == eIO_Success)
-        status  = s_FTPSyncCntl(xxx, timeout);
+        status  = x_FTPSyncCntl(xxx, timeout);
     if (status != eIO_Success)
         goto out;
     if (xxx->rest) {
@@ -1725,12 +1800,12 @@ static EIO_Status s_FTPExecute(SFTPConnector* xxx, const STimeout* timeout)
             xxx->rclr = 1/*true*/;
     }
     assert(xxx->cntl);
-    verify((size = BUF_Size(xxx->wbuf)) != 0);
+    size = BUF_Size(xxx->wbuf);
     if ((s = (char*) malloc(size + 1)) != 0
         &&  BUF_Read(xxx->wbuf, s, size) == size) {
         const char* c;
         assert(!memchr(s, '\n', size));
-        if (s[size - 1] == '\r')
+        if (size  &&  s[size - 1] == '\r')
             --size;
         s[size] = '\0';
         if (*s)
@@ -1740,10 +1815,13 @@ static EIO_Status s_FTPExecute(SFTPConnector* xxx, const STimeout* timeout)
         else
             size = (size_t)(c - s);
         assert(*c == ' '  ||  !*c);
-        if (size == 3  ||  size == 4) {
+        if (!size  ||  size == 3  ||  size == 4) {
             SOCK_SetTimeout(xxx->cntl, eIO_ReadWrite, timeout);
-            if         (size == 3  &&   strncasecmp(s, "REN",  3) == 0) {
-                /* special-case, non-standard command */
+            if (!size || (size == 4 && strncasecmp(s, "NOOP", 4) == 0 && !*c)){
+                /* Special, means to stop the current command and reach EOF */
+                status = size ? x_FTPNoop(xxx) : eIO_Success;
+            } else if  (size == 3  &&   strncasecmp(s, "REN",  3) == 0) {
+                /* Special-case, non-standard command */
                 status = s_FTPRename(xxx, c + strspn(c, " \t"));
             } else if ((size == 3  ||  toupper((unsigned char) c[-4]) == 'X')
                        &&          (strncasecmp(c - 3, "CWD",  3) == 0  ||
@@ -1779,11 +1857,6 @@ static EIO_Status s_FTPExecute(SFTPConnector* xxx, const STimeout* timeout)
             } else if  (size == 4  &&  (strncasecmp(s, "FEAT", 4) == 0  ||
                                         strncasecmp(s, "OPTS", 4) == 0)) {
                 status = s_FTPNegotiate(xxx, s);
-            } else if  (size == 4  &&   strncasecmp(s, "NOOP", 4) == 0 && !*c){
-                /* Special, means to stop the current command and reach EOF */
-                *s = '\0';
-                xxx->what = 0;
-                status = x_FTPNoop(xxx);
             } else
                 status = eIO_NotSupported;
         } else
@@ -1803,7 +1876,7 @@ static EIO_Status s_FTPExecute(SFTPConnector* xxx, const STimeout* timeout)
 }
 
 
-static EIO_Status s_FTPCompleteUpload(SFTPConnector*  xxx,
+static EIO_Status x_FTPCompleteUpload(SFTPConnector*  xxx,
                                       const STimeout* timeout)
 {
     EIO_Status status;
@@ -1940,6 +2013,7 @@ static EIO_Status s_VT_Open
                     ("[FTP; %s:%hu]  Too many failed attempts (%hu), giving up"
                      , xxx->info->host, xxx->info->port, ntry));
     }
+    assert(!(status == eIO_Success) == !xxx->cntl);
     assert(!xxx->what  &&  !xxx->data);
     xxx->r_status = status;
     xxx->w_status = status;
@@ -1962,7 +2036,7 @@ static EIO_Status s_VT_Wait
         if (xxx->data) {
             assert(xxx->open);
             if (event == eIO_Read)
-                return s_FTPCompleteUpload(xxx, timeout);
+                return x_FTPCompleteUpload(xxx, timeout);
             return SOCK_Wait(xxx->data, eIO_Write, timeout);
         }
         if (event == eIO_Write  ||  !xxx->open)
@@ -2005,7 +2079,7 @@ static EIO_Status s_VT_Write
     if (xxx->send) {
         if (!xxx->data)
             return eIO_Closed;
-        status = s_FTPPollCntl(xxx, timeout);
+        status = x_FTPPollCntl(xxx, timeout);
         if (status == eIO_Success) {
             SOCK_SetTimeout(xxx->data, eIO_Write, timeout);
             status = SOCK_Write(xxx->data, buf, size,n_written,eIO_WritePlain);
@@ -2016,52 +2090,67 @@ static EIO_Status s_VT_Write
                 x_FTPCloseData(xxx, eIO_Close/*silent close*/, 0);
             }
         }
-    } else if (size) {
-        size_t count;
-        const char* run = (const char*) memchr((const char*) buf, '\n', size);
-        *n_written = size; /* by default report the entire command consumed */
+        goto out;
+    }
+
+    if (size) {
+        const char* run = (const char*) memchr(buf, '\n', size);
+        *n_written = size; /* by default, report the entire output consumed */
         if (run  &&  run < (const char*) buf + --size) {
             /* reject multiple commands */
             BUF_Erase(xxx->wbuf);
-            return eIO_Unknown;
-        }
-        count = 0;
-        if (xxx->flag & fFTP_UncleanIAC) {
-            if (BUF_Write(&xxx->wbuf, buf, size))
-                count = size;
+            status = eIO_Unknown;
         } else {
-            static const char kIAC[] = { '\377'/*IAC*/, '\377' };
-            const char* s = (const char*) buf;
-            while (count < size) {
-                /* Escaped IAC (Interpret As Command) character, per RFC854 */
-                const char* p;
-                size_t part;
-                if (count) {
-                    if (!BUF_Write(&xxx->wbuf, kIAC, sizeof(kIAC)))
+            size_t count = 0;
+            if (xxx->flag & fFTP_UncleanIAC) {
+                if (BUF_Write(&xxx->wbuf, buf, size))
+                    count = size;
+            } else {
+                static const char kIAC[] = { '\377'/*IAC*/, '\377' };
+                const char* s = (const char*) buf;
+                while (count < size) {
+                    const char* p;
+                    size_t part;
+                    if (count) {
+                        /* Escaped IAC (Interpret As Command) char, RFC854 */
+                        if (!BUF_Write(&xxx->wbuf, kIAC, sizeof(kIAC)))
+                            break;
+                        assert(*s == kIAC[0]);
+                        ++count;
+                        ++s;
+                    }
+                    if (!(p = (const char*) memchr(s, kIAC[0], size - count)))
+                        part = size - count;
+                    else
+                        part = (size_t)(p - s);
+                    if (!BUF_Write(&xxx->wbuf, s, part))
                         break;
-                    ++count;
-                    ++s;
+                    count += part;
+                    s     += part;
                 }
-                if (!(p = (const char*) memchr(s, kIAC[0], size - count)))
-                    part = size - count;
-                else
-                    part = (size_t)(p - s);
-                if (!BUF_Write(&xxx->wbuf, s, part))
-                    break;
-                count += part;
-                s     += part;
+            }
+            if (count < size) {
+                /* short write */
+                *n_written = count;
+                status = eIO_Unknown;
+            } else if (!run) {
+                /* keep appending */
+                status = eIO_Success;
+            } else {
+                status = s_FTPExecute(xxx, timeout);
+                if (status == eIO_Closed  &&  !xxx->cntl)
+                    *n_written = 0;
+                return status;
             }
         }
-        if (count < size) {
-            /* short write */
-            *n_written = count;
-            status = eIO_Unknown;
-        } else if (!run) {
-            status = eIO_Success;
-        } else
-            return s_FTPExecute(xxx, timeout);
+        if (xxx->what  &&  status != eIO_Success) {
+            free((void*) xxx->what);
+            xxx->what = 0;
+        }
     } else
         status = eIO_Success;
+
+ out:
     xxx->w_status = status;
     return status;
 }
@@ -2106,7 +2195,7 @@ static EIO_Status s_VT_Read
             assert(!BUF_Size(xxx->rbuf));
             return eIO_Closed;
         }
-        status = s_FTPCompleteUpload(xxx, timeout);
+        status = x_FTPCompleteUpload(xxx, timeout);
         if (status != eIO_Success)
             return status;
         assert(!xxx->data  &&  !xxx->send);
@@ -2118,7 +2207,7 @@ static EIO_Status s_VT_Read
         status = eIO_Success;
     if (xxx->data) {
         assert(!xxx->send  &&  !BUF_Size(xxx->rbuf));
-        /* NB: Cannot use s_FTPPollCntl() here because a response about data
+        /* NB: Cannot use x_FTPPollCntl() here because a response about data
          * connection closure may be seen before the actual EOF in the
          * (heavily loaded) data connection. */
         SOCK_SetTimeout(xxx->data, eIO_Read, timeout);
@@ -2195,13 +2284,13 @@ static EIO_Status s_VT_Close
     if (xxx->cntl) {
         if (!data  &&  status == eIO_Success) {
             int code = 0/*any*/;
-            if (!timeout)
+            if (x_IsLongerTimeout(timeout, &kFailsafeTimeout))
                 timeout = &kFailsafeTimeout;
             SOCK_SetTimeout(xxx->cntl, eIO_ReadWrite, timeout);
             /* all implementations MUST support QUIT */
             status = s_FTPCommand(xxx, "QUIT", 0);
             if (status == eIO_Success) {
-                status  = s_FTPDrainReply(xxx, &code, code);
+                status  = x_FTPDrainReply(xxx, &code, code);
                 if (status == eIO_Success)
                     status  = eIO_Unknown;
                 if (status == eIO_Closed  &&  code == 221)
@@ -2319,7 +2408,7 @@ extern CONNECTOR s_CreateConnector(const SConnNetInfo*  info,
     xinfo->stateless = 0;
     xinfo->lb_disable = 0;
     xinfo->http_proxy_leak = 0;
-    if (!(flag & fFTP_UseProxy)) {
+    if (!(flag & fFTP_UseProxy)  ||  xinfo->http_proxy_only) {
         xinfo->http_proxy_host[0] = '\0';
         xinfo->http_proxy_port    =   0;
         xinfo->http_proxy_user[0] = '\0';
