@@ -1,4 +1,4 @@
-/* $Id: ncbi_gnutls.c 629075 2021-04-09 14:32:17Z ivanov $
+/* $Id: ncbi_gnutls.c 645948 2022-02-25 14:51:41Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,11 +35,16 @@
 #include "ncbi_priv.h"
 #include "ncbi_servicep.h"
 #include <connect/ncbi_gnutls.h>
+#include <connect/ncbi_tls.h>
 #include <stdlib.h>
 
+#define NCBI_USE_ERRCODE_X   Connect_TLS
+
+
 #ifdef HAVE_LIBGNUTLS
 
 #  include <gnutls/gnutls.h>
+#  include <gnutls/x509.h>
 
 #  if   defined(ENOTSUP)
 #    define NCBI_NOTSUPPORTED  ENOTSUP
@@ -76,9 +81,16 @@ static int gcry_user_mutex_init(void** lock)
 }
 static int gcry_user_mutex_destroy(void** lock)
 {
-    g_CORE_MT_Lock = MT_LOCK_Delete(*((MT_LOCK*) lock));
-    *lock = 0;
-    return 0;
+    if (*lock) {
+        if (!(*lock = MT_LOCK_Delete((MT_LOCK)(*lock)))) {
+            /* NB: Do not use CORE_SetLOCK() here! */
+            g_CORE_MT_Lock = 0;
+        } else
+            *lock = 0;
+        return 0;
+    }
+    CORE_LOG_X(49, eLOG_Warning, "NULL MT_LOCK deinit in GCRYPT");
+    return EINVAL;
 }
 static int gcry_user_mutex_lock(void** lock)
 {
@@ -86,7 +98,7 @@ static int gcry_user_mutex_lock(void** lock)
 }
 static int gcry_user_mutex_unlock(void** lock)
 {
-    return MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock)!=0 ? 0 : NCBI_NOTSUPPORTED;
+    return MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock)>0 ? 0 : NCBI_NOTSUPPORTED;
 }
 static struct gcry_thread_cbs gcry_threads_user = {
     GCRY_THREAD_OPTION_USER, NULL/*gcry_user_init*/,
@@ -111,9 +123,16 @@ static int gtls_user_mutex_init(void** lock)
 }
 static int gtls_user_mutex_deinit(void** lock)
 {
-    g_CORE_MT_Lock = MT_LOCK_Delete((MT_LOCK)(*lock));
-    *lock = 0;
-    return 0;
+    if (*lock) {
+        if (!(*lock = MT_LOCK_Delete((MT_LOCK)(*lock)))) {
+            /* NB: Do not use CORE_SetLOCK() here! */
+            g_CORE_MT_Lock = 0;
+        } else
+            *lock = 0;
+        return 0;
+    }
+    CORE_LOG_X(49, eLOG_Warning, "NULL MT_LOCK deinit in GNUTLS");
+    return EINVAL;
 }
 static int gtls_user_mutex_lock(void** lock)
 {
@@ -121,7 +140,7 @@ static int gtls_user_mutex_lock(void** lock)
 }
 static int gtls_user_mutex_unlock(void** lock)
 {
-    return MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock)!=0 ? 0 : NCBI_NOTSUPPORTED;
+    return MT_LOCK_Do((MT_LOCK)(*lock), eMT_Unlock)>0 ? 0 : NCBI_NOTSUPPORTED;
 }
 #    ifdef __cplusplus
 }
@@ -170,7 +189,7 @@ static const int kGnuTlsCompPrio[] = {
 #  endif /*LIBGNUTLS_VERSION_NUMBER<3.3.6*/
 
 
-static int                              s_GnuTlsLogLevel;
+static volatile int                     s_GnuTlsLogLevel;
 static gnutls_anon_client_credentials_t s_GnuTlsCredAnon;
 static gnutls_certificate_credentials_t s_GnuTlsCredCert;
 static volatile FSSLPull                s_Pull;
@@ -187,7 +206,8 @@ static void x_GnuTlsLogger(int level, const char* message)
         return;
     if (message[len - 1] == '\n')
         --len;
-    CORE_LOGF(eLOG_Note, ("GNUTLS%d: %.*s", level, (int) len, message));
+    CORE_LOGF_X(21, eLOG_Note,
+                ("GNUTLS%d: %.*s", level, (int) len, message));
 }
 
 
@@ -316,8 +336,10 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
 
     {{
         int x_err = error ? error : errno;
-        CORE_TRACEF(("CONNECT GNUTLS status %s -> %s %d",
-                     IO_StatusStr(status), error ? "error" : "errno", x_err));
+        CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+            CORE_TRACEF(("CONNECT GNUTLS status %s -> %s %d",
+                         IO_StatusStr(status),
+                         error ? "error" : "errno", x_err));
         if (!error)
             errno = x_err; /* restore errno that might have been clobbered */
     }}
@@ -339,8 +361,9 @@ static void* s_GnuTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
     int err;
 
     if (end == GNUTLS_SERVER) {
-        CORE_LOG(eLOG_Error, "Server-side SSL not yet supported with GNUTLS");
-        *error = 0;
+        CORE_LOG_X(22, eLOG_Critical,
+                   "Server-side SSL not yet supported with GNUTLS");
+        *error = NCBI_NOTSUPPORTED;
         return 0;
     }
 
@@ -352,19 +375,24 @@ static void* s_GnuTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
     if (!acred
         ||  (ctx->cred
              &&  (ctx->cred->type != eNcbiCred_GnuTls  ||  !ctx->cred->data))){
-        CORE_LOGF(eLOG_Error, ("Cannot %s GNUTLS credentials: %s",
-                               acred ? "use"            : "set",
-                               acred ? "Invalid format" : "Not initialized"));
+        CORE_LOGF_X(23, eLOG_Error,
+                    ("Cannot %s GNUTLS credentials: %s",
+                     acred ? "use"            : "set",
+                     acred ? "Invalid format" : "Not initialized"));
         /*FIXME: there's a NULL(data)-terminated array of credentials */
-        *error = 0;
+        *error = acred ? EINVAL : NCBI_NOTSUPPORTED;
         return 0;
     }
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACE("GnuTlsCreate(): Enter");
+
     if ((err = gnutls_init(&session, end)) != GNUTLS_E_SUCCESS/*0*/) {
         *error = err;
         return 0;
     }
 
+    /* Check CONN_GNUTLS_PRIORITY or [CONN]GNUTLS_PRIORITY */
     ConnNetInfo_GetValueInternal(0, "GNUTLS_PRIORITY", val, sizeof(val), 0);
 
     len = ctx->host ? strlen(ctx->host) : 0;
@@ -384,7 +412,9 @@ static void* s_GnuTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
 #  endif /*LIBGNUTLS_VERSION_NUMBER<3.3.6*/
         (err = gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE,
                                       ctx->cred
-                                      ? ctx->cred->data : xcred))      != 0  ||
+                                      ? (gnutls_certificate_credentials_t)
+                                         ctx->cred->data
+                                      : xcred))                        != 0  ||
         (err = gnutls_credentials_set(session, GNUTLS_CRD_ANON, acred))!= 0  ||
         (len  &&  (err = gnutls_server_name_set(session, GNUTLS_NAME_DNS,
                                                 ctx->host, len))       != 0)) {
@@ -402,6 +432,9 @@ static void* s_GnuTlsCreate(ESOCK_Side side, SNcbiSSLctx* ctx, int* error)
     gnutls_handshake_set_timeout(session, 0);
 #  endif /*LIBGNUTLS_VERSION_NUMBER>=3.0.0*/
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACEF(("GnuTlsCreate(): Leave(%p)", session));
+
     return session;
 }
 
@@ -411,6 +444,9 @@ static EIO_Status s_GnuTlsOpen(void* session, int* error, char** desc)
     EIO_Status status;
     int x_error;
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACEF(("GnuTlsOpen(%p): Enter", session));
+
     do {
         x_error = gnutls_handshake((gnutls_session_t) session);
     } while (x_error == GNUTLS_E_REHANDSHAKE);
@@ -436,6 +472,9 @@ static EIO_Status s_GnuTlsOpen(void* session, int* error, char** desc)
         status = eIO_Success;
     }
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACEF(("GnuTlsOpen(%p): Leave(%d)", session, status));
+
     return status;
 }
 
@@ -445,7 +484,7 @@ inline
 #endif /*__GNUC__*/
 static int/*bool*/ x_IfToLog(void)
 {
-    return 7 < s_GnuTlsLogLevel  &&  s_GnuTlsLogLevel <= 10 ? 1/*T*/ : 0/*F*/;
+    return 12 < s_GnuTlsLogLevel ? 1/*T*/ : 0/*F*/;
 }
 
 
@@ -614,11 +653,17 @@ static EIO_Status s_GnuTlsClose(void* session, int how, int* error)
 
     assert(session);
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACEF(("GnuTlsClose(%p): Enter", session));
+
     x_error = gnutls_bye((gnutls_session_t) session,
                          how == SOCK_SHUTDOWN_RDWR
                          ? GNUTLS_SHUT_RDWR
                          : GNUTLS_SHUT_WR);
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACEF(("GnuTlsClose(%p): Leave", session));
+
     if (x_error != GNUTLS_E_SUCCESS) {
         *error = x_error;
         return eIO_Unknown;
@@ -631,7 +676,13 @@ static void s_GnuTlsDelete(void* session)
 {
     assert(session);
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACEF(("GnuTlsDelete(%p): Enter", session));
+
     gnutls_deinit((gnutls_session_t) session);
+
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACEF(("GnuTlsDelete(%p): Leave", session));
 }
 
 
@@ -646,31 +697,35 @@ static EIO_Status x_InitLocking(void)
         ? eIO_Success
         : eIO_NotSupported;
 #      elif defined(NCBI_THREADS)
-    MT_LOCK lk = CORE_GetLOCK();
-    if (MT_LOCK_Do(lk, eMT_Lock) != -1) {
+    int locked;
+    MT_LOCK lock = CORE_GetLOCK();
+    if ((locked = MT_LOCK_Do(lock, eMT_Lock)) > 0) {
         status = gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_user) == 0
             ? eIO_Success
             : eIO_NotSupported;
-        MT_LOCK_Do(lk, eMT_Unlock);
+        MT_LOCK_Do(lock, eMT_Unlock);
     } else
-        status = lk ? eIO_Success : eIO_NotSupported;
+        status = !locked ? eIO_Unknown : lock ? eIO_Success : eIO_NotSupported;
 #      elif !defined(NCBI_NO_THREADS)  &&  defined(_MT)
-    CORE_LOG(eLOG_Critical,"LIBGCRYPT uninitialized: Unknown threading model");
+    CORE_LOG_X(24, eLOG_Critical,
+               "LIBGCRYPT uninitialized: Unknown threading model");
     status = eIO_NotSupported;
 #      endif /*NCBI_POSIX_THREADS*/
 #    endif /*HAVE_LIBGCRYPT*/
 #  elif defined(NCBI_THREADS)
-    MT_LOCK lk = CORE_GetLOCK();
-    if (MT_LOCK_Do(lk, eMT_Lock) != -1) {
+    int locked;
+    MT_LOCK lock = CORE_GetLOCK();
+    if ((locked = MT_LOCK_Do(lock, eMT_Lock)) > 0) {
         /* NB: calls global_deinit/global_init internally */
         gnutls_global_set_mutex(gtls_user_mutex_init, gtls_user_mutex_deinit,
                                 gtls_user_mutex_lock, gtls_user_mutex_unlock);
-        MT_LOCK_Do(lk, eMT_Unlock);
+        MT_LOCK_Do(lock, eMT_Unlock);
         status = eIO_Success;
     } else
-        status = lk ? eIO_Success : eIO_NotSupported;
+        status = !locked ? eIO_Unknown : lock ? eIO_Success : eIO_NotSupported;
 #  elif !defined(NCBI_NO_THREADS)  &&  defined(_MT)
-    CORE_LOG(eLOG_Critical,"GNUTLS locking uninited: Unknown threading model");
+    CORE_LOG_X(25, eLOG_Critical,
+               "GNUTLS locking uninited: Unknown threading model");
     status = eIO_NotSupported;
 #  else
     status = eIO_Success;
@@ -686,6 +741,7 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
     gnutls_anon_client_credentials_t acred;
     gnutls_certificate_credentials_t xcred;
     const char* version;
+    int/*bool*/ inited;
     EIO_Status status;
     const char* val;
     char buf[32];
@@ -694,20 +750,25 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
 
     version = gnutls_check_version(0);
     if (strcasecmp(GNUTLS_VERSION, version) != 0) {
-        CORE_LOGF(eLOG_Critical,
-                  ("GNUTLS version mismatch: %s headers vs. %s runtime",
-                   GNUTLS_VERSION, version));
+        CORE_LOGF_X(26, eLOG_Critical,
+                    ("GNUTLS version mismatch: %s headers vs. %s runtime",
+                     GNUTLS_VERSION, version));
         assert(0);
     }
 
+    CORE_TRACE("GnuTlsInit(): Enter");
+
     if (!pull  ||  !push)
         return eIO_InvalidArg;
 
-    val = ConnNetInfo_GetValueInternal(0, "GNUTLS_LOGLEVEL",
-                                       buf, sizeof(buf), 0);
+    /* Check CONN_[GNU]TLS_LOGLEVEL or [CONN][GNU]TLS_LOGLEVEL */
+    val = ConnNetInfo_GetValueInternal(0, "GNU" REG_CONN_TLS_LOGLEVEL,
+                                       buf, sizeof(buf),
+                                       DEF_CONN_TLS_LOGLEVEL);
     if (!val  ||  !*val) {
-        val = ConnNetInfo_GetValueInternal(0, "TLS_LOGLEVEL",
-                                           buf, sizeof(buf), 0);
+        val = ConnNetInfo_GetValueInternal(0, REG_CONN_TLS_LOGLEVEL,
+                                           buf, sizeof(buf),
+                                           DEF_CONN_TLS_LOGLEVEL);
     }
     CORE_LOCK_READ;
     if (!val  ||  !*val)
@@ -723,19 +784,24 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
             level = eLOG_Note;
         } else
             level = eLOG_Trace;
-        CORE_LOGF(level, ("GNUTLS V%s (LogLevel=%d)",
-                          version, s_GnuTlsLogLevel));
+        CORE_LOGF_X(27, level, ("GNUTLS V%s (LogLevel=%d)",
+                                version, s_GnuTlsLogLevel));
     } else
         CORE_UNLOCK;
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACE("GnuTlsInit(): Go-on");
+
+    inited = 0/*false*/;
     if ((status = x_InitLocking()) != eIO_Success)
         goto out;
 
-    if (!gnutls_check_version(LIBGNUTLS_VERSION)  ||
-        gnutls_global_init() != GNUTLS_E_SUCCESS/*0*/) {
+    if (!gnutls_check_version(LIBGNUTLS_VERSION)
+        ||  gnutls_global_init() != GNUTLS_E_SUCCESS/*0*/) {
         status = eIO_NotSupported;
         goto out;
     }
+    inited = 1/*true*/;
     if (gnutls_anon_allocate_client_credentials(&acred) != 0) {
         gnutls_global_deinit();
         status = eIO_Unknown;
@@ -747,17 +813,23 @@ static EIO_Status s_GnuTlsInit(FSSLPull pull, FSSLPush push)
         status = eIO_Unknown;
         goto out;
     }
+    assert(acred  &&  xcred);
 
     s_GnuTlsCredAnon = acred;
     s_GnuTlsCredCert = xcred;
     s_Pull           = pull;
     s_Push           = push;
 
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACE("GnuTlsInit(): Leave");
+
     return eIO_Success;
 
  out:
     gnutls_global_set_log_level(s_GnuTlsLogLevel = 0);
     gnutls_global_set_log_function(0);
+    if (inited)
+        gnutls_global_deinit();
     return status;
 }
 
@@ -768,7 +840,10 @@ static void s_GnuTlsExit(void)
     gnutls_anon_client_credentials_t acred = s_GnuTlsCredAnon;
     gnutls_certificate_credentials_t xcred = s_GnuTlsCredCert;
 
-    assert(acred);
+    assert(acred  &&  xcred);
+
+    CORE_DEBUG_ARG(if (s_GnuTlsLogLevel))
+        CORE_TRACE("GnuTlsExit(): Enter");
 
     s_Push           = 0;
     s_Pull           = 0;
@@ -785,9 +860,12 @@ static void s_GnuTlsExit(void)
     /* If GNUTLS is loaded as a DLL, it still has init count 1, so make sure
      * cleanup worked completely (MSVC2015 ReleaseDLL build breaks if not) */
     gnutls_global_deinit();
+
+    CORE_TRACE("GnuTlsExit(): Leave");
 }
 
- 
+
+/*ARGSUSED*/
 static const char* s_GnuTlsError(void* session, int error,
                                  char* buf/*unused*/, size_t size/*unused*/)
 {
@@ -799,18 +877,15 @@ static const char* s_GnuTlsError(void* session, int error,
         : gnutls_strerror(error);
 }
 
-
 #else
 
-
 /*ARGSUSED*/
 static EIO_Status s_GnuTlsInit(FSSLPull unused_pull, FSSLPush unused_push)
 {
-    CORE_LOG(eLOG_Critical, "Unavailable feature GNUTLS");
+    CORE_LOG_X(28, eLOG_Critical, "Unavailable feature GNUTLS");
     return eIO_NotSupported;
 }
 
-
 #endif /*HAVE_LIBGNUTLS*/
 
 
@@ -831,7 +906,7 @@ extern SOCKSSL NcbiSetupGnuTls(void)
 #endif /*HAVE_LIBGNUTLS*/
     };
 #ifndef HAVE_LIBGNUTLS
-    CORE_LOG(eLOG_Warning, "Unavailable feature GNUTLS");
+    CORE_LOG_X(29, eLOG_Warning, "Unavailable feature GNUTLS");
 #endif /*!HAVE_LIBGNUTLS*/
     return &kGnuTlsOps;
 }
@@ -846,3 +921,126 @@ extern NCBI_CRED NcbiCredGnuTls(void* xcred)
     }
     return cred;
 }
+
+
+#ifdef HAVE_LIBGNUTLS
+
+static gnutls_x509_crt_fmt_t x_GnuTlsX509Format(const gnutls_datum_t* data)
+{
+    assert(data);
+    /* a la logic in mbedTLS */
+    if (data->size  &&  data->data  &&  !data->data[data->size - 1]
+        &&  strstr((const char*) data->data, "-----BEGIN ")) {
+        return GNUTLS_X509_FMT_PEM;
+    }
+    return GNUTLS_X509_FMT_DER;
+}
+
+
+void NcbiDeleteGnuTlsCertCredentials(NCBI_CRED cred)
+{
+    if (cred->type / 100 == eNcbiCred_GnuTls / 100  &&  !(cred->type % 100)) {
+        if (cred->data) {
+            gnutls_certificate_free_credentials
+                ((gnutls_certificate_credentials_t) cred->data);
+        }
+    } else {
+        char who[80];
+        switch (cred->type / 100) {
+        case eNcbiCred_GnuTls / 100:
+            strcpy(who, "GNUTLS");
+            break;
+        case eNcbiCred_MbedTls / 100:
+            strcpy(who, "MBEDTLS");
+            break;
+        default:
+            sprintf(who, "TLS 0x%08X", cred->type);
+            break;
+        }
+        CORE_LOGF_X(30, eLOG_Critical,
+                    ("Deleting unknown certificate credentials (%s/%u)",
+                     who, cred->type % 100));
+        assert(0);
+    }
+    cred->type = (ENcbiCred) 0;
+    cred->data = 0;
+    free(cred);
+}
+
+
+NCBI_CRED NcbiCreateGnuTlsCertCredentials(const void* cert,
+                                          size_t      certsz,
+                                          const void* pkey,
+                                          size_t      pkeysz)
+{
+    struct SNcbiCred*                 ncbi_cred;
+    gnutls_certificate_credentials_t* gtls_xcred;
+    gnutls_x509_crt_t                 gtls_cert = 0;
+    gnutls_x509_privkey_t             gtls_pkey = 0;
+    gnutls_datum_t data;
+    int err;
+
+    if (!(ncbi_cred = (NCBI_CRED) calloc(2, sizeof(*ncbi_cred)))) {
+        CORE_LOGF_ERRNO_X(31, eLOG_Error, errno,
+                          ("Cannot allocate NCBI_CRED (%lu bytes)",
+                           (unsigned long)(2 * sizeof(*ncbi_cred))));
+        return 0;
+    }
+
+    ncbi_cred->type = eNcbiCred_GnuTls;
+    gtls_xcred = (gnutls_certificate_credentials_t*) &ncbi_cred->data;
+
+    err = gnutls_certificate_allocate_credentials(gtls_xcred);
+    if (err) {
+        CORE_LOG_ERRNO_EXX(32, eLOG_Error, err, gnutls_strerror(err),
+                           "GNUTLS cannot allocate certificate credentials");
+        *gtls_xcred = 0;
+        goto out;
+    }
+
+    data.data = (unsigned char*) cert;
+    data.size = certsz ? certsz : strlen((const char*) cert) + 1;
+    if ((err = gnutls_x509_crt_init (&gtls_cert)) != 0  ||
+        (err = gnutls_x509_crt_import(gtls_cert, &data,
+                                      x_GnuTlsX509Format(&data))) != 0) {
+        CORE_LOGF_ERRNO_EXX(33, eLOG_Error, err, gnutls_strerror(err),
+                            ("GNUTLS cannot %s X.509 certificate",
+                             gtls_cert ? "build" : "parse"));
+        goto out;
+    }
+
+    data.data = (unsigned char*) pkey;
+    data.size = pkeysz ? pkeysz : strlen((const char*) pkey) + 1;
+    if ((err = gnutls_x509_privkey_init (&gtls_pkey)) != 0  ||
+        (err = gnutls_x509_privkey_import(gtls_pkey, &data,
+                                          x_GnuTlsX509Format(&data))) != 0) {
+        CORE_LOGF_ERRNO_EXX(34, eLOG_Error, err, gnutls_strerror(err),
+                            ("GNUTLS cannot %s private key",
+                             gtls_pkey ? "build" : "parse"));
+        goto out;
+    }
+
+    err = gnutls_certificate_set_x509_key(*gtls_xcred,
+                                          &gtls_cert, 1, gtls_pkey);
+    if (err) {
+        CORE_LOG_ERRNO_EXX(35, eLOG_Error, err, gnutls_strerror(err),
+                           "GNUTLS certificate credentials setup failure");
+        goto out;
+    }
+
+    /* once they have been copied into xcred, they can be safely disposed of */
+    gnutls_x509_crt_deinit    (gtls_cert);
+    gnutls_x509_privkey_deinit(gtls_pkey);
+
+    return ncbi_cred;
+
+ out:
+    if (gtls_cert)
+        gnutls_x509_crt_deinit    (gtls_cert);
+    if (gtls_pkey)
+        gnutls_x509_privkey_deinit(gtls_pkey);
+    NcbiDeleteGnuTlsCertCredentials(ncbi_cred);
+    return 0;
+}
+
+#endif /*HAVE_LIBGNUTLS*/
