@@ -1,4 +1,4 @@
-/*  $Id: ncbi_http2_session.cpp 618648 2020-10-23 15:30:05Z sadyrovr $
+/*  $Id: ncbi_http2_session.cpp 674396 2023-10-25 18:31:34Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,9 +37,10 @@
 BEGIN_NCBI_SCOPE
 
 
-SH2S_Request::SStart::SStart(EReqMethod m, CUrl u, CHttpHeaders::THeaders h) :
+SH2S_Request::SStart::SStart(EReqMethod m, CUrl u, SUvNgHttp2_Tls::TCred c, CHttpHeaders::THeaders h) :
     method(m),
     url(move(u)),
+    cred(move(c)),
     headers(move(h))
 {
 }
@@ -84,6 +85,7 @@ const char* SH2S_Event<TBase>::GetTypeName() const
 }
 
 SH2S_ReaderWriter::SH2S_ReaderWriter(TUpdateResponse update_response, shared_ptr<TH2S_ResponseQueue> response_queue, TH2S_RequestEvent request) :
+    m_Io(SH2S_Io::GetInstance()),
     m_UpdateResponse(update_response),
     m_ResponseQueue(move(response_queue))
 {
@@ -292,10 +294,10 @@ constexpr size_t kWriteBufSize = 64 * 1024;
 constexpr uint32_t kMaxStreams = 200;
 
 template <class... TNgHttp2Cbs>
-SH2S_Session::SH2S_Session(uv_loop_t* loop, const SSocketAddress& address, bool https, TH2S_SessionsByQueues& sessions_by_queues, TNgHttp2Cbs&&... callbacks) :
+SH2S_Session::SH2S_Session(uv_loop_t* loop, const TAddrNCred& addr_n_cred, bool https, TH2S_SessionsByQueues& sessions_by_queues, TNgHttp2Cbs&&... callbacks) :
     SUvNgHttp2_SessionBase(
             loop,
-            address,
+            addr_n_cred,
             kReadBufSize,
             kWriteBufSize,
             https,
@@ -508,11 +510,13 @@ SH2S_IoCoordinator::~SH2S_IoCoordinator()
     m_Sessions.clear();
 }
 
-void SH2S_IoCoordinator::Process()
+void SH2S_IoCoordinator::Process(TH2S_RequestQueue& request_queue)
 {
+    m_Loop.Run(UV_RUN_NOWAIT);
+
     // Retrieve all events from the queue
     for (;;) {
-        auto queue_locked = SH2S_Io::GetInstance().request_queue.GetLock();
+        auto queue_locked = request_queue.GetLock();
 
         if (queue_locked->empty()) {
             break;
@@ -532,7 +536,7 @@ void SH2S_IoCoordinator::Process()
                     H2S_IOC_TRACE(response_queue << " pop " << outgoing);
                     auto& request = outgoing.GetStart();
 
-                    if (auto new_session = NewSession(request.url)) {
+                    if (auto new_session = NewSession(request)) {
                         if (new_session->Request(move(outgoing))) {
                             continue;
                         }
@@ -593,8 +597,9 @@ void SH2S_IoCoordinator::Process()
     m_Loop.Run(UV_RUN_NOWAIT);
 }
 
-SH2S_Session* SH2S_IoCoordinator::NewSession(const CUrl& url)
+SH2S_Session* SH2S_IoCoordinator::NewSession(const SH2S_Request::SStart& request)
 {
+    const auto& url = request.url;
     auto scheme = url.GetScheme();
     auto port = url.GetPort();
 
@@ -608,9 +613,10 @@ SH2S_Session* SH2S_IoCoordinator::NewSession(const CUrl& url)
         }
     }
 
-    SSocketAddress address(url.GetHost(), port);
+    SSocketAddress::SHost host(url.GetHost(), SSocketAddress::SHost::EName::eOriginal);
+    SH2S_Session::TAddrNCred addr_n_cred(SSocketAddress(move(host), port), request.cred);
     auto https = scheme == "https" || (scheme.empty() && (port == "443"));
-    auto range = m_Sessions.equal_range(address);
+    auto range = m_Sessions.equal_range(addr_n_cred);
 
     for (auto it = range.first; it != range.second; ++it) {
         if (!it->second.IsFull()) {
@@ -619,11 +625,22 @@ SH2S_Session* SH2S_IoCoordinator::NewSession(const CUrl& url)
     }
 
     // No such sessions yet or all are full
-    auto it = m_Sessions.emplace(piecewise_construct, forward_as_tuple(address), forward_as_tuple(&m_Loop, address, https, m_SessionsByQueues));
+    auto it = m_Sessions.emplace(piecewise_construct, forward_as_tuple(addr_n_cred), forward_as_tuple(&m_Loop, addr_n_cred, https, m_SessionsByQueues));
     return &it->second;
 }
 
-void CHttpSessionImpl2::UpdateResponse(CHttpRequest& req, CHttpHeaders::THeaders headers)
+CHttp2Session::CHttp2Session() :
+    CHttpSession_Base(CHttpSession_Base::eHTTP_2),
+    m_ApiLock(GetApiLock())
+{
+}
+
+CHttp2Session::TApiLock CHttp2Session::GetApiLock()
+{
+    return SH2S_Io::GetInstance();
+}
+
+void CHttp2Session::UpdateResponse(CHttpRequest& req, CHttpHeaders::THeaders headers)
 {
     int status_code = 0;
     auto status = headers.find(":status");
@@ -636,7 +653,7 @@ void CHttpSessionImpl2::UpdateResponse(CHttpRequest& req, CHttpHeaders::THeaders
     req.x_UpdateResponse(move(headers), status_code, {});
 }
 
-void CHttpSessionImpl2::StartRequest(CHttpSession_Base::EProtocol protocol, CHttpRequest& req, bool use_form_data)
+void CHttp2Session::x_StartRequest(CHttpSession_Base::EProtocol protocol, CHttpRequest& req, bool use_form_data)
 {
     if (protocol <= CHttpSession_Base::eHTTP_11) {
         req.x_InitConnection(use_form_data);
@@ -647,15 +664,20 @@ void CHttpSessionImpl2::StartRequest(CHttpSession_Base::EProtocol protocol, CHtt
 
     auto update_response = [&](CHttpHeaders::THeaders headers) { UpdateResponse(req, move(headers)); };
     auto response_queue = make_shared<TH2S_ResponseQueue>();
-    TH2S_RequestEvent request(SH2S_Request::SStart(req.m_Method, req.m_Url, req.m_Headers->Get()), response_queue);
+
+    const auto& req_cred = req.m_Credentials;
+    SUvNgHttp2_Tls::TCred cred(req_cred ? req_cred->GetCert() : string(), req_cred ? req_cred->GetPKey() : string());
+
+    // Cannot just pass req itself (accessing private members here)
+    TH2S_RequestEvent request(SH2S_Request::SStart(req.m_Method, req.m_Url, move(cred), req.m_Headers->Get()), response_queue);
 
     unique_ptr<IReaderWriter> rw(new SH2S_ReaderWriter(move(update_response), move(response_queue), move(request)));
     auto stream = make_shared<CRWStream>(rw.release(), 0, nullptr, CRWStreambuf::fOwnAll);
 
-    req.x_InitConnection2(move(stream), /* TODO: */ false);
+    req.x_InitConnection2(move(stream));
 }
 
-bool CHttpSessionImpl2::Downgrade(CHttpResponse& resp, CHttpSession_Base::EProtocol& protocol)
+bool CHttp2Session::x_Downgrade(CHttpResponse& resp, CHttpSession_Base::EProtocol& protocol) const
 {
     if (resp.GetStatusCode() || (protocol <= CHttpSession_Base::eHTTP_11)) {
         return false;
