@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_PRIV__H
 #define CONNECT___NCBI_PRIV__H
 
-/* $Id: ncbi_priv.h 627938 2021-03-22 18:18:42Z ivanov $
+/* $Id: ncbi_priv.h 668280 2023-06-01 15:35:47Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,13 +50,13 @@
  *    macro:           NCBI_CONNECT_SRAND_ADDEND
  * App name / NCBI ID / DTab support
  *    private globals: g_CORE_GetAppName
+ *                     g_CORE_GetReferer
  *                     g_CORE_GetRequestID
  *                     g_CORE_GetRequestDtab
  *
  */
 
-
-//#define NCBI_MONKEY
+/*#define NCBI_MONKEY*/
 
 #include "ncbi_assert.h"
 #include <connect/ncbi_util.h>
@@ -92,21 +92,21 @@ extern "C" {
 /* Here are only error codes used in C sources.  For error codes used in
  * C++ sources (in C++ Toolkit) see include/connect/error_codes.hpp.
  */
-NCBI_C_DEFINE_ERRCODE_X(Connect_Conn,     301,  36);
-NCBI_C_DEFINE_ERRCODE_X(Connect_Socket,   302, 160);
-NCBI_C_DEFINE_ERRCODE_X(Connect_Util,     303,   9);
-NCBI_C_DEFINE_ERRCODE_X(Connect_LBSM,     304,  30);
-NCBI_C_DEFINE_ERRCODE_X(Connect_FTP,      305,  13);
-NCBI_C_DEFINE_ERRCODE_X(Connect_SMTP,     306,  33);
-NCBI_C_DEFINE_ERRCODE_X(Connect_HTTP,     307,  25);
-NCBI_C_DEFINE_ERRCODE_X(Connect_Service,  308,  10);
-NCBI_C_DEFINE_ERRCODE_X(Connect_HeapMgr,  309,  34);
-NCBI_C_DEFINE_ERRCODE_X(Connect_LBOS,     310, 600); /*safe upper bound*/
-NCBI_C_DEFINE_ERRCODE_X(Connect_Mghbn,    311,  16);
-NCBI_C_DEFINE_ERRCODE_X(Connect_Crypt,    312,   5);
-NCBI_C_DEFINE_ERRCODE_X(Connect_LocalIP,  313,   5);
-NCBI_C_DEFINE_ERRCODE_X(Connect_Namerd,   314,  10);
-NCBI_C_DEFINE_ERRCODE_X(Connect_Linkerd,  315,   5);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Conn,          301,  36);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Socket,        302, 166);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Util,          303,  13);
+NCBI_C_DEFINE_ERRCODE_X(Connect_LBSM,          304,  30);
+NCBI_C_DEFINE_ERRCODE_X(Connect_FTP,           305,  13);
+NCBI_C_DEFINE_ERRCODE_X(Connect_SMTP,          306,  33);
+NCBI_C_DEFINE_ERRCODE_X(Connect_HTTP,          307,  26);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Service,       308,  10);
+NCBI_C_DEFINE_ERRCODE_X(Connect_HeapMgr,       309,  34);
+NCBI_C_DEFINE_ERRCODE_X(Connect_TLS,           310,  50);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Mghbn,         311,  16);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Crypt,         312,   5);
+NCBI_C_DEFINE_ERRCODE_X(Connect_LocalIP,       313,   5);
+NCBI_C_DEFINE_ERRCODE_X(Connect_NamerdLinkerd, 314,  14);
+
 
 /** Make one identifier from 2 parts */
 #define NCBI_C_CONCAT_IDENTIFIER(prefix, postfix)  prefix##postfix
@@ -290,7 +290,7 @@ extern NCBI_XCONNECT_EXPORT int g_NCBI_CoreCheckUnlock(void);
 
 extern NCBI_XCONNECT_EXPORT REG g_CORE_Registry;
 
-/* Always use the following macros and functions to access "g_CORE_Registry",
+/* Always use the following macros to access "g_CORE_Registry",
  * do not access/change it directly!
  */
 
@@ -300,7 +300,6 @@ extern NCBI_XCONNECT_EXPORT REG g_CORE_Registry;
 #define CORE_REG_SET(section, name, value, storage)                 \
     g_CORE_RegistrySET(section, name, value, storage)
 
-
 /* (private, to be used exclusively by the above macro CORE_REG_GET) */
 extern NCBI_XCONNECT_EXPORT const char* g_CORE_RegistryGET
 (const char*  section,
@@ -331,7 +330,7 @@ typedef enum {
 } ECORE_Set;
 typedef unsigned int TCORE_Set;
 
-
+/* Track settings modified from the user-land */
 extern TCORE_Set g_CORE_Set;
 
 
@@ -346,16 +345,25 @@ extern NCBI_XCONNECT_EXPORT unsigned int g_NCBI_ConnectSrandAddend(void);
 
 /******************************************************************************
  *  App name support (may return NULL; gets converted to "" at the user level)
+ *  The name is assumed to have static storage (not to be free()'d after use).
  */
 
 typedef const char* (*FNcbiGetAppName)(void);
 extern NCBI_XCONNECT_EXPORT FNcbiGetAppName g_CORE_GetAppName;
 
 
+/******************************************************************************
+ *  Referer support (return NULL or non-empty dynamically malloc()'ed string)
+ */
+
+typedef const char* (*FNcbiGetReferer)(void);
+extern NCBI_XCONNECT_EXPORT FNcbiGetReferer g_CORE_GetReferer;
+
+
 /******************************************************************************
  *  NCBI request ID support (return "as is" to the user)
  *  Return NULL on error;  otherwise, the returned ID is a non-empty string
- *  allocated on the heap, and must be free()'d when no longer needed.
+ *  allocated on heap, and must be free()'d when no longer needed.
  */
 
 typedef char* (*FNcbiGetRequestID)(ENcbiRequestID);
@@ -369,7 +377,6 @@ typedef const char* (*FNcbiGetRequestDtab)(void);
 extern NCBI_XCONNECT_EXPORT FNcbiGetRequestDtab g_CORE_GetRequestDtab;
 
 
-
 /******************************************************************************
  *  Miscellanea
  */
