@@ -1,4 +1,4 @@
-/* $Id: ncbi_sendmail.c 606555 2020-04-23 14:38:32Z lavr $
+/* $Id: ncbi_sendmail.c 681783 2024-04-15 14:14:32Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -267,7 +267,7 @@ static void s_MakeFrom(char* buf, size_t size, const char* from,
 
 
 static STimeout       s_MxTimeout;
-static char           s_MxHost[256];
+static char           s_MxHost[CONN_HOST_LEN + 1];
 static unsigned short s_MxPort;
 
 
@@ -495,7 +495,7 @@ extern const char* CORE_SendMailEx(const char*          to,
     if ((status = SOCK_CreateEx(info->mx_host, info->mx_port,
                                 &info->mx_timeout, &sock,
                                 0, 0, log)) != eIO_Success) {
-        sprintf(buffer, "%s:%hu (%s)", info->mx_host, info->mx_port,
+        sprintf(buffer, "%.512s:%hu (%s)", info->mx_host, info->mx_port,
                 IO_StatusStr(status));
         SENDMAIL_RETURN2(8, "Cannot connect to sendmail", buffer);
     }
@@ -658,7 +658,7 @@ extern const char* CORE_SendMailEx(const char*          to,
             const char* error = "Custom header write error";
             assert(status != eIO_Timeout);
             if (status != eIO_Success)
-                strncpy0(buffer, IO_StatusStr(status), sizeof(buffer) - 1);
+                strcpy(buffer, IO_StatusStr(status));
             else if (SENDMAIL_READ_RESPONSE(0, buffer))
                 error = "Spurious response while writing custom header";
             SENDMAIL_RETURN2(22, error, buffer);
@@ -705,7 +705,7 @@ extern const char* CORE_SendMailEx(const char*          to,
             const char* error = "Message body write error";
             assert(status == eIO_Timeout);
             if (status != eIO_Success)
-                strncpy0(buffer, IO_StatusStr(status), sizeof(buffer) - 1);
+                strcpy(buffer, IO_StatusStr(status));
             else if (SENDMAIL_READ_RESPONSE(0, buffer))
                 error = "Spurious response while writing message body";
             SENDMAIL_RETURN2(26, error, buffer);
