@@ -1,4 +1,4 @@
-/* $Id: ncbi_service_connector.c 606402 2020-04-21 14:50:28Z lavr $
+/* $Id: ncbi_service_connector.c 656746 2022-09-27 15:28:18Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,29 +44,32 @@
 
 
 typedef struct SServiceConnectorTag {
-    SMetaConnector      meta;           /* Low level comm.conn and its VT    */
+    SMetaConnector      meta;           /* Lower level comm.conn. and its VT */
     const char*         type;           /* Verbal connector type             */
     const char*         descr;          /* Verbal connector description      */
-    const SConnNetInfo* net_info;       /* R/O conn info with translated svc */
+    const SConnNetInfo* net_info;       /* R/O connection parameters         */
     const char*         user_header;    /* User header currently set         */
 
-    SERV_ITER           iter;           /* Dispatcher information            */
+    SERV_ITER           iter;           /* Dispatcher information (when open)*/
 
     SSERVICE_Extra      extra;          /* Extra params as passed to ctor    */
-    EIO_Status          status;         /* Status of last I/O op             */
+    EIO_Status          status;         /* Status of last I/O                */
 
-    unsigned short      retry;          /* Open retry count since last okay  */
     TSERV_TypeOnly      types;          /* Server types w/o any specials     */
-    unsigned            reset:1;        /* Non-zero if iter was just reset   */
-    unsigned            warned:1;       /* Non-zero when needed adj via HTTP */
-    unsigned            unused:5;
-    unsigned            secure:1;       /* Set when must start ssl on SOCK   */
+    unsigned short      retry;          /* Open retry count since last okay  */
 
     ticket_t            ticket;         /* Network byte order (none if zero) */
+
     unsigned int        host;           /* Parsed connection info... (n.b.o) */
     unsigned short      port;           /*                       ... (h.b.o) */
 
-    const char          service[1];     /* Untranslated (orig.) service name */
+    /* flags */
+    unsigned            reset:1;        /* Non-zero if iter was just reset   */
+    unsigned            warned:1;       /* Non-zero when needed adj via HTTP */
+    unsigned            unused:5;
+    unsigned            secure:1;       /* Set when must start SSL on SOCK   */
+
+    const char          name[1];        /* Original (unsubstituted) svc name */
 } SServiceConnector;
 
 
@@ -97,10 +100,10 @@ extern "C" {
 
 static int/*bool*/ s_OpenDispatcher(SServiceConnector* uuu)
 {
-    if (!(uuu->iter = SERV_Open(uuu->service, uuu->types,
+    if (!(uuu->iter = SERV_Open(uuu->name, uuu->types,
                                 SERV_LOCALHOST, uuu->net_info))) {
         CORE_LOGF_X(5, eLOG_Error,
-                    ("[%s]  Service not found", uuu->service));
+                    ("[%s]  Service not found", uuu->name));
         return 0/*false*/;
     }
     uuu->reset = 1/*true*/;
@@ -168,7 +171,7 @@ static EHTTP_HeaderParse s_ParseHeader(const char* header,
 #if defined(_DEBUG)  &&  !defined(NDEBUG)
                 if (uuu->net_info->debug_printout) {
                     CORE_LOGF_X(2, eLOG_Note,
-                                ("[%s]  Fallback to stateless", uuu->service));
+                                ("[%s]  Fallback to stateless", uuu->name));
                 }
 #endif /*_DEBUG && !NDEBUG*/
             } else {
@@ -243,6 +246,8 @@ static int/*bool*/ s_IsContentTypeDefined(const char*         service,
                                           EMIME_SubType       mime_s,
                                           EMIME_Encoding      mime_e)
 {
+    static const char   kContentType[] = "content-type:";
+    static const size_t kCTLen         = sizeof(kContentType)-1;
     const char* s;
 
     assert(net_info);
@@ -251,7 +256,8 @@ static int/*bool*/ s_IsContentTypeDefined(const char*         service,
             s++;
         if (!*s)
             break;
-        if (strncasecmp(s, "content-type: ", 14) == 0) {
+        if (strncasecmp(s, kContentType, kCTLen) == 0
+            &&  isspace((unsigned char) s[kCTLen])) {
 #if defined(_DEBUG)  &&  !defined(NDEBUG)
             EMIME_Type     m_t;
             EMIME_SubType  m_s;
@@ -270,7 +276,7 @@ static int/*bool*/ s_IsContentTypeDefined(const char*         service,
                 const char* c;
                 size_t len;
                 char* t;
-                for (s += 14;  *s;  s++) {
+                for (s += kCTLen + 1;  *s;  ++s) {
                     if (!isspace((unsigned char)(*s)))
                         break;
                 }
@@ -328,9 +334,8 @@ static const char* s_AdjustNetParams(const char*    service,
         ConnNetInfo_SetPath(net_info, cgi_path);
     if (args)
         ConnNetInfo_SetArgs(net_info, args);
-    ConnNetInfo_DeleteAllArgs(net_info, cgi_args);
 
-    if (ConnNetInfo_PrependArg(net_info, cgi_args, 0)) {
+    if (ConnNetInfo_PreOverrideArg(net_info, cgi_args, 0)) {
         size_t sh_len = static_header ? strlen(static_header) : 0;
         size_t eh_len = extend_header ? strlen(extend_header) : 0;
         char   c_t[CONN_CONTENT_TYPE_LEN+1];
@@ -397,11 +402,11 @@ static SSERV_InfoCPtr s_GetNextInfo(SServiceConnector* uuu, int/*bool*/ http)
 }
 
 
-static char* x_HostPort(const char* host, unsigned short nport)
+static char* x_HostPort(const char* host, unsigned short aport)
 {
     char*  hostport, port[16];
     size_t hostlen = strlen(host);
-    size_t portlen = (size_t) sprintf(port, ":%hu", nport) + 1;
+    size_t portlen = (size_t) sprintf(port, ":%hu", aport) + 1;
     hostport = (char*) malloc(hostlen + portlen);
     if (hostport) {
         memcpy(hostport,           host, hostlen);
@@ -431,7 +436,8 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
 
     flags |= (net_info->debug_printout == eDebugPrintout_Data
               ? fSOCK_LogOn : fSOCK_LogDefault);
-    if (net_info->http_proxy_host[0]  &&  net_info->http_proxy_port) {
+    if (net_info->http_proxy_host[0]  &&  net_info->http_proxy_port
+        &&  !net_info->http_proxy_only) {
         /* NB: ideally, should have pushed data:size here if proxy not buggy */
         *status = HTTP_CreateTunnel(net_info, fHTTP_NoAutoRetry, &sock);
         assert(!sock ^ !(*status != eIO_Success));
@@ -477,6 +483,8 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
             net_info->http_version = 0;
             net_info->http_push_auth = 0;
             net_info->http_proxy_leak = 0;
+            net_info->http_proxy_skip = 0;
+            net_info->http_proxy_only = 0;
             net_info->user[0] = '\0';
             net_info->pass[0] = '\0';
             net_info->path[0] = '\0';
@@ -510,8 +518,6 @@ static CONNECTOR s_SocketConnectorBuilder(SConnNetInfo* net_info,
         }
     }
     if (!(c = SOCK_CreateConnectorOnTopEx(sock, 1/*own*/, hostport))) {
-        if (*status == eIO_Success)
-            *status  = eIO_Unknown;
         SOCK_Abort(sock);
         SOCK_Close(sock);
     }
@@ -526,6 +532,7 @@ static int/*bool*/ x_SetHostPort(SConnNetInfo* net_info,
 
     if (vhost) {
         char* tag;
+        assert(info->vhost);
         if (!(tag = (char*) malloc(sizeof(kHttpHostTag) + info->vhost)))
             return 0/*failure*/;
         sprintf(tag, "%s%.*s", kHttpHostTag, (int) info->vhost, vhost);
@@ -534,8 +541,13 @@ static int/*bool*/ x_SetHostPort(SConnNetInfo* net_info,
             return 0/*failure*/;
         }
         free(tag);
+        if (!net_info->http_proxy_host[0]  ||  !net_info->http_proxy_port)
+            vhost = 0;
     }
-    if (info->host == SOCK_HostToNetLong((unsigned int)(-1L))) {
+    if (vhost) {
+        assert((size_t) info->vhost < sizeof(net_info->host));
+        strncpy0(net_info->host, vhost, (size_t) info->vhost);
+    } else if (info->host == SOCK_HostToNetLong((unsigned int)(-1))) {
         int/*bool*/ ipv6 = !NcbiIsIPv4(&info->addr);
         char* end = NcbiAddrToString(net_info->host + ipv6,
                                      sizeof(net_info->host) - (size_t)(2*ipv6),
@@ -549,8 +561,11 @@ static int/*bool*/ x_SetHostPort(SConnNetInfo* net_info,
             *end++ = ']';
             *end = '\0';
         }
-    } else
+    } else if (info->host) {
         SOCK_ntoa(info->host, net_info->host, sizeof(net_info->host));
+        assert(*net_info->host);
+    } else
+        *net_info->host = '\0';
 
     net_info->port = info->port;
     return 1/*success*/;
@@ -584,6 +599,7 @@ static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
     SSERV_InfoCPtr     info;
 
     assert(n  ||  uuu->extra.adjust);
+    assert(net_info != uuu->net_info);
     assert(!net_info->firewall  ||  net_info->stateless);
 
     if (n == (unsigned int)(-1))
@@ -603,7 +619,7 @@ static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
     switch (info->type) {
     case fSERV_Ncbid:
         user_header = "Connection-Mode: STATELESS\r\n"; /*default*/
-        user_header = s_AdjustNetParams(uuu->service, net_info,
+        user_header = s_AdjustNetParams(uuu->name, net_info,
                                         eReqMethod_Post,
                                         NCBID_WEBPATH,
                                         SERV_NCBID_ARGS(&info->u.ncbid),
@@ -616,7 +632,7 @@ static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
     case fSERV_HttpGet:
     case fSERV_HttpPost:
         user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-        user_header = s_AdjustNetParams(uuu->service, net_info,
+        user_header = s_AdjustNetParams(uuu->name, net_info,
                                         info->type == fSERV_HttpPost
                                         ?  eReqMethod_Post
                                         : (info->type == fSERV_HttpGet
@@ -632,7 +648,7 @@ static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
     case fSERV_Firewall:
     case fSERV_Standalone:
         user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-        user_header = s_AdjustNetParams(uuu->service, net_info,
+        user_header = s_AdjustNetParams(uuu->name, net_info,
                                         eReqMethod_Any,
                                         uuu->net_info->path, 0,
                                         ConnNetInfo_GetArgs(uuu->net_info),
@@ -678,12 +694,11 @@ static int/*bool*/ s_Adjust(SConnNetInfo* net_info,
 
 static void x_SetDefaultReferer(SConnNetInfo* net_info, SERV_ITER iter)
 {
-    const char* mapper = SERV_MapperName(iter);
     char* str, *referer;
+    const char* mapper = SERV_MapperName(iter);
+    assert(iter  &&  !net_info->http_referer);
+    assert(mapper  &&  *mapper);
 
-    assert(!net_info->http_referer);
-    if (!mapper  ||  !*mapper)
-        return;
     if (strcasecmp(mapper, "DISPD") == 0) {
         /* the swap is to make sure URL prints correctly */
         EBURLScheme scheme     = net_info->scheme;
@@ -723,7 +738,6 @@ static void x_SetDefaultReferer(SConnNetInfo* net_info, SERV_ITER iter)
             strcpy(strcpy(str, "?service="/*9*/) + 9, name);
     } else
         return;
-    assert(referer);
     net_info->http_referer = referer;
 }
 
@@ -750,6 +764,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
     char*       iter_header;
     EReqMethod  req_method;
 
+    *status = eIO_Success;
     assert(net_info->firewall  ||  info);
     ConnNetInfo_DeleteUserHeader(net_info, kHttpHostTag);
     if (!net_info->http_referer)
@@ -761,7 +776,6 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
            We know the connection point, so let's try to use it! */
         if ((info->type != fSERV_Standalone  ||  !net_info->stateless)
             &&  !x_SetHostPort(net_info, info)) {
-            *status = eIO_Unknown;
             return 0;
         }
 
@@ -776,7 +790,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
                 ? "Connection-Mode: STATELESS\r\n" /*default*/
                 /* We will be waiting for conn-info back */
                 : "Connection-Mode: STATEFUL\r\n";
-            user_header = s_AdjustNetParams(uuu->service, net_info, req_method,
+            user_header = s_AdjustNetParams(uuu->name, net_info, req_method,
                                             NCBID_WEBPATH,
                                             SERV_NCBID_ARGS(&info->u.ncbid),
                                             0, user_header, info->mime_t,
@@ -785,13 +799,13 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         case fSERV_Http:
         case fSERV_HttpGet:
         case fSERV_HttpPost:
-            /* Connection directly to CGI */
+            /* Connection directly to a CGI */
             net_info->stateless = 1/*true*/;
             req_method  = info->type == fSERV_HttpGet
                 ? eReqMethod_Get : (info->type == fSERV_HttpPost
                                     ? eReqMethod_Post : eReqMethod_Any);
             user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-            user_header = s_AdjustNetParams(uuu->service, net_info, req_method,
+            user_header = s_AdjustNetParams(uuu->name, net_info, req_method,
                                             SERV_HTTP_PATH(&info->u.http),
                                             SERV_HTTP_ARGS(&info->u.http),
                                             0, user_header, info->mime_t,
@@ -809,7 +823,7 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
             if (!net_info->scheme)
                 net_info->scheme = eURL_Https;
             user_header = "Client-Mode: STATELESS_ONLY\r\n"; /*default*/
-            user_header = s_AdjustNetParams(uuu->service, net_info,
+            user_header = s_AdjustNetParams(uuu->name, net_info,
                                             eReqMethod_Any, 0, 0,
                                             0, user_header, info->mime_t,
                                             info->mime_s, info->mime_e, 0);
@@ -856,15 +870,13 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         user_header = (net_info->stateless
                        ? "Client-Mode: STATELESS_ONLY\r\n" /*default*/
                        : "Client-Mode: STATEFUL_CAPABLE\r\n");
-        user_header = s_AdjustNetParams(uuu->service, net_info,
+        user_header = s_AdjustNetParams(uuu->name, net_info,
                                         req_method, 0, 0,
                                         0, user_header, mime_t,
                                         mime_s, mime_e, 0);
     }
-    if (!user_header) {
-        *status = eIO_Unknown;
+    if (!user_header)
         return 0;
-    }
 
     if ((iter_header = SERV_Print(uuu->iter, net_info, but_last)) != 0) {
         size_t uh_len;
@@ -882,14 +894,17 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
         user_header = 0; /* special case of assignment of literal "" */
 
     if (uuu->user_header) {
+        /* delete previously set user header first */
         ConnNetInfo_DeleteUserHeader(net_info, uuu->user_header);
         free((void*) uuu->user_header);
     }
+    /* then set a new one */
     uuu->user_header = user_header;
-    if (user_header && !ConnNetInfo_OverrideUserHeader(net_info, user_header)){
-        *status = eIO_Unknown;
+    if (user_header && !ConnNetInfo_OverrideUserHeader(net_info, user_header))
+        return 0;
+
+    if (!ConnNetInfo_SetupStandardArgs(net_info, uuu->iter->name))
         return 0;
-    }
 
     ConnNetInfo_ExtendUserHeader
         (net_info, "User-Agent: NCBIServiceConnector/"
@@ -901,12 +916,11 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
 #endif /*NCBI_CXX_TOOLKIT*/
          );
 
-    *status = eIO_Success;
     if (!net_info->stateless  &&  (net_info->firewall            ||
                                    info->type == fSERV_Firewall  ||
                                    info->type == fSERV_Ncbid)) {
         /* Auxiliary HTTP connector first */
-        EIO_Status temp = eIO_Success;
+        EIO_Status aux_status;
         CONNECTOR c;
         CONN conn;
 
@@ -922,57 +936,54 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
                                    s_ParseHeaderNoUCB, uuu/*user_data*/,
                                    0/*adjust*/, 0/*cleanup*/);
         /* Wait for connection info back from dispatcher */
-        if (c  &&  (temp = CONN_Create(c, &conn)) == eIO_Success) {
+        if (c  &&  (*status = CONN_Create(c, &conn)) == eIO_Success) {
             CONN_SetTimeout(conn, eIO_Open,      timeout);
             CONN_SetTimeout(conn, eIO_ReadWrite, timeout);
             CONN_SetTimeout(conn, eIO_Close,     timeout);
             /* Send all the HTTP data... */
-            if ((temp = CONN_Flush(conn)) != eIO_Success)
-                *status = temp;
+            *status = CONN_Flush(conn);
             /* ...then trigger the header callback */
-            if ((temp = CONN_Close(conn)) != eIO_Success  &&
-                 temp                     != eIO_Closed   &&
-                *status < temp) {
-                *status = temp;
+            aux_status = CONN_Close(conn);
+            if (aux_status != eIO_Success  &&
+                aux_status != eIO_Closed   &&
+                *status < aux_status) {
+                *status = aux_status;
             }
         } else {
             /* can only happen if we're out of memory */
             const char* error;
             if (c) {
-                error = IO_StatusStr(temp);
+                error = IO_StatusStr(*status);
                 x_DestroyConnector(c);
-                *status = temp;
             } else
                 error = 0;
             CORE_LOGF_X(4, eLOG_Error,
                         ("[%s]  Unable to create auxiliary HTTP %s%s%s",
-                         uuu->service, c ? "connection" : "connector",
+                         uuu->name, c ? "connection" : "connector",
                          error  &&  *error ? ": " : "", error ? error : ""));
             assert(!uuu->host);
         }
         if (uuu->host == (unsigned int)(-1)) {
             assert(net_info->firewall  ||  info->type == fSERV_Firewall);
             assert(!net_info->stateless);
+            assert(!uuu->port);
             net_info->stateless = 1/*true*/;
             /* Fallback to try to use stateless mode instead */
             return s_Open(uuu, timeout, info, net_info, status);
         }
         if (!uuu->host  ||  !uuu->port) {
             /* no connection info found */
-            if (*status == eIO_Success)
-                *status  = eIO_Unknown;
             if (!net_info->scheme)
                 net_info->scheme = eURL_Http;
             ConnNetInfo_SetArgs(net_info, 0);
             assert(!uuu->descr);
-            uuu->descr = ConnNetInfo_URL(net_info);
             return 0;
         }
         if (net_info->firewall == eFWMode_Fallback
             &&  !SERV_IsFirewallPort(uuu->port)) {
             CORE_LOGF_X(9, eLOG_Warning,
                         ("[%s]  Firewall port :%hu is not in the fallback set",
-                         uuu->service, uuu->port));
+                         uuu->name, uuu->port));
         }
         ConnNetInfo_DeleteUserHeader(net_info, uuu->user_header);
         SOCK_ntoa(uuu->host, net_info->host, sizeof(net_info->host));
@@ -991,7 +1002,6 @@ static CONNECTOR s_Open(SServiceConnector* uuu,
     else if (!net_info->scheme)
         net_info->scheme = eURL_Http;
     assert(!uuu->descr);
-    uuu->descr = ConnNetInfo_URL(net_info);
     return !uuu->extra.adjust
         ||  uuu->extra.adjust(net_info, uuu->extra.data, (unsigned int)(-1))
         ? HTTP_CreateConnectorEx(net_info,
@@ -1045,7 +1055,7 @@ static EIO_Status s_Close(CONNECTOR       connector,
 
     if (uuu->meta.list) {
         SMetaConnector* meta = connector->meta;
-        METACONN_Remove(meta, uuu->meta.list);
+        verify(METACONN_Remove(meta, uuu->meta.list) == eIO_Success);
         uuu->meta.list = 0;
         s_Reset(meta, connector);
     }
@@ -1057,7 +1067,7 @@ static EIO_Status s_Close(CONNECTOR       connector,
 static const char* s_VT_GetType(CONNECTOR connector)
 {
     SServiceConnector* uuu = (SServiceConnector*) connector->handle;
-    return uuu->type ? uuu->type : uuu->service;
+    return uuu->type ? uuu->type : uuu->name;
 }
 
 
@@ -1078,8 +1088,10 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
     for (uuu->retry = 0;  uuu->retry < uuu->net_info->max_try;  uuu->retry++) {
         SConnNetInfo* net_info;
         SSERV_InfoCPtr info;
+        const char* type;
         int stateless;
         CONNECTOR c;
+        char* descr;
 
         assert(!uuu->meta.list  &&  status != eIO_Success);
 
@@ -1088,7 +1100,7 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
 
         if (!(info = s_GetNextInfo(uuu, 0/*any*/))
             &&  (!uuu->net_info->firewall
-                 ||  strcasecmp(SERV_MapperName(uuu->iter), "local") == 0)) {
+                 ||  strcmp(SERV_MapperName(uuu->iter), "LOCAL") == 0)) {
             break;
         }
         if (uuu->type) {
@@ -1103,70 +1115,93 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
             status = eIO_Unknown;
             break;
         }
+        if (info  &&  strcmp(SERV_MapperName(uuu->iter), "LINKERD") == 0) {
+            /* LINKERD *is* a proxy, so just drop HTTP proxy (if any) here */
+            net_info->http_proxy_host[0] = '\0';
+            net_info->http_proxy_port    =   0;
+            net_info->http_proxy_user[0] = '\0';
+            net_info->http_proxy_pass[0] = '\0';
+            net_info->http_proxy_leak    =   0;
+            net_info->http_proxy_skip    =   0;
+            net_info->http_proxy_only    =   1;
+        }
 
         c = s_Open(uuu, timeout, info, net_info, &status);
+        descr = uuu->descr ? (char*) uuu->descr : ConnNetInfo_URL(net_info);
         stateless = net_info->stateless;
 
         ConnNetInfo_Destroy(net_info);
 
         if (!c) {
             if (status == eIO_Success)
-                status  = eIO_Closed;
+                status  = eIO_Unknown;
+            uuu->descr = descr;
             continue;
         }
-
-        /* Setup the new connector on a temporary meta-connector... */
-        memset(&uuu->meta, 0, sizeof(uuu->meta));
-        if ((status = METACONN_Insert(&uuu->meta, c)) != eIO_Success) {
-            x_DestroyConnector(c);
-            continue;
-        }
-        /* ...then link it in using current connection's meta */
+        {{
+            EIO_Status meta_status;
+            /* Setup the new connector on a temporary meta-connector... */
+            memset(&uuu->meta, 0, sizeof(uuu->meta));
+            meta_status = METACONN_Insert(&uuu->meta, c);
+            if (meta_status != eIO_Success) {
+                x_DestroyConnector(c);
+                status = meta_status;
+                uuu->descr = descr;
+                continue;
+            }
+        }}
+        /* ...then link it in using the current connection's meta */
         assert(c->meta == &uuu->meta);
         c->next = meta->list;
         meta->list = c;
 
-        if (!uuu->descr  &&  uuu->meta.descr)
+        if (!uuu->meta.descr)
+            uuu->descr = descr;
+        else if (!uuu->descr) {
+            if (descr)
+                free(descr);
             CONN_SET_METHOD(meta, descr,  uuu->meta.descr, uuu->meta.c_descr);
+        }
         CONN_SET_METHOD    (meta, wait,   uuu->meta.wait,  uuu->meta.c_wait);
         CONN_SET_METHOD    (meta, write,  uuu->meta.write, uuu->meta.c_write);
         CONN_SET_METHOD    (meta, flush,  uuu->meta.flush, uuu->meta.c_flush);
         CONN_SET_METHOD    (meta, read,   uuu->meta.read,  uuu->meta.c_read);
         CONN_SET_METHOD    (meta, status, uuu->meta.status,uuu->meta.c_status);
 
-        if (uuu->meta.get_type) {
-            const char* temp;
-            if ((temp = uuu->meta.get_type(uuu->meta.c_get_type)) != 0) {
-                size_t slen = strlen(uuu->service);
-                size_t tlen = strlen(temp);
-                char*  type = (char*) malloc(slen + tlen + 2);
-                if (type) {
-                    memcpy(type,        uuu->service, slen);
-                    type[slen++]      = '/';
-                    memcpy(type + slen, temp,         tlen);
-                    type[slen + tlen] = '\0';
-                    uuu->type = type;
-                }
+        type = uuu->meta.get_type
+            ? uuu->meta.get_type(uuu->meta.c_get_type) : 0;
+        if (type) {
+            size_t nlen = strlen(uuu->name);
+            size_t tlen = strlen(type);
+            char*  temp = (char*) malloc(nlen + tlen + 2);
+            if (temp) {
+                memcpy(temp,        uuu->name, nlen);
+                temp[nlen++]      = '/';
+                memcpy(temp + nlen, type,      tlen);
+                temp[nlen + tlen] = '\0';
+                uuu->type = temp;
             }
         }
 
-        if (!uuu->meta.open) {
-            s_Close(connector, timeout, 0/*retain*/);
-            status = eIO_NotSupported;
-            continue;
+        if (status == eIO_Success) {
+            if (!uuu->meta.open) {
+                s_Close(connector, timeout, 0/*retain*/);
+                status = eIO_NotSupported;
+                continue;
+            }
+
+            status = uuu->meta.open(uuu->meta.c_open, timeout);
+            if (status == eIO_Success)
+                break;
         }
-
-        status = uuu->meta.open(uuu->meta.c_open, timeout);
-        if (status == eIO_Success)
-            break;
-
         if (!stateless
-            &&  (uuu->net_info->firewall  ||  info->type == fSERV_Firewall)) {
+            &&  (uuu->net_info->firewall  ||  info->type == fSERV_Firewall)
+            &&  type  &&  strcmp(type, g_kNcbiSockNameAbbr) == 0) {
             static const char kFWDLink[] = CONN_FWD_LINK;
             CORE_LOGF_X(6, eLOG_Error,
                         ("[%s]  %s connection failure (%s) usually"
                          " indicates possible firewall configuration"
-                         " problem(s); please consult <%s>", uuu->service,
+                         " problem(s); please consult <%s>", uuu->name,
                          uuu->net_info->firewall ? "Firewall":"Stateful relay",
                          IO_StatusStr(status), kFWDLink));
         }
@@ -1177,7 +1212,7 @@ static EIO_Status s_VT_Open(CONNECTOR connector, const STimeout* timeout)
         &&  uuu->retry > 1  &&  uuu->retry >= uuu->net_info->max_try) {
         CORE_LOGF_X(10, eLOG_Error,
                     ("[%s]  Too many failed attempts (%hu), giving up",
-                     uuu->service, uuu->retry));
+                     uuu->name, uuu->retry));
     }
     uuu->status = status;
     return status;
@@ -1232,27 +1267,27 @@ static void s_Destroy(CONNECTOR connector)
  ***********************************************************************/
 
 extern CONNECTOR SERVICE_CreateConnectorEx
-(const char*           service,
+(const char*           name,
  TSERV_Type            types,
  const SConnNetInfo*   net_info,
  const SSERVICE_Extra* extra)
 {
-    char*              x_service;
     SConnNetInfo*      x_net_info;
-    CONNECTOR          ccc;
+    char*              x_name;
     size_t             len;
+    CONNECTOR          ccc;
     SServiceConnector* xxx;
 
-    if (!(x_service = SERV_ServiceName(service)))
+    if (!(x_name = SERV_ServiceName(name)))
         return 0;
 
     if (!(ccc = (SConnector*) malloc(sizeof(SConnector)))) {
-        free(x_service);
+        free(x_name);
         return 0;
     }
-    len = strlen(service);
-    if (!(xxx = (SServiceConnector*) calloc(1, sizeof(*xxx) + len))) {
-        free(x_service);
+    len = strlen(name);
+    if (!(xxx = (SServiceConnector*) calloc(1, sizeof(*xxx) + len + 1))) {
+        free(x_name);
         free(ccc);
         return 0;
     }
@@ -1264,20 +1299,18 @@ extern CONNECTOR SERVICE_CreateConnectorEx
     ccc->setup    = s_Setup;
     ccc->destroy  = s_Destroy;
 
-    xxx->types    = (TSERV_TypeOnly) types;
     x_net_info    = (net_info
                      ? ConnNetInfo_Clone(net_info)
-                     : ConnNetInfo_CreateInternal(x_service));
-    xxx->net_info = x_net_info;
-
-    if (!ConnNetInfo_SetupStandardArgs(x_net_info, x_service)) {
-        free(x_service);
+                     : ConnNetInfo_CreateInternal(x_name));
+    free(x_name);
+    if (!x_net_info) {
         s_Destroy(ccc);
         return 0;
     }
+    xxx->net_info = x_net_info;
+    xxx->types    = (TSERV_TypeOnly) types;
     /* NB: zero'ed block, no need to copy the trailing '\0' */
-    memcpy((char*) xxx->service, service, len);
-    free(x_service);
+    memcpy((char*) xxx->name, name, len);
 
     /* now get ready for first probe dispatching */
     if ( types & fSERV_Stateless )
