@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SOCKETP__H
 #define CONNECT___NCBI_SOCKETP__H
 
-/* $Id: ncbi_socketp.h 606556 2020-04-23 14:39:23Z lavr $
+/* $Id: ncbi_socketp.h 652320 2022-07-06 17:22:03Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,14 +37,14 @@
 /* OS must be specified in the command-line ("-D....") or in the conf. header
  */
 #if !defined(NCBI_OS_UNIX)  &&  !defined(NCBI_OS_MSWIN)
-#  error "Unknown OS, must be one of NCBI_OS_UNIX, NCBI_OS_MSWIN!"
+#  error "Unknown OS, must be either NCBI_OS_UNIX or NCBI_OS_MSWIN!"
 #endif /*supported platforms*/
 
 #include <connect/ncbi_socket.h>
 #include <connect/ncbi_buffer.h>
 
 
-/* Pull in a minial set of platform-specific system headers here.
+/* Pull in a minimal set of platform-specific system headers here.
  */
 
 #ifdef NCBI_OS_MSWIN
@@ -136,23 +136,9 @@ typedef int TRIGGER_Handle;
 #endif
 
 
-#if 0/*defined(__GNUC__)*/
-typedef ESwitch    EBSwitch;
-typedef EIO_Status EBIO_Status;
-#else
-typedef unsigned   EBSwitch;
-typedef unsigned   EBIO_Status;
-#endif
-
-
-typedef enum {
-    eListening = 0,
-    eTrigger   = 1,
-    eSocket    = 2,
-    eDatagram  = 3/*2|1*/
-} ESOCK_Type;
-
+typedef unsigned EBSwitch;
 typedef unsigned TBSOCK_Type;
+typedef unsigned EBSOCK_Side;
 
 
 typedef struct {
@@ -195,16 +181,6 @@ typedef struct TRIGGER_tag {
 } TRIGGER_struct;
 
 
-/* Sides of socket
- */
-typedef enum {
-    eSOCK_Server = 0,
-    eSOCK_Client = 1
-} ESOCK_Side;
-
-typedef unsigned EBSOCK_Side;
-
-
 /* Listening socket [must be in one-2-one binary correspondene with TRIGGER]
  */
 typedef struct LSOCK_tag {
@@ -294,7 +270,7 @@ typedef struct SOCK_tag {
 
     SNcbiSSLctx*     sslctx;    /* secure session context if SSL's on, else 0*/
 
-    /* timeouts */
+    /* timeouts: all normalized */
     struct timeval   r_tv;      /* finite read  timeout value                */
     struct timeval   w_tv;      /* finite write timeout value                */
     struct timeval   c_tv;      /* finite close timeout value                */
@@ -403,9 +379,21 @@ const char* SOCK_StringToHostPortEx(const char*     str,
                                     unsigned short* port,
                                     int/*bool*/     flag);
 
+/* Addtl socket API for internal use:  if flag != 0 and host != 0, then
+ * surround the host IP with square brackets.
+ */
+size_t SOCK_HostPortToStringEx(unsigned int   host,
+                               unsigned short port,
+                               char*          buf,
+                               size_t         size,
+                               int/*bool*/    flag);
 
+
+/* See: SOCK_SetupSSL[Ex] */
 void SOCK_SetupSSLInternal(FSSLSetup setup, int/*bool*/ init);
 
+EIO_Status SOCK_SetupSSLInternalEx(FSSLSetup setup, int/*bool*/ init);
+
 
 #ifdef __cplusplus
 } /* extern "C" */
