@@ -1,4 +1,4 @@
-/* $Id: ncbi_util.c 620617 2020-11-25 05:25:50Z lavr $
+/* $Id: ncbi_util.c 668369 2023-06-03 01:48:35Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,9 +42,9 @@
 #  include <stdlib.h>
 #  include <time.h>
 #endif
-#if !defined(NCBI_OS_DARWIN)  &&  defined(HAVE_POLL_H)
+#if defined(HAVE_POLL_H)  &&  !defined(NCBI_OS_DARWIN)
 #  include <poll.h>
-#endif /*!NCBI_OS_DARWIN && HAVE_POLL_H*/
+#endif /*HAVE_POLL_H && !NCBI_OS_DARWIN*/
 #if defined(NCBI_OS_UNIX)
 #  ifndef HAVE_GETPWUID
 #    error "HAVE_GETPWUID is undefined on a UNIX system!"
@@ -77,6 +77,19 @@
 #define NCBI_USE_PRECOMPILED_CRC32_TABLES 1
 
 
+#if 0
+static char* x_getenv(const char* name)
+{
+    char* env = getenv(name);
+    CORE_TRACEF(("getenv(\"%s\") = %s%s%s", name,
+                 &"\""[!env], env ? env : "NULL", &"\""[!env]));
+    return env;
+}
+#else
+#  define x_getenv  getenv
+#endif
+
+
 /******************************************************************************
  *  MT locking
  */
@@ -86,6 +99,7 @@ extern void CORE_SetLOCK(MT_LOCK lk)
     MT_LOCK old_lk = g_CORE_MT_Lock;
     g_CORE_MT_Lock = lk;
     g_CORE_Set |= eCORE_SetLOCK;
+    CORE_TRACEF(("CORE_SetLOCK(%p->%p)", old_lk, lk));
     if (old_lk  &&  old_lk != lk)
         MT_LOCK_Delete(old_lk);
 }
@@ -106,11 +120,13 @@ extern MT_LOCK CORE_GetLOCK(void)
 extern void CORE_SetLOG(LOG lg)
 {
     LOG old_lg;
+    CORE_DEBUG_ARG(if (!lg) CORE_TRACEF(("CORE_SetLOG(%p->NULL)",g_CORE_Log)));
     CORE_LOCK_WRITE;
     old_lg = g_CORE_Log;
     g_CORE_Log = lg;
     g_CORE_Set |= eCORE_SetLOG;
     CORE_UNLOCK;
+    CORE_TRACEF(("CORE_SetLOG(%p->%p)", old_lg, lg));
     if (old_lg  &&  old_lg != lg)
         LOG_Delete(old_lg);
 }
@@ -126,8 +142,7 @@ extern void CORE_SetLOGFILE_Ex
 (FILE*       fp,
  ELOG_Level  cut_off,
  ELOG_Level  fatal_err,
- int/*bool*/ auto_close
- )
+ int/*bool*/ auto_close)
 {
     LOG lg = LOG_Create(0, 0, 0, 0);
     LOG_ToFILE_Ex(lg, fp, cut_off, fatal_err, auto_close);
@@ -139,7 +154,7 @@ extern void CORE_SetLOGFILE
 (FILE*       fp,
  int/*bool*/ auto_close)
 {
-    CORE_SetLOGFILE_Ex(fp, eLOG_Trace, eLOG_Fatal, auto_close);
+    CORE_SetLOGFILE_Ex(fp, CORE_LOGFILE_CUTOFF_LEVEL, eLOG_Fatal, auto_close);
 }
 
 
@@ -155,16 +170,16 @@ extern int/*bool*/ CORE_SetLOGFILE_NAME_Ex
         return 0/*false*/;
     }
 
-    CORE_SetLOGFILE_Ex(fp, cut_off, fatal_err, 1/*autoclose*/);
+    CORE_SetLOGFILE_Ex(fp, cut_off, fatal_err, 1/*auto_close*/);
     return 1/*true*/;
 }
 
 
 extern int/*bool*/ CORE_SetLOGFILE_NAME
-(const char* logfile
- )
+(const char* logfile)
 {
-    return CORE_SetLOGFILE_NAME_Ex(logfile, eLOG_Trace, eLOG_Fatal);
+    return CORE_SetLOGFILE_NAME_Ex(logfile,
+                                   CORE_LOGFILE_CUTOFF_LEVEL, eLOG_Fatal);
 }
 
 
@@ -173,7 +188,6 @@ static TLOG_FormatFlags s_LogFormatFlags = fLOG_Default;
 extern TLOG_FormatFlags CORE_SetLOGFormatFlags(TLOG_FormatFlags flags)
 {
     TLOG_FormatFlags old_flags = s_LogFormatFlags;
-
     s_LogFormatFlags = flags;
     return old_flags;
 }
@@ -181,103 +195,112 @@ extern TLOG_FormatFlags CORE_SetLOGFormatFlags(TLOG_FormatFlags flags)
 
 extern size_t UTIL_PrintableStringSize(const char* data, size_t size)
 {
-    const unsigned char* c;
     size_t count;
     if (!data)
         return 0;
     if (!size)
         size = strlen(data);
-    count = size;
-    for (c = (const unsigned char*) data;  count;  --count, ++c) {
-        if (*c == '\t'  ||  *c == '\v'  ||  *c == '\b'  ||
-            *c == '\r'  ||  *c == '\f'  ||  *c == '\a'  ||
-            *c == '\\'  ||  *c == '\''  ||  *c == '"') {
+    for (count = size;  count;  --count) {
+        unsigned char c = (unsigned char)(*data++);
+        if (c == '\a'  ||  c == '\b'  ||  c == '\f'  ||
+            c == '\r'  ||  c == '\t'  ||  c == '\v'  ||
+            c == '\\'  ||  c == '\''  ||  c == '"'   ||  c == '?') {
             size++;
-        } else if (*c == '\n'  ||  !isascii(*c)  ||  !isprint(*c))
+        } else if (c == '\n'  ||  !isascii(c)  ||  !isprint(c))
             size += 3;
     }
-    return size;
+    return size +
+        (((size + UTIL_PRINTABLE_WIDTH - 1) / UTIL_PRINTABLE_WIDTH) << 1);
 }
 
 
-extern char* UTIL_PrintableString(const char* data, size_t size,
-                                  char* buf, int/*bool*/ flags)
+extern char* UTIL_PrintableStringEx(const char* src, size_t size, char* dst,
+                                    int/*bool*/ flags, int width)
 {
-    const unsigned char* s;
-    unsigned char* d;
+    const char *s, *w;
 
-    if (!data  ||  !buf)
+    if (!src  ||  !dst)
         return 0;
     if (!size)
-        size = strlen(data);
+        size = strlen(src);
+    if (width < UTIL_PRINTABLE_WIDTH  &&  width)
+        width = UTIL_PRINTABLE_WIDTH;
 
-    d = (unsigned char*) buf;
-    for (s = (const unsigned char*) data;  size;  --size, ++s) {
-        switch (*s) {
-        case '\t':
-            *d++ = '\\';
-            *d++ = 't';
-            continue;
-        case '\v':
-            *d++ = '\\';
-            *d++ = 'v';
+    for (w = dst, s = src;  size;  --size, ++s) {
+        unsigned char c = (unsigned char)(*s);
+        if (width  &&  width <= (size_t)(dst - w)) {
+            *dst++ = '\\';
+            *dst++ = '\n';
+            w = dst;
+        }
+        switch (c) {
+        case '\a':
+            *dst++ = '\\';
+            *dst++ = 'a';
             continue;
         case '\b':
-            *d++ = '\\';
-            *d++ = 'b';
-            continue;
-        case '\r':
-            *d++ = '\\';
-            *d++ = 'r';
+            *dst++ = '\\';
+            *dst++ = 'b';
             continue;
         case '\f':
-            *d++ = '\\';
-            *d++ = 'f';
+            *dst++ = '\\';
+            *dst++ = 'f';
             continue;
-        case '\a':
-            *d++ = '\\';
-            *d++ = 'a';
+        case '\r':
+            *dst++ = '\\';
+            *dst++ = 'r';
+            continue;
+        case '\t':
+            *dst++ = '\\';
+            *dst++ = 't';
+            continue;
+        case '\v':
+            *dst++ = '\\';
+            *dst++ = 'v';
             continue;
         case '\n':
-            *d++ = '\\';
-            *d++ = 'n';
-            if (flags & eUTIL_PrintableNoNewLine)
+            *dst++ = '\\';
+            *dst++ = 'n';
+            if (flags & fUTIL_PrintableNoNewLine)
                 continue;
+            w = dst + 2;
             /*FALLTHRU*/
         case '\\':
         case '\'':
         case '"':
-            *d++ = '\\';
+        case '?':
+            if (c != '?'
+                ||  (size > 1  &&  s[ 1] == '?')
+                ||  (s > src   &&  s[-1] == '?')) {
+                *dst++ = '\\';
+            }
             break;
         default:
-            if (!isascii(*s)  ||  !isprint(*s)) {
-                int/*bool*/ reduce;
+            if (!isascii(c)  ||  !isprint(c)) {
                 unsigned char v;
-                if (flags & eUTIL_PrintableFullOctal)
-                    reduce = 0/*false*/;
-                else {
-                    reduce = (size == 1  ||
-                              s[1] < '0' || s[1] > '7' ? 1/*T*/ : 0/*F*/);
-                }
-                *d++     = '\\';
-                v =  *s >> 6;
+                int/*bool*/ reduce = !(flags & fUTIL_PrintableFullOctal)
+                    &&  (size == 1/*last c*/  ||  s[1] < '0'  ||  '7' < s[1])
+                    ? 1/*true*/ : 0/*false*/;
+                *dst++     = '\\';
+                v =  c >> 6;
                 if (v  ||  !reduce) {
-                    *d++ = (unsigned char)('0' + v);
-                    reduce = 0;
+                    *dst++ = (char)('0' + v);
+                    reduce = 0/*false*/;
                 }
-                v = (*s >> 3) & 7;
+                v = (c >> 3) & 7;
                 if (v  ||  !reduce)
-                    *d++ = (unsigned char)('0' + v);
-                v = *s & 7;
-                *d++ = (unsigned char)('0' + v);
+                    *dst++ = (char)('0' + v);
+                v =  c       & 7;
+                *dst++     = (char)('0' + v);
                 continue;
             }
             break;
         }
-        *d++ = *s;
+        *dst++ = (char) c;
     }
-
-    return (char*) d;
+    if (width  &&  w != dst)
+        *dst++ = '\\';
+    return dst;
 }
 
 
@@ -403,13 +426,10 @@ extern char* LOG_ComposeMessage
  TLOG_FormatFlags    flags)
 {
     static const char kRawData_Beg[] =
-        "\n#################### [BEGIN] Raw Data (%lu byte%s):\n";
+        "\n#################### [BEGIN] Raw Data (%lu byte%s):%s";
     static const char kRawData_End[] =
         "\n#################### [_END_] Raw Data\n";
 
-    const char *function, *level = 0;
-    char *str, *s, datetime[32];
-
     /* Calculated length of ... */
     size_t datetime_len  = 0;
     size_t level_len     = 0;
@@ -420,6 +440,9 @@ extern char* LOG_ComposeMessage
     size_t data_len      = 0;
     size_t total_len;
 
+    const char* level;
+    char *str, *s, datetime[32];
+
     /* Adjust formatting flags */
     if (mess->level == eLOG_Trace  &&  !(flags & fLOG_None))
         flags |= fLOG_Full;
@@ -465,34 +488,26 @@ extern char* LOG_ComposeMessage
         &&  (mess->level != eLOG_Note  ||  !(flags & fLOG_OmitNoteLevel))) {
         level = LOG_LevelStr(mess->level);
         level_len = strlen(level) + 2;
-    }
-    if ((flags & fLOG_Module) != 0
-        &&  mess->module  &&  *mess->module) {
-        module_len = strlen(mess->module) + 3;
-    }
-    if ((flags & fLOG_Function) != 0
-        &&  mess->func  &&  *mess->func) {
-        function = mess->func;
-        if (!module_len)
-            function_len = 3;
-        function_len += strlen(function) + 2;
-        if (strncmp(function, "::", 2) == 0  &&  !*(function += 2))
-            function_len = 0;
     } else
-        function = 0;
-    if ((flags & fLOG_FileLine) != 0
-        &&  mess->file  &&  *mess->file) {
-        file_line_len = 12 + strlen(mess->file) + 11;
+        level = 0;
+    if ((flags & fLOG_Module) != 0  &&  mess->module  &&  *mess->module)
+        module_len = strlen(mess->module) + 3;
+    if ((flags & fLOG_Function) != 0  &&  mess->func  &&  *mess->func) {
+        const char* function = mess->func;
+        if (strncmp(function, "::", 2) != 0  ||  *(function += 2))
+            function_len = strlen(function) + (module_len ? 2 : 5);
     }
+    if ((flags & fLOG_FileLine) != 0  &&  mess->file  &&  *mess->file)
+        file_line_len = 12 + strlen(mess->file) + 11;
     if (mess->message  &&  *mess->message)
         message_len = strlen(mess->message);
-
     if (mess->raw_size) {
-        data_len = (sizeof(kRawData_Beg)
-                    + 20 + UTIL_PrintableStringSize((const char*)
-                                                    mess->raw_data,
-                                                    mess->raw_size) +
-                    sizeof(kRawData_End));
+        if (mess->raw_data) {
+            data_len = UTIL_PrintableStringSize((const char*)
+                                                mess->raw_data,
+                                                mess->raw_size);
+        }
+        data_len += sizeof(kRawData_Beg) + 20 + sizeof(kRawData_End);
     }
 
     /* Allocate memory for the resulting message */
@@ -509,10 +524,8 @@ extern char* LOG_ComposeMessage
         memcpy(s, datetime, datetime_len);
         s += datetime_len;
     }
-    if (file_line_len) {
-        s += sprintf(s, "\"%s\", line %d: ",
-                     mess->file, (int) mess->line);
-    }
+    if (file_line_len)
+        s += sprintf(s, "\"%s\", line %d: ", mess->file, mess->line);
     if (module_len | function_len)
         *s++ = '[';
     if (module_len) {
@@ -522,7 +535,7 @@ extern char* LOG_ComposeMessage
     if (function_len) {
         memcpy(s, "::", 2);
         s += 2;
-        memcpy(s, function, function_len -= (module_len ? 2 : 5));
+        memcpy(s, mess->func, function_len -= (module_len ? 2 : 5));
         s += function_len;
     }
     if (module_len | function_len) {
@@ -540,16 +553,19 @@ extern char* LOG_ComposeMessage
         s += message_len;
     }
     if (data_len) {
+        assert(mess->raw_size);
         s += sprintf(s, kRawData_Beg,
                      (unsigned long) mess->raw_size,
-                     &"s"[mess->raw_size == 1]);
-
-        s = UTIL_PrintableString((const char*)
-                                 mess->raw_data,
-                                 mess->raw_size,
-                                 s, flags & fLOG_FullOctal
-                                 ? eUTIL_PrintableFullOctal : 0);
-
+                     &"s"[mess->raw_size == 1],
+                     mess->raw_data ? "\n" : " <NULL>");
+        if (mess->raw_data) {
+            s = UTIL_PrintableStringEx((const char*)
+                                       mess->raw_data,
+                                       mess->raw_size,
+                                       s, flags & fLOG_FullOctal
+                                       ? fUTIL_PrintableFullOctal : 0,
+                                       UTIL_PRINTABLE_WIDTH);
+        }
         memcpy(s, kRawData_End, sizeof(kRawData_End));
     } else
         *s = '\0';
@@ -564,7 +580,7 @@ typedef struct {
     ELOG_Level  cut_off;
     ELOG_Level  fatal_err;
     int/*bool*/ auto_close;
-} SLogData;
+} SFILELogData;
 
 
 /* Callback for LOG_ToFILE[_Ex]() */
@@ -573,7 +589,7 @@ extern "C" {
 #endif /*__cplusplus*/
 static void s_LOG_FileHandler(void* data, const SLOG_Message* mess)
 {
-    SLogData* logdata = (SLogData*) data;
+    SFILELogData* logdata = (SFILELogData*) data;
     assert(logdata->fp);
 
     if (mess->level >= logdata->cut_off  ||
@@ -607,13 +623,14 @@ extern "C" {
 #endif /*__cplusplus*/
 static void s_LOG_FileCleanup(void* data)
 {
-    SLogData* logdata = (SLogData*) data;
+    SFILELogData* logdata = (SFILELogData*) data;
 
     assert(logdata->fp);
     if (logdata->auto_close)
         fclose(logdata->fp);
     else
         fflush(logdata->fp);
+    logdata->fp = 0;
     free(logdata);
 }
 #ifdef __cplusplus
@@ -629,10 +646,10 @@ extern void LOG_ToFILE_Ex
  int/*bool*/ auto_close
  )
 {
-    SLogData* logdata;
+    SFILELogData* logdata;
     if (fp) {
         fflush(fp);
-        logdata = (SLogData*) malloc(sizeof(*logdata));
+        logdata = (SFILELogData*) malloc(sizeof(*logdata));
     } else
         logdata = 0;
     if (logdata) {
@@ -713,16 +730,16 @@ extern char* CORE_GetNcbiRequestID(ENcbiRequestID reqid)
     }
     switch (reqid) {
     case eNcbiRequestID_SID:
-        id = getenv("HTTP_NCBI_SID");
+        id = x_getenv("HTTP_NCBI_SID");
         if (id  &&  *id)
             break;
-        id = getenv("NCBI_LOG_SESSION_ID");
+        id = x_getenv("NCBI_LOG_SESSION_ID");
         break;
     case eNcbiRequestID_HitID:
-        id = getenv("HTTP_NCBI_PHID");
+        id = x_getenv("HTTP_NCBI_PHID");
         if (id  &&  *id)
             break;
-        id = getenv("NCBI_LOG_HIT_ID");
+        id = x_getenv("NCBI_LOG_HIT_ID");
         break;
     default:
         id = 0;
@@ -756,7 +773,7 @@ extern const char* CORE_GetPlatform(void)
  * CORE_GetUsername
  */
 
-static char* x_Savestr(const char* str, char* buf, size_t bufsize)
+static char* x_SaveStr(const char* str, char* buf, size_t bufsize)
 {
     assert(str);
     if (buf) {
@@ -826,13 +843,13 @@ extern const char* CORE_GetUsernameEx(char* buf, size_t bufsize,
         assert(size < sizeof(temp)/sizeof(temp[0]) - 1);
         temp[size] = (TCHAR) '\0';
         login = UTIL_TcharToUtf8(temp);
-        buf = x_Savestr(login, buf, bufsize);
+        buf = x_SaveStr(login, buf, bufsize);
         UTIL_ReleaseBuffer(login);
         return buf;
     }
     CORE_LOCK_READ;
-    if ((login = getenv("USERNAME")) != 0) {
-        buf = x_Savestr(login, buf, bufsize);
+    if ((login = x_getenv("USERNAME")) != 0) {
+        buf = x_SaveStr(login, buf, bufsize);
         CORE_UNLOCK;
         return buf;
     }
@@ -865,7 +882,7 @@ extern const char* CORE_GetUsernameEx(char* buf, size_t bufsize,
         CORE_LOCK_WRITE;
 #    endif /*!NCBI_OS_SOLARIS*/
         if ((login = getlogin()) != 0)
-            buf = x_Savestr(login, buf, bufsize);
+            buf = x_SaveStr(login, buf, bufsize);
 #    ifndef NCBI_OS_SOLARIS
         CORE_UNLOCK;
 #    endif /*!NCBI_OS_SOLARIS*/
@@ -874,7 +891,7 @@ extern const char* CORE_GetUsernameEx(char* buf, size_t bufsize,
 #  else
         if (getlogin_r(temp, sizeof(temp) - 1) == 0) {
             temp[sizeof(temp) - 1] = '\0';
-            return x_Savestr(temp, buf, bufsize);
+            return x_SaveStr(temp, buf, bufsize);
         }
 #  endif /*NCBI_OS_SOLARIS || !HAVE_GETLOGIN_R*/
         /*FALLTHRU*/
@@ -895,7 +912,7 @@ extern const char* CORE_GetUsernameEx(char* buf, size_t bufsize,
 #    endif /*!NCBI_OS_SOLARIS*/
     if ((pwd = getpwuid(uid)) != 0) {
         if (pwd->pw_name)
-            buf = x_Savestr(pwd->pw_name, buf, bufsize);
+            buf = x_SaveStr(pwd->pw_name, buf, bufsize);
         else
             pwd = 0;
     }
@@ -919,16 +936,16 @@ extern const char* CORE_GetUsernameEx(char* buf, size_t bufsize,
 #      error "Unknown value of NCBI_HAVE_GETPWUID_R: 4 or 5 expected."
 #    endif /*NCBI_HAVE_GETPWUID_R*/
     if (pwd  &&  pwd->pw_name)
-        return x_Savestr(pwd->pw_name, buf, bufsize);
+        return x_SaveStr(pwd->pw_name, buf, bufsize);
 #  endif /*NCBI_HAVE_GETPWUID_R*/
 
 #endif /*!NCBI_OS_UNIX*/
 
     /* last resort */
     CORE_LOCK_READ;
-    if (!(login = getenv("USER"))  &&  !(login = getenv("LOGNAME")))
+    if (!(login = x_getenv("USER"))  &&  !(login = x_getenv("LOGNAME")))
         login = "";
-    buf = x_Savestr(login, buf, bufsize);
+    buf = x_SaveStr(login, buf, bufsize);
     CORE_UNLOCK;
     return buf;
 }
@@ -1000,14 +1017,14 @@ extern void CORE_Msdelay(unsigned long ms)
     struct timespec ts;
     ts.tv_sec  = (time_t)(ms / 1000);
     ts.tv_nsec = (long) ((ms % 1000) * 1000000);
-    nanosleep(&ts, 0);
-#  elif !defined(NCBI_OS_DARWIN)  &&  defined(HAVE_POLL_H)
-    poll(0, 0, (int) ms);
+    (void) nanosleep(&ts, 0);
+#  elif defined(HAVE_POLL_H)  &&  !defined(NCBI_OS_DARWIN)
+    (void) poll(0, 0, (int) ms);
 #  else
     struct timeval tv;
     tv.tv_sec  = (long) (ms / 1000);
     tv.tv_usec = (long)((ms % 1000) * 1000);
-    select(0, 0, 0, 0, &tv);
+    (void) select(0, 0, 0, 0, &tv);
 #  endif /*HAVE_NANOSLEEP*/
 #else
 #  error "Unsupported platform."
@@ -1388,6 +1405,17 @@ extern char* UTIL_NcbiLocalHostName(char* hostname)
 }
 
 
+extern int/*bool*/ UTIL_HelpRequested(int argc, char** argv)
+{
+    return argc == 2  &&  ((*argv[1] == '-'  ||  *argv[1] == '/')  &&
+                           (strcmp    (argv[1] + 1, "?")      == 0  ||
+                            strcasecmp(argv[1] + 1, "h")      == 0  ||
+                            strcasecmp(argv[1] + 1, "help")   == 0  ||
+                            strcasecmp(argv[1],     "--help") == 0))
+        ? 1/*true*/ : 0/*false*/;
+}
+
+
 #ifdef NCBI_OS_MSWIN
 
 
