@@ -1,4 +1,4 @@
-/*  $Id: ncbi_uv_nghttp2.cpp 627580 2021-03-15 19:15:45Z ivanov $
+/*  $Id: ncbi_uv_nghttp2.cpp 674396 2023-10-25 18:31:34Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -29,6 +29,8 @@
 
 #include <ncbi_pch.hpp>
 
+#define HAVE_LOCAL_NCBI_BUILD_VER_H 1
+
 #include "connect_misc_impl.hpp"
 
 #include <connect/impl/ncbi_uv_nghttp2.hpp>
@@ -202,24 +204,16 @@ SUv_Tcp::SUv_Tcp(uv_loop_t *l, const SSocketAddress& address, size_t rd_buf_size
 
 int SUv_Tcp::Write()
 {
+    if (m_State == eClosing) {
+        m_State = eRestarting;
+    }
+
     if (m_State == eClosed) {
-        auto rv = uv_tcp_init(m_Loop, this);
+        auto rv = Connect();
 
         if (rv < 0) {
-            NCBI_UV_TCP_TRACE(this << " init failed: " << SUvNgHttp2_Error::LibuvStr(rv));
             return rv;
         }
-
-        rv = m_Connect(this, s_OnConnect);
-
-        if (rv < 0) {
-            NCBI_UV_TCP_TRACE(this << " pre-connect failed: " << SUvNgHttp2_Error::LibuvStr(rv));
-            Close();
-            return rv;
-        }
-
-        NCBI_UV_TCP_TRACE(this << " connecting");
-        m_State = eConnecting;
     }
 
     if (m_State == eConnected) {
@@ -237,7 +231,32 @@ int SUv_Tcp::Write()
     return 0;
 }
 
-void SUv_Tcp::Close()
+// uv_tcp_close_reset was added in libuv v1.32.0
+#if UV_VERSION_HEX < 0x12000
+bool SUv_Tcp::CloseReset(ECloseType)
+{
+    return false;
+}
+#else
+bool SUv_Tcp::CloseReset(ECloseType close_type)
+{
+    if (close_type == eNormalClose) {
+        return false;
+    }
+
+    auto rv = uv_tcp_close_reset(this, s_OnClose);
+
+    if (rv < 0) {
+        NCBI_UV_TCP_TRACE(this << " close reset failed: " << SUvNgHttp2_Error::LibuvStr(rv));
+        return false;
+    } else {
+        NCBI_UV_TCP_TRACE(this << " close resetting");
+        return true;
+    }
+}
+#endif
+
+void SUv_Tcp::Close(ECloseType close_type)
 {
     if (m_State == eConnected) {
         auto rv = uv_read_stop(reinterpret_cast<uv_stream_t*>(this));
@@ -251,15 +270,45 @@ void SUv_Tcp::Close()
 
     m_Write.Reset();
 
-    if ((m_State != eClosing) && (m_State != eClosed)) {
-        NCBI_UV_TCP_TRACE(this << " closing");
-        m_State = eClosing;
-        SUv_Handle<uv_tcp_t>::Close();
+    if ((m_State == eClosing) || (m_State == eRestarting)) {
+        NCBI_UV_TCP_TRACE(this << " already closing");
+
+    } else if (m_State == eClosed) {
+        NCBI_UV_TCP_TRACE(this << " already closed");
+
     } else {
-        NCBI_UV_TCP_TRACE(this << " already closing/closed");
+        m_State = eClosing;
+
+        if (!CloseReset(close_type)) {
+            SUv_Handle<uv_tcp_t>::Close();
+            NCBI_UV_TCP_TRACE(this << " closing");
+        }
     }
 }
 
+int SUv_Tcp::Connect()
+{
+    auto rv = uv_tcp_init(m_Loop, this);
+
+    if (rv < 0) {
+        NCBI_UV_TCP_TRACE(this << " init failed: " << SUvNgHttp2_Error::LibuvStr(rv));
+        return rv;
+    }
+
+    rv = m_Connect(this, s_OnConnect);
+
+    if (rv < 0) {
+        NCBI_UV_TCP_TRACE(this << " pre-connect failed: " << SUvNgHttp2_Error::LibuvStr(rv));
+        Close();
+        return rv;
+    }
+
+    NCBI_UV_TCP_TRACE(this << " connecting");
+    m_State = eConnecting;
+
+    return 0;
+}
+
 void SUv_Tcp::OnConnect(uv_connect_t*, int status)
 {
     if (status >= 0) {
@@ -269,10 +318,24 @@ void SUv_Tcp::OnConnect(uv_connect_t*, int status)
             status = uv_read_start((uv_stream_t*)this, s_OnAlloc, s_OnRead);
 
             if (status >= 0) {
-                NCBI_UV_TCP_TRACE(this << " connected");
-                m_State = eConnected;
-                m_ConnectCb(status);
-                return;
+                struct sockaddr_storage name;
+                auto namelen = static_cast<int>(sizeof(name));
+
+                status = uv_tcp_getsockname(this, reinterpret_cast<sockaddr*>(&name), &namelen);
+
+                if (status >= 0) {
+                    if (name.ss_family == AF_INET) {
+                        auto sin = reinterpret_cast<sockaddr_in*>(&name);
+                        m_LocalPort = CSocketAPI::NetToHostShort(sin->sin_port);
+                    }
+
+                    NCBI_UV_TCP_TRACE(this << " connected: " << m_LocalPort);
+                    m_State = eConnected;
+                    m_ConnectCb(status);
+                    return;
+                } else {
+                    NCBI_UV_TCP_TRACE(this << " getsockname failed: " << SUvNgHttp2_Error::LibuvStr(status));
+                }
             } else {
                 NCBI_UV_TCP_TRACE(this << " read start failed: " << SUvNgHttp2_Error::LibuvStr(status));
             }
@@ -289,6 +352,7 @@ void SUv_Tcp::OnConnect(uv_connect_t*, int status)
 
 void SUv_Tcp::OnAlloc(uv_handle_t*, size_t suggested_size, uv_buf_t* buf)
 {
+    NCBI_UV_TCP_TRACE(this << " alloc: " << suggested_size);
     m_ReadBuffer.resize(suggested_size);
     buf->base = m_ReadBuffer.data();
     buf->len = static_cast<decltype(buf->len)>(m_ReadBuffer.size());
@@ -322,7 +386,10 @@ void SUv_Tcp::OnWrite(uv_write_t* req, int status)
 void SUv_Tcp::OnClose(uv_handle_t*)
 {
     NCBI_UV_TCP_TRACE(this << " closed");
-    m_State = eClosed;
+
+    if (exchange(m_State, eClosed) == eRestarting) {
+        Connect();
+    }
 }
 
 struct SUvNgHttp2_UserAgentImpl : string
@@ -375,7 +442,7 @@ SNgHttp2_Session::SNgHttp2_Session(void* user_data, uint32_t max_streams,
         nghttp2_on_data_chunk_recv_callback on_data,
         nghttp2_on_stream_close_callback    on_stream_close,
         nghttp2_on_header_callback          on_header,
-        nghttp2_error_callback              on_error,
+        nghttp2_error_callback2             on_error,
         nghttp2_on_frame_recv_callback      on_frame_recv) :
     m_UserData(user_data),
     m_OnData(on_data),
@@ -398,7 +465,7 @@ int SNgHttp2_Session::Init()
     nghttp2_session_callbacks_set_on_data_chunk_recv_callback(callbacks, m_OnData);
     nghttp2_session_callbacks_set_on_stream_close_callback(   callbacks, m_OnStreamClose);
     nghttp2_session_callbacks_set_on_header_callback(         callbacks, m_OnHeader);
-    nghttp2_session_callbacks_set_error_callback(             callbacks, m_OnError);
+    nghttp2_session_callbacks_set_error_callback2(            callbacks, m_OnError);
     if (m_OnFrameRecv) nghttp2_session_callbacks_set_on_frame_recv_callback(callbacks, m_OnFrameRecv);
 
     nghttp2_session_client_new(&m_Session, callbacks, m_UserData);
@@ -414,9 +481,9 @@ int SNgHttp2_Session::Init()
         return x_DelOnError(rv);
     }
 
-    NCBI_NGHTTP2_SESSION_TRACE(this << " initialized");
     auto max_streams = nghttp2_session_get_remote_settings(m_Session, NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS);
     m_MaxStreams.first = min(max_streams, m_MaxStreams.second);
+    NCBI_NGHTTP2_SESSION_TRACE(this << " initialized: " << m_MaxStreams.first);
     return 0;
 }
 
@@ -549,7 +616,7 @@ private:
 
 struct SUvNgHttp2_TlsImpl : SUvNgHttp2_Tls
 {
-    SUvNgHttp2_TlsImpl(const SSocketAddress& address, size_t rd_buf_size, size_t wr_buf_size, TGetWriteBuf get_write_buf);
+    SUvNgHttp2_TlsImpl(const TAddrNCred& addr_n_cred, size_t rd_buf_size, size_t wr_buf_size, TGetWriteBuf get_write_buf);
     ~SUvNgHttp2_TlsImpl() override;
 
     int Read(const char*& buf, ssize_t& nread) override;
@@ -560,6 +627,13 @@ struct SUvNgHttp2_TlsImpl : SUvNgHttp2_Tls
     vector<char>& GetWriteBuffer() override { return m_WriteBuffer; }
 
 private:
+    // Provides scope-restricted access to incoming TLS data
+    struct SIncomingData : pair<const char**, ssize_t*>
+    {
+        struct SReset { void operator()(first_type* p) const { *p = nullptr; } };
+        auto operator()(first_type b, second_type l) { first = b; second = l; return unique_ptr<first_type, SReset>(&first); }
+    };
+
     SUvNgHttp2_TlsImpl(const SUvNgHttp2_TlsImpl&) = delete;
     SUvNgHttp2_TlsImpl(SUvNgHttp2_TlsImpl&&) = delete;
 
@@ -592,13 +666,15 @@ private:
 
     vector<char> m_ReadBuffer;
     vector<char> m_WriteBuffer;
-    pair<const char**, ssize_t*> m_IncomingData;
+    SIncomingData m_IncomingData;
     TGetWriteBuf m_GetWriteBuf;
 
     mbedtls_ssl_context m_Ssl;
     mbedtls_ssl_config m_Conf;
     mbedtls_ctr_drbg_context m_CtrDrbg;
     mbedtls_entropy_context m_Entropy;
+    mbedtls_x509_crt m_Cert;
+    mbedtls_pk_context m_Pkey;
     array<const char*, 2> m_Protocols;
 };
 
@@ -607,16 +683,14 @@ bool s_WantReadOrWrite(int rv)
     return (rv == MBEDTLS_ERR_SSL_WANT_READ) || (rv == MBEDTLS_ERR_SSL_WANT_WRITE);
 }
 
-SUvNgHttp2_TlsImpl::SUvNgHttp2_TlsImpl(const SSocketAddress& address, size_t rd_buf_size, size_t wr_buf_size, TGetWriteBuf get_write_buf) :
+SUvNgHttp2_TlsImpl::SUvNgHttp2_TlsImpl(const TAddrNCred& addr_n_cred, size_t rd_buf_size, size_t wr_buf_size, TGetWriteBuf get_write_buf) :
     m_ReadBuffer(rd_buf_size),
-    m_IncomingData(nullptr, nullptr),
     m_GetWriteBuf(get_write_buf),
     m_Protocols({ "h2", nullptr })
 {
     NCBI_UVNGHTTP2_TLS_TRACE(this << " created");
     m_WriteBuffer.reserve(wr_buf_size),
 
-    SOCK_SetupSSLEx(NcbiSetupTls);
     mbedtls_ssl_config_init(&m_Conf);
 
     auto c_rv = mbedtls_ssl_config_defaults(&m_Conf, MBEDTLS_SSL_IS_CLIENT, MBEDTLS_SSL_TRANSPORT_STREAM, MBEDTLS_SSL_PRESET_DEFAULT);
@@ -629,6 +703,8 @@ SUvNgHttp2_TlsImpl::SUvNgHttp2_TlsImpl(const SSocketAddress& address, size_t rd_
     mbedtls_ssl_conf_authmode(&m_Conf, MBEDTLS_SSL_VERIFY_NONE);
     mbedtls_entropy_init(&m_Entropy);
     mbedtls_ctr_drbg_init(&m_CtrDrbg);
+    mbedtls_x509_crt_init(&m_Cert);
+    mbedtls_pk_init(&m_Pkey);
 
     auto d_rv = mbedtls_ctr_drbg_seed(&m_CtrDrbg, mbedtls_entropy_func, &m_Entropy, nullptr, 0);
 
@@ -648,7 +724,7 @@ SUvNgHttp2_TlsImpl::SUvNgHttp2_TlsImpl(const SSocketAddress& address, size_t rd_
         return;
     }
 
-    const auto host_name = address.GetHostName();
+    const auto host_name = addr_n_cred.first.GetHostName();
     auto h_rv = mbedtls_ssl_set_hostname(&m_Ssl, host_name.c_str());
 
     if (h_rv) {
@@ -657,10 +733,40 @@ SUvNgHttp2_TlsImpl::SUvNgHttp2_TlsImpl(const SSocketAddress& address, size_t rd_
     }
 
     mbedtls_ssl_set_bio(&m_Ssl, this, s_OnSend, s_OnRecv, nullptr);
+
+    const auto& cert = addr_n_cred.second.first;
+    const auto& pkey = addr_n_cred.second.second;
+
+    if (cert.empty() || pkey.empty()) {
+        return;
+    }
+
+    auto cp_rv = mbedtls_x509_crt_parse(&m_Cert, reinterpret_cast<const unsigned char*>(cert.data()), cert.size() + 1);
+
+    if (cp_rv) {
+        NCBI_UVNGHTTP2_TLS_TRACE(this << " mbedtls_x509_crt_parse: " << SUvNgHttp2_Error::MbedTlsStr(cp_rv));
+        return;
+    }
+
+    auto pk_rv = mbedtls_pk_parse_key(&m_Pkey, reinterpret_cast<const unsigned char*>(pkey.data()), pkey.size() + 1, nullptr, 0);
+
+    if (pk_rv) {
+        NCBI_UVNGHTTP2_TLS_TRACE(this << " mbedtls_pk_parse_key: " << SUvNgHttp2_Error::MbedTlsStr(pk_rv));
+        return;
+    }
+
+    auto oc_rv = mbedtls_ssl_conf_own_cert(&m_Conf, &m_Cert, &m_Pkey);
+
+    if (oc_rv) {
+        NCBI_UVNGHTTP2_TLS_TRACE(this << " mbedtls_ssl_conf_own_cert: " << SUvNgHttp2_Error::MbedTlsStr(oc_rv));
+        return;
+    }
 }
 
 SUvNgHttp2_TlsImpl::~SUvNgHttp2_TlsImpl()
 {
+    mbedtls_x509_crt_free(&m_Cert);
+    mbedtls_pk_free(&m_Pkey);
     mbedtls_entropy_free(&m_Entropy);
     mbedtls_ctr_drbg_free(&m_CtrDrbg);
     mbedtls_ssl_config_free(&m_Conf);
@@ -681,7 +787,6 @@ int SUvNgHttp2_TlsImpl::Init()
             break;
     }
 
-    m_WriteBuffer.clear();
     auto rv = mbedtls_ssl_session_reset(&m_Ssl);
 
     if (rv < 0) {
@@ -719,7 +824,7 @@ int SUvNgHttp2_TlsImpl::GetReady()
 
 int SUvNgHttp2_TlsImpl::Read(const char*& buf, ssize_t& nread)
 {
-    m_IncomingData = make_pair(&buf, &nread);
+    auto scope_guard = m_IncomingData(&buf, &nread);
 
     if (auto rv = Init()) return rv;
 
@@ -771,6 +876,7 @@ int SUvNgHttp2_TlsImpl::Close()
         case eReady:       break;
     }
 
+    m_WriteBuffer.clear();
     m_State = eClosed;
     return mbedtls_ssl_close_notify(&m_Ssl);
 }
@@ -802,10 +908,10 @@ int SUvNgHttp2_TlsImpl::OnSend(const unsigned char* buf, size_t len)
     return static_cast<int>(len);
 }
 
-SUvNgHttp2_Tls* SUvNgHttp2_Tls::Create(bool https, const SSocketAddress& address, size_t rd_buf_size, size_t wr_buf_size, TGetWriteBuf get_write_buf)
+SUvNgHttp2_Tls* SUvNgHttp2_Tls::Create(bool https, const TAddrNCred& addr_n_cred, size_t rd_buf_size, size_t wr_buf_size, TGetWriteBuf get_write_buf)
 {
     if (https) {
-        return new SUvNgHttp2_TlsImpl(address, rd_buf_size, wr_buf_size, get_write_buf);
+        return new SUvNgHttp2_TlsImpl(addr_n_cred, rd_buf_size, wr_buf_size, get_write_buf);
     }
 
     return new SUvNgHttp2_TlsNoOp(get_write_buf);
@@ -819,7 +925,7 @@ bool SUvNgHttp2_SessionBase::Send()
         Reset(SUvNgHttp2_Error::FromNgHttp2(send_rv, "on send"));
 
     } else if (send_rv == SNgHttp2_Session::eWantsClose) {
-        Reset("nghttp2 asked to drop connection");
+        Reset("nghttp2 asked to drop connection", SUv_Tcp::eNormalClose);
 
     } else {
         auto tls_rv = m_Tls->Write();
@@ -838,6 +944,12 @@ bool SUvNgHttp2_SessionBase::Send()
     return false;
 }
 
+int SUvNgHttp2_SessionBase::OnError(nghttp2_session*, int lib_error_code, const char* msg, size_t len)
+{
+    NCBI_UVNGHTTP2_SESSION_TRACE(this << " error: " << SUvNgHttp2_Error::NgHttp2Str(lib_error_code) << ". " << CTempString(msg, len));
+    return 0;
+}
+
 void SUvNgHttp2_SessionBase::OnConnect(int status)
 {
     NCBI_UVNGHTTP2_SESSION_TRACE(this << " connected: " << status);
@@ -873,7 +985,7 @@ void SUvNgHttp2_SessionBase::OnRead(const char* buf, ssize_t nread)
         if (read_rv == 0) {
             m_Session.Del();
             m_Tls->Close();
-            m_Tcp.Close();
+            m_Tcp.Close(SUv_Tcp::eNormalClose);
 
         } else if (s_WantReadOrWrite(read_rv)) {
             if (nread == 0) break;
@@ -903,12 +1015,12 @@ void SUvNgHttp2_SessionBase::OnRead(const char* buf, ssize_t nread)
     Send();
 }
 
-void SUvNgHttp2_SessionBase::Reset(SUvNgHttp2_Error error)
+void SUvNgHttp2_SessionBase::Reset(SUvNgHttp2_Error error, SUv_Tcp::ECloseType close_type)
 {
     NCBI_UVNGHTTP2_SESSION_TRACE(this << " resetting with " << error);
     m_Session.Del();
     m_Tls->Close();
-    m_Tcp.Close();
+    m_Tcp.Close(close_type);
     OnReset(move(error));
 }
 
