@@ -1,4 +1,4 @@
-/*  $Id: grid_worker.cpp 620109 2020-11-16 19:42:44Z sadyrovr $
+/*  $Id: grid_worker.cpp 638391 2021-09-29 18:15:12Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -307,6 +307,22 @@ private:
 };
 
 
+SSuspendResume::EState SSuspendResume::CheckState() volatile
+{
+    switch (m_Event.exchange(eNoEvent)) {
+        case eNoEvent:
+            break;
+        case eSuspend:
+            return m_IsSuspended.exchange(true) ? eSuspended : eSuspending;
+        case eResume:
+            m_IsSuspended.store(false);
+            return eRunning;
+    }
+
+    return m_IsSuspended.load() ? eSuspended : eRunning;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 //
 const IRegistry& SGridWorkerNodeImpl::GetConfig() const
@@ -356,7 +372,7 @@ SGridWorkerNodeImpl::SGridWorkerNodeImpl(CNcbiApplicationAPI& app,
     m_ThreadPool(NULL),
     m_MaxThreads(1),
     m_NSTimeout(DEFAULT_NS_TIMEOUT),
-    m_CommitJobInterval(COMMIT_JOB_INTERVAL_DEFAULT),
+    m_CommitJobInterval(2),
     m_CheckStatusPeriod(2),
     m_ExclusiveJobSemaphore(1, 1),
     m_IsProcessingExclusiveJob(false),
@@ -364,11 +380,6 @@ SGridWorkerNodeImpl::SGridWorkerNodeImpl(CNcbiApplicationAPI& app,
     m_TotalTimeLimit(0),
     m_StartupTime(0),
     m_CleanupEventSource(new CWorkerNodeCleanup()),
-    m_SuspendResumeEvent(NO_EVENT),
-    m_TimelineIsSuspended(false),
-    m_CurrentJobGeneration(0),
-    m_DefaultPullbackTimeout(0),
-    m_JobPullbackTime(0),
     m_Listener(new CGridWorkerNodeApp_Listener()),
     m_App(app),
     m_SingleThreadForced(false)
@@ -386,6 +397,7 @@ void SGridWorkerNodeImpl::Init()
 
     m_Listener->OnInit(this);
 
+    // This parameter is deprecated, "[Diag]Merge_Lines" should be used instead
     if (m_SynRegistry->Get("log", "merge_lines", false)) {
         SetDiagPostFlag(eDPF_PreMergeLines);
         SetDiagPostFlag(eDPF_MergeLines);
@@ -402,31 +414,38 @@ void CGridWorkerNode::Init()
 }
 
 void CGridWorkerNode::Suspend(bool pullback, unsigned timeout)
+{
+    m_Impl->m_SuspendResume.GetLock()->Suspend(pullback, timeout);
+}
+
+void SSuspendResume::Suspend(bool pullback, unsigned timeout)
 {
     if (pullback)
-        m_Impl->SetJobPullbackTimer(timeout);
-    if (SwapPointers(&m_Impl->m_SuspendResumeEvent, SUSPEND_EVENT) == NO_EVENT)
+        SetJobPullbackTimer(timeout);
+    if (m_Event.exchange(eSuspend) == eNoEvent)
         CGridGlobals::GetInstance().InterruptUDPPortListening();
 }
 
 void CGridWorkerNode::Resume()
 {
-    if (SwapPointers(&m_Impl->m_SuspendResumeEvent, RESUME_EVENT) == NO_EVENT)
+    m_Impl->m_SuspendResume->Resume();
+}
+
+void SSuspendResume::Resume() volatile
+{
+    if (m_Event.exchange(eResume) == eNoEvent)
         CGridGlobals::GetInstance().InterruptUDPPortListening();
 }
 
-void SGridWorkerNodeImpl::SetJobPullbackTimer(unsigned seconds)
+void SSuspendResume::SetJobPullbackTimer(unsigned seconds)
 {
-    CFastMutexGuard mutex_guard(m_JobPullbackMutex);
     m_JobPullbackTime = CDeadline(seconds);
     ++m_CurrentJobGeneration;
 }
 
-bool SGridWorkerNodeImpl::CheckForPullback(unsigned job_generation)
+bool SSuspendResume::IsJobPullbackTimerExpired()
 {
-    CFastMutexGuard mutex_guard(m_JobPullbackMutex);
-    return job_generation != m_CurrentJobGeneration &&
-            m_JobPullbackTime.IsExpired();
+    return m_JobPullbackTime.IsExpired();
 }
 
 void SGridWorkerNodeImpl::x_WNCoreInit()
@@ -595,15 +614,15 @@ int SGridWorkerNodeImpl::Run(
         LOG_POST_X(42, Warning << "All hosts from admin_hosts were ignored");
     }
 
-    m_CommitJobInterval = m_SynRegistry->Get("server", "commit_job_interval", COMMIT_JOB_INTERVAL_DEFAULT);
-    if (m_CommitJobInterval == 0)
-        m_CommitJobInterval = 1;
+    auto commit_job_interval = m_SynRegistry->Get("server", "commit_job_interval", m_CommitJobInterval);
+    m_CommitJobInterval = static_cast<unsigned>(max(1, commit_job_interval));
 
     m_CheckStatusPeriod = m_SynRegistry->Get("server", "check_status_period", 2);
     if (m_CheckStatusPeriod == 0)
         m_CheckStatusPeriod = 1;
 
-    m_DefaultPullbackTimeout = m_SynRegistry->Get("server", "default_pullback_timeout", 0);
+    auto default_timeout = m_SynRegistry->Get("server", "default_pullback_timeout", 0);
+    m_SuspendResume.GetLock()->SetDefaultPullbackTimeout(default_timeout);
 
     if (m_SynRegistry->Has("server", "wait_server_timeout")) {
         ERR_POST_X(52, "[server"
@@ -1130,7 +1149,7 @@ IWorkerNodeCleanupEventSource* CGridWorkerNode::GetCleanupEventSource()
 
 bool CGridWorkerNode::IsSuspended() const
 {
-    return m_Impl->m_TimelineIsSuspended;
+    return m_Impl->m_SuspendResume->IsSuspended();
 }
 
 const string& CGridWorkerNode::GetQueueName() const
