@@ -1,4 +1,4 @@
-/*  $Id: json_over_uttp.cpp 586511 2019-05-17 13:26:10Z satskyse $
+/*  $Id: json_over_uttp.cpp 623939 2021-01-22 18:14:49Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,6 +43,8 @@ BEGIN_NCBI_SCOPE
 class CJsonParser
 {
 public:
+    CJsonParser(CJsonNode::TParseFlags flags) : m_Flags(flags) {}
+
     CJsonNode ParseObject(const string& ns_output)
     {
         m_Ch = (m_NSOutput = ns_output).c_str();
@@ -81,6 +83,7 @@ private:
 
     string m_NSOutput;
     const char* m_Ch;
+    CJsonNode::TParseFlags m_Flags;
 };
 
 const char* CJsonException::GetErrCodeString() const
@@ -944,12 +947,13 @@ static void s_Repr_Value(string& os, const CJsonNode& node,
         os.append(1, ']');
         break;
     case CJsonNode::eString:
-        os.append(1, '"');
-        if (flags & CJsonNode::fStandardJson)
-            os.append(NStr::JsonEncode(node.AsString()));
-        else
+        if (flags & CJsonNode::fStandardJson) {
+            os.append(NStr::JsonEncode(node.AsString(), NStr::eJsonEnc_Quoted));
+        } else {
+            os.append(1, '"');
             os.append(NStr::PrintableString(node.AsString()));
-        os.append(1, '"');
+            os.append(1, '"');
+        }
         break;
     case CJsonNode::eInteger:
         os.append(NStr::NumericToString(node.AsInteger()));
@@ -1042,8 +1046,10 @@ CJsonNode CJsonParser::ParseJSON(const string& json)
 
 string CJsonParser::ParseString(size_t max_len)
 {
+    const bool standard_json = m_Flags & CJsonNode::fStandardJson;
+    auto str = CTempString(m_Ch, max_len);
     size_t len;
-    string val(NStr::ParseQuoted(CTempString(m_Ch, max_len), &len));
+    auto val = standard_json ? NStr::JsonDecode(move(str), &len) : NStr::ParseQuoted(move(str), &len);
 
     m_Ch += len;
     return val;
@@ -1240,19 +1246,19 @@ CJsonNode CJsonParser::ParseValue()
     INVALID_FORMAT_ERROR();
 }
 
-CJsonNode CJsonNode::ParseObject(const string& json)
+CJsonNode CJsonNode::ParseObject(const string& json, TParseFlags flags)
 {
-    return CJsonParser().ParseObject(json);
+    return CJsonParser(flags).ParseObject(json);
 }
 
-CJsonNode CJsonNode::ParseArray(const string& json)
+CJsonNode CJsonNode::ParseArray(const string& json, TParseFlags flags)
 {
-    return CJsonParser().ParseArray(json);
+    return CJsonParser(flags).ParseArray(json);
 }
 
-CJsonNode CJsonNode::ParseJSON(const string& json)
+CJsonNode CJsonNode::ParseJSON(const string& json, TParseFlags flags)
 {
-    return CJsonParser().ParseJSON(json);
+    return CJsonParser(flags).ParseJSON(json);
 }
 
 const char* CJsonOverUTTPException::GetErrCodeString() const
