@@ -1,4 +1,4 @@
-/*  $Id: neticache_client.cpp 593074 2019-09-11 18:58:14Z sadyrovr $
+/*  $Id: neticache_client.cpp 617404 2020-09-30 19:02:00Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -163,7 +163,8 @@ struct SNetICacheClientImpl : public SNetCacheAPIImpl
         size_t* blob_size_ptr,
         int* version,
         ICache::EBlobVersionValidity* validity,
-        unsigned max_age, unsigned* actual_age);
+        unsigned max_age, unsigned* actual_age,
+        const CNamedParameterList* optional = NULL);
 
     IReader* GetReadStreamPart(const string& key,
         int version, const string& subkey,
@@ -686,6 +687,15 @@ IReader* CNetICacheClient::GetReadStream(const string&  key,
             nc_caching_mode = CNetCacheAPI::eCaching_AppDefault);
 }
 
+IReader* CNetICacheClientExt::GetReadStream(const string& key,
+        const string& subkey, int* version,
+        size_t* blob_size_ptr, const CNamedParameterList* optional)
+{
+    ICache::EBlobVersionValidity not_used;
+    return m_Impl->ReadCurrentBlobNotOlderThan(key, subkey, blob_size_ptr,
+            version, &not_used, 0, NULL, optional);
+}
+
 IReader* CNetICacheClient::GetReadStream(const string& key,
         const string& subkey, int* version,
         ICache::EBlobVersionValidity* validity)
@@ -699,23 +709,21 @@ IReader* SNetICacheClientImpl::ReadCurrentBlobNotOlderThan(const string& key,
         size_t* blob_size_ptr,
         int* version,
         ICache::EBlobVersionValidity* validity,
-        unsigned max_age, unsigned* actual_age)
+        unsigned max_age, unsigned* actual_age,
+        const CNamedParameterList* optional)
 {
     try {
         string blob_id(s_KeySubkeyToBlobID(key, subkey));
+        CNetCacheAPIParameters parameters(&m_DefaultParameters);
 
-        string cmd;
-
-        if (max_age == 0)
-            cmd = MakeStdCmd("READLAST", blob_id, &m_DefaultParameters);
-        else {
-            CNetCacheAPIParameters parameters(&m_DefaultParameters);
+        parameters.LoadNamedParameters(optional);
 
+        if (max_age != 0) {
             parameters.SetMaxBlobAge(max_age);
-
-            cmd = MakeStdCmd("READLAST", blob_id, &parameters);
         }
 
+        string cmd = MakeStdCmd("READLAST", blob_id, &parameters);
+
         CNetServer::SExecResult exec_result(ChooseServerAndExec(cmd,
                 key, false, &m_DefaultParameters));
 
@@ -967,7 +975,7 @@ vector<CNetICacheClient::CBlobInfo> CNetICacheClient::Search(
     return result;
 }
 
-void CNetICacheClientExt::ProlongBlobLifetime(const string& key,
+void CNetICacheClientExt::ProlongBlobLifetime(const string& key, const string& subkey,
         const CTimeout& ttl, const CNamedParameterList* optional)
 {
     CNetCacheAPIParameters parameters(&m_Impl->m_DefaultParameters);
@@ -977,7 +985,9 @@ void CNetICacheClientExt::ProlongBlobLifetime(const string& key,
     cmd += NStr::PrintableString(parameters.GetCacheName());
     cmd += "\" \"";
     cmd += key;
-    cmd += "\" \"\" ttl=";
+    cmd += "\" \"";
+    cmd += subkey;
+    cmd += "\" ttl=";
     cmd += NStr::NumericToString((unsigned)ttl.GetAsDouble());
 
     m_Impl->AppendClientIPSessionIDHitID(&cmd);
