@@ -1,4 +1,4 @@
-/*  $Id: ncbi_config.cpp 534857 2017-05-03 12:26:07Z ivanov $
+/*  $Id: ncbi_config.cpp 663254 2023-02-15 17:18:08Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,8 +55,8 @@ static const char* kNodeName          = ".NodeName";
 static const char* kIncludeSections   = ".Include";
 
 
-static
-void s_List2Set(const list<string>& src, set<string>* dst)
+template<class ResultSet>
+void s_List2Set(const list<string>& src, ResultSet* dst)
 {
     ITERATE(list<string>, it, src) {
         dst->insert(*it);
@@ -64,6 +64,7 @@ void s_List2Set(const list<string>& src, set<string>* dst)
 }
 
 
+static
 bool s_IsSubNode(const string& str)
 {
     if (NStr::CompareNocase(kSubNode, str) == 0) {
@@ -80,7 +81,7 @@ typedef CConfig::TParamTree TParamTree;
 typedef CConfig::TParamValue TParamValue;
 typedef map<TParamTree*, set<string> > TSectionMap;
 
-
+static
 void s_AddOrReplaceSubNode(TParamTree*   node_ptr,
                            const string& element_name,
                            const string& element_value)
@@ -97,6 +98,7 @@ void s_AddOrReplaceSubNode(TParamTree*   node_ptr,
 }
 
 
+static
 TParamTree* s_FindSubNode(const string& path,
                           TParamTree*   tree_root)
 {
@@ -110,6 +112,7 @@ TParamTree* s_FindSubNode(const string& path,
 }
 
 
+static
 void s_ParseSubNodes(const string& sub_nodes,
                      TParamTree*   parent_node,
                      TSectionMap&  inc_sections,
@@ -118,10 +121,11 @@ void s_ParseSubNodes(const string& sub_nodes,
     list<string> sub_list;
     NStr::Split(sub_nodes, ",; \t\n\r", sub_list,
         NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
-    set<string> sub_set;
+    typedef set<string, PNocase_Conditional> TSubSet;
+    TSubSet sub_set(parent_node->GetKeyEqual());
     s_List2Set(sub_list, &sub_set);
-    ITERATE(set<string>, sub_it, sub_set) {
-        unique_ptr<TParamTree> sub_node(new TParamTree);
+    ITERATE(TSubSet, sub_it, sub_set) {
+        unique_ptr<TParamTree> sub_node(new TParamTree(parent_node->GetKeyEqual()));
         size_t pos = sub_it->rfind('/');
         if (pos == string::npos) {
             sub_node->GetKey() = *sub_it;
@@ -136,6 +140,7 @@ void s_ParseSubNodes(const string& sub_nodes,
 }
 
 
+static
 bool s_IsParentNode(TParamTree* parent, TParamTree* child)
 {
     TParamTree* node = child;
@@ -149,6 +154,7 @@ bool s_IsParentNode(TParamTree* parent, TParamTree* child)
 }
 
 
+static
 void s_IncludeNode(TParamTree*       parent_node,
                    const TParamTree* inc_node)
 {
@@ -169,6 +175,7 @@ void s_IncludeNode(TParamTree*       parent_node,
 }
 
 
+static
 void s_ExpandSubNodes(TSectionMap& inc_sections,
                       TParamTree*  tree_root,
                       TParamTree*  node)
@@ -232,12 +239,42 @@ ETreeTraverseCode SNodeNameUpdater::operator()(TParamTree& node,
 }
 
 
-CConfig::TParamTree* CConfig::ConvertRegToTree(const IRegistry& reg)
+static void x_Print(const CConfig::TParamTree* tree, const string& section)
 {
-    unique_ptr<TParamTree> tree_root(new TParamTree);
+    bool started_section = false;
+    for ( auto iter = tree->SubNodeBegin(); iter != tree->SubNodeEnd(); ++iter ) {
+        auto node = *iter;
+        if ( node->IsLeaf() ) {
+            if ( !started_section ) {
+                cout << "[" << section << "]\n";
+                started_section = true;
+            }
+            cout << node->GetKey() << "=" << node->GetValue().value << "\n";
+        }
+    }
+    for ( auto iter = tree->SubNodeBegin(); iter != tree->SubNodeEnd(); ++iter ) {
+        auto node = *iter;
+        if ( !node->IsLeaf() ) {
+            x_Print(node, section + (section.empty()? "": "/") + node->GetKey());
+        }
+    }
+}
+
+
+CConfig::TParamTree* CConfig::ConvertRegToTree(const IRegistry& reg,
+                                               NStr::ECase use_case)
+{
+    const bool debug = 0;
+    unique_ptr<TParamTree> tree_root(new TParamTree(TParamTree::TKeyGetter(use_case)));
 
     list<string> sections;
     reg.EnumerateSections(&sections);
+    if ( debug ) {
+        cout << "Sections:\n";
+        for ( auto& s : sections ) {
+            cout << s << "\n";
+        }
+    }
 
     // find the non-redundant set of top level sections
 
@@ -253,7 +290,7 @@ CConfig::TParamTree* CConfig::ConvertRegToTree(const IRegistry& reg)
         const string& section_name = *name_it;
         TParamTree* node_ptr;
         if (section_name.find('/') == string::npos) {
-            unique_ptr<TParamTree> node(new TParamTree);
+            unique_ptr<TParamTree> node(new TParamTree(TParamTree::TKeyGetter(use_case)));
             node->GetKey() = section_name;
             tree_root->AddNode(node_ptr = node.release());
         } else {
@@ -268,6 +305,12 @@ CConfig::TParamTree* CConfig::ConvertRegToTree(const IRegistry& reg)
         // Create section entries
         list<string> entries;
         reg.EnumerateEntries(section_name, &entries);
+        if ( debug ) {
+            cout << "Entries of " << section_name << ":\n";
+            for ( auto& s : entries ) {
+                cout << s << "\n";
+            }
+        }
 
         ITERATE(list<string>, eit, entries) {
             const string& element_name = *eit;
@@ -291,6 +334,9 @@ CConfig::TParamTree* CConfig::ConvertRegToTree(const IRegistry& reg)
                 continue;
             }
 
+            if ( debug ) {
+                cout << "[" << section_name << "]: "<<element_name<<"="<<element_value<<"\n";
+            }
             s_AddOrReplaceSubNode(node_ptr, element_name, element_value);
         }
         // Force node name to prevent overriding it by includes
@@ -344,7 +390,7 @@ CConfig::TParamTree* CConfig::ConvertRegToTree(const IRegistry& reg)
 
         TParamTree* node_ptr;
         if (section_name.find('/') == string::npos) {
-            unique_ptr<TParamTree> node(new TParamTree);
+            unique_ptr<TParamTree> node(new TParamTree(TParamTree::TKeyGetter(use_case)));
             node->GetKey() = section_name;
             tree_root->AddNode(node_ptr = node.release());
         } else {
@@ -383,30 +429,34 @@ CConfig::TParamTree* CConfig::ConvertRegToTree(const IRegistry& reg)
     } // ITERATE sit
     */
 
+    if ( debug ) {
+        x_Print(tree_root.get(), "");
+    }
+    
     return tree_root.release();
 }
 
 
-CConfig::CConfig(TParamTree* param_tree, EOwnership own)
+CConfig::CConfig(TParamTree* param_tree, EOwnership own, NStr::ECase use_case)
     : m_ParamTree(param_tree, own)
 {
     if ( !param_tree ) {
-        m_ParamTree.reset(new TParamTree, eTakeOwnership);
+        m_ParamTree.reset(new TParamTree(TParamTree::TKeyGetter(use_case)), eTakeOwnership);
     }
 }
 
 
-CConfig::CConfig(const IRegistry& reg)
+CConfig::CConfig(const IRegistry& reg, NStr::ECase use_case)
 {
-    m_ParamTree.reset(ConvertRegToTree(reg), eTakeOwnership);
+    m_ParamTree.reset(ConvertRegToTree(reg, use_case), eTakeOwnership);
     _ASSERT(m_ParamTree.get());
 }
 
 
-CConfig::CConfig(const TParamTree* param_tree)
+CConfig::CConfig(const TParamTree* param_tree, NStr::ECase use_case)
 {
     if ( !param_tree ) {
-        m_ParamTree.reset(new TParamTree, eTakeOwnership);
+        m_ParamTree.reset(new TParamTree(TParamTree::TKeyGetter(use_case)), eTakeOwnership);
     }
     else {
         m_ParamTree.reset(const_cast<TParamTree*>(param_tree), eNoOwnership);
