@@ -1,4 +1,4 @@
-/*  $Id: ncbi_url.cpp 605874 2020-04-16 11:24:34Z ivanov $
+/*  $Id: ncbi_url.cpp 658652 2022-11-04 17:10:53Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,10 @@
 
 BEGIN_NCBI_SCOPE
 
+NCBI_PARAM_DECL(bool, CUrl, enable_parsing_as_index);
+NCBI_PARAM_DEF(bool, CUrl, enable_parsing_as_index, false);
+typedef NCBI_PARAM_TYPE(CUrl, enable_parsing_as_index) TCUrlEnableParsingAsIndex;
+
 //////////////////////////////////////////////////////////////////////////////
 //
 // CUrlArgs_Parser
@@ -105,12 +109,19 @@ void CUrlArgs_Parser::SetQueryString(const string& query,
 
     // If no '=' present in the parsed string then try to parse it as ISINDEX
     // RFC3875
-    if (query.find("=") == NPOS) {
+    if (((m_Flags & fEnableParsingAsIndex) || TCUrlEnableParsingAsIndex::GetDefault()) && (query.find("=") == NPOS)) {
         x_SetIndexString(query, *encoder);
         return;
     }
 
     // Parse into entries
+    string mid_seps = "=&";
+    string end_seps = "&";
+    if (m_Flags & fSemicolonIsArgDelimiter)
+    {
+        mid_seps += ';';
+        end_seps += ';';
+    }
     unsigned int position = 1;
     for (SIZE_TYPE beg = 0; beg < len; ) {
         // ignore ampersand and "&amp;"
@@ -122,21 +133,13 @@ void CUrlArgs_Parser::SetQueryString(const string& query,
             continue;
         }
         // Alternative separator - ';'
-        else if (!m_SemicolonIsNotArgDelimiter  &&  query[beg] == ';')
+        else if ((m_Flags & fSemicolonIsArgDelimiter)  &&  query[beg] == ';')
         {
             ++beg;
             continue;
         }
 
         // parse and URL-decode name
-        string mid_seps = "=&";
-        string end_seps = "&";
-        if (!m_SemicolonIsNotArgDelimiter)
-        {
-            mid_seps += ';';
-            end_seps += ';';
-        }
-
         SIZE_TYPE mid = query.find_first_of(mid_seps, beg);
         // '=' is the first char (empty name)? Skip to the next separator.
         if (mid == beg) {
@@ -177,24 +180,27 @@ void CUrlArgs_Parser::SetQueryString(const string& query,
 // CUrlArgs
 //
 
-CUrlArgs::CUrlArgs(void)
-    : m_Case(NStr::eNocase),
+CUrlArgs::CUrlArgs(TFlags flags)
+    : CUrlArgs_Parser(flags),
+      m_Case(NStr::eNocase),
       m_IsIndex(false)
 {
     return;
 }
 
 
-CUrlArgs::CUrlArgs(const string& query, NStr::EUrlEncode decode)
-    : m_Case(NStr::eNocase),
+CUrlArgs::CUrlArgs(const string& query, NStr::EUrlEncode decode, TFlags flags)
+    : CUrlArgs_Parser(flags),
+      m_Case(NStr::eNocase),
       m_IsIndex(false)
 {
     SetQueryString(query, decode);
 }
 
 
-CUrlArgs::CUrlArgs(const string& query, const IUrlEncoder* encoder)
-    : m_Case(NStr::eNocase),
+CUrlArgs::CUrlArgs(const string& query, const IUrlEncoder* encoder, TFlags flags)
+    : CUrlArgs_Parser(flags),
+      m_Case(NStr::eNocase),
       m_IsIndex(false)
 {
     SetQueryString(query, encoder);
@@ -366,6 +372,40 @@ CUrl& CUrl::operator=(const CUrl& url)
 }
 
 
+bool CUrl::x_IsHostPort(const string& scheme, string& unparsed, const IUrlEncoder& encoder)
+{
+    static set<string> s_StdSchemes{
+        "http", "https", "file", "ftp"
+    };
+
+    // Check for special case: host:port[/path[...]] (see CXX-11455)
+    if (scheme.empty()) return false;
+    string sch_lower = scheme;
+    NStr::ToLower(sch_lower);
+    if (s_StdSchemes.find(sch_lower) != s_StdSchemes.end()) return false;
+
+    SIZE_TYPE port_end = unparsed.find_first_of("/?#");
+    string port = unparsed.substr(0, port_end);
+
+    if (port.empty() ||
+        port[0] == '0' ||
+        port.size() > 5 ||
+        port.find_first_not_of("0123456789") != NPOS) return false;
+    int port_val = atoi(port.c_str());
+    if (port_val > 65535) return false;
+
+    x_SetHost(scheme, encoder);
+    x_SetPort(port, encoder);
+    if (port_end != NPOS) {
+        unparsed = unparsed.substr(port_end);
+    }
+    else {
+        unparsed.clear();
+    }
+    return true;
+}
+
+
 void CUrl::SetUrl(const string& orig_url, const IUrlEncoder* encoder)
 {
     m_Scheme.clear();
@@ -457,7 +497,7 @@ void CUrl::SetUrl(const string& orig_url, const IUrlEncoder* encoder)
             try {
                 x_SetPort(host.substr(pos + 1), *encoder);
             }
-            catch (CStringException) {
+            catch (const CStringException&) {
                 NCBI_THROW2(CUrlParserException, eFormat,
                     "Invalid port value: \"" + orig_url + "\"", pos + 1);
             }
@@ -482,8 +522,12 @@ void CUrl::SetUrl(const string& orig_url, const IUrlEncoder* encoder)
         // No authority, check scheme.
         SIZE_TYPE scheme_end = orig_url.find(':');
         if (scheme_end != NPOS) {
-            x_SetScheme(orig_url.substr(0, scheme_end), *encoder);
+            string scheme = orig_url.substr(0, scheme_end);
             unparsed = orig_url.substr(scheme_end + 1);
+
+            if (!x_IsHostPort(scheme, unparsed, *encoder)) {
+                x_SetScheme(orig_url.substr(0, scheme_end), *encoder);
+            }
         }
         else {
             unparsed = orig_url;
@@ -678,6 +722,9 @@ void CUrl::Adjust(const CUrl& other, TAdjustFlags flags)
             if (m_Path.back() == '/'  &&  other.m_Path.front() == '/') {
                 other_p = 1;
             }
+            else if (m_Path.back() != '/'  &&  other.m_Path.front() != '/') {
+                m_Path.append(1, '/');
+            }
             m_Path.append(other.m_Path.substr(other_p));
         }
     }
