@@ -1,4 +1,4 @@
-/*  $Id: ncbiapp.cpp 620642 2020-11-25 17:54:32Z lavr $
+/*  $Id: ncbiapp.cpp 671295 2023-08-11 12:10:38Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,6 +32,9 @@
  */
 
 #include <ncbi_pch.hpp>
+#ifndef HAVE_LOCAL_NCBI_BUILD_VER_H
+#  define HAVE_LOCAL_NCBI_BUILD_VER_H 1
+#endif
 #include <common/ncbi_source_ver.h>
 #include <corelib/ncbiapp.hpp>
 #undef CNcbiApplication
@@ -69,13 +72,14 @@ BEGIN_NCBI_SCOPE
 //  Constants
 //
 
-static const char* s_ArgLogFile         = "-logfile";
-static const char* s_ArgCfgFile         = "-conffile";
-static const char* s_ArgVersion         = "-version";
-static const char* s_ArgFullVersion     = "-version-full";
-static const char* s_ArgFullVersionXml  = "-version-full-xml";
-static const char* s_ArgFullVersionJson = "-version-full-json";
-static const char* s_ArgDryRun          = "-dryrun";
+extern const char* s_ArgLogFile;
+extern const char* s_ArgCfgFile;
+extern const char* s_ArgVersion;
+extern const char* s_ArgFullVersion;
+extern const char* s_ArgFullVersionXml;
+extern const char* s_ArgFullVersionJson;
+extern const char* s_ArgDryRun;
+extern const char* s_ArgDelimiter;
 
 
 /////////////////////////////////////////////////////////////////////////////
@@ -352,7 +356,13 @@ static void s_MacArgMunging(CNcbiApplicationAPI&   app,
     }
 
     // stash them away in the standard argc and argv places.
-    *argcPtr = v.size();
+    if (v.size() > kMax_Int) {
+        ERR_POST_X(24,
+                   Critical << "Overly high argument count " << v.size()
+                   << "; truncating to " << kMax_Int);
+        v.resize(kMax_Int);
+    }
+    *argcPtr = static_cast<int>(v.size());
 
     char** argv =  new char*[v.size()];
     int c = 0;
@@ -658,9 +668,10 @@ void CNcbiApplicationAPI::x_TryMain(EAppDiagStream diag,
         if (e.GetErrCode() == CArgHelpException::eHelpXml) {
             m_ArgDesc->PrintUsageXml(cout);
         } else {
+            CArgHelpException::TErrCode code =  e.GetErrCode();
             string str;
-            m_ArgDesc->PrintUsage
-                (str, e.GetErrCode() == CArgHelpException::eHelpFull);
+            m_ArgDesc->ShowAllArguments(code == CArgHelpException::eHelpShowAll)->PrintUsage
+                (str, code == CArgHelpException::eHelpFull || code == CArgHelpException::eHelpShowAll);
             cout << str;
         }
         *exit_code = e.GetErrCode() == CArgHelpException::eHelpErr ? 2 : 0;
@@ -804,6 +815,9 @@ int CNcbiApplicationAPI::AppMain
             if ( !argv[i] ) {
                 continue;
             }
+            if ( NStr::strcmp(argv[i], s_ArgDelimiter) == 0 ) {
+                break;
+            }
             if ( NStr::strcmp(argv[i], s_ArgLogFile) == 0 ) {
                 if (!argv[++i]) {
                     continue;
@@ -817,6 +831,7 @@ int CNcbiApplicationAPI::AppMain
             }
         }
     }
+
     // Setup logging as soon as possible.
     // Setup for diagnostics
     try {
@@ -877,14 +892,23 @@ int CNcbiApplicationAPI::AppMain
 
     // Check command line for presence special arguments
     // "-logfile", "-conffile", "-version"
+    const char* conf_arg = nullptr;
     if (!m_DisableArgDesc && argc > 1  &&  argv) {
         const char** v = new const char*[argc];
         v[0] = argv[0];
         int real_arg_index = 1;
+        bool skip = false;
         for (int i = 1;  i < argc;  i++) {
             if ( !argv[i] ) {
                 continue;
             }
+            if ( NStr::strcmp(argv[i], s_ArgDelimiter) == 0 ) {
+                skip = true;
+            }
+            if (skip) {
+                v[real_arg_index++] = argv[i];
+                continue;
+            }
             // Log file - ignore if diag is eDS_User - the user wants to
             // take care about logging.
             if ( diag != eDS_User  &&
@@ -901,14 +925,14 @@ int CNcbiApplicationAPI::AppMain
                 }
                 v[real_arg_index++] = argv[i - 1];
                 v[real_arg_index++] = argv[i];
-                conf = argv[i];
+                conf_arg = argv[i];
 
             }
             else if (NStr::StartsWith(argv[i], s_ArgCfgFile)) {
                 v[real_arg_index++] = argv[i];
                 const char* a = argv[i] + strlen(s_ArgCfgFile);
                 if (*a == '=') {
-                    conf = ++a;
+                    conf_arg = ++a;
                 }
 
                 // Version
@@ -954,6 +978,15 @@ int CNcbiApplicationAPI::AppMain
             argc = real_arg_index;
             argv = v;
         }
+        if (conf_arg) {
+            if (CFile(conf_arg).Exists()) {
+                conf = conf_arg;
+            }
+            else {
+                ERR_POST_X(23, Critical << "Registry file \"" << conf_arg << "\" not found, aborting.");
+                Abort();
+            }
+        }
     }
 
     // Reset command-line args and application name
@@ -1146,27 +1179,7 @@ void CNcbiApplicationAPI::SetupArgDescriptions(CArgDescriptions* arg_desc)
     if ( arg_desc ) {
         if ( !m_DisableArgDesc ) {
             for(CArgDescriptions* desc : m_ArgDesc->GetAllDescriptions()) {
-            // Add logfile and conffile arguments
-            if (!desc->Exist(s_ArgLogFile + 1) ) {
-                desc->AddOptionalKey
-                    (s_ArgLogFile+1, "File_Name",
-                        "File to which the program log should be redirected",
-                        CArgDescriptions::eOutputFile);
-            }
-            if (!desc->Exist(s_ArgCfgFile + 1) ) {
-                if (m_DefaultConfig.empty()) {
-                    desc->AddOptionalKey
-                        (s_ArgCfgFile + 1, "File_Name",
-                            "Program's configuration (registry) data file",
-                            CArgDescriptions::eInputFile);
-                } else {
-                    desc->AddDefaultKey
-                        (s_ArgCfgFile + 1, "File_Name",
-                            "Program's configuration (registry) data file",
-                            CArgDescriptions::eInputFile,
-                            m_DefaultConfig);
-                }
-            }
+                desc->AddDefaultFileArguments(m_DefaultConfig);
             }
         }
         m_Args.reset(arg_desc->CreateArgs(GetArguments()));
@@ -1322,100 +1335,11 @@ void CNcbiApplicationAPI::x_SetupStdio(void)
 
 void CNcbiApplicationAPI::x_AddDefaultArgs(void)
 {
-    if ( !m_DisableArgDesc ) {
-        for(CArgDescriptions* desc : m_ArgDesc->GetAllDescriptions())
-        {
-            if (desc->IsAutoHelpEnabled()) {
-                if ((m_HideArgs & fHideHelp) != 0) {
-                    if (desc->Exist("h")) {
-                        desc->Delete("h");
-                    }
-                }
-            }
-            if ((m_HideArgs & fHideFullHelp) != 0) {
-                if (desc->Exist("help")) {
-                    desc->Delete("help");
-                }
-            }
-            if ((m_HideArgs & fHideXmlHelp) != 0) {
-                if (desc->Exist("xmlhelp")) {
-                    desc->Delete("xmlhelp");
-                }
-            }
-            if ((m_HideArgs & fHideLogfile) != 0) {
-                if (desc->Exist(s_ArgLogFile + 1)) {
-                    desc->Delete(s_ArgLogFile + 1);
-                }
-            } else {
-                if (!desc->Exist(s_ArgLogFile + 1)) {
-                    desc->AddOptionalKey
-                        (s_ArgLogFile+1, "File_Name",
-                            "File to which the program log should be redirected",
-                            CArgDescriptions::eOutputFile);
-                }
-            }
-            if ((m_HideArgs & fHideConffile) != 0) {
-                if (desc->Exist(s_ArgCfgFile + 1)) {
-                    desc->Delete(s_ArgCfgFile + 1);
-                }
-            } else {
-                if (!desc->Exist(s_ArgCfgFile + 1)) {
-                    desc->AddOptionalKey
-                        (s_ArgCfgFile + 1, "File_Name",
-                            "Program's configuration (registry) data file",
-                            CArgDescriptions::eInputFile);
-                }
-            }
-            if ((m_HideArgs & fHideVersion) != 0) {
-                if (desc->Exist(s_ArgVersion + 1)) {
-                    desc->Delete(s_ArgVersion + 1);
-                }
-            } else {
-                if (!desc->Exist(s_ArgVersion + 1)) {
-                    desc->AddFlag
-                        (s_ArgVersion + 1,
-                            "Print version number;  ignore other arguments");
-                }
-            }
-            if ((m_HideArgs & fHideFullVersion) != 0) {
-                if (desc->Exist(s_ArgFullVersion + 1)) {
-                    desc->Delete(s_ArgFullVersion + 1);
-                }
-                if (desc->Exist(s_ArgFullVersionXml+ 1)) {
-                    desc->Delete(s_ArgFullVersionXml + 1);
-                }
-                if (desc->Exist(s_ArgFullVersionJson + 1)) {
-                    desc->Delete(s_ArgFullVersionJson + 1);
-                }
-            } else {
-                if (!desc->Exist(s_ArgFullVersion + 1)) {
-                    desc->AddFlag
-                        (s_ArgFullVersion + 1,
-                            "Print extended version data;  ignore other arguments");
-                }
-                if (!desc->Exist(s_ArgFullVersionXml + 1)) {
-                    desc->AddFlag
-                        (s_ArgFullVersionXml + 1,
-                            "Print extended version data in XML format;  ignore other arguments");
-                }
-                if (!desc->Exist(s_ArgFullVersionJson + 1)) {
-                    desc->AddFlag
-                        (s_ArgFullVersionJson + 1,
-                            "Print extended version data in JSON format;  ignore other arguments");
-                }
-            }
-            if ((m_HideArgs & fHideDryRun) != 0) {
-                if (desc->Exist(s_ArgDryRun + 1)) {
-                    desc->Delete(s_ArgDryRun + 1);
-                }
-            } else {
-                if (!desc->Exist(s_ArgDryRun + 1)) {
-                    desc->AddFlag
-                        (s_ArgDryRun + 1,
-                            "Dry run the application: do nothing, only test all preconditions");
-                }
-            }
-        }
+    THideStdArgs mask = m_DisableArgDesc ? (fHideAll & ~fHideFullHelp & ~fHideHelp) : m_HideArgs;
+    for(CArgDescriptions* desc : m_ArgDesc->GetAllDescriptions())
+    {
+        desc->AddStdArguments(mask);
+        mask = mask | fHideFullVersion | fHideVersion;
     }
 }
 
@@ -1430,8 +1354,7 @@ void CNcbiApplicationAPI::SetProgramDisplayName(const string& app_name)
 }
 
 
-string CNcbiApplicationAPI::GetAppName(EAppNameType name_type, int argc,
-                                    const char* const* argv)
+string CNcbiApplicationAPI::GetAppName(EAppNameType name_type, int argc, const char* const* argv)
 {
     CNcbiApplicationGuard instance = InstanceGuard();
     string app_name;
@@ -1467,13 +1390,30 @@ string CNcbiApplicationAPI::GetAppName(EAppNameType name_type, int argc,
 }
 
 
-string CNcbiApplicationAPI::FindProgramExecutablePath
-(int                           argc,
- const char* const*            argv,
- string*                       real_path)
+string CNcbiApplicationAPI::FindProgramExecutablePath(
+    int                argc,
+    const char* const* argv,
+    string*            real_path
+)
 {
+    // Cache program executable path, it doesn't change during execution
+
+    static CSafeStatic<string> s_Path;
+    static CSafeStatic<string> s_RealPath;
+    static bool s_Init = false;
+
+    if (s_Init) {
+        // Return cached value
+        if (real_path) {
+            *real_path = *s_RealPath;
+        }
+        return *s_Path;
+    }
+    s_Init = true;
+
     CNcbiApplicationGuard instance = InstanceGuard();
     string ret_val;
+
     _ASSERT(argc >= 0); // formally signed for historical reasons
     _ASSERT(argv != NULL  ||  argc == 0);
     if (argc > 0  &&  argv[0] != NULL  &&  argv[0][0] != '\0') {
@@ -1485,8 +1425,8 @@ string CNcbiApplicationAPI::FindProgramExecutablePath
 #if defined(NCBI_OS_MSWIN)  ||  defined(NCBI_OS_UNIX)
 
 #  ifdef NCBI_OS_MSWIN
-    // MS Windows: Try more accurate method of detection
-    // XXX - use this only for real_path?
+
+    // MS Windows: Try more accurate method of detection.
     try {
         // Load PSAPI dynamic library -- it should exist on MS-Win NT/2000/XP
         CDll dll_psapi("psapi.dll", CDll::eLoadNow, CDll::eAutoUnload);
@@ -1499,9 +1439,7 @@ string CNcbiApplicationAPI::FindProgramExecutablePath
                  LPDWORD lpcbNeeded    // number of bytes required
                  ) = NULL;
 
-        dllEnumProcessModules =
-            dll_psapi.GetEntryPoint_Func("EnumProcessModules",
-                                         &dllEnumProcessModules);
+        dllEnumProcessModules = dll_psapi.GetEntryPoint_Func("EnumProcessModules", &dllEnumProcessModules);
         if ( !dllEnumProcessModules ) {
             NCBI_THROW(CException, eUnknown, kEmptyStr);
         }
@@ -1512,18 +1450,18 @@ string CNcbiApplicationAPI::FindProgramExecutablePath
         DWORD   needed  = 0;
 
         // Get first module of current process (it should be .exe file)
-        if ( dllEnumProcessModules(process,
-                                   &module, sizeof(HMODULE), &needed) ) {
+        if ( dllEnumProcessModules(process, &module, sizeof(HMODULE), &needed) ) {
             if ( needed  &&  module ) {
                 TXChar buf[MAX_PATH + 1];
                 DWORD ncount = GetModuleFileName(module, buf, MAX_PATH);
                 if (ncount > 0) {
-                    ret_val = _T_STDSTRING(buf);
+                    // Save to cache and return
+                    *s_Path = _T_STDSTRING(buf);
+                    *s_RealPath = CDirEntry::NormalizePath(*s_Path, eFollowLinks);
                     if (real_path) {
-                        *real_path = CDirEntry::NormalizePath(ret_val,
-                                                              eFollowLinks);
+                        *real_path = *s_RealPath;
                     }
-                    return ret_val;
+                    return *s_Path;
                 }
             }
         }
@@ -1540,13 +1478,15 @@ string CNcbiApplicationAPI::FindProgramExecutablePath
         real_path = &ret_val;
     }
     if (real_path) {
-        char   buf[PATH_MAX + 1];
+        char buf[PATH_MAX + 1];
         string procfile = "/proc/" + NStr::IntToString(getpid()) + "/exe";
         int    ncount   = (int)readlink((procfile).c_str(), buf, PATH_MAX);
         if (ncount > 0) {
             real_path->assign(buf, ncount);
             if (real_path == &ret_val  ||  ret_val.empty()) {
-                // XXX - could also parse /proc/self/cmdline.
+                // Save to cache and return
+                *s_Path = *real_path;
+                *s_RealPath = *real_path;
                 return *real_path;
             }
             real_path = 0;
@@ -1556,11 +1496,23 @@ string CNcbiApplicationAPI::FindProgramExecutablePath
 
     if (ret_val.empty()) {
         // nothing to go on :-/
+        // don't need to update cached values, they are empty
         if (real_path) {
             real_path->erase();
         }
+        // This can happens on non-Windows/Linux platforms if logging is used 
+        // before CNcbiApplication initialization, and no argc/argv valuies passed in arguments.
+        // As a fallback reset the initialization state, so it can be (maybe) initialized correctly later.
+#if !defined(NCBI_OS_MSWIN) && !defined(NCBI_OS_LINUX)
+        if (argc == 0) {
+            s_Init = false;
+        }
+#endif
         return kEmptyStr;
     }
+
+    // Default approach (using value from arguments):
+
     string app_path = ret_val;
 
     if ( !CDirEntry::IsAbsolutePath(app_path) ) {
@@ -1575,7 +1527,7 @@ string CNcbiApplicationAPI::FindProgramExecutablePath
 #  endif
         if ( CFile(app_path).Exists() ) {
             // Relative path from the the current directory
-            app_path = CDir::GetCwd() + CDirEntry::GetPathSeparator()+app_path;
+            app_path = CDir::GetCwd() + CDirEntry::GetPathSeparator() + app_path;
             if ( !CFile(app_path).Exists() ) {
                 app_path = kEmptyStr;
             }
@@ -1606,22 +1558,26 @@ string CNcbiApplicationAPI::FindProgramExecutablePath
             }
         }
     }
-    ret_val = CDirEntry::NormalizePath
-        ((app_path.empty() && argv != NULL && argv[0] != NULL) ? argv[0]
-         : app_path);
+    ret_val = CDirEntry::NormalizePath(
+        (app_path.empty() && argv != NULL && argv[0] != NULL) ? argv[0] : app_path);
 
 #else  // defined (NCBI_OS_MSWIN)  ||  defined(NCBI_OS_UNIX)
 
 #  error "Unsupported platform, sorry -- please contact NCBI"
+
 #endif
+
+    // Save to cache and return
+    *s_Path = ret_val;
+    *s_RealPath = CDirEntry::NormalizePath(ret_val, eFollowLinks);
     if (real_path) {
-        *real_path = CDirEntry::NormalizePath(ret_val, eFollowLinks);
+        *real_path = *s_RealPath;
     }
-    return ret_val;
+    return *s_Path;
 }
 
 
-void CNcbiApplicationAPI::x_HonorStandardSettings( IRegistry* reg)
+void CNcbiApplicationAPI::x_HonorStandardSettings(IRegistry* reg)
 {
     if (reg == 0) {
         reg = m_Config.GetPointer();
