@@ -1,4 +1,4 @@
-/*  $Id: ncbiexec.cpp 604332 2020-03-26 14:49:21Z ivanov $
+/*  $Id: ncbiexec.cpp 663939 2023-03-03 17:10:24Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -230,7 +230,7 @@ s_SpawnUnix(ESpawnFunc func, CExec::EMode full_mode,
 string CExec::QuoteArg(const string& arg)
 {
     // Enclose argument in quotes if it is empty,
-    // or contains spaces and not contains quotes.
+    // or contains spaces and does not contain quotes.
     if ( arg.empty()  ||
         (arg.find(' ') != NPOS  &&  arg.find('"') == NPOS) ) {
         return '"' + arg + '"';
@@ -492,7 +492,7 @@ void s_Create_Env(
         NCBI_THROW(CExecException, eSpawn, "CExec::" #func "() failed"); \
     } \
     CResult result; \
-    if ((mode & fModeMask) == eWait) { \
+    if ((mode & static_cast<EMode>(fModeMask)) == eWait) { \
         result.m_Flags = CResult::fExitCode; \
         result.m_Result.exitcode = (TExitCode)status; \
     } else { \
