@@ -1,4 +1,4 @@
-/*  $Id: ncbireg.cpp 591546 2019-08-16 16:59:06Z vasilche $
+/*  $Id: ncbireg.cpp 663939 2023-03-03 17:10:24Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -619,7 +619,7 @@ IRWRegistry* IRWRegistry::Read(CNcbiIstream& is, TFlags flags,
     if (ef == eEncodingForm_Utf16Native  ||  ef == eEncodingForm_Utf16Foreign) {
         CStringUTF8 s;
         ReadIntoUtf8(is, &s, ef);
-        CNcbiIstrstream iss(s.c_str());
+        CNcbiIstrstream iss(s);
         return x_Read(iss, flags, path);
     } else {
         return x_Read(is, flags, path);
@@ -868,7 +868,8 @@ bool IRWRegistry::Unset(const string& section, const string& name,
                         TFlags flags)
 {
     x_CheckFlags("IRWRegistry::Unset", flags,
-                 fTPFlags | fCountCleared | fSectionlessEntries);
+                 static_cast<TFlags>(fTPFlags) | fCountCleared
+                 | fSectionlessEntries);
     string clean_section = NStr::TruncateSpaces(section);
     if ( !IsNameSection(clean_section, flags) ) {
         _TRACE("IRWRegistry::Unset: bad section name \""
@@ -1654,6 +1655,7 @@ bool CNcbiRegistry::IncludeNcbircIfAllowed(TFlags flags)
             ERR_POST_X(5, Warning << "Resetting m_SysRegistry");
             m_SysRegistry.Reset(entry.registry);
         }
+        if (!entry.actual_name.empty()) return true;
     } catch (CRegistryException& e) {
         ERR_POST_X(6, Critical << "CNcbiRegistry: "
                       "Syntax error in system-wide configuration file: "
