@@ -1,4 +1,4 @@
-/* $Id: ncbistre.cpp 605723 2020-04-14 20:08:31Z lavr $
+/* $Id: ncbistre.cpp 667615 2023-05-18 14:01:14Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -214,6 +214,7 @@ bool NcbiStreamCopy(CNcbiOstream& os, CNcbiIstream& is)
         return false;
     if (CT_EQ_INT_TYPE(is.peek(), CT_EOF)) {
         // NB: C++ Std says nothing about eofbit (27.6.1.3.27)
+        _ASSERT(!is.good());
         return !is.bad();
     }
     os << is.rdbuf();
@@ -234,6 +235,64 @@ void NcbiStreamCopyThrow(CNcbiOstream& os, CNcbiIstream& is)
 }
 
 
+void NcbiStreamCopyHead(CNcbiOstream& os, CNcbiIstream& is, SIZE_TYPE count)
+{
+    if ( !is.good() ) {
+        is.setstate(NcbiFailbit);
+        NCBI_THROW(CCoreException, eCore, "Input stream already in bad state");
+    }
+    if ( os.bad() ) {
+        os.setstate(NcbiFailbit);
+        NCBI_THROW(CCoreException, eCore,
+                   "Output stream already in bad state");
+    }
+    if (CT_EQ_INT_TYPE(is.peek(), CT_EOF)) {
+        // NB: C++ Std says nothing about eofbit (27.6.1.3.27)
+        _ASSERT( !is.good() );
+        if (is.bad()) {
+            NCBI_THROW(CCoreException, eCore,
+                       "Input stream already in bad state (at EOF)");
+        }
+    }
+    char      buffer[16384];
+    SIZE_TYPE ninstream = count, ninbuffer = 0;
+    auto      outbuf = os.rdbuf();
+    while (ninstream > 0  ||  ninbuffer > 0) {
+        _ASSERT(ninbuffer < sizeof(buffer));
+        auto nwanted  = min(sizeof(buffer) - ninbuffer, ninstream);
+        streamsize nread = 0;
+        if (nwanted > 0) {
+            is.read(buffer + ninbuffer, nwanted);
+            nread = is.gcount();
+            if ( !is.good() ) {
+                is.setstate(NcbiFailbit);
+                ninstream = nread;
+            }
+        }
+        auto nwritten = outbuf->sputn(buffer, nread + ninbuffer);
+        if ( os.bad()  ||  nwritten == 0) {
+            os.setstate(NcbiFailbit);
+            NCBI_THROW(CCoreException, eCore, "Write error");
+        }
+        _ASSERT(static_cast<SIZE_TYPE>(nwritten) <= nread + ninbuffer);
+        ninbuffer = nread + ninbuffer - nwritten;
+        if (ninbuffer > 0) {
+            memmove(buffer, buffer + nwritten, ninbuffer);
+        }
+        _ASSERT(static_cast<SIZE_TYPE>(nread) <= ninstream);
+        ninstream -= nread;
+    }
+    if ( !os.flush() ) {
+        NCBI_THROW(CCoreException, eCore, "Flush error");
+    }
+    // Deferred to ensure writing as much as possible when reading fails
+    // and writing sometimes yields leftovers.
+    if ( is.bad() ) {
+        NCBI_THROW(CCoreException, eCore, "Read error");
+    }
+}
+
+
 size_t NcbiStreamToString(string* str, CNcbiIstream& is, size_t pos)
 {
     if (!is.good()) {
@@ -360,7 +419,7 @@ bool NcbiStreamCompareText(CNcbiIstream& is1, CNcbiIstream& is2,
 bool NcbiStreamCompareText(CNcbiIstream& is, const string& str,
                            ECompareTextMode mode, size_t buf_size)
 {
-    CNcbiIstrstream istr(str.data(), str.size());
+    CNcbiIstrstream istr(str);
     return NcbiStreamCompareText(is, istr, mode, buf_size);
 }
 
@@ -449,24 +508,24 @@ string Printable(char c)
 
     string s;
     switch ( c ) {
-    case '\0':  s += "\\0";   break;
-    case '\t':  s += "\\t";   break;
-    case '\v':  s += "\\v";   break;
-    case '\b':  s += "\\b";   break;
-    case '\r':  s += "\\r";   break;
-    case '\f':  s += "\\f";   break;
-    case '\a':  s += "\\a";   break;
-    case '\n':  s += "\\n";   break;
-    case '\\':  s += "\\\\";  break;
-    case '\'':  s += "\\'";   break;
-    case '"':   s += "\\\"";  break;
+    case '\0':  s = "\\0";   break;
+    case '\a':  s = "\\a";   break;
+    case '\b':  s = "\\b";   break;
+    case '\f':  s = "\\f";   break;
+    case '\n':  s = "\\n";   break;
+    case '\r':  s = "\\r";   break;
+    case '\t':  s = "\\t";   break;
+    case '\v':  s = "\\v";   break;
+    case '\\':  s = "\\\\";  break;
+    case '\'':  s = "\\'";   break;
+    case '"':   s = "\\\"";  break;
     default:
         if ( !isprint((unsigned char) c) ) {
-            s += "\\x";
+            s  = "\\x";
             s += kHex[(unsigned char) c / 16];
             s += kHex[(unsigned char) c % 16];
         } else
-            s += c;
+            s  = c;
         break;
     }
     return s;
@@ -474,29 +533,25 @@ string Printable(char c)
 
 
 static inline
-bool s_IsQuoted(char c)
-{
-    return (c == '\t'  ||   c == '\v'  ||  c == '\b'  ||
-            c == '\r'  ||   c == '\f'  ||  c == '\a'  ||
-            c == '\n'  ||   c == '\\'  ||  c == '\''  ||
-            c == '"'   ||  !isprint((unsigned char) c) ? true : false);
-}
-
-
-static inline
-void s_WritePrintable(CNcbiOstream& out, char c, char n)
+void s_WritePrintable(CNcbiOstream& out, char p, char c, char n)
 {
     switch ( c ) {
+    case '\a':  out.write("\\a",  2);  return;
+    case '\b':  out.write("\\b",  2);  return;
+    case '\f':  out.write("\\f",  2);  return;
+    case '\n':  out.write("\\n",  2);  return;
+    case '\r':  out.write("\\r",  2);  return;
     case '\t':  out.write("\\t",  2);  return;
     case '\v':  out.write("\\v",  2);  return;
-    case '\b':  out.write("\\b",  2);  return;
-    case '\r':  out.write("\\r",  2);  return;
-    case '\f':  out.write("\\f",  2);  return;
-    case '\a':  out.write("\\a",  2);  return;
-    case '\n':  out.write("\\n",  2);  return;
     case '\\':  out.write("\\\\", 2);  return;
     case '\'':  out.write("\\'",  2);  return;
     case '"':   out.write("\\\"", 2);  return;
+    case '?':
+        if (p == '?'  ||  n == '?')
+            out.write("\\?", 2);
+        else
+            out.put(c);
+        return;
     default:
         if ( isprint((unsigned char) c) ) {
             out.put(c);
@@ -505,13 +560,13 @@ void s_WritePrintable(CNcbiOstream& out, char c, char n)
         break;
     }
 
-    bool full = !s_IsQuoted(n)  &&  '0' <= n  &&  n <= '7' ? true : false;
+    bool full = '0' <= n  &&  n <= '7' ? true : false;
     unsigned char v;
     char octal[4];
     int k = 1;
 
     *octal = '\\';
-    v = (unsigned char)((unsigned char) c >> 6);
+    v =  (unsigned char) c >> 6;
     if (v  ||  full) {
         octal[k++] = char('0' + v);
         full = true;
@@ -520,8 +575,8 @@ void s_WritePrintable(CNcbiOstream& out, char c, char n)
     if (v  ||  full) {
         octal[k++] = char('0' + v);
     }
-    v = (unsigned char) c & 7;
-    octal[k++] = char('0' + v);
+    v =  (unsigned char) c       & 7;
+    octal[k++]     = char('0' + v);
     out.write(octal, k);
 }
 
@@ -530,11 +585,13 @@ CNcbiOstream& operator<<(CNcbiOstream& out, CPrintableStringConverter s)
 {
     size_t size = s.m_String.size();
     if (size) {
+        char prev = '\0';
         const char* data = s.m_String.data();
         for (size_t i = 0;  i < size - 1;  ++i) {
-            s_WritePrintable(out, data[i], data[i + 1]);
+            s_WritePrintable(out, prev, data[i], data[i + 1]);
+            prev = data[i];
         }
-        s_WritePrintable(out, data[size - 1], '\0');
+        s_WritePrintable(out, prev, data[size - 1], '\0');
     }
     return out;
 }
@@ -542,11 +599,13 @@ CNcbiOstream& operator<<(CNcbiOstream& out, CPrintableStringConverter s)
 
 CNcbiOstream& operator<<(CNcbiOstream& out, CPrintableCharPtrConverter s)
 {
-    const char* p = s.m_String;
-    char        c = *p;
+    const char* t = s.m_String;
+    char        p = '\0';
+    char        c = *t;
     while (c) {
-        char n = *++p;
-        s_WritePrintable(out, c, n);
+        char n = *++t;
+        s_WritePrintable(out, p, c, n);
+        p = c;
         c = n;
     }
     return out;
