@@ -1,4 +1,4 @@
-/*  $Id: request_ctx.cpp 632469 2021-06-02 11:12:10Z ivanov $
+/*  $Id: request_ctx.cpp 664132 2023-03-08 14:20:02Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -45,15 +45,123 @@
 BEGIN_NCBI_SCOPE
 
 
-static const char* kPassThrough_Sid = "ncbi_sid";
-static const char* kPassThrough_Phid = "ncbi_phid";
+static const char* kPassThrough_Sid      = "ncbi_sid";
+static const char* kPassThrough_Phid     = "ncbi_phid";
 static const char* kPassThrough_ClientIp = "ncbi_client_ip";
-static const char* kPassThrough_Dtab = "ncbi_dtab";
+static const char* kPassThrough_Dtab     = "ncbi_dtab";
 
 
 CAtomicCounter CRequestContext::sm_VersionCounter;
 
 
+// What to do on a bad hit ID -- allow-and-report by default
+
+enum EOnBadHitID {
+    eOnBadPHID_Allow,
+    eOnBadPHID_AllowAndReport,
+    eOnBadPHID_Ignore,
+    eOnBadPHID_IgnoreAndReport,
+    eOnBadPHID_Throw
+};
+
+NCBI_PARAM_ENUM_DECL(EOnBadHitID, Log, On_Bad_Hit_Id);
+NCBI_PARAM_ENUM_ARRAY(EOnBadHitID, Log, On_Bad_Hit_Id)
+{
+    {"Allow", eOnBadPHID_Allow},
+    {"AllowAndReport", eOnBadPHID_AllowAndReport},
+    {"Ignore", eOnBadPHID_Ignore},
+    {"IgnoreAndReport", eOnBadPHID_IgnoreAndReport},
+    {"Throw", eOnBadPHID_Throw}
+};
+NCBI_PARAM_ENUM_DEF_EX(EOnBadHitID, Log, On_Bad_Hit_Id, 
+                       eOnBadPHID_AllowAndReport,
+                       eParam_NoThread,
+                       LOG_ON_BAD_HIT_ID);
+typedef NCBI_PARAM_TYPE(Log, On_Bad_Hit_Id) TOnBadHitId;
+
+
+static const char* kAllowedIdMarkchars = "_-.:@";
+
+
+static bool IsValidHitID(const string& hit) 
+{
+    string id_std = kAllowedIdMarkchars;
+    size_t pos = 0;
+    size_t sep_pos = NPOS;
+    // Allow aphanumeric and some markup before the first separator (dot).
+    for (; pos < hit.size(); pos++) {
+        char c = hit[pos];
+        if (c == '.') {
+            sep_pos = pos;
+            break;
+        }
+        if (!isalnum(c)  &&  id_std.find(c) == NPOS) {
+            return false;
+        }
+    }
+    // Hit id must be present before the first separator.
+    // Note that empty hit id is still allowed if there are no sub-hit ids.
+    if (sep_pos == 0) return false;
+    if (sep_pos == NPOS) return true;
+    // Separator found - make sure the rest of the id contains only separators
+    // and valid sub-hit ids: a prefix consisting of allowed chars and some
+    // digits (XYZ.1.J2.T34).
+    size_t last_digit = sep_pos;
+    for (pos = sep_pos + 1; pos < hit.size(); pos++) {
+        char c = hit[pos];
+        if (c == '.') {
+            // Need at least one char between separators.
+            if (pos == sep_pos + 1) return false;
+            // Need at least one digit before separator.
+            if (last_digit + 1 != pos) return false;
+            sep_pos = pos;
+            continue;
+        }
+        if (!isalnum(c)  &&  id_std.find(c) == NPOS) {
+            return false;
+        }
+        if ( isdigit(c) ) last_digit = pos;
+    }
+    // Make sure the last char is digit.
+    return pos > sep_pos + 1;
+}
+
+
+void CSharedHitId::x_SetHitId(const string& hit_id)
+{
+    if (GetHitId() == hit_id) {
+        return;
+    }
+    if (IsValidHitID(hit_id)) {
+        m_HitId = hit_id;
+    } else {
+        static CSafeStatic<TOnBadHitId> action;
+        switch (action->Get()) {
+        case eOnBadPHID_Allow:
+        case eOnBadPHID_AllowAndReport:
+            // Regardless of the allow status lets sanitize bad value, 
+            // allow letters, digits and selected marks only, remlace all bad chars with '_'.
+            // Resulting PHID still can be invalid, see CXX-12891
+            m_HitId = NStr::Sanitize(hit_id, kAllowedIdMarkchars, "", '_', NStr::fSS_alnum | NStr::fSS_NoMerge);
+            if (action->Get() == eOnBadPHID_AllowAndReport) {
+                ERR_POST_X(27, Warning << "Bad hit ID format: " << NStr::PrintableString(hit_id)
+                                       << ", sanitized value will be used: " << m_HitId);
+            }
+            break;
+        case eOnBadPHID_Ignore:
+            return;
+        case eOnBadPHID_IgnoreAndReport:
+            ERR_POST_X(27, Warning << "Bad hit ID format: " << NStr::PrintableString(hit_id));
+            return;
+        case eOnBadPHID_Throw:
+            NCBI_THROW(CRequestContextException, eBadHit, "Bad hit ID format: " + NStr::PrintableString(hit_id));
+            break;
+        }
+    }
+}
+
+
+
 CRequestContext::CRequestContext(TContextFlags flags)
     : m_RequestID(0),
       m_AppState(eDiagAppState_NotSet),
@@ -238,12 +346,18 @@ void CRequestContext::StartRequest(void)
     GetRequestTimer().Restart();
     m_IsRunning = true;
     x_LogHitID();
+    if (m_Tracer) {
+        m_Tracer->OnRequestStart(*this);
+    }
 }
 
 
 void CRequestContext::StopRequest(void)
 {
     if (!x_CanModify()) return;
+    if (m_Tracer) {
+        m_Tracer->OnRequestStop(*this);
+    }
     if ((m_HitIDLoggedFlag & fLoggedOnRequest) == 0) {
         // Hit id has not been set or logged yet. Try to log the default one.
         x_GetHitID(CDiagContext::eHitID_NoCreate);
@@ -272,109 +386,19 @@ bool CRequestContext::GetDefaultAutoIncRequestIDOnPost(void)
 }
 
 
-enum EOnBadHitID {
-    eOnBadPHID_Allow,
-    eOnBadPHID_AllowAndReport,
-    eOnBadPHID_Ignore,
-    eOnBadPHID_IgnoreAndReport,
-    eOnBadPHID_Throw
-};
 
-NCBI_PARAM_ENUM_DECL(EOnBadHitID, Log, On_Bad_Hit_Id);
-NCBI_PARAM_ENUM_ARRAY(EOnBadHitID, Log, On_Bad_Hit_Id)
-{
-    {"Allow", eOnBadPHID_Allow},
-    {"AllowAndReport", eOnBadPHID_AllowAndReport},
-    {"Ignore", eOnBadPHID_Ignore},
-    {"IgnoreAndReport", eOnBadPHID_IgnoreAndReport},
-    {"Throw", eOnBadPHID_Throw}
-};
-NCBI_PARAM_ENUM_DEF_EX(EOnBadHitID, Log, On_Bad_Hit_Id,
-                       eOnBadPHID_AllowAndReport,
-                       eParam_NoThread,
-                       LOG_ON_BAD_HIT_ID);
-typedef NCBI_PARAM_TYPE(Log, On_Bad_Hit_Id) TOnBadHitId;
-
-
-static const char* kAllowedIdMarkchars = "_-.:@";
-
-
-static bool IsValidHitID(const string& hit) {
-    string id_std = kAllowedIdMarkchars;
-    size_t pos = 0;
-    size_t sep_pos = NPOS;
-    // Allow aphanumeric and some markup before the first separator (dot).
-    for (; pos < hit.size(); pos++) {
-        char c = hit[pos];
-        if (c == '.') {
-            sep_pos = pos;
-            break;
-        }
-        if (!isalnum(hit[pos])  &&  id_std.find(hit[pos]) == NPOS) {
-            return false;
-        }
-    }
-    // Hit id must be present before the first separator.
-    // Note that empty hit id is still allowed if there are no sub-hit ids.
-    if (sep_pos == 0) return false;
-    if (sep_pos == NPOS) return true;
-    // Separator found - make sure the rest of the id contains only separators
-    // and valid sub-hit ids: a prefix consisting of allowed chars and some
-    // digits (XYZ.1.J2.T34).
-    size_t last_digit = sep_pos;
-    for (pos = sep_pos + 1; pos < hit.size(); pos++) {
-        char c = hit[pos];
-        if (c == '.') {
-            // Need at least one char between separators.
-            if (pos == sep_pos + 1) return false;
-            // Need at least one digit before separator.
-            if (last_digit + 1 != pos) return false;
-            sep_pos = pos;
-            continue;
-        }
-        if (!isalnum(hit[pos])  &&  id_std.find(hit[pos]) == NPOS) {
-            return false;
-        }
-        if ( isdigit(c) ) last_digit = pos;
-    }
-    // Make sure the last char is digit.
-    return pos > sep_pos + 1;
-}
 
 
 void CRequestContext::x_SetHitID(const CSharedHitId& hit_id)
 {
     if (!x_CanModify()) return;
+
     const string& hit = hit_id.GetHitId();
     if (m_HitIDLoggedFlag & fLoggedOnRequest) {
         // Show warning when changing hit id after is has been logged.
-        ERR_POST_X(28, Warning << "Changing hit ID after one has been logged. "
-            "New hit id is: " << hit);
-    }
-
-    if (m_HitID.GetHitId() != hit) {
-        static CSafeStatic<TOnBadHitId> action;
-        if (!IsValidHitID(hit)) {
-            switch (action->Get()) {
-            case eOnBadPHID_Ignore:
-                return;
-            case eOnBadPHID_AllowAndReport:
-                ERR_POST_X(27, Warning << "Bad hit ID format: " << hit);
-                break;
-            case eOnBadPHID_IgnoreAndReport:
-                ERR_POST_X(27, Warning << "Bad hit ID format: " << hit);
-                return;
-            case eOnBadPHID_Throw:
-                NCBI_THROW(CRequestContextException, eBadHit,
-                    "Bad hit ID format: " + hit);
-                break;
-            case eOnBadPHID_Allow:
-                break;
-            }
-        }
+        ERR_POST_X(28, Warning << "Changing hit ID after one has been logged. New hit id is: " << hit);
     }
     x_SetProp(eProp_HitID);
-
     m_SubHitIDCache.clear();
     m_HitID = hit_id;
     x_Modify();
@@ -578,7 +602,8 @@ string CRequestContext::SelectLastHitID(const string& hit_ids)
         return hit_ids;
     }
     list<string> ids;
-    NStr::Split(hit_ids, ", ", ids,
+    // '+' is a temporary workaround for mismatched encoding/decoding of spaces.
+    NStr::Split(hit_ids, ", +", ids,
         NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
     return ids.empty() ? kEmptyStr : ids.back();
 }
@@ -876,7 +901,7 @@ const char* CRequestContextException::GetErrCodeString(void) const
 CRequestContextGuard_Base::CRequestContextGuard_Base(CRequestContext* context, TFlags flags)
     : m_Flags(flags)
 {
-    m_OriginatesFromThrow = uncaught_exception();
+    m_OriginatesFromThrow = uncaught_exceptions();
     CDiagContext& ctx = GetDiagContext();
     if ( context ) {
         m_SavedContext.Reset(&ctx.GetRequestContext());
@@ -899,7 +924,7 @@ CRequestContextGuard_Base::~CRequestContextGuard_Base(void)
     // If released, do not perform any actions.
     if ( !m_RequestContext ) return;
 
-    if ( uncaught_exception()  &&  !m_OriginatesFromThrow ) {
+    if ( uncaught_exceptions()  &&  !m_OriginatesFromThrow ) {
         SetStatus(m_ErrorStatus);
     }
     else if ( !m_RequestContext->IsSetRequestStatus() ) {
