@@ -1,4 +1,4 @@
-/*  $Id: version.cpp 619781 2020-11-10 18:40:38Z gouriano $
+/*  $Id: version.cpp 657706 2022-10-14 15:58:58Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -168,7 +168,7 @@ string CVersionInfo::PrintJson(void) const
     }
     if ( !m_Name.empty() ) {
         if ( need_separator ) os << ", ";
-        os << "\"name\": \"" << NStr::JsonEncode(m_Name) << "\"";
+        os << "\"name\": " << NStr::JsonEncode(m_Name, NStr::eJsonEnc_Quoted);
     }
     os << "}";
     return CNcbiOstrstreamToString(os);
@@ -432,12 +432,9 @@ string CComponentVersionInfoAPI::PrintXml(void) const
 string CComponentVersionInfoAPI::PrintJson(void) const
 {
     CNcbiOstrstream os;
-    os << "{ \"name\": \"" <<
-        NStr::JsonEncode(GetComponentName()) <<
-            "\", \"version_info\": " <<
-            CVersionInfo::PrintJson() << ",\n" <<
-            "        \"build_info\": " <<
-            m_BuildInfo.PrintJson() << "}";
+    os << "{ \"name\": " << NStr::JsonEncode(GetComponentName(),NStr::eJsonEnc_Quoted) <<
+            ", \"version_info\": " << CVersionInfo::PrintJson() << ",\n" <<
+            "        \"build_info\": " << m_BuildInfo.PrintJson() << "}";
     return CNcbiOstrstreamToString(os);
 }
 
@@ -495,6 +492,7 @@ string SBuildInfo::ExtraName(EExtra key)
     case eDevelopmentVersion:      return "Development-Version";
     case eProductionVersion:       return "Production-Version";
     case eBuiltAs:                 return "Built-As";
+    case eRevision:                return "Revision";
     }
     return "Unknown";
 }
@@ -514,6 +512,7 @@ string SBuildInfo::ExtraNameAppLog(EExtra key)
     case eDevelopmentVersion:      return "ncbi_app_dev_version";
     case eProductionVersion:       return "ncbi_app_prod_version";
     case eBuiltAs:                 return "ncbi_app_built_as";
+    case eRevision:                return "ncbi_app_revision";
     }
     return "ncbi_app_unk";
 }
@@ -569,30 +568,40 @@ string SBuildInfo::PrintXml(void) const
     return CNcbiOstrstreamToString(os);
 }
 
-
 string SBuildInfo::PrintJson(void) const
 {
     CNcbiOstrstream os;
     bool need_separator = false;
     os << '{';
     if ( !date.empty() ) {
-        os << "\"" << ExtraNameJson(eBuildDate) << "\": \"" << NStr::JsonEncode(date) << '\"';
+        os << "\"" << ExtraNameJson(eBuildDate) << "\": " << NStr::JsonEncode(date, NStr::eJsonEnc_Quoted);
         need_separator = true;
     }
     if ( !tag.empty() ) {
         if ( need_separator ) os << ", ";
-        os << '\"' << ExtraNameJson(eBuildTag) << "\": \"" << NStr::JsonEncode(tag) << '\"';
+        os << '\"' << ExtraNameJson(eBuildTag) << "\": " << NStr::JsonEncode(tag, NStr::eJsonEnc_Quoted);
         need_separator = true;
     }
     for( const auto& e : m_extra) {
         if ( need_separator ) os << ", ";
-        os << '\"' << ExtraNameJson(e.first) << "\": \"" << NStr::JsonEncode(e.second) << '\"';
+        os << '\"' << ExtraNameJson(e.first) << "\": " << NStr::JsonEncode(e.second, NStr::eJsonEnc_Quoted);
         need_separator = true;
     }
     os << '}';
     return CNcbiOstrstreamToString(os);
 }
 
+CTime SBuildInfo::GetBuildTime(void) const
+{
+    // According to the documentation, if the compiler cannot get the time of day 
+    // it will give you question marks for __DATE__ and __TIME__. 
+    try {
+        return CTime(date, "b D Y h:m:s");
+    }
+    catch (CTimeException&) {}
+    return CTime();
+}
+
 
 /////////////////////////////////////////////////////////////////////////////
 //  CVersion
@@ -809,7 +818,7 @@ string CVersionAPI::PrintJson(const string& appname, TPrintFlags flags) const
 
     if (flags & fVersionInfo) {
         if ( !appname.empty() ) {
-            os << "    \"appname\": \"" << NStr::JsonEncode(appname) << "\",\n";
+            os << "    \"appname\": " << NStr::JsonEncode(appname, NStr::eJsonEnc_Quoted) << ",\n";
         }
         os << "    \"version_info\": " << m_VersionInfo->PrintJson();
         need_separator = true;
@@ -832,11 +841,11 @@ string CVersionAPI::PrintJson(const string& appname, TPrintFlags flags) const
     if (flags & ( fPackageShort | fPackageFull )) {
         if ( need_separator ) os << ",\n";
         os << "    \"package\": {\n" <<
-            "      \"name\": \"" << NStr::JsonEncode(GetPackageName()) << "\",\n" <<
+            "      \"name\": " << NStr::JsonEncode(GetPackageName(), NStr::eJsonEnc_Quoted) << ",\n" <<
             "      \"version_info\": " << GetPackageVersion().PrintJson() << ",\n" <<
             "      \"build_info\": " << NCBI_SBUILDINFO_DEFAULT().PrintJson();
         if (flags & fPackageFull) {
-            os << ",\n      \"config\": \"" << NStr::JsonEncode(GetPackageConfig()) << "\"";
+            os << ",\n      \"config\": " << NStr::JsonEncode(GetPackageConfig(), NStr::eJsonEnc_Quoted);
         }
         os << "}";
         need_separator = true;
@@ -846,7 +855,7 @@ string CVersionAPI::PrintJson(const string& appname, TPrintFlags flags) const
 #ifdef NCBI_SIGNATURE
     if (flags & fBuildSignature) {
         if ( need_separator ) os << ",\n";
-        os << "    \"build_signature\": \"" << NStr::JsonEncode(NCBI_SIGNATURE) << "\"";
+        os << "    \"build_signature\": " << NStr::JsonEncode(NCBI_SIGNATURE, NStr::eJsonEnc_Quoted);
         need_separator = true;
     }
 #endif
