@@ -1,4 +1,4 @@
-/*  $Id: dbapi_conn_factory.cpp 630683 2021-05-06 15:49:27Z ivanov $
+/*  $Id: dbapi_conn_factory.cpp 682489 2024-05-01 12:40:05Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -65,11 +65,6 @@ m_TryServerToo(false)
     ConfigureFromRegistry(registry);
 }
 
-CDBConnectionFactory::~CDBConnectionFactory(void)
-{
-    CDB_UserHandler::ClearExceptions(m_Errors);
-}
-
 void
 CDBConnectionFactory::ConfigureFromRegistry(const IRegistry* registry)
 {
@@ -109,7 +104,7 @@ CDBConnectionFactory::ConfigureFromRegistry(const IRegistry* registry)
 void
 CDBConnectionFactory::Configure(const IRegistry* registry)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CWriteLockGuard guard(m_Lock);
 
     ConfigureFromRegistry(registry);
 }
@@ -117,7 +112,7 @@ CDBConnectionFactory::Configure(const IRegistry* registry)
 void
 CDBConnectionFactory::SetMaxNumOfConnAttempts(unsigned int max_num)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CWriteLockGuard guard(m_Lock);
 
     m_MaxNumOfConnAttempts = max_num;
 }
@@ -125,7 +120,7 @@ CDBConnectionFactory::SetMaxNumOfConnAttempts(unsigned int max_num)
 void
 CDBConnectionFactory::SetMaxNumOfValidationAttempts(unsigned int max_num)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CWriteLockGuard guard(m_Lock);
 
     m_MaxNumOfValidationAttempts = max_num;
 }
@@ -133,7 +128,7 @@ CDBConnectionFactory::SetMaxNumOfValidationAttempts(unsigned int max_num)
 void
 CDBConnectionFactory::SetMaxNumOfServerAlternatives(unsigned int max_num)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CWriteLockGuard guard(m_Lock);
 
     m_MaxNumOfServerAlternatives = max_num;
 }
@@ -141,7 +136,7 @@ CDBConnectionFactory::SetMaxNumOfServerAlternatives(unsigned int max_num)
 void
 CDBConnectionFactory::SetMaxNumOfDispatches(unsigned int max_num)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CWriteLockGuard guard(m_Lock);
 
     m_MaxNumOfDispatches = max_num;
 }
@@ -149,7 +144,7 @@ CDBConnectionFactory::SetMaxNumOfDispatches(unsigned int max_num)
 void
 CDBConnectionFactory::SetConnectionTimeout(unsigned int timeout)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CWriteLockGuard guard(m_Lock);
 
     m_ConnectionTimeout = timeout;
 }
@@ -157,50 +152,11 @@ CDBConnectionFactory::SetConnectionTimeout(unsigned int timeout)
 void
 CDBConnectionFactory::SetLoginTimeout(unsigned int timeout)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CWriteLockGuard guard(m_Lock);
 
     m_LoginTimeout = timeout;
 }
 
-unsigned int
-CDBConnectionFactory::CalculateConnectionTimeout
-(const I_DriverContext& ctx) const
-{
-    unsigned int timeout = 30;
-
-    unsigned int to = GetConnectionTimeout();
-    if (to != 0) {
-        timeout = to;
-    }
-    else {
-        to = ctx.GetTimeout();
-        if (to != 0) {
-            timeout = to;
-        }
-    }
-
-    return timeout;
-}
-
-unsigned int
-CDBConnectionFactory::CalculateLoginTimeout(const I_DriverContext& ctx) const
-{
-    unsigned int timeout = 30;
-
-    unsigned int to = GetLoginTimeout();
-    if (to != 0) {
-        timeout = to;
-    }
-    else {
-        to = ctx.GetLoginTimeout();
-        if (to != 0) {
-            timeout = to;
-        }
-    }
-
-    return timeout;
-}
-
 CDBConnectionFactory::CRuntimeData&
 CDBConnectionFactory::GetRuntimeData(const CRef<IConnValidator> validator)
 {
@@ -214,6 +170,7 @@ CDBConnectionFactory::GetRuntimeData(const CRef<IConnValidator> validator)
 CDBConnectionFactory::CRuntimeData&
 CDBConnectionFactory::GetRuntimeData(const string& validator_name)
 {
+    CFastMutexGuard guard(m_ValidatorSetMutex);
     TValidatorSet::iterator it = m_ValidatorSet.find(validator_name);
     if (it != m_ValidatorSet.end()) {
         return it->second;
@@ -227,34 +184,6 @@ CDBConnectionFactory::GetRuntimeData(const string& validator_name)
 
 
 ///////////////////////////////////////////////////////////////////////////////
-class CDB_DBLB_Delegate : public CDBConnParamsDelegate
-{
-public:
-    CDB_DBLB_Delegate(
-            const string& srv_name,
-            Uint4 host,
-            Uint2 port,
-            const CDBConnParams& other);
-    virtual ~CDB_DBLB_Delegate(void);
-
-public:
-    virtual string GetServerName(void) const;
-    virtual Uint4 GetHost(void) const;
-    virtual Uint2  GetPort(void) const;
-    virtual const impl::CDBHandlerStack& GetOpeningMsgHandlers(void) const;
-    impl::CDBHandlerStack& SetOpeningMsgHandlers(void);
-
-private:
-    // Non-copyable.
-    CDB_DBLB_Delegate(const CDB_DBLB_Delegate& other);
-    CDB_DBLB_Delegate& operator =(const CDB_DBLB_Delegate& other);
-
-private:
-    const string m_ServerName;
-    const Uint4  m_Host;
-    const Uint2  m_Port;
-    impl::CDBHandlerStack m_OpeningMsgHandlers;
-};
 
 
 CDB_DBLB_Delegate::CDB_DBLB_Delegate(
@@ -304,23 +233,27 @@ impl::CDBHandlerStack& CDB_DBLB_Delegate::SetOpeningMsgHandlers(void)
 
 ///////////////////////////////////////////////////////////////////////////////
 CDBConnectionFactory::SOpeningContext::SOpeningContext
-(I_DriverContext& driver_ctx_in)
-    : driver_ctx(driver_ctx_in), conn_status(IConnValidator::eInvalidConn)
+(I_DriverContext& driver_ctx_in, CServiceInfo& service_info_in,
+ CDB_UserHandler::TExceptions& exceptions)
+    : driver_ctx(driver_ctx_in), service_info(service_info_in),
+      conn_status(IConnValidator::eInvalidConn), errors(exceptions)
 {
 }
 
 CDB_Connection*
 CDBConnectionFactory::MakeDBConnection(
     I_DriverContext& ctx,
-    const CDBConnParams& params)
+    const CDBConnParams& params,
+    CDB_UserHandler::TExceptions& exceptions)
 {
-    CFastMutexGuard mg(m_Mtx);
-
-    CDB_UserHandler::ClearExceptions(m_Errors);
+    CReadLockGuard guard(m_Lock);
 
     unique_ptr<CDB_Connection> t_con;
     CRuntimeData& rt_data = GetRuntimeData(params.GetConnValidator());
-    TSvrRef dsp_srv = rt_data.GetDispatchedServer(params.GetServerName());
+    CServiceInfo& service_info
+        = rt_data.GetServiceInfo(params.GetServerName());
+    CServiceInfo::TGuard service_guard(service_info);
+    TSvrRef dsp_srv = service_info.GetDispatchedServer();
 
     // Prepare to collect messages, whose proper severity depends on whether
     // ANY attempt succeeds.
@@ -336,18 +269,11 @@ CDBConnectionFactory::MakeDBConnection(
             ultimate_handlers.Push(&CDB_UserHandler::GetDefault());
         }
     }}
-    SOpeningContext opening_ctx(ctx);
+    SOpeningContext opening_ctx(ctx, service_info, exceptions);
     CRef<CDB_UserHandler_Deferred> handler
         (new CDB_UserHandler_Deferred(ultimate_handlers));
     opening_ctx.handlers.Push(&*handler, eTakeOwnership);
     
-    // Store original query timeout ...
-    unsigned int query_timeout = ctx.GetTimeout();
-
-    // Set "validation timeouts" ...
-    ctx.SetTimeout(CalculateConnectionTimeout(ctx));
-    ctx.SetLoginTimeout(CalculateLoginTimeout(ctx));
-
     if ( dsp_srv.Empty() ) {
         // We are here either because server name was never dispatched or
         // because a named connection pool has been used before.
@@ -362,7 +288,7 @@ CDBConnectionFactory::MakeDBConnection(
         // We probably need to re-dispatch it ...
         if (single_server != "true"
             &&  ((GetMaxNumOfDispatches()
-                  &&  (rt_data.GetNumOfDispatches(params.GetServerName())
+                  &&  (service_info.GetNumOfDispatches()
                        >= GetMaxNumOfDispatches()))
                  || (dsp_srv->GetExpireTime() != 0
                      &&  (CurrentTime(CTime::eUTC).GetTimeT()
@@ -370,10 +296,10 @@ CDBConnectionFactory::MakeDBConnection(
             // We definitely need to re-dispatch it ...
 
             // Clean previous info ...
-            rt_data.CleanExcluded(params.GetServerName());
-            rt_data.SetDispatchedServer(params.GetServerName(), TSvrRef());
+            service_info.CleanExcluded();
+            service_info.SetDispatchedServer(TSvrRef());
             if (is_pooled == "true") {
-                rt_data.GetServerOptions(params.GetServerName()).clear();
+                service_info.ClearOptions();
             }
             t_con.reset(DispatchServerName(opening_ctx, params));
         } else if (single_server != "true"  &&  is_pooled == "true") {
@@ -401,7 +327,7 @@ CDBConnectionFactory::MakeDBConnection(
                                                 cur_params));
 
             } catch (CDB_Exception& ex) {
-                // m_Errors.push_back(ex.Clone());
+                // opening_ctx.errors.push_back(ex.Clone());
                 opening_ctx.handlers.PostMsg(&ex);
                 if (params.GetConnValidator()) {
                     opening_ctx.conn_status
@@ -416,8 +342,8 @@ CDBConnectionFactory::MakeDBConnection(
                     // Check conn_status ...
                     if (opening_ctx.conn_status
                         == IConnValidator::eTempInvalidConn) {
-                        rt_data.IncNumOfValidationFailures
-                            (params.GetServerName(), opening_ctx.last_tried);
+                        service_info.IncNumOfValidationFailures(
+                            opening_ctx.last_tried);
                     }
 
                     // Re-dispatch ...
@@ -426,20 +352,14 @@ CDBConnectionFactory::MakeDBConnection(
             } else {
                 // Dispatched server is already set, but calling of this method
                 // will increase number of successful dispatches.
-                rt_data.SetDispatchedServer(params.GetServerName(),
-                                            opening_ctx.last_tried);
+                service_info.SetDispatchedServer(opening_ctx.last_tried);
             }
         }
     }
 
-    // Restore original connection timeout ...
-    ctx.SetTimeout(query_timeout);
-
-    // Restore original query timeout ...
-    if (t_con.get() != NULL) {
-        t_con->SetTimeout(query_timeout);
+    if (t_con.get() == nullptr) {
+        service_info.ClearOptions();
     }
-
     x_LogConnection(opening_ctx, t_con.get(), params);
     handler->Flush((t_con.get() == NULL) ? eDiagSevMax : eDiag_Warning);
 
@@ -460,20 +380,25 @@ CDBConnectionFactory::DispatchServerName(
     Uint4 cur_host = 0;
     Uint2  cur_port = 0;
 
-    CRuntimeData& rt_data = GetRuntimeData(params.GetConnValidator());
-
     // Try to connect up to a given number of alternative servers ...
-    unsigned int alternatives = GetMaxNumOfServerAlternatives();
+    unsigned int max_alternatives = GetMaxNumOfServerAlternatives();
     list<TSvrRef> tried_servers;
     bool full_retry_made = false;
     CRef<CDBPoolBalancer> balancer;
+    CServiceInfo& service_info = ctx.service_info;
+    unsigned int alternatives
+        = static_cast<unsigned int>(service_info.GetOptions().size());
+    if (alternatives < 1) {
+        alternatives = min(2U, max_alternatives);
+    } else {
+        alternatives = min(alternatives * 2U, max_alternatives);
+    }
 
-    if ( !do_not_dispatch  &&  params.GetParam("is_pooled") == "true"
-        &&  !service_name.empty()  ) {
+    if ( !do_not_dispatch  &&  !service_name.empty() ) {
         balancer.Reset(new CDBPoolBalancer
-                       (service_name, params.GetParam("pool_name"),
-                        rt_data.GetServerOptions(service_name),
-                        &ctx.driver_ctx));
+                       (service_info, params.GetParam("pool_name"),
+                        &ctx.driver_ctx,
+                        params.GetParam("is_pooled") == "true"));
     }
     for ( ; !t_con && alternatives > 0; --alternatives ) {
         TSvrRef dsp_srv;
@@ -488,13 +413,19 @@ CDBConnectionFactory::DispatchServerName(
         // In this case we even won't try to map it.
         else if (!service_name.empty()) {
             if (balancer.NotEmpty()) {
-                dsp_srv = balancer->GetServer(&t_con, &params);
+                ctx.excluded = ctx.service_info.GetExcluded();
+                try {
+                    dsp_srv = balancer->GetServer(&t_con, &params);
+                } catch (CDB_Exception& ex) {
+                    ctx.errors.push_back(ex.Clone());
+                    continue;
+                }
             }
             if (dsp_srv.Empty()) {
-                dsp_srv = rt_data.GetDispatchedServer(service_name);
+                dsp_srv = service_info.GetDispatchedServer();
             }
             if (dsp_srv.Empty()) {
-                dsp_srv = rt_data.GetDBServiceMapper().GetServer(service_name);
+                dsp_srv = service_info.GetMappedServer();
             }
 
             if (tried_servers.empty()
@@ -502,16 +433,20 @@ CDBConnectionFactory::DispatchServerName(
                      ||  (dsp_srv->GetName() == service_name
                           &&  dsp_srv->GetHost() == 0
                           &&  dsp_srv->GetPort() == 0
-                          &&  !rt_data.GetExcluded(service_name).empty() ))) {
+                          &&  service_info.HasExclusions() ))) {
                 _TRACE("List of servers for service " << service_name
                        << " is exhausted. Giving excluded a try.");
-                rt_data.CleanExcluded(service_name);
-                dsp_srv = rt_data.GetDBServiceMapper().GetServer(service_name);
+                service_info.CleanExcluded();
+                dsp_srv = service_info.GetMappedServer();
             }
 
             if (dsp_srv.Empty()) {
-                m_Errors.push_back(new CDB_Exception(DIAG_COMPILE_INFO, NULL, CDB_Exception::EErrCode(0),
-                                   "Service mapper didn't return any server for " + service_name, eDiag_Error, 0));
+                ctx.errors.push_back(
+                    new CDB_Exception(
+                        DIAG_COMPILE_INFO, NULL, CDB_Exception::EErrCode(0),
+                        "Service mapper didn't return any server for "
+                        + service_name,
+                        eDiag_Error, 0));
                 return NULL;
             }
             if (dsp_srv->GetName() == service_name
@@ -520,8 +455,13 @@ CDBConnectionFactory::DispatchServerName(
             {
                 if (full_retry_made) {
                     if (!m_TryServerToo) {
-                        m_Errors.push_back(new CDB_Exception(DIAG_COMPILE_INFO, NULL, CDB_Exception::EErrCode(0),
-                                           "No more servers to try (didn't try " + service_name + " as server name)", eDiag_Error, 0));
+                        ctx.errors.push_back(
+                            new CDB_Exception(
+                                DIAG_COMPILE_INFO, NULL,
+                                CDB_Exception::EErrCode(0),
+                                "No more servers to try (didn't try "
+                                + service_name + " as server name)",
+                                eDiag_Error, 0));
                         return NULL;
                     }
                 }
@@ -529,15 +469,15 @@ CDBConnectionFactory::DispatchServerName(
                     _TRACE("List of servers for service " << service_name
                            << " is exhausted. Giving excluded a try.");
 
-                    rt_data.CleanExcluded(service_name);
+                    service_info.CleanExcluded();
                     ITERATE(list<TSvrRef>, it, tried_servers) {
-                        rt_data.Exclude(service_name, *it);
+                        service_info.Exclude(*it);
                     }
                     if (balancer.NotEmpty()) {
+                        service_info.ClearOptions();
                         balancer.Reset
                             (new CDBPoolBalancer
-                             (service_name, params.GetParam("pool_name"),
-                              rt_data.GetServerOptions(service_name, true),
+                             (service_info, params.GetParam("pool_name"),
                               &ctx.driver_ctx));
                     }
                     full_retry_made = true;
@@ -545,20 +485,6 @@ CDBConnectionFactory::DispatchServerName(
                 }
             }
 
-            // If connection attempts will take too long mapper can return
-            // the same server once more. Let's not try to connect to it
-            // again.
-            bool found = false;
-            ITERATE(list<TSvrRef>, it, tried_servers) {
-                if (**it == *dsp_srv) {
-                    rt_data.Exclude(service_name, dsp_srv);
-                    found = true;
-                    break;
-                }
-            }
-            if (found)
-                continue;
-
             // curr_conn_attr.srv_name = dsp_srv->GetName();
             cur_srv_name = dsp_srv->GetName();
             cur_host = dsp_srv->GetHost();
@@ -597,7 +523,7 @@ CDBConnectionFactory::DispatchServerName(
                     break;
                 }
             } catch (CDB_Exception& ex) {
-                // m_Errors.push_back(ex.Clone());
+                // ctx.errors.push_back(ex.Clone());
                 ctx.handlers.PostMsg(&ex);
                 if (params.GetConnValidator()) {
                     ctx.conn_status
@@ -616,35 +542,43 @@ CDBConnectionFactory::DispatchServerName(
             if (cur_srv_name == service_name  &&  cur_host == 0  &&  cur_port == 0
                 &&  (ctx.conn_status != IConnValidator::eTempInvalidConn
                         ||  (GetMaxNumOfValidationAttempts()
-                             &&  rt_data.GetNumOfValidationFailures(service_name)
+                             &&  service_info.GetNumOfValidationFailures()
                                                >= GetMaxNumOfValidationAttempts())))
             {
-                m_Errors.push_back(new CDB_Exception(DIAG_COMPILE_INFO, NULL, CDB_Exception::EErrCode(0),
-                                   "No more services/servers to try", eDiag_Error, 0));
+                ctx.errors.push_back(
+                    new CDB_Exception(DIAG_COMPILE_INFO, NULL,
+                                      CDB_Exception::EErrCode(0),
+                                      "No more services/servers to try",
+                                      eDiag_Error, 0));
                 return NULL;
             }
 
             if (need_exclude) {
+                if (balancer.NotEmpty()) {
+                    balancer->LocallyPenalize(ctx.last_tried);
+                }
                 // Server might be temporarily unavailable ...
                 // Check conn_status ...
                 if (ctx.conn_status == IConnValidator::eTempInvalidConn) {
-                    rt_data.IncNumOfValidationFailures(service_name,
-                                                       ctx.last_tried);
+                    service_info.IncNumOfValidationFailures(ctx.last_tried);
                 } else {
                     // conn_status == IConnValidator::eInvalidConn
-                    rt_data.Exclude(service_name, ctx.last_tried);
+                    service_info.Exclude(ctx.last_tried);
                     tried_servers.push_back(ctx.last_tried);
                 }
             }
         } else {
-            rt_data.SetDispatchedServer(service_name, dsp_srv);
+            service_info.SetDispatchedServer(dsp_srv);
         }
     }
 
     if (!t_con) {
-        m_Errors.push_back(new CDB_Exception(DIAG_COMPILE_INFO, NULL, CDB_Exception::EErrCode(0),
-                           "Can't try any more alternatives (max number is "
-                           + NStr::UIntToString(GetMaxNumOfServerAlternatives()) + ")", eDiag_Error, 0));
+        ctx.errors.push_back(
+            new CDB_Exception(
+                DIAG_COMPILE_INFO, NULL, CDB_Exception::EErrCode(0),
+                "Can't try any more alternatives (max number is "
+                + NStr::UIntToString(GetMaxNumOfServerAlternatives()) + ")",
+                eDiag_Error, 0));
     }
     return t_con;
 }
@@ -669,18 +603,18 @@ CDBConnectionFactory::MakeValidConnection(
     unique_ptr<CDB_Connection> conn;
     try {
         if (candidate == NULL) {
+            ctx.excluded = ctx.service_info.GetExcluded();
+            CServiceInfo::TAntiGuard anti_guard(ctx.service_info);
             conn.reset(CtxMakeConnection(ctx.driver_ctx, params));
         } else {
             conn.reset(candidate);
         }
     } catch (...) {
-        CRef<IConnValidator> validator = params.GetConnValidator();
-        CRuntimeData&        rt_data   = GetRuntimeData(validator);
-        const string&        service   = params.GetServerName();
-        ctx.last_tried.Reset(rt_data.GetDispatchedServer(service));
+        ctx.last_tried.Reset(ctx.service_info.GetDispatchedServer());
         if (ctx.last_tried.Empty()) {
             ctx.last_tried.Reset
-                (new CDBServer(service, params.GetHost(), params.GetPort()));
+                (new CDBServer(params.GetServerName(), params.GetHost(),
+                               params.GetPort()));
         }
         throw;
     }
@@ -717,11 +651,12 @@ CDBConnectionFactory::MakeValidConnection(
 
         ctx.conn_status = IConnValidator::eInvalidConn;
         try {
+            CServiceInfo::TAntiGuard anti_guard(ctx.service_info);
             ctx.conn_status = validator.Validate(*conn);
             if (ctx.conn_status != IConnValidator::eValidConn) {
                 if (conn->IsReusable()) {
                     static_cast<impl::CConnection&>(conn->GetExtraFeatures())
-                        .m_Reusable = false;
+                        .x_DisableReuse();
                 }
                 CDB_Exception ex(DIAG_COMPILE_INFO, NULL,
                                  CDB_Exception::EErrCode(0),
@@ -729,7 +664,7 @@ CDBConnectionFactory::MakeValidConnection(
                                  + params.GetServerName(),
                                  eDiag_Error, 0);
                 ex.SetRetriable(eRetriable_No);
-                // m_Errors.push_back(ex.Clone());
+                // ctx.errors.push_back(ex.Clone());
                 ctx.handlers.PostMsg(&ex);
                 return NULL;
             }
@@ -741,9 +676,9 @@ CDBConnectionFactory::MakeValidConnection(
             if (ctx.conn_status != IConnValidator::eValidConn) {
                 if (conn->IsReusable()) {
                     static_cast<impl::CConnection&>(conn->GetExtraFeatures())
-                        .m_Reusable = false;
+                        .x_DisableReuse();
                 }
-                // m_Errors.push_back(ex.Clone());
+                // ctx.errors.push_back(ex.Clone());
                 ctx.handlers.PostMsg(&ex);
                 return NULL;
             }
@@ -762,34 +697,27 @@ CDBConnectionFactory::MakeValidConnection(
         conn->FinishOpening();
     }
     else {
-        CRef<IConnValidator> validator = params.GetConnValidator();
-        CRuntimeData&        rt_data   = GetRuntimeData(validator);
-        const string&        service   = params.GetServerName();
-        ctx.last_tried.Reset(rt_data.GetDispatchedServer(service));
+        ctx.last_tried.Reset(ctx.service_info.GetDispatchedServer());
         if (ctx.last_tried.Empty()) {
             ctx.last_tried.Reset
-                (new CDBServer(service, params.GetHost(), params.GetPort()));
+                (new CDBServer(params.GetServerName(), params.GetHost(),
+                               params.GetPort()));
         }
 
-        m_Errors.push_back(new CDB_Exception(DIAG_COMPILE_INFO, NULL, CDB_Exception::EErrCode(0),
-                           "Parameters prohibited creating connection", eDiag_Error, 0));
+        ctx.errors.push_back(
+            new CDB_Exception(DIAG_COMPILE_INFO, NULL,
+                              CDB_Exception::EErrCode(0),
+                              "Parameters prohibited creating connection",
+                              eDiag_Error, 0));
     }
     return conn.release();
 }
 
-CDB_UserHandler::TExceptions*
-CDBConnectionFactory::GetExceptions(void)
-{
-    return &m_Errors;
-}
-
 void
 CDBConnectionFactory::GetServersList(const string& validator_name,
                                      const string& service_name,
                                      list<string>* serv_list)
 {
-    CFastMutexGuard mg(m_Mtx);
-
     const IDBServiceMapper& mapper
                         = GetRuntimeData(validator_name).GetDBServiceMapper();
     mapper.GetServersList(service_name, serv_list);
@@ -800,11 +728,14 @@ CDBConnectionFactory::WorkWithSingleServer(const string& validator_name,
                                            const string& service_name,
                                            const string& server)
 {
-    CFastMutexGuard mg(m_Mtx);
-
     CRuntimeData& rt_data = GetRuntimeData(validator_name);
-    TSvrRef svr(new CDBServer(server, 0, 0, numeric_limits<unsigned int>::max()));
-    rt_data.SetDispatchedServer(service_name, svr);
+    TSvrRef svr;
+    if ( !server.empty() ) {
+        svr.Reset(new CDBServer(server, 0, 0, kMax_UInt));
+    }
+    auto& service_info = rt_data.GetServiceInfo(service_name);
+    IDBServiceInfo::TGuard guard(service_info);
+    service_info.SetDispatchedServer(svr);
 }
 
 // Make an effort to give each request its own connection numbering.
@@ -840,8 +771,8 @@ void CDBConnectionFactory::x_LogConnection(const SOpeningContext& ctx,
     CDBServer stub_dsp_srv;
 
     CRef<IConnValidator> validator = params.GetConnValidator();
-    CRuntimeData&        rt_data   = GetRuntimeData(validator);
     const string&        service   = params.GetServerName();
+    CServiceInfo&        svc_info  = ctx.service_info;
     TSvrRef              dsp_srv   = ctx.last_tried;
 
     CDiagContext_Extra   extra     = GetDiagContext().Extra();
@@ -927,8 +858,7 @@ void CDBConnectionFactory::x_LogConnection(const SOpeningContext& ctx,
         }
     }}
 
-    extra.Print(prefix + "name_mapper",
-                rt_data.GetDBServiceMapper().GetName());
+    extra.Print(prefix + "name_mapper", svc_info.GetMapperName());
 
     size_t retries = ctx.tried.size();
     if (ctx.conn_status != IConnValidator::eValidConn) {
@@ -938,9 +868,8 @@ void CDBConnectionFactory::x_LogConnection(const SOpeningContext& ctx,
     if ( !ctx.tried.empty() ) {
         extra.Print(prefix + "tried", NStr::Join(ctx.tried, " "));
     }
-    const string& excluded = rt_data.GetExcluded(service);
-    if ( !excluded.empty() ) {
-        extra.Print(prefix + "excluded", excluded);
+    if ( !ctx.excluded.empty() ) {
+        extra.Print(prefix + "excluded", ctx.excluded);
     }
     if (connection != NULL) {
         extra.Print(prefix + "conn_reuse_count", connection->GetReuseCount());
@@ -961,97 +890,68 @@ void CDBConnectionFactory::x_LogConnection(const SOpeningContext& ctx,
 
 
 ///////////////////////////////////////////////////////////////////////////////
-CDBConnectionFactory::CRuntimeData::CRuntimeData(
-    const CDBConnectionFactory& parent,
-    const CRef<IDBServiceMapper>& mapper
-    ) :
-m_Parent(&parent),
-m_DBServiceMapper(mapper)
+const IDBServiceMapper::TOptions&
+CDBConnectionFactory::CServiceInfo::GetOptions(void)
 {
-}
-
-IDBServiceMapper::TOptions&
-CDBConnectionFactory::CRuntimeData::GetServerOptions(const string& svc_name,
-                                                     bool force_refresh)
-{
-    auto& options = m_ServerOptionsMap[svc_name];
-    if (force_refresh  ||  options.empty()) {
-        m_DBServiceMapper->GetServerOptions(svc_name, &options);
+    if (m_Options.empty()) {
+        m_Mapper->GetServerOptions(m_ServiceName, &m_Options);
         // OK to leave empty if nothing turns up; service mappers can and
         // do take responsibility for temporarily remembering negative
         // results if checking from scratch is slow, and higher-level logic
         // on this end falls back on GetServer as needed.
+    } else {
+        auto now = CurrentTime().GetTimeT();
+        bool refresh = false;
+        for (const auto &it : m_Options) {
+            if (it->GetExpireTime() < now) {
+                refresh = true;
+                break;
+            }
+        }
+        if (refresh) {
+            IDBServiceMapper::TOptions options;
+            m_Mapper->GetServerOptions(m_ServiceName, &options);
+            if ( !options.empty() ) {
+                swap(options, m_Options);
+            }
+        }
     }
-    return options;
+    return m_Options;
 }
 
-TSvrRef
-CDBConnectionFactory::CRuntimeData::GetDispatchedServer(
-    const string& service_name
-    )
-{
-    return m_DispatchedSet[service_name];
-}
-
-void
-CDBConnectionFactory::CRuntimeData::SetDispatchedServer(
-    const string& service_name,
-    const TSvrRef& server
-    )
+void CDBConnectionFactory::CServiceInfo::SetDispatchedServer(
+    const TSvrRef& server)
 {
     if (server.Empty()) {
-        m_DispatchNumMap[service_name] = 0;
+        m_NumDispatches = 0;
     } else {
-        ++m_DispatchNumMap[service_name];
+        ++m_NumDispatches;
     }
 
-    m_DispatchedSet[service_name] = server;
+    m_Dispatched = server;
 }
 
-unsigned int
-CDBConnectionFactory::CRuntimeData::GetNumOfDispatches(
-    const string& service_name
-    )
+void CDBConnectionFactory::CServiceInfo::IncNumOfValidationFailures(
+    const TSvrRef& dsp_srv)
 {
-    return m_DispatchNumMap[service_name];
-}
+    ++m_NumValidationFailures;
+    auto limit = m_Factory.GetMaxNumOfValidationAttempts();
 
-unsigned int
-CDBConnectionFactory::CRuntimeData::GetNumOfValidationFailures(
-    const string& service_name
-    )
-{
-    return m_ValidationFailureMap[service_name];
-}
-
-void
-CDBConnectionFactory::CRuntimeData::IncNumOfValidationFailures(
-    const string& server_name,
-    const TSvrRef& dsp_srv
-    )
-{
-    ++m_ValidationFailureMap[server_name];
-
-    if (GetParent().GetMaxNumOfValidationAttempts() &&
-        GetNumOfValidationFailures(server_name) >=
-            GetParent().GetMaxNumOfValidationAttempts()) {
+    if (limit != 0  &&  m_NumValidationFailures >= limit) {
         // It is time to finish with this server ...
-        Exclude(server_name, dsp_srv);
+        Exclude(dsp_srv);
     }
 }
 
-string CDBConnectionFactory::CRuntimeData::GetExcluded
-(const string& service_name)
+string CDBConnectionFactory::CServiceInfo::GetExcluded(void)
 {
-    IDBServiceMapper& mapper = GetDBServiceMapper();
-    if ( !mapper.HasExclusions(service_name) ) {
+    if ( !m_Mapper->HasExclusions(m_ServiceName) ) {
         return kEmptyStr;
     }
 
-    IDBServiceMapper::TOptions options;
-    mapper.GetServerOptions(service_name, &options);
+    m_Mapper->GetServerOptions(m_ServiceName, &m_Options);
     string result, delim;
-    for (const auto& it : options) {
+    for (const auto &it : m_Options) {
         if (it->IsExcluded()) {
             string exclusion;
             if (it->GetHost() != 0) {
@@ -1071,6 +971,29 @@ string CDBConnectionFactory::CRuntimeData::GetExcluded
     return result;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+CDBConnectionFactory::CRuntimeData::CRuntimeData(
+    const CDBConnectionFactory& parent,
+    const CRef<IDBServiceMapper>& mapper
+    ) :
+m_Mutex(new CFastMutex),
+m_Parent(&parent),
+m_DBServiceMapper(mapper)
+{
+}
+
+CDBConnectionFactory::CServiceInfo&
+CDBConnectionFactory::CRuntimeData::GetServiceInfo(const string& service_name)
+{
+    CFastMutexGuard guard(*m_Mutex);
+    auto& entry = m_ServiceInfoMap[service_name];
+    if (entry.Empty()) {
+        entry.Reset(
+            new CServiceInfo(*m_Parent, *m_DBServiceMapper, service_name));
+    }
+    return *entry;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 CDBConnectionFactory::CMapperFactory::CMapperFactory(
     IDBServiceMapper::TFactory svc_mapper_factory,
@@ -1151,7 +1074,7 @@ CConnValidatorCoR::~CConnValidatorCoR(void)
 IConnValidator::EConnStatus
 CConnValidatorCoR::Validate(CDB_Connection& conn)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CFastReadGuard guard(m_Lock);
 
     NON_CONST_ITERATE(TValidators, vr_it, m_Validators) {
         EConnStatus status = (*vr_it)->Validate(conn);
@@ -1169,7 +1092,7 @@ CConnValidatorCoR::GetName(void) const
 {
     string result("CConnValidatorCoR");
 
-    CFastMutexGuard mg(m_Mtx);
+    CFastReadGuard guard(m_Lock);
 
     ITERATE(TValidators, vr_it, m_Validators) {
         result += (*vr_it)->GetName();
@@ -1182,7 +1105,7 @@ void
 CConnValidatorCoR::Push(const CRef<IConnValidator>& validator)
 {
     if (validator.NotNull()) {
-        CFastMutexGuard mg(m_Mtx);
+        CFastWriteGuard guard(m_Lock);
 
         m_Validators.push_back(validator);
     }
@@ -1191,7 +1114,7 @@ CConnValidatorCoR::Push(const CRef<IConnValidator>& validator)
 void
 CConnValidatorCoR::Pop(void)
 {
-    CFastMutexGuard mg(m_Mtx);
+    CFastWriteGuard guard(m_Lock);
 
     m_Validators.pop_back();
 }
@@ -1199,7 +1122,7 @@ CConnValidatorCoR::Pop(void)
 CRef<IConnValidator>
 CConnValidatorCoR::Top(void) const
 {
-    CFastMutexGuard mg(m_Mtx);
+    CFastReadGuard guard(m_Lock);
 
     return m_Validators.back();
 }
@@ -1207,7 +1130,7 @@ CConnValidatorCoR::Top(void) const
 bool
 CConnValidatorCoR::Empty(void) const
 {
-    CFastMutexGuard mg(m_Mtx);
+    CFastReadGuard guard(m_Lock);
 
     return m_Validators.empty();
 }
