@@ -1,4 +1,4 @@
-/*  $Id: dbapi_impl_context.cpp 610920 2020-06-25 13:37:30Z ivanov $
+/*  $Id: dbapi_impl_context.cpp 662123 2023-01-25 18:36:58Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,7 @@
 #include <dbapi/driver/impl/dbapi_impl_context.hpp>
 #include <dbapi/driver/impl/dbapi_impl_connection.hpp>
 #include <dbapi/driver/dbapi_driver_conn_params.hpp>
+#include <dbapi/driver/dbapi_conn_factory.hpp>
 #include <dbapi/error_codes.hpp>
 
 #include <corelib/resource_info.hpp>
@@ -98,7 +99,7 @@ CDriverContext::~CDriverContext(void)
 void
 CDriverContext::SetApplicationName(const string& app_name)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
 
     m_AppName = app_name;
 }
@@ -106,7 +107,7 @@ CDriverContext::SetApplicationName(const string& app_name)
 string
 CDriverContext::GetApplicationName(void) const
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CReadLockGuard guard(x_GetCtxLock());
 
     return m_AppName;
 }
@@ -114,7 +115,7 @@ CDriverContext::GetApplicationName(void) const
 void
 CDriverContext::SetHostName(const string& host_name)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
 
     m_HostName = host_name;
 }
@@ -122,21 +123,21 @@ CDriverContext::SetHostName(const string& host_name)
 string
 CDriverContext::GetHostName(void) const
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CReadLockGuard guard(x_GetCtxLock());
 
     return m_HostName;
 }
 
 unsigned int CDriverContext::GetLoginTimeout(void) const 
 { 
-    CMutexGuard mg(x_GetCtxMtx());
+    CReadLockGuard guard(x_GetCtxLock());
 
     return m_LoginTimeout; 
 }
 
 bool CDriverContext::SetLoginTimeout (unsigned int nof_secs)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
 
     m_LoginTimeout = nof_secs;
 
@@ -145,7 +146,7 @@ bool CDriverContext::SetLoginTimeout (unsigned int nof_secs)
 
 unsigned int CDriverContext::GetTimeout(void) const 
 { 
-    CMutexGuard mg(x_GetCtxMtx());
+    CReadLockGuard guard(x_GetCtxLock());
 
     return m_Timeout; 
 }
@@ -153,7 +154,7 @@ unsigned int CDriverContext::GetTimeout(void) const
 bool CDriverContext::SetTimeout(unsigned int nof_secs)
 {
     bool success = true;
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
 
     try {
         m_Timeout = nof_secs;
@@ -170,7 +171,7 @@ bool CDriverContext::SetTimeout(unsigned int nof_secs)
 
 unsigned int CDriverContext::GetCancelTimeout(void) const 
 { 
-    CMutexGuard mg(x_GetCtxMtx());
+    CReadLockGuard guard(x_GetCtxLock());
 
     return m_CancelTimeout;
 }
@@ -178,7 +179,7 @@ unsigned int CDriverContext::GetCancelTimeout(void) const
 bool CDriverContext::SetCancelTimeout(unsigned int nof_secs)
 {
     bool success = true;
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
 
     try {
         m_CancelTimeout = nof_secs;
@@ -191,7 +192,7 @@ bool CDriverContext::SetCancelTimeout(unsigned int nof_secs)
 
 bool CDriverContext::SetMaxBlobSize(size_t nof_bytes)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
 
     m_MaxBlobSize = nof_bytes;
 
@@ -203,26 +204,26 @@ bool CDriverContext::SetMaxBlobSize(size_t nof_bytes)
 void CDriverContext::PushCntxMsgHandler(CDB_UserHandler* h,
                                          EOwnership ownership)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
     m_CntxHandlers.Push(h, ownership);
 }
 
 void CDriverContext::PopCntxMsgHandler(CDB_UserHandler* h)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
     m_CntxHandlers.Pop(h);
 }
 
 void CDriverContext::PushDefConnMsgHandler(CDB_UserHandler* h,
                                             EOwnership ownership)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
     m_ConnHandlers.Push(h, ownership);
 }
 
 void CDriverContext::PopDefConnMsgHandler(CDB_UserHandler* h)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
     m_ConnHandlers.Pop(h);
 
     // Remove this handler from all connections
@@ -284,7 +285,7 @@ void CDriverContext::ResetEnvSybase(void)
 
 void CDriverContext::x_Recycle(CConnection* conn, bool conn_reusable)
 {
-    CMutexGuard mg(m_PoolMutex);
+    CWriteLockGuard guard(m_PoolLock);
 
     TConnPool::iterator it = find(m_InUse.begin(), m_InUse.end(), conn);
 
@@ -293,16 +294,19 @@ void CDriverContext::x_Recycle(CConnection* conn, bool conn_reusable)
     }
 
 #ifdef NCBI_THREADS
-    bool for_sem = false;
-    if ( !m_PoolSemSubject.empty()
-         &&  (m_PoolSemSubjectHasPoolName
-              ? conn->PoolName() == m_PoolSemSubject
-              : s_Matches(conn, kEmptyStr, m_PoolSemSubject))) {
-        for_sem = true;
-        m_PoolSemSubject.erase();
-        m_PoolSemConn = NULL;
-        m_PoolSem.TryWait(); // probably redundant, but ensures Post will work
-        m_PoolSem.Post();
+    CConnection** recipient = nullptr;
+    for (auto &consumer : m_PoolSemConsumers) {
+        if ( !consumer.selected
+            &&  (consumer.subject_is_pool
+                 ? conn->PoolName() == consumer.subject
+                 : s_Matches(conn, kEmptyStr, consumer.subject))) {
+            consumer.selected = true;
+            recipient = &consumer.conn;
+            // probably redundant, but ensures Post will work
+            m_PoolSem.TryWait();
+            m_PoolSem.Post();
+            break;
+        }
     }
 #endif
 
@@ -337,8 +341,8 @@ void CDriverContext::x_Recycle(CConnection* conn, bool conn_reusable)
         }
         m_NotInUse.push_back(conn);
 #ifdef NCBI_THREADS
-        if (for_sem) {
-            m_PoolSemConn = conn;
+        if (recipient != nullptr) {
+            *recipient = conn;
         }
 #endif
     } else {
@@ -353,7 +357,7 @@ void CDriverContext::x_Recycle(CConnection* conn, bool conn_reusable)
 void CDriverContext::x_AdjustCounts(const CConnection* conn, int delta)
 {
     if (delta != 0  &&  conn->IsReusable()) {
-        CMutexGuard mg(m_PoolMutex);
+        CWriteLockGuard guard(m_PoolLock);
         string server_name = conn->GetServerName();
         if (conn->Host() != 0  &&  server_name.find('@') == NPOS) {
             server_name += '@' + ConvertN2A(conn->Host());
@@ -375,7 +379,7 @@ void CDriverContext::x_AdjustCounts(const CConnection* conn, int delta)
 void CDriverContext::x_GetCounts(const TCountsMap& main_map,
                                  const string& name, TCounts* counts) const
 {
-    CMutexGuard mg(m_PoolMutex);
+    CReadLockGuard guard(m_PoolLock);
     auto it = main_map.find(name);
     if (it == main_map.end()) {
         counts->clear();
@@ -389,7 +393,7 @@ void CDriverContext::CloseUnusedConnections(const string&   srv_name,
                                             const string&   pool_name,
                                             unsigned int    max_closings)
 {
-    CMutexGuard mg(m_PoolMutex);
+    CWriteLockGuard guard(m_PoolLock);
 
     TConnPool::value_type con;
 
@@ -414,7 +418,7 @@ void CDriverContext::CloseUnusedConnections(const string&   srv_name,
 unsigned int CDriverContext::NofConnections(const TSvrRef& svr_ref,
                                             const string& pool_name) const
 {
-    CMutexGuard mg(m_PoolMutex);
+    CReadLockGuard guard(m_PoolLock);
 
     if ((!svr_ref  ||  !svr_ref->IsValid())  &&  pool_name.empty()) {
         return static_cast<unsigned int>(m_InUse.size() + m_NotInUse.size());
@@ -463,7 +467,7 @@ CDB_Connection* CDriverContext::MakeCDBConnection(CConnection* connection,
                                                   int delta)
 {
     connection->m_CleanupTime.Clear();
-    CMutexGuard mg(m_PoolMutex);
+    CWriteLockGuard guard(m_PoolLock);
     m_InUse.push_back(connection);
     x_AdjustCounts(connection, delta);
 
@@ -474,7 +478,7 @@ CDB_Connection*
 CDriverContext::MakePooledConnection(const CDBConnParams& params)
 {
     if (params.GetParam("is_pooled") == "true") {
-        CMutexGuard mg(m_PoolMutex);
+        CWriteLockGuard guard(m_PoolLock);
 
         string pool_name  (params.GetParam("pool_name")),
                server_name(params.GetServerName());
@@ -523,34 +527,65 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
                     if ( !timeout_str.empty()  &&  timeout_str != "default") {
                         timeout_val = NStr::StringToDouble(timeout_str);
                     }
-                    CTimeout timeout(timeout_val);
+                    CDeadline deadline{CTimeout(timeout_val)};
+                    CTempString subject;
                     if (pool_name.empty()) {
-                        m_PoolSemSubject = server_name;
-                        m_PoolSemSubjectHasPoolName = false;
+                        subject = server_name;
                     } else {
-                        m_PoolSemSubject = pool_name;
-                        m_PoolSemSubjectHasPoolName = true;
+                        subject = pool_name;
                     }
-                    mg.Release();
-                    if (m_PoolSem.TryWait(timeout)) {
-                        mg.Guard(m_PoolMutex);
+                    auto target = (m_PoolSemConsumers
+                                   .emplace(m_PoolSemConsumers.end(),
+                                            subject, !pool_name.empty()));
+                    guard.Release();
+                    bool timed_out = true;
+                    bool relocked = false;
+                    while (m_PoolSem.TryWait(deadline.GetRemainingTime())) {
+                        guard.Guard(m_PoolLock);
+                        relocked = true;
+                        if (target->selected) {
+                            timed_out = false;
+                            for (const auto &it : m_PoolSemConsumers) {
+                                if (&it != &*target  &&  it.selected) {
+                                    m_PoolSem.TryWait();
+                                    m_PoolSem.Post();
+                                    break;
+                                }
+                            }
+                        } else {
+                            m_PoolSem.TryWait();
+                            m_PoolSem.Post();
+                            relocked = false;
+                            guard.Release();
+                            continue;
+                        }
                         CConnection* t_con = NULL;
                         NON_CONST_REVERSE_ITERATE(TConnPool, it, m_NotInUse) {
-                            if (*it == m_PoolSemConn) {
-                                t_con = m_PoolSemConn;
-                                m_PoolSemConn = NULL;
+                            if (*it == target->conn) {
+                                t_con = target->conn;
                                 m_NotInUse.erase((++it).base());
                                 break;
                             }
                         }
                         if (t_con != NULL) {
+                            m_PoolSemConsumers.erase(target);
                             return MakeCDBConnection(t_con, 0);
                         }
-                    } else
+                        break;
+                    }
+                    if ( !relocked ) {
+                        guard.Guard(m_PoolLock);
+                    }                        
+                    m_PoolSemConsumers.erase(target);
+                    if (timed_out)
 #endif
                     if (params.GetParam("pool_allow_temp_overflow")
                         != "true") {
-                        return NULL;
+                        string msg = FORMAT("Connection pool full (size "
+                                            << pool_max << ") for "
+                                            << (pool_name.empty() ? server_name
+                                                : pool_name));
+                        DATABASE_DRIVER_ERROR(msg, 200011);
                     }
                 }
             }
@@ -586,7 +621,7 @@ CDriverContext::MakePooledConnection(const CDBConnParams& params)
 void
 CDriverContext::CloseAllConn(void)
 {
-    CMutexGuard mg(m_PoolMutex);
+    CWriteLockGuard guard(m_PoolLock);
     // close all connections first
     ITERATE(TConnPool, it, m_NotInUse) {
         x_AdjustCounts(*it, -1);
@@ -602,7 +637,7 @@ CDriverContext::CloseAllConn(void)
 void
 CDriverContext::DeleteAllConn(void)
 {
-    CMutexGuard mg(m_PoolMutex);
+    CWriteLockGuard guard(m_PoolLock);
     // close all connections first
     ITERATE(TConnPool, it, m_NotInUse) {
         x_AdjustCounts(*it, -1);
@@ -929,6 +964,12 @@ CDriverContext::ReadDBConfParams(const string&  service_name,
         params->pool_max_conn_use
             = reg.Get(section_name, "conn_pool_max_conn_use");
     }
+    if (reg.HasEntry(section_name, "log_minor_messages",
+                     IRegistry::fCountCleared)) {
+        params->flags += SDBConfParams::fLogMinorMsgsSet;
+        params->log_minor_messages
+            = reg.Get(section_name, "log_minor_messages");
+    }
     if (reg.HasEntry(section_name, "args", IRegistry::fCountCleared)) {
         params->flags += SDBConfParams::fArgsSet;
         params->args = reg.Get(section_name, "args");
@@ -938,7 +979,7 @@ CDriverContext::ReadDBConfParams(const string&  service_name,
 bool
 CDriverContext::SatisfyPoolMinimum(const CDBConnParams& params)
 {
-    CMutexGuard mg(m_PoolMutex);
+    CWriteLockGuard guard(m_PoolLock);
 
     string pool_min_str = params.GetParam("pool_minsize");
     if (pool_min_str.empty()  ||  pool_min_str == "default")
@@ -966,14 +1007,14 @@ CDriverContext::SatisfyPoolMinimum(const CDBConnParams& params)
             ++total_cnt;
         }
     }
-    mg.Release();
+    guard.Release();
     vector< AutoPtr<CDB_Connection> > conns(pool_min);
     for (int i = total_cnt; i < pool_min; ++i) {
         try {
             conns.push_back(MakeConnection(params));
         }
         catch (CDB_Exception& ex) {
-            LOG_POST_X(1, "Error filling connection pool: " << ex);
+            ERR_POST_X(1, "Error filling connection pool: " << ex);
             return false;
         }
     }
@@ -983,8 +1024,6 @@ CDriverContext::SatisfyPoolMinimum(const CDBConnParams& params)
 CDB_Connection* 
 CDriverContext::MakeConnection(const CDBConnParams& params)
 {
-    CMutexGuard mg(x_GetCtxMtx());
-
     CMakeConnActualParams act_params(params);
 
     SDBConfParams conf_params;
@@ -993,10 +1032,8 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
         ReadDBConfParams(params.GetServerName(), &conf_params);
     }
 
-    int was_timeout = GetTimeout();
-    int was_login_timeout = GetLoginTimeout();
     unique_ptr<CDB_Connection> t_con;
-    try {
+    {
         string server_name = (conf_params.IsServerSet()?   conf_params.server:
                                                            params.GetServerName());
         string user_name   = (conf_params.IsUsernameSet()? conf_params.username:
@@ -1005,39 +1042,60 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
                                                            params.GetDatabaseName());
         string password    = (conf_params.IsPasswordSet()? conf_params.password:
                                                            params.GetPassword());
-        if (conf_params.IsLoginTimeoutSet()) {
+        CRef<IDBConnectionFactory> factory
+            = CDbapiConnMgr::Instance().GetConnectionFactory();
+        auto cfactory
+            = dynamic_cast<const CDBConnectionFactory*>(
+                factory.GetPointerOrNull());
+        unsigned int timeout;
+        if (cfactory != nullptr) {
+            timeout = cfactory->GetLoginTimeout();
+        } else if (conf_params.IsLoginTimeoutSet()) {
             if (conf_params.login_timeout.empty()) {
-                SetLoginTimeout(0);
+                timeout = 0;
+            } else {
+                NStr::StringToNumeric(conf_params.login_timeout, &timeout);
             }
-            else {
-                SetLoginTimeout(NStr::StringToInt(conf_params.login_timeout));
-            }
-        }
-        else {
+        } else {
             string value(params.GetParam("login_timeout"));
             if (value == "default") {
-                SetLoginTimeout(0);
-            }
-            else if (!value.empty()) {
-                SetLoginTimeout(NStr::StringToInt(value));
+                timeout = 0;
+            } else if (!value.empty()) {
+                NStr::StringToNumeric(value, &timeout);
+            } else {
+                timeout = GetLoginTimeout();
             }
         }
+        if (timeout == 0  &&  cfactory != nullptr) {
+            timeout = 30;
+        }
+        act_params.SetParam("login_timeout", NStr::NumericToString(timeout));
+
         if (conf_params.IsIOTimeoutSet()) {
             if (conf_params.io_timeout.empty()) {
-                SetTimeout(0);
+                timeout = 0;
+            } else {
+                NStr::StringToNumeric(conf_params.io_timeout, &timeout);
             }
-            else {
-                SetTimeout(NStr::StringToInt(conf_params.io_timeout));
+        } else {
+            string value(params.GetParam("timeout"));
+            if (value == "default") {
+                timeout = 0;
+            } else if (!value.empty()) {
+                NStr::StringToNumeric(value, &timeout);
+            } else {
+                timeout = GetTimeout();
             }
         }
-        else {
-            string value(params.GetParam("io_timeout"));
-            if (value == "default") {
-                SetTimeout(0);
-            }
-            else if (!value.empty()) {
-                SetTimeout(NStr::StringToInt(value));
+        if (cfactory != nullptr) {
+            auto validation_timeout = cfactory->GetConnectionTimeout();
+            if (validation_timeout == 0) {
+                validation_timeout = timeout ? timeout : 30;
             }
+            act_params.SetParam("timeout",
+                                NStr::NumericToString(validation_timeout));
+        } else {
+            act_params.SetParam("timeout", NStr::NumericToString(timeout));
         }
         if (conf_params.IsCancelTimeoutSet()) {
             if (conf_params.cancel_timeout.empty()) {
@@ -1139,6 +1197,21 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
         else if (params.GetParam("continue_after_raiserror") == "default") {
             act_params.SetParam("continue_after_raiserror", "false");
         }
+        if (conf_params.IsLogMinorMessagesSet()) {
+            if (conf_params.log_minor_messages.empty()) {
+                act_params.SetParam("log_minor_messages", "false");
+            }
+            else {
+                act_params.SetParam
+                    ("log_minor_messages", 
+                     NStr::BoolToString(
+                         NStr::StringToBool(
+                             conf_params.log_minor_messages)));
+            }
+        }
+        else if (params.GetParam("log_minor_messages") == "default") {
+            act_params.SetParam("log_minor_messages", "false");
+        }
 
         s_TransformLoginData(server_name, user_name, db_name, password);
         act_params.SetServerName(server_name);
@@ -1146,8 +1219,8 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
         act_params.SetDatabaseName(db_name);
         act_params.SetPassword(password);
 
-        CRef<IDBConnectionFactory> factory = CDbapiConnMgr::Instance().GetConnectionFactory();
-        t_con.reset(factory->MakeDBConnection(*this, act_params));
+        CDB_UserHandler::TExceptions expts;
+        t_con.reset(factory->MakeDBConnection(*this, act_params, expts));
 
         if (t_con.get() == NULL) {
             if (act_params.GetParam("do_not_connect") == "true") {
@@ -1160,26 +1233,18 @@ CDriverContext::MakeConnection(const CDBConnParams& params)
 
             CDB_ClientEx ex(DIAG_COMPILE_INFO, NULL, err, eDiag_Error, 100011);
             ex.SetRetriable(eRetriable_No);
-            CDB_UserHandler::TExceptions* expts = factory->GetExceptions();
-            if (expts) {
-                NON_CONST_REVERSE_ITERATE(CDB_UserHandler::TExceptions, it, *expts) {
-                    ex.AddPrevious(*it);
-                }
+            NON_CONST_REVERSE_ITERATE(CDB_UserHandler::TExceptions, it, expts)
+            {
+                ex.AddPrevious(*it);
             }
             throw ex;
         }
 
+        t_con->SetTimeout(timeout);
         // Set database ...
         t_con->SetDatabaseName(act_params.GetDatabaseName());
 
     }
-    catch (exception&) {
-        SetTimeout(was_timeout);
-        SetLoginTimeout(was_login_timeout);
-        throw;
-    }
-    SetTimeout(was_timeout);
-    SetLoginTimeout(was_login_timeout);
 
     return t_con.release();
 }
@@ -1188,7 +1253,7 @@ size_t CDriverContext::CloseConnsForPool(const string& pool_name,
                                          Uint4 keep_host_ip, Uint2 keep_port)
 {
     size_t      invalidated_count = 0;
-    CMutexGuard mg(m_PoolMutex);
+    CWriteLockGuard guard(m_PoolLock);
 
     ITERATE(TConnPool, it, m_InUse) {
         CConnection* t_con(*it);
@@ -1216,7 +1281,7 @@ size_t CDriverContext::CloseConnsForPool(const string& pool_name,
 void CDriverContext::CloseOldIdleConns(unsigned int max_closings,
                                        const string& pool_name)
 {
-    CMutexGuard mg(m_PoolMutex);
+    CWriteLockGuard guard(m_PoolLock);
     if (max_closings == 0) {
         return;
     }
@@ -1269,7 +1334,7 @@ void CDriverContext::DestroyConnImpl(CConnection* impl)
 
 void CDriverContext::SetClientCharset(const string& charset)
 {
-    CMutexGuard mg(x_GetCtxMtx());
+    CWriteLockGuard guard(x_GetCtxLock());
 
     m_ClientCharset = charset;
     m_ClientEncoding = eEncoding_Unknown;
