@@ -1,4 +1,4 @@
-/*  $Id: dbapi_pool_balancer.cpp 626337 2021-02-25 18:42:32Z ivanov $
+/*  $Id: dbapi_pool_balancer.cpp 673284 2023-09-27 13:32:30Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -61,30 +61,71 @@ public:
             }
         }
 };
+
+
+class CDB_FakeConnection : public IBalanceable
+{
+public:
+    CDB_FakeConnection(TSvrRef server)
+        : m_Server(server)
+        { _ASSERT(server.NotEmpty()); }
+
+    const string& ServerName(void) const override
+        { return m_Server->GetName(); }
+    Uint4 Host(void) const override
+        { return m_Server->GetHost(); }
+    Uint2 Port(void) const override
+        { return m_Server->GetPort(); }
+
+private:
+    TSvrRef m_Server;
+};
     
 
-CDBPoolBalancer::CDBPoolBalancer(const string& service_name,
+CDBPoolBalancer::CDBPoolBalancer(IDBServiceInfo& service_info,
                                  const string& pool_name,
-                                 const IDBServiceMapper::TOptions& options,
-                                 I_DriverContext* driver_ctx)
-    : CPoolBalancer(service_name, options,
-                    driver_ctx != nullptr
-                    &&  !NStr::StartsWith(driver_ctx->GetDriverName(),
-                                          "ftds")),
-      m_DriverCtx(driver_ctx)
+                                 I_DriverContext* driver_ctx,
+                                 bool is_pooled)
+    : CPoolBalancer(service_info.GetServiceName(), service_info.GetOptions(),
+                    x_GetFlags(driver_ctx, is_pooled)),
+      m_ServiceInfo(&service_info), m_PoolName(pool_name),
+      m_DriverCtx(driver_ctx), m_IsPooled(is_pooled)
+{
+    x_ReinitFromCounts();
+}
+
+
+CPoolBalancer::TFlags CDBPoolBalancer::x_GetFlags(I_DriverContext* driver_ctx,
+                                                  bool is_pooled)
+{
+    TFlags flags = static_cast<EFlags>(0);
+    if (driver_ctx != nullptr
+        &&  !NStr::StartsWith(driver_ctx->GetDriverName(), "ftds")) {
+        flags |= fIgnoreRawIPs;
+    }
+    if ( !is_pooled ) {
+        flags |= fNoPooling;
+    }
+    return flags;
+}
+
+
+void CDBPoolBalancer::x_ReinitFromCounts(void)
 {
     const impl::CDriverContext* ctx_impl
-        = dynamic_cast<const impl::CDriverContext*>(driver_ctx);
+        = dynamic_cast<const impl::CDriverContext*>(m_DriverCtx);
     impl::CDriverContext::TCounts counts;
     if (ctx_impl == NULL) {
-        if (driver_ctx != nullptr) {
+        if (m_DriverCtx != nullptr) {
             ERR_POST_X(1, Warning <<
                        "Called with non-standard IDriverContext");
         }
-    } else if (pool_name.empty()) {
-        ctx_impl->GetCountsForService(service_name, &counts);
+    } else if (x_NoPooling()) {
+    } else if (m_PoolName.empty()) {
+        ctx_impl->GetCountsForService(m_ServiceInfo->GetServiceName(),
+                                      &counts);
     } else {
-        ctx_impl->GetCountsForPool(pool_name, &counts);
+        ctx_impl->GetCountsForPool(m_PoolName, &counts);
     }
     x_InitFromCounts(counts);
 }
@@ -96,10 +137,25 @@ IBalanceable* CDBPoolBalancer::x_TryPool(const void* params_in)
     _ASSERT(params != nullptr);
     if (m_DriverCtx == nullptr) {
         return nullptr;
+    } else if (x_NoPooling()) {
+        auto server = m_ServiceInfo->GetMappedServer();
+        if (server.Empty()) {
+            return nullptr;
+        } else {
+            return new CDB_FakeConnection(server);
+        }
     } else {
         CDBConnParams_DNC dnc_params(*params);
-        return IDBConnectionFactory::CtxMakeConnection(*m_DriverCtx,
-                                                       dnc_params);
+        IBalanceable * conn = nullptr;
+        {{
+            IDBServiceInfo::TAntiGuard anti_guard(*m_ServiceInfo);
+            conn = IDBConnectionFactory::CtxMakeConnection(*m_DriverCtx,
+                                                           dnc_params);
+        }}
+        if (conn == nullptr) {
+            x_ReinitFromCounts();
+        }
+        return conn;
     }
 }
 
@@ -132,11 +188,14 @@ void CDBPoolBalancer::x_Discard(const void* params_in, IBalanceable* conn_in)
 {
     auto params = static_cast<const CDBConnParams*>(params_in);
     _ASSERT(params != nullptr);
-    auto conn = static_cast<const CDB_Connection*>(conn_in);
-    _ASSERT(conn != nullptr);
+    auto conn = dynamic_cast<const CDB_Connection*>(conn_in);
+    if (conn == nullptr) {
+        _ASSERT(dynamic_cast<const CDB_FakeConnection*>(conn_in) != nullptr);
+        return;
+    }
     _TRACE("Proceeding to request turnover");
-    const string&  server_name   = conn->ServerName();
-    bool           was_reusable  = conn->IsReusable();
+    string  server_name   = conn->ServerName();
+    bool    was_reusable  = conn->IsReusable();
     delete conn;
     if (was_reusable) {
         // This call might not close the exact connection we
