@@ -1,4 +1,4 @@
-/*  $Id: snpptis.cpp 618512 2020-10-21 16:05:54Z grichenk $
+/*  $Id: snpptis.cpp 637979 2021-09-21 16:18:17Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -70,9 +70,22 @@ const float kDefault_TimeoutMul = 1.5;
 const float kDefault_TimeoutInc = 0;
 const float kDefault_TimeoutMax = 10;
 const float kDefault_WaitTime    = 0.5;
-const float kDefault_WaitTimeMul = 1.2;
-const float kDefault_WaitTimeInc = 0.2;
+const float kDefault_WaitTimeMul = 1.2f;
+const float kDefault_WaitTimeInc = 0.2f;
 const float kDefault_WaitTimeMax = 5;
+const int kDefault_Debug = 1; // errors only
+
+NCBI_PARAM_DECL(int, ID2SNP, PTIS_DEBUG);
+NCBI_PARAM_DEF(int, ID2SNP, PTIS_DEBUG, kDefault_Debug);
+
+static int GetDebugLevel()
+{
+#ifdef HAVE_LIBGRPC
+    return NCBI_PARAM_TYPE(ID2SNP, PTIS_DEBUG)::GetDefault();
+#else
+    return 0;
+#endif
+}
 #endif
 
 
@@ -139,14 +152,14 @@ CSnpPtisClient_Impl::CSnpPtisClient_Impl()
     //LOG_POST(Trace<<"CSnpPtisClient: connecting to "<<address);
     channel = grpc::CreateCustomChannel(address, grpc::InsecureChannelCredentials(), args);
     max_retries = g_GetConfigInt(kSection, kParam_Retry, nullptr, kDefault_Retry);
-    timeout     = g_GetConfigDouble(kSection, kParam_Timeout   , nullptr, kDefault_Timeout   );
-    timeout_mul = g_GetConfigDouble(kSection, kParam_TimeoutMul, nullptr, kDefault_TimeoutMul);
-    timeout_inc = g_GetConfigDouble(kSection, kParam_TimeoutInc, nullptr, kDefault_TimeoutInc);
-    timeout_max = g_GetConfigDouble(kSection, kParam_TimeoutMax, nullptr, kDefault_TimeoutMax);
-    wait_time     = g_GetConfigDouble(kSection, kParam_WaitTime   , nullptr, kDefault_WaitTime   );
-    wait_time_mul = g_GetConfigDouble(kSection, kParam_WaitTimeMul, nullptr, kDefault_WaitTimeMul);
-    wait_time_inc = g_GetConfigDouble(kSection, kParam_WaitTimeInc, nullptr, kDefault_WaitTimeInc);
-    wait_time_max = g_GetConfigDouble(kSection, kParam_WaitTimeMax, nullptr, kDefault_WaitTimeMax);
+    timeout     = (float)g_GetConfigDouble(kSection, kParam_Timeout   , nullptr, kDefault_Timeout   );
+    timeout_mul = (float)g_GetConfigDouble(kSection, kParam_TimeoutMul, nullptr, kDefault_TimeoutMul);
+    timeout_inc = (float)g_GetConfigDouble(kSection, kParam_TimeoutInc, nullptr, kDefault_TimeoutInc);
+    timeout_max = (float)g_GetConfigDouble(kSection, kParam_TimeoutMax, nullptr, kDefault_TimeoutMax);
+    wait_time     = (float)g_GetConfigDouble(kSection, kParam_WaitTime   , nullptr, kDefault_WaitTime   );
+    wait_time_mul = (float)g_GetConfigDouble(kSection, kParam_WaitTimeMul, nullptr, kDefault_WaitTimeMul);
+    wait_time_inc = (float)g_GetConfigDouble(kSection, kParam_WaitTimeInc, nullptr, kDefault_WaitTimeInc);
+    wait_time_max = (float)g_GetConfigDouble(kSection, kParam_WaitTimeMax, nullptr, kDefault_WaitTimeMax);
     
     stub = ncbi::grpcapi::dbsnp::primary_track::DbSnpPrimaryTrack::NewStub(channel);
 }
@@ -160,6 +173,9 @@ CSnpPtisClient_Impl::~CSnpPtisClient_Impl()
 string CSnpPtisClient_Impl::GetPrimarySnpTrackForGi(TGi gi)
 {
     ncbi::grpcapi::dbsnp::primary_track::SeqIdRequestStringAccverUnion request;
+    if ( GetDebugLevel() >= 5 ) {
+        LOG_POST(Info << "CSnpPtisClient: asking for gi "<<gi);
+    }
     request.set_gi(GI_TO(TIntId, gi));
     return x_GetPrimarySnpTrack(request);
 }
@@ -167,6 +183,9 @@ string CSnpPtisClient_Impl::GetPrimarySnpTrackForGi(TGi gi)
 string CSnpPtisClient_Impl::GetPrimarySnpTrackForAccVer(const string& acc_ver)
 {
     ncbi::grpcapi::dbsnp::primary_track::SeqIdRequestStringAccverUnion request;
+    if ( GetDebugLevel() >= 5 ) {
+        LOG_POST(Info << "CSnpPtisClient: asking for acc_ver "<<acc_ver);
+    }
     request.set_accver(acc_ver);
     return x_GetPrimarySnpTrack(request);
 }
@@ -188,19 +207,28 @@ string CSnpPtisClient_Impl::x_GetPrimarySnpTrack(const TRequest& request)
         auto status = stub->ForSeqId(&context, request, &reply);
         
         if ( status.ok() ) {
+            if ( GetDebugLevel() >= 5 ) {
+                LOG_POST(Info << "CSnpPtisClient: received "<<reply.na_track_acc_with_filter());
+            }
             return reply.na_track_acc_with_filter();
         }
         
         if ( status.error_code() == grpc::StatusCode::NOT_FOUND ) {
+            if ( GetDebugLevel() >= 5 ) {
+                LOG_POST(Info << "CSnpPtisClient: received NOT_FOUND");
+            }
             return string();
         }
         if ( ++cur_retry >= max_retries ) {
+            if ( GetDebugLevel() >= 5 ) {
+                LOG_POST(Info << "CSnpPtisClient: failed: "<<status.error_message());
+            }
             NCBI_THROW(CException, eUnknown, status.error_message());
         }
         ERR_POST(Trace<<
                  "CSnpPtisClient: failed : "<<status.error_message()<<". "
                  "Waiting "<<cur_wait_time<<" seconds before retry...");
-        SleepMicroSec(Int8(cur_wait_time*1e6));
+        SleepMicroSec((unsigned long)(cur_wait_time*1e6));
         cur_timeout = min(cur_timeout*timeout_mul + timeout_inc, timeout_max);
         cur_wait_time = min(cur_wait_time*wait_time_mul + wait_time_inc, wait_time_max);
     }
