@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 #define OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 
-/*  $Id: seq_id_tree.hpp 576166 2018-12-11 15:51:54Z grichenk $
+/*  $Id: seq_id_tree.hpp 662785 2023-02-06 21:49:17Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -39,6 +39,7 @@
 #include <corelib/ncbi_limits.hpp>
 
 #include <objects/general/Date.hpp>
+#include <objects/general/Date_std.hpp>
 #include <objects/general/Dbtag.hpp>
 #include <objects/general/Object_id.hpp>
 
@@ -302,7 +303,7 @@ class CSeq_id_Gi_Info : public CSeq_id_Info
 public:
     CSeq_id_Gi_Info(CSeq_id_Mapper* mapper);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant /*variant*/) const;
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant /*variant*/) const override;
 };
 
 
@@ -345,7 +346,7 @@ class CSeq_id_Textseq_PlainInfo : public CSeq_id_Info
 public:
     CSeq_id_Textseq_PlainInfo(const CConstRef<CSeq_id>& seq_id, CSeq_id_Mapper* mapper);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const override;
     
     TVariant ParseCaseVariant(const string& acc) const;
     TVariant ParseCaseVariant(const CTextseq_id& id) const;
@@ -465,8 +466,10 @@ public:
     static TPacked Pack(const TKey& key, const CTextseq_id& id);
     static TVariant ParseCaseVariant(const CSeq_id_Info* info, const string& acc);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const override;
     
+    int CompareOrdered(const CSeq_id_Info& other, const CSeq_id_Handle& h_this, const CSeq_id_Handle& h_other) const override;
+
 private:
     TKey m_Key;
 };
@@ -719,7 +722,7 @@ public:
     TVariant ParseCaseVariant(const string& str) const;
     TVariant ParseCaseVariant(const CObject_id& oid) const;
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const override;
     
 private:
     bool m_IsId;
@@ -786,8 +789,10 @@ public:
 
     static TPacked Pack(const TKey& key, const CDbtag& id);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const override;
     
+    int CompareOrdered(const CSeq_id_Info& other, const CSeq_id_Handle& h_this, const CSeq_id_Handle& h_other) const override;
+
 private:
     TKey m_Key;
 };
@@ -858,9 +863,13 @@ public:
     static TKey Parse(const CDbtag& id);
     static TPacked Pack(const TKey& key, const CDbtag& id);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const override;
     
+    int CompareOrdered(const CSeq_id_Info& other, const CSeq_id_Handle& h_this, const CSeq_id_Handle& h_other) const override;
+
 private:
+    string x_GetStr(TPacked param) const;
+
     TKey m_Key;
 };
 
@@ -869,7 +878,7 @@ class CSeq_id_General_PlainInfo : public CSeq_id_Info {
 public:
     CSeq_id_General_PlainInfo(const CDbtag& dbid, CSeq_id_Mapper* mapper);
     
-    virtual CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const;
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const override;
     
     TVariant ParseCaseVariant(const CDbtag& dbtag) const;
 };
@@ -1001,6 +1010,42 @@ private:
 // e_PDB tree
 
 
+class CSeq_id_PDB_Info : public CSeq_id_Info {
+public:
+    CSeq_id_PDB_Info(const CConstRef<CSeq_id>& seq_id, CSeq_id_Mapper* mapper);
+    
+    CConstRef<CSeq_id> GetPackedSeqId(TPacked packed, TVariant variant) const override;
+    
+    static pair<CConstRef<CSeq_id>, TVariant> Normalize(const CSeq_id& seq_id);
+
+private:
+    static TVariant x_NormalizeDate(const CDate_std& date_std);
+    
+    // variant contains bits describing if chain fields are set,
+    // and normalized date fields - year, month, day, hour, minute, and second
+
+    // base constants
+    static const int kSecondBits = 6; // 0-63, 63 means not set
+    static const int kMinuteBits = 6; // 0-63, 63 means not set
+    static const int kHourBits = 5; // 0-31, 31 means not set
+    static const int kDayBits = 5; // 1-31, 0 means not set
+    static const int kMonthBits = 4; // 1-12, 0 means not set
+    static const int kYearBits = 12; // 1-4095
+    static const int kMolLowerCaseBits = 3; // up to 3 'mol' letters
+
+    // derived constants
+    static const int kNoChainOffset = 0;
+    static const int kNoChain_idOffset = kNoChainOffset + 1;
+    static const int kSecondOffset = kNoChain_idOffset + 1;
+    static const int kMinuteOffset = kSecondOffset + kSecondBits;
+    static const int kHourOffset = kMinuteOffset + kMinuteBits;
+    static const int kDayOffset = kHourOffset + kHourBits;
+    static const int kMonthOffset = kDayOffset + kDayBits;
+    static const int kYearOffset = kMonthOffset + kMonthBits;
+    static const int kMolLowerCaseOffset = kYearOffset + kYearBits;
+};
+
+
 class CSeq_id_PDB_Tree : public CSeq_id_Which_Tree
 {
 public:
@@ -1032,7 +1077,7 @@ private:
 
     // Index by mol+chain, no date - too complicated
     typedef vector<CSeq_id_Info*>  TSubMolList;
-    typedef map<string, TSubMolList, PNocase> TMolMap;
+    typedef map<string, TSubMolList, PCase> TMolMap;
 
     TMolMap m_MolMap;
 };
