@@ -1,4 +1,4 @@
-/*  $Id: annot_selector.cpp 603744 2020-03-16 17:42:16Z vasilche $
+/*  $Id: annot_selector.cpp 666880 2023-05-03 16:12:32Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -50,8 +50,6 @@
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 
-static const size_t NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX_LEN = 2;
-
 ////////////////////////////////////////////////////////////////////
 //
 //  SAnnotSelector
@@ -84,7 +82,8 @@ SAnnotSelector::SAnnotSelector(TAnnotType annot,
       m_HasWildcardInAnnotsNames(false),
       m_FilterMask(0),
       m_FilterBits(0),
-      m_ExcludeIfGeneIsSuppressed(false)
+      m_ExcludeIfGeneIsSuppressed(false),
+      m_SNPScaleLimit(CSeq_id::eSNPScaleLimit_Default)
 {
     if ( feat != CSeqFeatData::e_not_set ) {
         SetFeatType(feat);
@@ -118,7 +117,8 @@ SAnnotSelector::SAnnotSelector(TFeatType feat,
       m_HasWildcardInAnnotsNames(false),
       m_FilterMask(0),
       m_FilterBits(0),
-      m_ExcludeIfGeneIsSuppressed(false)
+      m_ExcludeIfGeneIsSuppressed(false),
+      m_SNPScaleLimit(CSeq_id::eSNPScaleLimit_Default)
 {
 }
 
@@ -148,7 +148,8 @@ SAnnotSelector::SAnnotSelector(TFeatSubtype feat_subtype)
       m_HasWildcardInAnnotsNames(false),
       m_FilterMask(0),
       m_FilterBits(0),
-      m_ExcludeIfGeneIsSuppressed(false)
+      m_ExcludeIfGeneIsSuppressed(false),
+      m_SNPScaleLimit(CSeq_id::eSNPScaleLimit_Default)
 {
 }
 
@@ -206,6 +207,7 @@ SAnnotSelector& SAnnotSelector::operator=(const SAnnotSelector& sel)
         }
         m_IgnoreFarLocationsForSorting = sel.m_IgnoreFarLocationsForSorting;
         m_ExcludeIfGeneIsSuppressed = sel.m_ExcludeIfGeneIsSuppressed;
+        m_SNPScaleLimit = sel.m_SNPScaleLimit;
     }
     return *this;
 }
@@ -963,85 +965,28 @@ IFeatComparator::~IFeatComparator()
 bool ExtractZoomLevel(const string& full_name,
                       string* acc_ptr, int* zoom_level_ptr)
 {
-    SIZE_TYPE pos = full_name.find(NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX);
-    if ( pos != NPOS ) {
-        if ( acc_ptr ) {
-            *acc_ptr = full_name.substr(0, pos);
-        }
-        SIZE_TYPE num_pos = pos + NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX_LEN;
-        // assuming single asterisk "*" as wildcard for all zoom levels
-        if ( num_pos+1 == full_name.size() && full_name[num_pos] == '*' ) {
-            if ( zoom_level_ptr ) {
-                *zoom_level_ptr = -1;
-            }
-            return true;
-        }
-        else {
-            try {
-                int zoom_level = NStr::StringToInt(full_name.substr(num_pos));
-                if ( zoom_level_ptr ) {
-                    *zoom_level_ptr = zoom_level;
-                }
-                return true;
-            }
-            catch ( CException& ) {
-                // invalid zoom level suffix, assume no zoom level
-            }
-        }
-    }
-    // no explicit zoom level
-    if ( acc_ptr ) {
-        *acc_ptr = full_name;
-    }
-    if ( zoom_level_ptr ) {
-        *zoom_level_ptr = 0;
-    }
-    return false;
+    return CSeq_annot::ExtractZoomLevel(full_name, acc_ptr, zoom_level_ptr);
 }
 
 
 string CombineWithZoomLevel(const string& acc, int zoom_level)
 {
-    int incl_level;
-    if ( !ExtractZoomLevel(acc, 0, &incl_level) ) {
-        if ( zoom_level == -1 ) {
-            // wildcard
-            return acc + NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX "*";
-        }
-        else {
-            return acc +
-                NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX +
-                NStr::IntToString(zoom_level);
-        }
+    try {
+        return CSeq_annot::CombineWithZoomLevel(acc, zoom_level);
     }
-    else if ( incl_level != zoom_level ) {
-        // different zoom level
-        NCBI_THROW_FMT(CAnnotException, eOtherError,
-                       "AddZoomLevel: Incompatible zoom levels: "
-                       <<acc<<" vs "<<zoom_level);
+    catch (CSeqAnnotException& ex) {
+        NCBI_THROW_FMT(CAnnotException, eOtherError, ex.GetMsg());
     }
-    return acc;
 }
 
 
 void AddZoomLevel(string& acc, int zoom_level)
 {
-    int incl_level;
-    if ( !ExtractZoomLevel(acc, 0, &incl_level) ) {
-        if ( zoom_level == -1 ) {
-            // wildcard
-            acc += NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX "*";
-        }
-        else {
-            acc += NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX;
-            acc += NStr::IntToString(zoom_level);
-        }
+    try {
+        CSeq_annot::AddZoomLevel(acc, zoom_level);
     }
-    else if ( incl_level != zoom_level ) {
-        // different zoom level
-        NCBI_THROW_FMT(CAnnotException, eOtherError,
-                       "AddZoomLevel: Incompatible zoom levels: "
-                       <<acc<<" vs "<<zoom_level);
+    catch (CSeqAnnotException& ex) {
+        NCBI_THROW_FMT(CAnnotException, eOtherError, ex.GetMsg());
     }
 }
 
