@@ -1,4 +1,4 @@
-/*  $Id: data_loader.cpp 610968 2020-06-26 12:55:17Z grichenk $
+/*  $Id: data_loader.cpp 659592 2022-11-30 18:45:10Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -716,6 +716,32 @@ void CDataLoader::GetSequenceHashes(const TIds& ids, TLoaded& loaded,
 }
 
 
+void CDataLoader::GetCDDAnnots(const TSeqIdSets& id_sets, TLoaded& loaded, TCDD_Locks& ret)
+{
+    SAnnotSelector sel;
+    sel.AddNamedAnnots("CDD")
+        .IncludeFeatSubtype(CSeqFeatData::eSubtype_region)
+        .IncludeFeatSubtype(CSeqFeatData::eSubtype_site);
+    for (size_t i = 0; i < id_sets.size(); ++i) {
+        for (auto id : id_sets[i]) {
+            TTSE_LockSet locks = GetRecordsNoBlobState(id, eBioseqCore);
+            CConstRef<CBioseq_Info> bs_info;
+            ITERATE(TTSE_LockSet, it, locks) {
+                bs_info = (*it)->FindMatchingBioseq(id);
+                if (!bs_info) continue;
+                TProcessedNAs nas;
+                const auto& id_locks = GetExternalAnnotRecordsNA(*bs_info, &sel, &nas);
+                if (id_locks.empty()) continue;
+                ret[i] = *id_locks.begin();
+                loaded[i] = true;
+                break;
+            }
+            if (loaded[i]) break;
+        }
+    }
+}
+
+
 void CDataLoader::GetBlobs(TTSE_LockSets& tse_sets)
 {
     NON_CONST_ITERATE(TTSE_LockSets, tse_set, tse_sets) {
@@ -941,6 +967,11 @@ unsigned CDataLoader::GetDefaultBlobCacheSizeLimit(void) const
     return kMax_UInt;
 }
 
+bool CDataLoader::GetTrackSplitSeq() const
+{
+    return false;
+}
+
 
 /////////////////////////////////////////////////////////////////////////////
 // CBlobId
