@@ -1,4 +1,4 @@
-/*  $Id: data_source.cpp 610968 2020-06-26 12:55:17Z grichenk $
+/*  $Id: data_source.cpp 669340 2023-06-27 13:19:52Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -69,6 +69,7 @@
 #include <corelib/ncbimtx.hpp>
 #include <corelib/ncbi_param.hpp>
 #include <objmgr/scope.hpp>
+#include <common/ncbi_sanitizers.h>
 
 #include <algorithm>
 
@@ -100,11 +101,23 @@ unsigned CDataSource::GetDefaultBlobCacheSizeLimit(void)
 }
 
 
+NCBI_PARAM_DECL(bool, OBJMGR, BULK_CHUNKS);
+NCBI_PARAM_DEF_EX(bool, OBJMGR, BULK_CHUNKS, true,
+                  eParam_NoThread, OBJMGR_BULK_CHUNKS);
+
+static bool s_GetBulkChunks(void)
+{
+    static bool value = NCBI_PARAM_TYPE(OBJMGR, BULK_CHUNKS)::GetDefault();
+    return value;
+}
+
+
 CDataSource::CDataSource(void)
     : m_DefaultPriority(CObjectManager::kPriority_Entry),
       m_Blob_Cache_Size(0),
       m_Blob_Cache_Size_Limit(GetDefaultBlobCacheSizeLimit()),
-      m_StaticBlobCounter(0)
+      m_StaticBlobCounter(0),
+      m_TrackSplitSeq(false)
 {
 }
 
@@ -115,7 +128,8 @@ CDataSource::CDataSource(CDataLoader& loader)
       m_Blob_Cache_Size(0),
       m_Blob_Cache_Size_Limit(min(GetDefaultBlobCacheSizeLimit(),
                                   loader.GetDefaultBlobCacheSizeLimit())),
-      m_StaticBlobCounter(0)
+      m_StaticBlobCounter(0),
+      m_TrackSplitSeq(loader.GetTrackSplitSeq())
 {
     m_Loader->SetTargetDataSource(*this);
 }
@@ -126,7 +140,8 @@ CDataSource::CDataSource(const CObject& shared_object, const CSeq_entry& entry)
       m_DefaultPriority(CObjectManager::kPriority_Entry),
       m_Blob_Cache_Size(0),
       m_Blob_Cache_Size_Limit(GetDefaultBlobCacheSizeLimit()),
-      m_StaticBlobCounter(0)
+      m_StaticBlobCounter(0),
+      m_TrackSplitSeq(false)
 {
     CTSE_Lock tse_lock = AddTSE(const_cast<CSeq_entry&>(entry));
     m_StaticBlobs.PutLock(tse_lock);
@@ -162,8 +177,12 @@ void CDataSource::DropAllTSEs(void)
     // First clear all indices
     m_InfoMap.clear();
     
-    m_TSE_seq.clear();
-
+    {{
+        TSeqLock::TWriteLockGuard guard2(m_DSSeqLock);
+        m_TSE_seq.clear();
+        m_TSE_split_seq.clear();
+    }}
+    
     {{
         TAnnotLock::TWriteLockGuard guard2(m_DSAnnotLock);
         m_TSE_seq_annot.clear();
@@ -415,13 +434,163 @@ CDataSource::FindTSE_Lock(const CSeq_entry& tse,
 }
 
 
+static ostream& s_Format(ostream& out, const CSeq_id_Handle& seq_id)
+{
+    return out << seq_id;
+}
+
+
+static ostream& s_Format(ostream& out, const string& str)
+{
+    return out << str;
+}
+
+
+static ostream& s_Format(ostream& out, const pair<const string, int>& na_with_zoom)
+{
+    out << na_with_zoom.first;
+    if ( na_with_zoom.second > 0 ) {
+        out << NCBI_ANNOT_TRACK_ZOOM_LEVEL_SUFFIX << na_with_zoom.second;
+    }
+    return out;
+}
+
+
+template<class Value>
+static ostream& s_Format(ostream& out, const pair<const CSeq_id_Handle, Value>& request_pair)
+{
+    return out << request_pair.first;
+}
+
+
+template<class Container>
+static ostream& s_Format(ostream& out, const Container& container)
+{
+    char separator = '(';
+    for ( auto& element : container ) {
+        out << separator; separator = ',';
+        s_Format(out, element);
+    }
+    return out << ')';
+}
+
+
+static string s_FormatCall(const char* method_name,
+                           const CSeq_id_Handle& seq_id)
+{
+    ostringstream out;
+    out << method_name << '(' << seq_id << ')';
+    return out.str();
+}
+
+
+static string s_FormatCall(const char* method_name,
+                           const CBioseq_Handle::TId& ids)
+{
+    ostringstream out;
+    out << method_name << '(';
+    s_Format(out, ids);
+    out << ')';
+    return out.str();
+}
+
+
+static string s_FormatCall(const char* method_name,
+                           const CDataSource::TSeqIdSets& id_sets)
+{
+    ostringstream out;
+    out << method_name << '(';
+    s_Format(out, id_sets);
+    out << ')';
+    return out.str();
+}
+
+
+static string s_FormatCall(const char* method_name,
+                           const CDataLoader::TTSE_LockSets& tse_set)
+{
+    ostringstream out;
+    out << method_name << '(';
+    s_Format(out, tse_set);
+    out << ')';
+    return out.str();
+}
+
+
+static string s_FormatCall(const char* method_name,
+                           const CSeq_id_Handle& seq_id,
+                           CDataLoader::EChoice choice)
+{
+    ostringstream out;
+    out << method_name << '(' << seq_id << ')';
+    return out.str();
+}
+
+
+static string s_FormatCall(const char* method_name,
+                           const CDataLoader::TSeq_idSet& ids,
+                           const SAnnotSelector* sel,
+                           CDataLoader::TProcessedNAs* processed_nas)
+{
+    ostringstream out;
+    out << method_name << '(';
+    s_Format(out, ids);
+    if ( sel && sel->IsIncludedAnyNamedAnnotAccession() ) {
+        out << ',';
+        s_Format(out, sel->GetNamedAnnotAccessions());
+        if ( processed_nas && !processed_nas->empty() ) {
+            out << '-';
+            s_Format(out, *processed_nas);
+        }
+    }
+    out << ')';
+    return out.str();
+}
+
+
+static string s_FormatCall(const char* method_name,
+                           const CBioseq_Info& bioseq,
+                           const SAnnotSelector* sel,
+                           CDataLoader::TProcessedNAs* processed_nas)
+{
+    ostringstream out;
+    out << method_name << '(';
+    s_Format(out, bioseq.GetId());
+    if ( sel && sel->IsIncludedAnyNamedAnnotAccession() ) {
+        out << ',';
+        s_Format(out, sel->GetNamedAnnotAccessions());
+        if ( processed_nas && !processed_nas->empty() ) {
+            out << '-';
+            s_Format(out, *processed_nas);
+        }
+    }
+    out << ')';
+    return out.str();
+}
+
+
+static string s_FormatCall(const char* method_name,
+                           const CDataLoader::TBlobId& blob_id)
+{
+    ostringstream out;
+    out << method_name << '(' << blob_id << ')';
+    return out.str();
+}
+
+
 CDataSource::TSeq_entry_Lock
 CDataSource::GetSeq_entry_Lock(const CBlobIdKey& blob_id)
 {
     TSeq_entry_Lock ret;
     {{
+        try {
+            ret.first = m_Loader->GetBlobById(blob_id);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetBlobById", blob_id));
+            throw;
+        }
         TMainLock::TWriteLockGuard guard(m_DSMainLock);
-        ret.first = m_Loader->GetBlobById(blob_id);
         if ( ret.first ) {
             x_SetLock(ret.second, ConstRef(&ret.first->GetTSE_Info()));
         }
@@ -520,38 +689,52 @@ CDataSource::FindSeq_feat_Lock(const CSeq_id_Handle& loc_id,
 }
 
 
+void CDataSource::x_CollectBlob_ids(const CSeq_id_Handle& idh,
+                                    const TSeq_id2TSE_Set& index,
+                                    TLoadedBlob_ids_Set& blob_ids)
+{
+    TSeq_id2TSE_Set::const_iterator tse_set = index.find(idh);
+    if ( tse_set != index.end() ) {
+        for ( auto& tse : tse_set->second ) {
+            blob_ids.insert(tse->GetBlobId());
+        }
+    }
+}
+
+
+void CDataSource::x_CollectBlob_ids(const CSeq_id_Handle& idh,
+                                    const TSeq_id2SplitInfoSet& index,
+                                    TLoadedBlob_ids_Set& blob_ids)
+{
+    auto iter = index.find(idh);
+    if ( iter != index.end() ) {
+        for ( auto& split : iter->second ) {
+            blob_ids.insert(split->GetBlobId());
+        }
+    }
+}
+
+
 void CDataSource::x_GetLoadedBlob_ids(const CSeq_id_Handle& idh,
                                       TLoadedTypes types,
-                                      TLoadedBlob_ids_Set& ids) const
+                                      TLoadedBlob_ids_Set& blob_ids) const
 {
-    if ( types & fLoaded_bioseqs ) {
-        TMainLock::TReadLockGuard guard(m_DSMainLock);
-        TSeq_id2TSE_Set::const_iterator tse_set = m_TSE_seq.find(idh);
-        if (tse_set != m_TSE_seq.end()) {
-            ITERATE(TTSE_Set, tse, tse_set->second) {
-                ids.insert((*tse)->GetBlobId());
-            }
+    if ( types & (fLoaded_bioseqs|fSplit_bioseqs) ) {
+        TSeqLock::TReadLockGuard guard(m_DSSeqLock);
+        if ( types & fLoaded_bioseqs ) {
+            x_CollectBlob_ids(idh, m_TSE_seq, blob_ids);
+        }
+        if ( x_IsTrackingSplitSeq() && (types & fSplit_bioseqs) ) {
+            x_CollectBlob_ids(idh, m_TSE_split_seq, blob_ids);
         }
     }
     if ( types & fLoaded_annots ) {
         TAnnotLock::TReadLockGuard guard(m_DSAnnotLock);
         if ( types & fLoaded_bioseq_annots ) {
-            TSeq_id2TSE_Set::const_iterator tse_set =
-                m_TSE_seq_annot.find(idh);
-            if (tse_set != m_TSE_seq_annot.end()) {
-                ITERATE(TTSE_Set, tse, tse_set->second) {
-                    ids.insert((*tse)->GetBlobId());
-                }
-            }
+            x_CollectBlob_ids(idh, m_TSE_seq_annot, blob_ids);
         }
         if ( types & fLoaded_orphan_annots ) {
-            TSeq_id2TSE_Set::const_iterator tse_set =
-                m_TSE_orphan_annot.find(idh);
-            if (tse_set != m_TSE_orphan_annot.end()) {
-                ITERATE(TTSE_Set, tse, tse_set->second) {
-                    ids.insert((*tse)->GetBlobId());
-                }
-            }
+            x_CollectBlob_ids(idh, m_TSE_orphan_annot, blob_ids);
         }
     }
 }
@@ -808,7 +991,14 @@ CDataSource::x_GetRecords(const CSeq_id_Handle& idh,
 {
     TTSE_LockSet tse_set;
     if ( m_Loader ) {
-        CDataLoader::TTSE_LockSet tse_set2 = m_Loader->GetRecords(idh, choice);
+        CDataLoader::TTSE_LockSet tse_set2;
+        try {
+            tse_set2 = move(m_Loader->GetRecords(idh, choice));
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetRecords", idh, choice));
+            throw;
+        }
         ITERATE ( CDataLoader::TTSE_LockSet, it, tse_set2 ) {
             tse_set.AddLock(*it);
             (*it)->x_GetRecords(idh, choice == CDataLoader::eBioseqCore);
@@ -899,8 +1089,14 @@ void CDataSource::GetTSESetWithOrphanAnnots(const TSeq_idSet& ids,
         // with loader installed we look only in TSEs reported by loader.
 
         // collect set of TSEs with orphan annotations
-        CDataLoader::TTSE_LockSet tse_set =
-            m_Loader->GetOrphanAnnotRecordsNA(ids, sel, processed_nas);
+        CDataLoader::TTSE_LockSet tse_set;
+        try {
+            tse_set = move(m_Loader->GetOrphanAnnotRecordsNA(ids, sel, processed_nas));
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetOrphanAnnotRecordsNA(", ids, sel, processed_nas));
+            throw;
+        }
 
         ITERATE ( CDataLoader::TTSE_LockSet, tse_it, tse_set ) {
             x_AddTSEOrphanAnnots(ret, ids, *tse_it);
@@ -951,8 +1147,14 @@ void CDataSource::GetTSESetWithBioseqAnnots(const CBioseq_Info& bioseq,
         // and TSEs with external annotations reported by the loader.
 
         // external annotations
-        CDataLoader::TTSE_LockSet tse_set2 =
-            m_Loader->GetExternalAnnotRecordsNA(bioseq, sel, processed_nas);
+        CDataLoader::TTSE_LockSet tse_set2;
+        try {
+            tse_set2 = move(m_Loader->GetExternalAnnotRecordsNA(bioseq, sel, processed_nas));
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetExternalAnnotRecordsNA", bioseq, sel, processed_nas));
+            throw;
+        }
         ITERATE ( CDataLoader::TTSE_LockSet, tse_it, tse_set2 ) {
             x_AddTSEBioseqAnnots(ret, bioseq, *tse_it);
         }
@@ -1009,7 +1211,7 @@ void CDataSource::GetTSESetWithBioseqAnnots(const CBioseq_Info& bioseq,
 }
 
 
-void CDataSource::x_IndexTSE(TSeq_id2TSE_Set& tse_map,
+bool CDataSource::x_IndexTSE(TSeq_id2TSE_Set& tse_map,
                              const CSeq_id_Handle& id,
                              CTSE_Info* tse_info)
 {
@@ -1018,7 +1220,7 @@ void CDataSource::x_IndexTSE(TSeq_id2TSE_Set& tse_map,
         it = tse_map.insert(it, TSeq_id2TSE_Set::value_type(id, TTSE_Set()));
     }
     _ASSERT(it != tse_map.end() && it->first == id);
-    it->second.insert(Ref(tse_info));
+    return it->second.insert(Ref(tse_info)).second;
 }
 
 
@@ -1037,22 +1239,97 @@ void CDataSource::x_UnindexTSE(TSeq_id2TSE_Set& tse_map,
 }
 
 
+void CDataSource::x_IndexSplitInfo(TSeq_id2SplitInfoSet& split_map,
+                                   const CSeq_id_Handle& id,
+                                   CTSE_Split_Info* split_info)
+{
+    split_map[id].insert(Ref(split_info));
+}
+
+
+void CDataSource::x_UnindexSplitInfo(TSeq_id2SplitInfoSet& split_map,
+                                     const CSeq_id_Handle& id,
+                                     CTSE_Split_Info* split_info)
+{
+    auto it = split_map.find(id);
+    if ( it != split_map.end() ) {
+        it->second.erase(Ref(split_info));
+        if ( it->second.empty() ) {
+            split_map.erase(it);
+        }
+    }
+}
+
+
+void CDataSource::x_IndexSeqTSELocked(const CSeq_id_Handle& id,
+                                      CTSE_Info* tse_info)
+{
+    if ( x_IndexTSE(m_TSE_seq, id, tse_info) &&
+         x_IsTrackingSplitSeq() &&
+         tse_info->HasSplitInfo() ) {
+        x_UnindexSplitInfo(m_TSE_split_seq, id, &tse_info->GetSplitInfo());
+    }
+}
+
+
 void CDataSource::x_IndexSeqTSE(const CSeq_id_Handle& id,
                                 CTSE_Info* tse_info)
 {
-    // no need to lock as it's locked by callers
-    TMainLock::TWriteLockGuard guard(m_DSMainLock);
-    x_IndexTSE(m_TSE_seq, id, tse_info);
+    TSeqLock::TWriteLockGuard guard(m_DSSeqLock);
+    x_IndexSeqTSELocked(id, tse_info);
+}
+
+
+void CDataSource::x_IndexSplitInfo(const CSeq_id_Handle& id,
+                                   CTSE_Split_Info* split_info)
+{
+    if ( x_IsTrackingSplitSeq() ) {
+        TSeqLock::TWriteLockGuard guard(m_DSSeqLock);
+        x_IndexSplitInfo(m_TSE_split_seq, id, split_info);
+    }
+}
+
+
+void CDataSource::x_IndexSplitInfo(const vector<CSeq_id_Handle>& ids,
+                                   CTSE_Split_Info* split_info)
+{
+    if ( x_IsTrackingSplitSeq() ) {
+        TSeqLock::TWriteLockGuard guard(m_DSSeqLock);
+        for ( auto& id : ids ) {
+            x_IndexSplitInfo(m_TSE_split_seq, id, split_info);
+        }
+    }
+}
+
+
+void CDataSource::x_UnindexSplitInfo(const CSeq_id_Handle& id,
+                                     CTSE_Split_Info* split_info)
+{
+    if ( x_IsTrackingSplitSeq() ) {
+        TSeqLock::TWriteLockGuard guard(m_DSSeqLock);
+        x_UnindexSplitInfo(m_TSE_split_seq, id, split_info);
+    }
+}
+
+
+void CDataSource::x_UnindexSplitInfo(const vector<CSeq_id_Handle>& ids,
+                                     CTSE_Split_Info* split_info)
+{
+    if ( x_IsTrackingSplitSeq() ) {
+        TSeqLock::TWriteLockGuard guard(m_DSSeqLock);
+        for ( auto& id : ids ) {
+            x_UnindexSplitInfo(m_TSE_split_seq, id, split_info);
+        }
+    }
 }
 
 
 void CDataSource::x_IndexSeqTSE(const vector<CSeq_id_Handle>& ids,
                                 CTSE_Info* tse_info)
 {
-    // no need to lock as it's locked by callers
-    TMainLock::TWriteLockGuard guard(m_DSMainLock);
-    ITERATE ( vector<CSeq_id_Handle>, it, ids ) {
-        x_IndexTSE(m_TSE_seq, *it, tse_info);
+    TSeqLock::TWriteLockGuard guard(m_DSSeqLock);
+    for ( auto& id : ids ) {
+        x_IndexSeqTSELocked(id, tse_info);
     }
 }
 
@@ -1060,8 +1337,7 @@ void CDataSource::x_IndexSeqTSE(const vector<CSeq_id_Handle>& ids,
 void CDataSource::x_UnindexSeqTSE(const CSeq_id_Handle& id,
                                   CTSE_Info* tse_info)
 {
-    // no need to lock as it's locked by callers
-    TMainLock::TWriteLockGuard guard(m_DSMainLock);
+    TSeqLock::TWriteLockGuard guard(m_DSSeqLock);
     x_UnindexTSE(m_TSE_seq, id, tse_info);
 }
 
@@ -1119,7 +1395,7 @@ CDataSource::x_FindBestTSE(const CSeq_id_Handle& handle,
 {
     TTSE_LockSet all_tse;
     {{
-        TMainLock::TReadLockGuard guard(m_DSMainLock);
+        TSeqLock::TReadLockGuard guard(m_DSSeqLock);
 #ifdef DEBUG_MAPS
         debug::CReadGuard<TSeq_id2TSE_Set> g1(m_TSE_seq);
 #endif
@@ -1225,7 +1501,7 @@ CDataSource::TSeqMatches CDataSource::GetMatches(const CSeq_id_Handle& idh,
     TSeqMatches ret;
 
     if ( !history.empty() ) {
-        TMainLock::TReadLockGuard guard(m_DSMainLock);
+        TSeqLock::TReadLockGuard guard(m_DSSeqLock);
         TSeq_id2TSE_Set::const_iterator tse_set = m_TSE_seq.find(idh);
         if ( tse_set != m_TSE_seq.end() ) {
             ITERATE ( TTSE_Set, it, tse_set->second ) {
@@ -1253,7 +1529,13 @@ void CDataSource::GetIds(const CSeq_id_Handle& idh, TIds& ids)
     }
     // Bioseq not found - try to request ids from loader if any.
     if ( m_Loader ) {
-        m_Loader->GetIds(idh, ids);
+        try {
+            m_Loader->GetIds(idh, ids);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetIds", idh));
+            throw;
+        }
     }
 }
 
@@ -1267,7 +1549,13 @@ CDataSource::SAccVerFound CDataSource::GetAccVer(const CSeq_id_Handle& idh)
         ret.sequence_found = true;
     }
     else if ( m_Loader ) {
-        ret = m_Loader->GetAccVerFound(idh);
+        try {
+            ret = m_Loader->GetAccVerFound(idh);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetAccVer", idh));
+            throw;
+        }
     }
     return ret;
 }
@@ -1282,7 +1570,13 @@ CDataSource::SGiFound CDataSource::GetGi(const CSeq_id_Handle& idh)
         ret.sequence_found = true;
     }
     else if ( m_Loader ) {
-        ret = m_Loader->GetGiFound(idh);
+        try {
+            ret = m_Loader->GetGiFound(idh);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetGi", idh));
+            throw;
+        }
     }
     return ret;
 }
@@ -1303,7 +1597,13 @@ string CDataSource::GetLabel(const CSeq_id_Handle& idh)
         }
     }
     if ( m_Loader ) {
-        ret = m_Loader->GetLabel(idh);
+        try {
+            ret = m_Loader->GetLabel(idh);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetLabel", idh));
+            throw;
+        }
     }
     return ret;
 }
@@ -1317,7 +1617,13 @@ TTaxId CDataSource::GetTaxId(const CSeq_id_Handle& idh)
         ret = match.m_Bioseq->GetTaxId();
     }
     else if ( m_Loader ) {
-        ret = m_Loader->GetTaxId(idh);
+        try {
+            ret = m_Loader->GetTaxId(idh);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetTaxId", idh));
+            throw;
+        }
     }
     return ret;
 }
@@ -1331,7 +1637,13 @@ TSeqPos CDataSource::GetSequenceLength(const CSeq_id_Handle& idh)
         ret = match.m_Bioseq->GetBioseqLength();
     }
     else if ( m_Loader ) {
-        ret = m_Loader->GetSequenceLength(idh);
+        try {
+            ret = m_Loader->GetSequenceLength(idh);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetSequenceLength", idh));
+            throw;
+        }
     }
     return ret;
 }
@@ -1347,7 +1659,13 @@ CDataSource::GetSequenceType(const CSeq_id_Handle& idh)
         ret.sequence_found = true;
     }
     else if ( m_Loader ) {
-        ret = m_Loader->GetSequenceTypeFound(idh);
+        try {
+            ret = m_Loader->GetSequenceTypeFound(idh);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetSequenceType", idh));
+            throw;
+        }
     }
     return ret;
 }
@@ -1361,7 +1679,13 @@ int CDataSource::GetSequenceState(const CSeq_id_Handle& idh)
         ret = match.m_Bioseq->GetTSE_Info().GetBlobState();
     }
     else if ( m_Loader ) {
-        ret = m_Loader->GetSequenceState(idh);
+        try {
+            ret = m_Loader->GetSequenceState(idh);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetSequenceState", idh));
+            throw;
+        }
     }
     return ret;
 }
@@ -1386,7 +1710,13 @@ void CDataSource::GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret)
         }
     }
     if ( remaining && m_Loader ) {
-        m_Loader->GetAccVers(ids, loaded, ret);
+        try {
+            m_Loader->GetAccVers(ids, loaded, ret);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetAccVers", ids));
+            throw;
+        }
     }
 }
 
@@ -1410,7 +1740,13 @@ void CDataSource::GetGis(const TIds& ids, TLoaded& loaded, TGis& ret)
         }
     }
     if ( remaining && m_Loader ) {
-        m_Loader->GetGis(ids, loaded, ret);
+        try {
+            m_Loader->GetGis(ids, loaded, ret);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetGis", ids));
+            throw;
+        }
     }
 }
 
@@ -1434,7 +1770,13 @@ void CDataSource::GetLabels(const TIds& ids, TLoaded& loaded, TLabels& ret)
         }
     }
     if ( remaining && m_Loader ) {
-        m_Loader->GetLabels(ids, loaded, ret);
+        try {
+            m_Loader->GetLabels(ids, loaded, ret);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetLabels", ids));
+            throw;
+        }
     }
 }
 
@@ -1458,7 +1800,13 @@ void CDataSource::GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret)
         }
     }
     if ( remaining && m_Loader ) {
-        m_Loader->GetTaxIds(ids, loaded, ret);
+        try {
+            m_Loader->GetTaxIds(ids, loaded, ret);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetTaxIds", ids));
+            throw;
+        }
     }
 }
 
@@ -1483,7 +1831,13 @@ void CDataSource::GetSequenceLengths(const TIds& ids, TLoaded& loaded,
         }
     }
     if ( remaining && m_Loader ) {
-        m_Loader->GetSequenceLengths(ids, loaded, ret);
+        try {
+            m_Loader->GetSequenceLengths(ids, loaded, ret);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetSequenceLengths", ids));
+            throw;
+        }
     }
 }
 
@@ -1508,7 +1862,13 @@ void CDataSource::GetSequenceTypes(const TIds& ids, TLoaded& loaded,
         }
     }
     if ( remaining && m_Loader ) {
-        m_Loader->GetSequenceTypes(ids, loaded, ret);
+        try {
+            m_Loader->GetSequenceTypes(ids, loaded, ret);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetSequenceTypes", ids));
+            throw;
+        }
     }
 }
 
@@ -1533,7 +1893,13 @@ void CDataSource::GetSequenceStates(const TIds& ids, TLoaded& loaded,
         }
     }
     if ( remaining && m_Loader ) {
-        m_Loader->GetSequenceStates(ids, loaded, ret);
+        try {
+            m_Loader->GetSequenceStates(ids, loaded, ret);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetSequenceStates", ids));
+            throw;
+        }
     }
 }
 
@@ -1543,7 +1909,13 @@ CDataSource::GetSequenceHash(const CSeq_id_Handle& idh)
 {
     SHashFound ret;
     if ( m_Loader ) {
-        ret = m_Loader->GetSequenceHashFound(idh);
+        try {
+            ret = m_Loader->GetSequenceHashFound(idh);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetSequenceHash", idh));
+            throw;
+        }
     }
     return ret;
 }
@@ -1553,7 +1925,47 @@ void CDataSource::GetSequenceHashes(const TIds& ids, TLoaded& loaded,
                                     TSequenceHashes& ret, THashKnown& known)
 {
     if ( m_Loader ) {
-        m_Loader->GetSequenceHashes(ids, loaded, ret, known);
+        try {
+            m_Loader->GetSequenceHashes(ids, loaded, ret, known);
+        }
+        catch ( CLoaderException& exc ) {
+            exc.SetFailedCall(s_FormatCall("GetSequenceHashes", ids));
+            throw;
+        }
+    }
+}
+
+
+void CDataSource::GetCDDAnnots(const TSeqIdSets& id_sets, TLoaded& loaded, TCDD_Locks& ret)
+{
+    if (!m_Loader) return;
+    try {
+        m_Loader->GetCDDAnnots(id_sets, loaded, ret);
+    }
+    catch ( CLoaderException& exc ) {
+        exc.SetFailedCall(s_FormatCall("GetCDDAnnots", id_sets));
+        throw;
+    }
+}
+
+
+static void s_GetBlobs(CDataLoader* loader,
+                       CDataLoader::TTSE_LockSets& all_tse_sets,
+                       CDataLoader::TTSE_LockSets& current_tse_sets)
+{
+    try {
+        loader->GetBlobs(current_tse_sets);
+    }
+    catch ( CLoaderException& exc ) {
+        exc.SetFailedCall(s_FormatCall("GetBlobs", current_tse_sets));
+        throw;
+    }
+    if ( all_tse_sets.empty() ) {
+        swap(all_tse_sets, current_tse_sets);
+    }
+    else {
+        all_tse_sets.insert(current_tse_sets.begin(), current_tse_sets.end());
+        current_tse_sets.clear();
     }
 }
 
@@ -1565,13 +1977,40 @@ void CDataSource::GetBlobs(TSeqMatchMap& match_map)
     }
     if ( m_Loader ) {
         CDataLoader::TTSE_LockSets tse_sets;
+        CDataLoader::TTSE_LockSets current_tse_sets;
+        size_t current_tse_sets_size = 0;
+#ifdef NCBI_USE_TSAN
+        const size_t limit_blobs_request = 15;
+#else
+        const size_t limit_blobs_request = 200;
+#endif
         ITERATE(TSeqMatchMap, match, match_map) {
             _ASSERT( !match->second );
-            tse_sets.insert(tse_sets.end(),
+            current_tse_sets.insert(
                 CDataLoader::TTSE_LockSets::value_type(
                 match->first, CDataLoader::TTSE_LockSet()));
+            if ( ++current_tse_sets_size >= limit_blobs_request ) {
+                s_GetBlobs(m_Loader, tse_sets, current_tse_sets);
+                current_tse_sets_size = 0;
+            }
+        }
+        if ( !current_tse_sets.empty() ) {
+            s_GetBlobs(m_Loader, tse_sets, current_tse_sets);
+        }
+        if ( s_GetBulkChunks() ) {
+            // bulk chunk loading
+            vector<CConstRef<CTSE_Chunk_Info>> chunks;
+            for ( auto& tse_set : tse_sets ) {
+                for ( auto& tse_lock : tse_set.second ) {
+                    if ( tse_lock->HasSplitInfo() ) {
+                        tse_lock->GetSplitInfo().x_AddChunksForGetRecords(chunks, tse_set.first);
+                    }
+                }
+            }
+            if ( !chunks.empty() ) {
+                CTSE_Split_Info::x_LoadChunks(m_Loader, chunks);
+            }
         }
-        m_Loader->GetBlobs(tse_sets);
         ITERATE(CDataLoader::TTSE_LockSets, tse_set, tse_sets) {
             TTSE_LockSet locks;
             ITERATE(CDataLoader::TTSE_LockSet, it, tse_set->second) {
@@ -1796,12 +2235,17 @@ CTSE_LoadLock CDataSource::GetTSE_LoadLockIfLoaded(const TBlobId& blob_id)
 
 
 CTSE_LoadLock CDataSource::GetLoadedTSE_Lock(const TBlobId& blob_id, const CTimeout& timeout)
+{
+    return GetLoadedTSE_Lock(blob_id, CDeadline(timeout));
+}
+
+
+CTSE_LoadLock CDataSource::GetLoadedTSE_Lock(const TBlobId& blob_id, const CDeadline& deadline)
 {
     CTSE_LoadLock lock = GetTSE_LoadLock(blob_id);
     if ( IsLoaded(*lock) ) {
         return lock;
     }
-    CDeadline deadline(timeout);
     while ( lock.x_GetGuard().WaitForSignal(deadline) ) {
         if ( IsLoaded(*lock) ) {
             return lock;
@@ -1924,6 +2368,7 @@ void CDataSource::x_SetLoadLock(CTSE_LoadLock& load, CTSE_Lock& lock)
     load.m_Info->m_LockCounter.Add(1);
     if ( !IsLoaded(*load) ) {
         _ASSERT(load->m_LoadMutex);
+        CThread::GetSystemID(&load.m_LoadLockOwner);
         load.m_LoadLock.Reset(new CTSE_LoadLockGuard(this, load->m_LoadMutex));
         if ( IsLoaded(*load) ) {
             load.ReleaseLoadLock();
@@ -1944,6 +2389,7 @@ void CDataSource::x_SetLoadLock(CTSE_LoadLock& load,
     load.m_Info.Reset(&tse);
     if ( !IsLoaded(tse) ) {
         _ASSERT(load_mutex);
+        CThread::GetSystemID(&load.m_LoadLockOwner);
         load.m_LoadLock.Reset(new CTSE_LoadLockGuard(this, load_mutex));
         if ( IsLoaded(tse) ) {
             load.ReleaseLoadLock();
@@ -1963,10 +2409,6 @@ void CDataSource::x_ReleaseLastLock(CTSE_Lock& lock)
 void CDataSource::x_ReleaseLastLoadLock(CTSE_LoadLock& lock)
 {
     CRef<CTSE_Info> info = lock.m_Info;
-    if ( info->m_LoadState == CTSE_Info::eNotLoaded ) {
-        // reset TSE info into empty state
-        info->x_Reset();
-    }
     lock.m_Info.Reset();
     lock.m_DataSource.Reset();
     x_ReleaseLastTSELock(info);
@@ -1987,6 +2429,7 @@ CTSE_LoadLock& CTSE_LoadLock::operator=(const CTSE_LoadLock& lock)
         Reset();
         m_Info = lock.m_Info;
         m_DataSource = lock.m_DataSource;
+        m_LoadLockOwner = lock.m_LoadLockOwner;
         m_LoadLock = lock.m_LoadLock;
         if ( *this ) {
             TSE_LOCK_TRACE("TSE_LoadLock("<<&*lock<<") "<<this<<" lock");
@@ -2034,7 +2477,15 @@ void CTSE_LoadLock::ReleaseLoadLock(void)
 {
     if ( m_LoadLock ) {
         if ( IsLoaded() ) {
-            x_GetGuard().Release();
+            TThreadSystemID current;
+            CThread::GetSystemID(&current);
+            if ( m_LoadLockOwner == current ) {
+                x_GetGuard().Release();
+            }
+        }
+        else {
+            // reset TSE info into empty state
+            m_Info->x_Reset();
         }
         m_LoadLock.Reset();
     }
