@@ -1,4 +1,4 @@
-/*  $Id: scope.cpp 610058 2020-06-10 16:19:48Z ivanov $
+/*  $Id: scope.cpp 659592 2022-11-30 18:45:10Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -146,6 +146,18 @@ CScope::TBioseqHandles CScope::GetBioseqHandles(const TIds& ids)
 }
 
 
+CScope::TCDD_Entries CScope::GetCDDAnnots(const TIds& idhs)
+{
+    return m_Impl->GetCDDAnnots(idhs);
+}
+
+
+CScope::TCDD_Entries CScope::GetCDDAnnots(const TBioseqHandles& bhs)
+{
+    return m_Impl->GetCDDAnnots(bhs);
+}
+
+
 CBioseq_Handle CScope::GetBioseqHandle(const CSeq_id_Handle& id,
                                        EGetBioseqFlag get_flag)
 {
@@ -459,13 +471,13 @@ string CScope::GetLabel(const CSeq_id_Handle& idh, TGetFlags flags)
 }
 
 
-int CScope::GetTaxId(const CSeq_id& id, TGetFlags flags)
+TTaxId CScope::GetTaxId(const CSeq_id& id, TGetFlags flags)
 {
     return GetTaxId(CSeq_id_Handle::GetHandle(id), flags);
 }
 
 
-int CScope::GetTaxId(const CSeq_id_Handle& idh, TGetFlags flags)
+TTaxId CScope::GetTaxId(const CSeq_id_Handle& idh, TGetFlags flags)
 {
     return m_Impl->GetTaxId(idh, flags);
 }
