@@ -1,4 +1,4 @@
-/*  $Id: scope_info.cpp 630071 2021-04-26 17:10:15Z ivanov $
+/*  $Id: scope_info.cpp 669333 2023-06-27 13:18:56Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -450,37 +450,39 @@ CDataSource_ScopeInfo::GetTSE_Lock(const CTSE_Lock& lock)
     }
     TTSE_ScopeInfo info;
     {{
-        TTSE_InfoMapMutex::TWriteLockGuard guard(m_TSE_InfoMapMutex);
-        TTSE_ScopeInfo& slot = m_TSE_InfoMap[lock->GetBlobId()];
-        if ( !slot ) {
-            slot = info = new CTSE_ScopeInfo(*this, lock,
-                                             m_NextTSEIndex++,
-                                             m_CanBeUnloaded);
-            if ( m_CanBeUnloaded ) {
-                // add this TSE into index by SeqId
-                x_IndexTSE(*info);
+        {{
+            TTSE_InfoMapMutex::TWriteLockGuard guard(m_TSE_InfoMapMutex);
+            TTSE_ScopeInfo& slot = m_TSE_InfoMap[lock->GetBlobId()];
+            if ( !slot ) {
+                slot = info = new CTSE_ScopeInfo(*this, lock,
+                                                 m_NextTSEIndex++,
+                                                 m_CanBeUnloaded);
+                if ( m_CanBeUnloaded ) {
+                    // add this TSE into index by SeqId
+                    x_IndexTSE(*info);
+                }
             }
-        }
-        else {
-            info = slot;
-        }
+            else {
+                info = slot;
+            }
+        }}
         _ASSERT(info->IsAttached() && &info->GetDSInfo() == this);
-        info->m_TSE_LockCounter.Add(1);
-        info->m_UserLockCounter.Add(1);
+        ++info->m_TSE_LockCounter;
+        ++info->m_UserLockCounter;
         {{
             // first remove the TSE from unlock queue
             TTSE_LockSetMutex::TWriteLockGuard guard2(m_TSE_UnlockQueueMutex);
             // TSE must be locked already by caller
-            _ASSERT(info->m_TSE_LockCounter.Get() > 0);
+            _ASSERT(info->m_TSE_LockCounter > 0);
             m_TSE_UnlockQueue.Erase(info);
             // TSE must be still locked by caller even after removing it
             // from unlock queue
-            _ASSERT(info->m_TSE_LockCounter.Get() > 0);
+            _ASSERT(info->m_TSE_LockCounter > 0);
         }}
         info->SetTSE_Lock(lock);
         ret.Reset(info);
-        _VERIFY(info->m_UserLockCounter.Add(-1) > 0);
-        _VERIFY(info->m_TSE_LockCounter.Add(-1) > 0);
+        _VERIFY(--info->m_UserLockCounter > 0);
+        _VERIFY(--info->m_TSE_LockCounter > 0);
         _ASSERT(info->GetTSE_Lock() == lock);
     }}
     return ret;
@@ -492,18 +494,21 @@ void CDataSource_ScopeInfo::AttachTSE(CTSE_ScopeInfo& info,
 {
     _ASSERT(m_CanBeUnloaded == info.CanBeUnloaded());
     _ASSERT(!info.m_DS_Info);
+    _ASSERT(!info.m_TSE_LockAssigned);
     _ASSERT(!info.m_TSE_Lock);
     _ASSERT(lock && &lock->GetDataSource() == &GetDataSource());
-    TTSE_InfoMapMutex::TWriteLockGuard guard(m_TSE_InfoMapMutex);
-    _VERIFY(m_TSE_InfoMap.insert(TTSE_InfoMap::value_type
-                                 (lock->GetBlobId(),
-                                  //STSE_Key(*lock, m_CanBeUnloaded),
-                                  Ref(&info))).second);
-    if ( m_CanBeUnloaded ) {
-        // add this TSE into index by SeqId
-        x_IndexTSE(info);
-    }
-    info.m_DS_Info = this;
+    {{
+        TTSE_InfoMapMutex::TWriteLockGuard guard(m_TSE_InfoMapMutex);
+        _VERIFY(m_TSE_InfoMap.insert(TTSE_InfoMap::value_type
+                                     (lock->GetBlobId(),
+                                      //STSE_Key(*lock, m_CanBeUnloaded),
+                                      Ref(&info))).second);
+        if ( m_CanBeUnloaded ) {
+            // add this TSE into index by SeqId
+            x_IndexTSE(info);
+        }
+        info.m_DS_Info = this;
+    }}
     info.SetTSE_Lock(lock);
 }
 
@@ -553,7 +558,7 @@ void CDataSource_ScopeInfo::ReleaseTSEUserLock(CTSE_ScopeInfo& tse)
     {{
         CTSE_ScopeInternalLock unlocked;
         TTSE_LockSetMutex::TWriteLockGuard tse_guard(m_TSE_UnlockQueueMutex);
-        if ( tse.m_UserLockCounter.Get() > 0 ) {
+        if ( tse.m_UserLockCounter > 0 ) {
             // relocked already
             return;
         }
@@ -578,10 +583,10 @@ void CDataSource_ScopeInfo::AcquireTSEUserLock(CTSE_ScopeInfo& tse)
         TTSE_LockSetMutex::TWriteLockGuard tse_guard(m_TSE_UnlockQueueMutex);
         m_TSE_UnlockQueue.Erase(&tse);
     }}
-    if ( !tse.m_TSE_Lock ) {
+    if ( !tse.m_TSE_LockAssigned ) {
         CDataSource_ScopeInfo* ds = tse.m_DS_Info;
         if ( !ds ) {
-            tse.m_UserLockCounter.Add(-1);
+            --tse.m_UserLockCounter;
             NCBI_THROW(CCoreException, eNullPtr,
                        "CTSE_ScopeInfo is not attached to CScope");
         }
@@ -590,8 +595,9 @@ void CDataSource_ScopeInfo::AcquireTSEUserLock(CTSE_ScopeInfo& tse)
         _ASSERT(lock);
         tse.SetTSE_Lock(lock);
         _ASSERT(tse.GetTSE_Lock() == lock);
-        _ASSERT(tse.m_UserLockCounter.Get() > 0);
+        _ASSERT(tse.m_UserLockCounter > 0);
     }
+    _ASSERT(tse.m_TSE_LockAssigned);
     _ASSERT(tse.m_TSE_Lock);
 }
 
@@ -600,11 +606,11 @@ void CDataSource_ScopeInfo::AcquireTSEUserLock(CTSE_ScopeInfo& tse)
 // CTSE_ScopeInternalLocks are stored in m_TSE_UnlockQueue 
 void CDataSource_ScopeInfo::ForgetTSELock(CTSE_ScopeInfo& tse)
 {
-    if ( tse.m_TSE_LockCounter.Get() > 0 ) {
+    if ( tse.m_TSE_LockCounter > 0 ) {
         // relocked already
         return;
     }
-    if ( !tse.GetTSE_Lock() ) {
+    if ( !tse.m_TSE_LockAssigned ) {
         // already unlocked
         return;
     }
@@ -681,14 +687,16 @@ void CDataSource_ScopeInfo::RemoveFromHistory(CTSE_ScopeInfo& tse,
                                               bool drop_from_ds)
 {
     tse.ReleaseUsedTSEs();
-    TTSE_InfoMapMutex::TWriteLockGuard guard1(m_TSE_InfoMapMutex);
-    if ( tse.CanBeUnloaded() ) {
-        x_UnindexTSE(tse);
-    }
-    tse.RestoreReplacedTSE();
-    _VERIFY(m_TSE_InfoMap.erase(tse.GetBlobId()));
+    {{
+        TTSE_InfoMapMutex::TWriteLockGuard guard1(m_TSE_InfoMapMutex);
+        if ( tse.CanBeUnloaded() ) {
+            x_UnindexTSE(tse);
+        }
+        tse.RestoreReplacedTSE();
+        _VERIFY(m_TSE_InfoMap.erase(tse.GetBlobId()));
+    }}
     // prevent storing into m_TSE_UnlockQueue
-    _VERIFY(tse.m_UserLockCounter.Add(1) > 0);
+    _VERIFY(++tse.m_UserLockCounter > 0);
     // remove TSE lock completely
     {{
         TTSE_LockSetMutex::TWriteLockGuard guard2(m_TSE_UnlockQueueMutex);
@@ -705,7 +713,7 @@ void CDataSource_ScopeInfo::RemoveFromHistory(CTSE_ScopeInfo& tse,
         tse.ResetTSE_Lock();
     }
     tse.x_DetachDS();
-    _VERIFY(tse.m_UserLockCounter.Add(-1) >= 0); // restore lock counter
+    _VERIFY(--tse.m_UserLockCounter >= 0); // restore lock counter
     _ASSERT(!tse.GetTSE_Lock());
     _ASSERT(!tse.m_DS_Info);
 }
@@ -837,6 +845,7 @@ SSeqMatch_Scope CDataSource_ScopeInfo::BestResolve(const CSeq_id_Handle& idh,
         _ASSERT(ret.m_Seq_id);
         _ASSERT(ret.m_Bioseq);
         _ASSERT(ret.m_TSE_Lock);
+        _ASSERT(ret.m_TSE_Lock->m_TSE_LockAssigned);
         _ASSERT(ret.m_Bioseq == ret.m_TSE_Lock->m_TSE_Lock->FindBioseq(ret.m_Seq_id));
     }
 #endif
@@ -1042,6 +1051,9 @@ CTSE_ScopeInfo::CTSE_ScopeInfo(CDataSource_ScopeInfo& ds_info,
                                bool can_be_unloaded)
     : m_DS_Info(&ds_info),
       m_LoadIndex(load_index),
+      m_TSE_LockCounter(0),
+      m_UserLockCounter(0),
+      m_TSE_LockAssigned(false),
       m_UsedByTSE(0)
 {
     _ASSERT(lock);
@@ -1052,7 +1064,7 @@ CTSE_ScopeInfo::CTSE_ScopeInfo(CDataSource_ScopeInfo& ds_info,
     else {
         // permanent lock
         _TRACE_TSE_LOCK("CTSE_ScopeInfo("<<this<<") perm lock");
-        m_TSE_LockCounter.Add(1);
+        ++m_TSE_LockCounter;
         x_SetTSE_Lock(lock);
         _ASSERT(m_TSE_Lock == lock);
     }
@@ -1063,12 +1075,13 @@ CTSE_ScopeInfo::~CTSE_ScopeInfo(void)
 {
     if ( !CanBeUnloaded() ) {
         // remove permanent lock
-        _TRACE_TSE_LOCK("CTSE_ScopeInfo("<<this<<") perm unlock: "<<m_TSE_LockCounter.Get());
-        _VERIFY(m_TSE_LockCounter.Add(-1) == 0);
+        _TRACE_TSE_LOCK("CTSE_ScopeInfo("<<this<<") perm unlock: "<<m_TSE_LockCounter);
+        _VERIFY(--m_TSE_LockCounter == 0);
     }
     x_DetachDS();
-    _TRACE_TSE_LOCK("CTSE_ScopeInfo("<<this<<") final: "<<m_TSE_LockCounter.Get());
-    _ASSERT(m_TSE_LockCounter.Get() == 0);
+    _TRACE_TSE_LOCK("CTSE_ScopeInfo("<<this<<") final: "<<m_TSE_LockCounter);
+    _ASSERT(m_TSE_LockCounter == 0);
+    _ASSERT(!m_TSE_LockAssigned);
     _ASSERT(!m_TSE_Lock);
     _ASSERT(!m_UsedByTSE);
     _ASSERT(m_UsedTSE_Set.empty());
@@ -1077,7 +1090,7 @@ CTSE_ScopeInfo::~CTSE_ScopeInfo(void)
 
 CTSE_ScopeInfo::TBlobOrder CTSE_ScopeInfo::GetBlobOrder(void) const
 {
-    if ( !m_TSE_Lock ) {
+    if ( !m_TSE_LockAssigned ) {
         _ASSERT(m_UnloadedInfo.get());
         return m_UnloadedInfo->m_BlobOrder;
     }
@@ -1117,13 +1130,13 @@ const CTSE_ScopeInfo::TSeqIds& CTSE_ScopeInfo::GetBioseqsIds(void) const
 
 void CTSE_ScopeInfo::x_InternalLockTSE(void)
 {
-    _VERIFY(m_TSE_LockCounter.Add(1) > 0);
+    _VERIFY(++m_TSE_LockCounter > 0);
 }
 
 
 void CTSE_ScopeInfo::x_InternalUnlockTSE(void)
 {
-    if ( m_TSE_LockCounter.Add(-1) == 0 ) {
+    if ( --m_TSE_LockCounter == 0 ) {
         _ASSERT(CanBeUnloaded());
         if ( IsAttached() ) {
             GetDSInfo().ForgetTSELock(*this);
@@ -1134,7 +1147,7 @@ void CTSE_ScopeInfo::x_InternalUnlockTSE(void)
 
 void CTSE_ScopeInfo::x_UserLockTSE(void)
 {
-    if ( m_UserLockCounter.Add(1) == 1 || !GetTSE_Lock() ) {
+    if ( ++m_UserLockCounter == 1 || !GetTSE_Lock() ) {
         //_ASSERT(CanBeUnloaded());
         if ( IsAttached() ) {
             // TODO: possible race - if the TSE becomes detached after the above check
@@ -1155,7 +1168,7 @@ void CTSE_ScopeInfo::x_UserLockTSE(void)
 
 void CTSE_ScopeInfo::x_UserUnlockTSE(void)
 {
-    if ( m_UserLockCounter.Add(-1) == 0 ) {
+    if ( --m_UserLockCounter == 0 ) {
         //_ASSERT(CanBeUnloaded());
         if ( IsAttached() ) {
             GetDSInfo().ReleaseTSEUserLock(*this);
@@ -1206,7 +1219,7 @@ void CTSE_ScopeUserLocker::Unlock(CTSE_ScopeInfo* tse) const
 
 bool CTSE_ScopeInfo::x_SameTSE(const CTSE_Info& tse) const
 {
-    return m_TSE_LockCounter.Get() > 0 && m_TSE_Lock && &*m_TSE_Lock == &tse;
+    return m_TSE_LockCounter > 0 && m_TSE_LockAssigned && &*m_TSE_Lock == &tse;
 }
 
 
@@ -1235,7 +1248,7 @@ bool CTSE_ScopeInfo::AddUsedTSE(const CTSE_ScopeUserLock& used_tse) const
     CTSE_ScopeInfo& add_info = const_cast<CTSE_ScopeInfo&>(*used_tse);
     if ( &add_info == this || // the same TSE
          !add_info.CanBeUnloaded() || // added is permanentrly locked
-         m_TSE_LockCounter.Get() == 0 ) { // this one is unlocked
+         m_TSE_LockCounter == 0 ) { // this one is unlocked
         return false;
     }
     CFastMutexGuard guard(sx_UsedTSEMutex);
@@ -1258,8 +1271,9 @@ bool CTSE_ScopeInfo::AddUsedTSE(const CTSE_ScopeUserLock& used_tse) const
 void CTSE_ScopeInfo::x_SetTSE_Lock(const CTSE_Lock& lock)
 {
     _ASSERT(lock);
-    if ( !m_TSE_Lock ) {
+    if ( !m_TSE_LockAssigned ) {
         m_TSE_Lock = lock;
+        m_TSE_LockAssigned = true;
         GetDSInfo().AddTSE_Lock(lock);
     }
     _ASSERT(m_TSE_Lock == lock);
@@ -1268,8 +1282,9 @@ void CTSE_ScopeInfo::x_SetTSE_Lock(const CTSE_Lock& lock)
 
 void CTSE_ScopeInfo::x_ResetTSE_Lock(void)
 {
-    if ( m_TSE_Lock ) {
+    if ( m_TSE_LockAssigned ) {
         CTSE_Lock lock;
+        m_TSE_LockAssigned = false;
         lock.Swap(m_TSE_Lock);
         GetDSInfo().RemoveTSE_Lock(lock);
     }
@@ -1280,7 +1295,7 @@ void CTSE_ScopeInfo::x_ResetTSE_Lock(void)
 void CTSE_ScopeInfo::SetTSE_Lock(const CTSE_Lock& lock)
 {
     _ASSERT(lock);
-    if ( !m_TSE_Lock ) {
+    if ( !m_TSE_LockAssigned ) {
         CMutexGuard guard(m_TSE_LockMutex);
         x_SetTSE_Lock(lock);
     }
@@ -1290,7 +1305,7 @@ void CTSE_ScopeInfo::SetTSE_Lock(const CTSE_Lock& lock)
 
 void CTSE_ScopeInfo::ResetTSE_Lock(void)
 {
-    if ( m_TSE_Lock ) {
+    if ( m_TSE_LockAssigned ) {
         CMutexGuard guard(m_TSE_LockMutex);
         x_ResetTSE_Lock();
     }
@@ -1300,8 +1315,9 @@ void CTSE_ScopeInfo::ResetTSE_Lock(void)
 
 void CTSE_ScopeInfo::DropTSE_Lock(void)
 {
-    if ( m_TSE_Lock ) {
+    if ( m_TSE_LockAssigned ) {
         CMutexGuard guard(m_TSE_LockMutex);
+        m_TSE_LockAssigned = false;
         m_TSE_Lock.Reset();
     }
     _ASSERT(!m_TSE_Lock);
@@ -1316,29 +1332,36 @@ void CTSE_ScopeInfo::SetEditTSE(const CTSE_Lock& new_tse_lock,
     _ASSERT(&new_tse_lock->GetDataSource() == &new_ds.GetDataSource());
 
     CUnlockedTSEsGuard unlocked_guard;
-    CMutexGuard guard(m_TSE_LockMutex);
-    _ASSERT(m_TSE_Lock);
-    _ASSERT(&m_TSE_Lock->GetDataSource() == &GetDSInfo().GetDataSource());
-    CTSE_Lock old_tse_lock = m_TSE_Lock;
+    CTSE_Lock old_tse_lock;
+    {{
+        CMutexGuard guard(m_TSE_LockMutex);
+        _ASSERT(m_TSE_LockAssigned);
+        _ASSERT(m_TSE_Lock);
+        _ASSERT(&m_TSE_Lock->GetDataSource() == &GetDSInfo().GetDataSource());
+        old_tse_lock = m_TSE_Lock;
+    }}
     
     TScopeInfoMap old_map; // save old scope info map
-    CMutexGuard guard2(m_ScopeInfoMapMutex);
-    old_map.swap(m_ScopeInfoMap);
     TBioseqById old_bioseq_map; // save old bioseq info map
-    old_bioseq_map.swap(m_BioseqById);
+    {{
+        CMutexGuard guard2(m_ScopeInfoMapMutex);
+        old_map.swap(m_ScopeInfoMap);
+        old_bioseq_map.swap(m_BioseqById);
+    }}
 
     GetDSInfo().RemoveFromHistory(*this); // remove tse from old ds
+    _ASSERT(!m_TSE_LockAssigned);
     _ASSERT(!m_TSE_Lock);
     _ASSERT(!m_DS_Info);
     if ( CanBeUnloaded() ) {
         m_UnloadedInfo.reset(); // edit tse cannot be unloaded
-        m_TSE_LockCounter.Add(1);
+        ++m_TSE_LockCounter;
     }
 
     // convert scope info map
     const TEditInfoMap& edit_map = new_tse_lock->m_BaseTSE->m_ObjectCopyMap;
     NON_CONST_ITERATE ( TScopeInfoMap, it, old_map ) {
-        CConstRef<CObject> old_obj(it->first);
+        CConstRef<CTSE_Info_Object> old_obj(it->first);
         _ASSERT(old_obj);
         TEditInfoMap::const_iterator iter = edit_map.find(old_obj);
         TScopeInfoMapKey new_obj;
@@ -1352,8 +1375,9 @@ void CTSE_ScopeInfo::SetEditTSE(const CTSE_Lock& new_tse_lock,
         _ASSERT(new_obj);
         _ASSERT(&*new_obj != &*old_obj);
         TScopeInfoMapValue info = it->second;
-        _ASSERT(info->m_ObjectInfo == old_obj);
+        _ASSERT(info->HasObject(*old_obj));
         info->m_ObjectInfo = new_obj;
+        _ASSERT(info->HasObject(*new_obj));
         _VERIFY(m_ScopeInfoMap.insert
                 (TScopeInfoMap::value_type(new_obj, info)).second);
     }
@@ -1363,6 +1387,7 @@ void CTSE_ScopeInfo::SetEditTSE(const CTSE_Lock& new_tse_lock,
     new_ds.AttachTSE(*this, new_tse_lock);
 
     _ASSERT(&GetDSInfo() == &new_ds);
+    _ASSERT(m_TSE_LockAssigned);
     _ASSERT(m_TSE_Lock == new_tse_lock);
     
     const_cast<CTSE_Info&>(*new_tse_lock).m_BaseTSE->m_ObjectCopyMap.clear();
@@ -1395,27 +1420,29 @@ void CTSE_ScopeInfo::ReplaceTSE(const CTSE_Info& old_tse)
 // Action A4.
 void CTSE_ScopeInfo::ForgetTSE_Lock(void)
 {
-    if ( m_TSE_LockCounter.Get() > 0 ) {
+    if ( m_TSE_LockCounter > 0 ) {
         // relocked already
         return;
     }
     ReleaseUsedTSEs();
-    if ( !m_TSE_Lock ) {
+    if ( !m_TSE_LockAssigned ) {
         return;
     }
     CMutexGuard guard(m_TSE_LockMutex);
-    if ( m_TSE_LockCounter.Get() > 0 ) {
+    if ( m_TSE_LockCounter > 0 ) {
         // relocked already
         return;
     }
-    if ( !m_TSE_Lock ) {
+    if ( !m_TSE_LockAssigned ) {
         return;
     }
     {{
         CMutexGuard guard2(m_ScopeInfoMapMutex);
         NON_CONST_ITERATE ( TScopeInfoMap, it, m_ScopeInfoMap ) {
-            _ASSERT(!it->second->m_TSE_Handle.m_TSE);
+            _ASSERT(!it->second->m_TSE_HandleAssigned);
+            it->second->m_ObjectInfoAssigned = false;
             it->second->m_ObjectInfo.Reset();
+            _ASSERT(!it->second->HasObject());
             if ( it->second->IsTemporary() ) {
                 it->second->x_DetachTSE(this);
             }
@@ -1436,13 +1463,16 @@ void CTSE_ScopeInfo::x_DetachDS(void)
     {{
         CMutexGuard guard2(m_ScopeInfoMapMutex);
         NON_CONST_ITERATE ( TScopeInfoMap, it, m_ScopeInfoMap ) {
-            //_ASSERT(!it->second->m_TSE_Handle.m_TSE);
+            it->second->m_TSE_HandleAssigned = false;
             it->second->m_TSE_Handle.Reset();
+            it->second->m_ObjectInfoAssigned = false;
             it->second->m_ObjectInfo.Reset();
+            _ASSERT(!it->second->HasObject());
             it->second->x_DetachTSE(this);
         }
         m_ScopeInfoMap.clear();
     }}
+    m_TSE_LockAssigned = false;
     m_TSE_Lock.Reset();
     while ( !m_BioseqById.empty() ) {
         CRef<CBioseq_ScopeInfo> bioseq = m_BioseqById.begin()->second;
@@ -1474,8 +1504,8 @@ pair<bool, CScopeInfo_Base*> CTSE_ScopeInfo::GetUserLockState(const CTSE_Handle*
     }
     // now we have one handle already
     _ASSERT(&tseh->x_GetScopeInfo() == this);
-    _ASSERT(m_UserLockCounter.Get() >= 1);
-    if ( m_UserLockCounter.Get() > 1 ) {
+    _ASSERT(m_UserLockCounter >= 1);
+    if ( m_UserLockCounter > 1 ) {
         // there are more sub-object handles
         ret.first = true;
         return ret;
@@ -1485,9 +1515,9 @@ pair<bool, CScopeInfo_Base*> CTSE_ScopeInfo::GetUserLockState(const CTSE_Handle*
     CMutexGuard guard(m_ScopeInfoMapMutex);
     for ( auto& s : m_ScopeInfoMap ) {
         if ( &s.second->m_TSE_Handle == tseh ) {
-            _ASSERT(s.second->m_LockCounter.Get() >= 1);
+            _ASSERT(s.second->m_LockCounter >= 1);
             ret.second = s.second.GetNCPointer();
-            ret.first = s.second->m_LockCounter.Get() > 1;
+            ret.first = s.second->m_LockCounter > 1;
             return ret;
         }
     }
@@ -1514,6 +1544,7 @@ void CTSE_ScopeInfo::RemoveFromHistory(const CTSE_Handle* tseh,
     }
     CTSE_Handle tse;
     if ( locked.second ) {
+        locked.second->m_TSE_HandleAssigned = false;
         tse.Swap(locked.second->m_TSE_Handle);
         _ASSERT(&tse.x_GetScopeInfo() == this);
     }
@@ -1589,13 +1620,13 @@ CTSE_ScopeInfo::x_GetScopeLock(const CTSE_Handle& tse,
         TScopeInfoMap::iterator iter = m_ScopeInfoMap.lower_bound(key);
         if ( iter == m_ScopeInfoMap.end() || iter->first != key ) {
             scope_info = new TScopeInfo(tse, info);
+            _ASSERT(scope_info->HasObject(info));
             TScopeInfoMapValue value(scope_info);
             m_ScopeInfoMap.insert(iter, TScopeInfoMap::value_type(key, value));
-            value->m_ObjectInfo = &info;
+            _ASSERT(value->HasObject(info));
         }
         else {
-            _ASSERT(iter->second->HasObject());
-            _ASSERT(&iter->second->GetObjectInfo_Base() == &info);
+            _ASSERT(iter->second->HasObject(info));
             scope_info = &dynamic_cast<TScopeInfo&>(*iter->second);
         }
     }}
@@ -1637,8 +1668,9 @@ CTSE_ScopeInfo::GetBioseqLock(CRef<CBioseq_ScopeInfo> info,
 {
     // TODO: possible deadlock (1), m_TSE_LockMutex is taken before m_TSE_UnlockQueueMutex
     // this thread calls GetBioseqHandle()
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     CTSE_ScopeUserLock tse(this);
+    _ASSERT(m_TSE_LockAssigned);
     _ASSERT(m_TSE_Lock);
     if ( !info ) {
         // find CBioseq_ScopeInfo
@@ -1649,6 +1681,7 @@ CTSE_ScopeInfo::GetBioseqLock(CRef<CBioseq_ScopeInfo> info,
             // named bioseq, look in Seq-id index only
             info = x_FindBioseqInfo(ids);
             if ( !info ) {
+                CMutexGuard guard2(m_ScopeInfoMapMutex);
                 info = x_CreateBioseqInfo(ids);
             }
         }
@@ -1661,18 +1694,19 @@ CTSE_ScopeInfo::GetBioseqLock(CRef<CBioseq_ScopeInfo> info,
                 if ( iter == m_ScopeInfoMap.end() || iter->first != key ) {
                     info = new CBioseq_ScopeInfo(*this);
                     TScopeInfoMapValue value(info);
-                    iter = m_ScopeInfoMap
-                        .insert(iter, TScopeInfoMap::value_type(key, value));
+                    m_ScopeInfoMap.insert(iter, TScopeInfoMap::value_type(key, value));
+                    _ASSERT(!value->HasObject());
                     value->m_ObjectInfo = &*bioseq;
+                    value->m_ObjectInfoAssigned = true;
+                    _ASSERT(value->HasObject(*bioseq));
                 }
                 else {
-                    _ASSERT(iter->second->HasObject());
-                    _ASSERT(&iter->second->GetObjectInfo_Base() == &*bioseq);
+                    _ASSERT(iter->second->HasObject(*bioseq));
                     info.Reset(&dynamic_cast<CBioseq_ScopeInfo&>(*iter->second));
                 }
             }}
             TBioseq_Lock ret(*info);
-            ret->x_SetTSE_Handle(tse);
+            ret->x_SetTSE_Lock(tse, *bioseq);
             return ret;
         }
     }
@@ -1698,8 +1732,7 @@ CTSE_ScopeInfo::GetBioseqLock(CRef<CBioseq_ScopeInfo> info,
         CMutexGuard guard2(m_ScopeInfoMapMutex);
         TScopeInfoMapKey key(bioseq);
         TScopeInfoMapValue value(info);
-        _VERIFY(m_ScopeInfoMap
-                .insert(TScopeInfoMap::value_type(key, value)).second);
+        m_ScopeInfoMap.insert(TScopeInfoMap::value_type(key, value));
     }
     TBioseq_Lock ret(*info);
     if ( bioseq ) {
@@ -1727,10 +1760,11 @@ CTSE_ScopeInfo::GetBioseqInfo(const SSeqMatch_Scope& match)
     const CBioseq_Info::TId& ids = match.m_Bioseq->GetId();
     _ASSERT(find(ids.begin(), ids.end(), match.m_Seq_id) != ids.end());
 
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     
     info = x_FindBioseqInfo(ids);
     if ( !info ) {
+        CMutexGuard guard2(m_ScopeInfoMapMutex);
         info = x_CreateBioseqInfo(ids);
     }
     return info;
@@ -1782,7 +1816,7 @@ void CTSE_ScopeInfo::x_UnindexBioseq(const CSeq_id_Handle& id,
 // Action A2.
 void CTSE_ScopeInfo::ResetEntry(CSeq_entry_ScopeInfo& info)
 {
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     _ASSERT(info.IsAttached());
     CScopeInfo_Ref<CScopeInfo_Base> child;
     if ( info.GetObjectInfo().Which() == CSeq_entry::e_Set ) {
@@ -1805,7 +1839,7 @@ void CTSE_ScopeInfo::ResetEntry(CSeq_entry_ScopeInfo& info)
 // Action A2.
 void CTSE_ScopeInfo::RemoveEntry(CSeq_entry_ScopeInfo& info)
 {
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     _ASSERT(info.IsAttached());
     CSeq_entry_Info& entry = info.GetNCObjectInfo();
     entry.GetParentBioseq_set_Info().RemoveEntry(Ref(&entry));
@@ -1816,7 +1850,7 @@ void CTSE_ScopeInfo::RemoveEntry(CSeq_entry_ScopeInfo& info)
 // Action A2.
 void CTSE_ScopeInfo::RemoveAnnot(CSeq_annot_ScopeInfo& info)
 {
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     _ASSERT(info.IsAttached());
     _ASSERT(info.GetObjectInfo().BelongsToTSE_Info(*m_TSE_Lock));
     CSeq_annot_Info& annot = info.GetNCObjectInfo();
@@ -1834,12 +1868,12 @@ void CTSE_ScopeInfo::x_CheckAdded(CScopeInfo_Base& parent,
 {
     _ASSERT(parent.IsAttached());
     _ASSERT(parent.HasObject());
-    _ASSERT(parent.m_LockCounter.Get() > 0);
+    _ASSERT(parent.m_LockCounter > 0);
     _ASSERT(child.IsDetached());
     _ASSERT(child.m_DetachedInfo);
     _ASSERT(child.HasObject());
     _ASSERT(!child.GetObjectInfo_Base().HasParent_Info());
-    _ASSERT(child.m_LockCounter.Get() > 0);
+    _ASSERT(child.m_LockCounter > 0);
     _ASSERT(x_SameTSE(parent.GetTSE_Handle().x_GetTSE_Info()));
 }
 #else  /* _DEBUG */
@@ -1854,7 +1888,7 @@ void CTSE_ScopeInfo::AddEntry(CBioseq_set_ScopeInfo& parent,
                               CSeq_entry_ScopeInfo& child,
                               int index)
 {
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     x_CheckAdded(parent, child);
     parent.GetNCObjectInfo().AddEntry(Ref(&child.GetNCObjectInfo()), index, true);
     x_RestoreAdded(parent, child);
@@ -1866,7 +1900,7 @@ void CTSE_ScopeInfo::AddEntry(CBioseq_set_ScopeInfo& parent,
 void CTSE_ScopeInfo::AddAnnot(CSeq_entry_ScopeInfo& parent,
                               CSeq_annot_ScopeInfo& child)
 {
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     _ASSERT(!child.GetObjectInfo().HasTSE_Info());
     x_CheckAdded(parent, child);
     parent.GetNCObjectInfo().AddAnnot(Ref(&child.GetNCObjectInfo()));
@@ -1880,7 +1914,7 @@ void CTSE_ScopeInfo::AddAnnot(CSeq_entry_ScopeInfo& parent,
 void CTSE_ScopeInfo::SelectSet(CSeq_entry_ScopeInfo& parent,
                                CBioseq_set_ScopeInfo& child)
 {
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     x_CheckAdded(parent, child);
     _ASSERT(parent.GetObjectInfo().Which() == CSeq_entry::e_not_set);
     parent.GetNCObjectInfo().SelectSet(child.GetNCObjectInfo());
@@ -1893,7 +1927,7 @@ void CTSE_ScopeInfo::SelectSet(CSeq_entry_ScopeInfo& parent,
 void CTSE_ScopeInfo::SelectSeq(CSeq_entry_ScopeInfo& parent,
                                CBioseq_ScopeInfo& child)
 {
-    CMutexGuard guard(m_TSE_LockMutex);
+    //CMutexGuard guard(m_TSE_LockMutex);
     x_CheckAdded(parent, child);
     _ASSERT(parent.GetObjectInfo().Which() == CSeq_entry::e_not_set);
     parent.GetNCObjectInfo().SelectSeq(child.GetNCObjectInfo());
@@ -1924,6 +1958,7 @@ void CTSE_ScopeInfo::x_SaveRemoved(CScopeInfo_Base& info)
         for ( TScopeInfoMap::iterator it = m_ScopeInfoMap.begin(); it != m_ScopeInfoMap.end(); ) {
             if ( !it->first->BelongsToTSE_Info(*m_TSE_Lock) ) {
                 _TRACE(" "<<it->second<<" " << it->first);
+                it->second->m_TSE_HandleAssigned = false;
                 it->second->m_TSE_Handle.Reset();
                 it->second->x_DetachTSE(this);
                 if ( &*it->second != &info ) {
@@ -1961,12 +1996,12 @@ void CTSE_ScopeInfo::x_RestoreAdded(CScopeInfo_Base& parent,
 {
     _ASSERT(parent.IsAttached()); // parent is attached
     _ASSERT(parent.m_TSE_Handle); // and locked
-    _ASSERT(parent.m_LockCounter.Get() > 0);
+    _ASSERT(parent.m_LockCounter > 0);
     _ASSERT(child.IsDetached()); // child is detached
     _ASSERT(child.m_DetachedInfo); // and contain m_DetachedInfo
     _ASSERT(child.HasObject()); // it contains pointer to removed object
     _ASSERT(child.GetObjectInfo_Base().HasParent_Info());//and is connected
-    _ASSERT(child.m_LockCounter.Get() > 0);
+    _ASSERT(child.m_LockCounter > 0);
 
     _TRACE("x_RestoreAdded("<<&child<<") TSE: "<<this);
 
@@ -1982,7 +2017,7 @@ void CTSE_ScopeInfo::x_RestoreAdded(CScopeInfo_Base& parent,
         ITERATE ( TDetachedInfo, it, infos->GetData() ) {
             _TRACE(" "<<it->second<<" " << it->first);
             CScopeInfo_Base& info = it->second.GetNCObject();
-            if ( info.m_LockCounter.Get() > 0 ) {
+            if ( info.m_LockCounter > 0 ) {
                 info.x_AttachTSE(this);
                 _VERIFY(m_ScopeInfoMap.insert
                         (TScopeInfoMap::value_type(it->first, it->second)).second);
