@@ -1,4 +1,4 @@
-/*  $Id: tse_split_info.cpp 544387 2017-08-22 19:28:06Z vasilche $
+/*  $Id: tse_split_info.cpp 669339 2023-06-27 13:19:44Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -46,6 +46,7 @@
 #include <objmgr/seq_map.hpp>
 #include <objmgr/prefetch_manager.hpp>
 #include <objects/seq/Seq_literal.hpp>
+#include <common/ncbi_sanitizers.h>
 
 #include <algorithm>
 
@@ -82,6 +83,7 @@ CTSE_Split_Info::CTSE_Split_Info(TBlobId blob_id, TBlobVersion blob_ver)
 
 CTSE_Split_Info::~CTSE_Split_Info(void)
 {
+    CMutexGuard guard(m_ChunksMutex);
     NON_CONST_ITERATE ( TChunks, it, m_Chunks ) {
         it->second->x_DropAnnotObjects();
     }
@@ -93,6 +95,7 @@ CTSE_Split_Info::~CTSE_Split_Info(void)
 void CTSE_Split_Info::x_TSEAttach(CTSE_Info& tse, CRef<ITSE_Assigner>& lsnr)
 {
     m_TSE_Set.insert(TTSE_Set::value_type(&tse, lsnr));
+    CMutexGuard guard(m_ChunksMutex);
     NON_CONST_ITERATE ( TChunks, it, m_Chunks ) {
         it->second->x_TSEAttach(tse, *lsnr);
     }
@@ -120,6 +123,16 @@ void CTSE_Split_Info::x_DSAttach(CDataSource& ds)
     if ( !m_DataLoader && ds.GetDataLoader() ) {
         m_DataLoader = &ds;
         _ASSERT(m_DataLoader);
+        if ( ds.x_IsTrackingSplitSeq() && ContainsBioseqs() ) {
+            vector<CSeq_id_Handle> ids;
+            {{
+                CMutexGuard guard(m_ChunksMutex);
+                for ( auto& chunk : m_Chunks ) {
+                    chunk.second->GetBioseqsIds(ids);
+                }
+            }}
+            ds.x_IndexSplitInfo(ids, this);
+        }
     }
 }
 
@@ -127,6 +140,16 @@ void CTSE_Split_Info::x_DSAttach(CDataSource& ds)
 void CTSE_Split_Info::x_DSDetach(CDataSource& ds)
 {
     if ( m_DataLoader == &ds ) {
+        if ( ds.x_IsTrackingSplitSeq() && ContainsBioseqs() ) {
+            vector<CSeq_id_Handle> ids;
+            {{
+                CMutexGuard guard(m_ChunksMutex);
+                for ( auto& chunk : m_Chunks ) {
+                    chunk.second->GetBioseqsIds(ids);
+                }
+            }}
+            ds.x_UnindexSplitInfo(ids, this);
+        }
         m_DataLoader = 0;
     }
 }
@@ -178,6 +201,7 @@ CDataLoader& CTSE_Split_Info::GetDataLoader(void) const
 
 bool CTSE_Split_Info::x_HasDelayedMainChunk(void) const
 {
+    CMutexGuard guard(m_ChunksMutex);
     TChunks::const_iterator iter = m_Chunks.end(), begin = m_Chunks.begin();
     return iter != begin && (--iter)->first == CTSE_Chunk_Info::kDelayedMain_ChunkId;
 }
@@ -185,6 +209,7 @@ bool CTSE_Split_Info::x_HasDelayedMainChunk(void) const
 
 bool CTSE_Split_Info::x_NeedsDelayedMainChunk(void) const
 {
+    CMutexGuard guard(m_ChunksMutex);
     TChunks::const_iterator iter = m_Chunks.end(), begin = m_Chunks.begin();
     if ( iter == begin || (--iter)->first != CTSE_Chunk_Info::kDelayedMain_ChunkId ) {
         // no delayed main chunk
@@ -197,9 +222,17 @@ bool CTSE_Split_Info::x_NeedsDelayedMainChunk(void) const
 
 void CTSE_Split_Info::x_LoadDelayedMainChunk(void) const
 {
-    TChunks::const_iterator iter = m_Chunks.end(), begin = m_Chunks.begin();
-    while ( iter != begin && (--iter)->first >= kMax_Int-1 ) {
-        iter->second->Load();
+    CRef<CTSE_Chunk_Info> delayed_chunk;
+    {{
+        CMutexGuard guard(m_ChunksMutex);
+        TChunks::const_iterator iter = m_Chunks.end(), begin = m_Chunks.begin();
+        while ( iter != begin && (--iter)->first >= kMax_Int-1 ) {
+            delayed_chunk = iter->second;
+            break;
+        }
+    }}
+    if ( delayed_chunk ) {
+        delayed_chunk->Load();
     }
 }
 
@@ -207,20 +240,23 @@ void CTSE_Split_Info::x_LoadDelayedMainChunk(void) const
 // chunk attach
 void CTSE_Split_Info::AddChunk(CTSE_Chunk_Info& chunk_info)
 {
-    CMutexGuard guard(m_SeqIdToChunksMutex);
-    _ASSERT(m_Chunks.find(chunk_info.GetChunkId()) == m_Chunks.end());
-    _ASSERT(m_Chunks.empty() || chunk_info.GetChunkId() != chunk_info.kDelayedMain_ChunkId);
-    bool need_update = x_HasDelayedMainChunk();
-    m_Chunks[chunk_info.GetChunkId()].Reset(&chunk_info);
-    chunk_info.x_SplitAttach(*this);
-    if ( need_update ) {
-        chunk_info.x_EnableAnnotIndex();
-    }
+    _ASSERT(!chunk_info.IsLoaded());
+    {{
+        CMutexGuard guard(m_ChunksMutex);
+        _ASSERT(m_Chunks.find(chunk_info.GetChunkId()) == m_Chunks.end());
+        _ASSERT(m_Chunks.empty() || chunk_info.GetChunkId() != chunk_info.kDelayedMain_ChunkId);
+        m_Chunks[chunk_info.GetChunkId()].Reset(&chunk_info);
+    }}
+    {{
+        CMutexGuard guard(m_AttachMutex);
+        chunk_info.x_SplitAttach(*this);
+    }}
 }
 
 
 CTSE_Chunk_Info& CTSE_Split_Info::GetChunk(TChunkId chunk_id)
 {
+    CMutexGuard guard(m_ChunksMutex);
     TChunks::iterator iter = m_Chunks.find(chunk_id);
     if ( iter == m_Chunks.end() ) {
         NCBI_THROW(CObjMgrException, eAddDataError,
@@ -232,6 +268,7 @@ CTSE_Chunk_Info& CTSE_Split_Info::GetChunk(TChunkId chunk_id)
 
 const CTSE_Chunk_Info& CTSE_Split_Info::GetChunk(TChunkId chunk_id) const
 {
+    CMutexGuard guard(m_ChunksMutex);
     TChunks::const_iterator iter = m_Chunks.find(chunk_id);
     if ( iter == m_Chunks.end() ) {
         NCBI_THROW(CObjMgrException, eAddDataError,
@@ -243,6 +280,7 @@ const CTSE_Chunk_Info& CTSE_Split_Info::GetChunk(TChunkId chunk_id) const
 
 CTSE_Chunk_Info& CTSE_Split_Info::GetSkeletonChunk(void)
 {
+    CMutexGuard guard(m_ChunksMutex);
     TChunks::iterator iter = m_Chunks.find(0);
     if ( iter != m_Chunks.end() ) {
         return *iter->second;
@@ -314,11 +352,39 @@ void CTSE_Split_Info::x_SetContainedId(const TBioseqId& id,
                                        TChunkId chunk_id,
                                        bool bioseq)
 {
+    CMutexGuard guard(m_SeqIdToChunksMutex);
     m_SeqIdToChunksSorted = false;
-    if ( bioseq && !m_ContainsBioseqs ) {
-        m_ContainsBioseqs = true;
+    if ( bioseq && !ContainsBioseqs() ) {
+        m_ContainsBioseqs.store(true, memory_order_relaxed);
     }
     m_SeqIdToChunks.push_back(pair<CSeq_id_Handle, TChunkId>(id, chunk_id));
+    if ( bioseq && m_DataLoader ) {
+        m_DataLoader->x_IndexSplitInfo(id, this);
+    }
+}
+
+
+void CTSE_Split_Info::x_SetContainedSeqIds(const vector<TBioseqId>& seq_ids,
+                                           const set<TBioseqId>& annot_ids,
+                                           TChunkId chunk_id)
+{
+    if ( seq_ids.empty() && annot_ids.empty() ) {
+        return;
+    }
+    CMutexGuard guard(m_SeqIdToChunksMutex);
+    m_SeqIdToChunksSorted = false;
+    if ( !seq_ids.empty() && !ContainsBioseqs() ) {
+        m_ContainsBioseqs.store(true, memory_order_relaxed);
+    }
+    for ( auto& id : seq_ids ) {
+        m_SeqIdToChunks.push_back(pair<CSeq_id_Handle, TChunkId>(id, chunk_id));
+    }
+    for ( auto& id : annot_ids ) {
+        m_SeqIdToChunks.push_back(pair<CSeq_id_Handle, TChunkId>(id, chunk_id));
+    }
+    if ( m_DataLoader && !seq_ids.empty() ) {
+        m_DataLoader->x_IndexSplitInfo(seq_ids, this);
+    }
 }
 
 
@@ -337,46 +403,24 @@ bool CTSE_Split_Info::x_CanAddBioseq(const TBioseqId& id) const
 void CTSE_Split_Info::x_UpdateFeatIdIndex(CSeqFeatData::E_Choice type,
                                           EFeatIdType id_type)
 {
-    NON_CONST_ITERATE ( TChunks, it, m_Chunks ) {
-        CTSE_Chunk_Info& chunk = *it->second;
-        if ( !chunk.IsLoaded() && !chunk.m_AnnotIndexEnabled &&
-             chunk.x_ContainsFeatIds(type, id_type) ) {
-            x_UpdateAnnotIndex(chunk);
-        }
-    }
 }
 
 
 void CTSE_Split_Info::x_UpdateFeatIdIndex(CSeqFeatData::ESubtype subtype,
                                           EFeatIdType id_type)
 {
-    NON_CONST_ITERATE ( TChunks, it, m_Chunks ) {
-        CTSE_Chunk_Info& chunk = *it->second;
-        if ( !chunk.IsLoaded() && !chunk.m_AnnotIndexEnabled &&
-             chunk.x_ContainsFeatIds(subtype, id_type) ) {
-            x_UpdateAnnotIndex(chunk);
-        }
-    }
 }
 
 
 void CTSE_Split_Info::x_UpdateAnnotIndex(void)
 {
-    NON_CONST_ITERATE ( TChunks, it, m_Chunks ) {
-        x_UpdateAnnotIndex(*it->second);
-    }
 }
 
 
 void CTSE_Split_Info::x_UpdateAnnotIndex(CTSE_Chunk_Info& chunk)
 {
-    if ( !chunk.IsLoaded() && !chunk.m_AnnotIndexEnabled ) {
-        NON_CONST_ITERATE ( TTSE_Set, it, m_TSE_Set ) {
-            CTSE_Info& tse = *it->first;
-            ITSE_Assigner& listener = *it->second;
-            listener.UpdateAnnotIndex(tse, chunk);
-        }
-        chunk.m_AnnotIndexEnabled = true;
+    ITERATE ( TTSE_Set, it, m_TSE_Set ) {
+        chunk.x_UpdateAnnotIndex(*it->first);
     }
 }
 
@@ -397,29 +441,51 @@ CTSE_Split_Info::x_FindChunk(const CSeq_id_Handle& id) const
 // load requests
 void CTSE_Split_Info::x_GetRecords(const CSeq_id_Handle& id, bool bioseq) const
 {
-    if ( bioseq && !m_ContainsBioseqs ) {
+    if ( bioseq && !ContainsBioseqs() ) {
+        // shortcut - this TSE doesn't contain any Bioseqs
+        return;
+    }
+    vector<TChunkId> chunk_ids;
+    {{
+        CMutexGuard guard(m_SeqIdToChunksMutex);
+        for ( TSeqIdToChunks::const_iterator iter = x_FindChunk(id);
+              iter != m_SeqIdToChunks.end() && iter->first == id; ++iter ) {
+            chunk_ids.push_back(iter->second);
+        }
+    }}
+    for ( auto& chunk_id : chunk_ids ) {
+        const CTSE_Chunk_Info& chunk = GetChunk(chunk_id);
+        if ( !chunk.IsLoaded() ) {
+            chunk.x_GetRecords(id, bioseq);
+        }
+    }
+}
+
+
+// load requests
+void CTSE_Split_Info::x_AddChunksForGetRecords(vector<CConstRef<CTSE_Chunk_Info>>& chunks,
+                                               const CSeq_id_Handle& id) const
+{
+    if ( !ContainsBioseqs() ) {
         // shortcut - this TSE doesn't contain any Bioseqs
         return;
     }
-    vector< CConstRef<CTSE_Chunk_Info> > chunks;
     {{
         CMutexGuard guard(m_SeqIdToChunksMutex);
         for ( TSeqIdToChunks::const_iterator iter = x_FindChunk(id);
               iter != m_SeqIdToChunks.end() && iter->first == id; ++iter ) {
             const CTSE_Chunk_Info& chunk = GetChunk(iter->second);
-            if ( !chunk.IsLoaded() ) {
+            if ( !chunk.IsLoaded() && chunk.ContainsBioseq(id) ) {
                 chunks.push_back(ConstRef(&chunk));
             }
         }
     }}
-    ITERATE ( vector< CConstRef<CTSE_Chunk_Info> >, it, chunks ) {
-        (*it)->x_GetRecords(id, bioseq);
-    }
 }
 
 
 void CTSE_Split_Info::GetBioseqsIds(TSeqIds& ids) const
 {
+    CMutexGuard guard(m_ChunksMutex);
     ITERATE ( TChunks, it, m_Chunks ) {
         it->second->GetBioseqsIds(ids);
     }
@@ -428,7 +494,7 @@ void CTSE_Split_Info::GetBioseqsIds(TSeqIds& ids) const
 
 bool CTSE_Split_Info::ContainsBioseq(const CSeq_id_Handle& id) const
 {
-    if ( !m_ContainsBioseqs ) {
+    if ( !ContainsBioseqs() ) {
         // shortcut - this TSE doesn't contain any Bioseqs
         return false;
     }
@@ -466,10 +532,16 @@ void CTSE_Split_Info::x_LoadChunks(const TChunkIds& chunk_ids) const
     sort(sorted_ids.begin(), sorted_ids.end());
     sorted_ids.erase(unique(sorted_ids.begin(), sorted_ids.end()),
         sorted_ids.end());
+#ifdef NCBI_USE_TSAN
+    const size_t limit_chunks_request = 15;
+#else
+    const size_t limit_chunks_request = 200;
+#endif
+    size_t reserve_size = min(sorted_ids.size(), limit_chunks_request);
     TChunkRefs chunks;
-    chunks.reserve(sorted_ids.size());
+    chunks.reserve(reserve_size);
     TInitGuards guards;
-    guards.reserve(sorted_ids.size());
+    guards.reserve(reserve_size);
     // Collect and lock all chunks to be loaded
     ITERATE(TChunkIds, id, sorted_ids) {
         CRef<CTSE_Chunk_Info> chunk(&info_nc.GetChunk(*id));
@@ -480,9 +552,79 @@ void CTSE_Split_Info::x_LoadChunks(const TChunkIds& chunk_ids) const
         }
         chunks.push_back(chunk);
         guards.push_back(guard);
+        if ( guards.size() >= limit_chunks_request ) {
+            // Load chunks
+            info_nc.GetDataLoader().GetChunks(chunks);
+            guards.clear();
+            chunks.clear();
+        }
+    }
+    if ( !guards.empty() ) {
+        // Load chunks
+        info_nc.GetDataLoader().GetChunks(chunks);
+        guards.clear();
+        chunks.clear();
+    }
+}
+
+
+void CTSE_Split_Info::x_LoadChunks(CDataLoader* loader,
+                                   const vector<CConstRef<CTSE_Chunk_Info>>& src_chunks)
+{
+    vector<CRef<CTSE_Chunk_Info>> sorted_chunks;
+    sorted_chunks.reserve(src_chunks.size());
+    for ( auto& chunk : src_chunks ) {
+        _ASSERT(&chunk->GetSplitInfo().GetDataLoader() == loader);
+        sorted_chunks.push_back(Ref(const_cast<CTSE_Chunk_Info*>(chunk.GetPointer())));
+    }
+    sort(sorted_chunks.begin(), sorted_chunks.end(),
+         [](const CRef<CTSE_Chunk_Info>& a,
+            const CRef<CTSE_Chunk_Info>& b) -> bool
+             {
+                 auto blob_id_a = a->GetBlobId();
+                 auto blob_id_b = b->GetBlobId();
+                 return ((blob_id_a < blob_id_b) ||
+                         (!(blob_id_b < blob_id_a) && a->GetChunkId() < b->GetChunkId()));
+             });
+    sorted_chunks.erase(unique(sorted_chunks.begin(), sorted_chunks.end(),
+                               [](const CRef<CTSE_Chunk_Info>& a,
+                                  const CRef<CTSE_Chunk_Info>& b) -> bool
+                                   {
+                                       return ((&a->GetSplitInfo() == &b->GetSplitInfo()) &&
+                                               (a->GetChunkId() == b->GetChunkId()));
+                                   }),
+                        sorted_chunks.end());
+#ifdef NCBI_USE_TSAN
+    const size_t limit_chunks_request = 15;
+#else
+    const size_t limit_chunks_request = 200;
+#endif
+    size_t reserve_size = min(sorted_chunks.size(), limit_chunks_request);
+    CDataLoader::TChunkSet chunks;
+    chunks.reserve(reserve_size);
+    vector< AutoPtr<CInitGuard> > guards;
+    guards.reserve(reserve_size);
+    // Collect and lock all chunks to be loaded
+    for ( auto& chunk : sorted_chunks ) {
+        AutoPtr<CInitGuard> guard = chunk->GetLoadInitGuard();
+        if ( !guard.get() || !*guard.get() ) {
+            continue;
+        }
+        chunks.push_back(chunk);
+        guards.push_back(guard);
+        if ( guards.size() >= limit_chunks_request ) {
+            // Load chunks
+            loader->GetChunks(chunks);
+            guards.clear();
+            chunks.clear();
+        }
+    }
+    if ( !guards.empty() ) {
+        // Load chunks
+        loader->GetChunks(chunks);
+        guards.clear();
+        chunks.clear();
     }
-    // Load chunks
-    info_nc.GetDataLoader().GetChunks(chunks);
 }
 
 
