@@ -1,4 +1,4 @@
-/*  $Id: autodef.cpp 629259 2021-04-13 13:28:40Z ivanov $
+/*  $Id: autodef.cpp 668931 2023-06-15 12:32:29Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -33,12 +33,14 @@
 #include <ncbi_pch.hpp>
 #include <objmgr/util/autodef.hpp>
 #include <corelib/ncbimisc.hpp>
+#include <objmgr/annot_ci.hpp>
 #include <objmgr/seqdesc_ci.hpp>
 #include <objmgr/bioseq_ci.hpp>
 #include <objmgr/util/feature.hpp>
 #include <objmgr/util/sequence.hpp>
 #include <objmgr/util/seq_loc_util.hpp>
 
+#include <objects/misc/sequence_macros.hpp>
 #include <objects/seq/Seq_descr.hpp>
 #include <objects/seq/Seqdesc.hpp>
 #include <objects/seq/Bioseq.hpp>
@@ -90,7 +92,6 @@ bool s_NeedFeatureClause(const CBioseq& b)
 
 void CAutoDef::AddSources (CSeq_entry_Handle se)
 {
-    
     // add sources to modifier combination groups
     CBioseq_CI seq_iter(se, CSeq_inst::eMol_na);
     for ( ; seq_iter; ++seq_iter ) {
@@ -106,6 +107,16 @@ void CAutoDef::AddSources (CSeq_entry_Handle se)
     m_OrigModCombo.SetExcludeSpOrgs (m_OrigModCombo.GetDefaultExcludeSp());
 }
 
+void CAutoDef::AddDescriptors(const TSources& sources)
+{
+    for (const auto& it : sources) {
+        // Feature clauses are not necessary at this point. They are formed later on anyway.
+        m_OrigModCombo.AddSource(*it);
+    }
+
+    // set default exclude_sp values
+    m_OrigModCombo.SetExcludeSpOrgs(m_OrigModCombo.GetDefaultExcludeSp());
+}
 
 void CAutoDef::x_SortModifierListByRank(TModifierIndexVector &index_list, CAutoDefSourceDescription::TAvailableModifierVector &modifier_list) 
 {
@@ -549,8 +560,13 @@ string CAutoDef::x_GetFeatureClauses(const CBioseq_Handle& bh)
 
     CSeqdesc_CI d(bh, CSeqdesc::e_User);
     while (d) {
-        if (x_IsHumanSTR(d->GetUser())) {
-            return x_GetHumanSTRFeatureClauses(bh, d->GetUser());
+        const auto& uo = d->GetUser();
+        if (x_IsHumanSTR(uo)) {
+            return (
+                uo.HasField("Bracketed record seq.", "") ?
+                    x_GetHumanSTRv2FeatureClauses(bh, uo) :
+                    x_GetHumanSTRFeatureClauses(bh, uo)
+            );
         }
         ++d;
     }
@@ -734,12 +750,12 @@ string OrganelleByGenome(unsigned int genome_val)
 
 static unsigned int s_GetProductFlagFromCDSProductNames (CBioseq_Handle bh)
 {
-	unsigned int product_flag = CBioSource::eGenome_unknown;
-	string::size_type pos;
+    unsigned int product_flag = CBioSource::eGenome_unknown;
+    string::size_type pos;
 
-	SAnnotSelector sel(CSeqFeatData::eSubtype_cdregion);
+    SAnnotSelector sel(CSeqFeatData::eSubtype_cdregion);
     CFeat_CI feat_ci(bh, sel);
-	while (feat_ci && product_flag == CBioSource::eGenome_unknown) {
+    while (feat_ci && product_flag == CBioSource::eGenome_unknown) {
         if (feat_ci->IsSetProduct()) {
             string label;
             CConstRef<CSeq_feat> prot
@@ -772,8 +788,8 @@ static unsigned int s_GetProductFlagFromCDSProductNames (CBioseq_Handle bh)
                 }
             }
         }
-		++feat_ci;
-	}
+        ++feat_ci;
+    }
     return product_flag;
 }
 
@@ -782,15 +798,15 @@ string CAutoDef::x_GetFeatureClauseProductEnding(const string& feature_clauses,
                                                  CBioseq_Handle bh)
 {
     bool pluralize = false;
-	unsigned int product_flag_to_use;
+    unsigned int product_flag_to_use;
     unsigned int nuclear_copy_flag = CBioSource::eGenome_unknown;
     
-	if (m_Options.GetSpecifyNuclearProduct()) {
-	    product_flag_to_use = s_GetProductFlagFromCDSProductNames (bh);
-	} else {
-		product_flag_to_use = m_Options.GetProductFlag();
+    if (m_Options.GetSpecifyNuclearProduct()) {
+        product_flag_to_use = s_GetProductFlagFromCDSProductNames (bh);
+    } else {
+        product_flag_to_use = m_Options.GetProductFlag();
         nuclear_copy_flag = m_Options.GetNuclearCopyFlag();
-	}
+    }
     if (NStr::Find(feature_clauses, "genes") != NCBI_NS_STD::string::npos) {
         pluralize = true;
     } else {
@@ -881,7 +897,7 @@ string CAutoDef::x_GetNonFeatureListEnding()
             end = " sequence.";
             break;
         case CAutoDefOptions::eWholeGenomeShotgunSequence:
-            end = " whole genome shotgun sequence.";
+            end = ", whole genome shotgun sequence.";
             break;
         default:
             break;
@@ -955,8 +971,38 @@ string CAutoDef::x_GetOneNonFeatureClause(CBioseq_Handle bh, unsigned int genome
 string CAutoDef::GetOneFeatureClauseList(CBioseq_Handle bh, unsigned int genome_val)
 {
     string feature_clauses;
-    if (m_Options.GetFeatureListType() == CAutoDefOptions::eListAllFeatures ||
-        (IsBioseqmRNA(bh) && IsInGenProdSet(bh))) {
+    bool listAllFeatures = (m_Options.GetFeatureListType() == CAutoDefOptions::eListAllFeatures);
+    if (listAllFeatures) {
+        int numGenes = 0;
+        int numCDSs = 0;
+        CSeq_annot_CI annot_ci(bh);
+        for (; annot_ci; ++annot_ci) {
+            const CSeq_annot_Handle& annt = *annot_ci;
+            CConstRef<CSeq_annot> pAnnot = annt.GetCompleteSeq_annot();
+            const CSeq_annot& antx = *pAnnot;
+            FOR_EACH_SEQFEAT_ON_SEQANNOT (feat_it, antx) {
+                const CSeq_feat& sft = **feat_it;
+                const CSeqFeatData& data = sft.GetData();
+                CSeqFeatData::ESubtype subtype = data.GetSubtype();
+                if (subtype == CSeqFeatData::eSubtype_gene) {
+                    numGenes++;
+                } else if (subtype == CSeqFeatData::eSubtype_cdregion) {
+                    numCDSs++;
+                }
+            }
+        }
+        if (numGenes + numCDSs > 100) {
+            // too many features will drastically slow down performance, bypass (RW-1578)
+            const string& custom = m_Options.GetCustomFeatureClause();
+            if (!NStr::IsBlank(custom)) {
+                return custom;
+            }
+            feature_clauses = x_GetOneNonFeatureClause(bh, genome_val);
+            return feature_clauses;
+        }
+    }
+    
+    if (listAllFeatures || (IsBioseqmRNA(bh) && IsInGenProdSet(bh))) {
         feature_clauses = x_GetFeatureClauses(bh);
         if (NStr::IsBlank(feature_clauses)) {
             feature_clauses = x_GetOneNonFeatureClause(bh, genome_val);
@@ -1011,8 +1057,45 @@ string CAutoDef::GetKeywordPrefix(CBioseq_Handle bh)
 }
 
 
-string CAutoDef::GetOneDefLine(CAutoDefModifierCombo *mod_combo, const CBioseq_Handle& bh)
+std::string x_trim(const std::string &s)
 {
+   auto start = s.begin();
+   while (start != s.end() && std::isspace(*start)) {
+       start++;
+   }
+
+   auto end = s.end();
+   do {
+       end--;
+   } while (std::distance(start, end) > 0 && std::isspace(*end));
+
+   return std::string(start, end + 1);
+}
+
+static bool s_IsBioseqGenomic(CBioseq_Handle bsh)
+{
+    for (CSeqdesc_CI desc(bsh, CSeqdesc::e_Molinfo); desc; ++desc) {
+        if (desc->GetMolinfo().CanGetBiomol() && desc->GetMolinfo().GetBiomol() == CMolInfo::eBiomol_genomic) {
+            return true;
+        }
+    }
+    return false;
+}
+
+static bool s_IsRefSeq(CBioseq_Handle bsh)
+{
+    for (CSeq_id_Handle sid : bsh.GetId()) {
+        if (sid.Which() == NCBI_SEQID(Other)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+string CAutoDef::GetOneDefLine(CAutoDefModifierCombo *mod_combo, const CBioseq_Handle& bh, CRef<feature::CFeatTree> featTree)
+{
+    m_Feat_Tree = featTree;
+
     // for protein sequences, use sequence::GetTitle
     if (bh.CanGetInst() && bh.GetInst().CanGetMol() && bh.GetInst().GetMol() == CSeq_inst::eMol_aa) {
         return sequence::CDeflineGenerator()
@@ -1043,7 +1126,20 @@ string CAutoDef::GetOneDefLine(CAutoDefModifierCombo *mod_combo, const CBioseq_H
 
     string keyword = GetKeywordPrefix(bh);
 
-    return keyword + org_desc + feature_clauses;
+    if (!NStr::IsBlank(feature_clauses)) {
+        string key_org = x_trim(keyword + org_desc);
+        feature_clauses = x_trim(feature_clauses);
+        if (NStr::StartsWith(feature_clauses, ",")) {
+            return keyword + org_desc + feature_clauses;
+        }
+        if (genome_val == CBioSource::eGenome_chromosome &&
+            s_IsBioseqGenomic(bh) && s_IsRefSeq(bh) &&
+            !NStr::IsBlank(m_Options.GetCustomFeatureClause())) {
+            return keyword + org_desc + ", " + feature_clauses;
+        }
+        return keyword + org_desc + " " + feature_clauses;
+    }
+    return keyword + org_desc;
 }
 
 
@@ -1102,6 +1198,11 @@ void CAutoDef::SetOptionsObject(const CUser_object& user)
     m_Options.InitFromUserObject(user);
 }
 
+void CAutoDef::SetOptions(const CAutoDefModifierCombo& mod_combo)
+{
+    mod_combo.InitOptions(m_Options);
+}
+
 
 //starting here, remove when separating autodef from taxonomy options
 CConstRef<CUser_object> s_GetOptionsForSet(CBioseq_set_Handle set)
@@ -1211,6 +1312,7 @@ string CAutoDef::x_GetHumanSTRFeatureClauses(CBioseq_Handle bh, const CUser_obje
     string locus_name;
     string allele;
     string repeat;
+    string flanking;
     string assay;
 
     if (comment.IsSetData()) {
@@ -1224,6 +1326,8 @@ string CAutoDef::x_GetHumanSTRFeatureClauses(CBioseq_Handle bh, const CUser_obje
                     allele = (*it)->GetData().GetStr();
                 } else if (NStr::EqualNocase(label, "Bracketed repeat")) {
                     repeat = (*it)->GetData().GetStr();
+                } else if (NStr::EqualNocase(label, "Flanking region")) {
+                    flanking = (*it)->GetData().GetStr();
                 } else if (NStr::EqualNocase(label, "Sequencing assay code")) {
                     assay = (*it)->GetData().GetStr();
                 }
@@ -1248,6 +1352,9 @@ string CAutoDef::x_GetHumanSTRFeatureClauses(CBioseq_Handle bh, const CUser_obje
         }
         ++f;
     }
+    if (flanking != "") {
+        clause += " " + flanking;
+    }
     if (assay != "") {
         clause += " " + assay;
     }
@@ -1256,6 +1363,45 @@ string CAutoDef::x_GetHumanSTRFeatureClauses(CBioseq_Handle bh, const CUser_obje
 }
 
 
+string CAutoDef::x_GetHumanSTRv2FeatureClauses(CBioseq_Handle bh, const CUser_object& comment)
+{
+    string locusName;
+    string lengthBasedAllele;
+    string bracketedRecordSeq;
+    string assayCode;
+
+    if (comment.IsSetData()) {
+        ITERATE(CUser_object::TData, it, comment.GetData()) {
+            if ((*it)->IsSetData() && (*it)->GetData().IsStr() &&
+                (*it)->IsSetLabel() && (*it)->GetLabel().IsStr()) {
+
+                const string& label = (*it)->GetLabel().GetStr();
+                if (NStr::EqualNocase(label, "STR locus name")) {
+                    locusName = (*it)->GetData().GetStr();
+                }
+                else if (NStr::EqualNocase(label, "Length-based allele")) {
+                    lengthBasedAllele = (*it)->GetData().GetStr();
+                }
+                else if (NStr::EqualNocase(label, "Bracketed record seq.")) {
+                    bracketedRecordSeq = (*it)->GetData().GetStr();
+                }
+                else if (NStr::EqualNocase(label, "Sequencing assay code")) {
+                    assayCode = (*it)->GetData().GetStr();
+                }
+            }
+        }
+    }
+
+    string clause = "microsatellite " + locusName + " " + lengthBasedAllele + 
+        " " + bracketedRecordSeq;
+    if (assayCode != "") {
+        clause += " " + assayCode;
+    }
+    clause += " sequence";
+    return clause;
+}
+
+
 bool s_ChooseModInModList(bool is_org_mod, int subtype, bool require_all, CAutoDefSourceDescription::TAvailableModifierVector& modifiers)
 {
     bool rval = false;
