@@ -1,4 +1,4 @@
-/*  $Id: objutil.cpp 593093 2019-09-11 20:18:39Z dondosha $
+/*  $Id: objutil.cpp 660112 2022-12-09 21:22:42Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1660,7 +1660,7 @@ NCBI_XOBJEDIT_EXPORT const char* strLinkBaseGeneOntology =
 NCBI_XOBJEDIT_EXPORT const char* strLinkBaseGeneOntologyRef =
     "http://www.geneontology.org/cgi-bin/references.cgi#GO_REF:"; // not government site
 NCBI_XOBJEDIT_EXPORT const char* strLinkBaseUSPTO =
-    "http://patft.uspto.gov/netacgi/nph-Parser?patentnumber=";
+    "https://ppubs.uspto.gov/pubwebapp/external.html?q=";
 NCBI_XOBJEDIT_EXPORT const char* strLinkBaseUniProt =
     "https://www.uniprot.org/uniprot/";
 
