@@ -1,4 +1,4 @@
-/*  $Id: showalign_unit_test.cpp 631562 2021-05-19 13:53:13Z ivanov $
+/*  $Id: showalign_unit_test.cpp 667737 2023-05-19 19:23:15Z kans $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -140,12 +140,12 @@ BOOST_AUTO_TEST_CASE(TestAccConversionExt)
     ds.SetSeqLocChar(CDisplaySeqalign::eLowerCase);
     CNcbiOstrstream output_stream;
     ds.DisplaySeqalign(output_stream);
-    string output = CNcbiOstrstreamToString(output_stream);        
+    string output = CNcbiOstrstreamToString(output_stream);
     BOOST_REQUIRE(output.find("NP_001263315.1") != NPOS);
     BOOST_REQUIRE(output.find("XP_015137661.1") != NPOS);
     BOOST_REQUIRE(output.find("AUD54591.1") != NPOS);
     BOOST_REQUIRE(output.find("BAM13279.1") != NPOS);
-    BOOST_REQUIRE(output.find("P01014.1") != NPOS);
+    BOOST_REQUIRE(output.find("P01013.1") != NPOS);
     scope->GetObjectManager().RevokeAllDataLoaders();                
 }
 
@@ -188,7 +188,8 @@ bool TestSimpleAlignment(CBlastOM::ELocation location, bool long_seqids)
         BOOST_REQUIRE(output.find(">gi|1788470|gb|AE000304.1|AE000304 ") != NPOS);
     }
     BOOST_REQUIRE(output.find("Escherichia coli K-12 MG1655 section 9 of 400 of ") != NPOS ||
-                  output.find("Escherichia coli K12 MG1655 section 9 of 400 of ") != NPOS);
+                  output.find("Escherichia coli K12 MG1655 section 9 of 400 of ") != NPOS ||
+                  output.find("Escherichia coli K12 substr. MG1655") != NPOS);
     BOOST_REQUIRE(output.find("Sbjct  259   GCCTGATGCGACGCTGGCGCGTCTTATCAGGCCTAC  294") != NPOS);
     BOOST_REQUIRE(output.find("Length=11852") != NPOS);
     BOOST_REQUIRE(output.find("Query  5636  GTAGG-CAGGATAAGGCGTTCACGCCGCATCCGGCA  5670") != NPOS);
@@ -225,4 +226,42 @@ BOOST_AUTO_TEST_CASE(TestSimpleAlignment_RmtBlastDBLoader)
     BOOST_REQUIRE(TestSimpleAlignment(CBlastOM::eRemote, true));
 }
 
+#ifndef NCBI_INT4_GI
+BOOST_AUTO_TEST_CASE(TestLargeGi)
+{
+    const string seqAlignFileName_in = "data/large_gi.asn";
+    CRef<CSeq_annot> san(new CSeq_annot);
+
+    ifstream in(seqAlignFileName_in.c_str());
+    in >> MSerial_AsnText >> *san;
+    in.close();
+
+    CRef<CSeq_align_set> fileSeqAlignSet(new CSeq_align_set);
+    fileSeqAlignSet->Set() = san->GetData().GetAlign();
+
+    const string kDbName("data/nucl_32b_gi");
+    const CBlastDbDataLoader::EDbType kDbType(CBlastDbDataLoader::eNucleotide);
+    TestUtil::CBlastOM tmp_data_loader(kDbName, kDbType, CBlastOM::eLocal);
+    CRef<CScope> scope = tmp_data_loader.NewScope();
+
+    CDisplaySeqalign ds(*fileSeqAlignSet, *scope);
+    ds.SetDbName(kDbName);
+    ds.SetDbType((kDbType == CBlastDbDataLoader::eNucleotide));
+    int flags = CDisplaySeqalign::eShowBlastInfo |
+        CDisplaySeqalign::eShowBlastStyleId |
+        CDisplaySeqalign::eShowGi;
+    ds.SetAlignOption(flags);
+    ds.SetSeqLocChar(CDisplaySeqalign::eLowerCase);
+    CNcbiOstrstream output_stream;
+    ds.DisplaySeqalign(output_stream);
+    string output = CNcbiOstrstreamToString(output_stream);
+    BOOST_REQUIRE(output.find("2271278971") != NPOS);
+    BOOST_REQUIRE(output.find("698036805") != NPOS);
+    BOOST_REQUIRE(output.find("4294967295") != NPOS);
+    BOOST_REQUIRE(output.find("2966748774") != NPOS);
+    scope->GetObjectManager().RevokeAllDataLoaders();
+}
+#endif
+
+
 BOOST_AUTO_TEST_SUITE_END()
