@@ -1,4 +1,4 @@
-/*  $Id: tabularinof_unit_test.cpp 631540 2021-05-19 13:51:06Z ivanov $
+/*  $Id: tabularinof_unit_test.cpp 676159 2023-12-11 14:55:46Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -214,7 +214,7 @@ BOOST_AUTO_TEST_CASE(TaxonomyOutput) {
 
     const list<CRef<CSeq_align> > & seqalign_list = sa->Get();
 
-    const string kDbName("nr");
+    const string kDbName("data/tabular_db");
     const CBlastDbDataLoader::EDbType kDbType(CBlastDbDataLoader::eProtein);
     TestUtil::CBlastOM tmp_data_loader(kDbName, kDbType, CBlastOM::eLocal);
     CRef<CScope> scope = tmp_data_loader.NewScope();
@@ -232,7 +232,7 @@ BOOST_AUTO_TEST_CASE(TaxonomyOutput) {
        ctab2.Print();
     }
     const string ref[5] = {
-    		"XP_003443710	XP_003443710	8128;47969	Oreochromis niloticus;Oreochromis aureus	Nile tilapia;Oreochromis aureus	bony fishes	Eukaryota",
+    		"XP_003443710	XP_003443710	8128;9669;47969	Oreochromis niloticus;Mustela putorius furo;Oreochromis aureus	Nile tilapia;domestic ferret;Oreochromis aureus	bony fishes;carnivores	Eukaryota",
     		"XP_003443710	XP_004568121	8153;8154;106582;303518	Haplochromis burtoni;Astatotilapia calliptera;Maylandia zebra;Pundamilia nyererei	Burton's mouthbrooder;eastern happy;zebra mbuna;Pundamilia nyererei	bony fishes	Eukaryota",
     		"XP_003443710	XP_030605880	63155	Archocentrus centrarchus	flier cichlid	bony fishes	Eukaryota",
     		"XP_003443710	XP_006794996	32507	Neolamprologus brichardi	Neolamprologus brichardi	bony fishes	Eukaryota",
@@ -271,7 +271,7 @@ BOOST_AUTO_TEST_CASE(SubjectTitlesOutput) {
 
     const list<CRef<CSeq_align> > & seqalign_list = sa->Get();
 
-    const string kDbName("nr");
+    const string kDbName("data/tabular_db");
     const CBlastDbDataLoader::EDbType kDbType(CBlastDbDataLoader::eProtein);
     TestUtil::CBlastOM tmp_data_loader(kDbName, kDbType, CBlastOM::eLocal);
     CRef<CScope> scope = tmp_data_loader.NewScope();
@@ -317,8 +317,8 @@ BOOST_AUTO_TEST_CASE(SubjectTitlesOutput) {
         const string ref[5] = {
         		"O19910,NP_045082,phycocyanin alpha subunit [Cyanidium caldarium]<>RecName: Full=C-phycocyanin alpha subunit [Cyanidium caldarium]",
         		"O19910,AAB01593,cpcA [Cyanidium caldarium]",
-        		"O19910,P00306,RecName: Full=C-phycocyanin alpha chain [Galdieria sulphuraria]<>C-phycocyanin alpha chain [validated] - red alga (Cyanidium caldarium) [Cyanidium caldarium]<>phycocyanin alpha subunits [Cyanidium caldarium]<>Structure Of Phycocyanin From Cyanidium Caldarium At 1.65a Resolution [Cyanidium caldarium]",
-        		"O19910,YP_009051179,phycocyanin alpha subunit [Galdieria sulphuraria]<>[pt] C-phycocyanin alpha chain [Galdieria sulphuraria]<>Crystal Structure of C-Phycocyanin from Galdieria sulphuraria [Galdieria sulphuraria]<>The high resolution structure of C-Phycocyanin from Galdieria Sulphuraria [Galdieria sulphuraria]<>phycocyanin alpha subunit [Galdieria sulphuraria]<>[pt] C-phycocyanin alpha chain [Galdieria sulphuraria]",
+        		"O19910,P00306,RecName: Full=C-phycocyanin alpha chain [Galdieria sulphuraria]<>C-phycocyanin alpha chain [validated] - red alga (Cyanidium caldarium) [Cyanidium caldarium]<>phycocyanin alpha subunits [Cyanidium caldarium]<>Chain A, Phycocyanin [Cyanidium caldarium]",
+        		"O19910,YP_009051179,phycocyanin alpha subunit [Galdieria sulphuraria]<>[pt] C-phycocyanin alpha chain [Galdieria sulphuraria]<>Chain A, C-phycocyanin alpha chain [unidentified]<>Chain A, C-phycocyanin alpha chain [Galdieria sulphuraria]<>phycocyanin alpha subunit [Galdieria sulphuraria]<>[pt] C-phycocyanin alpha chain [Galdieria sulphuraria]",
         		"O19910,YP_009297463,phycocyanin alpha subunit [Erythrotrichia carnea]<>phycocyanin alpha subunit [Erythrotrichia carnea]"};
     	for(unsigned int i=0; i < 5; i++) {
             CNcbiOstrstream oss;
@@ -343,7 +343,7 @@ BOOST_AUTO_TEST_CASE(ExtractCorrectSeqIdWhenThereIsSourceObjectInDescription) {
 
     const CSeq_align_set::Tdata& seqalign_list = ba->GetResults().GetAlignments().Get();
 
-    const string kDbName("nt.14");
+    const string kDbName("nt.014");
     const CBlastDbDataLoader::EDbType kDbType(CBlastDbDataLoader::eNucleotide);
     TestUtil::CBlastOM tmp_data_loader(kDbName, kDbType, CBlastOM::eLocal);
     CRef<CScope> scope = tmp_data_loader.NewScope();
