@@ -1,4 +1,4 @@
-/*  $Id: blastdb_dataextract.cpp 610974 2020-06-26 12:59:33Z grichenk $
+/*  $Id: blastdb_dataextract.cpp 683929 2024-06-10 12:07:52Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -574,11 +574,11 @@ string CBlastDBExtractor::ExtractSeqData() {
         x_ExtractMaskingData(masked_ranges, m_FiltAlgoId);
         ITERATE(CSeqDB::TSequenceRanges, mask, masked_ranges) {
             transform(&seq[mask->first], &seq[mask->second],
-                      &seq[mask->first], (int (*)(int))tolower);
+                      &seq[mask->first], (int (*)(int))::tolower);
         }
         if (m_Strand == eNa_strand_minus) {
             CSeqManip::ReverseComplement(seq, CSeqUtil::e_Iupacna,
-                                         0, seq.size());
+                                         0, static_cast<ncbi::TSeqPos>(seq.size()));
         }
     } catch (CSeqDBException& e) {
         //FIXME: change the enumeration when it's availble
@@ -598,7 +598,7 @@ string CBlastDBExtractor::ExtractSeqLen() {
 string CBlastDBExtractor::ExtractHash() {
     string seq;
     m_BlastDb.GetSequenceAsString(m_Oid, seq);
-    return NStr::IntToString(CBlastSeqUtil::GetSeqHash(seq.c_str(), seq.size()));
+    return NStr::IntToString(CBlastSeqUtil::GetSeqHash(seq.c_str(), static_cast<ncbi::TSeqPos>(seq.size())));
 }
 
 #define CTRL_A "\001"
@@ -1050,7 +1050,7 @@ void CBlastSeqUtil::ApplySeqMask(string & seq, const CSeqDB::TSequenceRanges & m
 	if(r.Empty()) {
 		ITERATE(CSeqDB::TSequenceRanges, itr, masks) {
 			transform(&seq[itr->first], &seq[itr->second],
-			                      &seq[itr->first], (int (*)(int))tolower);
+			                      &seq[itr->first], (int (*)(int))::tolower);
 		}
 	}
 	else {
@@ -1063,7 +1063,7 @@ void CBlastSeqUtil::ApplySeqMask(string & seq, const CSeqDB::TSequenceRanges & m
 			TSeqRange  tmp = r.IntersectionWith(mask);
 			if(!tmp.Empty()) {
 				transform(&seq[tmp.GetFrom() -r_from], &seq[tmp.GetToOpen() - r_from],
-		                  &seq[tmp.GetFrom() -r_from], (int (*)(int))tolower);
+		                  &seq[tmp.GetFrom() -r_from], (int (*)(int))::tolower);
 			}
 		}
 	}
@@ -1071,7 +1071,7 @@ void CBlastSeqUtil::ApplySeqMask(string & seq, const CSeqDB::TSequenceRanges & m
 
 void CBlastSeqUtil::GetReverseStrandSeq(string & seq)
 {
-	CSeqManip::ReverseComplement(seq, CSeqUtil::e_Iupacna, 0, seq.size());
+	CSeqManip::ReverseComplement(seq, CSeqUtil::e_Iupacna, 0, static_cast<ncbi::TSeqPos>(seq.size()));
 }
 
 string CBlastSeqUtil::GetMasksString(const CSeqDB::TSequenceRanges & masks)
