@@ -1,4 +1,4 @@
-/*  $Id: seqdb.cpp 611131 2020-06-29 18:42:01Z grichenk $
+/*  $Id: seqdb.cpp 683929 2024-06-10 12:07:52Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -264,6 +264,11 @@ void CSeqDB::GetDBTaxIds(set<TTaxId> & tax_ids) const
      m_Impl->GetDBTaxIds(tax_ids);
 }
 
+void CSeqDB::GetTaxIdsForOids(const vector<blastdb::TOid> & oids, set<TTaxId> & tax_ids) const
+{
+	m_Impl->GetTaxIdsForOids(oids, tax_ids);
+}
+
 // This could become the primary constructor for SeqDB, and those
 // taking positive and negative lists could be deprecated.  This
 // implies refactoring of code using SeqDB, addition of the third
@@ -1467,6 +1472,7 @@ Int8 CSeqDB::GetDiskUsage() const
     vector<string> extn;
     const bool is_protein(GetSequenceType() == CSeqDB::eProtein);
     SeqDB_GetFileExtensions(is_protein, extn, GetBlastDbVersion());
+    string blastdb_dirname;
 
     ITERATE(vector<string>, path, paths) {
         ITERATE(vector<string>, ext, extn) {
@@ -1475,6 +1481,37 @@ Int8 CSeqDB::GetDiskUsage() const
                 Int8 length = file.GetLength();
                 if (length != -1) {
                     retval += length;
+                    LOG_POST(Trace << "File " << file.GetPath() << " " << length << " bytes");
+                    blastdb_dirname = file.GetDir();
+                } else {
+                    ERR_POST(Error << "Error retrieving file size for "
+                                   << file.GetPath());
+                }
+            }
+        }
+    }
+    // For multi-volume databases, take into account files that apply to the
+    // entire BLASTDB
+    if (paths.size() > 1) {
+        _ASSERT( !blastdb_dirname.empty() );
+        auto dbname = GetDBNameList();
+        vector<string> dblist;
+        NStr::Split(dbname, " ", dblist, NStr::fSplit_Tokenize);
+        if (dblist.size() > 1) {
+            CNcbiOstrstream oss;
+            oss << "Cannot compute disk usage for multiple BLASTDBs (i.e.: '"
+                << dbname << "') at once. Please try again using one BLASTDB "
+                << "at a time.";
+            NCBI_THROW(CSeqDBException, eArgErr, CNcbiOstrstreamToString(oss));
+        }
+
+        for (const auto& ext: extn) {
+            CFile file(CDirEntry::MakePath(blastdb_dirname, dbname, ext));
+            if (file.Exists()) {
+                Int8 length = file.GetLength();
+                if (length != -1) {
+                    retval += length;
+                    LOG_POST(Trace << "File " << file.GetPath() << " " << length << " bytes");
                 } else {
                     ERR_POST(Error << "Error retrieving file size for "
                                    << file.GetPath());
@@ -1485,6 +1522,7 @@ Int8 CSeqDB::GetDiskUsage() const
     return retval;
 }
 
+
 CSeqDB::ESeqType
 ParseMoleculeTypeString(const string& s)
 {
@@ -1568,5 +1606,160 @@ EBlastDbVersion CSeqDB::GetBlastDbVersion() const
 	 return m_Impl->GetBlastDbVersion();
 }
 
+
+void CSeqDB::x_GetDBFilesMetaData(Int8 & disk_bytes, Int8 & cached_bytes, vector<string> & db_files, const string & user_path) const
+{
+    vector<string> paths;
+    vector<string> alias;
+    m_Impl->FindVolumePaths(paths, alias, true);
+    _ASSERT( !paths.empty() );
+
+    db_files.clear();
+    cached_bytes = 0;
+    disk_bytes = 0;
+
+    ITERATE(vector<string>, a, alias) {
+   		CFile af(*a);
+   		if (af.Exists()) {
+   			string fn = user_path + af.GetName();
+   			db_files.push_back(fn);
+   			Int8 afl = af.GetLength();
+   			if (afl != -1) {
+   				disk_bytes += afl;
+   			} else {
+   				ERR_POST(Error << "Error retrieving file size for " << af.GetPath());
+   			}
+   		}
+    }
+
+    vector<string> extn;
+    const bool is_protein(GetSequenceType() == CSeqDB::eProtein);
+    SeqDB_GetFileExtensions(is_protein, extn, EBlastDbVersion::eBDB_Version4);
+
+    const string kExtnMol(1, is_protein ? 'p' : 'n');
+    const string index_ext = kExtnMol + "in";
+    const string seq_ext = kExtnMol + "sq";
+
+    ITERATE(vector<string>, path, paths) {
+        ITERATE(vector<string>, ext, extn) {
+            CFile file(*path + "." + *ext);
+            if (file.Exists()) {
+            	string f = user_path + file.GetName();
+            	db_files.push_back(f);
+                Int8 length = file.GetLength();
+                if (length != -1) {
+                    disk_bytes += length;
+                    if((*ext == index_ext) || (*ext == seq_ext)) {
+                    	cached_bytes += length;
+                    }
+                } else {
+                    ERR_POST(Error << "Error retrieving file size for "
+                                   << file.GetPath());
+                }
+            }
+        }
+    }
+
+    if (GetBlastDbVersion() == EBlastDbVersion::eBDB_Version5) {
+    	vector<string> lmdb_list;
+    	m_Impl->GetLMDBFileNames(lmdb_list);
+
+     	ITERATE(vector<string>, l, lmdb_list) {
+     		CFile file(*l);
+     		if (file.Exists()) {
+     		   string f = user_path + file.GetName();
+      	 	   db_files.push_back(f);
+      	 	   Int8 length = file.GetLength();
+       	   	   if (length != -1) {
+       	   		   disk_bytes += length;
+       	   	   } else {
+       	   		   ERR_POST(Error << "Error retrieving file size for " << file.GetPath());
+       	   	   }
+       	   	   static const char * v5_exts[]={"os", "ot", "tf", "to", NULL};
+       	   	   for(const char ** p=v5_exts; *p != NULL; p++) {
+       	    		CFile v(file.GetDir() + file.GetBase() + "." + kExtnMol + (*p));
+       	    		if (v.Exists()) {
+       	    			string vf = user_path + v.GetName();
+       	    			db_files.push_back(vf);
+       	    			Int8 vl = v.GetLength();
+       	    			if (vl != -1) {
+       	    				disk_bytes += vl;
+       	    			} else {
+       	    				ERR_POST(Error << "Error retrieving file size for " << v.GetPath());
+       	    			}
+       	    		}
+       	   	   }
+     	   }
+    	}
+    }
+
+    // FIXME: increase the version for this new file type
+    //string ext;
+    //SeqDB_GetMetadataFileExtension(is_protein, ext);
+    //const CFile dbfile(paths.front());
+    //db_files.push_back(user_path + dbfile.GetBase() + "." + ext);
+
+    sort(db_files.begin(), db_files.end());
+}
+
+CRef<CBlast_db_metadata> CSeqDB::GetDBMetaData(string user_path)
+{
+	CRef<CBlast_db_metadata> m (new CBlast_db_metadata());
+	int num_seqs = 0;
+	Uint8 total_length = 0;
+
+	GetTotals(CSeqDB::eFilteredAll, &num_seqs, &total_length, true);
+	vector<string> dblist;
+    NStr::Split(GetDBNameList(), " ", dblist, NStr::fSplit_Tokenize);
+    NON_CONST_ITERATE(vector<string>, itr, dblist) {
+        size_t off = (*itr).find_last_of(CFile::GetPathSeparator());
+        if (off != string::npos ) {
+        	(*itr).erase(0, off+1);
+        }
+    }
+
+	string dbnames = NStr::Join(dblist, " ");
+	m->SetDbname(dbnames);
+
+    m->SetDbtype(GetSequenceType() == CSeqDB::eProtein ? "Protein" : "Nucleotide" );
+	m->SetDb_version(GetBlastDbVersion() == EBlastDbVersion::eBDB_Version5?5:4);
+	m->SetDescription(GetTitle());
+	m->SetNumber_of_letters(total_length);
+	m->SetNumber_of_sequences(num_seqs);
+
+	CTimeFormat timeFmt = CTimeFormat::GetPredefined(CTimeFormat::eISO8601_DateTimeSec);
+	string fmt = "b d, Y  H:m P";
+	CTime date(GetDate(), fmt);
+	m->SetLast_updated(date.AsString(timeFmt));
+
+	Int8 disk_bytes(0), cached_bytes(0);
+	x_GetDBFilesMetaData(disk_bytes, cached_bytes, m->SetFiles(), user_path);
+	m->SetBytes_total(disk_bytes);
+	m->SetBytes_to_cache(cached_bytes);
+
+	m->SetNumber_of_volumes(m_Impl->GetNumOfVols());
+
+	if (GetBlastDbVersion() == EBlastDbVersion::eBDB_Version5) {
+		set<TTaxId> tax_ids;
+		GetDBTaxIds(tax_ids);
+		if((tax_ids.size() > 1) || ((tax_ids.size() == 1) && (0 != *tax_ids.begin()))){
+			m->SetNumber_of_taxids(static_cast<int>(tax_ids.size()));
+		}
+	}
+	return m;
+}
+
+void CSeqDB::GetTaxIdsForAccession(const string & accs, vector<TTaxId> & taxids)
+{
+	CSeq_id seqid(accs, CSeq_id::fParse_RawText | CSeq_id::fParse_ValidLocal);
+	m_Impl->GetTaxIdsForSeqId(seqid, taxids);
+}
+
+void CSeqDB::GetTaxIdsForSeqId(const CSeq_id & seq_id, vector<TTaxId> & taxids)
+{
+	m_Impl->GetTaxIdsForSeqId(seq_id, taxids);
+}
+
+
 END_NCBI_SCOPE
 
