@@ -1,4 +1,4 @@
-/*  $Id: seqdbalias.cpp 578711 2019-01-24 15:40:06Z fongah2 $
+/*  $Id: seqdbalias.cpp 651939 2022-06-28 12:33:29Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -68,7 +68,8 @@ CSeqDBAliasFile::CSeqDBAliasFile(CSeqDBAtlas     & atlas,
       m_MembBit          (-1),
       m_HasTitle         (false),
       m_NeedTotalsScan   (-1),
-      m_HasFilters       (0)
+      m_HasFilters       (0),
+      m_OidMaskType      (0)
 {
     if (name_list.size() && prot_nucl != '-') {
         m_Node.Reset(new CSeqDBAliasNode(atlas,
@@ -181,8 +182,7 @@ bool
 CSeqDBAliasSets::x_FindBlastDBPath(const string   & dbname,
                                    char             dbtype,
                                    bool             exact,
-                                   string         & resolved,
-                                   CSeqDBLockHold & locked)
+                                   string         & resolved)
 {
     map<string,string>::iterator i = m_PathLookup.find(dbname);
     
@@ -191,8 +191,7 @@ CSeqDBAliasSets::x_FindBlastDBPath(const string   & dbname,
                                          dbtype,
                                          0,
                                          exact,
-                                         m_Atlas,
-                                         locked);
+                                         m_Atlas);
         
         m_PathLookup[dbname] = resolved;
     } else {
@@ -215,7 +214,7 @@ CSeqDBAliasSets::FindAliasPath(const CSeqDB_Path & dbpath,
     
     CSeqDB_Path resolved_aset;
     
-    if (! FindBlastDBPath(aset_path, resolved_aset, locked)) {
+    if (! FindBlastDBPath(aset_path, resolved_aset)) {
         return false;
     }
     
@@ -269,7 +268,7 @@ void CSeqDBAliasNode::x_ResolveNames(char prot_nucl, CSeqDBLockHold & locked)
             CSeqDB_BasePath resolved_bp;
             
             // search for X/base.nal/nin
-            if (m_AliasSets.FindBlastDBPath(base, prot_nucl, resolved_bp, locked)) {
+            if (m_AliasSets.FindBlastDBPath(base, prot_nucl, resolved_bp)) {
                 resolved_path = CSeqDB_Path(resolved_bp, prot_nucl, 'a', 'l');
             }
         }
@@ -307,8 +306,7 @@ void CSeqDBAliasNode::x_ResolveNames(char prot_nucl, CSeqDBLockHold & locked)
                                   prot_nucl,
                                   & search_path,
                                   false,
-                                  m_Atlas,
-                                  locked);
+                                  m_Atlas);
             
             ostringstream oss;
             oss << "No alias or index file found for " << p_or_n
@@ -882,7 +880,7 @@ void CSeqDBAliasNode::x_ExpandAliases(const CSeqDB_BasePath & this_name,
         CSeqDB_BasePath result;
         CSeqDB_BasePath restart(m_DBList[i]);
         
-        if (m_AliasSets.FindBlastDBPath(restart, prot_nucl, result, locked)) {
+        if (m_AliasSets.FindBlastDBPath(restart, prot_nucl, result)) {
             // either alias or volume file exists for this entry
             CSeqDB_Path new_alias( result, prot_nucl, 'a', 'l' );
             CSeqDB_Path new_volume( result, prot_nucl, 'i', 'n' );
@@ -1416,6 +1414,60 @@ private:
     int m_Value;
 };
 
+/// Walker for oid mask type
+///
+/// This just searches alias files for the oid mask type if one is
+/// specified.
+
+class CSeqDB_OidMaskTypeWalker : public CSeqDB_AliasWalker {
+public:
+    /// Constructor
+    CSeqDB_OidMaskTypeWalker()
+    {
+        m_Value = 0;
+    }
+
+    /// This provides the alias file key used for this field.
+    virtual const char * GetFileKey() const
+    {
+        return "OID_MASK_TYPE";
+    }
+
+    /// Collect data from the volume
+    ///
+    /// If the MEMB_BIT field is not specified in an alias file, then
+    /// it is not needed.  This field is intended to allow filtration
+    /// of deflines by taxonomic category, which is only needed if an
+    /// alias file reduces the taxonomic scope.
+    virtual void Accumulate(const CSeqDBVol &)
+    {
+        // Volumes don't have this data, only alias files.
+    }
+
+    /// Collect data from an alias file
+    ///
+    /// If the MEMB_BIT field is specified in an alias file, it will
+    /// be used unmodified.  No attempt is made to combine or collect
+    /// bit values - currently, only one can be used at a time.
+    ///
+    /// @param value
+    ///   A database volume
+    virtual void AddString(const string & value)
+    {
+        m_Value = NStr::StringToUInt(value);
+    }
+
+    /// Returns the oid mask type.
+    int GetOidMaskType() const
+    {
+        return m_Value;
+    }
+
+private:
+    /// The oid mask type.
+    int m_Value;
+};
+
 
 /// Test for completeness of GI list alias file values.
 ///
@@ -1496,6 +1548,11 @@ public:
             return true;
         }
         
+        if (vars.find("OIDLIST") != vars.end()) {
+            m_NeedScan = true;
+            return true;
+        }
+
         // If none of those conditions is met, traversal proceeds.
         return false;
     }
@@ -1642,6 +1699,15 @@ bool CSeqDBAliasNode::NeedTotalsScan(const CSeqDBVolSet & volset) const
     return explore.NeedScan();
 }
 
+int CSeqDBAliasNode::GetOidMaskType(const CSeqDBVolSet & volset) const
+{
+    CSeqDB_OidMaskTypeWalker walk;
+    WalkNodes(& walk, volset);
+
+    return walk.GetOidMaskType();
+}
+
+
 
 void CSeqDBAliasNode::
 CompleteAliasFileValues(const CSeqDBVolSet & volset)
@@ -1826,6 +1892,14 @@ void CSeqDBAliasNode::BuildFilterTree(CSeqDB_FilterTree & ftree) const
     }
 }
 
+int CSeqDBAliasFile::GetOidMaskType(const CSeqDBVolSet & volset) const
+{
+    // Default is zero.
+    m_OidMaskType = m_Node->GetOidMaskType(volset);
+
+    return m_OidMaskType;
+}
+
 CRef<CSeqDB_FilterTree> CSeqDBAliasFile::GetFilterTree()
 {
     if (m_TopTree.Empty()) {
@@ -1882,6 +1956,7 @@ void CSeqDBAliasNode::ComputeMasks(bool & has_filters)
     TVarList::iterator l_oid_iter = m_Values.find(string("LAST_OID"));
     TVarList::iterator mbit_iter  = m_Values.find(string("MEMB_BIT"));
     TVarList::iterator taxid_iter  = m_Values.find(string("TAXIDLIST"));
+    TVarList::iterator oid_mask_type_iter  = m_Values.find(string("OID_MASK_TYPE"));
     
     if (! m_DBList.empty()) {
         if (oid_iter   != m_Values.end() ||
@@ -1923,8 +1998,17 @@ void CSeqDBAliasNode::ComputeMasks(bool & has_filters)
             if (oid_iter != m_Values.end()) {
                 CSeqDB_FileName lst(oid_iter->second);
                 CSeqDB_Path lst_path(m_DBPath, lst);
-                
-                CRef<TMask> mask(new TMask(TMask::eOidList, lst_path));
+                CFile oid_f(lst_path.GetPathS());
+                if (!oid_f.Exists() && m_VolNames.size() != 0) {
+                	CSeqDB_Path tmp(m_VolNames[0].FindDirName(), lst.GetFileNameSub());
+                	lst_path = tmp;
+                }
+
+                int oid_mask_type = 0;
+                if(oid_mask_type_iter != m_Values.end()) {
+                	oid_mask_type = NStr::StringToUInt(oid_mask_type_iter->second);
+                }
+                CRef<TMask> mask(new TMask(TMask::eOidList, lst_path, oid_mask_type));
                 m_NodeMasks.push_back(mask);
             }
             
