@@ -1,4 +1,4 @@
-/*  $Id: seqdbcommon.cpp 631511 2021-05-19 13:47:54Z ivanov $
+/*  $Id: seqdbcommon.cpp 672619 2023-09-12 12:55:53Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -96,7 +96,7 @@ CSeqDB_Substring SeqDB_RemoveExtn(CSeqDB_Substring s)
 
         if (extn[0] == '.'     &&
             (extn[1] == 'n' || extn[1] == 'p') &&
-            (extn2 == "al"  || extn2 == "in")) {
+            (extn2 == "al"  || extn2 == "in" || extn2 == "db")) {
             /*
             isalpha( s[slen-3] ) &&
             isalpha( s[slen-2] ) &&
@@ -507,7 +507,7 @@ void SeqDB_JoinDelim(string & a, const string & b, const string & delim)
 
 
 CSeqDBGiList::CSeqDBGiList()
-    : m_CurrentOrder(eNone)
+    : m_CurrentOrder(eNone), m_MaskOpts(0)
 {
 }
 
@@ -2666,6 +2666,14 @@ void SeqDB_GetLMDBFileExtensions(bool db_is_protein, vector<string>& extn)
 	}
 }
 
+
+void SeqDB_GetMetadataFileExtension(bool db_is_protein, string & extn)
+{
+	const string kExtnMol(1, db_is_protein ? 'p' : 'n');
+	extn = kExtnMol + "js";
+	return;
+}
+
 bool IsStringId(const CSeq_id & id)
 {
 	 switch(id.Which()) {
@@ -2694,5 +2702,18 @@ string GetBlastSeqIdString(const CSeq_id & seqid, bool version)
 	return seqid.GetSeqIdString(version);
 }
 
+const string SeqDB_GetOidMaskFileExt(bool db_is_protein, EOidMaskType t)
+{
+	switch (t) {
+		case EOidMaskType::fExcludeModel:
+			return  (db_is_protein ? "pxm":"nxm");
+		break;
+		default:
+			NCBI_THROW(CSeqDBException, eArgErr, "Invalid oid mask type.");
+		break;
+	}
+}
+
+
 END_NCBI_SCOPE
 
