@@ -1,4 +1,4 @@
-/*  $Id: seqdbimpl.cpp 631567 2021-05-19 13:53:46Z ivanov $
+/*  $Id: seqdbimpl.cpp 683929 2024-06-10 12:07:52Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -121,7 +121,6 @@ CSeqDBImpl::CSeqDBImpl(const string       & db_name_list,
 
             // This is a whole-database scan; it's always done in
             // approximate length mode.
-
             x_ScanTotals(true, & m_NumSeqs, & m_TotalLength,
                                & m_MaxLength, & m_MinLength, locked);
 
@@ -138,6 +137,9 @@ CSeqDBImpl::CSeqDBImpl(const string       & db_name_list,
         }
         m_NumSeqsStats     = x_GetNumSeqsStats();
         m_TotalLengthStats = x_GetTotalLengthStats();
+
+        LOG_POST(Info << "Num of Seqs: " << m_NumSeqs);
+        LOG_POST(Info << "Total Length: " << m_TotalLength);
     }
     catch(CSeqDBException & e) {
         m_UserGiList.Reset();
@@ -338,7 +340,7 @@ CSeqDBImpl::GetNextOIDChunk(int         & begin_chunk, // out
     if (m_NumThreads) {
         SSeqResBuffer * buffer = m_CachedSeqs[cacheID];
         x_FillSeqBuffer(buffer, begin_chunk);
-        end_chunk = begin_chunk + buffer->results.size();
+        end_chunk = begin_chunk + static_cast<int>(buffer->results.size());
     } else {
         end_chunk = begin_chunk + oid_size;
     }
@@ -1338,6 +1340,18 @@ void CSeqDBImpl::GetDBTaxIds(set<TTaxId> & tax_ids)
     return;
 }
 
+void CSeqDBImpl::GetTaxIdsForOids(const vector<blastdb::TOid> & oids, set<TTaxId> & tax_ids)
+{
+    if (m_LMDBSet.IsBlastDBVersion5()) {
+   		m_LMDBSet.GetTaxIdsForOids(oids, tax_ids);
+    }
+    else {
+    	NCBI_THROW(CSeqDBException, eArgErr,
+    			           "Taxonomy list is not supported in v4 BLAST db");
+    }
+
+}
+
 void CSeqDBImpl::AccessionsToOids(const vector<string>& accs, vector<blastdb::TOid>& oids)
 {
     CHECK_MARKER();
@@ -1532,6 +1546,13 @@ CSeqDBImpl::FindVolumePaths(vector<string> & paths, bool recursive) const
     m_Aliases.FindVolumePaths(paths, NULL, recursive);
 }
 
+void
+CSeqDBImpl::FindVolumePaths(vector<string> & paths, vector<string> & alias, bool recursive) const
+{
+    CHECK_MARKER();
+    m_Aliases.FindVolumePaths(paths, &alias, recursive);
+}
+
 void CSeqDBImpl::GetAliasFileValues(TAliasFileValues & afv)
 {
     CHECK_MARKER();
@@ -1988,7 +2009,7 @@ int CSeqDBImpl::x_GetColumnId(const string   & title,
         if (found) {
             CRef<CSeqDB_ColumnEntry> obj(new CSeqDB_ColumnEntry(vol_ids));
 
-            col_id = m_ColumnInfo.size();
+            col_id = static_cast<int>(m_ColumnInfo.size());
             m_ColumnInfo.push_back(obj);
         } else {
             col_id = kColumnNotFound;
@@ -2563,11 +2584,6 @@ void CSeqDBImpl::SetNumberOfThreads(int num_threads, bool force_mt)
             m_CachedSeqs.push_back(new SSeqResBuffer());
         }
 
-        for(int vol_idx = 0; vol_idx < m_VolSet.GetNumVols(); vol_idx++) {
-            m_VolSet.GetVol(vol_idx)->OpenSeqFile(locked);
-        }
-        //m_Atlas.SetSliceSize();
-
     } else if (num_threads < m_NumThreads) {
 
         for (int thread = num_threads; thread < m_NumThreads; ++thread) {
@@ -2613,6 +2629,14 @@ void CSeqDBImpl::SetVolsMemBit(int mbit)
     }
 }
 
+void CSeqDBImpl::SetVolsOidMaskType(int oid_mask_type)
+{
+    int nvols = m_VolSet.GetNumVols();
+    for (int vol = 0; vol < nvols; ++vol) {
+        m_VolSet.GetVolNonConst(vol)->SetOidMaskType(oid_mask_type);
+    }
+}
+
 void CSeqDBImpl::DebugDump(CDebugDumpContext ddc, unsigned int depth) const
 {
     ddc.SetFrame("CSeqDBImpl");
@@ -2648,5 +2672,56 @@ EBlastDbVersion CSeqDBImpl::GetBlastDbVersion() const
 
 }
 
+int CSeqDBImpl::GetNumOfVols() const
+{
+  	return m_VolSet.GetNumVols();
+}
+
+void CSeqDBImpl::GetLMDBFileNames(vector<string> & lmdb_list) const
+{
+	m_LMDBSet.GetLMDBFileNames(lmdb_list);
+}
+
+
+void CSeqDBImpl::x_GetTaxIdsForSeqId(const CSeq_id & seq_id, int oid, CBlast_def_line::TTaxIds & taxid_set)
+{
+
+	CSeqDBLockHold locked(m_Atlas);
+    CRef<CBlast_def_line_set> defline_set = x_GetHdr(oid, locked);
+
+    if ((! defline_set.Empty()) && defline_set->CanGet()) {
+        ITERATE(list< CRef<CBlast_def_line> >, defline, defline_set->Get()) {
+            if (! (*defline)->CanGetSeqid()) {
+                continue;
+            }
+
+            ITERATE(list< CRef<CSeq_id> >, df_seqid, (*defline)->GetSeqid()) {
+            	if((*df_seqid)->Match(seq_id)) {
+            		CBlast_def_line::TTaxIds df_taxids = (*defline)->GetTaxIds();
+            		if(!df_taxids.empty()) {
+            			taxid_set.insert(df_taxids.begin(), df_taxids.end());
+            		}
+            		return;
+            	}
+	        }
+	    }
+    }
+}
+
+void CSeqDBImpl::GetTaxIdsForSeqId(const CSeq_id & seq_id, vector<TTaxId> & taxids)
+{
+	vector<int>  oids;
+	SeqidToOids(seq_id, oids, true);
+	taxids.clear();
+	CBlast_def_line::TTaxIds taxid_set;
+	for (unsigned int i=0; i < oids.size(); i++) {
+		x_GetTaxIdsForSeqId(seq_id, oids[i], taxid_set);
+	}
+
+	if (!taxid_set.empty()) {
+		taxids.insert(taxids.begin(), taxid_set.begin(), taxid_set.end());
+	}
+}
+
 END_NCBI_SCOPE
 
