@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBIMPL_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBIMPL_HPP
 
-/*  $Id: seqdbimpl.hpp 631537 2021-05-19 13:50:49Z ivanov $
+/*  $Id: seqdbimpl.hpp 654652 2022-08-18 16:44:30Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -676,6 +676,8 @@ public:
     ///   If true the search will traverse the full alias node tree
     void FindVolumePaths(vector<string> & paths, bool recursive) const;
 
+    void FindVolumePaths(vector<string> & paths, vector<string> & alias, bool recursive) const;
+
     /// Set Iteration Range
     ///
     /// This method sets the iteration range as a pair of OIDs.
@@ -1056,6 +1058,8 @@ public:
     /// Set the membership bit of all volumes
     void SetVolsMemBit(int mbit);
 
+    void SetVolsOidMaskType(int oid_mask_type);
+
     /// Dump debug information for this object
     /// @sa CDebugDumpable
     void DebugDump(CDebugDumpContext ddc, unsigned int depth) const;
@@ -1072,6 +1076,14 @@ public:
     /// @param tax_ids	return taxonomy ids in db
     void GetDBTaxIds(set<TTaxId> & tax_ids);
 
+    void GetTaxIdsForOids(const vector<blastdb::TOid> & oids, set<TTaxId> & tax_ids);
+
+    int GetNumOfVols() const;
+
+    void GetLMDBFileNames(vector<string> & lmdb_list) const;
+
+    void GetTaxIdsForSeqId(const CSeq_id & seq_id, vector<TTaxId> & taxids);
+
 private:
     CLASS_MARKER_FIELD("IMPL")
 
@@ -1260,6 +1272,8 @@ private:
     int x_GetCacheID(CSeqDBLockHold &locked) const;
 
     
+    void x_GetTaxIdsForSeqId(const CSeq_id & seq_id, int oid, CBlast_def_line::TTaxIds & taxid_set);
+
     /// Memory management layer guard (RIIA) object.
     CSeqDBAtlasHolder m_AtlasHolder;
 
