@@ -1,4 +1,4 @@
-/*  $Id: seqdbisam.cpp 631527 2021-05-19 13:49:38Z ivanov $
+/*  $Id: seqdbisam.cpp 683929 2024-06-10 12:07:52Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1304,7 +1304,7 @@ void CSeqDBIsam::StringToOids(const string   & acc,
         bool is_version = false;
 
         if (pos != string::npos) {
-            int ver_len = acc.size() - pos - 1;
+            int ver_len = static_cast<int>(acc.size() - pos) - 1;
 
             is_version = (ver_len <= 3 && ver_len >= 1);
 
