@@ -1,4 +1,4 @@
-/*  $Id: seqdboidlist.cpp 610974 2020-06-26 12:59:33Z grichenk $
+/*  $Id: seqdboidlist.cpp 683929 2024-06-10 12:07:52Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,6 +116,7 @@ void CSeqDBOIDList::x_Setup(const CSeqDBVolSet       & volset,
     while(m_NumOIDs && (! x_IsSet(m_NumOIDs - 1))) {
         -- m_NumOIDs;
     }
+    LOG_POST(Info << "Num Of Oids: " << m_NumOIDs);
 }
 
 CRef<CSeqDB_BitSet>
@@ -137,7 +138,7 @@ CSeqDBOIDList::x_ComputeFilters(const CSeqDB_FilterTree & filters,
     // Step 1: Compute the bitmap representing the filtering done by
     // all subnodes.  This is a "union".
     
-    int vols = ft->GetVolumes().size();
+    int vols = static_cast<int>(ft->GetVolumes().size());
     
     _ASSERT(vols || ft->GetNodes().size());
     
@@ -229,6 +230,7 @@ CSeqDBOIDList::x_ComputeFilters(const CSeqDB_FilterTree & filters,
         switch(mask.GetType()) {
         case CSeqDB_AliasMask::eOidList:
             f = x_GetOidMask(mask.GetPath(), vol_start, vol_end);
+            vol.Vol()->SetOidMaskType(mask.GetOidMaskType());
             break;
             
         case CSeqDB_AliasMask::eSiList:
@@ -266,8 +268,26 @@ CSeqDBOIDList::x_ComputeFilters(const CSeqDB_FilterTree & filters,
         filter->IntersectWith(*f, true);
     }
     
+    if (gis.GetUserMaskOpts()) {
+
+   		const CSeqDBVol * v = vol.Vol();
+    	bool is_protein= v->GetSeqType() == 'p' ? true : false;
+    	string p_str = v->GetVolName()+ ".";
+
+    	if ( gis.GetUserMaskOpts() & EOidMaskType::fExcludeModel ) {
+    		string path_str(p_str + SeqDB_GetOidMaskFileExt(is_protein, EOidMaskType::fExcludeModel));
+    		CSeqDB_Path mask_path(path_str);
+    		CFile check_file(path_str);
+    		if (!check_file.Exists()) {
+    			NCBI_THROW(CSeqDBException, eArgErr, "Oid Mask file not found");
+    		}
+
+    		CRef<CSeqDB_BitSet> m = x_GetOidMask( mask_path,vol_start,vol_end);
+    		filter->IntersectWith (*m,true);
+    	}
+    }
+
     volume_map->IntersectWith(*filter, true);
-    
     return volume_map;
 }
 
@@ -477,7 +497,6 @@ CSeqDBOIDList::x_GetOidMask(const CSeqDB_Path & fn,
     }
     CRef<CSeqDB_BitSet> bitset(new CSeqDB_BitSet(vol_start, vol_end, bitmap, bitend));
     
-    
     // Disable any enabled bits occuring after the volume end point
     // [this should not normally occur.]
     
