@@ -1,4 +1,4 @@
-/*  $Id: writedb_impl.cpp 618513 2020-10-21 16:07:12Z grichenk $
+/*  $Id: writedb_impl.cpp 675078 2023-11-14 13:36:47Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -46,6 +46,7 @@
 
 #include <iostream>
 #include <sstream>
+#include <cmath>
 
 BEGIN_NCBI_SCOPE
 
@@ -60,7 +61,9 @@ CWriteDB_Impl::CWriteDB_Impl(const string & dbname,
                              bool           long_ids,
                              bool           use_gi_mask,
                              EBlastDbVersion    dbver,
-                             bool           limit_defline)
+                             bool           limit_defline,
+                             Uint8          oid_masks,
+                             bool           scan_bioseq_4_cfastareader_usrobj)
     : m_Dbname           (dbname),
       m_Protein          (protein),
       m_Title            (title),
@@ -78,7 +81,9 @@ CWriteDB_Impl::CWriteDB_Impl(const string & dbname,
       m_HaveSequence     (false),
       m_LongSeqId        (long_ids),
       m_LmdbOid          (0),
-      m_limitDefline     (protein? limit_defline: false)
+      m_limitDefline     (protein? limit_defline: false),
+      m_OidMasks         (oid_masks),
+      m_ScanBioseq4CFastaReaderUsrObjct(scan_bioseq_4_cfastareader_usrobj)
 {
     CTime now(CTime::eCurrent);
 
@@ -339,6 +344,14 @@ void CWriteDB_Impl::Close()
         if (m_VolumeList.size() == 1) {
             m_Volume->RenameSingle();
         }
+        else if(m_VolumeList.size() > 100){
+        	unsigned int num_digits = log10(m_VolumeList.size()) +1;
+        	for(unsigned i=0; i < m_VolumeList.size(); i++) {
+        		CRef<CWriteDB_Volume> & v = m_VolumeList[i];
+        		v->RenameFileIndex(num_digits);
+        	}
+        	LOG_POST(Info << "Rename files index to " << num_digits << " digits");
+        }
 
         // disable the check for duplicate ids across volumes
         /*
@@ -374,6 +387,8 @@ void CWriteDB_Impl::Close()
         		vol_num_oids[i] = v->GetOID();
         	}
             m_Lmdbdb->InsertVolumesInfo(vol_names, vol_num_oids);
+            m_Lmdbdb.Reset();
+            m_Taxdb.Reset();
         }
 
         m_Volume.Reset();
@@ -393,7 +408,8 @@ void CWriteDB_Impl::x_MakeAlias()
             if (dblist.size())
                 dblist += " ";
 
-            dblist += CDirEntry(CWriteDB_File::MakeShortName(m_Dbname, i)).GetName();
+        		CRef<CWriteDB_Volume> & v = m_VolumeList[i];
+        		dblist += CDirEntry(v->GetVolumeName()).GetName();
         }
     } else {
         dblist = m_Dbname;
@@ -756,7 +772,8 @@ CWriteDB_Impl::x_ExtractDeflines(CConstRef<CBioseq>             & bioseq,
                                  int                              OID,
                                  bool                             parse_ids,
                                  bool                             long_ids,
-                                 bool                             limit_defline)
+                                 bool                             limit_defline,
+                                 bool scan_bioseq_4_cfastareader_usrobj)
 {
     bool use_bin = (deflines.Empty() && pig == 0);
 
@@ -783,6 +800,7 @@ CWriteDB_Impl::x_ExtractDeflines(CConstRef<CBioseq>             & bioseq,
         }
 
         if (bin_hdr.empty()) {
+	    try {
             x_GetFastaReaderDeflines(*bioseq,
                                      deflines,
                                      membbits,
@@ -790,7 +808,11 @@ CWriteDB_Impl::x_ExtractDeflines(CConstRef<CBioseq>             & bioseq,
                                      pig,
                                      false,
                                      parse_ids,
-                                     long_ids);
+                                     long_ids,
+                                     scan_bioseq_4_cfastareader_usrobj);
+	      } catch ( const CSeqIdException&e ) {
+		//LOG_POST(Info << "x_GetFastaReaderDeflines " << e.GetMsg() );
+	    }
         }
 
         if(!s_UseFastaReaderDeflines(bioseq, deflines, long_ids)) {
@@ -894,7 +916,8 @@ void CWriteDB_Impl::x_CookHeader()
                       OID,
                       m_ParseIDs,
                       m_LongSeqId,
-                      m_limitDefline);
+                      m_limitDefline,
+                      m_ScanBioseq4CFastaReaderUsrObjct);
 
     x_CookIds();
 }
@@ -919,7 +942,9 @@ void CWriteDB_Impl::x_CookIds()
         const list< CRef<CSeq_id> > & ids = (**iter).GetSeqid();
         // m_Ids.insert(m_Ids.end(), ids.begin(), ids.end());
         // Spelled out for WorkShop. :-/
-        m_Ids.reserve(m_Ids.size() + ids.size());
+        // ID-6757 : STL containers have efficient internal memory maintenance,
+        // the following line is, on the contrary, very inefficient. 
+        // m_Ids.reserve(m_Ids.size() + ids.size());
         ITERATE (list<CRef<CSeq_id> >, it, ids) {
             m_Ids.push_back(*it);
         }
@@ -1167,7 +1192,8 @@ void CWriteDB_Impl::x_Publish()
                                                m_MaxFileSize,
                                                m_MaxVolumeLetters,
                                                m_Indices,
-                                               m_DbVersion));
+                                               m_DbVersion,
+                                               m_OidMasks));
 
             m_VolumeList.push_back(m_Volume);
 
@@ -1546,7 +1572,8 @@ void CWriteDB_Impl::SetMaxVolumeLetters(Uint8 sz)
 
 CRef<CBlast_def_line_set>
 CWriteDB_Impl::ExtractBioseqDeflines(const CBioseq & bs, bool parse_ids,
-                                     bool long_seqids)
+                                     bool long_seqids,
+                                     bool scan_bioseq_4_cfastareader_usrobj)
 {
     // Get information
 
@@ -1554,10 +1581,11 @@ CWriteDB_Impl::ExtractBioseqDeflines(const CBioseq & bs, bool parse_ids,
     string binary_header;
     vector< vector<int> > v1, v2;
     set<TTaxId> t;
+    const bool kLimitDefline = false;
 
     CConstRef<CBioseq> bsref(& bs);
     x_ExtractDeflines(bsref, deflines, binary_header, v2, v2, 0, t, -1, parse_ids,
-                      long_seqids);
+                      long_seqids, kLimitDefline, scan_bioseq_4_cfastareader_usrobj);
 
     // Convert to return type
 
@@ -1695,7 +1723,8 @@ x_GetFastaReaderDeflines(const CBioseq                  & bioseq,
                          int                              pig,
                          bool                             accept_gt,
                          bool                             parse_ids,
-                         bool                             long_seqids)
+                         bool                             long_seqids,
+                         bool                             scan_bioseq_4_cfastareader_usrobj)
 {
     if (! bioseq.CanGetDescr()) {
         return;
@@ -1705,33 +1734,35 @@ x_GetFastaReaderDeflines(const CBioseq                  & bioseq,
 
     // Scan the CBioseq for the CFastaReader user object.
 
-    ITERATE(list< CRef< CSeqdesc > >, iter, bioseq.GetDescr().Get()) {
-        const CSeqdesc & desc = **iter;
+    if (scan_bioseq_4_cfastareader_usrobj) {
+        ITERATE(list< CRef< CSeqdesc > >, iter, bioseq.GetDescr().Get()) {
+            const CSeqdesc & desc = **iter;
 
-        if (desc.IsUser() &&
-            desc.GetUser().CanGetType() &&
-            desc.GetUser().GetType().IsStr() &&
-            desc.GetUser().GetType().GetStr() == "CFastaReader" &&
-            desc.GetUser().CanGetData()) {
+            if (desc.IsUser() &&
+                desc.GetUser().CanGetType() &&
+                desc.GetUser().GetType().IsStr() &&
+                desc.GetUser().GetType().GetStr() == "CFastaReader" &&
+                desc.GetUser().CanGetData()) {
 
-            const vector< CRef< CUser_field > > & D = desc.GetUser().GetData();
+                const vector< CRef< CUser_field > > & D = desc.GetUser().GetData();
 
-            ITERATE(vector< CRef< CUser_field > >, iter, D) {
-                const CUser_field & f = **iter;
+                ITERATE(vector< CRef< CUser_field > >, iter, D) {
+                    const CUser_field & f = **iter;
 
-                if (f.CanGetLabel() &&
-                    f.GetLabel().IsStr() &&
-                    f.GetLabel().GetStr() == "DefLine" &&
-                    f.CanGetData() &&
-                    f.GetData().IsStr()) {
-                    fasta = NStr::Replace(f.GetData().GetStr(), "\\t", TAB_REPLACEMENT);
-                    fasta = NStr::ParseEscapes(fasta);
-                    break;
+                    if (f.CanGetLabel() &&
+                        f.GetLabel().IsStr() &&
+                        f.GetLabel().GetStr() == "DefLine" &&
+                        f.CanGetData() &&
+                        f.GetData().IsStr()) {
+                        fasta = NStr::Replace(f.GetData().GetStr(), "\\t", TAB_REPLACEMENT);
+                        fasta = NStr::ParseEscapes(fasta);
+                        break;
+                    }
                 }
             }
         }
-    }
 
+    }
     if (fasta.empty())
         return;
 
