@@ -1,4 +1,4 @@
-/*  $Id: writedb_isam.cpp 631534 2021-05-19 13:50:32Z ivanov $
+/*  $Id: writedb_isam.cpp 662402 2023-01-30 14:32:39Z madden $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -113,7 +113,8 @@ CWriteDB_Isam::CWriteDB_Isam(EIsamType      itype,
                                          protein,
                                          index,
                                          m_DFile,
-                                         sparse));
+                                         sparse,
+                                         max_file_size));
 }
 
 CWriteDB_Isam::~CWriteDB_Isam()
@@ -156,16 +157,23 @@ void CWriteDB_Isam::RenameSingle()
     m_DFile->RenameSingle();
 }
 
+void CWriteDB_Isam::RenameFileIndex(unsigned int num_digits)
+{
+    m_IFile->RenameFileIndex(num_digits);
+    m_DFile->RenameFileIndex(num_digits);
+}
+
 CWriteDB_IsamIndex::CWriteDB_IsamIndex(EWriteDBIsamType        itype,
                                        const string          & dbname,
                                        bool                    protein,
                                        int                     index,
                                        CRef<CWriteDB_IsamData> datafile,
-                                       bool                    sparse)
+                                       bool                    sparse,
+                                       Int8                    max_file_size)
     : CWriteDB_File  (dbname,
                       s_IsamExtension(itype, protein, true),
                       index,
-                      0,
+                      max_file_size,
                       false),
       m_Type         (itype),
       m_Sparse       (sparse),
@@ -656,7 +664,6 @@ void CWriteDB_IsamIndex::x_AddPdb(int             oid,
 
     // mol is always 4 characters long. If chain exists, add a sting in which the
     // '|' delimiter is changed to a space.
-    int len = short_id.size();
     if (short_id[4] == '|')
         short_id[4] = ' ';
     x_AddStdString(oid, short_id);
