@@ -1,4 +1,4 @@
-/*  $Id: capitalization_string.cpp 632626 2021-06-03 17:38:42Z ivanov $
+/*  $Id: capitalization_string.cpp 681619 2024-04-11 12:27:56Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -238,6 +238,24 @@ static const string mouse_strain_fixes[] = {
 typedef CStaticPairArrayMap<const char*, const char*, PCase_CStr> TCStringPairsMap;
 DEFINE_STATIC_ARRAY_MAP(TCStringPairsMap,k_state_abbrev, map_state_to_abbrev);
 
+MAKE_CONST_MAP(Canada_map_state_to_abbrev, ct::tagStrNocase, ct::tagStrNocase, 
+{
+    { "alberta", "AB"},
+    { "british columbia", "BC"},
+    { "manitoba", "MB"},
+    { "new brunswick", "NB"},
+    { "newfoundland and labrador", "NL"},
+    { "northwest territories", "NT"},
+    { "nova scotia", "NS"},
+    { "nunavut", "NU"},
+    { "ontario", "ON"},
+    { "prince edward island", "PE"},
+    { "quebec", "QC"},
+    { "saskatchewan", "SK"},
+    { "yukon", "YT"}
+});
+
+
 static const SStaticPair<const char*, const char*> set_short_words[] =
 {
     {"\\bA\\b", "a" },
@@ -615,7 +633,7 @@ const string& GetValidCountryCode(unsigned int i)
 };
 
 
-void FixCapitalizationInString (objects::CSeq_entry_Handle seh, string& str, ECapChange capchange_opt)
+void FixCapitalizationInString (CSeq_entry_Handle seh, string& str, ECapChange capchange_opt)
 {
     if (NStr::IsBlank(str) || capchange_opt == eCapChange_none) {
         return;
@@ -722,7 +740,7 @@ static bool s_ReplaceInPlaceWholeWordNoCase(string& str, const string& search, c
     return modified;
 }
 
-void FixOrgNames(objects::CSeq_entry_Handle seh, string& result)
+void FixOrgNames(CSeq_entry_Handle seh, string& result)
 {
     vector<string> taxnames;
     FindOrgNames(seh, taxnames);
@@ -737,18 +755,21 @@ void FixOrgNames(objects::CSeq_entry_Handle seh, string& result)
     }
 }
 
-void FindOrgNames(objects::CSeq_entry_Handle seh, vector<string>& taxnames)
+void FindOrgNames(CSeq_entry_Handle seh, vector<string>& taxnames)
 {
     if (!seh) return;
-    objects::CBioseq_CI b_iter(seh, objects::CSeq_inst::eMol_na);
+    set<string> names;
+    CBioseq_CI b_iter(seh, CSeq_inst::eMol_na);
     for ( ; b_iter ; ++b_iter ) {
-        objects::CSeqdesc_CI it (*b_iter, objects::CSeqdesc::e_Source);
-        if (it) {
-            if (it->GetSource().IsSetTaxname()) {
-                taxnames.push_back(it->GetSource().GetTaxname());
+        CSeqdesc_CI it (*b_iter, CSeqdesc::e_Source);
+        if (it && it->GetSource().IsSetTaxname()) {
+            auto& tax_name = it->GetSource().GetTaxname();
+            if (!NStr::IsBlank(tax_name)) {
+                names.insert(tax_name);
             }
         }
     }
+    taxnames.assign(names.begin(), names.end());
 }
 
 void RemoveFieldNameFromString( const string& field_name, string& str)
@@ -776,6 +797,17 @@ void GetStateAbbreviation(string& state)
         NStr::ToUpper(state);
 }
 
+void GetCanadaStateAbbreviation(string& state)
+{
+    NStr::ReplaceInPlace(state, "  ", " ");
+    NStr::TruncateSpacesInPlace(state);
+    auto found = Canada_map_state_to_abbrev.find(state);
+    if (found != Canada_map_state_to_abbrev.end())
+        state = found->second;
+    else
+        NStr::ToUpper(state);
+}
+
 bool FixStateAbbreviationsInCitSub(CCit_sub& sub)
 {
     bool modified = false;
@@ -907,7 +939,7 @@ void CapitalizeAfterApostrophe(string& input)
     for (;;) {
         pattern.GetMatch(input, start, 0, CRegexp::fMatch_default, true);
         if (pattern.NumFound() > 0) {
-            const int* rslt = pattern.GetResults(0);
+            const auto* rslt = pattern.GetResults(0);
             if (rslt[0] != start)
                 result += input.substr(start,rslt[0]-start);
             string tmp = input.substr(rslt[0], rslt[1] - rslt[0]);
@@ -946,7 +978,7 @@ void FixAffiliationShortWordsInElement(string& result)
         for (;;) {
             pattern.GetMatch(result, start, 0, CRegexp::fMatch_default, true);
             if (pattern.NumFound() > 0) {
-                const int* rslt = pattern.GetResults(0);
+                const auto* rslt = pattern.GetResults(0);
                 if (rslt[0] != start)
                     temp += result.substr(start,rslt[0]-start);
                 string tmp = result.substr(rslt[0], rslt[1] - rslt[0]);
@@ -973,7 +1005,7 @@ void FixOrdinalNumbers(string& result)
         for (;;) {
             pattern.GetMatch(result, start, 0, CRegexp::fMatch_default, true);
             if (pattern.NumFound() > 0) {
-                const int* rslt = pattern.GetResults(0);
+                const auto* rslt = pattern.GetResults(0);
                 if (rslt[0] != start)
                     temp += result.substr(start,rslt[0]-start);
                 string tmp = result.substr(rslt[0], rslt[1] - rslt[0]);
