@@ -1,4 +1,4 @@
-/* $Id: cleanup_pub.cpp 632626 2021-06-03 17:38:42Z ivanov $
+/* $Id: cleanup_pub.cpp 657065 2022-10-03 21:09:26Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -345,7 +345,7 @@ CRef<CPubCleaner> PubCleanerFactory(CPub& pub)
         return CRef<CPubCleaner>(new CMedlineEntryCleaner(pub.SetMedline()));
         break;
     default:
-        return CRef<CPubCleaner>(NULL);
+        return CRef<CPubCleaner>();
     }
 }
 
@@ -479,16 +479,16 @@ bool CCitSubCleaner::IsEmpty()
 }
 
 
-bool CCitArtCleaner::Clean(bool fix_initials, bool strip_serial)
+bool CCitArtCleaner::CleanArticle(CCit_art& art, bool fix_initials, bool strip_serial)
 {
     bool change = false;
-    if (m_Art.IsSetAuthors()) {
-        if (CCleanup::CleanupAuthList(m_Art.SetAuthors(), fix_initials)) {
+    if (art.IsSetAuthors()) {
+        if (CCleanup::CleanupAuthList(art.SetAuthors(), fix_initials)) {
             change = true;
         }
     }
-    if (m_Art.IsSetFrom()) {
-        auto& from = m_Art.SetFrom();
+    if (art.IsSetFrom()) {
+        auto& from = art.SetFrom();
         if (from.IsBook()) {
             CCitBookCleaner cleaner(from.SetBook());
             change |= cleaner.Clean(fix_initials, strip_serial);
