@@ -1,4 +1,4 @@
-/* $Id: cleanup_utils.cpp 632626 2021-06-03 17:38:42Z ivanov $
+/* $Id: cleanup_utils.cpp 681442 2024-04-09 14:53:56Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -156,7 +156,7 @@ bool CleanVisStringJunk( string &str, bool allow_ellipses )
     // while we're at it, also check if the junk contains a tilde and/or period
     bool isPeriod = false;
     bool isTilde = false;
-    int start_of_junk_pos = str.length() - 1;
+    int start_of_junk_pos = (int)str.length() - 1;
     for( ; start_of_junk_pos >= 0 ; --start_of_junk_pos ) {
         const char ch = str[start_of_junk_pos];
         if (ch <= ' ' || ch == '.' || ch == ',' || ch == '~' || ch == ';') {
@@ -184,7 +184,7 @@ bool CleanVisStringJunk( string &str, bool allow_ellipses )
         // holds the suffix to add after we remove the junk
         const char * suffix = ""; // by default, just remove junk
 
-        const int chars_in_junk = ( str.length() - start_of_junk_pos );
+        const int chars_in_junk = ( (int)str.length() - start_of_junk_pos );
         _ASSERT( chars_in_junk >= 1 );
         // allow one period at end
         if (isPeriod) {
@@ -340,7 +340,6 @@ bool Asn2gnbkCompressSpaces(string& val)
     curr = *in;
     in++;
 
-    next = 0;
     two_chars = curr;
 
     while (curr != '\0') {
@@ -428,20 +427,20 @@ bool Asn2gnbkCompressSpaces(string& val)
             ch = *ptr;
         }
         *dst = '\0';
-        dst = NULL;
+        dst = nullptr;
         ptr = str;
         ch = *ptr;
         while (ch != '\0') {
             if (ch != ' ') {
-                dst = NULL;
+                dst = nullptr;
             }
-            else if (dst == NULL) {
+            else if (!dst) {
                 dst = ptr;
             }
             ptr++;
             ch = *ptr;
         }
-        if (dst != NULL) {
+        if (dst) {
             *dst = '\0';
         }
     }
@@ -491,39 +490,39 @@ bool TrimSpacesSemicolonsAndCommas(string& val)
         }
         *dst = '\0';
     }
-    amp = NULL;
-    dst = NULL;
+    amp = nullptr;
+    dst = nullptr;
     ptr = str;
     ch = *ptr;
     while (ch != '\0') {
         if (ch == '&') {
             amp = ptr;
-            dst = NULL;
+            dst = nullptr;
         }
         else if (ch <= ' ') {
-            if (dst == NULL) {
+            if (!dst) {
                 dst = ptr;
             }
-            amp = NULL;
+            amp = nullptr;
         }
         else if (ch == ';') {
-            if (dst == NULL && amp == NULL) {
+            if (!dst && !amp) {
                 dst = ptr;
             }
         }
         else if (ch == ',') {
-            if (dst == NULL) {
+            if (!dst) {
                 dst = ptr;
             }
-            amp = NULL;
+            amp = nullptr;
         }
         else {
-            dst = NULL;
+            dst = nullptr;
         }
         ptr++;
         ch = *ptr;
     }
-    if (dst != NULL) {
+    if (dst) {
         *dst = '\0';
     }
 
@@ -630,27 +629,25 @@ static ProteinAbbrevData abbreviation_list[] =
 // or three-letter abbreviation.
 // Use X if the abbreviation is not found.
 
-char ValidAminoAcid (const string& abbrev)
+char x_ValidAminoAcid(string_view abbrev)
 {
-    char ch = 'X';
-
-    for (unsigned int k = 0; k < sizeof(abbreviation_list) / sizeof (ProteinAbbrevData); k++) {
-        if (NStr::EqualNocase (abbrev, abbreviation_list[k].abbreviation)) {
-            ch = abbreviation_list[k].letter;
-            break;
-        }
-    }
-
-    if (abbrev.length() == 1) {
-        for (unsigned int k = 0; k < sizeof(abbreviation_list) / sizeof (ProteinAbbrevData); k++) {
-            if (abbrev.c_str()[0] == abbreviation_list[k].letter) {
-                ch = abbreviation_list[k].letter;
-                break;
+    if (abbrev.length() >= 3) {
+        for (unsigned k = 0; k < ArraySize(abbreviation_list); ++k) {
+            if (NStr::EqualNocase(abbrev, abbreviation_list[k].abbreviation)) {
+                return abbreviation_list[k].letter;
             }
         }
     }
 
-    return ch;
+    if (abbrev.length() == 1) {
+        for (unsigned k = 0; k < ArraySize(abbreviation_list); ++k) {
+            if (abbrev[0] == abbreviation_list[k].letter) {
+                return abbreviation_list[k].letter;
+            }
+        }
+    }
+
+    return 'X';
 }
 
 
