@@ -1,7 +1,7 @@
 #ifndef OBJECTS_GENERAL___CLEANUP_UTILS__HPP
 #define OBJECTS_GENERAL___CLEANUP_UTILS__HPP
 
-/* $Id: cleanup_utils.hpp 632626 2021-06-03 17:38:42Z ivanov $
+/* $Id: cleanup_utils.hpp 681442 2024-04-09 14:53:56Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -378,7 +378,7 @@ if ((o).IsSet##x()) { \
 CRef<CSeq_loc> ReadLocFromText(const string& text, const CSeq_id *id, CScope *scope);
 
 // for finding the correct amino acid letter given an abbreviation
-char NCBI_CLEANUP_EXPORT ValidAminoAcid (const string& abbrev);
+char x_ValidAminoAcid(string_view abbrev);
 
 // for sorting and uniquing dbtags
 bool s_DbtagCompare (const CRef<CDbtag>& dbt1, const CRef<CDbtag>& dbt2);
