@@ -1,4 +1,4 @@
-/*  $Id: fix_feature_id.cpp 632626 2021-06-03 17:38:42Z ivanov $
+/*  $Id: fix_feature_id.cpp 671838 2023-08-24 19:43:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,9 +38,9 @@ BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 
 
-CObject_id::TId CFixFeatureId::s_FindHighestFeatureId(const CSeq_entry_Handle& entry)
+CFixFeatureId::TId CFixFeatureId::s_FindHighestFeatureId(const CSeq_entry_Handle& entry)
 {
-    CObject_id::TId feat_id = 0;
+    TId feat_id = 0;
     for (CFeat_CI feat_it(entry); feat_it; ++feat_it) {
         if (feat_it->IsSetId()) {
             const CFeat_id &id = feat_it->GetId();
@@ -52,7 +52,10 @@ CObject_id::TId CFixFeatureId::s_FindHighestFeatureId(const CSeq_entry_Handle& e
     return feat_id;
 }
 
-static void FindNextOffset(const unordered_set<int> &existing_ids, const unordered_set<int> &new_existing_ids, const unordered_set<int> &current_ids, int &offset)
+static void FindNextOffset(const CFixFeatureId::TIdSet &existing_ids,
+        const CFixFeatureId::TIdSet &new_existing_ids,
+        const CFixFeatureId::TIdSet &current_ids,
+        CFixFeatureId::TId &offset)
 {
     do
     {
@@ -62,32 +65,28 @@ static void FindNextOffset(const unordered_set<int> &existing_ids, const unorder
             current_ids.find(offset) != current_ids.end());
 }
 
-void CFixFeatureId::s_UpdateFeatureIds(const CSeq_entry_Handle& entry, map<CSeq_feat_Handle, CRef<CSeq_feat> > &changed_feats, unordered_set<int> &existing_ids, int &offset)
+
+void CFixFeatureId::s_UpdateFeatureIds(const CSeq_entry_Handle& entry, map<CSeq_feat_Handle, CRef<CSeq_feat> > &changed_feats,
+        TIdSet &existing_ids, TId &offset)
 {
-    unordered_map<int, int> remapped_ids; // map between old id to new id within the current seq-entry only
-    unordered_set<int> new_existing_ids; // id's which were left unchanged in the current seq-entry
-    unordered_set<int>  current_ids; //  newly created (mapped) ids in the current seq-entry
+    unordered_map<TId, TId> remapped_ids; // map between old id to new id within the current seq-entry only
+    TIdSet new_existing_ids; // id's which were left unchanged in the current seq-entry
+    TIdSet new_ids; //  newly created (mapped) ids in the current seq-entry
+
+
     CFeat_CI feat_it(entry);
     for ( ; feat_it; ++feat_it )
     {
-        bool modified = false;
-        CRef<CSeq_feat> edited;
         CSeq_feat_Handle fh = feat_it->GetSeq_feat_Handle();
-        if (changed_feats.find(fh) != changed_feats.end())
-        {
-            edited = changed_feats[fh];
-        }
-        else
-        {
-            edited.Reset(new CSeq_feat);
-            edited->Assign(feat_it->GetOriginalFeature());
-        }
+        const auto& feat = feat_it->GetOriginalFeature();
 
-        if (edited->IsSetId() && edited->GetId().IsLocal() && edited->GetId().GetLocal().IsId())
+        if (feat.IsSetId() && feat.GetId().IsLocal() && feat.GetId().GetLocal().IsId())
         {
-            int id = edited->GetId().GetLocal().GetId();
+            TId id = feat.GetId().GetLocal().GetId();
             if (existing_ids.find(id) != existing_ids.end() ||
-                current_ids.find(id) != current_ids.end())  // remap id if it's found in other seq-entries or among newly created ids in the current seq-entry, do not remap existing duplicate ids within the same seq-entry
+                new_ids.find(id) != new_ids.end())
+                // remap id if it's found in other seq-entries or among newly created ids in the current seq-entry,
+                // do not remap existing duplicate ids within the same seq-entry
             {
                 auto it = remapped_ids.find(id);
                 if (it != remapped_ids.end())
@@ -96,29 +95,26 @@ void CFixFeatureId::s_UpdateFeatureIds(const CSeq_entry_Handle& entry, map<CSeq_
                 }
                 else
                 {
-                    FindNextOffset(existing_ids, new_existing_ids, current_ids, offset); // find id which does not exist among either of the 3 sets
+                    FindNextOffset(existing_ids, new_existing_ids, new_ids, offset); // find id which does not exist among either of the 3 sets
                     remapped_ids[id] = offset;
                 }
+                CRef<CSeq_feat> edited(new CSeq_feat());
+                edited->Assign(feat);
                 edited->SetId().SetLocal().SetId(offset);
-                current_ids.insert(offset);
-                modified = true;
+                new_ids.insert(offset);
+                changed_feats[fh] = edited;
             }
             else
             {
                 new_existing_ids.insert(id);
             }
         }
-        if (modified)
-        {
-            changed_feats[fh] = edited;
-        }
     }
     existing_ids.insert(new_existing_ids.begin(), new_existing_ids.end());
-    existing_ids.insert(current_ids.begin(), current_ids.end());
+    existing_ids.insert(new_ids.begin(), new_ids.end());
     feat_it.Rewind();
     for ( ; feat_it; ++feat_it )
     {
-        bool modified = false;
         CRef<CSeq_feat> edited;
         CSeq_feat_Handle fh = feat_it->GetSeq_feat_Handle();
         if (changed_feats.find(fh) != changed_feats.end())
@@ -133,34 +129,32 @@ void CFixFeatureId::s_UpdateFeatureIds(const CSeq_entry_Handle& entry, map<CSeq_
 
         if (edited->IsSetXref())
         {
-            CSeq_feat::TXref::iterator xref_it = edited->SetXref().begin();
-            while ( xref_it != edited->SetXref().end() )
-            {
-                if ((*xref_it)-> IsSetId() && (*xref_it)->GetId().IsLocal() && (*xref_it)->GetId().GetLocal().IsId())
-                {
-                    int id = (*xref_it)->GetId().GetLocal().GetId();
+            bool modified = false;
+            for (auto pXref : edited->SetXref()) {
+                if (pXref->IsSetId() &&
+                    pXref->GetId().IsLocal() &&
+                    pXref->GetId().GetLocal().IsId()) {
+                    TId id = pXref->GetId().GetLocal().GetId();
                     auto it = remapped_ids.find(id);
-                    if (it != remapped_ids.end())
-                    {
-                        (*xref_it)->SetId().SetLocal().SetId(it->second); // remap xrefs if necessary
+                    if (it != remapped_ids.end()){
+                        pXref->SetId().SetLocal().SetId(it->second);
                         modified = true;
                     }
                 }
-                ++xref_it;
+                if (modified) {
+                    changed_feats[fh] = edited;
+                }
             }
         }
-        if (modified)
-        {
-            changed_feats[fh] = edited;
-        }
     }
 }
 
-
+//LCOV_EXCL_START
+// Exercised by test_fix_feature_ids
 void CFixFeatureId::s_ApplyToSeqInSet(CSeq_entry_Handle tse, map<CSeq_feat_Handle, CRef<CSeq_feat> > &changed_feats)
 {
-    int offset = 0;
-    unordered_set<int> existing_ids;
+    TId offset = 0;
+    TIdSet existing_ids;
     if (tse && tse.IsSet() && tse.GetSet().IsSetClass() && tse.GetSet().GetClass() == CBioseq_set::eClass_genbank)
     {
         for(CSeq_entry_CI direct_child_ci( tse.GetSet(), CSeq_entry_CI::eNonRecursive ); direct_child_ci; ++direct_child_ci )
@@ -170,11 +164,37 @@ void CFixFeatureId::s_ApplyToSeqInSet(CSeq_entry_Handle tse, map<CSeq_feat_Handl
         }
     }
 }
+//LCOV_EXCL_STOP
 
-// This function maps existing feature ids to the sequential ints - 1,2,3,...
-void CFixFeatureId::s_MakeIDPairs(const CSeq_entry_Handle& entry, map<int,int> &id_pairs)
+void CFixFeatureId::s_ApplyToSeqInSet(CSeq_entry_Handle tse)
 {
-    int feat_id = 0;
+    if (tse && tse.IsSet() && tse.GetSet().IsSetClass() && tse.GetSet().GetClass() == CBioseq_set::eClass_genbank)
+    {
+        TId offset = 0;
+        TIdSet existing_ids;
+        for(CSeq_entry_CI direct_child_ci( tse.GetSet(), CSeq_entry_CI::eNonRecursive ); direct_child_ci; ++direct_child_ci )
+        {
+            TFeatMap changed_feats;
+            const CSeq_entry_Handle& entry = *direct_child_ci;
+            s_UpdateFeatureIds(entry, changed_feats, existing_ids, offset);
+            for (auto entry : changed_feats)
+            {
+                auto orig_feat = entry.first;
+                auto new_feat = entry.second;
+                CSeq_feat_EditHandle feh(orig_feat);
+                feh.Replace(*new_feat);
+            }
+        }
+    }
+}
+
+//LCOV_EXCL_START
+// The following 2 methods are exercised in test_fix_feature_ids.
+//
+// This function maps existing feature ids to the sequential ints - 1,2,3,...
+void CFixFeatureId::s_MakeIDPairs(const CSeq_entry_Handle& entry, map<TId,TId> &id_pairs)
+{
+    TId feat_id = 0;
     for (CFeat_CI feat_it(entry); feat_it; ++feat_it) {
         if (feat_it->IsSetId()) {
             const CFeat_id &id = feat_it->GetId();
@@ -191,7 +211,7 @@ void CFixFeatureId::s_ReassignFeatureIds(const CSeq_entry_Handle& entry, map<CSe
 {
     if (!entry)
         return;
-    map<int,int> id_pairs;
+    map<TId,TId> id_pairs;
     CFixFeatureId::s_MakeIDPairs(entry, id_pairs);
 
     for ( CFeat_CI feat_it(entry); feat_it; ++feat_it )
@@ -211,7 +231,7 @@ void CFixFeatureId::s_ReassignFeatureIds(const CSeq_entry_Handle& entry, map<CSe
 
         if (edited->IsSetId() && edited->GetId().IsLocal() && edited->GetId().GetLocal().IsId())
         {
-            int id = id_pairs[edited->GetId().GetLocal().GetId()];
+            TId id = id_pairs[edited->GetId().GetLocal().GetId()];
             edited->SetId().SetLocal().SetId(id);
             modified = true;
         }
@@ -225,7 +245,7 @@ void CFixFeatureId::s_ReassignFeatureIds(const CSeq_entry_Handle& entry, map<CSe
                     modified = true;
                     if (id_pairs.find((*xref_it)->GetId().GetLocal().GetId()) != id_pairs.end())
                         {
-                            int id = id_pairs[(*xref_it)->GetId().GetLocal().GetId()];
+                            TId id = id_pairs[(*xref_it)->GetId().GetLocal().GetId()];
                             (*xref_it)->SetId().SetLocal().SetId(id);
                         }
                     else
@@ -246,5 +266,6 @@ void CFixFeatureId::s_ReassignFeatureIds(const CSeq_entry_Handle& entry, map<CSe
        }
     }
 }
+//LCOV_EXCL_STOP
 
 END_NCBI_SCOPE
