@@ -121,7 +121,7 @@ class CNewCleanup_imp
 {
 public:
 
-    static const int NCBI_CLEANUP_VERSION;
+    static const int NCBI_CLEANUP_VERSION = 1;
 
     // some cleanup functions will return a value telling you whether
     // to erase the cleaned value ( or whatever action may be
@@ -221,6 +221,8 @@ public:
 
     static bool ShouldRemoveAnnot(const CSeq_annot& annot);
 
+    static void AddNcbiCleanupObject(CSeq_descr& descr);
+
 private:
 
     // many more methods and variables ...
@@ -317,14 +319,12 @@ private:
 
     void MoveDbxrefs(CSeq_feat& sf);
     void MoveStandardName(CSeq_feat& sf);
-    void CreatePubFromFeat(CSeq_feat& sf);
     void ResynchProteinPartials ( CSeq_feat& feat );
     void ResynchPeptidePartials( CBioseq& seq );
     void x_SetPartialsForProtein(CBioseq& prot, bool partial5, bool partial3, bool feat_partial);
     void RemoveBadProteinTitle(CBioseq& seq);
     void MoveCitationQuals(CBioseq& seq);
-    void x_RemoveUnseenTitles(CBioseq& seq);
-    void x_RemoveUnseenTitles(CBioseq_set& set);
+    void x_RemoveUnseenTitles(CSeq_descr& seq_descr);
     void KeepLatestDateDesc(CSeq_descr & seq_descr);
     void x_SingleSeqSetToSeq(CBioseq_set& set);
     void x_MergeDupBioSources(CSeq_descr & seq_descr);
@@ -424,6 +424,7 @@ private:
     void x_OrgModBC( COrgMod & orgmod );
 
     void x_FixUnsetMolFromBiomol( CMolInfo& molinfo, CBioseq &bioseq );
+    void FixUnsetMolFromBiomol(CMolInfo::TBiomol biomol, CBioseq& bioseq);
 
     void x_AddPartialToProteinTitle( CBioseq &bioseq );
 
@@ -461,13 +462,11 @@ private:
     // string cleanup funcs
     void x_CleanupStringMarkChanged( std::string &str );
     void x_CleanupStringJunkMarkChanged( std::string &str );
-    void x_CleanupVisStringMarkChanged( std::string &str );
     void x_ConvertDoubleQuotesMarkChanged( std::string &str );
     bool x_CompressSpaces( string &str );
     void x_CompressStringSpacesMarkChanged( std::string &str );
     void x_StripSpacesMarkChanged( std::string& str );
     void x_RemoveSpacesBetweenTildesMarkChanged( std::string & str );
-    void X_CommentTildeFixes( std::string & str );
     void x_TruncateSpacesMarkChanged( std::string & str );
     void x_TrimInternalSemicolonsMarkChanged( std::string & str );
 
@@ -500,18 +499,19 @@ private:
     void x_NotePubdescOrAnnotPubs_RecursionHelper(
         const CPub_equiv &pub_equiv, int &muid, int &pmid );
     void x_RememberPubOldLabel( CPub &pub );
-    void x_RememberMuidThatMightBeConvertibleToPmid( int &muid, CPub &pub );
+    void x_RememberMuidThatMightBeConvertibleToPmid(CPub &pub );
     void x_RememberSeqFeatCitPubs( CPub &pub );
 
     void x_DecodeXMLMarkChanged( std::string & str );
+    void AddMolInfo(CBioseq_set& set, const CMolInfo& mol);
+    void AddMolInfo(CBioseq& seq, const CMolInfo& mol);
 
 private:
     void x_SortSeqDescs( CSeq_entry & seq_entry );
 
-    void x_FixStructuredCommentKeywords( CBioseq & bioseq );
+    void x_FixStructuredCommentKeywords(CSeq_descr& descr);
 
     void x_RemoveDupBioSource( CBioseq_set & bioseq_set );
-    void x_RemoveDupBioSource(CBioseq & bioseq);
     void x_RemoveDupBioSource(CSeq_entry& se, const CBioSource& src);
 
     void x_RemoveDupPubs(CSeq_descr & descr);
@@ -563,8 +563,6 @@ private:
     void x_RemovePopPhyBioSource(CBioseq_set& set, const COrg_ref& org);
     void x_RemovePopPhyBioSource(CBioseq& seq, const COrg_ref& org);
     void x_RemovePopPhyMolInfo(CBioseq_set& set);
-    void x_RemovePopPhyMolInfo(CBioseq_set& set, const CMolInfo& mol);
-    void x_RemovePopPhyMolInfo(CBioseq& seq, const CMolInfo& mol);
     void x_BioseqSetNucProtEC( CBioseq_set & bioseq_set );
     void x_BioseqSetGenBankEC(CBioseq_set & bioseq_set);
     void x_RemoveNestedGenBankSet(CBioseq_set & bioseq_set);
@@ -599,9 +597,9 @@ protected:
     /// For simplicity, the same CScope is used wherever possible
     CRef<CScope>          m_Scope;
     /// Set via m_Options when we're in gpipe cleanup mode
-    bool                  m_IsGpipe;
+    bool                  m_IsGpipe      { false };
     /// Set via m_Options to synchronize Cdregion genetic codes with BioSource
-    bool                  m_SyncGenCodes;
+    bool                  m_SyncGenCodes { false };
 
     void SetGlobalFlags(const CSeq_entry& se, bool reset = true);
     void SetGlobalFlags(const CSeq_submit& ss);
@@ -610,11 +608,12 @@ protected:
     void ResetGlobalFlags() { m_StripSerial = true; m_IsEmblOrDdbj = false; }
     /// set if references should NOT have serial numbers
     /// under this entry.
-    bool m_StripSerial;
+    bool m_StripSerial     { true };
     /// tells if any Seq-id on any Bioseq in the blob being cleaned is embl or ddbj.
-    bool m_IsEmblOrDdbj;
+    bool m_IsEmblOrDdbj    { false };
 
-    bool m_KeepTopNestedSet;
+    bool m_KeepTopNestedSet { false };
+    bool m_KeepSingleSeqSet { false };
 
     friend class CAutogeneratedCleanup;
     friend class CAutogeneratedExtendedCleanup;
