@@ -1,4 +1,4 @@
-/*  $Id: gbnative.cpp 620489 2020-11-23 18:09:39Z sadyrovr $
+/*  $Id: gbnative.cpp 675419 2023-11-21 19:45:39Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -83,10 +83,6 @@ BEGIN_SCOPE(objects)
 //
 
 
-NCBI_PARAM_DEF_EX(string, GENBANK, LOADER_METHOD, "",
-                  eParam_NoThread, GENBANK_LOADER_METHOD);
-typedef NCBI_PARAM_TYPE(GENBANK, LOADER_METHOD) TGenbankLoaderMethod;
-
 NCBI_PARAM_DECL(string, GENBANK, READER_NAME);
 NCBI_PARAM_DEF_EX(string, GENBANK, READER_NAME, "",
                   eParam_NoThread, GENBANK_READER_NAME);
@@ -98,10 +94,11 @@ NCBI_PARAM_DEF_EX(string, GENBANK, WRITER_NAME, "",
 typedef NCBI_PARAM_TYPE(GENBANK, WRITER_NAME) TGenbankWriterName;
 
 #if defined(HAVE_PUBSEQ_OS)
-static const char* const DEFAULT_DRV_ORDER = "ID2:PUBSEQOS:ID1";
+static const char* const DEFAULT_DRV_ORDER = "id2:pubseqos";
 #else
-static const char* const DEFAULT_DRV_ORDER = "ID2:ID1";
+static const char* const DEFAULT_DRV_ORDER = "id2";
 #endif
+static const char* const DEFAULT_HUP_DRV_ORDER = "pubseqos2:pubseqos";
 
 #define GBLOADER_NAME "GBLOADER"
 #define GBLOADER_HUP_NAME "GBLOADER-HUP"
@@ -121,24 +118,24 @@ public:
         {
             return *m_Loader;
         }
-    virtual CGBDataLoader_Native* GetLoaderPtr(void);
+    virtual CGBDataLoader_Native* GetLoaderPtr(void) override;
 
     //virtual TConn GetConn(void);
     //virtual void ReleaseConn(void);
-    virtual CTSE_LoadLock GetTSE_LoadLock(const TKeyBlob& blob_id);
-    virtual CTSE_LoadLock GetTSE_LoadLockIfLoaded(const TKeyBlob& blob_id);
+    virtual CTSE_LoadLock GetTSE_LoadLock(const TKeyBlob& blob_id) override;
+    virtual CTSE_LoadLock GetTSE_LoadLockIfLoaded(const TKeyBlob& blob_id) override;
     virtual void GetLoadedBlob_ids(const CSeq_id_Handle& idh,
-                                   TLoadedBlob_ids& blob_ids) const;
+                                   TLoadedBlob_ids& blob_ids) const override;
 
-    virtual operator CInitMutexPool&(void) { return GetMutexPool(); }
+    virtual operator CInitMutexPool&(void) override { return GetMutexPool(); }
 
     CInitMutexPool& GetMutexPool(void) { return m_Loader->m_MutexPool; }
 
-    virtual TExpirationTime GetIdExpirationTimeout(GBL::EExpirationType type) const;
+    virtual TExpirationTime GetIdExpirationTimeout(GBL::EExpirationType type) const override;
 
-    virtual bool GetAddWGSMasterDescr(void) const;
+    virtual bool GetAddWGSMasterDescr(void) const override;
 
-    virtual EGBErrorAction GetPTISErrorAction(void) const;
+    virtual EGBErrorAction GetPTISErrorAction(void) const override;
     
     friend class CGBDataLoader_Native;
 
@@ -147,115 +144,15 @@ private:
 };
 
 
-CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::ConvertRegInfo(const TGBMaker::TRegisterInfo& info)
-{
-    TRegisterLoaderInfo ret;
-    ret.Set(info.GetLoader(), info.IsCreated());
-    return ret;
-}
-
-
-CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::RegisterInObjectManager(
-    CObjectManager& om,
-    CReader*        reader_ptr,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority  priority)
-{
-    CGBLoaderParams params(reader_ptr);
-    TGBMaker maker(params);
-    CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
-}
-
-
-CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::RegisterInObjectManager(
-    CObjectManager& om,
-    const string&   reader_name,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority  priority)
-{
-    CGBLoaderParams params(reader_name);
-    TGBMaker maker(params);
-    CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
-}
-
-
-CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::RegisterInObjectManager(
-    CObjectManager& om,
-    EIncludeHUP     include_hup,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority  priority)
-{
-    return RegisterInObjectManager(om, include_hup, NcbiEmptyString, is_default,
-                                   priority);
-}
-
-
-CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::RegisterInObjectManager(
-    CObjectManager& om,
-    EIncludeHUP     /*include_hup*/,
-    const string& web_cookie,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority  priority)
-{
-    CGBLoaderParams params("PUBSEQOS2:PUBSEQOS");
-    params.SetHUPIncluded(true, web_cookie);
-    TGBMaker maker(params);
-    CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
-}
-
-
-CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::RegisterInObjectManager(
-    CObjectManager& om,
-    const string&   reader_name,
-    EIncludeHUP     include_hup,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority  priority)
-{
-    return RegisterInObjectManager(om, reader_name, include_hup, NcbiEmptyString,
-                                   is_default, priority);
-}
-
-CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::RegisterInObjectManager(
-    CObjectManager& om,
-    const string&   reader_name,
-    EIncludeHUP     /*include_hup*/,
-    const string& web_cookie,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority  priority)
-{
-    CGBLoaderParams params(reader_name);
-    params.SetHUPIncluded(true, web_cookie);
-    TGBMaker maker(params);
-    CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
-}
-
-
-CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::RegisterInObjectManager(
-    CObjectManager& om,
-    const TParamTree& param_tree,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority  priority)
-{
-    CGBLoaderParams params(&param_tree);
-    TGBMaker maker(params);
-    CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
-}
-
-
 CGBDataLoader::TRegisterLoaderInfo CGBDataLoader_Native::RegisterInObjectManager(
     CObjectManager& om,
     const CGBLoaderParams& params,
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
-    TGBMaker maker(params);
+    CGBLoaderMaker<CGBDataLoader_Native> maker(params);
     CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
+    return maker.GetRegisterInfo();
 }
 
 
@@ -275,20 +172,16 @@ CGBDataLoader_Native::~CGBDataLoader_Native(void)
 }
 
 
-CObjectManager::TPriority CGBDataLoader_Native::GetDefaultPriority(void) const
+CGBDataLoader::TRealBlobId
+CGBDataLoader_Native::x_GetRealBlobId(const TBlobId& blob_id) const
 {
-    CObjectManager::TPriority priority = CDataLoader::GetDefaultPriority();
-    if ( HasHUPIncluded() ) {
-        // HUP data loader has lower priority by default
-        priority += 1;
-    }
-    return priority;
+    return dynamic_cast<const CBlob_id&>(*blob_id);
 }
 
 
 void CGBDataLoader_Native::x_CreateDriver(const CGBLoaderParams& params)
 {
-    auto_ptr<TParamTree> app_params;
+    unique_ptr<TParamTree> app_params;
     const TParamTree* gb_params = 0;
     if ( params.GetParamTree() ) {
         gb_params = GetLoaderParams(params.GetParamTree());
@@ -442,7 +335,7 @@ void CGBDataLoader_Native::x_CreateDriver(const CGBLoaderParams& params)
         if ( NStr::StartsWith(reader_name, "pubseqos") )
             m_WebCookie = params.GetWebCookie();
     
-        if ( x_CreateReaders(reader_name, gb_params, preopen) ) {
+        if ( x_CreateReaders(reader_name, gb_params, params, preopen) ) {
             if ( reader_name == "cache" ||
                  NStr::StartsWith(reader_name, "cache;") ) {
                 x_CreateWriters("cache", gb_params);
@@ -450,8 +343,8 @@ void CGBDataLoader_Native::x_CreateDriver(const CGBLoaderParams& params)
         }
     }
     else {
-        pair<string, string> rw_name = GetReaderWriterName(gb_params);
-        if ( x_CreateReaders(rw_name.first, gb_params, preopen) ) {
+        pair<string, string> rw_name = GetReaderWriterName(gb_params, params);
+        if ( x_CreateReaders(rw_name.first, gb_params, params, preopen) ) {
             x_CreateWriters(rw_name.second, gb_params);
         }
     }
@@ -459,22 +352,29 @@ void CGBDataLoader_Native::x_CreateDriver(const CGBLoaderParams& params)
 
 
 pair<string, string>
-CGBDataLoader_Native::GetReaderWriterName(const TParamTree* params) const
+CGBDataLoader_Native::GetReaderWriterName(const TParamTree* params, const CGBLoaderParams& loader_params) const
 {
     pair<string, string> ret;
+    if ( HasHUPIncluded() ) {
+        // use default HUP readers
+        ret.first = DEFAULT_HUP_DRV_ORDER;
+        return ret;
+    }
     ret.first = GetParam(params, NCBI_GBLOADER_PARAM_READER_NAME);
     if ( ret.first.empty() ) {
         ret.first = TGenbankReaderName::GetDefault();
     }
-    ret.second = GetParam(params, NCBI_GBLOADER_PARAM_WRITER_NAME);
-    if ( ret.first.empty() ) {
-        ret.first = TGenbankWriterName::GetDefault();
+    ret.second = loader_params.GetWriterName();
+    if (ret.second.empty()) {
+        ret.second = GetParam(params, NCBI_GBLOADER_PARAM_WRITER_NAME);
+    }
+    if ( ret.second.empty() ) {
+        ret.second = TGenbankWriterName::GetDefault();
     }
     if ( ret.first.empty() || ret.second.empty() ) {
-        string method = GetParam(params, NCBI_GBLOADER_PARAM_LOADER_METHOD);
+        string method = loader_params.GetLoaderMethod();
         if ( method.empty() ) {
-            // try config first
-            method = TGenbankLoaderMethod::GetDefault();
+            method = x_GetLoaderMethod(params);
         }
         if ( method.empty() ) {
             // fall back default reader list
@@ -495,14 +395,15 @@ CGBDataLoader_Native::GetReaderWriterName(const TParamTree* params) const
 
 
 bool CGBDataLoader_Native::x_CreateReaders(const string& str,
-                                    const TParamTree* params,
-                                    CGBLoaderParams::EPreopenConnection preopen)
+                                           const TParamTree* params,
+                                           const CGBLoaderParams& gb_params,
+                                           CGBLoaderParams::EPreopenConnection preopen)
 {
     vector<string> str_list;
     NStr::Split(str, ";", str_list);
     size_t reader_count = 0;
     for ( size_t i = 0; i < str_list.size(); ++i ) {
-        CRef<CReader> reader(x_CreateReader(str_list[i], params));
+        CRef<CReader> reader(x_CreateReader(str_list[i], gb_params, params));
         if( reader ) {
             if ( HasHUPIncluded() ) {
                 reader->SetIncludeHUP(true, m_WebCookie);
@@ -528,7 +429,7 @@ void CGBDataLoader_Native::x_CreateWriters(const string& str,
     vector<string> str_list;
     NStr::Split(str, ";", str_list);
     for ( size_t i = 0; i < str_list.size(); ++i ) {
-        if ( HasHUPIncluded() ) {
+        if ( HasHUPIncluded() || m_Dispatcher->HasReaderWithHUPIncluded() ) {
             NCBI_THROW(CObjMgrException, eRegisterError,
                        "HUP GBLoader cannot have cache");
         }
@@ -588,7 +489,8 @@ static bool s_ForceDriver(const string& name)
 
 
 CReader* CGBDataLoader_Native::x_CreateReader(const string& name,
-                                       const TParamTree* params)
+                                              const CGBLoaderParams& gb_params,
+                                              const TParamTree* params)
 {
     CRef<TReaderManager> manager = x_GetReaderManager();
     CReader* ret = manager->CreateInstanceFromList(params, name);
@@ -600,6 +502,11 @@ CReader* CGBDataLoader_Native::x_CreateReader(const string& name,
         }
     }
     else {
+        CReaderParams reader_params;
+        reader_params.SetEnableWGS(gb_params.m_EnableWGS);
+        reader_params.SetEnableSNP(gb_params.m_EnableSNP);
+        reader_params.SetEnableCDD(gb_params.m_EnableCDD);
+        ret->SetParams(reader_params);
         ret->InitializeCache(m_CacheManager, params);
     }
     return ret;
@@ -959,7 +866,7 @@ void CGBDataLoader_Native::GetSequenceHashes(const TIds& ids, TLoaded& loaded,
 
 CDataLoader::TBlobVersion CGBDataLoader_Native::GetBlobVersion(const TBlobId& id)
 {
-    const TRealBlobId& blob_id = GetRealBlobId(id);
+    TRealBlobId blob_id = GetRealBlobId(id);
     CGBReaderRequestResult result(this, CSeq_id_Handle());
     CLoadLockBlobVersion lock(result, blob_id);
     if ( !lock.IsLoadedBlobVersion() ) {
@@ -980,7 +887,7 @@ CGBDataLoader_Native::ResolveConflict(const CSeq_id_Handle& handle,
 
     ITERATE(TTSE_LockSet, sit, tse_set) {
         const CTSE_Info& tse = **sit;
-
+        TRealBlobId rbid = GetRealBlobId(tse);
         CLoadLockBlob blob(result, GetRealBlobId(tse));
         _ASSERT(blob);
 
@@ -1201,7 +1108,7 @@ bool CGBDataLoader_Native::CanGetBlobById(void) const
 CDataLoader::TTSE_Lock
 CGBDataLoader_Native::GetBlobById(const TBlobId& id)
 {
-    const TRealBlobId& blob_id = GetRealBlobId(id);
+    TRealBlobId blob_id = GetRealBlobId(id);
 
     CGBReaderRequestResult result(this, CSeq_id_Handle());
     CLoadLockBlob blob(result, blob_id);
@@ -1639,7 +1546,7 @@ void CGBReaderRequestResult::GetLoadedBlob_ids(const CSeq_id_Handle& idh,
 {
     CDataSource::TLoadedBlob_ids blob_ids2;
     m_Loader->GetDataSource()->GetLoadedBlob_ids(idh,
-                                                 CDataSource::fLoaded_bioseqs,
+                                                 CDataSource::fKnown_bioseqs,
                                                  blob_ids2);
     ITERATE(CDataSource::TLoadedBlob_ids, id, blob_ids2) {
         blob_ids.push_back(m_Loader->GetRealBlobId(*id));
