@@ -1,4 +1,4 @@
-/*  $Id: processors.cpp 631409 2021-05-18 11:56:25Z ivanov $
+/*  $Id: processors.cpp 667656 2023-05-18 18:41:31Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -416,13 +416,13 @@ NCBI_PARAM_DEF_EX(Int8, GENBANK, GI_OFFSET, 0,
 
 TIntId CProcessor::GetGiOffset(void)
 {
-    static volatile TIntId gi_offset;
-    static volatile bool initialized;
-    if ( !initialized ) {
-        gi_offset = NCBI_PARAM_TYPE(GENBANK, GI_OFFSET)::GetDefault();
-        initialized = true;
+    static atomic<TIntId> gi_offset;
+    static atomic<bool> initialized;
+    if ( !initialized.load(memory_order_acquire) ) {
+        gi_offset.store(NCBI_PARAM_TYPE(GENBANK, GI_OFFSET)::GetDefault(), memory_order_relaxed);
+        initialized.store(true, memory_order_release);
     }
-    return gi_offset;
+    return gi_offset.load(memory_order_relaxed);
 }
 
 
@@ -898,14 +898,14 @@ void CProcessor_ID1_SNP::ProcessObjStream(CReaderRequestResult& result,
                        "CProcessor_ID1_SNP: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
     }
-    CTSE_SetObjectInfo set_info;
+    CRef<CTSE_SetObjectInfo> set_info(new CTSE_SetObjectInfo);
     CID1server_back reply;
     {{
         CReaderRequestResultRecursion r(result);
             
         CSeq_annot_SNP_Info_Reader::Parse(obj_stream,
                                           Begin(reply),
-                                          set_info);
+                                          *set_info);
             
         LogStat(r, blob_id,
                 CGBRequestStatistics::eStat_LoadSNPBlob,
@@ -922,7 +922,7 @@ void CProcessor_ID1_SNP::ProcessObjStream(CReaderRequestResult& result,
     
     CWriter* writer = GetWriter(result);
     if ( writer && version >= 0 ) {
-        if ( set_info.m_Seq_annot_InfoMap.empty() || !entry.first ) {
+        if ( set_info->m_Seq_annot_InfoMap.empty() || !entry.first ) {
             const CProcessor_ID1* prc =
                 dynamic_cast<const CProcessor_ID1*>
                 (&m_Dispatcher->GetProcessor(eType_ID1));
@@ -936,7 +936,7 @@ void CProcessor_ID1_SNP::ProcessObjStream(CReaderRequestResult& result,
                 (&m_Dispatcher->GetProcessor(eType_St_Seq_entry_SNPT));
             if ( prc ) {
                 prc->SaveSNPBlob(result, blob_id, chunk_id, writer,
-                                 *entry.first, entry.second, set_info);
+                                 *entry.first, entry.second, *set_info);
             }
         }
     }
@@ -944,8 +944,8 @@ void CProcessor_ID1_SNP::ProcessObjStream(CReaderRequestResult& result,
     CLoadLockSetter setter(blob);
     if ( !setter.IsLoaded() ) {
         if ( entry.first ) {
-            OffsetAllGisToOM(*entry.first, &set_info);
-            setter.SetSeq_entry(*entry.first, &set_info);
+            OffsetAllGisToOM(*entry.first, set_info);
+            setter.SetSeq_entry(*entry.first, set_info);
         }
         setter.SetLoaded();
     }
@@ -1096,7 +1096,7 @@ void CProcessor_SE_SNP::ProcessObjStream(CReaderRequestResult& result,
                        "CProcessor_SE_SNP: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
     }
-    CTSE_SetObjectInfo set_info;
+    CRef<CTSE_SetObjectInfo> set_info(new CTSE_SetObjectInfo);
     CRef<CSeq_entry> seq_entry(new CSeq_entry);
     {{
         CWriter* writer = x_GetWriterToSaveBlob(result, blob_id, setter, "SE_SNP");
@@ -1106,7 +1106,7 @@ void CProcessor_SE_SNP::ProcessObjStream(CReaderRequestResult& result,
             
             CSeq_annot_SNP_Info_Reader::Parse(obj_stream,
                                               Begin(*seq_entry),
-                                              set_info);
+                                              *set_info);
             
             LogStat(r, blob_id,
                     CGBRequestStatistics::eStat_ParseSNPBlob,
@@ -1115,7 +1115,7 @@ void CProcessor_SE_SNP::ProcessObjStream(CReaderRequestResult& result,
         }}
 
         if ( writer ) {
-            if ( set_info.m_Seq_annot_InfoMap.empty() || !seq_entry ) {
+            if ( set_info->m_Seq_annot_InfoMap.empty() || !seq_entry ) {
                 const CProcessor_St_SE* prc =
                     dynamic_cast<const CProcessor_St_SE*>
                     (&m_Dispatcher->GetProcessor(eType_St_Seq_entry));
@@ -1136,13 +1136,13 @@ void CProcessor_SE_SNP::ProcessObjStream(CReaderRequestResult& result,
                     (&m_Dispatcher->GetProcessor(eType_St_Seq_entry_SNPT));
                 if ( prc ) {
                     prc->SaveSNPBlob(result, blob_id, chunk_id, writer,
-                                     *seq_entry, setter.GetBlobState(), set_info);
+                                     *seq_entry, setter.GetBlobState(), *set_info);
                 }
             }
         }
     }}
-    OffsetAllGisToOM(*seq_entry, &set_info);
-    setter.SetSeq_entry(*seq_entry, &set_info);
+    OffsetAllGisToOM(*seq_entry, set_info);
+    setter.SetSeq_entry(*seq_entry, set_info);
     setter.SetLoaded();
 }
 
@@ -1400,13 +1400,13 @@ void CProcessor_St_SE_SNPT::ProcessStream(CReaderRequestResult& result,
     result.SetAndSaveBlobState(blob_id, blob_state);
 
     CRef<CSeq_entry> seq_entry(new CSeq_entry);
-    CTSE_SetObjectInfo set_info;
+    CRef<CTSE_SetObjectInfo> set_info(new CTSE_SetObjectInfo);
 
     {{
         CReaderRequestResultRecursion r(result);
         Int8 size = NcbiStreamposToInt8(stream.tellg());
         
-        CSeq_annot_SNP_Info_Reader::Read(stream, Begin(*seq_entry), set_info);
+        CSeq_annot_SNP_Info_Reader::Read(stream, Begin(*seq_entry), *set_info);
 
         size = NcbiStreamposToInt8(stream.tellg()) - size;
         LogStat(r, blob_id,
@@ -1417,10 +1417,10 @@ void CProcessor_St_SE_SNPT::ProcessStream(CReaderRequestResult& result,
     
     CWriter* writer = GetWriter(result);
     if ( writer ) {
-        SaveSNPBlob(result, blob_id, chunk_id, writer, *seq_entry, blob_state, set_info);
+        SaveSNPBlob(result, blob_id, chunk_id, writer, *seq_entry, blob_state, *set_info);
     }
-    OffsetAllGisToOM(*seq_entry, &set_info);
-    setter.SetSeq_entry(*seq_entry, &set_info);
+    OffsetAllGisToOM(*seq_entry, set_info);
+    setter.SetSeq_entry(*seq_entry, set_info);
     setter.SetLoaded();
 }
 
@@ -1512,6 +1512,14 @@ void CProcessor_ID2::ProcessData(CReaderRequestResult& result,
 {
     CLoadLockBlob blob(result, blob_id, chunk_id);
     if ( blob.IsLoadedChunk() ) {
+        if ( chunk_id >= 0 ) {
+            // the same chunk can be load multiple times by getblob request
+            // do not treat this situation as an error since
+            // the chunk was actually loaded successfully
+            ERR_POST_X(4, Info << "CProcessor_ID2: "
+                       "double load of "<<blob_id<<'/'<<chunk_id);
+            return;
+        }
         NCBI_THROW_FMT(CLoaderException, eLoaderFailed,
                        "CProcessor_ID2: "
                        "double load of "<<blob_id<<'/'<<chunk_id);
@@ -1788,8 +1796,8 @@ CObjectIStream* CProcessor_ID2::x_OpenDataStream(const CID2_Reply_Data& data)
         NCBI_THROW(CLoaderException, eLoaderFailed,
                    "CId2Reader::x_ReadData(): unknown data format");
     }
-    auto_ptr<IReader> reader(new COSSReader(data.GetData()));
-    auto_ptr<CNcbiIstream> stream;
+    unique_ptr<IReader> reader(new COSSReader(data.GetData()));
+    unique_ptr<CNcbiIstream> stream;
     switch ( data.GetData_compression() ) {
     case CID2_Reply_Data::eData_compression_none:
         break;
@@ -1812,7 +1820,7 @@ CObjectIStream* CProcessor_ID2::x_OpenDataStream(const CID2_Reply_Data& data)
         stream.reset(new CRStream(reader.release(),
                                   0, 0, CRWStreambuf::fOwnAll));
     }
-    auto_ptr<CObjectIStream> in;
+    unique_ptr<CObjectIStream> in;
     in.reset(CObjectIStream::Open(format, *stream.release(), eTakeOwnership));
     return in.release();
 }
@@ -1822,7 +1830,7 @@ void CProcessor_ID2::x_ReadData(const CID2_Reply_Data& data,
                                 const CObjectInfo& object,
                                 size_t& data_size)
 {
-    auto_ptr<CObjectIStream> in(x_OpenDataStream(data));
+    unique_ptr<CObjectIStream> in(x_OpenDataStream(data));
     switch ( data.GetData_type() ) {
     case CID2_Reply_Data::eData_type_seq_entry:
         if ( object.GetTypeInfo() != CSeq_entry::GetTypeInfo() ) {
@@ -1857,8 +1865,8 @@ void CProcessor_ID2::x_ReadData(const CID2_Reply_Data& data,
 void CProcessor_ID2::DumpDataAsText(const CID2_Reply_Data& data,
                                     CNcbiOstream& out_stream)
 {
-    auto_ptr<CObjectIStream> in(x_OpenDataStream(data));
-    auto_ptr<CObjectOStream> out(CObjectOStream::Open(eSerial_AsnText,
+    unique_ptr<CObjectIStream> in(x_OpenDataStream(data));
+    unique_ptr<CObjectOStream> out(CObjectOStream::Open(eSerial_AsnText,
                                                       out_stream));
     TTypeInfo type;
     switch ( data.GetData_type() ) {
