@@ -1,4 +1,4 @@
-/*  $Id: psg_loader.cpp 624399 2021-01-28 20:09:32Z ivanov $
+/*  $Id: psg_loader.cpp 675419 2023-11-21 19:45:39Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -51,9 +51,99 @@
 #if defined(HAVE_PSG_LOADER)
 
 BEGIN_NCBI_SCOPE
+
+NCBI_PARAM_ENUM_ARRAY(objects::CSeq_id::ESNPScaleLimit, PSG_LOADER, SNP_SCALE_LIMIT) {
+    {"", objects::CSeq_id::eSNPScaleLimit_Default},
+    {"Unit", objects::CSeq_id::eSNPScaleLimit_Unit},
+    {"Contig", objects::CSeq_id::eSNPScaleLimit_Contig},
+    {"Supercontig", objects::CSeq_id::eSNPScaleLimit_Supercontig},
+    {"Chromosome", objects::CSeq_id::eSNPScaleLimit_Chromosome},
+};
+NCBI_PARAM_ENUM_DECL(objects::CSeq_id::ESNPScaleLimit, PSG_LOADER, SNP_SCALE_LIMIT);
+NCBI_PARAM_ENUM_DEF_EX(objects::CSeq_id::ESNPScaleLimit, PSG_LOADER, SNP_SCALE_LIMIT,
+                       objects::CSeq_id::eSNPScaleLimit_Default,
+                       eParam_NoThread, PSG_LOADER_SNP_SCALE_LIMIT);
+typedef NCBI_PARAM_TYPE(PSG_LOADER, SNP_SCALE_LIMIT) TSNP_Scale_Limit;
+
 BEGIN_SCOPE(objects)
 
-class CDataLoader;
+/////////////////////////////////////////////////////////////////////////////
+// CPsgBlobId
+/////////////////////////////////////////////////////////////////////////////
+
+
+CPsgBlobId::CPsgBlobId(const string& id)
+    : m_Id(id)
+{
+}
+
+
+CPsgBlobId::CPsgBlobId(const string& id, const string& id2_info)
+    : m_Id(id),
+      m_Id2Info(id2_info)
+{
+}
+
+
+CPsgBlobId::~CPsgBlobId()
+{
+}
+
+
+string CPsgBlobId::ToString(void) const
+{
+    return m_Id;
+}
+
+
+bool CPsgBlobId::operator==(const CBlobId& id_ref) const
+{
+    const CPsgBlobId* id = dynamic_cast<const CPsgBlobId*>(&id_ref);
+    return id && m_Id == id->m_Id;
+}
+
+
+bool CPsgBlobId::operator<(const CBlobId& id_ref) const
+{
+    const CPsgBlobId* id = dynamic_cast<const CPsgBlobId*>(&id_ref);
+    if ( !id ) {
+        return LessByTypeId(id_ref);
+    }
+    return m_Id < id->m_Id;
+}
+
+
+bool CPsgBlobId::GetSatSatkey(int& sat, int& satkey) const
+{
+    string ssat, ssatkey;
+    NStr::SplitInTwo(m_Id, ".", ssat, ssatkey);
+    if (ssat.empty() || ssatkey.empty()) return false;
+    try {
+        sat = NStr::StringToNumeric<int>(ssat);
+        satkey = NStr::StringToNumeric<int>(ssatkey);
+    }
+    catch (CStringException&) {
+        return false;
+    }
+    return true;
+}
+
+
+CConstRef<CPsgBlobId> CPsgBlobId::GetPsgBlobId(const CBlobId& blob_id)
+{
+    if ( auto psg_blob_id = dynamic_cast<const CPsgBlobId*>(&blob_id) ) {
+        return ConstRef(psg_blob_id);
+    }
+    const CBlob_id* gb_blob_id = dynamic_cast<const CBlob_id*>(&blob_id);
+    if (!gb_blob_id) {
+        NCBI_THROW(CLoaderException, eOtherError, "Incompatible blob-id: " + blob_id.ToString());
+    }
+    return ConstRef(new CPsgBlobId(
+                        NStr::NumericToString(gb_blob_id->GetSat()) +
+                        '.' +
+                        NStr::NumericToString(gb_blob_id->GetSatKey())));
+}
+
 
 /////////////////////////////////////////////////////////////////////////////
 // CPSGDataLoader
@@ -61,44 +151,40 @@ class CDataLoader;
 
 
 #define PSGLOADER_NAME "GBLOADER"
-#define PSGLOADER_HUP_NAME "GBLOADER-HUP"
 
 const char kDataLoader_PSG_DriverName[] = "psg";
 
+
+CSeq_id::ESNPScaleLimit CPSGDataLoader::GetSNP_Scale_Limit(void)
+{
+    return TSNP_Scale_Limit::GetDefault();
+}
+
+
+void CPSGDataLoader::SetSNP_Scale_Limit(CSeq_id::ESNPScaleLimit value)
+{
+    TSNP_Scale_Limit::SetDefault(value);
+}
+
+
+CPSGDataLoader::TRegisterLoaderInfo CPSGDataLoader::RegisterInObjectManager(
+    CObjectManager& om,
+    CObjectManager::EIsDefault is_default,
+    CObjectManager::TPriority priority)
+{
+    return RegisterInObjectManager(om, CGBLoaderParams(), is_default, priority);
+}
+
+
 CPSGDataLoader::TRegisterLoaderInfo CPSGDataLoader::RegisterInObjectManager(
     CObjectManager& om,
     const CGBLoaderParams& params,
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority priority)
 {
-    TMaker maker(params);
+    CGBLoaderMaker<CPSGDataLoader> maker(params);
     CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
-}
-
-
-CPSGDataLoader::TRegisterLoaderInfo CPSGDataLoader::RegisterInObjectManager(
-    CObjectManager& om,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority priority)
-{
-    CGBLoaderParams params;
-    TMaker maker(params);
-    CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
-}
-
-
-CPSGDataLoader::TRegisterLoaderInfo CPSGDataLoader::RegisterInObjectManager(
-    CObjectManager& om,
-    const TParamTree& param_tree,
-    CObjectManager::EIsDefault is_default,
-    CObjectManager::TPriority priority)
-{
-    CGBLoaderParams params(&param_tree);
-    TMaker maker(params);
-    CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
-    return ConvertRegInfo(maker.GetRegisterInfo());
+    return maker.GetRegisterInfo();
 }
 
 
@@ -150,11 +236,53 @@ CPSGDataLoader::TTSE_LockSet CPSGDataLoader::GetOrphanAnnotRecordsNA(const CSeq_
 }
 
 
-CPSGDataLoader::TTSE_LockSet CPSGDataLoader::GetExternalAnnotRecordsNA(const CSeq_id_Handle& idh,
-    const SAnnotSelector* sel,
-    TProcessedNAs* processed_nas)
-{
-    return m_Impl->GetAnnotRecordsNA(GetDataSource(), idh, sel, processed_nas);
+namespace {
+    struct SBetterId
+    {
+        int GetScore(const CSeq_id_Handle& id1) const
+            {
+                if ( id1.IsGi() ) {
+                    return 100;
+                }
+                if ( !id1 ) {
+                    return -1;
+                }
+                CConstRef<CSeq_id> seq_id = id1.GetSeqId();
+                const CTextseq_id* text_id = seq_id->GetTextseq_Id();
+                if ( text_id ) {
+                    int score;
+                    if ( text_id->IsSetAccession() ) {
+                        if ( text_id->IsSetVersion() ) {
+                            score = 99;
+                        }
+                        else {
+                            score = 50;
+                        }
+                    }
+                    else {
+                        score = 0;
+                    }
+                    return score;
+                }
+                if ( seq_id->IsGeneral() ) {
+                    return 10;
+                }
+                if ( seq_id->IsLocal() ) {
+                    return 0;
+                }
+                return 1;
+            }
+        bool operator()(const CSeq_id_Handle& id1,
+                        const CSeq_id_Handle& id2) const
+            {
+                int score1 = GetScore(id1);
+                int score2 = GetScore(id2);
+                if ( score1 != score2 ) {
+                    return score1 > score2;
+                }
+                return id1 < id2;
+            }
+    };
 }
 
 
@@ -162,15 +290,9 @@ CPSGDataLoader::TTSE_LockSet CPSGDataLoader::GetExternalAnnotRecordsNA(const CBi
     const SAnnotSelector* sel,
     TProcessedNAs* processed_nas)
 {
-    TTSE_LockSet ret;
-    ITERATE(CBioseq_Info::TId, it, bioseq.GetId()) {
-        TTSE_LockSet ret2 = m_Impl->GetAnnotRecordsNA(GetDataSource(), *it, sel, processed_nas);
-        if (!ret2.empty()) {
-            ret.swap(ret2);
-            break;
-        }
-    }
-    return ret;
+    TIds ids = bioseq.GetId();
+    sort(ids.begin(), ids.end(), SBetterId());
+    return m_Impl->GetAnnotRecordsNA(GetDataSource(), ids, sel, processed_nas);
 }
 
 
@@ -192,11 +314,17 @@ void CPSGDataLoader::GetBlobs(TTSE_LockSets& tse_sets)
 }
 
 
+void CPSGDataLoader::GetCDDAnnots(const TSeqIdSets& id_sets, TLoaded& loaded, TCDD_Locks& ret)
+{
+    m_Impl->GetCDDAnnots(GetDataSource(), id_sets, loaded, ret);
+}
+
+
 CDataLoader::TTSE_Lock
 CPSGDataLoader::GetBlobById(const TBlobId& blob_id)
 {
     return m_Impl->GetBlobById(GetDataSource(),
-                               dynamic_cast<const CPsgBlobId&>(*blob_id));
+                               *CPsgBlobId::GetPsgBlobId(*blob_id));
 }
 
 
@@ -227,6 +355,12 @@ TTaxId CPSGDataLoader::GetTaxId(const CSeq_id_Handle& idh)
 }
 
 
+void CPSGDataLoader::GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret)
+{
+    m_Impl->GetTaxIds(ids, loaded, ret);
+}
+
+
 TSeqPos CPSGDataLoader::GetSequenceLength(const CSeq_id_Handle& idh)
 {
     return m_Impl->GetSequenceLength(idh);
@@ -249,7 +383,7 @@ CPSGDataLoader::GetSequenceTypeFound(const CSeq_id_Handle& idh)
 
 int CPSGDataLoader::GetSequenceState(const CSeq_id_Handle& idh)
 {
-    return m_Impl->GetSequenceState(idh);
+    return m_Impl->GetSequenceState(GetDataSource(), idh);
 }
 
 
@@ -271,6 +405,18 @@ void CPSGDataLoader::GetGis(const TIds& ids, TLoaded& loaded, TGis& ret)
 }
 
 
+void CPSGDataLoader::GetSequenceLengths(const TIds& ids, TLoaded& loaded, TSequenceLengths& ret)
+{
+    m_Impl->GetSequenceLengths(ids, loaded, ret);
+}
+
+
+void CPSGDataLoader::GetSequenceTypes(const TIds& ids, TLoaded& loaded, TSequenceTypes& ret)
+{
+    m_Impl->GetSequenceTypes(ids, loaded, ret);
+}
+
+
 CGBDataLoader::TNamedAnnotNames
 CPSGDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih)
 {
@@ -359,11 +505,22 @@ CPSGDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih,
 }
 
 
-CGBDataLoader::TRegisterLoaderInfo CPSGDataLoader::ConvertRegInfo(const TMaker::TRegisterInfo& info)
+CGBDataLoader::TRealBlobId
+CPSGDataLoader::x_GetRealBlobId(const TBlobId& blob_id) const
 {
-    TRegisterLoaderInfo ret;
-    ret.Set(info.GetLoader(), info.IsCreated());
-    return ret;
+    const CPsgBlobId* psg_blob_id = dynamic_cast<const CPsgBlobId*>(&*blob_id);
+    if (psg_blob_id) {
+        int sat, satkey;
+        if (psg_blob_id->GetSatSatkey(sat, satkey)) {
+            CBlob_id gb_blob_id;
+            gb_blob_id.SetSat(sat);
+            gb_blob_id.SetSatKey(satkey);
+            return gb_blob_id;
+        }
+    }
+    const CBlob_id* gb_blob_id = dynamic_cast<const CBlob_id*>(&*blob_id);
+    if (gb_blob_id) return *gb_blob_id;
+    return CBlob_id();
 }
 
 
