@@ -1,4 +1,4 @@
-/*  $Id: reader.cpp 610970 2020-06-26 12:56:46Z grichenk $
+/*  $Id: reader.cpp 675188 2023-11-16 13:46:47Z ivanov $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -112,6 +112,7 @@ CReader::CReader(void)
     : m_Dispatcher(0),
       m_MaxConnections(0),
       m_PreopenConnection(DEFAULT_PREOPEN),
+      m_IncludeHUP(false),
       m_NextNewConnection(0),
       m_NumFreeConnections(0, 1000),
       m_MaximumRetryCount(3),
@@ -911,10 +912,21 @@ bool CReader::LoadBlobs(CReaderRequestResult& result,
             }
             
             if ( info.IsSetAnnotInfo() ) {
-                CProcessor_AnnotInfo::LoadBlob(result, info);
-                _ASSERT(blob.IsLoadedBlob());
-                ++loaded_count;
-                continue;
+                if ( info.GetAnnotInfo()->GetAnnotInfo().empty() ) {
+                    // no actual annot info - only name
+                    auto& names = info.GetAnnotInfo()->GetNamedAnnotNames();
+                    if ( names.size() == 1 ) {
+                        CLoadLockSetter setter(blob);
+                        setter.GetTSE_LoadLock()->SetName(*names.begin());
+                        setter.AllowIncompleteLoading();
+                    }
+                }
+                else {
+                    CProcessor_AnnotInfo::LoadBlob(result, info);
+                    _ASSERT(blob.IsLoadedBlob());
+                    ++loaded_count;
+                    continue;
+                }
             }
 
             m_Dispatcher->LoadBlob(result, blob_id);
@@ -1243,7 +1255,7 @@ void CReader::SetAndSaveSeq_idBlob_ids(CReaderRequestResult& result,
                                        const CFixedBlob_ids& blob_ids) const
 {
     TRACE_SET("SetAndSaveSeq_idBlob_ids("<<seq_id<<")");
-    if ( !lock.SetLoadedBlob_ids(blob_ids) ) {
+    if ( !lock.SetLoadedBlob_ids(sel, blob_ids) ) {
         return;
     }
     TRACE_SET("SetAndSaveSeq_idBlob_ids("<<seq_id<<") set");
@@ -1260,7 +1272,7 @@ void CReader::SetAndSaveSeq_idBlob_ids(CReaderRequestResult& result,
                                        const CLoadLockBlobIds& blob_ids) const
 {
     TRACE_SET("SetAndSaveSeq_idBlob_ids("<<seq_id<<")");
-    if ( !lock.SetLoadedBlob_ids(blob_ids) ) {
+    if ( !lock.SetLoadedBlob_ids(sel, blob_ids.GetBlob_ids(), blob_ids.GetExpirationTime()) ) {
         return;
     }
     TRACE_SET("SetAndSaveSeq_idBlob_ids("<<seq_id<<") set");
@@ -1320,6 +1332,11 @@ void CReader::SetIncludeHUP(bool include_hup, const string& web_cookie)
 }
 
 
+void CReader::SetParams(const CReaderParams& params)
+{
+}
+
+
 CReaderCacheManager::CReaderCacheManager(void)
 {
 }
