@@ -1,4 +1,4 @@
-/*  $Id: request_result.cpp 578138 2019-01-14 15:36:05Z vasilche $
+/*  $Id: request_result.cpp 667661 2023-05-18 18:47:34Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -56,11 +56,11 @@ NCBI_PARAM_DEF_EX(int, GENBANK, TRACE_LOAD, 0,
 static
 int s_GetLoadTraceLevel(void)
 {
-    static volatile int load_trace_level;
-    static volatile bool initialized;
-    if ( !initialized ) {
+    static atomic<int> saved_load_trace_level{-1};
+    auto load_trace_level = saved_load_trace_level.load(memory_order_acquire);
+    if ( load_trace_level < 0 ) {
         load_trace_level = NCBI_PARAM_TYPE(GENBANK, TRACE_LOAD)::GetDefault();
-        initialized = true;
+        saved_load_trace_level.store(load_trace_level, memory_order_release);
     }
     return load_trace_level;
 }
@@ -247,6 +247,18 @@ void CFixedBlob_ids::SetNotFound(void)
 }
 
 
+CFixedBlob_ids CFixedBlob_ids::Filter(const SAnnotSelector* sel) const
+{
+    TList infos;
+    for ( auto& blob_info : Get() ) {
+        if ( blob_info.Matches(fBlobHasExtAnnot|fBlobHasNamedAnnot, sel) ) {
+            infos.push_back(blob_info);
+        }
+    }
+    return CFixedBlob_ids(eTakeOwnership, infos);
+}
+
+
 CNcbiOstream& operator<<(CNcbiOstream& out, const CFixedBlob_ids& ids)
 {
     const char* sep = "( ";
@@ -565,6 +577,7 @@ END_LOCAL_NAMESPACE;
 
 
 CLoadLockSetter::CLoadLockSetter(CLoadLockBlob& lock)
+    : m_AllowIncompleteLoading(false)
 {
     x_Init(lock, lock.GetSelectedChunkId());
 }
@@ -572,6 +585,7 @@ CLoadLockSetter::CLoadLockSetter(CLoadLockBlob& lock)
 
 CLoadLockSetter::CLoadLockSetter(CLoadLockBlob& lock,
                                  TChunkId chunk_id)
+    : m_AllowIncompleteLoading(false)
 {
     x_Init(lock, chunk_id);
 }
@@ -580,7 +594,8 @@ CLoadLockSetter::CLoadLockSetter(CLoadLockBlob& lock,
 CLoadLockSetter::CLoadLockSetter(CReaderRequestResult& result,
                                  const CBlob_id& blob_id,
                                  TChunkId chunk_id)
-    : TParent(result.GetLoadLockBlob(blob_id))
+    : TParent(result.GetLoadLockBlob(blob_id)),
+      m_AllowIncompleteLoading(false)
 {
     x_ObtainTSE_LoadLock(result, blob_id);
     if ( chunk_id != kMain_ChunkId ) {
@@ -609,13 +624,19 @@ void CLoadLockSetter::x_Init(CLoadLockBlob& lock, TChunkId chunk_id)
         else {
             x_SelectChunk(chunk_id);
         }
+        if ( m_Chunk ) {
+            m_ChunkLoadLock = m_Chunk->GetLoadInitGuard();
+            if ( !m_ChunkLoadLock.get() || !*m_ChunkLoadLock.get() ) {
+                m_ChunkLoadLock.reset();
+            }
+        }
     }
 }
 
 
 CLoadLockSetter::~CLoadLockSetter(void)
 {
-    if ( !IsLoaded() ) {
+    if ( !m_AllowIncompleteLoading && !IsLoaded() ) {
         ERR_POST("Incomplete loading");
     }
 }
@@ -632,6 +653,12 @@ bool CLoadLockSetter::IsLoaded(void) const
 }
 
 
+void CLoadLockSetter::AllowIncompleteLoading()
+{
+    m_AllowIncompleteLoading = true;
+}
+
+
 void CLoadLockSetter::SetLoaded(void)
 {
     _ASSERT(!IsLoaded());
@@ -652,6 +679,7 @@ void CLoadLockSetter::SetLoaded(void)
             LOG_POST(Info<<"GBLoader:"<<SChunkId(*m_Chunk)<<" loaded");
         }
         m_Chunk->SetLoaded();
+        m_ChunkLoadLock.reset();
     }
 }
 
@@ -1039,6 +1067,33 @@ CLoadLockBlobIds::CLoadLockBlobIds(CReaderRequestResult& result,
 }
 
 
+bool CLoadLockBlobIds::SetLoadedBlob_ids(const SAnnotSelector* sel,
+                                         const TData& blob_ids)
+{
+    if ( !sel || sel->IsIncludedNamedAnnotAccession("NA*") ) {
+        // no filtering
+        return SetLoadedBlob_ids(blob_ids);
+    }
+    else {
+        return SetLoadedBlob_ids(blob_ids.Filter(sel));
+    }
+}
+
+
+bool CLoadLockBlobIds::SetLoadedBlob_ids(const SAnnotSelector* sel,
+                                         const TData& blob_ids,
+                                         TExpirationTime expiration)
+{
+    if ( !sel || sel->IsIncludedNamedAnnotAccession("NA*") ) {
+        // no filtering
+        return SetLoadedBlob_ids(blob_ids, expiration);
+    }
+    else {
+        return SetLoadedBlob_ids(blob_ids.Filter(sel), expiration);
+    }
+}
+
+
 CLoadLockBlobState::CLoadLockBlobState(CReaderRequestResult& result,
                                        const CBlob_id& id)
     : TParent(result.GetLoadLockBlobState(id))
@@ -1237,7 +1292,7 @@ bool
 CReaderRequestResult::SetLoadedSeqIdsFromZeroGi(const CSeq_id_Handle& id,
                                                 const CLoadLockGi& gi_lock)
 {
-    _ASSERT(gi_lock.IsLoadedGi() && !gi_lock.GetGi(gi_lock.GetGi()));
+    _ASSERT(gi_lock.IsLoadedGi() && gi_lock.GetGi(gi_lock.GetGi()) == ZERO_GI );
     if ( s_GetLoadTraceLevel() > 0 ) {
         LOG_POST(Info<<"GBLoader:SeqId("<<id<<") seq_ids = null");
     }
@@ -1252,7 +1307,7 @@ CReaderRequestResult::SetLoadedBlobIdsFromZeroGi(const CSeq_id_Handle& id,
                                                  const SAnnotSelector* sel,
                                                  const CLoadLockGi& gi_lock)
 {
-    _ASSERT(gi_lock.IsLoadedGi() && !gi_lock.GetGi(gi_lock.GetGi()));
+    _ASSERT(gi_lock.IsLoadedGi() && gi_lock.GetGi(gi_lock.GetGi()) == ZERO_GI);
     if ( s_GetLoadTraceLevel() > 0 ) {
         LOG_POST(Info<<"GBLoader:SeqId("<<id<<") blob_ids = null");
     }
@@ -1676,6 +1731,13 @@ CReaderRequestResult::s_KeyBlobIds(const CSeq_id_Handle& seq_id,
         ITERATE ( SAnnotSelector::TNamedAnnotAccessions, it,
                   sel->GetNamedAnnotAccessions() ) {
             key.second += it->first;
+            if (key.second == "SNP") {
+                auto scale_limit = sel->GetSNPScaleLimit();
+                if (scale_limit != CSeq_id::eSNPScaleLimit_Default) {
+                    key.second += "@";
+                    key.second += CSeq_id::GetSNPScaleLimit_Name(scale_limit);
+                }
+            }
             key.second += ',';
         }
     }
@@ -1761,7 +1823,9 @@ bool CReaderRequestResult::SetLoadedBlobVersion(const CBlob_id& blob_id,
             if ( old_version < 0 ) {
                 blob.GetTSE_LoadLock()->SetBlobVersion(version);
             }
-            _ASSERT(blob.GetKnownBlobVersion() == version);
+            if ( blob.GetKnownBlobVersion() != version ) {
+                LOG_POST(Warning<<"GBLoader:"<<blob_id<<" set version "<<version<<", but GetKnownBlobVersion() returns "<<blob.GetKnownBlobVersion());
+            }
         }
     }
     return changed && exp_type == GBL::eExpire_normal;
