@@ -1,4 +1,4 @@
-/*  $Id: dblink_field.cpp 632623 2021-06-03 17:38:11Z ivanov $
+/*  $Id: dblink_field.cpp 642609 2021-12-23 13:29:04Z stakhovv $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -270,7 +270,7 @@ void CDBLinkField::SetConstraint(const string& field_name, CConstRef<CStringCons
     // constraints only apply for authors, since there could be more than one
     m_ConstraintFieldType = GetTypeForLabel(field_name);
     if (m_ConstraintFieldType == eDBLinkFieldType_Unknown || !string_constraint) {
-        string_constraint.Reset(NULL);
+        string_constraint.Reset();
     } else {
         m_StringConstraint = new CStringConstraint(" ");
         m_StringConstraint->Assign(*string_constraint);
@@ -355,7 +355,7 @@ vector<CConstRef<CObject> > CDBLinkField::GetObjects(CSeq_entry_Handle seh, cons
     vector<CConstRef<CObject> > objs;
     CRef<CScope> scope(&seh.GetScope());
 
-    CBioseq_CI bi (seh, objects::CSeq_inst::eMol_na);
+    CBioseq_CI bi (seh, CSeq_inst::eMol_na);
     while (bi) {
         if (NStr::EqualNocase(constraint_field, kFieldTypeSeqId)) {
             if (CSeqIdGuesser::DoesSeqMatchConstraint(*bi, string_constraint)) {
@@ -459,7 +459,7 @@ vector<CConstRef<CObject> > CDBLinkField::GetRelatedObjects(const CApplyObject&
     const CSeq_feat * obj_feat = dynamic_cast<const CSeq_feat *>(&object);
 
    if (obj_feat) {
-        related = GetObjects(object.GetSEH(), "", CRef<CStringConstraint>(NULL));
+        related = GetObjects(object.GetSEH(), "", CRef<CStringConstraint>());
     } else if (obj_desc) {
         if (obj_desc->IsUser() && IsDBLink(obj_desc->GetUser())) {
             CConstRef<CObject> obj(obj_desc);
