@@ -1,4 +1,4 @@
-/*  $Id: feattable_edit.cpp 632623 2021-06-03 17:38:11Z ivanov $
+/*  $Id: feattable_edit.cpp 677913 2024-01-25 14:13:17Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -176,61 +176,73 @@ void CFeatTableEdit::GenerateMissingMrnaForCds()
     sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_cdregion);
     CFeat_CI it(mHandle, sel);
     for (; it; ++it) {
-        const CSeq_feat& cds = it->GetOriginalFeature();
-        CConstRef<CSeq_feat> pOverlappingRna =
-            sequence::GetBestOverlappingFeat(
-                cds.GetLocation(),
-                CSeqFeatData::e_Rna,
-                sequence::eOverlap_CheckIntRev,
-                *mpScope);
-        if (pOverlappingRna) {
-            continue;
-        }
-        CRef<CSeq_feat> pRna(new CSeq_feat);
-        pRna->SetData().SetRna().SetType(CRNA_ref::eType_mRNA);
-        pRna->SetLocation().Assign(cds.GetLocation());
-        pRna->SetLocation().SetPartialStart(false, eExtreme_Positional);
-        pRna->SetLocation().SetPartialStop(false, eExtreme_Positional);
-        pRna->ResetPartial();
-        //product name
-        pRna->SetData().SetRna().SetExt().SetName(
-            sGetCdsProductName(cds, *mpScope));
-
-        //find proper name for rna
-        string rnaId(xNextFeatId());
-        pRna->SetId().SetLocal().SetStr(rnaId);
-
-        //add rna xref to cds
-        CSeq_feat_EditHandle feh(mpScope->GetObjectHandle(cds));
-        feh.AddFeatXref(rnaId);
-
-        //add cds xref to rna
-        CRef<CFeat_id> pFeatId(new CFeat_id);
-        pFeatId->Assign(cds.GetId());
-        CRef<CSeqFeatXref> pRnaXref(new CSeqFeatXref);
-        pRnaXref->SetId(*pFeatId);
-        pRna->SetXref().push_back(pRnaXref);
-
-        CMappedFeat parentGene = feature::GetBestGeneForFeat(*it, &mTree);
-        if (parentGene) {
-            //if gene exists, add gene xref to rna
-            CSeq_feat_EditHandle geh(mpScope->GetObjectHandle(
-                parentGene.GetOriginalFeature()));
-            geh.AddFeatXref(rnaId);
-            //if gene exists, add rna xref to gene
-            CRef<CFeat_id> pGeneId(new CFeat_id);
-            pGeneId->Assign(parentGene.GetId());
-            CRef<CSeqFeatXref> pRnaXref(new CSeqFeatXref);
-            pRnaXref->SetId(*pGeneId);
-            pRna->SetXref().push_back(pRnaXref);
-        }
-
-        //add new rna to feature table
-        mEditHandle.AddFeat(*pRna);
-        mTree.AddFeature(mpScope->GetObjectHandle(*pRna));
+        xGenerateMissingMrnaForCds(*it);
     }
 }
 
+
+//  -------------------------------------------------------------------------
+void CFeatTableEdit::xGenerateMissingMrnaForCds(const CMappedFeat& mappedCds)
+//  -------------------------------------------------------------------------
+{        
+
+    const CSeq_feat& cds = mappedCds.GetOriginalFeature();
+    CConstRef<CSeq_feat> pOverlappingRna =
+        sequence::GetBestOverlappingFeat(
+            cds.GetLocation(),
+            CSeqFeatData::e_Rna,
+            sequence::eOverlap_CheckIntRev,
+            *mpScope);
+    
+    if (pOverlappingRna) {
+        return;
+    }
+
+    CRef<CSeq_feat> pRna(new CSeq_feat);
+    pRna->SetData().SetRna().SetType(CRNA_ref::eType_mRNA);
+    pRna->SetLocation().Assign(cds.GetLocation());
+    pRna->SetLocation().SetPartialStart(false, eExtreme_Positional);
+    pRna->SetLocation().SetPartialStop(false, eExtreme_Positional);
+    pRna->ResetPartial();
+        
+    //product name
+    pRna->SetData().SetRna().SetExt().SetName(
+        sGetCdsProductName(cds, *mpScope));
+
+    //find proper name for rna
+    string rnaId(xNextFeatId());
+    pRna->SetId().SetLocal().SetStr(rnaId);
+
+    //add rna xref to cds
+    CSeq_feat_EditHandle feh(mpScope->GetObjectHandle(cds));
+    feh.AddFeatXref(rnaId);
+
+    //add cds xref to rna
+    CRef<CFeat_id> pFeatId(new CFeat_id);
+    pFeatId->Assign(cds.GetId());
+    CRef<CSeqFeatXref> pRnaXref(new CSeqFeatXref);
+    pRnaXref->SetId(*pFeatId);
+    pRna->SetXref().push_back(pRnaXref);
+
+    CMappedFeat parentGene = feature::GetBestGeneForFeat(mappedCds, &mTree);
+    if (parentGene) {
+        //if gene exists, add gene xref to rna
+        CSeq_feat_EditHandle geh(mpScope->GetObjectHandle(
+                    parentGene.GetOriginalFeature()));
+        geh.AddFeatXref(rnaId);
+        //if gene exists, add rna xref to gene
+        CRef<CFeat_id> pGeneId(new CFeat_id);
+        pGeneId->Assign(parentGene.GetId());
+        CRef<CSeqFeatXref> pRnaXref(new CSeqFeatXref);
+        pRnaXref->SetId(*pGeneId);
+        pRna->SetXref().push_back(pRnaXref);
+    }
+
+    //add new rna to feature table
+    mEditHandle.AddFeat(*pRna);
+    mTree.AddFeature(mpScope->GetObjectHandle(*pRna));
+}
+
 //  -------------------------------------------------------------------------
 void CFeatTableEdit::InferParentMrnas()
 //  -------------------------------------------------------------------------
@@ -309,7 +321,6 @@ void CFeatTableEdit::SubmitFixProducts()
 //  ----------------------------------------------------------------------------
 {
     SAnnotSelector sel;
-    //sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_mRNA);
     sel.IncludeFeatType(CSeqFeatData::e_Rna);
     sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_cdregion);
     for (CFeat_CI it(mHandle, sel); it; ++it) {
@@ -457,25 +468,22 @@ void CFeatTableEdit::GenerateProteinAndTranscriptIds()
 // ---------------------------------------------------------------------------
 {
     mProcessedMrnas.clear();
-
-    { {
-            SAnnotSelector sel;
-            sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_cdregion);
-            for (CFeat_CI it(mHandle, sel); it; ++it) {
-                CMappedFeat cds = *it;
-                xAddTranscriptAndProteinIdsToCdsAndParentMrna(cds);
-            }
-        }}
-
-    { {
-            SAnnotSelector sel;
-            sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_mRNA);
-            for (CFeat_CI it(mHandle, sel); it; ++it) {
-                CMappedFeat mrna = *it;
-                xAddTranscriptAndProteinIdsToUnmatchedMrna(mrna);
-            }
-        }}
-
+    {
+        SAnnotSelector sel;
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_cdregion);
+        for (CFeat_CI it(mHandle, sel); it; ++it) {
+            CMappedFeat cds = *it;
+            xAddTranscriptAndProteinIdsToCdsAndParentMrna(cds);
+        }
+    }
+    {
+        SAnnotSelector sel;
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_mRNA);
+        for (CFeat_CI it(mHandle, sel); it; ++it) {
+            CMappedFeat mrna = *it;
+            xAddTranscriptAndProteinIdsToUnmatchedMrna(mrna);
+        }
+    }
 }
 
 
@@ -502,108 +510,107 @@ static string s_GetTranscriptIdFromMrna(const CMappedFeat& mrna)
     return transcript_id;
 }
 
+static bool s_IsGenbankId(const string& id)
+{
+    return NStr::StartsWith(id, "gb|");
+}
+
+static bool s_IsGeneralId(const string& id)
+{
+    return NStr::StartsWith(id, "gnl|");
+}
 
 // ---------------------------------------------------------------------------
 void CFeatTableEdit::xAddTranscriptAndProteinIdsToCdsAndParentMrna(CMappedFeat& cds)
 // ---------------------------------------------------------------------------
 {
     CMappedFeat mrna = feature::GetBestMrnaForCds(cds, &mTree);
-    string protein_id = cds.GetNamedQual("protein_id");
-    const bool no_protein_id_qual = NStr::IsBlank(protein_id);
-    if (no_protein_id_qual) { // no protein_id qual on CDS - check mRNA and ID qual
+
+    string proteinId = cds.GetNamedQual("protein_id");
+    const bool noProteinIdQual = NStr::IsBlank(proteinId);
+    if (noProteinIdQual) { // no protein_id qual on CDS - check mRNA and ID qual
         if (mrna) {
-            protein_id = mrna.GetNamedQual("protein_id");
+            proteinId = mrna.GetNamedQual("protein_id");
         }
-        if (NStr::IsBlank(protein_id)) {
-            protein_id = cds.GetNamedQual("ID");
+        if (NStr::IsBlank(proteinId)) {
+            proteinId = cds.GetNamedQual("ID");
         }
     }
-    bool is_genbank_protein = NStr::StartsWith(protein_id, "gb|");
 
-    string transcript_id = cds.GetNamedQual("transcript_id");
-    const bool no_transcript_id_qual = NStr::IsBlank(transcript_id);
-    if (no_transcript_id_qual) { // no transcript_id qual on CDS - check mRNA
-        if (mrna) {
-            transcript_id = s_GetTranscriptIdFromMrna(mrna);
-        }
+    const bool isGenbankProtein = s_IsGenbankId(proteinId);
+
+    string transcriptId = cds.GetNamedQual("transcript_id");
+    const bool noTranscriptIdQual = NStr::IsBlank(transcriptId);
+    if (noTranscriptIdQual && mrna) { // no transcript_id qual on CDS - check mRNA
+        transcriptId = s_GetTranscriptIdFromMrna(mrna);
     }
-    bool is_genbank_transcript = NStr::StartsWith(transcript_id, "gb|");
+
+    const bool isGenbankTranscript = s_IsGenbankId(transcriptId);
 
 
-    if ((is_genbank_protein ||
-        NStr::StartsWith(protein_id, "gnl|")) &&
-        (is_genbank_transcript ||
-            NStr::StartsWith(transcript_id, "gnl|"))) { // No further processing of ids is required - simply assign to features
-        if (no_protein_id_qual) { // protein_id from ID qualifier or mRNA => need to add protein_id qual to CDS
-            xFeatureSetQualifier(cds, "protein_id", protein_id);
-        }
-
-        if (no_transcript_id_qual) { // transcript_id from mRNA => need to add transcript_id qual to CDS
-            xFeatureSetQualifier(cds, "transcript_id", transcript_id);
+    if ((isGenbankProtein || s_IsGeneralId(proteinId)) &&
+            (isGenbankTranscript || s_IsGeneralId(transcriptId))) { 
+        
+        // No further processing of ids is required - simply assign to features
+        if (noProteinIdQual) { 
+            // proteinId from ID qualifier or mRNA => need to add protein_id qual to CDS
+            xFeatureSetQualifier(cds, "protein_id", proteinId);
         }
 
         if (mrna) {
-            xAddTranscriptAndProteinIdsToMrna(transcript_id,
-                protein_id,
-                mrna);
+            if (noTranscriptIdQual) { 
+                // transcript_id from mRNA => need to add transcript_id qual to CDS
+                xFeatureSetQualifier(cds, "transcript_id", transcriptId);
+            }   
+            xAddTranscriptAndProteinIdsToMrna(transcriptId, proteinId, mrna);
         }
         return;
     }
 
     // else need to generate and/or process ids before we add them to features
-    bool has_protein_id = !NStr::IsBlank(protein_id);
-    bool has_transcript_id = !NStr::IsBlank(transcript_id);
+    bool hasProteinId = !NStr::IsBlank(proteinId);
+    bool hasTranscriptId = !NStr::IsBlank(transcriptId);
 
-    if (has_protein_id &&
-        has_transcript_id) {
-        if (!is_genbank_protein &&
-            (transcript_id == protein_id)) {
-            protein_id = "cds." + protein_id;
+    if (hasProteinId  &&  hasTranscriptId) {
+        if (!isGenbankProtein  &&  (transcriptId == proteinId)) {
+            proteinId = "cds." + proteinId;
         }
     }
     else {
-        if (has_protein_id &&
-            !is_genbank_protein) {
-            transcript_id = "mrna." + protein_id;
-            has_transcript_id = true;
-
+        if (hasProteinId  &&  !isGenbankProtein) {
+            transcriptId = "mrna." + proteinId;
+            hasTranscriptId = true;
         }
-        else
-            if (has_transcript_id &&
-                !is_genbank_transcript) {
-                protein_id = "cds." + transcript_id;
-                has_protein_id = true;
-            }
-
-        // Generate new transcript_id and protein_id if necessary
-        if (!has_transcript_id) {
-            transcript_id = xNextTranscriptId(cds);
+        else if (hasTranscriptId  &&  !isGenbankTranscript) {
+            proteinId = "cds." + transcriptId;
+            hasProteinId = true;
         }
 
-        if (!has_protein_id) {
-            protein_id = xNextProteinId(cds);
+        // Generate new transcriptId and proteinId if necessary
+        if (!hasTranscriptId && mrna) {
+            transcriptId = xNextTranscriptId(cds);
+        }
+        if (!hasProteinId) {
+            proteinId = xNextProteinId(cds);
         }
     }
 
-    xConvertToGeneralIds(cds, transcript_id, protein_id);
+    xConvertToGeneralIds(cds, transcriptId, proteinId);
 
     if (mrna) {
-        xAddTranscriptAndProteinIdsToMrna(transcript_id,
-            protein_id,
-            mrna);
+        xAddTranscriptAndProteinIdsToMrna(transcriptId, proteinId, mrna);
+        xFeatureSetQualifier(cds, "transcript_id", transcriptId);
     }
-
-
-    xFeatureSetQualifier(cds, "transcript_id", transcript_id);
-    xFeatureSetQualifier(cds, "protein_id", protein_id);
+    xFeatureSetQualifier(cds, "protein_id", proteinId);
 }
 
 
 // ---------------------------------------------------------------------------
-void CFeatTableEdit::xAddTranscriptAndProteinIdsToMrna(const string& cds_transcript_id,
+void CFeatTableEdit::xAddTranscriptAndProteinIdsToMrna(
+    const string& cds_transcript_id,
     const string& cds_protein_id,
     CMappedFeat& mrna)
-    // ---------------------------------------------------------------------------
+// ---------------------------------------------------------------------------
 {
     if (mProcessedMrnas.find(mrna) != mProcessedMrnas.end()) {
         return;
@@ -624,15 +631,15 @@ void CFeatTableEdit::xAddTranscriptAndProteinIdsToMrna(const string& cds_transcr
         protein_id = cds_protein_id;
     }
     else {
-        if ((protein_id == transcript_id) &&
-            !NStr::StartsWith(protein_id, "gb|")) {
+        if ((protein_id == transcript_id)  &&  !NStr::StartsWith(protein_id, "gb|")) {
             protein_id = "cds." + protein_id;
         }
         use_local_ids = true;
     }
 
-    if (use_local_ids) { // protein id and/or transcript id specified on mRNA.
-                         // Process these ids and check that the processed quals match the qualifiers on the child CDS feature
+    if (use_local_ids) { 
+        // protein id and/or transcript id specified on mRNA.
+        // Process these ids and check that the processed quals match the qualifiers on the child CDS feature
         xConvertToGeneralIds(mrna, transcript_id, protein_id);
 
         if (transcript_id != cds_transcript_id) {
@@ -659,53 +666,51 @@ void CFeatTableEdit::xAddTranscriptAndProteinIdsToUnmatchedMrna(CMappedFeat& mrn
         return;
     }
 
-    string transcript_id = mrna.GetNamedQual("transcript_id");
-    bool no_transcript_id_qual = NStr::IsBlank(transcript_id);
-    if (no_transcript_id_qual) {
-        transcript_id = mrna.GetNamedQual("ID");
+    string transcriptId = mrna.GetNamedQual("transcript_id");
+    bool noTranscriptIdQual = NStr::IsBlank(transcriptId);
+    if (noTranscriptIdQual) {
+        transcriptId = mrna.GetNamedQual("ID");
     }
-    const bool is_genbank_transcript = NStr::StartsWith(transcript_id, "gb|");
+    const bool isGenbankTranscript = s_IsGenbankId(transcriptId);
 
-    string protein_id = mrna.GetNamedQual("protein_id");
-    const bool is_genbank_protein = NStr::StartsWith(protein_id, "gb|");
+    string proteinId = mrna.GetNamedQual("protein_id");
+    const bool isGenbankProtein = s_IsGenbankId(proteinId);
 
-    if ((is_genbank_protein ||
-        NStr::StartsWith(protein_id, "gnl|")) &&
-        (is_genbank_transcript ||
-            NStr::StartsWith(transcript_id, "gnl|"))) {
-        if (no_transcript_id_qual) {
-            xFeatureSetQualifier(mrna, "transcript_id", transcript_id);
+    if ((isGenbankProtein || s_IsGeneralId(proteinId)) &&
+        (isGenbankTranscript || s_IsGeneralId(transcriptId))) {
+        if (noTranscriptIdQual) {
+            xFeatureSetQualifier(mrna, "transcript_id", transcriptId);
         }
         return;
     }
 
-    if (!NStr::IsBlank(protein_id) &&
-        !NStr::IsBlank(transcript_id)) {
-        if ((transcript_id == protein_id) &&
-            !is_genbank_transcript) {
-            protein_id = "cds." + protein_id;
+    if (!NStr::IsBlank(proteinId) &&
+        !NStr::IsBlank(transcriptId)) {
+        if ((transcriptId == proteinId) &&
+            !isGenbankTranscript) {
+            proteinId = "cds." + proteinId;
         }
     }
-    else
-        if (!is_genbank_protein && !NStr::IsBlank(protein_id)) {
-            transcript_id = "mrna." + protein_id;
+    else { // Empty proteinId and/or transcriptId
+        if (!isGenbankProtein && !NStr::IsBlank(proteinId)) {
+            transcriptId = "mrna." + proteinId;
+        }
+        else if (!isGenbankTranscript && !NStr::IsBlank(transcriptId)) {
+            proteinId = "cds." + transcriptId;
         }
-        else
-            if (!is_genbank_transcript && !NStr::IsBlank(transcript_id)) {
-                protein_id = "cds." + transcript_id;
-            }
-
-    if (NStr::IsBlank(protein_id)) {
-        protein_id = xNextProteinId(mrna);
-    }
-    if (NStr::IsBlank(transcript_id)) {
-        transcript_id = xNextTranscriptId(mrna);
     }
 
-    xConvertToGeneralIds(mrna, transcript_id, protein_id);
+    if (NStr::IsBlank(proteinId)) {
+        proteinId = xNextProteinId(mrna);
+    }
+    if (NStr::IsBlank(transcriptId)) {
+        transcriptId = xNextTranscriptId(mrna);
+    }
 
-    xFeatureSetQualifier(mrna, "transcript_id", transcript_id);
-    xFeatureSetQualifier(mrna, "protein_id", protein_id);
+    xConvertToGeneralIds(mrna, transcriptId, proteinId);
+
+    xFeatureSetQualifier(mrna, "transcript_id", transcriptId);
+    xFeatureSetQualifier(mrna, "protein_id", proteinId);
 
     mProcessedMrnas.insert(mrna);
 }
@@ -776,35 +781,55 @@ void CFeatTableEdit::InstantiateProducts()
 
 //  ---------------------------------------------------------------------------
 void CFeatTableEdit::xGenerateMissingGeneForChoice(
-    CSeqFeatData::E_Choice choice)
+    CSeqFeatData::E_Choice choice,
+    const CGff3LocationMerger* pMerger)
     //  ---------------------------------------------------------------------------
 {
     SAnnotSelector sel;
     sel.IncludeFeatType(choice);
+    xGenerateMissingGeneForFeats(sel, pMerger);
+}
+
+
+//  ---------------------------------------------------------------------------
+void CFeatTableEdit::xGenerateMissingGeneForFeats(
+    const SAnnotSelector& sel,
+    const CGff3LocationMerger* pMerger)
+//  ---------------------------------------------------------------------------
+{
     CFeat_CI it(mHandle, sel);
     for (; it; ++it) {
         CMappedFeat mf = *it;
-        if (xCreateMissingParentGene(mf)) {
+        auto seqId = mf.GetLocationId().AsString();
+        auto sequenceSize = 
+            (pMerger ? pMerger->GetSequenceSize(seqId) : mSequenceSize);
+        if (xCreateMissingParentGene(mf, sequenceSize)) {
             xAdjustExistingParentGene(mf);
         }
     }
 }
 
 
+//  ---------------------------------------------------------------------------
+void CFeatTableEdit::xGenerateMissingGeneForFeat(
+        const CMappedFeat& feat)
+//  ---------------------------------------------------------------------------
+{
+    if (xCreateMissingParentGene(feat, mSequenceSize)) {
+        xAdjustExistingParentGene(feat);
+    }
+}
+
+
 //  ---------------------------------------------------------------------------
 void CFeatTableEdit::xGenerateMissingGeneForSubtype(
-    CSeqFeatData::ESubtype subType)
-    //  ---------------------------------------------------------------------------
+    CSeqFeatData::ESubtype subType,
+    const CGff3LocationMerger* pMerger)
+//  ---------------------------------------------------------------------------
 {
     SAnnotSelector sel;
     sel.IncludeFeatSubtype(subType);
-    CFeat_CI it(mHandle, sel);
-    for (; it; ++it) {
-        CMappedFeat mf = *it;
-        if (xCreateMissingParentGene(mf)) {
-            xAdjustExistingParentGene(mf);
-        }
-    }
+    xGenerateMissingGeneForFeats(sel, pMerger);
 }
 
 //  ----------------------------------------------------------------------------
@@ -836,10 +861,14 @@ CFeatTableEdit::xAdjustExistingParentGene(
 //  ----------------------------------------------------------------------------
 bool
 CFeatTableEdit::xCreateMissingParentGene(
-    CMappedFeat mf)
+    CMappedFeat mf,
+    TSeqPos sequenceSize)
     //  ----------------------------------------------------------------------------
 {
-    CRef<CSeq_feat> pGene = xMakeGeneForFeature(mf);
+    if (sequenceSize == 0) {
+        sequenceSize = mSequenceSize;
+    }
+    CRef<CSeq_feat> pGene = xMakeGeneForFeature(mf, sequenceSize);
     if (!pGene) {
         return false;
     }
@@ -1391,34 +1420,62 @@ void CFeatTableEdit::GenerateLocusTags()
 
 //  ----------------------------------------------------------------------------
 void CFeatTableEdit::GenerateMissingParentFeatures(
-    bool forEukaryote)
+    bool forEukaryote,
+    const CGff3LocationMerger* pMerger)
     //  ----------------------------------------------------------------------------
 {
     if (forEukaryote) {
-        GenerateMissingParentFeaturesForEukaryote();
+        GenerateMissingParentFeaturesForEukaryote(pMerger);
     }
-    else {
-        GenerateMissingParentFeaturesForProkaryote();
+    else {       
+        GenerateMissingParentFeaturesForProkaryote(pMerger);
     }
     mTree = feature::CFeatTree(mHandle);
 }
 
 
 //  ----------------------------------------------------------------------------
-void CFeatTableEdit::GenerateMissingParentFeaturesForEukaryote()
+void CFeatTableEdit::GenerateMissingParentFeaturesForEukaryote(
+    const CGff3LocationMerger* pMerger)
 //  ----------------------------------------------------------------------------
 {
-    GenerateMissingMrnaForCds();
-    xGenerateMissingGeneForChoice(CSeqFeatData::e_Rna);
+    {
+        SAnnotSelector sel;
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_cdregion);
+        for (CFeat_CI it(mHandle, sel); it; ++it) {
+            if (auto parent = mTree.GetParent(*it); parent) {
+                auto subtype = parent.GetOriginalFeature().GetData().GetSubtype();
+                if (subtype == CSeqFeatData::eSubtype_mRNA  ||
+                    subtype == CSeqFeatData::eSubtype_V_segment ||
+                    subtype == CSeqFeatData::eSubtype_D_segment ||
+                    subtype == CSeqFeatData::eSubtype_J_segment ||
+                    subtype == CSeqFeatData::eSubtype_C_region) {
+                        continue;
+                }
+            }
+            xGenerateMissingMrnaForCds(*it);
+        }
+    }
+
+    {
+        SAnnotSelector sel;
+        sel.IncludeFeatType(CSeqFeatData::e_Rna);
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_V_segment);
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_D_segment);
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_J_segment);
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_C_region);
+        xGenerateMissingGeneForFeats(sel, nullptr); //sequence not circular for eukaryote
+    }
 }
 
 
 //  ----------------------------------------------------------------------------
-void CFeatTableEdit::GenerateMissingParentFeaturesForProkaryote()
+void CFeatTableEdit::GenerateMissingParentFeaturesForProkaryote(
+    const CGff3LocationMerger* pMerger)
 //  ----------------------------------------------------------------------------
 {
-    xGenerateMissingGeneForChoice(CSeqFeatData::e_Cdregion);
-    xGenerateMissingGeneForChoice(CSeqFeatData::e_Rna);
+    xGenerateMissingGeneForChoice(CSeqFeatData::e_Cdregion, pMerger);
+    xGenerateMissingGeneForChoice(CSeqFeatData::e_Rna, pMerger);
 }
 
 
@@ -1589,7 +1646,8 @@ string CFeatTableEdit::xNextTranscriptId(
 
 //  ----------------------------------------------------------------------------
 CRef<CSeq_loc> CFeatTableEdit::xGetGeneLocation(
-    const CSeq_loc& baseLoc)
+    const CSeq_loc& baseLoc,
+    TSeqPos sequenceSize)
 //  ----------------------------------------------------------------------------
 {
     CRef<CSeq_loc> pEnvelope(new CSeq_loc);
@@ -1598,7 +1656,7 @@ CRef<CSeq_loc> CFeatTableEdit::xGetGeneLocation(
     auto baseStop = baseLoc.GetStop(eExtreme_Positional);
     auto& baseId = *baseLoc.GetId();
 
-    if (mSequenceSize == 0  ||  baseStart <= baseStop) {
+    if (sequenceSize == 0  ||  baseStart <= baseStop) {
         pEnvelope->SetInt();
         pEnvelope->SetId(baseId);
         pEnvelope->SetInt().SetFrom(baseStart);
@@ -1610,7 +1668,7 @@ CRef<CSeq_loc> CFeatTableEdit::xGetGeneLocation(
             CRef<CSeq_interval> pTop(new CSeq_interval);
             pTop->SetId().Assign(baseId);
             pTop->SetFrom(baseStart);
-            pTop->SetTo(mSequenceSize);
+            pTop->SetTo(sequenceSize-1);
             pTop->SetStrand(baseLoc.GetStrand());
             pEnvelope->SetPacked_int().AddInterval(*pTop);
             CRef<CSeq_interval> pBottom(new CSeq_interval);
@@ -1631,7 +1689,7 @@ CRef<CSeq_loc> CFeatTableEdit::xGetGeneLocation(
             CRef<CSeq_interval> pTop(new CSeq_interval);
             pTop->SetId().Assign(baseId);
             pTop->SetFrom(baseStart);
-            pTop->SetTo(mSequenceSize);
+            pTop->SetTo(sequenceSize-1);
             pTop->SetStrand(baseLoc.GetStrand());
             pEnvelope->SetPacked_int().AddInterval(*pTop);
             pEnvelope->ChangeToMix();
@@ -1642,11 +1700,11 @@ CRef<CSeq_loc> CFeatTableEdit::xGetGeneLocation(
 
 //  ----------------------------------------------------------------------------
 CRef<CSeq_feat> CFeatTableEdit::xMakeGeneForFeature(
-    const CMappedFeat& rna)
+    const CMappedFeat& rna,
+    TSeqPos sequenceSize)
 //  ----------------------------------------------------------------------------
 {
     CRef<CSeq_feat> pGene;
-    //const CSeq_loc& loc = rna.GetOriginalFeature().GetLocation();
     CSeq_feat_Handle sfh = mpScope->GetSeq_featHandle(rna.GetOriginalFeature());
     CSeq_annot_Handle sah = sfh.GetAnnot();
     if (!sah) {
@@ -1657,7 +1715,7 @@ CRef<CSeq_feat> CFeatTableEdit::xMakeGeneForFeature(
         return pGene;
     }
     pGene.Reset(new CSeq_feat);
-    pGene->SetLocation(*xGetGeneLocation(rna.GetLocation()));
+    pGene->SetLocation(*xGetGeneLocation(rna.GetLocation(), sequenceSize));
     pGene->SetData().SetGene();
     return pGene;
 }
@@ -1696,7 +1754,6 @@ CFeatTableEdit::xPutErrorMissingLocustag(
         subName;
 
     string message = subName + " feature is missing locus tag.";
-
     xPutError(message);
 }
 
@@ -1718,7 +1775,6 @@ CFeatTableEdit::xPutErrorMissingTranscriptId(
         subName;
 
     string message = subName + " feature is missing transcript ID.";
-
     xPutError(message);
 }
 
@@ -1772,9 +1828,6 @@ CFeatTableEdit::xPutErrorDifferingProteinIds(
         return;
     }
     xPutError("Protein ID on mRNA feature differs from protein ID on child CDS.");
-
-    //    xPutError(ILineError::eProblem_InconsistentQualifiers,
-    //        "Protein ID on mRNA feature differs from protein ID on child CDS.");
 }
 
 void
@@ -1949,7 +2002,7 @@ static void s_SetProtRef(const CSeq_feat& cds,
     } // prot_ref.IsSetName()
 }
 
-static bool AssignLocalIdIfEmpty(ncbi::objects::CSeq_feat& feature, unsigned& id)
+static bool AssignLocalIdIfEmpty(CSeq_feat& feature, unsigned& id)
 {
     if (feature.IsSetId())
         return true;
