@@ -1,4 +1,4 @@
-/*  $Id: gaps_edit.cpp 632623 2021-06-03 17:38:11Z ivanov $
+/*  $Id: gaps_edit.cpp 642609 2021-12-23 13:29:04Z stakhovv $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -400,7 +400,7 @@ void CGapsEditor::x_SetGapParameters(CDelta_seq& lit, bool length_unknown)
     }
 }
 
-void CGapsEditor::ConvertNs2Gaps(objects::CSeq_entry& entry)
+void CGapsEditor::ConvertNs2Gaps(CSeq_entry& entry)
 {
     if (m_gapNmin == 0 && m_gap_Unknown_length > 0)
         return;
