@@ -1,4 +1,4 @@
-/*  $Id: source_edit.cpp 632623 2021-06-03 17:38:11Z ivanov $
+/*  $Id: source_edit.cpp 642609 2021-12-23 13:29:04Z stakhovv $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -48,7 +48,7 @@ BEGIN_SCOPE(objects)
 BEGIN_SCOPE(edit)
 
 
-bool CleanupForTaxnameChange( objects::CBioSource& src )
+bool CleanupForTaxnameChange( CBioSource& src )
 {
     bool rval = RemoveOldName(src);
     rval |= RemoveMod(src, COrgMod::eSubtype_type_material);
@@ -64,12 +64,12 @@ bool CleanupForTaxnameChange( objects::CBioSource& src )
     return rval;
 }
 
-bool RemoveOldName( objects::CBioSource& src )
+bool RemoveOldName( CBioSource& src )
 {
     return RemoveMod(src, COrgMod::eSubtype_old_name);
 }
 
-bool RemoveMod( objects::CBioSource& src, objects::COrgMod::ESubtype subtype )
+bool RemoveMod( CBioSource& src, COrgMod::ESubtype subtype )
 {
     bool erased = false;
     if (src.IsSetOrg() && src.GetOrg().IsSetOrgname()
@@ -90,7 +90,7 @@ bool RemoveMod( objects::CBioSource& src, objects::COrgMod::ESubtype subtype )
     return erased;
 }
 
-bool RemoveTaxId( objects::CBioSource& src )
+bool RemoveTaxId( CBioSource& src )
 {
     bool erased = false;
     if (src.IsSetOrg() && src.GetOrg().IsSetDb()) {
@@ -159,16 +159,15 @@ void AddMissingCommonOrgMods(const COrg_ref& o1, const COrg_ref& o2, COrg_ref& c
 }
 
 
-CRef<CBioSource> MakeCommonBioSource(const objects::CBioSource& src1, const objects::CBioSource& src2)
+CRef<CBioSource> MakeCommonBioSource(const CBioSource& src1, const CBioSource& src2)
 {
-    CRef<CBioSource> common(NULL);
+    CRef<CBioSource> common;
 
     if (!src1.IsSetOrg() || !src2.IsSetOrg()) {
         return common;
     }
 
-    CTaxon3 taxon3;
-    taxon3.Init();
+    CTaxon3 taxon3(CTaxon3::initialize::yes);
 
     // do lookup before attempting to merge
     vector<CRef<COrg_ref> > rq_list;
@@ -181,7 +180,7 @@ CRef<CBioSource> MakeCommonBioSource(const objects::CBioSource& src1, const obje
     CRef<CTaxon3_reply> reply = taxon3.SendOrgRefList(rq_list);
     if (!reply || reply->GetReply().size() != 2) {
         ERR_POST("Taxonomy service failure");
-        return CRef<CBioSource>(NULL);
+        return CRef<CBioSource>();
     }
     if (!s_ProcessReply(*(reply->GetReply().front()), o1) ||
         !s_ProcessReply(*(reply->GetReply().back()), o2)) {
@@ -214,7 +213,7 @@ CRef<CBioSource> MakeCommonBioSource(const objects::CBioSource& src1, const obje
         CRef<CTaxon3_reply> reply = taxon3.SendRequest(request);
         if (!reply || reply->GetReply().size() != 1) {
             ERR_POST("Taxonomy service failure" + err_nums);
-            return CRef<CBioSource>(NULL);
+            return CRef<CBioSource>();
         }
         const CT3Reply& join_reply = *(reply->GetReply().front());
         if (join_reply.IsData()) {
