@@ -1,4 +1,4 @@
-/*  $Id: context.cpp 607319 2020-05-02 00:45:07Z kans $
+/*  $Id: context.cpp 665431 2023-04-04 17:43:43Z stakhovv $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -80,7 +80,7 @@ CBioseqContext::CBioseqContext
  CMasterContext* mctx,
  CTopLevelSeqEntryContext *tlsec) :
     m_Handle(seq),
-    m_pOpticalMapPoints(NULL),
+    m_pOpticalMapPoints(nullptr),
     m_Repr(CSeq_inst::eRepr_not_set),
     m_Mol(CSeq_inst::eMol_not_set),
     m_HasParts(false),
@@ -122,10 +122,11 @@ CBioseqContext::CBioseqContext
     m_IsCrossKingdom(false),
     m_UsePDBCompoundForComment(false),
     m_fUnverified(fUnverified_None),
+    m_fUnreviewed(fUnreviewed_None),
     m_ShowAnnotCommentAsCOMMENT(false),
     m_ShowAnnotCommentAsCOMMENT_checked(false),
     m_FFCtx(ffctx),
-    m_RefCache(0),
+    m_RefCache(nullptr),
     m_Master(mctx),
     m_TLSeqEntryCtx(tlsec)
 {
@@ -134,7 +135,7 @@ CBioseqContext::CBioseqContext
 
 
 CBioseqContext::CBioseqContext
-(const CBioseq_Handle& prev_seq, 
+(const CBioseq_Handle& prev_seq,
  const CBioseq_Handle& seq,
  const CBioseq_Handle& next_seq,
  CFlatFileContext& ffctx,
@@ -143,7 +144,7 @@ CBioseqContext::CBioseqContext
     m_PrevHandle(prev_seq),
     m_Handle(seq),
     m_NextHandle(next_seq),
-    m_pOpticalMapPoints(NULL),
+    m_pOpticalMapPoints(nullptr),
     m_Repr(CSeq_inst::eRepr_not_set),
     m_Mol(CSeq_inst::eMol_not_set),
     m_HasParts(false),
@@ -185,10 +186,11 @@ CBioseqContext::CBioseqContext
     m_IsCrossKingdom(false),
     m_UsePDBCompoundForComment(false),
     m_fUnverified(fUnverified_None),
+    m_fUnreviewed(fUnreviewed_None),
     m_ShowAnnotCommentAsCOMMENT(false),
     m_ShowAnnotCommentAsCOMMENT_checked(false),
     m_FFCtx(ffctx),
-    m_RefCache(0),
+    m_RefCache(nullptr),
     m_Master(mctx),
     m_TLSeqEntryCtx(tlsec)
 {
@@ -249,12 +251,11 @@ void CBioseqContext::x_Init(const CBioseq_Handle& seq, const CSeq_loc* user_loc)
     m_IsInNucProt = x_IsInNucProt();
 
     x_SetLocation(user_loc);
-    
     x_SetDataFromUserObjects();
 
     m_ShowAnnotCommentAsCOMMENT = false;
     m_ShowAnnotCommentAsCOMMENT_checked = false;
-    
+
     // m_HasOperon = x_HasOperon();
 
     if (IsRefSeq()) {
@@ -276,7 +277,7 @@ void CBioseqContext::x_SetLocation(const CSeq_loc* user_loc)
 {
     CRef<CSeq_loc> loc;
 
-    if (user_loc != NULL) {
+    if (user_loc) {
         // map the user location to the current bioseq
         CSeq_id_Handle idh1 = CSeq_id_Handle::GetHandle(*m_Handle.GetSeqId());
         CSeq_id_Handle idh2 = sequence::GetIdHandle(*user_loc, &m_Handle.GetScope());
@@ -292,7 +293,7 @@ void CBioseqContext::x_SetLocation(const CSeq_loc* user_loc)
             if (loc->IsWhole()) {
                 loc.Reset();
             } else if (loc->IsInt()) {
-                CSeq_loc::TRange range = loc->GetTotalRange();               
+                CSeq_loc::TRange range = loc->GetTotalRange();
                 if (!IsReverse(loc->GetStrand()) && range.GetFrom() == 0 && range.GetTo() == m_Handle.GetInst_Length() - 1) {
                     loc.Reset();
                 }
@@ -495,8 +496,8 @@ void CBioseqContext::SetPDBCompoundForComment(bool value)
 
 void CBioseqContext::x_SetFiletrackURL(const CUser_object& uo)
 {
-    if ( ! FIELD_IS_SET_AND_IS(uo, Type, Str) || 
-        ! NStr::EqualNocase(uo.GetType().GetStr(), "FileTrack")) 
+    if ( ! FIELD_IS_SET_AND_IS(uo, Type, Str) ||
+        ! NStr::EqualNocase(uo.GetType().GetStr(), "FileTrack"))
     {
         return;
     }
@@ -535,14 +536,14 @@ void CBioseqContext::x_SetFiletrackURL(const CUser_object& uo)
 
 void CBioseqContext::x_SetAuthorizedAccess(const CUser_object& uo)
 {
-    if ( ! FIELD_IS_SET_AND_IS(uo, Type, Str) || 
-        ! NStr::EqualNocase(uo.GetType().GetStr(), "AuthorizedAccess")) 
+    if ( ! FIELD_IS_SET_AND_IS(uo, Type, Str) ||
+        ! NStr::EqualNocase(uo.GetType().GetStr(), "AuthorizedAccess"))
     {
         return;
     }
     CConstRef<CUser_field> pAuthorizedAccessField =
         uo.GetFieldRef("Study");
-    if( ! pAuthorizedAccessField || 
+    if( ! pAuthorizedAccessField ||
         ! FIELD_IS_SET_AND_IS(*pAuthorizedAccessField, Data, Str) ||
         pAuthorizedAccessField->GetData().GetStr().empty() )
     {
@@ -552,8 +553,8 @@ void CBioseqContext::x_SetAuthorizedAccess(const CUser_object& uo)
 }
 
 void CBioseqContext::x_SetOpticalMapPoints(void)
-{  
-    if( GetRepr() != CSeq_inst::eRepr_map || 
+{
+    if( GetRepr() != CSeq_inst::eRepr_map ||
         ! FIELD_IS_SET_AND_IS(m_Handle, Inst_Ext, Map) )
     {
        return;
@@ -562,7 +563,7 @@ void CBioseqContext::x_SetOpticalMapPoints(void)
     const CMap_ext & map_ext = m_Handle.GetInst_Ext().GetMap();
     FOR_EACH_SEQFEAT_ON_MAPEXT(feat_it, map_ext ) {
         const CSeq_feat & feat = **feat_it;
-        if( ! FIELD_IS_SET_AND_IS(feat, Data, Rsite) || 
+        if( ! FIELD_IS_SET_AND_IS(feat, Data, Rsite) ||
             ! feat.IsSetLocation() )
         {
             continue;
@@ -574,7 +575,7 @@ void CBioseqContext::x_SetOpticalMapPoints(void)
 
             if( seq_point.IsSetPoint() ) {
                 m_pOpticalMapPointsDestroyer.reset( new CPacked_seqpnt );
-                CLONE_IF_SET_ELSE_RESET(*m_pOpticalMapPointsDestroyer, Fuzz, 
+                CLONE_IF_SET_ELSE_RESET(*m_pOpticalMapPointsDestroyer, Fuzz,
                     seq_point, Fuzz);
                 CLONE_IF_SET_ELSE_RESET(*m_pOpticalMapPointsDestroyer, Id,
                     seq_point, Id);
@@ -589,7 +590,7 @@ void CBioseqContext::x_SetOpticalMapPoints(void)
         case CSeq_loc::e_Packed_pnt:
             m_pOpticalMapPoints = & feat_loc.GetPacked_pnt();
             // in case a previous iteration set this
-            m_pOpticalMapPointsDestroyer.reset(); 
+            m_pOpticalMapPointsDestroyer.reset();
             break;
         default:
             // ignore other types
@@ -624,15 +625,15 @@ void CBioseqContext::x_SetDataFromUserObjects(void)
                             ! field.IsSetLabel() || ! field.GetLabel().IsStr() ) {
                             continue;
                         }
-                        if( field.GetLabel().GetStr() == "StructuredCommentPrefix" && 
-                            field.GetData().GetStr() == "##Genome-Assembly-Data-START##" ) 
+                        if( field.GetLabel().GetStr() == "StructuredCommentPrefix" &&
+                            field.GetData().GetStr() == "##Genome-Assembly-Data-START##" )
                         {
                             m_IsGenomeAssembly = true;
                         }
                         if( field.GetLabel().GetStr() == "Current Finishing Status" )
                         {
                             string asn_fin_stat = field.GetData().GetStr();
-                            replace( asn_fin_stat.begin(), asn_fin_stat.end(), ' ', '-' ); 
+                            replace( asn_fin_stat.begin(), asn_fin_stat.end(), ' ', '-' );
                             TFinStatMap::const_iterator new_fin_stat_iter = sc_FinStatMap.find(asn_fin_stat.c_str());
                             if( new_fin_stat_iter != sc_FinStatMap.end() ) {
                                 m_FinishingStatus = new_fin_stat_iter->second;
@@ -657,6 +658,10 @@ void CBioseqContext::x_SetDataFromUserObjects(void)
                 if( m_fUnverified == fUnverified_None ) {
                     m_fUnverified = fUnverified_SequenceOrAnnotation;
                 }
+            } else if (utype == CUser_object::eObjectType_Unreviewed) {
+                if (uo.IsUnreviewedUnannotated()) {
+                    m_fUnreviewed |= fUnreviewed_Unannotated;
+                }
             } else if ( utype == CUser_object::eObjectType_FileTrack ) {
                 x_SetFiletrackURL(uo);
             } else if ( NStr::EqualNocase(uo.GetType().GetStr(), "AuthorizedAccess") ) {
@@ -707,7 +712,7 @@ void CBioseqContext::x_CheckForShowComments() const
             // we finally got down to an annot desc user object.  See if it indicates any
             // relevant information
             const CUser_object & user_obj = one_desc.GetUser();
-            if( ! user_obj.IsSetType() || ! user_obj.GetType().IsStr() || 
+            if( ! user_obj.IsSetType() || ! user_obj.GetType().IsStr() ||
                 ! user_obj.IsSetData() ||
                 user_obj.GetType().GetStr() != "AnnotDescCommentPolicy" )
             {
@@ -724,7 +729,7 @@ void CBioseqContext::x_CheckForShowComments() const
                     continue;
                 }
 
-                if( policy_field.GetData().IsStr() ) { 
+                if( policy_field.GetData().IsStr() ) {
                     const string & policy_str = policy_field.GetData().GetStr();
                     if( policy_str == "ShowInComment" ) {
                         m_ShowAnnotCommentAsCOMMENT = true;
@@ -779,18 +784,18 @@ void CBioseqContext::x_SetId(void)
         }
         if ( obj.GetType().GetStr() == "TpaAssembly" ) {
             bTpaAssemblyPresent = true;
-            continue; 
+            continue;
         }
         if ( obj.GetType().GetStr() == "GenomeProjectsDB" ) {
             m_IsGbGenomeProject = true;
-            continue; 
+            continue;
         }
     }
 
     ITERATE (CBioseq::TId, id_iter, m_Handle.GetBioseqCore()->GetId()) {
         const CSeq_id& id = **id_iter;
         const CTextseq_id* tsip = id.GetTextseq_Id();
-        const string& acc = (tsip != 0  &&  tsip->CanGetAccession()) ?
+        const string& acc = (tsip && tsip->CanGetAccession()) ?
             tsip->GetAccession() : kEmptyStr;
 
         CSeq_id::EAccessionInfo acc_info = id.IdentifyAccession();
@@ -880,11 +885,11 @@ void CBioseqContext::x_SetId(void)
 
         // WGS
         m_IsWGS = m_IsWGS  ||  (acc_div == CSeq_id::eAcc_wgs);
-        
+
         if ( m_IsWGS  &&  !acc.empty() ) {
             /*
             size_t len = acc.length();
-            m_IsWGSMaster = 
+            m_IsWGSMaster =
                 ((len == 12  ||  len == 15)  &&  NStr::EndsWith(acc, "000000"))  ||
                 ((len == 14)  &&  NStr::EndsWith(acc, "00000000")) ||
                 ((len == 13 || len == 16 || len == 17)  &&  NStr::EndsWith(acc, "0000000"));
@@ -894,11 +899,11 @@ void CBioseqContext::x_SetId(void)
                 m_WGSMasterAccn = acc;
                 m_WGSMasterName = tsip->CanGetName() ? tsip->GetName() : kEmptyStr;
             }
-        } 
+        }
 
         // TSA
         m_IsTSA = m_IsTSA  ||  ( GetTech() == CMolInfo::eTech_tsa );
-        
+
         if ( m_IsTSA  &&  !acc.empty() ) {
             if ( m_Repr == CSeq_inst::eRepr_virtual) {
                 m_IsTSAMaster = true;
@@ -907,7 +912,7 @@ void CBioseqContext::x_SetId(void)
                 m_TSAMasterAccn = acc;
                 m_TSAMasterName = tsip->CanGetName() ? tsip->GetName() : kEmptyStr;
             }
-        } 
+        }
 
         // TLS
         m_IsTLS = m_IsTLS  ||  ( GetTech() == CMolInfo::eTech_targeted );
@@ -951,7 +956,7 @@ CSeq_inst::TRepr CBioseqContext::x_GetRepr(void) const
 const CMolInfo* CBioseqContext::x_GetMolInfo(void) const
 {
     CSeqdesc_CI desc(m_Handle, CSeqdesc::e_Molinfo);
-    return desc ? &desc->GetMolinfo() : 0;
+    return desc ? &desc->GetMolinfo() : nullptr;
 }
 
 
@@ -989,7 +994,7 @@ bool CBioseqContext::x_HasParts(void) const
     if ( !h ) {
         return false;
     }
-    
+
     // make sure the segmented set contains our bioseq
     {{
         bool has_seq = false;
@@ -1046,7 +1051,7 @@ SIZE_TYPE CBioseqContext::x_GetPartNumber(void)
 
 bool CBioseqContext::x_IsInSGS(void) const
 {
-    CSeq_entry_Handle e = 
+    CSeq_entry_Handle e =
         m_Handle.GetExactComplexityLevel(CBioseq_set::eClass_small_genome_set);
     return e;
 }
@@ -1054,7 +1059,7 @@ bool CBioseqContext::x_IsInSGS(void) const
 
 bool CBioseqContext::x_IsInGPS(void) const
 {
-    CSeq_entry_Handle e = 
+    CSeq_entry_Handle e =
         m_Handle.GetExactComplexityLevel(CBioseq_set::eClass_gen_prod_set);
     return e;
 }
@@ -1062,7 +1067,7 @@ bool CBioseqContext::x_IsInGPS(void) const
 
 bool CBioseqContext::x_IsInNucProt(void) const
 {
-    CSeq_entry_Handle e = 
+    CSeq_entry_Handle e =
         m_Handle.GetExactComplexityLevel(CBioseq_set::eClass_nuc_prot);
     return e;
 }
@@ -1160,7 +1165,7 @@ void CMasterContext::x_SetNumParts(void)
         if (part  &&
             part.IsSetInst_Repr()  &&
             part.GetInst_Repr() != CSeq_inst::eRepr_virtual) {
-            ++count;    
+            ++count;
         }
     }
     m_NumParts = count;
@@ -1183,7 +1188,7 @@ static void s_GetNameForBioseq(const CBioseq_Handle& seq, string& name)
 
     if (sip) {
         const CTextseq_id* tsip = sip->GetTextseq_Id();
-        if (tsip != NULL  &&  tsip->CanGetName()) {
+        if (tsip && tsip->CanGetName()) {
             name = tsip->GetName();
         }
     }
@@ -1245,11 +1250,69 @@ void CMasterContext::x_SetBaseName(void)
 //
 // CTopLevelSeqEntryContext
 
-CTopLevelSeqEntryContext::CTopLevelSeqEntryContext( const CSeq_entry_Handle &entry_handle )
+void CTopLevelSeqEntryContext::x_InitSeqs(const CSeq_entry& sep)
+{
+    if (sep.IsSeq()) {
+        // Is Bioseq
+        const CBioseq& bsp = sep.GetSeq();
+        for (auto& sid : bsp.GetId()) {
+            TSEQID_CHOICE chs = sid->Which();
+            switch (chs) {
+                case CSeq_id_Base::e_Genbank:
+                case CSeq_id_Base::e_Tpg:
+                    // Genbank allows merging only if it's the old-style 1 + 5 accessions
+                    {
+                        const CTextseq_id* tsid = sid->GetTextseq_Id ();
+                        if (tsid && tsid->IsSetAccession() && tsid->GetAccession().length() == 6) {
+                            m_CanSourcePubsBeFused = true;
+                        }
+                    }
+                    break;
+                case CSeq_id_Base::e_Embl:
+                case CSeq_id_Base::e_Ddbj:
+                case CSeq_id_Base::e_Pir:
+                case CSeq_id_Base::e_Swissprot:
+                case CSeq_id_Base::e_Prf:
+                case CSeq_id_Base::e_Pdb:
+                case CSeq_id_Base::e_Tpe:
+                case CSeq_id_Base::e_Tpd:
+                case CSeq_id_Base::e_Gpipe:
+                case CSeq_id_Base::e_Patent:
+                case CSeq_id_Base::e_Gibbsq:
+                case CSeq_id_Base::e_Gibbmt:
+                    // with some types, it's okay to merge
+                    m_CanSourcePubsBeFused = true;
+                    break;
+                default:
+                    break;
+            }
+        }
+    } else if (sep.IsSet()) {
+        // Is Bioseq-set
+        const CBioseq_set& bssp = sep.GetSet();
+        if (bssp.CanGetClass() && bssp.GetClass() == CBioseq_set::eClass_small_genome_set) {
+            m_HasSmallGenomeSet = true;
+        }
+        for (auto& seqentry : bssp.GetSeq_set()) {
+            // recursively explore current Bioseq-set
+            x_InitSeqs(*seqentry);
+        }
+    }
+}
+
+CTopLevelSeqEntryContext::CTopLevelSeqEntryContext( const CSeq_entry_Handle &entry_handle, bool useIndexedFasterSets )
 {
     m_CanSourcePubsBeFused = false;
     m_HasSmallGenomeSet = false;
 
+    if (useIndexedFasterSets) {
+        CSeq_entry_Handle tseh = entry_handle.GetTopLevelEntry();
+        CConstRef<CSeq_entry> tcsep = tseh.GetCompleteSeq_entry();
+        CSeq_entry& topsep = const_cast<CSeq_entry&>(*tcsep);
+        x_InitSeqs( topsep );
+        return;
+    }
+
     CBioseq_CI bioseq_iter( entry_handle.GetScope(), *entry_handle.GetSeq_entryCore() );
     for( ; bioseq_iter; ++bioseq_iter ) {
         ITERATE( CBioseq_Handle::TId, it, bioseq_iter->GetId() )  {
@@ -1261,7 +1324,7 @@ CTopLevelSeqEntryContext::CTopLevelSeqEntryContext( const CSeq_entry_Handle &ent
                         case CSeq_id_Base::e_Embl:
                         case CSeq_id_Base::e_Pir:
                         case CSeq_id_Base::e_Swissprot:
-                        case CSeq_id_Base::e_Patent:        
+                        case CSeq_id_Base::e_Patent:
                         case CSeq_id_Base::e_Ddbj:
                         case CSeq_id_Base::e_Prf:
                         case CSeq_id_Base::e_Pdb:
@@ -1270,11 +1333,11 @@ CTopLevelSeqEntryContext::CTopLevelSeqEntryContext( const CSeq_entry_Handle &ent
                         case CSeq_id_Base::e_Gpipe:
                             // with some types, it's okay to merge
                             m_CanSourcePubsBeFused = true;
-                            break;                        
+                            break;
                         case CSeq_id_Base::e_Genbank:
                         case CSeq_id_Base::e_Tpg:
                             // Genbank allows merging only if it's the old-style 1 + 5 accessions
-                            if( NULL != seqId->GetTextseq_Id() &&
+                            if( seqId->GetTextseq_Id() &&
                                 seqId->GetTextseq_Id()->IsSetAccession() &&
                                 seqId->GetTextseq_Id()->GetAccession().length() == 6 ) {
                                     m_CanSourcePubsBeFused = true;
@@ -1284,7 +1347,7 @@ CTopLevelSeqEntryContext::CTopLevelSeqEntryContext( const CSeq_entry_Handle &ent
                         case CSeq_id_Base::e_Local:
                         case CSeq_id_Base::e_Other:
                         case CSeq_id_Base::e_General:
-                        case CSeq_id_Base::e_Giim:                        
+                        case CSeq_id_Base::e_Giim:
                         case CSeq_id_Base::e_Gi:
                             break;
                         default:
@@ -1297,14 +1360,14 @@ CTopLevelSeqEntryContext::CTopLevelSeqEntryContext( const CSeq_entry_Handle &ent
     // check all Bioseq-sets, if any
     if( entry_handle.IsSet() ) {
         if( entry_handle.GetSet().CanGetClass() &&
-            entry_handle.GetSet().GetClass() == CBioseq_set::eClass_small_genome_set ) 
+            entry_handle.GetSet().GetClass() == CBioseq_set::eClass_small_genome_set )
         {
             m_HasSmallGenomeSet = true;
         } else {
             CSeq_entry_CI seq_entry_ci( entry_handle, CSeq_entry_CI::eRecursive );
             for( ; seq_entry_ci && ! m_HasSmallGenomeSet; ++seq_entry_ci ) {
                 if( seq_entry_ci->IsSet() && seq_entry_ci->GetSet().CanGetClass() &&
-                    seq_entry_ci->GetSet().GetClass() == CBioseq_set::eClass_small_genome_set ) 
+                    seq_entry_ci->GetSet().GetClass() == CBioseq_set::eClass_small_genome_set )
                 {
                     m_HasSmallGenomeSet = true;
                     break;
