@@ -1,4 +1,4 @@
-/*  $Id: dbsource_item.cpp 607132 2020-04-30 12:54:53Z grichenk $
+/*  $Id: dbsource_item.cpp 675915 2023-12-05 13:22:40Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -124,7 +124,7 @@ static bool s_HasLocalBioseq(const CSeq_loc& loc, const CSeq_entry_Handle& tse)
 {
     CScope& scope = tse.GetScope();
     for (CSeq_loc_CI li(loc); li; ++li) {
-        CBioseq_Handle local = 
+        CBioseq_Handle local =
             scope.GetBioseqHandleFromTSE(li.GetSeq_id(), tse);
         if (local) {
             return true;
@@ -148,20 +148,23 @@ void CDBSourceItem::x_GatherInfo(CBioseqContext& ctx)
     }
 
     switch (idh.Which()) {
+// LCOV_EXCL_START
     case CSeq_id::e_Pir:
         m_DBSource.push_back(x_FormatDBSourceID(idh));
         x_AddPIRBlock(ctx);
         break;
+// LCOV_EXCL_STOP
 
     case CSeq_id::e_Swissprot:
         m_DBSource.push_back(x_FormatDBSourceID(idh));
         x_AddSPBlock(ctx);
         break;
-
+// LCOV_EXCL_START
     case CSeq_id::e_Prf:
         m_DBSource.push_back(x_FormatDBSourceID(idh));
         x_AddPRFBlock(ctx);
         break;
+// LCOV_EXCL_STOP
 
     case CSeq_id::e_Pdb:
         m_DBSource.push_back(x_FormatDBSourceID(idh));
@@ -184,12 +187,12 @@ void CDBSourceItem::x_GatherInfo(CBioseqContext& ctx)
 
         // find generating feature
         const CSeq_feat* feat = sequence::GetCDSForProduct(seq);
-        if (feat == NULL) {
+        if (! feat) {
             // may also be protein product of mature peptide feature
             feat = sequence::GetPROTForProduct(seq);
         }
 
-        if (feat != NULL) {
+        if (feat) {
             const CSeq_loc& loc = feat->GetLocation();
             CSeq_entry_Handle topLevelEntry = seq.GetTopLevelEntry();
             if (s_HasLocalBioseq(loc, topLevelEntry)) {
@@ -198,10 +201,10 @@ void CDBSourceItem::x_GatherInfo(CBioseqContext& ctx)
                 }
             } /* else {
                 const CSeq_id *cds_seq_id = loc.GetId();
-                if( NULL != cds_seq_id && cds_seq_id->IsGi() ) {
+                if (cds_seq_id && cds_seq_id->IsGi()) {
                     CSeq_id_Base::TGi cds_gi = cds_seq_id->GetGi();
                     s_AddToUniqueIdList( CSeq_id_Handle::GetHandle(cds_gi), unique_ids);
-                } 
+                }
             } */
         }
 
@@ -219,14 +222,14 @@ void CDBSourceItem::x_GatherInfo(CBioseqContext& ctx)
             }
         }
 
-        if( m_DBSource.empty() && feat != NULL ) {
+        if (m_DBSource.empty() && feat) {
             const CSeq_loc& loc = feat->GetLocation();
             const CSeq_id *cds_seq_id = loc.GetId();
-            if( NULL != cds_seq_id && cds_seq_id->IsGi() ) {
+            if (cds_seq_id && cds_seq_id->IsGi()) {
                 CSeq_id_Base::TGi cds_gi = cds_seq_id->GetGi();
                 // s_AddToUniqueIdList( CSeq_id_Handle::GetHandle(cds_gi), unique_ids);
                 m_DBSource.push_back( x_FormatDBSourceID( CSeq_id_Handle::GetHandle(cds_gi) ) );
-            } 
+            }
         }
 
         if (m_DBSource.empty()) {
@@ -238,7 +241,7 @@ void CDBSourceItem::x_GatherInfo(CBioseqContext& ctx)
         m_DBSource.push_back("UNKNOWN");
     }
 
-    // turn double-quotes to single-quotes in all m_DBSources, 
+    // turn double-quotes to single-quotes in all m_DBSources,
     // except inside HTML tags
     NON_CONST_ITERATE( list<string>, it, m_DBSource ) {
         if( bHtml ) {
@@ -249,10 +252,11 @@ void CDBSourceItem::x_GatherInfo(CBioseqContext& ctx)
     }
 }
 
+// LCOV_EXCL_START
 void CDBSourceItem::x_AddPIRBlock(CBioseqContext& ctx)
 {
-    // In this function, the newlines seem weird because the C toolkit 
-    // outputs this way.  Hopefully in the future we can do something 
+    // In this function, the newlines seem weird because the C toolkit
+    // outputs this way.  Hopefully in the future we can do something
     // more consistent.
 
 
@@ -300,7 +304,7 @@ void CDBSourceItem::x_AddPIRBlock(CBioseqContext& ctx)
     if (pir.CanGetSeqref()) {
         list<string> xrefs;
         ITERATE (CPIR_block::TSeqref, it, pir.GetSeqref()) {
-            const char* type = 0;
+            const char* type = nullptr;
             switch ((*it)->Which()) {
             case CSeq_id::e_Genbank:    type = "genbank ";    break;
             case CSeq_id::e_Embl:       type = "embl ";       break;
@@ -343,6 +347,7 @@ void CDBSourceItem::x_AddPIRBlock(CBioseqContext& ctx)
         m_DBSource.front() += "\n";
     }
 }
+// LCOV_EXCL_STOP
 
 static void s_FormatDate(const CDate& date, string& str)
 {
@@ -414,7 +419,7 @@ void CDBSourceItem::x_AddSPBlock(CBioseqContext& ctx)
                 xrefs.push_back(acc);
             }
             /**
-            const char* s = 0;
+            const char* s = nullptr;
             switch ((*it)->Which()) {
             case CSeq_id::e_Genbank:  s = "genbank accession ";          break;
             case CSeq_id::e_Embl:     s = "embl accession ";             break;
@@ -456,7 +461,7 @@ void CDBSourceItem::x_AddSPBlock(CBioseqContext& ctx)
                     xrefs.push_back("MIM:" + id);
                 }
             } else {
-                // For exmaple, HGNC has HGNC as part of its identifier, so we may need to eliminate 
+                // For exmaple, HGNC has HGNC as part of its identifier, so we may need to eliminate
                 // such redundancies (example accession: Q02094.1)
                 if( id.substr(0, db.length() + 1) == (db + ":") ) {
                     xrefs.push_back(id); // in this case, id already has db at beginning
@@ -470,7 +475,7 @@ void CDBSourceItem::x_AddSPBlock(CBioseqContext& ctx)
     }
 }
 
-
+// LCOV_EXCL_START
 void CDBSourceItem::x_AddPRFBlock(CBioseqContext& ctx)
 {
     CSeqdesc_CI dsc(ctx.GetHandle(), CSeqdesc::e_Prf);
@@ -503,6 +508,7 @@ void CDBSourceItem::x_AddPRFBlock(CBioseqContext& ctx)
         *it += (&*it == &m_DBSource.back() ? '.' : ';');
     }
 }
+// LCOV_EXCL_STOP
 
 
 void CDBSourceItem::x_AddPDBBlock(CBioseqContext& ctx)
@@ -545,10 +551,6 @@ void CDBSourceItem::x_AddPDBBlock(CBioseqContext& ctx)
 
 string CDBSourceItem::x_FormatDBSourceID(const CSeq_id_Handle& idh)
 {
-#ifndef NEW_HTML_FMT
-    const bool is_html = ( GetContext()->Config().DoHTML() );
-#endif
-
     CConstRef<CSeq_id> id;
     if (idh) {
         id = idh.GetSeqId();
@@ -595,7 +597,7 @@ string CDBSourceItem::x_FormatDBSourceID(const CSeq_id_Handle& idh)
     default:
         {{
             const CTextseq_id* tsid = id->GetTextseq_Id();
-            if (tsid == NULL) {
+            if (! tsid) {
                 return kEmptyStr;
             }
             string s, sep, comma, ht;
@@ -620,7 +622,6 @@ string CDBSourceItem::x_FormatDBSourceID(const CSeq_id_Handle& idh)
                     choice != CSeq_id::e_Swissprot) {
                     acc += '.' + NStr::IntToString(tsid->GetVersion());
                 }
-#ifdef NEW_HTML_FMT
 #if 0
                 GetContext()->Config().GetHTMLFormatter().FormatNucId(ht, *idh.GetSeqId(), GetContext()->GetScope().GetGi(idh), acc);
 #else
@@ -632,15 +633,6 @@ string CDBSourceItem::x_FormatDBSourceID(const CSeq_id_Handle& idh)
                 }
 #endif
                 s += comma + sep + "accession " + ht;
-#else
-                if (is_html) {
-                    const TIntId gi = GetContext()->GetScope().GetGi(idh);
-                    s += comma + sep + "accession <a href=\"" + strLinkBaseNuc +
-                        NStr::NumericToString(gi) + "\">" + acc + "</a>";
-                } else {
-                    s += comma + sep + "accession " + acc;
-                }
-#endif
                 sep = " ";
             }
             /**
@@ -706,7 +698,7 @@ bool CDBSourceItem::x_ExtractLinkableSource(
     };
 
     const static size_t numPotentialPrefixes = sizeof(potentialPrefixes)/sizeof(potentialPrefixes[0]);
-    
+
     for( size_t idx = 0; idx < numPotentialPrefixes; ++idx ) {
         const string & prefix = potentialPrefixes[idx].m_prefix;
         const string & url = potentialPrefixes[idx].m_url;
