@@ -1,4 +1,4 @@
-/*  $Id: flat_file_generator.cpp 632198 2021-05-27 13:43:43Z ivanov $
+/*  $Id: flat_file_generator.cpp 671846 2023-08-24 20:01:19Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -27,11 +27,12 @@
 *
 * File Description:
 *   User interface for generating flat file reports from ASN.1
-*   
+*
 */
 #include <ncbi_pch.hpp>
 #include <corelib/ncbistd.hpp>
 #include <corelib/ncbiobj.hpp>
+#include <corelib/ncbitime.hpp>
 #include <connect/ncbi_conn_stream.hpp>
 
 #include <objects/seqset/Seq_entry.hpp>
@@ -193,6 +194,7 @@ void CFlatFileGenerator::ResetSeqEntryIndex(void)
 // Generate a flat-file report for a Seq-entry
 // (the other CFlatFileGenerator::Generate functions ultimately
 // call this)
+#if 0
 void CFlatFileGenerator::Generate
 (const CSeq_entry_Handle& entry,
  CFlatItemOStream& item_os)
@@ -253,10 +255,10 @@ void CFlatFileGenerator::Generate
     CRef<CFlatItemOStream> pItemOS( & item_os );
     // If there is a ICancel callback, wrap the item_os so
     // that every call checks it.
-    const ICanceled * pCanceled = 
+    const ICanceled * pCanceled =
         m_Ctx->GetConfig().GetCanceledCallback();
     if( pCanceled ) {
-        pItemOS.Reset( 
+        pItemOS.Reset(
             new CCancelableFlatItemOStreamWrapper(
             item_os, pCanceled) );
     }
@@ -283,46 +285,50 @@ void CFlatFileGenerator::Generate
             try {
                 CSeqEntryIndex::EPolicy policy = CSeqEntryIndex::eAdaptive;
                 CSeqEntryIndex::TFlags flags = CSeqEntryIndex::fDefault;
-                if ( m_Ctx->GetConfig().OnlyNearFeatures() ) {
+                const CFlatFileConfig& cfg = m_Ctx->GetConfig();
+                if ( cfg.OnlyNearFeatures() && ! ( cfg.IsPolicyFtp() || cfg.IsPolicyGenomes() ) ) {
                     policy = CSeqEntryIndex::eInternal;
                 }
-                if ( m_Ctx->GetConfig().HideSNPFeatures() ) {
+                if ( cfg.HideSNPFeatures() ) {
                     flags |= CSeqEntryIndex::fHideSNPFeats;
                 }
-                if ( m_Ctx->GetConfig().HideCDDFeatures() ) {
+                if ( cfg.HideCDDFeatures() ) {
                     flags |= CSeqEntryIndex::fHideCDDFeats;
                 }
-                if ( m_Ctx->GetConfig().ShowSNPFeatures() ) {
+                if ( cfg.ShowSNPFeatures() ) {
                     flags |= CSeqEntryIndex::fShowSNPFeats;
                 }
-                if ( m_Ctx->GetConfig().ShowCDDFeatures() ) {
+                if ( cfg.ShowCDDFeatures() ) {
                     flags |= CSeqEntryIndex::fShowCDDFeats;
                 }
-                if ( m_Ctx->GetConfig().HideExonFeatures() ) {
+                if ( cfg.HideExonFeatures() ) {
                     flags |= CSeqEntryIndex::fHideExonFeats;
                 }
-                if ( m_Ctx->GetConfig().HideIntronFeatures() ) {
+                if ( cfg.HideIntronFeatures() ) {
                     flags |= CSeqEntryIndex::fHideIntronFeats;
                 }
-                if ( m_Ctx->GetConfig().HideMiscFeatures() ) {
+                if ( cfg.HideMiscFeatures() ) {
                     flags |= CSeqEntryIndex::fHideMiscFeats;
                 }
-                if ( m_Ctx->GetConfig().GeneRNACDSFeatures() ) {
+                if ( cfg.GeneRNACDSFeatures() ) {
                     flags |= CSeqEntryIndex::fGeneRNACDSOnly;
                 }
-                if ( m_Ctx->GetConfig().IsPolicyInternal() ) {
+                if ( cfg.IsPolicyInternal() ) {
                     policy = CSeqEntryIndex::eInternal;
                 }
-                if ( m_Ctx->GetConfig().IsPolicyExternal() ) {
+                if ( cfg.IsPolicyExternal() ) {
                     policy = CSeqEntryIndex::eExternal;
                 }
-                if ( m_Ctx->GetConfig().IsPolicyExhaustive() ) {
+                if ( cfg.IsPolicyExhaustive() ) {
                     policy = CSeqEntryIndex::eExhaustive;
                 }
-                if ( m_Ctx->GetConfig().IsPolicyFtp() ) {
+                if ( cfg.IsPolicyFtp() ) {
                     policy = CSeqEntryIndex::eFtp;
                 }
-                if ( m_Ctx->GetConfig().IsPolicyWeb() ) {
+                if ( cfg.IsPolicyGenomes() ) {
+                    policy = CSeqEntryIndex::eGenomes;
+                }
+                if ( cfg.IsPolicyWeb() ) {
                     policy = CSeqEntryIndex::eWeb;
                 }
                 CRef<CSeqEntryIndex> idx(new CSeqEntryIndex( topseh, policy, flags ));
@@ -354,7 +360,7 @@ void CFlatFileGenerator::Generate
 
     for (CBioseq_CI bi(entry); bi; ++bi) {
         const CBioseq_Handle& bh = *bi;
-  
+
         const CBioseq& bsp = *(bi->GetCompleteBioseq());
 
         FOR_EACH_SEQID_ON_BIOSEQ (it, bsp) {
@@ -372,8 +378,8 @@ void CFlatFileGenerator::Generate
                     break;
                 case CSeq_id::e_Other:
                     {
-                         const CTextseq_id* tsid = sid.GetTextseq_Id ();
-                        if (tsid != NULL && tsid->IsSetAccession()) {
+                        const CTextseq_id* tsid = sid.GetTextseq_Id();
+                        if (tsid && tsid->IsSetAccession()) {
                             const string& acc = tsid->GetAccession().substr(0, 3);
                             if (acc == "NC_") {
                                 isNc = true;
@@ -599,25 +605,105 @@ void CFlatFileGenerator::Generate
         Generate(*loc, scope, item_os);
     }
 }
+void CFlatFileGenerator::Generate
+(const CSeq_submit& submit,
+ CScope& scope,
+ CNcbiOstream& os)
+{
+    CRef<CFlatItemOStream>
+        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+    Generate(submit, scope, *item_os);
+}
+
+
+void CFlatFileGenerator::Generate
+(const CBioseq& bioseq,
+ CScope& scope,
+ CNcbiOstream& os)
+{
+    CRef<CFlatItemOStream>
+        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+    const CBioseq_Handle bsh = scope.GetBioseqHandle(bioseq);
+    const CSeq_entry_Handle entry = bsh.GetSeq_entry_Handle();
+    Generate(entry, *item_os);
+}
+
+
+void CFlatFileGenerator::Generate
+(const CSeq_entry_Handle& entry,
+ CNcbiOstream& os)
+{
+    CRef<CFlatItemOStream>
+        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+    Generate(entry, *item_os);
+}
+
+
+void CFlatFileGenerator::Generate
+(const CBioseq_Handle& bsh,
+ CNcbiOstream& os)
+{
+    CRef<CFlatItemOStream>
+        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+    const CSeq_entry_Handle entry = bsh.GetSeq_entry_Handle();
+    Generate(entry, *item_os);
+}
+
+
+void CFlatFileGenerator::Generate
+(const CSeq_loc& loc,
+ CScope& scope,
+ CNcbiOstream& os)
+{
+    CRef<CFlatItemOStream>
+        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+    Generate(loc, scope, *item_os);
+}
+
+
+void CFlatFileGenerator::Generate
+(const CSeq_id& id,
+ const TRange& range,
+ ENa_strand strand,
+ CScope& scope,
+ CNcbiOstream& os)
+{
+    CRef<CFlatItemOStream>
+        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+    Generate(id, range, strand, scope, *item_os);
+}
+
+
+#endif
 
 
 // This version iterates Bioseqs within the Bioseq_set
-void CFlatFileGenerator::Generate
-(const CSeq_entry_Handle& entry,
- CFlatItemOStream& item_os,
- bool useSeqEntryIndexing,
- CNcbiOstream* m_Os,
- CNcbiOstream* m_On,
- CNcbiOstream* m_Og,
- CNcbiOstream* m_Or,
- CNcbiOstream* m_Op,
- CNcbiOstream* m_Ou
- )
+void CFlatFileGenerator::Generate(
+    const CSeq_entry_Handle& entry,
+    CFlatItemOStream& item_os,
+    const multiout& mo)
 {
+    // useSeqEntryIndexing now turned ON by default
+    bool useSeqEntryIndexing = true;
+
+    /*
     // useSeqEntryIndexing argument also set by relevant flags in CFlatFileConfig
     if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
         useSeqEntryIndexing = true;
     }
+    */
+
+    // Setting the -custom 1048576 bit flag turns OFF default indexing
+    if ( m_Ctx->GetConfig().DisableDefaultIndex() ) {
+        useSeqEntryIndexing = false;
+        m_Ctx->SetConfig().SetUseSeqEntryIndexer(false);
+    }
 
     if (useSeqEntryIndexing) {
         m_Ctx->SetConfig().SetUseSeqEntryIndexer(true);
@@ -629,8 +715,13 @@ void CFlatFileGenerator::Generate
 
     const CFlatFileConfig& cfg = m_Ctx->GetConfig();
 
+    bool showDebugTiming = cfg.ShowDebugTiming();
+
+    CStopWatch sw;
+
     bool doNuc = false;
     bool doProt = false;
+    bool doFastSets = false;
 
     // doNuc and doProt arguments also set by relevant flags in CFlatFileConfig
     if ( cfg.IsViewNuc() ) {
@@ -643,9 +734,15 @@ void CFlatFileGenerator::Generate
         doNuc = true;
         doProt = true;
     }
+    if ( cfg.FasterReleaseSets() ) {
+        doFastSets = true;
+    }
 
     if ( cfg.BasicCleanup() )
     {
+        if (showDebugTiming) {
+            sw.Start();
+        }
 
         entry.GetTopLevelEntry().GetCompleteObject();
         CSeq_entry_EditHandle tseh = entry.GetTopLevelEntry().GetEditHandle();
@@ -675,6 +772,15 @@ void CFlatFileGenerator::Generate
         else {
             tseh.SelectSeq(bseqh);
         }
+
+        if (showDebugTiming) {
+            NcbiCerr << "Cleanup: " << sw.Elapsed() << ", ";
+            sw.Reset();
+        }
+    }
+
+    if (showDebugTiming) {
+        sw.Start();
     }
 
     m_Ctx->SetSGS(false);
@@ -721,7 +827,7 @@ void CFlatFileGenerator::Generate
             try {
                 CSeqEntryIndex::EPolicy policy = CSeqEntryIndex::eAdaptive;
                 CSeqEntryIndex::TFlags flags = CSeqEntryIndex::fDefault;
-                if ( cfg.OnlyNearFeatures() ) {
+                if ( cfg.OnlyNearFeatures() && ! ( cfg.IsPolicyFtp() || cfg.IsPolicyGenomes() ) ) {
                     policy = CSeqEntryIndex::eInternal;
                 }
                 if ( cfg.HideSNPFeatures() ) {
@@ -746,6 +852,9 @@ void CFlatFileGenerator::Generate
                 if ( cfg.IsPolicyFtp() ) {
                     policy = CSeqEntryIndex::eFtp;
                 }
+                if ( cfg.IsPolicyGenomes() ) {
+                    policy = CSeqEntryIndex::eGenomes;
+                }
                 if ( cfg.IsPolicyWeb() ) {
                     policy = CSeqEntryIndex::eWeb;
                 }
@@ -773,10 +882,62 @@ void CFlatFileGenerator::Generate
     }
     formatter->SetContext(*m_Ctx);
 
+    if (showDebugTiming) {
+        // NcbiCerr << "Prepare: " << sw.Elapsed() << ", ";
+        sw.Reset();
+    }
+
+    const string accn_filt = cfg.GetSingleAccession();
+
     // internal Bioseq iterator loop moved up from x_GatherSeqEntry
     for (CBioseq_CI bioseq_it(entry);  bioseq_it;  ++bioseq_it) {
+
+        if (showDebugTiming) {
+            sw.Start();
+        }
+
         CBioseq_Handle bsh = *bioseq_it;
         if (! bsh) continue;
+
+        if ( ! accn_filt.empty()) {
+            bool okay = false;
+            const CBioseq& bsp = *bsh.GetBioseqCore();
+            for (auto& sid : bsp.GetId()) {
+                switch (sid->Which()) {
+                    case NCBI_SEQID(Genbank):
+                    case NCBI_SEQID(Embl):
+                    case NCBI_SEQID(Ddbj):
+                    case NCBI_SEQID(Tpg):
+                    case NCBI_SEQID(Tpe):
+                    case NCBI_SEQID(Tpd):
+                    case NCBI_SEQID(Local):
+                    case NCBI_SEQID(General):
+                    case NCBI_SEQID(Other):
+                    case NCBI_SEQID(Gpipe):
+                    {
+                        const string accn_string = sid->GetSeqIdString();
+                        if ( accn_filt == accn_string ) {
+                            okay = true;
+                        }
+                        const string fasta_str = sid->AsFastaString();
+                        if ( accn_filt == fasta_str ) {
+                            okay = true;
+                        }
+                        break;
+                    }
+                     default:
+                       break;
+                }
+            }
+            if ( ! okay) {
+                continue;
+            }
+        }
+
+        CSeq_id_Handle sidh = bsh.GetAccessSeq_id_Handle();
+        if (showDebugTiming) {
+            NcbiCerr << "Accession: " << sidh << ", ";
+        }
         const CSeq_entry_Handle ent = bsh.GetSeq_entry_Handle();
         CConstRef<CBioseq> bsr = bsh.GetCompleteBioseq();
 
@@ -831,33 +992,33 @@ void CFlatFileGenerator::Generate
             }
         }
 
-        if ( m_Os ) {
-            flatfile_os = m_Os;
+        if (mo.m_Os) {
+            flatfile_os = mo.m_Os;
             if ( bsh.IsNa() && ! doNuc ) continue;
             if ( bsh.IsAa() && ! doProt ) continue;
         } else if ( bsh.IsNa() ) {
-            if ( m_On ) {
-                flatfile_os = m_On;
-            } else if ( (is_genomic || ! closest_molinfo) && m_Og ) {
-                flatfile_os = m_Og;
-            } else if ( is_RNA && m_Or ) {
-                flatfile_os = m_Or;
+            if (mo.m_On) {
+                flatfile_os = mo.m_On;
+            } else if ((is_genomic || ! closest_molinfo) && mo.m_Og) {
+                flatfile_os = mo.m_Og;
+            } else if (is_RNA && mo.m_Or) {
+                flatfile_os = mo.m_Or;
             }
         } else if ( bsh.IsAa() ) {
-            if ( m_Op ) {
-                flatfile_os = m_Op;
+            if (mo.m_Op) {
+                flatfile_os = mo.m_Op;
             }
         } else {
-            if ( m_Ou ) {
-                flatfile_os = m_Ou;
-            } else if ( m_On ) {
-                flatfile_os = m_On;
+            if (mo.m_Ou) {
+                flatfile_os = mo.m_Ou;
+            } else if (mo.m_On) {
+                flatfile_os = mo.m_On;
             }
         }
 
         CRef<CFlatItemOStream> newitem_os;
-        if (flatfile_os == nullptr) {
-            if (!m_Os && !m_On && !m_Og && !m_Or && !m_Op && !m_Ou) {
+        if (! flatfile_os) {
+            if (! mo) {
                 if ( bsh.IsNa() && ! doNuc ) continue;
                 if ( bsh.IsAa() && ! doProt ) continue;
                 newitem_os.Reset(&item_os);
@@ -872,7 +1033,7 @@ void CFlatFileGenerator::Generate
 
         CRef<CFlatItemOStream> pItemOS( newitem_os );
         if( pCanceled ) {
-            pItemOS.Reset( 
+            pItemOS.Reset(
                 new CCancelableFlatItemOStreamWrapper(
                 *newitem_os, pCanceled) );
         }
@@ -884,150 +1045,110 @@ void CFlatFileGenerator::Generate
             NCBI_THROW(CFlatException, eInternal, "Unable to initialize gatherer");
         }
 
-        gatherer->Gather(*m_Ctx, *pItemOS, ent, bsh, useSeqEntryIndexing, doNuc, doProt);
+        gatherer->Gather(*m_Ctx, *pItemOS, ent, bsh, useSeqEntryIndexing, doNuc, doProt, doFastSets);
+
+        if (showDebugTiming) {
+            NcbiCerr << "Elapsed: " << sw.Elapsed() << NcbiEndl;
+            sw.Reset();
+        }
     }
 
+    /*
+    if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
+        m_Ctx->ResetSeqEntryIndex();
+    }
+    */
+
     /// reset the context, but preserve our selector
     /// we do this a bit oddly since resetting the context erases the selector;
     /// since the caller is reusing this object (most likely), we automatically
     /// restore the selector to its former glory
     m_Ctx->Reset();
     m_Ctx->SetAnnotSelector() = sel;
-
-    /*
-    if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
-        m_Ctx->ResetSeqEntryIndex();
-    }
-    */
 }
 
 
-void CFlatFileGenerator::Generate
-(const CBioseq_Handle& bsh,
- CNcbiOstream& os,
- bool useSeqEntryIndexing,
- CNcbiOstream* m_Os,
- CNcbiOstream* m_On,
- CNcbiOstream* m_Og,
- CNcbiOstream* m_Or,
- CNcbiOstream* m_Op,
- CNcbiOstream* m_Ou
- )
+void CFlatFileGenerator::Generate(
+    const CSeq_entry_Handle& entry,
+    CNcbiOstream& os,
+    const multiout& mo)
+{
+    CRef<CFlatItemOStream>
+        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+    Generate(entry, *item_os, mo);
+}
+
+
+void CFlatFileGenerator::Generate(
+    const CBioseq_Handle& bsh,
+    CNcbiOstream& os,
+    const multiout& mo)
 {
     CRef<CFlatItemOStream>
         item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
 
     const CSeq_entry_Handle entry = bsh.GetSeq_entry_Handle();
-    Generate(entry, *item_os, useSeqEntryIndexing, m_Os, m_On, m_Og, m_Or, m_Op, m_Ou);
-
+    Generate(entry, *item_os, mo);
 }
 
 
-void CFlatFileGenerator::Generate
-(const CSeq_submit& submit,
- CScope& scope,
- CNcbiOstream& os)
+void CFlatFileGenerator::Generate(
+    const CSeq_submit& submit,
+    CScope& scope,
+    CNcbiOstream& os,
+    const multiout& mo)
 {
-    CRef<CFlatItemOStream> 
-        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+    _ASSERT(submit.CanGetData());
+    _ASSERT(submit.CanGetSub());
+    _ASSERT(submit.GetData().IsEntrys());
+    _ASSERT(!submit.GetData().GetEntrys().empty());
 
-    Generate(submit, scope, *item_os);
+    // NB: though the spec specifies a submission may contain multiple entries
+    // this is not the case. A submission should only have a single Top-level
+    // Seq-entry
+    CConstRef<CSeq_entry> e(submit.GetData().GetEntrys().front());
+    if (e.NotEmpty()) {
+        // get Seq_entry_Handle from scope
+        CSeq_entry_Handle entry;
+        try {
+            entry = scope.GetSeq_entryHandle(*e);
+        } catch (CException&) {}
+
+        if (!entry) {  // add to scope if not already in it
+            entry = scope.AddTopLevelSeqEntry(*e);
+        }
+        // "remember" the submission block
+        m_Ctx->SetSubmit(submit.GetSub());
+
+        CRef<CFlatItemOStream>
+            item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+        Generate(entry, *item_os, mo);
+    }
 }
 
 
-void CFlatFileGenerator::Generate
-(const CBioseq& bioseq,
- CScope& scope,
- CNcbiOstream& os)
+void CFlatFileGenerator::Generate(
+    const CBioseq& bioseq,
+    CScope& scope,
+    CNcbiOstream& os,
+    const multiout& mo)
 {
-    CRef<CFlatItemOStream> 
+    CRef<CFlatItemOStream>
         item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
 
     const CBioseq_Handle bsh = scope.GetBioseqHandle(bioseq);
     const CSeq_entry_Handle entry = bsh.GetSeq_entry_Handle();
-    Generate(entry, *item_os);
+    Generate(entry, *item_os, mo);
 }
 
 
-void CFlatFileGenerator::Generate
-(const CSeq_entry_Handle& entry,
- CNcbiOstream& os)
-{
-    CRef<CFlatItemOStream> 
-        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
-
-    Generate(entry, *item_os);
-}
-
-
-void CFlatFileGenerator::Generate
-(const CBioseq_Handle& bsh,
- CNcbiOstream& os)
-{
-    CRef<CFlatItemOStream> 
-        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
-
-    const CSeq_entry_Handle entry = bsh.GetSeq_entry_Handle();
-    Generate(entry, *item_os);
-}
-
-
-void CFlatFileGenerator::Generate
-(const CSeq_loc& loc,
- CScope& scope,
- CNcbiOstream& os)
-{
-    CRef<CFlatItemOStream> 
-        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
-
-    Generate(loc, scope, *item_os);
-}
-
-
-void CFlatFileGenerator::Generate
-(const CSeq_id& id,
- const TRange& range, 
- ENa_strand strand,
- CScope& scope,
- CNcbiOstream& os)
-{
-    CRef<CFlatItemOStream> 
-        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
-
-    Generate(id, range, strand, scope, *item_os);
-}
-
-
-void CFlatFileGenerator::Generate
-(const CSeq_entry_Handle& entry,
- CNcbiOstream& os,
- bool useSeqEntryIndexing,
- CNcbiOstream* m_Os,
- CNcbiOstream* m_On,
- CNcbiOstream* m_Og,
- CNcbiOstream* m_Or,
- CNcbiOstream* m_Op,
- CNcbiOstream* m_Ou
- )
-{
-    CRef<CFlatItemOStream> 
-        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
-
-    Generate(entry, *item_os, useSeqEntryIndexing, m_Os, m_On, m_Og, m_Or, m_Op, m_Ou);
-}
-
-void CFlatFileGenerator::Generate
-(const CSeq_loc& loc,
- CScope& scope,
- CNcbiOstream& os,
- bool useSeqEntryIndexing,
- CNcbiOstream* m_Os,
- CNcbiOstream* m_On,
- CNcbiOstream* m_Og,
- CNcbiOstream* m_Or,
- CNcbiOstream* m_Op,
- CNcbiOstream* m_Ou
- )
+void CFlatFileGenerator::Generate(
+    const CSeq_loc& loc,
+    CScope& scope,
+    CNcbiOstream& os,
+    const multiout& mo)
 {
     CBioseq_Handle bsh = GetBioseqFromSeqLoc(loc, scope);
     if (!bsh) {
@@ -1046,7 +1167,30 @@ void CFlatFileGenerator::Generate
         cfg.SetStyleMaster();
     }
 
-    Generate(entry, os, useSeqEntryIndexing, m_Os, m_On, m_Og, m_Or, m_Op, m_Ou);
+    Generate(entry, os, mo);
+}
+
+
+void CFlatFileGenerator::Generate(
+    const CSeq_id& id,
+    const TRange& range,
+    ENa_strand strand,
+    CScope& scope,
+    CNcbiOstream& os,
+    const multiout& mo)
+{
+    CRef<CSeq_id> id2(new CSeq_id);
+    id2->Assign(id);
+    CRef<CSeq_loc> loc;
+    if ( range.IsWhole() ) {
+        loc.Reset(new CSeq_loc);
+        loc->SetWhole(*id2);
+    } else {
+        loc.Reset(new CSeq_loc(*id2, range.GetFrom(), range.GetTo(), strand));
+    }
+    if ( loc ) {
+        Generate(*loc, scope, os, mo);
+    }
 }
 
 
@@ -1060,7 +1204,8 @@ void CFlatFileGenerator::Generate
 string CFlatFileGenerator::GetSeqFeatText
 (const CMappedFeat& feat,
  CScope& scope,
- const CFlatFileConfig& cfg)
+ const CFlatFileConfig& cfg,
+ CRef<feature::CFeatTree> ftree)
 {
     CBioseq_Handle seq = sequence::GetBioseqFromSeqLoc(feat.GetLocation(), scope);
     if (!seq) {
@@ -1078,10 +1223,11 @@ string CFlatFileGenerator::GetSeqFeatText
 
     CBioseqContext bctx(seq, *ctx);
 
-    CSeq_entry_Handle tseh = seq.GetTopLevelEntry();
-    CFeat_CI iter (tseh);
-    CRef<feature::CFeatTree> ftree;
-    ftree.Reset (new feature::CFeatTree (iter));
+    if (ftree.Empty()) {
+        CSeq_entry_Handle tseh = seq.GetTopLevelEntry();
+        CFeat_CI iter (tseh);
+        ftree.Reset (new feature::CFeatTree (iter));
+    }
 
     CConstRef<IFlatItem> item;
     if (feat.GetData().IsBiosrc()) {
@@ -1097,6 +1243,7 @@ string CFlatFileGenerator::GetSeqFeatText
     return text;
 }
 
+
 void CFlatFileGenerator::x_GetLocation
 (const CSeq_entry_Handle& entry,
  TSeqPos from,
@@ -1185,7 +1332,7 @@ CBioseq_Handle CFlatFileGenerator::x_DeduceTarget(const CSeq_entry_Handle& entry
             "Cannot deduce target bioseq.");
 }
 
-void 
+void
 CFlatFileGenerator::CCancelableFlatItemOStreamWrapper::SetFormatter(
     IFormatter* formatter)
 {
@@ -1195,10 +1342,10 @@ CFlatFileGenerator::CCancelableFlatItemOStreamWrapper::SetFormatter(
 
 void
 CFlatFileGenerator::CCancelableFlatItemOStreamWrapper::AddItem(
-    CConstRef<IFlatItem> item) 
+    CConstRef<IFlatItem> item)
 {
     if( m_pCanceledCallback && m_pCanceledCallback->IsCanceled() ) {
-        NCBI_THROW(CFlatException, eHaltRequested, 
+        NCBI_THROW(CFlatException, eHaltRequested,
             "FlatFileGeneration canceled by ICancel callback");
     }
     m_pUnderlying->AddItem(item);
