@@ -1,4 +1,4 @@
-/*  $Id: flat_seqloc.cpp 603724 2020-03-16 16:38:27Z kans $
+/*  $Id: flat_seqloc.cpp 665431 2023-04-04 17:43:43Z stakhovv $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -82,7 +82,7 @@ static bool s_IsVirtualSeqInt
 static bool s_IsVirtualLocation(const CSeq_loc& loc, const CBioseq_Handle& seq)
 {
     const CSeq_id* id = loc.GetId();
-    return (id != NULL) ?
+    return id ?
         s_IsVirtualId(CSeq_id_Handle::GetHandle(*id), seq) :
         false;
 }
@@ -198,7 +198,7 @@ CFlatSeqLoc::CFlatSeqLoc
             for( unsigned int id_idx = 0; id_idx < handles_vec.size(); ++id_idx ) {
                 CSeq_id_Handle acc_handle = results[id_idx];
                 if( acc_handle ) {
-                    m_ToAccessionMap.Insert( 
+                    m_ToAccessionMap.Insert(
                         handles_vec[id_idx], acc_handle );
                 }
             }
@@ -269,10 +269,10 @@ bool CFlatSeqLoc::x_Add
     case CSeq_loc::e_Null:
     {{
         const CFlatGapLoc* gap = dynamic_cast<const CFlatGapLoc*>(&loc);
-        if (gap == 0) {
+        if (! gap) {
             oss << "gap()";
             break;
-        } 
+        }
         size_t uLength = gap->GetLength();
         const CInt_fuzz* fuzz = gap->GetFuzz();
         oss << "gap(";
@@ -315,7 +315,7 @@ bool CFlatSeqLoc::x_Add
     }}
     case CSeq_loc::e_Packed_int:
     {{
-        // Note: At some point, we should add the "join inside order" logic here (like for the e_Mix case), but since I didn't immediately find a 
+        // Note: At some point, we should add the "join inside order" logic here (like for the e_Mix case), but since I didn't immediately find a
         // test case in the repository, I'm taking the conservative approach and leaving it alone for now.
         oss << prefix;
         const char* delim = "";
@@ -351,7 +351,7 @@ bool CFlatSeqLoc::x_Add
         const char* delim = "";
         ITERATE (CPacked_seqpnt::TPoints, it, ppnt.GetPoints()) {
             oss << delim;
-            const CInt_fuzz* fuzz = ppnt.CanGetFuzz() ? &ppnt.GetFuzz() : 0;
+            const CInt_fuzz* fuzz = ppnt.CanGetFuzz() ? &ppnt.GetFuzz() : nullptr;
             if (!x_Add(*it, fuzz, oss, ( ctx.Config().DoHTML() ? eHTML_Yes : eHTML_None ), eForce_None, eSource_Other, show_all_accns, suppress_accession )) {
                 delim = "";
             } else {
@@ -387,7 +387,7 @@ bool CFlatSeqLoc::x_Add
 
              // save some work by using what was done on the last loop iteration
              // (this is set before the loop on the first iteration)
-             const bool this_is_virtual = next_is_virtual; 
+             const bool this_is_virtual = next_is_virtual;
 
              // get iterator to next one
              CSeq_loc_CI next = it;
@@ -396,7 +396,7 @@ bool CFlatSeqLoc::x_Add
              // begin join in order, if necessary
              next_is_virtual = ( ! next || next.GetEmbeddingSeq_loc().IsNull() || s_IsVirtualLocation( next.GetEmbeddingSeq_loc(), seq ) );
              if( is_flat_order ) {
-                 if( ( this_loc.IsInt() || this_loc.IsPnt() ) && 
+                 if( ( this_loc.IsInt() || this_loc.IsPnt() ) &&
                      ! join_inside_order && ! this_is_virtual && ! next_is_virtual ) {
                      oss << "join(";
                      join_inside_order = true;
@@ -483,7 +483,7 @@ bool CFlatSeqLoc::x_FuzzToDisplayed(const CSeq_interval& si) const
         return false;
     }
 
-    const CInt_fuzz& to_fuzz = si.GetFuzz_to(); 
+    const CInt_fuzz& to_fuzz = si.GetFuzz_to();
 /* Range fuzziness is not displayed
     if (to_fuzz.IsRange()) {
         return true;
@@ -530,16 +530,16 @@ bool CFlatSeqLoc::x_Add
     x_AddID(si.GetId(), oss, ctx, type, show_all_accns, suppress_accession);
 
     // get the fuzz we need, but certain kinds of fuzz do not belong in an interval
-    const CSeq_interval::TFuzz_from *from_fuzz = (si.IsSetFuzz_from() ? &si.GetFuzz_from() : 0);
+    const CSeq_interval::TFuzz_from *from_fuzz = (si.IsSetFuzz_from() ? &si.GetFuzz_from() : nullptr);
 
     x_Add(from, from_fuzz, oss, ( do_html ? eHTML_Yes : eHTML_None ));
 
-    if ( (type == eType_assembly) || 
-         ((from != to || x_FuzzToDisplayed(si)) ) ) 
+    if ( (type == eType_assembly) ||
+         ((from != to || x_FuzzToDisplayed(si)) ) )
     {
         oss << "..";
 
-        const CSeq_interval::TFuzz_from *to_fuzz = (si.IsSetFuzz_to() ? &si.GetFuzz_to() : 0);
+        const CSeq_interval::TFuzz_from *to_fuzz = (si.IsSetFuzz_to() ? &si.GetFuzz_to() : nullptr);
 
         x_Add(to, to_fuzz, oss, ( do_html ? eHTML_Yes : eHTML_None ));
     }
@@ -565,7 +565,7 @@ bool CFlatSeqLoc::x_Add
     }
 
     const bool do_html = ctx.Config().DoHTML();
-    const bool is_comp = 
+    const bool is_comp =
         pnt.IsSetStrand()  &&  IsReverse(pnt.GetStrand())  &&  show_comp;
 
     TSeqPos pos = pnt.GetPoint();
@@ -573,8 +573,8 @@ bool CFlatSeqLoc::x_Add
     if( is_comp ) {
         oss << "complement(";
     }
-    x_Add(pos, pnt.IsSetFuzz() ? &pnt.GetFuzz() : 0, 
-        oss, ( do_html ? eHTML_Yes : eHTML_None ), 
+    x_Add(pos, pnt.IsSetFuzz() ? &pnt.GetFuzz() : nullptr,
+        oss, ( do_html ? eHTML_Yes : eHTML_None ),
         ( (eType_assembly == type) ? eForce_ToRange : eForce_None ),
         eSource_Point );
     if( is_comp ) {
@@ -600,7 +600,7 @@ bool CFlatSeqLoc::x_Add
 
     const bool bFromSeqPoint = (source == eSource_Point);
 
-    if ( fuzz != 0 ) {
+    if (fuzz) {
         switch ( fuzz->Which() ) {
         case CInt_fuzz::e_P_m:
             {
@@ -708,7 +708,7 @@ void CFlatSeqLoc::x_AddID
 
     CConstRef<CSeq_id> idp;
     try {
-        CSeq_id_Handle handle = 
+        CSeq_id_Handle handle =
             m_ToAccessionMap.Get( CSeq_id_Handle::GetHandle(id) );
         if( handle ) {
             idp = handle.GetSeqId();
@@ -741,11 +741,11 @@ bool CFlatSeqLoc::x_IsAccessionVersion( CSeq_id_Handle id )
         return false;
     }
 
-    return ( seq_id->GetTextseq_Id() != NULL );
+    return (seq_id->GetTextseq_Id() != nullptr);
 }
 
-void 
-CFlatSeqLoc::CGuardedToAccessionMap::Insert( 
+void
+CFlatSeqLoc::CGuardedToAccessionMap::Insert(
     CSeq_id_Handle from, CSeq_id_Handle to )
 {
     CFastMutexGuard guard(m_MutexForTheMap);
