@@ -1,4 +1,4 @@
-/*  $Id: keywords_item.cpp 627647 2021-03-16 18:16:29Z ivanov $
+/*  $Id: keywords_item.cpp 683835 2024-06-06 12:08:29Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -112,7 +112,7 @@ DEFINE_STATIC_ARRAY_MAP(TStaticKeywordSet, sc_GSS_kw, sc_GSS);
 
 // STS keywords
 static const char* const sc_STS[] = {
-  "STS", "STS (sequence tagged site)", "STS sequence", 
+  "STS", "STS (sequence tagged site)", "STS sequence",
   "STS(sequence tagged site)", "sequence tagged site"
 };
 DEFINE_STATIC_ARRAY_MAP(TStaticKeywordSet, sc_STS_kw, sc_STS);
@@ -128,7 +128,7 @@ static bool s_CheckSpecialKeyword(const string& keyword, ETechFlags tech)
             return false;
         }
     }
-    
+
     if (tech == eSTS) {
         if (sc_EST_kw.find(keyword.c_str()) != sc_EST_kw.end()) {
             return false;
@@ -137,7 +137,7 @@ static bool s_CheckSpecialKeyword(const string& keyword, ETechFlags tech)
             return false;
         }
     }
-    
+
     if (tech == eGSS) {
         if (sc_EST_kw.find(keyword.c_str()) != sc_EST_kw.end()) {
             return false;
@@ -192,14 +192,11 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
     // add keywords based on mol-info
     ETechFlags tech = e_not_set;
     // don't do tech-related keywords if molinfo isn't set
-    if( ctx.GetMolinfo() != NULL ) {
+    if (ctx.GetMolinfo()) {
         switch ( ctx.GetTech() ) {
         case CMolInfo::eTech_est:
             tech = eEST;
             x_AddKeyword("EST");
-            if (is_env_sample) {
-                x_AddKeyword("ENV");
-            }
             break;
 
         case CMolInfo::eTech_sts:
@@ -210,9 +207,6 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
         case CMolInfo::eTech_survey:
             tech = eGSS;
             x_AddKeyword("GSS");
-            if (is_env_sample) {
-                x_AddKeyword("ENV");
-            }
             break;
 
         case CMolInfo::eTech_htgs_0:
@@ -259,10 +253,13 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
 
         case CMolInfo::eTech_unknown:
         case CMolInfo::eTech_standard:
+        case CMolInfo::eTech_concept_trans:
+        case CMolInfo::eTech_seq_pept:
+        case CMolInfo::eTech_both:
+        case CMolInfo::eTech_seq_pept_overlap:
+        case CMolInfo::eTech_seq_pept_homol:
+        case CMolInfo::eTech_concept_trans_a:
         case CMolInfo::eTech_other:
-            if (is_env_sample) {
-                x_AddKeyword("ENV");
-            }
             break;
 
         default:
@@ -270,6 +267,10 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
         }
     }
 
+    if (is_env_sample) {
+        x_AddKeyword("ENV");
+    }
+
     if (is_metagenome_source) {
         x_AddKeyword("Metagenome Assembled Genome");
         x_AddKeyword("MAG");
@@ -286,7 +287,7 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
                     CSeqdesc_CI desc_ci( nuc, CSeqdesc::e_Molinfo );
                     for( ; desc_ci; ++desc_ci ) {
                         if( desc_ci->GetMolinfo().CanGetTech() &&
-                            desc_ci->GetMolinfo().GetTech() == CMolInfo::eTech_tsa ) 
+                            desc_ci->GetMolinfo().GetTech() == CMolInfo::eTech_tsa )
                         {
                             x_AddKeyword("TSA");
                             x_AddKeyword("Transcriptome Shotgun Assembly");
@@ -341,7 +342,7 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
             FOR_EACH_STRING_IN_LIST ( s_itr, keywords ) {
                 x_AddKeyword(*s_itr);
             }
-        } catch (CException) {
+        } catch (CException&) {
         }
     }
 
@@ -362,6 +363,15 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
         x_AddKeyword("UNVERIFIED_CONTAMINANT");
     }
 
+    CBioseqContext::TUnreviewed urv = ctx.GetUnreviewedType();
+    if (urv == 0) {
+        //x_AddKeyword("UNREVIEWED");
+    }
+    if ((urv & CBioseqContext::fUnreviewed_Unannotated) != 0) {
+        x_AddKeyword("UNREVIEWED");
+        x_AddKeyword("UNREVIEWED_UNANNOTATED");
+    }
+
     if (ctx.IsEncode()) {
         x_AddKeyword("ENCODE");
     }
@@ -385,36 +395,30 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
     }
 
     for (CSeqdesc_CI it(ctx.GetHandle());  it;  ++it) {
-        const list<string>* keywords = NULL;
-        
+        const list<string>* keywords = nullptr;
+
         switch (it->Which()) {
-            
         case CSeqdesc::e_Pir:
             keywords = &(it->GetPir().GetKeywords());
             break;
-            
         case CSeqdesc::e_Genbank:
             keywords = &(it->GetGenbank().GetKeywords());
             break;
-            
         case CSeqdesc::e_Sp:
             keywords = &(it->GetSp().GetKeywords());
             break;
-            
         case CSeqdesc::e_Embl:
             keywords = &(it->GetEmbl().GetKeywords());
             break;
-            
         case CSeqdesc::e_Prf:
             keywords = &(it->GetPrf().GetKeywords());
             break;
-        
         default:
-            keywords = NULL;
+            keywords = nullptr;
             break;
         }
-        
-        if (keywords != NULL) {
+
+        if (keywords) {
             if (!IsSetObject()) {
                 x_SetObject(*it);
             }
