@@ -1,4 +1,4 @@
-/*  $Id: locus_item.cpp 577454 2019-01-03 22:58:25Z kans $
+/*  $Id: locus_item.cpp 665431 2023-04-04 17:43:43Z stakhovv $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -87,7 +87,7 @@ void CLocusItem::Format
 (IFormatter& formatter,
  IFlatTextOStream& text_os) const
 {
-    formatter.FormatLocus(*this, text_os); 
+    formatter.FormatLocus(*this, text_os);
 }
 
 
@@ -289,7 +289,7 @@ void CLocusItem::x_SetName(CBioseqContext& ctx)
         const static size_t MAX_LOCUS_ACCN_LEN = 38;
 
         const CTextseq_id* tsip = id.GetSeqId()->GetTextseq_Id();
-        if (tsip != NULL) {
+        if (tsip) {
             if (s_IsGenomeView(ctx)) {
                 if (tsip->IsSetAccession()) {
                     m_Name = tsip->GetAccession();
@@ -349,7 +349,7 @@ void CLocusItem::x_SetLength(CBioseqContext& ctx)
 void CLocusItem::x_SetStrand(CBioseqContext& ctx)
 {
     const CBioseq_Handle& bsh = ctx.GetHandle();
-    
+
     CSeq_inst::TMol bmol = bsh.IsSetInst_Mol() ?
         bsh.GetInst_Mol() : CSeq_inst::eMol_not_set;
 
@@ -365,12 +365,12 @@ void CLocusItem::x_SetStrand(CBioseqContext& ctx)
         if ( bmol == CSeq_inst::eMol_dna  &&  m_Strand == CSeq_inst::eStrand_ds ) {
             m_Strand = CSeq_inst::eStrand_not_set;
         }
-        
+
         // ss-any RNA don't show ss
         const bool is_nucleic_acid = bmol <= CSeq_inst::eMol_rna;
-        const bool is_in_set_1 = (m_Biomol >= CMolInfo::eBiomol_mRNA && 
+        const bool is_in_set_1 = (m_Biomol >= CMolInfo::eBiomol_mRNA &&
             m_Biomol <= CMolInfo::eBiomol_peptide);
-        const bool is_in_set_2 = (m_Biomol >= CMolInfo::eBiomol_cRNA && 
+        const bool is_in_set_2 = (m_Biomol >= CMolInfo::eBiomol_cRNA &&
             m_Biomol <= CMolInfo::eBiomol_tmRNA);
         if ( (! is_nucleic_acid  || is_in_set_1 || is_in_set_2)  &&
             m_Strand == CSeq_inst::eStrand_ss ) {
@@ -389,7 +389,7 @@ void CLocusItem::x_SetBiomol(CBioseqContext& ctx)
     }
 
     CSeq_inst::TMol bmol = ctx.GetHandle().GetBioseqMolType();
-    
+
     if ( bmol > CSeq_inst::eMol_aa ) {
         bmol = CSeq_inst::eMol_not_set;
     }
@@ -422,7 +422,7 @@ void CLocusItem::x_SetBiomol(CBioseqContext& ctx)
 void CLocusItem::x_SetTopology(CBioseqContext& ctx)
 {
     const CBioseq_Handle& bsh = ctx.GetHandle();
-    
+
     m_Topology = bsh.GetInst_Topology();
     const CSeq_loc& loc = ctx.GetLocation();
     if ( loc.IsWhole() ) {
@@ -451,7 +451,7 @@ static CTempString x_GetDivisionProcIdx(const CBioseqContext& ctx, const CBioseq
 {
     CTempString division;
 
-    const CBioSource* bsrc = 0;
+    const CBioSource* bsrc = nullptr;
 
     CRef<CSeqEntryIndex> idx = ctx.GetSeqEntryIndex();
     if (! idx) return division;
@@ -527,10 +527,16 @@ static CTempString x_GetDivisionProcIdx(const CBioseqContext& ctx, const CBioseq
         is_transgenic ) {
         division = "SYN";
     } else if (is_env_sample) {
-        if (tech == CMolInfo::eTech_unknown  ||
-            tech == CMolInfo::eTech_standard ||
-            tech == CMolInfo::eTech_htgs_3   ||
-            tech == CMolInfo::eTech_wgs      ||
+        if (tech == CMolInfo::eTech_unknown          ||
+            tech == CMolInfo::eTech_standard         ||
+            tech == CMolInfo::eTech_htgs_3           ||
+            tech == CMolInfo::eTech_wgs              ||
+            tech == CMolInfo::eTech_concept_trans    ||
+            tech == CMolInfo::eTech_seq_pept         ||
+            tech == CMolInfo::eTech_both             ||
+            tech == CMolInfo::eTech_seq_pept_overlap ||
+            tech == CMolInfo::eTech_seq_pept_homol   ||
+            tech == CMolInfo::eTech_concept_trans_a  ||
             tech == CMolInfo::eTech_other) {
             division = "ENV";
         }
@@ -547,46 +553,43 @@ static CTempString x_GetDivisionProcIdx(const CBioseqContext& ctx, const CBioseq
         }
     }
     if ( is_prot ) {
-        CRef<CFeatureIndex> sfxp = bsx->GetFeatureForProduct();
-        if (sfxp) {
-            CSeq_feat_Handle hdl = sfxp->GetSeqFeatHandle();
-            if (hdl) {
-                const CSeq_loc & cds_loc = hdl.GetLocation();
-                const CSeq_id *pSeqId = cds_loc.GetId();
-                CBioseq_Handle nuc;
-                if( pSeqId ) {
-                    nuc = bsh.GetScope().GetBioseqHandle(*pSeqId);
-                } else {
-                    // fall back on first Seq-id in location that is local to this Seq-entry
-                    // (example where this code is needed to prevent a crash: AL772325.4 proteins)
-                    ITERATE( CSeq_loc, cds_loc_iter, cds_loc ) {
-                        try {
-                            CSeq_id_Handle cds_loc_piece_id = cds_loc_iter.GetSeq_id_Handle();
-                            if( cds_loc_piece_id ) {
-                                nuc = bsh.GetScope().GetBioseqHandleFromTSE(
-                                    cds_loc_piece_id, bsh );
-                                if( nuc ) {
-                                    break;
-                                }
+        const CSeq_feat* cds = GetCDSForProduct(bsh);
+        if ( cds ) {
+            const CSeq_loc & cds_loc = cds->GetLocation();
+            const CSeq_id *pSeqId = cds_loc.GetId();
+            CBioseq_Handle nuc;
+            if( pSeqId ) {
+                nuc = bsh.GetScope().GetBioseqHandle(*pSeqId);
+            } else {
+                // fall back on first Seq-id in location that is local to this Seq-entry
+                // (example where this code is needed to prevent a crash: AL772325.4 proteins)
+                ITERATE( CSeq_loc, cds_loc_iter, cds_loc ) {
+                    try {
+                        CSeq_id_Handle cds_loc_piece_id = cds_loc_iter.GetSeq_id_Handle();
+                        if( cds_loc_piece_id ) {
+                            nuc = bsh.GetScope().GetBioseqHandleFromTSE(
+                                cds_loc_piece_id, bsh );
+                            if( nuc ) {
+                                break;
                             }
-                        } catch(...) {
-                            // ignore exceptions because it's common for us to fail
-                            // to find something since we're only looking locally
                         }
+                    } catch(...) {
+                        // ignore exceptions because it's common for us to fail
+                        // to find something since we're only looking locally
                     }
                 }
-                if( nuc ) {
-                    ITERATE (CBioseq_Handle::TId, iter, nuc.GetId()) {
-                        if (*iter  &&  iter->GetSeqId()->IsPatent()) {
-                            division = "PAT";
-                            break;
-                        }
+            }
+            if( nuc ) {
+                ITERATE (CBioseq_Handle::TId, iter, nuc.GetId()) {
+                    if (*iter  &&  iter->GetSeqId()->IsPatent()) {
+                        division = "PAT";
+                        break;
                     }
                 }
             }
         }
     }
-    
+
     // more complicated code for division, if necessary, goes here
 
     for (CSeqdesc_CI gb_desc(bsh, CSeqdesc::e_Genbank); gb_desc; ++gb_desc) {
@@ -614,7 +617,7 @@ static CTempString x_GetDivisionProc(const CBioseq_Handle& bsh, bool is_prot,
 {
     CTempString division;
 
-    const CBioSource* bsrc = 0;
+    const CBioSource* bsrc = nullptr;
 
     // Find the BioSource object for this sequnece.
     CSeqdesc_CI src_desc(bsh, CSeqdesc::e_Source);
@@ -625,7 +628,7 @@ static CTempString x_GetDivisionProc(const CBioseq_Handle& bsh, bool is_prot,
         if ( feat ) {
             bsrc = &(feat->GetData().GetBiosrc());
         } else if ( is_prot ) {
-            // if protein with no sources, get sources applicable to 
+            // if protein with no sources, get sources applicable to
             // DNA location of CDS
             const CSeq_feat* cds = GetCDSForProduct(bsh);
             if ( cds ) {
@@ -637,7 +640,7 @@ static CTempString x_GetDivisionProc(const CBioseq_Handle& bsh, bool is_prot,
                 if ( nuc_fsrc ) {
                     bsrc = &(nuc_fsrc->GetData().GetBiosrc());
                 } else {
-                    CBioseq_Handle nuc = 
+                    CBioseq_Handle nuc =
                         bsh.GetScope().GetBioseqHandle(cds->GetLocation());
                     CSeqdesc_CI nuc_dsrc(nuc, CSeqdesc::e_Source);
                     if ( nuc_dsrc ) {
@@ -703,10 +706,16 @@ static CTempString x_GetDivisionProc(const CBioseq_Handle& bsh, bool is_prot,
         is_transgenic ) {
         division = "SYN";
     } else if (is_env_sample) {
-        if (tech == CMolInfo::eTech_unknown  ||
-            tech == CMolInfo::eTech_standard ||
-            tech == CMolInfo::eTech_htgs_3   ||
-            tech == CMolInfo::eTech_wgs      ||
+        if (tech == CMolInfo::eTech_unknown          ||
+            tech == CMolInfo::eTech_standard         ||
+            tech == CMolInfo::eTech_htgs_3           ||
+            tech == CMolInfo::eTech_wgs              ||
+            tech == CMolInfo::eTech_concept_trans    ||
+            tech == CMolInfo::eTech_seq_pept         ||
+            tech == CMolInfo::eTech_both             ||
+            tech == CMolInfo::eTech_seq_pept_overlap ||
+            tech == CMolInfo::eTech_seq_pept_homol   ||
+            tech == CMolInfo::eTech_concept_trans_a  ||
             tech == CMolInfo::eTech_other) {
             division = "ENV";
         }
@@ -759,7 +768,7 @@ static CTempString x_GetDivisionProc(const CBioseq_Handle& bsh, bool is_prot,
             }
         }
     }
-    
+
     // more complicated code for division, if necessary, goes here
 
     for (CSeqdesc_CI gb_desc(bsh, CSeqdesc::e_Genbank); gb_desc; ++gb_desc) {
@@ -782,7 +791,6 @@ static CTempString x_GetDivisionProc(const CBioseq_Handle& bsh, bool is_prot,
 }
 
 string CLocusItem::GetDivision(const CBioseq_Handle& bsh, const CBioseqContext* ctx)
-
 {
     if ( bsh.IsSetInst_Repr() && bsh.GetInst_Repr() == CSeq_inst::eRepr_delta) {
         if (bsh.IsSetInst_Ext()) {
@@ -797,7 +805,7 @@ string CLocusItem::GetDivision(const CBioseq_Handle& bsh, const CBioseqContext*
         }
     }
 
-    CMolInfo::TTech tech = 0;
+    CMolInfo::TTech tech = CMolInfo::eTech_unknown;
 
     CSeqdesc_CI::TDescChoices desc_choices;
     desc_choices.reserve(1);
@@ -840,13 +848,12 @@ void CLocusItem::x_SetDivision(CBioseqContext& ctx)
 
     const CMolInfo* molinfo = dynamic_cast<const CMolInfo*>(GetObject());
     if ( ctx.Config().IsFormatEMBL() ) {
-        for ( CSeqdesc_CI embl_desc(bsh, CSeqdesc::e_Embl); 
-            embl_desc; 
+        for ( CSeqdesc_CI embl_desc(bsh, CSeqdesc::e_Embl);
+            embl_desc;
             ++embl_desc ) {
                 const CEMBL_block& embl = embl_desc->GetEmbl();
                 if ( embl.CanGetDiv() ) {
-                    if ( embl.GetDiv() == CEMBL_block::eDiv_other  &&
-                        molinfo == 0 ) {
+                    if (embl.GetDiv() == CEMBL_block::eDiv_other && ! molinfo) {
                             m_Division = "HUM";
                         } else {
                             m_Division = embl.GetDiv();
@@ -869,8 +876,8 @@ void CLocusItem::x_SetDate(CBioseqContext& ctx)
     const CBioseq_Handle& bsh = ctx.GetHandle();
 
     const CDate* date = x_GetDateForBioseq(bsh);
-    if ( date == 0 ) {
-        // if bioseq is product of CDS or mRNA feature, get 
+    if (! date) {
+        // if bioseq is product of CDS or mRNA feature, get
         // date from parent bioseq.
         CBioseq_Handle parent = GetNucleotideParent(bsh);
         if ( parent ) {
@@ -887,7 +894,7 @@ void CLocusItem::x_SetDate(CBioseqContext& ctx)
 
 const CDate* CLocusItem::x_GetDateForBioseq(const CBioseq_Handle& bsh) const
 {
-    const CDate* result = 0;
+    const CDate* result = nullptr;
 
     {{
         CSeqdesc_CI desc(bsh, CSeqdesc::e_Update_date);
@@ -897,7 +904,7 @@ const CDate* CLocusItem::x_GetDateForBioseq(const CBioseq_Handle& bsh) const
     }}
 
     {{
-        // temporarily (???) also look at genbank block entry date 
+        // temporarily (???) also look at genbank block entry date
         CSeqdesc_CI desc(bsh, CSeqdesc::e_Genbank);
         if ( desc ) {
             const CGB_block& gb = desc->GetGenbank();
@@ -965,11 +972,11 @@ const CDate* CLocusItem::x_GetDateForBioseq(const CBioseq_Handle& bsh) const
 
 const CDate* CLocusItem::x_GetLaterDate(const CDate* d1, const CDate* d2) const
 {
-    if ( d1 == 0 || d1->Which() == CDate::e_Str ) {
+    if (! d1 || d1->Which() == CDate::e_Str) {
         return d2;
     }
 
-    if ( d2 == 0 || d2->Which() == CDate::e_Str ) {
+    if (! d2 || d2->Which() == CDate::e_Str) {
         return d1;
     }
 
