@@ -1,4 +1,4 @@
-/*  $Id: qualifiers.cpp 629259 2021-04-13 13:28:40Z ivanov $
+/*  $Id: qualifiers.cpp 679820 2024-03-04 18:32:45Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -84,27 +84,25 @@ static bool s_StringIsJustQuotes(const string& str)
 }
 
 static string s_GetGOText(
-    const CUser_field& field, 
-    const bool is_ftable, 
+    const CUser_field& field,
+    const bool is_ftable,
     const bool is_html)
 {
-    const string* text_string = NULL,
-                * evidence = NULL,
-                * go_id = NULL,
-                * go_ref = NULL;
+    const string *text_string = nullptr,
+                 *evidence = nullptr,
+                 *go_id = nullptr,
+                 *go_ref = nullptr;
     string temp;
     int pmid = 0;
-    
-    
 
     ITERATE (CUser_field::C_Data::TFields, it, field.GetData().GetFields()) {
         if ( !(*it)->IsSetLabel()  ||  !(*it)->GetLabel().IsStr() ) {
             continue;
         }
-        
+
         const string& label = (*it)->GetLabel().GetStr();
         const CUser_field::C_Data& data = (*it)->GetData();
-        
+
         if (data.IsStr()) {
             if (label == "text string") {
                 text_string = &data.GetStr();
@@ -124,29 +122,29 @@ static string s_GetGOText(
             }
         }
     }
-    
+
     string go_text;
-    if (text_string != NULL) {
+    if (text_string) {
         go_text = *text_string;
     }
     if ( is_ftable ) {
         go_text += '|';
-        if (go_id != NULL) {
+        if (go_id) {
             go_text += *go_id;
         }
         go_text += '|';
         if ( pmid != 0 ) {
             go_text +=  NStr::IntToString(pmid);
         }
-        if (evidence != NULL) {
+        if (evidence) {
             go_text += '|';
             go_text += *evidence;
         }
-    } else { 
+    } else {
         bool add_dash = false;
         // RW-922 only make one link from GO:id - text.
         go_text.clear();
-        if (go_id != NULL) {
+        if (go_id) {
             if( is_html ) {
                 go_text += "<a href=\"";
                 go_text += strLinkBaseGeneOntology + *go_id + "\">";
@@ -155,34 +153,34 @@ static string s_GetGOText(
             go_text += *go_id;
             add_dash = true;
         }
-        if ( text_string != 0 && text_string->length() > 0 ) {
+        if (text_string && text_string->length() > 0) {
             if (add_dash) {
               go_text += string( " - " );
             }
             // NO, we NO LONGER have the dash here even if there's no go_id (RETAIN compatibility with CHANGE in C)
             go_text += *text_string;
         }
-        if( is_html && go_id != NULL ) {
+        if (is_html && go_id) {
             go_text += "</a>";
         }
-        if ( evidence != 0 ) {
+        if (evidence) {
             go_text += string( " [Evidence " ) + *evidence + string( "]" );
         }
         if ( pmid != 0 ) {
             string pmid_str = NStr::IntToString(pmid);
 
             go_text += " [PMID ";
-            if( is_html && go_id != NULL ) {
+            if (is_html && go_id) {
                 go_text += "<a href=\"";
                 go_text += strLinkBasePubmed + pmid_str + "\">";
             }
             go_text += pmid_str;
-            if( is_html && go_id != NULL ) {
+            if(is_html && go_id) {
                 go_text += "</a>";
             }
             go_text += "]";
         }
-        if ( go_ref != 0 ) {
+        if (go_ref) {
             go_text += " [GO Ref ";
             if( is_html ) {
                 go_text += "<a href=\"";
@@ -214,7 +212,7 @@ static void s_ReplaceUforT(string& codon)
 static char s_MakeDegenerateBase(const string &str1, const string& str2)
 {
     static const char kIdxToSymbol[] = "?ACMGRSVUWYHKDBN";
-    
+
     vector<char> symbol_to_idx(256, '\0');
     for (size_t i = 0; i < sizeof(kIdxToSymbol) - 1; ++i) {
         symbol_to_idx[kIdxToSymbol[i]] = i;
@@ -234,7 +232,7 @@ static size_t s_ComposeCodonRecognizedStr(const CTrna_ext& trna, string& recogni
     }
 
     list<string> codons;
-    
+
     ITERATE (CTrna_ext::TCodon, it, trna.GetCodon()) {
         string codon = CGen_code_table::IndexToCodon(*it);
         s_ReplaceUforT(codon);
@@ -278,7 +276,7 @@ static bool s_AltitudeIsValid(const string & str )
     // we use c-style strings so we don't have to worry about constantly
     // checking if we're out of bounds: we get a '\0' which lets us be cleaner
     const char *pch = str.c_str();
-    
+
     // optional sign at start
     if( *pch == '+' || *pch == '-' ) {
         ++pch;
@@ -310,14 +308,14 @@ static bool s_AltitudeIsValid(const string & str )
 
 CFormatQual::CFormatQual
 (const CTempString& name,
- const CTempString& value, 
+ const CTempString& value,
  const CTempString& prefix,
  const CTempString& suffix,
  TStyle style,
  TFlags flags,
  ETrim trim ) :
-  m_Name(name), 
-  m_Prefix(prefix), 
+  m_Name(name),
+  m_Prefix(prefix),
   m_Suffix(suffix),
   m_Style(style), m_Flags(flags), m_Trim(trim), m_AddPeriod(false)
 {
@@ -326,7 +324,7 @@ CFormatQual::CFormatQual
 
 
 CFormatQual::CFormatQual(const CTempString& name, const CTempString& value, TStyle style, TFlags flags, ETrim trim) :
-    m_Name(name), 
+    m_Name(name),
     m_Prefix(" "), m_Suffix(kEmptyStr),
     m_Style(style), m_Flags(flags), m_Trim(trim), m_AddPeriod(false)
 {
@@ -378,7 +376,7 @@ DEFINE_STATIC_ARRAY_MAP(TNameTildeStyleMap, sc_NameTildeStyleMap, kNameTildeStyl
 // a few kinds don't use the default tilde style
 ETildeStyle s_TildeStyleFromName( const string &name )
 {
-    TNameTildeStyleMap::const_iterator result = sc_NameTildeStyleMap.find( name.c_str() ); 
+    TNameTildeStyleMap::const_iterator result = sc_NameTildeStyleMap.find( name.c_str() );
     if( sc_NameTildeStyleMap.end() == result ) {
         return eTilde_space;
     } else {
@@ -404,7 +402,7 @@ void CFlatStringQVal::Format(TFlatQuals& q, const CTempString& name,
 
     ETildeStyle tilde_style = s_TildeStyleFromName( name );
     ExpandTildes(m_Value, tilde_style);
-                
+
     const bool is_note = s_IsNote(flags, ctx);
 
     // e.g. CP001398
@@ -419,18 +417,18 @@ void CFlatStringQVal::Format(TFlatQuals& q, const CTempString& name,
     // or something, but this is fine while we're only checking for
     // one type.
     // This prevents quals like /metagenomic="metagenomic" (e.g. EP508672)
-    const bool forceNoValue = ( 
+    const bool forceNoValue = (
         ! ctx.Config().SrcQualsToNote() &&
-        name == m_Value && 
+        name == m_Value &&
         name == "metagenomic" );
 
     const bool prependNewline = (flags & fPrependNewline) && ! q.empty();
     TFlatQual qual = x_AddFQ(
-        q, (is_note ? "note" : name), 
-        (  prependNewline ? CTempString("\n" + m_Value) : CTempString(m_Value) ), 
+        q, (is_note ? "note" : name),
+        (  prependNewline ? CTempString("\n" + m_Value) : CTempString(m_Value) ),
         ( forceNoValue ? CFormatQual::eEmpty : m_Style ),
         0, m_Trim );
-    
+
     if ((flags & fAddPeriod)  &&  qual) {
         qual->SetAddPeriod();
     }
@@ -511,7 +509,7 @@ void CFlatSiteQVal::Format
         m_Value = "transit peptide";
     }
     if (m_Value != "transit peptide" && m_Value != "signal peptide" &&
-        m_Value != "transmembrane region" && s_IsNote(flags, ctx)) 
+        m_Value != "transmembrane region" && s_IsNote(flags, ctx))
     {
         const static char *pchSiteSuffix = " site";
         if( ! NStr::EndsWith(m_Value, pchSiteSuffix) ) {
@@ -539,7 +537,7 @@ void CFlatStringListQVal::Format
         m_Suffix = &kSemicolon;
     }
 
-    x_AddFQ(q, 
+    x_AddFQ(q,
             (s_IsNote(flags, ctx) ? "note" : name),
             JoinString(m_Value, "; "),
             m_Style);
@@ -620,7 +618,7 @@ void CFlatCodeBreakQVal::Format(TFlatQuals& q, const CTempString& name,
         }
 
         string pos = CFlatSeqLoc( (*it)->GetLoc(), ctx).GetString();
-        x_AddFQ(q, name, "(pos:" + pos + ",aa:" + aa + ')', 
+        x_AddFQ(q, name, "(pos:" + pos + ",aa:" + aa + ')',
             CFormatQual::eUnquoted);
     }
 }
@@ -637,7 +635,7 @@ void CFlatNomenclatureQVal::Format(TFlatQuals& q, const CTempString& name,
     }
 
     // we build up the final result in this variable
-    string nomenclature; 
+    string nomenclature;
 
     // add the status part
     switch( m_Value->GetStatus() ) {
@@ -664,7 +662,7 @@ void CFlatNomenclatureQVal::Format(TFlatQuals& q, const CTempString& name,
     // add the source part, if any
     if( m_Value->CanGetSource() ) {
         const CGene_nomenclature_Base::TSource& source = m_Value->GetSource();
-        
+
         if( source.CanGetDb() && ! source.GetDb().empty() && source.CanGetTag() ) {
             if( source.GetTag().IsId() || ( source.GetTag().IsStr() && ! source.GetTag().GetStr().empty() ) ) {
                 nomenclature += " | Provided by: " + source.GetDb() + ":";
@@ -699,7 +697,7 @@ void CFlatCodonQVal::Format(TFlatQuals& q, const CTempString& name, CBioseqConte
 
 CFlatExperimentQVal::CFlatExperimentQVal(
     const string& value )
-    : m_str( value ) 
+    : m_str( value )
 {
     if ( m_str.empty() ) {
         m_str = "experimental evidence, no additional details recorded";
@@ -722,7 +720,7 @@ CFlatInferenceQVal::CFlatInferenceQVal( const string& gbValue ) :
     //  However, it can't be overriden to be just anything, only certain strings
     //  are allowed.
     //  The following code will change m_str from its default if gbValue is a
-    //  legal replacement for "non-experimental ...", and leave it alone 
+    //  legal replacement for "non-experimental ...", and leave it alone
     //  otherwise.
     //
     string prefix;
@@ -748,12 +746,12 @@ void CFlatIllegalQVal::Format(TFlatQuals& q, const CTempString& name, CBioseqCon
 
     // special case: in sequin mode, orig_protein_id and orig_transcript_id removed,
     // even though we usually keep illegal quals in sequin mode
-    if( m_Value->GetQual() == "orig_protein_id" || 
+    if( m_Value->GetQual() == "orig_protein_id" ||
         m_Value->GetQual() == "orig_transcript_id" )
     {
         return;
     }
-        
+
     x_AddFQ(q, m_Value->GetQual(), m_Value->GetVal());
 }
 
@@ -761,7 +759,7 @@ void CFlatIllegalQVal::Format(TFlatQuals& q, const CTempString& name, CBioseqCon
 void CFlatMolTypeQVal::Format(TFlatQuals& q, const CTempString& name,
                             CBioseqContext& ctx, IFlatQVal::TFlags flags) const
 {
-    const char* s = 0;
+    const char* s = nullptr;
     switch ( m_Biomol ) {
 
     default:
@@ -769,79 +767,79 @@ void CFlatMolTypeQVal::Format(TFlatQuals& q, const CTempString& name,
 
     case CMolInfo::eBiomol_unknown:
         switch ( m_Mol ) {
-        case CSeq_inst::eMol_dna:  
-            s = "unassigned DNA"; 
+        case CSeq_inst::eMol_dna:
+            s = "unassigned DNA";
             break;
-        case CSeq_inst::eMol_rna:  
-            s = "unassigned RNA"; 
+        case CSeq_inst::eMol_rna:
+            s = "unassigned RNA";
             break;
-        default:                   
+        default:
             break;
         }
         break;
 
     case CMolInfo::eBiomol_genomic:
         switch ( m_Mol ) {
-        case CSeq_inst::eMol_dna:  
-            s = "genomic DNA";  
+        case CSeq_inst::eMol_dna:
+            s = "genomic DNA";
             break;
-        case CSeq_inst::eMol_rna:  
-            s = "genomic RNA";  
+        case CSeq_inst::eMol_rna:
+            s = "genomic RNA";
             break;
-        default:                   
+        default:
             break;
         }
         break;
 
-    case CMolInfo::eBiomol_mRNA:     
-        s = "mRNA";      
+    case CMolInfo::eBiomol_mRNA:
+        s = "mRNA";
         break;
 
-    case CMolInfo::eBiomol_rRNA:     
-        s = "rRNA";      
-        break;
-    
-    case CMolInfo::eBiomol_tRNA:     
-        s = "tRNA";      
+    case CMolInfo::eBiomol_rRNA:
+        s = "rRNA";
         break;
 
-    case CMolInfo::eBiomol_pre_RNA:  
-    case CMolInfo::eBiomol_snRNA:    
-    case CMolInfo::eBiomol_scRNA:    
+    case CMolInfo::eBiomol_tRNA:
+        s = "tRNA";
+        break;
+
+    case CMolInfo::eBiomol_pre_RNA:
+    case CMolInfo::eBiomol_snRNA:
+    case CMolInfo::eBiomol_scRNA:
     case CMolInfo::eBiomol_snoRNA:
     case CMolInfo::eBiomol_ncRNA:
     case CMolInfo::eBiomol_tmRNA:
     case CMolInfo::eBiomol_transcribed_RNA:
-        s = "transcribed RNA";     
+        s = "transcribed RNA";
         break;
 
     case CMolInfo::eBiomol_other_genetic:
     case CMolInfo::eBiomol_other:
         switch ( m_Mol ) {
-        case CSeq_inst::eMol_dna:  
-            s = "other DNA";  
+        case CSeq_inst::eMol_dna:
+            s = "other DNA";
             break;
-        case CSeq_inst::eMol_rna:  
-            s = "other RNA";  
+        case CSeq_inst::eMol_rna:
+            s = "other RNA";
             break;
-        default:                   
+        default:
             break;
         }
         break;
 
     case CMolInfo::eBiomol_cRNA:
-        s = "viral cRNA";     
+        s = "viral cRNA";
         break;
 
     }
 
-    if ( s == 0 ) {
+    if (! s) {
         switch ( m_Mol ) {
         case CSeq_inst::eMol_rna:
             s = "unassigned RNA";
             break;
         case CSeq_inst::eMol_aa:
-            s = 0;
+            s = nullptr;
             break;
         case CSeq_inst::eMol_dna:
         default:
@@ -850,7 +848,7 @@ void CFlatMolTypeQVal::Format(TFlatQuals& q, const CTempString& name,
         }
     }
 
-    if ( s != 0 ) {
+    if (s) {
         x_AddFQ(q, name, s);
     }
 }
@@ -875,7 +873,7 @@ void CFlatSubmitterSeqidQVal::Format(TFlatQuals& q, const CTempString& name,
             }
             int num_letters = 0;
             int num_digits = 0;
-            int len = dbname.length();
+            int len = (int) dbname.length();
             if ( len != 6 && len != 8 ) continue;
             bool bail = false;
             for ( int i = 0; i < len; i++ ) {
@@ -922,11 +920,11 @@ void CFlatOrgModQVal::Format(TFlatQuals& q, const CTempString& name,
     CleanAndCompress(subname, subname.c_str());
     NStr::TruncateSpacesInPlace(subname);
     ExpandTildes(subname, ( (flags & IFlatQVal::fIsNote) != 0  ? eTilde_tilde : eTilde_space ) );
-    
+
     if (s_IsNote(flags, ctx)) {
         bool add_period = RemovePeriodFromEnd(subname, true);
         if (!subname.empty() || add_period ) {
-            bool is_src_orgmod_note = 
+            bool is_src_orgmod_note =
                 (flags & IFlatQVal::fIsSource)  &&  (name == "orgmod_note");
             if (is_src_orgmod_note) {
                 if (add_period) {
@@ -936,7 +934,7 @@ void CFlatOrgModQVal::Format(TFlatQuals& q, const CTempString& name,
                 m_Suffix = ( add_period ? &kEOL : &kSemicolonEOL );
                 qual = x_AddFQ(q, "note", subname);
             } else {
-                qual = x_AddFQ(q, "note", string(name) + ": " + subname, 
+                qual = x_AddFQ(q, "note", string(name) + ": " + subname,
                     CFormatQual::eQuoted, CFormatQual::fFlags_showEvenIfRedund );
             }
             if (add_period  &&  qual) {
@@ -955,14 +953,14 @@ void CFlatOrganelleQVal::Format(TFlatQuals& q, const CTempString& name,
                               CBioseqContext&, IFlatQVal::TFlags) const
 {
     const string& organelle
-        = CBioSource::GetTypeInfo_enum_EGenome()->FindName(m_Value, true);
+        = CBioSource::ENUM_METHOD_NAME(EGenome)()->FindName(m_Value, true);
 
     switch (m_Value) {
-    case CBioSource::eGenome_chloroplast: 
+    case CBioSource::eGenome_chloroplast:
     case CBioSource::eGenome_chromoplast:
-    case CBioSource::eGenome_cyanelle:    
+    case CBioSource::eGenome_cyanelle:
     case CBioSource::eGenome_apicoplast:
-    case CBioSource::eGenome_leucoplast:  
+    case CBioSource::eGenome_leucoplast:
     case CBioSource::eGenome_proplastid:
         x_AddFQ(q, name, "plastid:" + organelle);
         break;
@@ -971,7 +969,7 @@ void CFlatOrganelleQVal::Format(TFlatQuals& q, const CTempString& name,
         x_AddFQ(q, name, "mitochondrion:kinetoplast");
         break;
 
-    case CBioSource::eGenome_mitochondrion: 
+    case CBioSource::eGenome_mitochondrion:
     case CBioSource::eGenome_plastid:
     case CBioSource::eGenome_nucleomorph:
     case CBioSource::eGenome_hydrogenosome:
@@ -979,7 +977,7 @@ void CFlatOrganelleQVal::Format(TFlatQuals& q, const CTempString& name,
         x_AddFQ(q, name, organelle);
         break;
 
-    case CBioSource::eGenome_macronuclear: 
+    case CBioSource::eGenome_macronuclear:
     case CBioSource::eGenome_proviral:
         x_AddFQ(q, organelle, kEmptyStr, CFormatQual::eEmpty);
         break;
@@ -987,7 +985,7 @@ void CFlatOrganelleQVal::Format(TFlatQuals& q, const CTempString& name,
 //        x_AddFQ(q, organelle, kEmptyStr, CFormatQual::eEmpty);
         break;
 
-    case CBioSource::eGenome_plasmid: 
+    case CBioSource::eGenome_plasmid:
     case CBioSource::eGenome_transposon:
         x_AddFQ(q, organelle, kEmptyStr);
         break;
@@ -999,7 +997,6 @@ void CFlatOrganelleQVal::Format(TFlatQuals& q, const CTempString& name,
     default:
         break;
     }
-    
 }
 
 
@@ -1048,14 +1045,14 @@ void CFlatPubSetQVal::Format(TFlatQuals& q, const CTempString& name,
                     value = '[' + pub_id_str + ']';
                 }
                 x_AddFQ(q, name, value, CFormatQual::eUnquoted);
-                
+
                 pub_iter = unusedPubs.erase( pub_iter ); // only one citation should be created per reference
                 break; // break so we don't show the same ref more than once
             }
         }
     }
 
-    // out of the pubs which are still unused, we may still be able to salvage some 
+    // out of the pubs which are still unused, we may still be able to salvage some
     // under certain conditions.
     string pubmed;
     if (ctx.IsRefSeq() && !ctx.Config().IsModeRelease()) {
@@ -1083,9 +1080,9 @@ void CFlatPubSetQVal::Format(TFlatQuals& q, const CTempString& name,
     }
 }
 
-void CFlatIntQVal::Format(TFlatQuals& q, const CTempString& name, 
+void CFlatIntQVal::Format(TFlatQuals& q, const CTempString& name,
                           CBioseqContext& ctx, TFlags) const
-{ 
+{
     bool bHtml = ctx.Config().DoHTML();
 
     string value = NStr::IntToString(m_Value);
@@ -1098,7 +1095,7 @@ void CFlatIntQVal::Format(TFlatQuals& q, const CTempString& name,
         link += "</a>";
         value = link;
     }
-    x_AddFQ( q, name, value, CFormatQual::eUnquoted); 
+    x_AddFQ( q, name, value, CFormatQual::eUnquoted);
 }
 
 
@@ -1111,41 +1108,19 @@ void CFlatSeqIdQVal::Format(TFlatQuals& q, const CTempString& name,
     if ( m_Value->IsGi() ) {
         if ( m_GiPrefix ) {
             id_str = "GI:";
-            if ((ctx.Config().HideGI() || ctx.Config().IsPolicyFtp()) && name == "db_xref") return;
+            if ((ctx.Config().HideGI() || ctx.Config().IsPolicyFtp() || ctx.Config().IsPolicyGenomes()) && name == "db_xref") return;
         }
         m_Value->GetLabel(&id_str, CSeq_id::eContent);
     } else {
         id_str = m_Value->GetSeqIdString(true);
     }
 
-#ifdef NEW_HTML_FMT
     if (name == "protein_id") {
        ctx.Config().GetHTMLFormatter().FormatProteinId(id_str, *m_Value, string(id_str));
     }
     if (name == "transcript_id") {
        ctx.Config().GetHTMLFormatter().FormatTranscriptId(id_str, *m_Value, string(id_str));
     }
-#else
-    if (bHtml && name == "protein_id") {
-        string raw_id_str = id_str;
-        string raw_link_str = id_str;
-        CBioseq_Handle bsh = ctx.GetScope().GetBioseqHandle( *m_Value );
-        vector< CSeq_id_Handle > ids = bsh.GetId();
-        ITERATE( vector< CSeq_id_Handle >, it, ids ) {
-            CSeq_id_Handle hid = *it;
-            if ( hid.IsGi() ) {
-                raw_link_str = NStr::NumericToString(hid.GetGi());
-                break;
-            }
-        }
-        id_str = "<a href=\"";
-        id_str += strLinkBaseProt;
-        id_str += raw_link_str;
-        id_str += "\">";
-        id_str += raw_id_str;
-        id_str += "</a>";
-    }
-#endif
     x_AddFQ(q, name, id_str);
 }
 
@@ -1167,8 +1142,16 @@ void s_HtmlizeLatLon( string &subname ) {
     string lon;
     string east_or_west;
 
+    if (subname.length() < 1) {
+       return;
+    }
+    char ch = subname[0];
+    if (ch < '0' || ch > '9') {
+        return;
+    }
+
     // extract the pieces
-    CNcbiIstrstream lat_lon_stream( subname.c_str() );
+    CNcbiIstrstream lat_lon_stream( subname );
     lat_lon_stream >> lat;
     lat_lon_stream >> north_or_south;
     lat_lon_stream >> lon;
@@ -1217,26 +1200,26 @@ void s_HtmlizeLatLon( string &subname ) {
     // now we can form the HTML
     CNcbiOstrstream result;
     /*
-    result << "<a href=\"" << strLinkBaseLatLon << "?lat=" 
-        << lat 
-        << "&amp;lon=" 
-        << lon 
+    result << "<a href=\"" << strLinkBaseLatLon << "?lat="
+        << lat
+        << "&amp;lon="
+        << lon
         << "\">"
         << subname << "</a>";
     */
     result << "<a href=\"" << "https://www.google.com/maps/place/"
-        << lat 
-        << "+" 
-        << lon 
+        << lat
+        << "+"
+        << lon
         << "\">"
         << subname << "</a>";
     subname = CNcbiOstrstreamToString(result);
 }
 
 void CFlatSubSourcePrimer::Format(
-    TFlatQuals& q, 
+    TFlatQuals& q,
     const CTempString& name,
-    CBioseqContext& ctx, 
+    CBioseqContext& ctx,
     IFlatQVal::TFlags flags) const
 {
     vector< string > fwd_names;
@@ -1247,7 +1230,7 @@ void CFlatSubSourcePrimer::Format(
         }
         NStr::Split( fwd_name, ",", fwd_names );
     }
-    
+
     vector< string > rev_names;
     if ( ! m_rev_name.empty() ) {
         string rev_name = m_rev_name;
@@ -1302,6 +1285,57 @@ void CFlatSubSourcePrimer::Format(
     }
 }
 
+string s_TruncateLatLon( string &subname ) {
+    string lat;
+    string north_or_south;
+    string lon;
+    string east_or_west;
+
+    if (subname.length() < 1) {
+       return subname;
+    }
+    char ch = subname[0];
+    if (ch < '0' || ch > '9') {
+        return subname;
+    }
+
+    // extract the pieces
+    CNcbiIstrstream lat_lon_stream( subname );
+    lat_lon_stream >> lat;
+    lat_lon_stream >> north_or_south;
+    lat_lon_stream >> lon;
+    lat_lon_stream >> east_or_west;
+    if( lat_lon_stream.bad() ) {
+        return subname;
+    }
+
+    if( north_or_south != "N" && north_or_south != "S" ) {
+        return subname;
+    }
+
+    if( east_or_west != "E" && east_or_west != "W" ) {
+        return subname;
+    }
+
+    size_t pos = NStr::Find(lat, ".");
+    if (pos > 0) {
+        size_t len = lat.length();
+        if (pos + 9 < len) {
+            lat.erase(pos + 9);
+        }
+    }
+
+    pos = NStr::Find(lon, ".");
+    if (pos > 0) {
+        size_t len = lon.length();
+        if (pos + 9 < len) {
+            lon.erase(pos + 9);
+        }
+    }
+
+    return lat + " " + north_or_south + " " + lon + " " + east_or_west;
+}
+
 void CFlatSubSourceQVal::Format(TFlatQuals& q, const CTempString& name,
                               CBioseqContext& ctx, IFlatQVal::TFlags flags) const
 {
@@ -1329,7 +1363,7 @@ void CFlatSubSourceQVal::Format(TFlatQuals& q, const CTempString& name,
                 m_Suffix = ( add_period ? &kEOL : &kSemicolonEOL );
                 qual = x_AddFQ(q, "note", subname);
             } else {
-                qual = x_AddFQ(q, "note", string(name) + ": " + subname);        
+                qual = x_AddFQ(q, "note", string(name) + ": " + subname);
             }
             if (add_period  &&  qual) {
                 qual->SetAddPeriod();
@@ -1353,6 +1387,7 @@ void CFlatSubSourceQVal::Format(TFlatQuals& q, const CTempString& name,
             break;
 
         case CSubSource::eSubtype_lat_lon:
+            subname = s_TruncateLatLon(subname);
             if( ctx.Config().DoHTML() ) {
                 s_HtmlizeLatLon(subname);
             }
@@ -1362,8 +1397,8 @@ void CFlatSubSourceQVal::Format(TFlatQuals& q, const CTempString& name,
 
         case CSubSource::eSubtype_altitude:
             // skip invalid /altitudes
-            if( s_AltitudeIsValid(subname) || 
-                ( ! ctx.Config().IsModeRelease() && ! ctx.Config().IsModeEntrez() ) ) 
+            if( s_AltitudeIsValid(subname) ||
+                ( ! ctx.Config().IsModeRelease() && ! ctx.Config().IsModeEntrez() ) )
             {
                 x_AddFQ(q, name, subname);
             }
@@ -1381,10 +1416,10 @@ void CFlatSubSourceQVal::Format(TFlatQuals& q, const CTempString& name,
 
 struct SSortReferenceByName
 {
-   bool operator()(const CDbtag* dbt1,  const CDbtag* dbt2) 
-   {
-      return (dbt1->Compare(*dbt2)<0);
-   }
+    bool operator()(const CDbtag* dbt1,  const CDbtag* dbt2)
+    {
+        return (dbt1->Compare(*dbt2)<0);
+    }
 };
 
 
@@ -1425,7 +1460,7 @@ void CFlatXrefQVal::Format(TFlatQuals& q, const CTempString& name,
             // Special case for EST or GSS: we don't filter the dbtags as toughly
             CDbtag::EIsEstOrGss is_est_or_gss = CDbtag::eIsEstOrGss_No;
             const CMolInfo* mol_info = ctx.GetMolinfo();
-            if( NULL != mol_info && mol_info->CanGetTech() ) {
+            if (mol_info && mol_info->CanGetTech()) {
                 if( mol_info->GetTech() == CMolInfo::eTech_est || mol_info->GetTech() == CMolInfo::eTech_survey ) {
                     is_est_or_gss = CDbtag::eIsEstOrGss_Yes;
                 }
@@ -1442,7 +1477,7 @@ void CFlatXrefQVal::Format(TFlatQuals& q, const CTempString& name,
         id.clear();
         if (tag.IsId()) {
             id = NStr::IntToString(tag.GetId());
-        } else if (tag.IsStr()) {        
+        } else if (tag.IsStr()) {
             id = tag.GetStr();
             if (NStr::EqualNocase(db, "MGI")  ||  NStr::EqualNocase(db, "MGD")) {
                 if (NStr::StartsWith(id, "MGI:", NStr::eNocase)  ||
@@ -1515,7 +1550,7 @@ bool CFlatXrefQVal::x_XrefInGeneXref(const CDbtag& dbtag) const
     while (gxref != end  &&  gxref->first == eFQ_gene_xref) {
         const CFlatXrefQVal* xrefqv =
             dynamic_cast<const CFlatXrefQVal*>(gxref->second.GetPointerOrNull());
-        if (xrefqv != NULL) {
+        if (xrefqv) {
             ITERATE (TXref, dbt, xrefqv->m_Value) {
                 if (dbtag.Match(**dbt)) {
                     return true;
@@ -1528,7 +1563,7 @@ bool CFlatXrefQVal::x_XrefInGeneXref(const CDbtag& dbtag) const
     for ( TQCI it = gxrefs.first; it != gxrefs.second; ++it ) {
         const CFlatXrefQVal* xrefqv =
             dynamic_cast<const CFlatXrefQVal*>(it->second.GetPointerOrNull());
-        if ( xrefqv != 0 ) {
+        if (xrefqv) {
             ITERATE (TXref, dbt, xrefqv->m_Value) {
                 if ( dbtag.Match(**dbt) ) {
                     return true;
@@ -1547,7 +1582,7 @@ static size_t s_CountAccessions(const CUser_field& field)
     if (!field.IsSetData()  ||  !field.GetData().IsFields()) {
         return 0;
     }
-    
+
     //
     //  Each accession consists of yet another block of "Fields" one of which carries
     //  a label named "accession":
@@ -1555,7 +1590,7 @@ static size_t s_CountAccessions(const CUser_field& field)
     ITERATE (CUser_field::TData::TFields, it, field.GetData().GetFields()) {
         const CUser_field& uf = **it;
         if ( uf.CanGetData()  &&  uf.GetData().IsFields() ) {
-        
+
             ITERATE( CUser_field::TData::TFields, it2, uf.GetData().GetFields() ) {
                 const CUser_field& inner = **it2;
                 if ( inner.IsSetLabel() && inner.GetLabel().IsStr() ) {
@@ -1578,7 +1613,7 @@ void CFlatModelEvQVal::Format
 {
     size_t num_mrna = 0, num_prot = 0, num_est = 0, num_long_sra = 0;
     size_t rnaseq_base_coverage = 0, rnaseq_biosamples_introns_full = 0;
-    const string* method = 0;
+    const string* method = nullptr;
 
     ITERATE (CUser_object::TData, it, m_Value->GetData()) {
         const CUser_field& field = **it;
@@ -1628,13 +1663,13 @@ void CFlatModelEvQVal::Format
 
     CNcbiOstrstream text;
     text << "Derived by automated computational analysis";
-    if (method != NULL  &&  !NStr::IsBlank(*method)) {
+    if (method && ! NStr::IsBlank(*method)) {
          text << " using gene prediction method: " << *method;
     }
     text << ".";
 
     if (num_mrna > 0  ||  num_est > 0  ||  num_prot > 0 || num_long_sra > 0 ||
-        rnaseq_base_coverage > 0 ) 
+        rnaseq_base_coverage > 0 )
     {
         text << " Supporting evidence includes similarity to:";
     }
@@ -1755,7 +1790,7 @@ int CFlatGoQVal::GetPubmedId(void) const
     if( ! pmidData.IsInt() ) {
         return 0;
     }
-    
+
     return pmidData.GetInt();
 }
 
@@ -1845,7 +1880,7 @@ void CFlatProductNamesQVal::Format
         return;
     }
     bool note = s_IsNote(flags, ctx);
-    
+
     CProt_ref::TName::const_iterator it = m_Value.begin();
     ++it;  // first is used for /product
     while (it != m_Value.end()  &&  !NStr::IsBlank(*it)) {
