@@ -1,4 +1,4 @@
-/*  $Id: reference_item.cpp 629259 2021-04-13 13:28:40Z ivanov $
+/*  $Id: reference_item.cpp 670941 2023-08-04 12:24:26Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -44,7 +44,6 @@
 #include <objects/general/Dbtag.hpp>
 #include <objects/general/Object_id.hpp>
 #include <objects/medline/Medline_entry.hpp>
-#include <objects/mla/mla_client.hpp>
 #include <objects/pub/Pub.hpp>
 #include <objects/pub/Pub_equiv.hpp>
 #include <objects/pub/Pub_set.hpp>
@@ -139,7 +138,6 @@ void CReferenceItem::FormatAffil(const CAffil& affil, string& result, bool gen_s
                 }
                 result += std.GetAffil();
             }
-            
         } else {
             if (std.IsSetAffil()) {
                 result = std.GetAffil();
@@ -149,7 +147,7 @@ void CReferenceItem::FormatAffil(const CAffil& affil, string& result, bool gen_s
                     result += ", ";
                 }
                 result = std.GetDiv();
-            } 
+            }
         }
         if (std.IsSetStreet()) {
             if (!result.empty()) {
@@ -196,11 +194,11 @@ CReferenceItem::CReferenceItem(const CSeqdesc& desc, CBioseqContext& ctx) :
     m_JustUids(true), m_Elect(false)
 {
     _ASSERT(desc.IsPub());
-    
+
     x_SetObject(desc);
     m_Pubdesc.Reset(&(desc.GetPub()));
 
-    if (ctx.GetMapper() != NULL) {
+    if (ctx.GetMapper()) {
         m_Loc.Reset(ctx.GetMapper()->Map(ctx.GetLocation()));
     } else {
         m_Loc.Reset(&ctx.GetLocation());
@@ -223,10 +221,10 @@ CReferenceItem::CReferenceItem
     x_SetObject(feat);
 
     m_Pubdesc.Reset(&(feat.GetData().GetPub()));
-    
-    if (loc != NULL) {
+
+    if (loc) {
         m_Loc.Reset(loc);
-    } else if ( ctx.GetMapper() != 0 ) {
+    } else if (ctx.GetMapper()) {
         m_Loc.Reset(ctx.GetMapper()->Map(feat.GetLocation()));
     } else {
         m_Loc.Reset(&(feat.GetLocation()));
@@ -275,7 +273,7 @@ void CReferenceItem::SetLoc(const CConstRef<CSeq_loc>& loc)
 
 void CReferenceItem::SetRemark( const CPubdesc::TFig* new_fig,
     const CPubdesc::TMaploc *new_maploc,
-    const CPubdesc::TPoly_a *new_poly_a ) 
+    const CPubdesc::TPoly_a *new_poly_a )
 {
     _ASSERT( m_Pubdesc.NotEmpty() && GetContext() );
 
@@ -325,7 +323,7 @@ static bool s_ShouldRemoveRef
     if( curr_ref.GetPMID() != ZERO_ENTREZ_ID && prev_ref.GetPMID() != ZERO_ENTREZ_ID) {
         return ( curr_ref.GetPMID() == prev_ref.GetPMID() );
     }
-        
+
     // same MUID ( and overlap )
     if( curr_ref.GetMUID() != ZERO_ENTREZ_ID && prev_ref.GetMUID() != ZERO_ENTREZ_ID) {
         return ( curr_ref.GetMUID() == prev_ref.GetMUID() );
@@ -340,10 +338,17 @@ static bool s_ShouldRemoveRef
         CReferenceItem::FormatAuthors(prev_ref.GetAuthors(), auth2);
     }
     const bool authorsSame = NStr::EqualNocase(auth1, auth2);
+    const bool locationsEqual = curr_ref.GetLoc().Equals( prev_ref.GetLoc() );
+
+    // both submission citations
+    if ( curr_ref.GetCategory() == CReferenceItem::eSubmission && prev_ref.GetCategory() == CReferenceItem::eSubmission ) {
+        if ( authorsSame && locationsEqual && curr_ref.GetRemark() != prev_ref.GetRemark() ) {
+            return false;
+        }
+    }
 
     const string& curr_unique_str = curr_ref.GetUniqueStr();
     const string& prev_unique_str = prev_ref.GetUniqueStr();
-    const bool locationsEqual = curr_ref.GetLoc().Equals( prev_ref.GetLoc() );
     if( NStr::EqualNocase( curr_unique_str, prev_unique_str ) &&
         locationsEqual &&
         authorsSame ) {
@@ -366,7 +371,7 @@ static void s_CombineRefs
         CConstRef<CSeq_id> prev_id( prev_loc->GetId() );
         CConstRef<CSeq_id> curr_id( curr_loc->GetId() );
 
-        const bool is_circular = 
+        const bool is_circular =
             ( ctx.GetHandle().CanGetInst_Topology() && ctx.GetHandle().GetInst_Topology() == CSeq_inst_Base::eTopology_circular );
 
         CRef<CSeq_loc> new_loc = Seq_loc_Add( *prev_loc, *curr_loc,
@@ -386,13 +391,13 @@ static void s_CombineRefs
     const bool same_muid = ( curr_ref.GetMUID() != ZERO_ENTREZ_ID && (prev_ref.GetMUID() == curr_ref.GetMUID()) );
     const bool same_pmid = ( curr_ref.GetPMID() != ZERO_ENTREZ_ID && (prev_ref.GetPMID() == curr_ref.GetPMID()) );
     if( (same_muid || same_pmid) &&
-        ( prev_ref.GetRemark() != curr_ref.GetRemark() )  ) 
+        ( prev_ref.GetRemark() != curr_ref.GetRemark() ) )
     {
         const CPubdesc& prev_pubdesc = prev_ref.GetPubdesc();
         const CPubdesc& curr_pubdesc = curr_ref.GetPubdesc();
 
-        const CPubdesc::TFig* new_fig = ( (! prev_pubdesc.IsSetFig() && curr_pubdesc.IsSetFig()) ? &curr_pubdesc.GetFig() : NULL );
-        const CPubdesc::TMaploc *new_maploc = ( (! prev_pubdesc.IsSetMaploc() && curr_pubdesc.IsSetMaploc()) ? &curr_pubdesc.GetMaploc() : NULL );
+        const CPubdesc::TFig* new_fig = ((! prev_pubdesc.IsSetFig() && curr_pubdesc.IsSetFig()) ? &curr_pubdesc.GetFig() : nullptr);
+        const CPubdesc::TMaploc* new_maploc = ((! prev_pubdesc.IsSetMaploc() && curr_pubdesc.IsSetMaploc()) ? &curr_pubdesc.GetMaploc() : nullptr );
         // the "false" is arbitrary and won't get ever used
         const CPubdesc::TPoly_a new_poly_a =
             ( curr_pubdesc.IsSetPoly_a() ? curr_pubdesc.GetPoly_a() : false );
@@ -400,7 +405,7 @@ static void s_CombineRefs
         prev_ref.SetRemark(
             new_fig,
             new_maploc,
-            ( (! prev_pubdesc.IsSetPoly_a() && curr_pubdesc.IsSetPoly_a()) ? &new_poly_a : NULL )
+            ((! prev_pubdesc.IsSetPoly_a() && curr_pubdesc.IsSetPoly_a()) ? &new_poly_a : nullptr)
         );
     }
 }
@@ -410,7 +415,7 @@ static void s_MergeDuplicates
  CBioseqContext& ctx)
 {
     /**
-    static const CSeq_loc::TOpFlags kMergeFlags = 
+    static const CSeq_loc::TOpFlags kMergeFlags =
         CSeq_loc::fSort | CSeq_loc::fStrand_Ignore | CSeq_loc::fMerge_All;
         **/
 
@@ -424,7 +429,7 @@ static void s_MergeDuplicates
     }
 
     // see if merging is allowed
-    if( ! ctx.CanGetTLSeqEntryCtx() || 
+    if( ! ctx.CanGetTLSeqEntryCtx() ||
         ! ctx.GetTLSeqEntryCtx().GetCanSourcePubsBeFused() )
     {
       return;
@@ -469,7 +474,7 @@ static void s_MergeDuplicates
         if( curr != refs.begin() ) {
             CReferenceItem& prev_ref = **(curr-1);
 
-            if( prev_ref.GetReftype() == CPubdesc::eReftype_seq && 
+            if( prev_ref.GetReftype() == CPubdesc::eReftype_seq &&
                     curr_ref.GetReftype() != CPubdesc::eReftype_seq ) {
                 combine_allowed = false;
             }
@@ -494,7 +499,7 @@ static void s_MergeDuplicates
 void CReferenceItem::Rearrange(TReferences& refs, CBioseqContext& ctx)
 {
     {{
-        stable_sort(refs.begin(), refs.end(), 
+        stable_sort(refs.begin(), refs.end(),
             LessThan(LessThan::eSerialFirst_No, ctx.IsRefSeq()));
     }}
 
@@ -511,10 +516,10 @@ void CReferenceItem::Rearrange(TReferences& refs, CBioseqContext& ctx)
 
     {{
         // re-sort, take serial number into consideration.
-        stable_sort(refs.begin(), refs.end(), 
+        stable_sort(refs.begin(), refs.end(),
             LessThan(LessThan::eSerialFirst_Yes, ctx.IsRefSeq()));
     }}
-    
+
     // assign final serial numbers
     size_t size = refs.size();
     int current_serial = 1;
@@ -560,7 +565,7 @@ static bool s_IsOnlySerial(const CPub& pub)
 
     const CCit_gen& gen = pub.GetGen();
 
-    if ( !gen.IsSetCit() ) 
+    if ( !gen.IsSetCit() )
     {
         if (!gen.IsSetJournal()  &&  !gen.IsSetDate()  &&
             gen.IsSetSerial_number()  &&  gen.GetSerial_number() > 0) {
@@ -571,7 +576,7 @@ static bool s_IsOnlySerial(const CPub& pub)
     return false;
 }
 
-    
+
 void CReferenceItem::x_CreateUniqueStr(void) const
 {
     if (!NStr::IsBlank(m_UniqueStr)) {  // already created
@@ -650,7 +655,7 @@ void CReferenceItem::x_GatherInfo(CBioseqContext& ctx)
 
     const CPubdesc::TPub& pub = m_Pubdesc->GetPub();
 
-    /*if (ctx.GetSubmitBlock() != NULL) {
+    /*if (ctx.GetSubmitBlock()) {
         m_Title = "Direct Submission";
         m_Sub.Reset(&ctx.GetSubmitBlock()->GetCit());
         m_PubType = ePub_sub;
@@ -661,73 +666,6 @@ void CReferenceItem::x_GatherInfo(CBioseqContext& ctx)
         x_Init(**it, ctx);
     }
 
-    // if just pmid or just muid, we look it up, assuming the user has
-    // somehow given permission for remote lookups 
-    const static string kGbLoader = "GBLOADER";
-    if( IsJustUids() && 
-        ctx.GetScope().GetObjectManager().FindDataLoader(kGbLoader) ) 
-    {
-        // TODO: To avoid repeated connections, in the future we should have 
-        // one CMLAClient shared by the whole program
-        CMLAClient mlaClient;
-
-        vector< CRef<CPub> > new_pubs;
-
-        ITERATE( CPub_equiv::Tdata, it, pub.Get() ) {
-            const CPub & pub = **it;
-            CRef<CPub> new_pub;
-
-            try {
-                switch(pub.Which()) {
-                case CPub::e_Pmid:
-                    {
-                        const TEntrezId pmid = pub.GetPmid().Get();
-
-                        CPubMedId req(pmid);
-                        CMLAClient::TReply reply;
-                        new_pub = mlaClient.AskGetpubpmid(req, &reply);
-                    }
-                    break;
-                case CPub::e_Muid:
-                    {
-                        pub.GetMuid();
-                        // RW-1040: removed mlaClient.AskUidtopmid and AskGetpubpmid
-                    }
-                    break;
-                default:
-                    // ignore if type unknown
-                    break;
-                }
-            } catch(...) {
-                // don't worry if we can't look it up
-            }
-
-            if( new_pub ) {
-                // authors come back in a weird format that we need
-                // to convert to ISO
-                ChangeMedlineAuthorsToISO(new_pub);
-
-                new_pubs.push_back(new_pub);
-            }
-        }
-
-        if( ! new_pubs.empty() ) {
-            ITERATE( vector< CRef<CPub> >, new_pub_iter, new_pubs ) {
-                x_Init( **new_pub_iter, ctx );
-            }
-
-            // we have to add the new_pubs to m_Pubdesc->GetPub() but m_Pubdesc
-            // is const.  The solution is to copy it, modify the copy, and 
-            // set the copy to have CConstRef
-            CRef<CPubdesc> new_pubdesc( new CPubdesc );
-            new_pubdesc->Assign(*m_Pubdesc);
-            CPub_equiv::Tdata & new_pub_list = new_pubdesc->SetPub().Set();
-            copy( new_pubs.begin(), new_pubs.end(),
-                back_inserter(new_pub_list) );
-            m_Pubdesc = new_pubdesc;
-        }
-    }
-
     // gather Genbank specific fields (formats: Genbank, GBSeq, DDBJ)
     if ( ctx.IsGenbankFormat() || ctx.Config().IsFormatGBSeq() || ctx.Config().IsFormatINSDSeq() ) {
         x_GatherRemark(ctx);
@@ -735,7 +673,7 @@ void CReferenceItem::x_GatherInfo(CBioseqContext& ctx)
 
     x_CleanData();
 }
- 
+
 
 void CReferenceItem::x_Init(const CPub& pub, CBioseqContext& ctx)
 {
@@ -851,7 +789,7 @@ void CReferenceItem::x_Init(const CCit_gen& gen, CBioseqContext& ctx)
             gen.IsSetSerial_number()  &&  gen.GetSerial_number() == 0) {
             x_SetSkip();
             return;
-        } 
+        }
     } else if ((!gen.IsSetJournal()  ||  !m_Date)  &&  m_Serial == 0) {
         x_SetSkip();
         return;
@@ -875,7 +813,7 @@ void CReferenceItem::x_Init(const CCit_gen& gen, CBioseqContext& ctx)
     if (!NStr::IsBlank(m_Title)  &&  NStr::StartsWith(m_Title, "(er)")) {
         m_Elect = true;
     }
-    
+
     // Authors
     if (gen.CanGetAuthors()) {
         x_AddAuthors(gen.GetAuthors());
@@ -885,7 +823,7 @@ void CReferenceItem::x_Init(const CCit_gen& gen, CBioseqContext& ctx)
     if (gen.CanGetMuid()  &&  m_MUID == ZERO_ENTREZ_ID) {
         m_MUID = gen.GetMuid();
     }
-    
+
     // PMID
     if (gen.CanGetPmid()  &&  m_PMID == ZERO_ENTREZ_ID) {
         m_PMID = gen.GetPmid().Get();
@@ -909,7 +847,7 @@ void CReferenceItem::x_Init(const CCit_sub& sub, CBioseqContext& ctx)
     // Date
     if (sub.CanGetDate()) {
         m_Date.Reset(&sub.GetDate());
-    } 
+    }
     if (sub.CanGetImp()) {
         x_AddImprint(sub.GetImp(), ctx);
     }
@@ -940,7 +878,7 @@ static bool PiiOk(const string& str)
 {
     if (NStr::IsBlank(str)) return false;
 
-    int max = str.length();
+    int max = (int) str.length();
     int i = 0;
 
     char ch = str[i];
@@ -1010,7 +948,7 @@ void CReferenceItem::x_Init(const CCit_art& art, CBioseqContext& ctx)
                 }
                 if (imp.IsSetPubstatus()) {
                     CImprint::TPubstatus pubstatus = imp.GetPubstatus();
-                    if (pubstatus == 3) {
+                    if (pubstatus == ePubStatus_epublish) {
                         is_epublish = true;
                     }
                 }
@@ -1122,7 +1060,7 @@ void CReferenceItem::x_InitProc(const CCit_book& proc, CBioseqContext& ctx)
     m_Book.Reset();
     if (!m_Authors  &&  proc.IsSetAuthors()) {
         x_AddAuthors(proc.GetAuthors());
-    } 
+    }
     if (proc.IsSetTitle()) {
         m_Title = proc.GetTitle().GetTitle();
     }
@@ -1139,11 +1077,10 @@ void CReferenceItem::x_Init
     m_Book.Reset(&book);
     if (!m_Authors  &&  book.IsSetAuthors()) {
         x_AddAuthors(book.GetAuthors());
-    } 
+    }
     if (book.CanGetImp()) {
         x_AddImprint(book.GetImp(), ctx);
     }
-
 }
 
 
@@ -1177,7 +1114,7 @@ void CReferenceItem::x_Init(const CCit_let& man, CBioseqContext& ctx)
     }
 
     m_PubType = ePub_thesis;
-  
+
     if (man.IsSetCit()) {
         const CCit_book& book = man.GetCit();
         x_Init(book, ctx);
@@ -1195,7 +1132,7 @@ void CReferenceItem::x_AddAuthors(const CAuth_list& auth_list)
     if (!auth_list.CanGetNames()) {
         return;
     }
-    
+
     // also populate the consortium (if available).
     // note: there may be more than one, and they all want to be listed.
     if (!NStr::IsBlank(m_Consortium)) {
@@ -1203,7 +1140,7 @@ void CReferenceItem::x_AddAuthors(const CAuth_list& auth_list)
     }
 
     const CAuth_list::TNames& names = auth_list.GetNames();
-    
+
     if (names.IsStd()) {
         ITERATE (CAuth_list::TNames::TStd, it, names.GetStd()) {
             const CAuthor& auth = **it;
@@ -1225,7 +1162,7 @@ void CReferenceItem::x_AddImprint(const CImprint& imp, CBioseqContext& ctx)
     // electronic journal
     if (imp.IsSetPubstatus()) {
         CImprint::TPubstatus pubstatus = imp.GetPubstatus();
-        m_Elect = (pubstatus == 3  || pubstatus == 10);
+        m_Elect = (pubstatus == ePubStatus_epublish || pubstatus == ePubStatus_aheadofprint);
     }
 
     // date
@@ -1237,7 +1174,7 @@ void CReferenceItem::x_AddImprint(const CImprint& imp, CBioseqContext& ctx)
     if (imp.IsSetPrepub()) {
         CImprint::TPrepub prepub = imp.GetPrepub();
         //m_Prepub = imp.GetPrepub();
-        m_Category = 
+        m_Category =
             prepub != CImprint::ePrepub_in_press ? eUnpublished : ePublished;
     } else {
         m_Category = ePublished;
@@ -1246,7 +1183,7 @@ void CReferenceItem::x_AddImprint(const CImprint& imp, CBioseqContext& ctx)
 
 
 void CReferenceItem::GetAuthNames(const CAuth_list& alp, TStrList& authors)
-{   
+{
     authors.clear();
 
     const CAuth_list::TNames& names = alp.GetNames();
@@ -1258,21 +1195,21 @@ void CReferenceItem::GetAuthNames(const CAuth_list& alp, TStrList& authors)
             }
             const CPerson_id& pid = (*it)->GetName();
             if (pid.IsName()  ||  pid.IsMl()  ||  pid.IsStr()) {
-                authors.push_back(kEmptyStr);               
+                authors.push_back(kEmptyStr);
                 string& name = authors.back();
                 pid.GetLabel(&name, CPerson_id::eGenbank);
             }
         }
         break;
-        
+
     case CAuth_list::TNames::e_Ml:
         authors.insert(authors.end(), names.GetMl().begin(), names.GetMl().end());
         break;
-        
+
     case CAuth_list::TNames::e_Str:
         authors.insert(authors.end(), names.GetStr().begin(), names.GetStr().end());
         break;
-        
+
     default:
         break;
     }
@@ -1357,7 +1294,7 @@ public:
 };
 
 static
-bool s_ContainsNoLowercase( const string &str ) 
+bool s_ContainsNoLowercase( const string &str )
 {
     return ( find_if( str.begin(), str.end(), CIsLowercase() ) == str.end() );
 }
@@ -1397,120 +1334,16 @@ void CReferenceItem::x_CapitalizeTitleIfNecessary()
     }
 }
 
-void CReferenceItem::ChangeMedlineAuthorsToISO( CRef<CPub> pub )
+
+void CReferenceItem::ChangeMedlineAuthorsToISO(CRef<CPub> pub)
 {
     // leave early if it doesn't need to be changed
-    if( ! pub || ! pub->IsArticle() || ! pub->IsSetAuthors() || ! pub->GetAuthors().IsSetNames() ||
-        ! pub->GetAuthors().GetNames().IsMl() )
-    {
+    if (!pub || !pub->IsArticle() || !pub->IsSetAuthors() || !pub->GetAuthors().IsSetNames() ||
+        !pub->GetAuthors().GetNames().IsMl()) {
         return;
     }
 
-    // build our new authors list in here
-    CAuth_list::C_Names::TStd new_authors;
-
-    const CAuth_list::C_Names::TMl & ml_names = pub->GetAuthors().GetNames().GetMl();
-    ITERATE( CAuth_list::C_Names::TMl, ml_name_iter, ml_names ) {
-        string author_name = *ml_name_iter;
-
-        // we will fill in these 3 as we go along
-        string lastname;
-        string initials;
-        string suffix;
-
-        // this scope fills in lastname, initials, and suffix
-        {
-            NStr::TruncateSpacesInPlace(author_name);
-            vector<string> tokens;
-            NStr::Split(author_name, " ", tokens, 
-                        NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
-
-            // get suffix if it exists, and remove it from the list
-            if( tokens.size() >= 3 && 
-                ! x_StringIsJustCapitalLetters(tokens.back()) && 
-                x_StringIsJustCapitalLetters(tokens[tokens.size() - 2]) ) 
-            {
-                suffix = tokens.back();
-                tokens.pop_back();
-            }
-
-            // get initials if they exist, and remove them from the list
-            if( tokens.size() >= 2 &&
-                x_StringIsJustCapitalLetters(tokens.back()) )
-            {
-                initials = tokens.back();
-                tokens.pop_back();
-            }
-
-            // remaining pieces belong to the last name
-            lastname = NStr::Join(tokens, " ");
-        }
-
-        // put period in initials. e.g. "MJ" -> "M.J."
-        {
-            string new_initials;
-            ITERATE( string, ch_iter, initials ) {
-                new_initials += *ch_iter;
-                new_initials += '.';
-            }
-            // swap is faster than assignment
-            initials.swap(new_initials);
-        }
-
-        // a couple of static transformations for the suffix
-        typedef SStaticPair<const char*, const char*> TSufElem;
-        static const TSufElem sc_suf_map[] = {
-            { "1d",  "I" },
-            { "1st", "I" },
-            { "2d",  "II" },
-            { "2nd", "II" },
-            { "3d",  "III" },
-            { "3rd", "III" },
-            { "4th", "IV" },
-            { "5th", "V" },
-            { "6th", "VI" },
-            { "Jr",  "Jr." },
-            { "Sr",  "Sr."}
-        };
-        typedef CStaticArrayMap<const char *, const char *, PCase_CStr> TSufMap;
-        DEFINE_STATIC_ARRAY_MAP(TSufMap, sc_SufMap, sc_suf_map);
-
-        TSufMap::const_iterator suf_find_iter = sc_SufMap.find(suffix.c_str());
-        if( suf_find_iter != sc_SufMap.end() ) {
-            suffix = suf_find_iter->second;
-        }
-
-        CRef<CAuthor> new_author( new CAuthor );
-        CPerson_id_Base::TName & name = new_author->SetName().SetName();
-        name.SetLast( lastname );
-        if( ! initials.empty() ) {
-            name.SetInitials( initials );
-        }
-        if( ! suffix.empty() ) {
-            name.SetSuffix( suffix );
-        }
-
-        new_authors.push_back( new_author );
-    }
-    
-    copy( new_authors.begin(), 
-        new_authors.end(),
-        back_inserter( pub->SetArticle().SetAuthors().SetNames().SetStd() ) );
-}
-
-bool CReferenceItem::x_StringIsJustCapitalLetters( const string & str )
-{
-    if( str.empty() ) {
-        return false;
-    }
-
-    ITERATE(string, ch_iter, str ) {
-        if( ! isupper(*ch_iter) ) {
-            return false;
-        }
-    }
-
-    return true;
+    pub->SetArticle().SetAuthors().ConvertMlToStandard(true);
 }
 
 
@@ -1545,7 +1378,7 @@ void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
 
     if ( m_Pubdesc->IsSetComment()  &&  !m_Pubdesc->GetComment().empty() ) {
         const string& comment = m_Pubdesc->GetComment();
-        
+
         if ( sc_Remarks.find(comment.c_str()) == sc_Remarks.end() ) {
             l.push_back(comment);
         }
@@ -1568,7 +1401,7 @@ void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
         }
     }
     if ( gibbsq > 0 ) {
-        static const string str1 = 
+        static const string str1 =
             "GenBank staff at the National Library of Medicine created this entry [NCBI gibbsq ";
         static const string str2 = "] from the original journal article.";
         l.push_back(str1 + NStr::IntToString(gibbsq) + str2);
@@ -1595,12 +1428,12 @@ void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
             }
         }
     }
-    
+
     if ( m_Pubdesc->CanGetPub() ) {
         ITERATE (CPubdesc::TPub::Tdata, it, m_Pubdesc->GetPub().Get()) {
             if ( (*it)->IsArticle() ) {
                 if ( (*it)->GetArticle().GetFrom().IsJournal() ) {
-                    const CCit_jour& jour = 
+                    const CCit_jour& jour =
                         (*it)->GetArticle().GetFrom().GetJournal();
                     if ( jour.IsSetImp() ) {
                         const CCit_jour::TImp& imp = jour.GetImp();
@@ -1641,10 +1474,10 @@ void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
                         if ( imp.CanGetPubstatus() ) {
                             CImprint::TPubstatus pubstatus = imp.GetPubstatus();
                             switch ( pubstatus ) {
-                            case 3:
+                            case ePubStatus_epublish:
                                 l.push_back( "Publication Status: Online-Only" );
                                 break;
-                            case 10:
+                            case ePubStatus_aheadofprint:
                                 l.push_back( "Publication Status: Available-Online prior to print" );
                                 break;
                             default:
@@ -1809,7 +1642,7 @@ bool LessThan::operator()
     } else if (!ref1->IsSetDate()  &&  ref2->IsSetDate()) {
         return ! m_IsRefSeq;
     }
-    
+
     if (ref1->IsSetDate()  &&  ref2->IsSetDate()) {
         CDate::ECompare status = s_CompareDates(ref1->GetDate(), ref2->GetDate());
         _ASSERT( status != CDate::eCompare_unknown ); // unknown would produce invalid ordering
@@ -1820,7 +1653,7 @@ bool LessThan::operator()
     }
     //}
     // after: dates are the same, or both missing.
-    
+
     // distinguish by uids (swap order for RefSeq)
     if ( ref1->GetPMID() != ZERO_ENTREZ_ID &&  ref2->GetPMID() != ZERO_ENTREZ_ID &&
          !(ref1->GetPMID() == ref2->GetPMID()) ) {
@@ -1886,11 +1719,11 @@ bool LessThan::operator()
     // put pub descriptors before features, sort features by location
     const CSeq_feat* f1 = dynamic_cast<const CSeq_feat*>(ref1->GetObject());
     const CSeq_feat* f2 = dynamic_cast<const CSeq_feat*>(ref2->GetObject());
-    if (f1 == NULL  &&  f2 != NULL) {
+    if (! f1  &&  f2) {
         return true;
-    } else if (f1 != NULL  &&  f2 == NULL) {
+    } else if (f1 && ! f2) {
         return false;
-    } else if (f1 != NULL  &&  f2 != NULL) {
+    } else if (f1 && f2) {
         CSeq_loc::TRange r1 = f1->GetLocation().GetTotalRange();
         CSeq_loc::TRange r2 = f2->GetLocation().GetTotalRange();
         if (r1 < r2) {
