@@ -1,4 +1,4 @@
-/*  $Id: agp_util.cpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: agp_util.cpp 678856 2024-02-12 19:15:03Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -356,11 +356,12 @@ const CAgpRow::TStr CAgpRow::gap_types[CAgpRow::eGapCount] = {
     "centromere",
     "short_arm",
     "heterochromatin",
-    "telomere"
+    "telomere",
+    "unknown"
 };
 
 CSafeStatic<CAgpRow::TMapStrEGap> CAgpRow::gap_type_codes(
-    & CAgpRow::gap_type_codes_creator, NULL );
+    &CAgpRow::gap_type_codes_creator, nullptr);
 
 // static
 CAgpRow::TMapStrEGap * CAgpRow::gap_type_codes_creator(void)
@@ -395,15 +396,15 @@ CAgpRow::~CAgpRow()
 {
 }
 
-TSeqPos CAgpRow::ReadSeqPos(const CTempString seq_pos_str, const string& details,
+TAgpPos CAgpRow::ReadSeqPos(const CTempString seq_pos_str, const string& details,
     int *perror_code, bool log_errors)
 {
     Int8 pos = NStr::StringToInt8( seq_pos_str, NStr::fConvErr_NoThrow );
-    TSeqPos ret_value = 0;
+    TAgpPos ret_value = 0;
     int error_code = 0;
     if(pos<=0) {
         error_code = CAgpErr::E_MustBePositive;
-    } else if (pos > std::numeric_limits<TSeqPos>::max()) {
+    } else if (pos > std::numeric_limits<TAgpPos>::max()) {
         error_code = CAgpErr::E_MustFitSeqPosType;
     }
     if (error_code) {
@@ -415,7 +416,7 @@ TSeqPos CAgpRow::ReadSeqPos(const CTempString seq_pos_str, const string& details
         }
     }
     else {
-        ret_value = static_cast<TSeqPos>(pos);
+        ret_value = static_cast<TAgpPos>(pos);
     }
     return ret_value;
 }
@@ -438,7 +439,7 @@ int CAgpRow::FromString(const string& line)
         NStr::Split(line.substr(0, pcomment), "\t", cols);
     }
     else {
-      int pos=line.size();
+      SIZE_TYPE pos=line.size();
       if(pos == 0) {
           m_AgpErr->Msg(CAgpErr::E_EmptyLine);
           return CAgpErr::E_EmptyLine;
@@ -529,7 +530,7 @@ int CAgpRow::FromString(const string& line)
 
             int code=ParseComponentCols();
             if(code==0) {
-                TSeqPos component_range_len=component_end-component_beg+1;
+                TAgpPos component_range_len=component_end-component_beg+1;
                 if(component_range_len != object_range_len) {
                     m_AgpErr->Msg( CAgpErr::E_ObjRangeNeComp, string(": ") +
                         i2s(object_range_len   ) + " != " +
@@ -704,7 +705,8 @@ int CAgpRow::ParseGapCols(bool log_errors)
             gap_type != eGapRepeat &&
             gap_type != eGapFragment &&
             gap_type != eGapScaffold &&
-            gap_type != eGapContamination
+            gap_type != eGapContamination &&
+            gap_type != eGapUnknown
             )
         {
             if(log_errors) m_AgpErr->Msg(CAgpErr::E_InvalidLinkage, " \"yes\" for gap_type "+GetGapType() );
@@ -843,7 +845,7 @@ void CAgpRow::SetErrorHandler(CAgpErr* arg)
     m_AgpErr=arg;
 }
 
-bool CAgpRow::CheckComponentEnd( const string& comp_id, TSeqPos comp_end, TSeqPos comp_len,
+bool CAgpRow::CheckComponentEnd( const string& comp_id, TAgpPos comp_end, TAgpLen comp_len,
   CAgpErr& agp_err)
 {
     if( comp_end > comp_len) {
@@ -1628,7 +1630,7 @@ void CAgpErrEx::LineDone(const string& s, int line_num, bool invalid_line)
 
     m_line_num_pp = m_line_num_prev; m_line_num_prev = line_num;
     m_line_pp     = m_line_prev    ; m_line_prev     = s;
-    m_filenum_pp  = m_filenum_prev ; m_filenum_prev  = m_InputFiles.size()-1;
+    m_filenum_pp  = m_filenum_prev ; m_filenum_prev  = GetFileNum()-1;
 
     if(invalid_line) {
         m_lines_skipped++;
@@ -1640,7 +1642,7 @@ void CAgpErrEx::LineDone(const string& s, int line_num, bool invalid_line)
 void CAgpErrEx::StartFile(const string& s)
 {
     // might need to set it here in case some file is empty and LineDone() is never called
-    m_filenum_pp=m_filenum_prev; m_filenum_prev=m_InputFiles.size()-1;
+    m_filenum_pp = m_filenum_prev; m_filenum_prev = GetFileNum()-1;
     m_filename=s;
     m_InputFiles.push_back(s);
 }
