@@ -1,4 +1,4 @@
-/*  $Id: aln_reader.cpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: aln_reader.cpp 671755 2023-08-23 17:41:23Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -123,7 +123,7 @@ class CDefaultIdErrorReporter
 public:
     CDefaultIdErrorReporter(CAlnErrorReporter* pErrorReporter)
         : m_pErrorReporter(pErrorReporter)
-        { _ASSERT(m_pErrorReporter != nullptr); }
+        { _ASSERT(m_pErrorReporter); }
 
 
     void operator()(EDiagSev severity,
@@ -814,7 +814,7 @@ void CAlnReader::x_AddMods(const SLineInfo& defline_info,
     }
 
     auto pFirstID = bioseq.GetFirstId();
-    _ASSERT(pFirstID != nullptr);
+    _ASSERT(pFirstID);
     const auto idString = pFirstID->AsFastaString();
 
     CDefaultModErrorReporter
@@ -856,7 +856,7 @@ void CAlnReader::x_AddTitle(const string& title, CBioseq& bioseq)
     }
     auto pDesc = Ref(new CSeqdesc());
     pDesc->SetTitle() = title;
-    bioseq.SetDescr().Set().push_back(move(pDesc));
+    bioseq.SetDescr().Set().push_back(std::move(pDesc));
 }
 
 
