@@ -1,4 +1,4 @@
-/*  $Id: fasta.cpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: fasta.cpp 680777 2024-03-27 13:52:05Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -218,6 +218,44 @@ inline static bool s_ASCII_IsUnAmbigNuc(unsigned char c)
     }
 }
 
+void
+CFastaReader::AddStringFlags(
+    const list<string>& stringFlags,
+    TFlags& baseFlags)
+{
+    static const map<string, CFastaReader::TReaderFlags> flagsMap = {
+        { "AssumeNuc",    CFastaReader::fAssumeNuc},
+        { "AssumeProt",   CFastaReader::fAssumeProt},
+        { "ForceType",    CFastaReader::fForceType},
+        { "NoParseID",    CFastaReader::fNoParseID},
+        { "ParseGaps",    CFastaReader::fParseGaps},
+        { "OneSeq",       CFastaReader::fOneSeq},
+        { "NoSeqData",    CFastaReader::fNoSeqData},
+        { "RequireID",    CFastaReader::fRequireID},
+        { "DLOptional",   CFastaReader::fDLOptional},
+        { "ParseRawID",   CFastaReader::fParseRawID},
+        { "SkipCheck",    CFastaReader::fSkipCheck},
+        { "NoSplit",      CFastaReader::fNoSplit},
+        { "Validate",     CFastaReader::fValidate},
+        { "UniqueIDs",    CFastaReader::fUniqueIDs},
+        { "StrictGuess",  CFastaReader::fStrictGuess},
+        { "LaxGuess",     CFastaReader::fLaxGuess},
+        { "AddMods",      CFastaReader::fAddMods},
+        { "LetterGaps",   CFastaReader::fLetterGaps},
+        { "NoUserObjs",   CFastaReader::fNoUserObjs},
+        { "LeaveAsText",  CFastaReader::fLeaveAsText},
+        { "QuickIDCheck", CFastaReader::fQuickIDCheck},
+        { "UseIupacaa",   CFastaReader::fUseIupacaa},
+        { "HyphensIgnoreAndWarn", CFastaReader::fHyphensIgnoreAndWarn},
+        { "DisableNoResidues",    CFastaReader::fDisableNoResidues},
+        { "DisableParseRange",    CFastaReader::fDisableParseRange},
+        { "IgnoreMods",           CFastaReader::fIgnoreMods}
+    };
+
+    return CReaderBase::xAddStringFlagsWithMap(stringFlags, flagsMap, baseFlags);
+};
+
+
 CFastaReader::CFastaReader(ILineReader& reader, TFlags flags, FIdCheck f_idcheck)
     : m_LineReader(&reader), m_MaskVec(0),
       m_gapNmin(0), m_gap_Unknown_length(0),
@@ -369,7 +407,7 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(ILineErrorListener * pMessageListener)
                     line = strmodified;
                 }
                 ParseDataLine(line, pMessageListener);
-            } catch(CBadResiduesException & e) {
+            } catch(const CBadResiduesException & e) {
                 // we have to catch this exception so we can build up
                 // information on all lines, not just the first line
                 // with a bad residue
@@ -415,7 +453,7 @@ CRef<CSeq_entry> CFastaReader::ReadSet(int max_seqs, ILineErrorListener * pMessa
             }
             if (entry2.NotEmpty())
               entry->SetSet().SetSeq_set().push_back(entry2);
-        } catch (CObjReaderParseException& e) {
+        } catch (const CObjReaderParseException& e) {
             if (e.GetErrCode() == CObjReaderParseException::eEOF) {
                 break;
             } else {
@@ -457,7 +495,7 @@ void CFastaReader::SetMaxIDLength(Uint4 max_len)
 // CRef<CSeq_interval> to access range information - RW-26
 void CFastaReader::ParseDefLine(const TStr& defLine,
                                 const SDefLineParseInfo& info,
-                                const TIgnoredProblems& ignoredErrors,
+                                const TIgnoredProblems& /*ignoredErrors*/,
                                 list<CRef<CSeq_id>>& ids,
                                 bool& hasRange,
                                 TSeqPos& rangeStart,
@@ -472,11 +510,11 @@ void CFastaReader::ParseDefLine(const TStr& defLine,
         data,
         pMessageListener);
 
-    ids = move(data.ids);
+    ids        = std::move(data.ids);
     hasRange   = data.has_range;
     rangeStart = data.range_start;
     rangeEnd   = data.range_end;
-    seqTitles  = move(data.titles);
+    seqTitles  = std::move(data.titles);
 }
 
 
@@ -533,7 +571,7 @@ void CFastaReader::ParseDefLine(const TStr& s, ILineErrorListener * pMessageList
     CFastaDeflineReader::SDeflineData data;
     CFastaDeflineReader::ParseDefline(s, parseInfo, data, pMessageListener, m_fIdCheck);
 
-    m_CurrentSeqTitles = move(data.titles);
+    m_CurrentSeqTitles = std::move(data.titles);
 
     if (data.ids.empty()) {
         if (TestFlag(fRequireID)) {
@@ -580,7 +618,7 @@ void CFastaReader::ParseDefLine(const TStr& s, ILineErrorListener * pMessageList
 
 void CFastaReader::PostProcessIDs(
     const CBioseq::TId& defline_ids,
-    const string& defline,
+    const string& /*defline*/,
     const bool has_range,
     const TSeqPos range_start,
     const TSeqPos range_end)
@@ -778,7 +816,7 @@ void CFastaReader::ParseDataLine(
     m_SeqData.resize(m_CurrentPos + s_len);
 
     // these will stay as -1 and empty unless there's an error
-    int bad_pos_line_num = -1;
+    Int8 bad_pos_line_num = -1;
     vector<TSeqPos> bad_pos_vec;
 
     const bool bHyphensIgnoreAndWarn = TestFlag(fHyphensIgnoreAndWarn);
@@ -959,7 +997,7 @@ void CFastaReader::ParseDataLine(
 
     if( ! bad_pos_vec.empty() ) {
         if (TestFlag(fValidate)) {
-            NCBI_THROW2(CBadResiduesException, eBadResidues,
+                        NCBI_THROW2(CBadResiduesException, eBadResidues,
                 "CFastaReader: There are invalid " + x_NucOrProt() + "residue(s) in input sequence",
                 CBadResiduesException::SBadResiduePositions( m_BestID, bad_pos_vec, bad_pos_line_num ) );
         } else {
@@ -978,7 +1016,7 @@ void CFastaReader::ParseDataLine(
 }
 
 void CFastaReader::x_CloseGap(
-    TSeqPos len, bool atStartOfLine, ILineErrorListener * pMessageListener)
+    TSeqPos len, bool atStartOfLine, ILineErrorListener * /*pMessageListener*/)
 {
     _ASSERT(len > 0  &&  TestFlag(fParseGaps));
 
@@ -1011,7 +1049,7 @@ void CFastaReader::x_CloseGap(
             // (do NOT treat a lone 'N' or 'X' as unknown length)
             if (len == 1 && m_CurrentGapChar == '-' ) {
                 TSeqPos l = m_SeqData.length();
-                if ((l == pos  ||  l == pos + (*GetLineReader()).length())  && atStartOfLine) {
+                if ((l == pos) || (l == pos + (*GetLineReader()).length() && atStartOfLine)) {
                     //and it's not the first col of the line
                     len = 0;
                     eKnownSize = SGap::eKnownSize_No;
@@ -1147,7 +1185,7 @@ bool CFastaReader::ParseGapLine(
 
     // string to value maps
     const CEnumeratedTypeValues::TNameToValue & linkage_evidence_to_value_map =
-        CLinkage_evidence::GetTypeInfo_enum_EType()->NameToValue();
+        CLinkage_evidence::ENUM_METHOD_NAME(EType)()->NameToValue();
 
     // remember if there is a gap-type conflict
     bool bConflictingGapTypes = false;
@@ -1452,12 +1490,12 @@ void CFastaReader::AssembleSeq(ILineErrorListener * pMessageListener)
                             CSeq_gap::TLinkage_evidence::value_type pNewLinkEvid(
                                 new CLinkage_evidence );
                             pNewLinkEvid->SetType( *link_evid_it );
-                            vecLinkEvids.push_back(move(pNewLinkEvid));
+                            vecLinkEvids.push_back(std::move(pNewLinkEvid));
                         }
                     }
                 }
             }
-            delta_ext.Set().push_back(move(gap_ds));
+            delta_ext.Set().push_back(std::move(gap_ds));
 
             TSeqPos next_start = (i == n-1) ? m_CurrentPos : m_Gaps[i+1]->m_uPos;
             if (next_start != m_Gaps[i]->m_uPos) {
@@ -1481,7 +1519,7 @@ static void s_AddBiomol(CMolInfo::EBiomol biomol, CBioseq& bioseq)
 {
     auto pDesc = Ref(new CSeqdesc());
     pDesc->SetMolinfo().SetBiomol(biomol);
-    bioseq.SetDescr().Set().emplace_back(move(pDesc));
+    bioseq.SetDescr().Set().emplace_back(std::move(pDesc));
 }
 
 static bool sRefineNaMol(const char* beginSeqData, const char* endSeqData, CBioseq& bioseq)
@@ -1507,7 +1545,7 @@ static bool sRefineNaMol(const char* beginSeqData, const char* endSeqData, CBios
 }
 
 
-void CFastaReader::AssignMolType(ILineErrorListener * pMessageListener)
+void CFastaReader::AssignMolType(ILineErrorListener * /*pMessageListener*/)
 {
     CSeq_inst::EMol             default_mol;
     CFormatGuess::ESTStrictness strictness;
@@ -1688,7 +1726,7 @@ CRef<CSeq_entry> CFastaReader::x_ReadSeqsToAlign(TIds& ids,
             _ASSERT(lengths.size() == size_t(m_Row) + 1);
             // redundant if there was a trailing gap, but that should be okay
             m_Starts[lengths[m_Row]][m_Row] = CFastaAlignmentBuilder::kNoPos;
-        } catch (CObjReaderParseException&) {
+        } catch (const CObjReaderParseException&) {
             if (GetLineReader().AtEOF()) {
                 break;
             } else {
@@ -1925,7 +1963,7 @@ void ScanFastaFile(IFastaEntryScan* scanner,
             if (se->IsSeq()) {
                 scanner->EntryFound(se, pos);
             }
-        } catch (CObjReaderParseException&) {
+        } catch (const CObjReaderParseException&) {
             if ( !lr->AtEOF() ) {
                 throw;
             }
@@ -1956,6 +1994,27 @@ void CFastaReader::SetExcludedMods(const vector<string>& excluded_mods)
 }
 
 
+void CFastaReader::SetIgnoredMods(const list<string>& ignored_mods)
+{
+    m_ModHandler.SetIgnoredMods(ignored_mods);
+}
+
+void CFastaReader::SetPostponedMods(const list<string>& postponed_mods)
+{
+    m_PostponedMods.clear();
+    transform(postponed_mods.begin(), postponed_mods.end(),
+            inserter(m_PostponedMods, m_PostponedMods.end()),
+            [](const string& mod_name) { return CModHandler::GetCanonicalName(mod_name); });
+}
+
+
+const CFastaReader::TPostponedModMap&
+CFastaReader::GetPostponedModMap() const
+{
+    return m_PostponedModMap;
+}
+
+
 void CFastaReader::x_ApplyMods(
      const string& title,
      TSeqPos line_number,
@@ -1964,31 +2023,7 @@ void CFastaReader::x_ApplyMods(
 {
     string processed_title = title;
     if (TestFlag(fAddMods)) {
-        string remainder;
-        CModHandler::TModList mods;
-        CTitleParser::Apply(processed_title, mods, remainder);
-
-        const auto pFirstID = bioseq.GetFirstId();
-        _ASSERT(pFirstID != nullptr);
-        const auto idString = pFirstID->AsFastaString();
-
-        CDefaultModErrorReporter
-            errorReporter(idString, line_number, pMessageListener);
-
-        CModHandler::TModList rejected_mods;
-        m_ModHandler.Clear();
-        m_ModHandler.AddMods(mods, CModHandler::eReplace, rejected_mods, errorReporter);
-        s_AppendMods(rejected_mods, remainder);
-
-        CModHandler::TModList skipped_mods;
-        const bool logInfo =
-            pMessageListener ?
-            pMessageListener->SevEnabled(eDiag_Info) :
-            false;
-        CModAdder::Apply(m_ModHandler, bioseq, skipped_mods, logInfo, errorReporter);
-        s_AppendMods(skipped_mods, remainder);
-
-        processed_title = remainder;
+        x_AddMods(line_number, bioseq, processed_title, pMessageListener);
     }
     else
     if (!TestFlag(fIgnoreMods) &&
@@ -2004,7 +2039,77 @@ void CFastaReader::x_ApplyMods(
     if (!processed_title.empty()) {
         auto pDesc = Ref(new CSeqdesc());
         pDesc->SetTitle() = processed_title;
-        bioseq.SetDescr().Set().push_back(move(pDesc));
+        bioseq.SetDescr().Set().push_back(std::move(pDesc));
+    }
+}
+
+
+void CFastaReader::x_AddMods(
+        TSeqPos line_number,
+        CBioseq& bioseq,
+        string& processed_title,
+        ILineErrorListener* pMessageListener)
+{
+    string remainder;
+    CModHandler::TModList mods;
+    CTitleParser::Apply(processed_title, mods, remainder);
+    if (mods.empty()) {
+        return;
+    }
+
+    const auto* pFirstID = bioseq.GetFirstId();
+    _ASSERT(pFirstID);
+    const auto idString = pFirstID->AsFastaString();
+
+    x_CheckForPostponedMods(idString, line_number, mods);
+    if (mods.empty()) {
+        return;
+    }
+
+    CDefaultModErrorReporter
+        errorReporter(idString, line_number, pMessageListener);
+
+    CModHandler::TModList rejected_mods;
+    m_ModHandler.Clear();
+    m_ModHandler.AddMods(mods, CModHandler::eReplace, rejected_mods, errorReporter);
+    s_AppendMods(rejected_mods, remainder);
+
+    CModHandler::TModList skipped_mods;
+    const bool logInfo =
+        pMessageListener ?
+        pMessageListener->SevEnabled(eDiag_Info) :
+        false;
+    CModAdder::Apply(m_ModHandler, bioseq, skipped_mods, logInfo, errorReporter);
+    s_AppendMods(skipped_mods, remainder);
+
+    processed_title = remainder;
+}
+
+
+void CFastaReader::x_CheckForPostponedMods(const string& idString,
+        TSeqPos line_number,
+        CModHandler::TModList& mods)
+{
+    if (mods.empty() || m_PostponedMods.empty()) {
+        return;
+    }
+
+    auto it = mods.begin();
+    while(it != mods.end()) {
+        if (m_PostponedMods.find(CModHandler::GetCanonicalName(it->GetName()))
+            != m_PostponedMods.end()) {
+
+            if (auto mit = m_PostponedModMap.find(idString);
+                mit != m_PostponedModMap.end()) {
+                mit->second.second.push_back(*it);
+            }
+            else {
+                m_PostponedModMap[idString] = {line_number, {*it}};
+            }
+            it = mods.erase(it);
+        } else {
+            ++it;
+        }
     }
 }
 
@@ -2045,7 +2150,7 @@ CFastaReader::SGap::SGap(
     TSeqPos uPos,
     TSignedSeqPos uLen,
     EKnownSize eKnownSize,
-    TSeqPos uLineNumber,
+    Uint8 uLineNumber,
     TNullableGapType pGapType,
     const set<CLinkage_evidence::EType> & setOfLinkageEvidence ) :
         m_uPos(uPos),
@@ -2099,7 +2204,7 @@ void CFastaReader::PostWarning(
     string sSeqId = ( m_BestID ? m_BestID->AsFastaString() : kEmptyStr);
     AutoPtr<CObjReaderLineException> pLineExpt(
         CObjReaderLineException::Create(
-        (_eSeverity), _uLineNum,
+        (_eSeverity), static_cast<unsigned>(_uLineNum),
         _MessageStrmOps,
         (_eProblem),
         sSeqId, (_sFeature),
@@ -2121,4 +2226,3 @@ void CFastaReader::IgnoreProblem(ILineError::EProblem problem)
 
 END_SCOPE(objects)
 END_NCBI_SCOPE
-
