@@ -1,4 +1,4 @@
- /*  $Id: gvf_reader.cpp 632531 2021-06-02 17:25:37Z ivanov $
+ /*  $Id: gvf_reader.cpp 663694 2023-02-27 16:15:10Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -149,7 +149,7 @@ bool CGvfReadRecord::SanityCheck() const
 
 //  ----------------------------------------------------------------------------
 CGvfReader::CGvfReader(
-    unsigned int uFlags,
+    TReaderFlags uFlags,
     const string& name,
     const string& title,
     CReaderListener* pRL):
@@ -237,60 +237,6 @@ void CGvfReader::xPostProcessAnnot(
     }
 }
 
-//  ----------------------------------------------------------------------------
-CRef<CSeq_annot> CGvfReader::x_GetAnnotById(
-    TAnnots& annots,
-    const string& strId )
-//  ----------------------------------------------------------------------------
-{
-    for ( TAnnotIt it = annots.begin(); it != annots.end(); ++it ) {
-        CSeq_annot& annot = **it;
-        if ( ! annot.CanGetId() || annot.GetId().size() != 1 ) {
-            // internal error
-            return CRef<CSeq_annot>();
-        }
-
-        CRef< CAnnot_id > pId = *( annot.GetId().begin() );
-        if ( ! pId->IsLocal() ) {
-            // internal error
-            return CRef<CSeq_annot>();
-        }
-        if ( strId == pId->GetLocal().GetStr() ) {
-            return *it;
-        }
-    }
-    CRef<CSeq_annot> pNewAnnot( new CSeq_annot );
-    annots.insert(annots.begin(), pNewAnnot);
-
-    CRef< CAnnot_id > pAnnotId( new CAnnot_id );
-    pAnnotId->SetLocal().SetStr( strId );
-    pNewAnnot->SetId().push_back( pAnnotId );
-    pNewAnnot->SetData().SetFtable();
-
-    // if available, add current browser information
-    if ( m_CurrentBrowserInfo ) {
-        pNewAnnot->SetDesc().Set().push_back( m_CurrentBrowserInfo );
-    }
-
-    // if available, add current track information
-    if (m_pTrackDefaults->ContainsData()) {
-        xAssignTrackData(*pNewAnnot);
-    }
-
-    if ( !m_AnnotName.empty() ) {
-        pNewAnnot->SetNameDesc(m_AnnotName);
-    }
-    if ( !m_AnnotTitle.empty() ) {
-        pNewAnnot->SetTitleDesc(m_AnnotTitle);
-    }
-
-    // if available, add gvf pragma information
-    if ( m_Pragmas ) {
-        pNewAnnot->SetDesc().Set().push_back( m_Pragmas );
-    }
-    return pNewAnnot;
-}
-
 //  ----------------------------------------------------------------------------
 bool CGvfReader::xMergeRecord(
     const CGvfReadRecord& record,
@@ -330,191 +276,6 @@ bool CGvfReader::xFeatureSetLocation(
 }
 
 
-//  ----------------------------------------------------------------------------
-bool CGvfReader::xSetLocation(
-        const CGff2Record& record,
-        CSeq_loc& location)
-//  ----------------------------------------------------------------------------
-{
-
-    if (record.SeqStart() < record.SeqStop()) {
-        return xSetLocationInterval(record, location);
-    }
-    else { // record.SeqStart() == record.SeqStop()
-        return xSetLocationPoint(record, location);
-    }
-}
-
-
-
-//  ----------------------------------------------------------------------------
-bool CGvfReader::xSetLocationInterval(
-    const CGff2Record& record,
-    CSeq_loc& location)
-//  ----------------------------------------------------------------------------
-{
-    CRef< CSeq_id > pId = mSeqIdResolve(record.Id(), m_iFlags, true);
-
-    location.SetInt().SetId(*pId);
-    location.SetInt().SetFrom(record.SeqStart());
-    location.SetInt().SetTo(record.SeqStop());
-    if (record.IsSetStrand()) {
-        location.SetInt().SetStrand( record.Strand() );
-    }
-
-    //  deal with fuzzy range indicators / lower end:
-    string strRange;
-    list<string> range_borders;
-    TSeqPos lower, upper;
-    if (record.GetAttribute( "Start_range", strRange ) )
-    {
-        NStr::Split( strRange, ",", range_borders, 0 );
-        if ( range_borders.size() != 2 ) {
-            CReaderMessage error(
-                eDiag_Error,
-                m_uLineNumber,
-                "Bad Start_range attribute: Start_range=" + strRange + ".");
-            throw error;
-        }
-        try {
-            if ( range_borders.back() == "." ) {
-                lower = upper = NStr::StringToUInt( range_borders.front() );
-                location.SetInt().SetFuzz_from().SetLim( CInt_fuzz::eLim_gt );
-            }
-            else if ( range_borders.front() == "." ) {
-                lower = upper = NStr::StringToUInt( range_borders.back() );
-                location.SetInt().SetFuzz_from().SetLim( CInt_fuzz::eLim_lt );
-            }
-            else {
-                lower = NStr::StringToUInt( range_borders.front() );
-                upper = NStr::StringToUInt( range_borders.back() );
-                location.SetInt().SetFuzz_from().SetRange().SetMin( lower-1 );
-                location.SetInt().SetFuzz_from().SetRange().SetMax( upper-1 );
-            }
-        }
-        catch ( std::exception& ) {
-            CReaderMessage error(
-                eDiag_Error,
-                m_uLineNumber,
-                "Bad Start_range attribute: Start_range=" + strRange + ".");
-            throw error;
-        }
-    }
-
-    //  deal with fuzzy range indicators / upper end:
-    range_borders.clear();
-    if (record.GetAttribute( "End_range", strRange ) )
-    {
-        NStr::Split( strRange, ",", range_borders, 0 );
-        if ( range_borders.size() != 2 ) {
-            CReaderMessage error(
-                eDiag_Error,
-                m_uLineNumber,
-                "Bad End_range attribute: End_range=" + strRange + ".");
-            throw error;
-        }
-        try {
-            if ( range_borders.back() == "." ) {
-                lower = upper = NStr::StringToUInt( range_borders.front() );
-                location.SetInt().SetFuzz_to().SetLim( CInt_fuzz::eLim_gt );
-            }
-            else if ( range_borders.front() == "." ) {
-                lower = upper = NStr::StringToUInt( range_borders.back() );
-                location.SetInt().SetFuzz_to().SetLim( CInt_fuzz::eLim_lt );
-            }
-            else {
-                lower = NStr::StringToUInt( range_borders.front() );
-                upper = NStr::StringToUInt( range_borders.back() );
-                location.SetInt().SetFuzz_to().SetRange().SetMin( lower-1 );
-                location.SetInt().SetFuzz_to().SetRange().SetMax( upper-1 );
-            }
-        }
-        catch (std::exception&) {
-            CReaderMessage error(
-                eDiag_Error,
-                m_uLineNumber,
-                "Bad End_range attribute: End_range=" + strRange + ".");
-            throw error;
-        }
-    }
-
-    return true;
-}
-
-
-//  ----------------------------------------------------------------------------
-bool CGvfReader::xSetLocationPoint(
-    const CGff2Record& record,
-    CSeq_loc& location)
-//  ----------------------------------------------------------------------------
-{
-    CRef< CSeq_id > pId = mSeqIdResolve(record.Id(), m_iFlags, true);
-    location.SetPnt().SetId(*pId);
-    if (record.Type() == "insertion") {
-        //for insertions, GVF uses insert-after logic while NCBI uses insert-before
-        location.SetPnt().SetPoint(record.SeqStart()+1);
-    }
-    else {
-        location.SetPnt().SetPoint(record.SeqStart());
-    }
-    if (record.IsSetStrand()) {
-        location.SetStrand(record.Strand());
-    }
-
-    string strRangeLower, strRangeUpper;
-    bool hasLower = record.GetAttribute("Start_range", strRangeLower);
-    bool hasUpper = record.GetAttribute("End_range", strRangeUpper);
-    if (hasLower  &&  hasUpper  &&  strRangeLower != strRangeUpper) {
-        CReaderMessage error(
-            eDiag_Error,
-            m_uLineNumber,
-            "Bad range attribute: Conflicting fuzz ranges for single point location.");
-        throw error;
-    }
-    if (!hasLower  &&  !hasUpper) {
-        return true;
-    }
-    if (!hasLower) {
-        strRangeLower = strRangeUpper;
-    }
-
-    list<string> bounds;
-    TSeqPos lower, upper;
-    NStr::Split( strRangeLower, ",", bounds, 0 );
-    if (bounds.size() != 2) {
-        CReaderMessage error(
-            eDiag_Error,
-            m_uLineNumber,
-            "Bad range attribute: XXX_range=" + strRangeLower + ".");
-        throw error;
-    }
-    try {
-        if (bounds.back() == ".") {
-            lower = upper = NStr::StringToUInt(bounds.front());
-            location.SetPnt().SetFuzz().SetLim(CInt_fuzz::eLim_gt);
-        }
-        else if (bounds.front() == ".") {
-            lower = upper = NStr::StringToUInt(bounds.back());
-            location.SetPnt().SetFuzz().SetLim( CInt_fuzz::eLim_lt );
-        }
-        else {
-            lower = NStr::StringToUInt(bounds.front());
-            upper = NStr::StringToUInt(bounds.back());
-            location.SetPnt().SetFuzz().SetRange().SetMin(lower-1);
-            location.SetPnt().SetFuzz().SetRange().SetMax(upper-1);
-        }
-    }
-    catch (std::exception&) {
-        CReaderMessage error(
-            eDiag_Error,
-            m_uLineNumber,
-            "Bad range attribute: XXX_range=" + strRangeLower + ".");
-        throw error;
-    }
-    return true;
-}
-
-
 //  ----------------------------------------------------------------------------
 bool CGvfReader::xFeatureSetLocationInterval(
     const CGff2Record& record,
