@@ -1,4 +1,4 @@
-/*  $Id: mod_reader.cpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: mod_reader.cpp 676196 2023-12-11 21:07:17Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -94,7 +94,8 @@ static const unordered_map<string, string> s_ModNameMap =
  {"db-xref", "dbxref"},
  {"pubmed", "pmid"},
  {"ft-url-mod", "ft-mod"},
- {"ft-url", "ft-map"}
+ {"ft-url", "ft-map"},
+ {"geo-loc-name", "country"}
  };
 //);
 
@@ -202,6 +203,14 @@ void CModHandler::SetExcludedMods(const vector<string>& excluded_mods)
             [](const string& mod_name) { return GetCanonicalName(mod_name); });
 }
 
+void CModHandler::SetIgnoredMods(const list<string>& ignored_mods) 
+{
+    m_IgnoredModifiers.clear();
+    transform(ignored_mods.begin(), ignored_mods.end(),
+            inserter(m_IgnoredModifiers, m_IgnoredModifiers.end()),
+            [](const string& mod_name) { return GetCanonicalName(mod_name); });
+}
+
 
 void CModHandler::SetMods(const TMods& mods)
 {
@@ -232,6 +241,12 @@ void CModHandler::AddMods(const TModList& mods,
             }
         }
 
+        if (m_IgnoredModifiers.find(canonical_name) !=
+            m_IgnoredModifiers.end()) {
+                rejected_mods.push_back(mod);
+                continue;
+        }
+
         if (m_ExcludedModifiers.find(canonical_name) !=
             m_ExcludedModifiers.end()) {
             string message = "The following modifier is unsupported in this context and will be ignored: " + mod.GetName() + ".";
@@ -305,7 +320,7 @@ void CModHandler::AddMods(const TModList& mods,
         rejected_mods.splice(rejected_mods.end(), conflicts.second);
     }
 
-    x_SaveMods(move(accepted_mods), handle_existing, m_Mods);
+    x_SaveMods(std::move(accepted_mods), handle_existing, m_Mods);
 }
 
 
@@ -330,10 +345,10 @@ void CModHandler::x_SaveMods(TMods&& mods, EHandleExisting handle_existing, TMod
             if (x_MultipleValuesAllowed(canonical_name)){
                 dest_mod_list.splice(
                         dest_mod_list.end(),
-                        move(mod_entry.second));
+                    std::move(mod_entry.second));
             }
             else {
-                dest_mod_list = move(mod_entry.second);
+                dest_mod_list = std::move(mod_entry.second);
             }
         }
     }
@@ -343,13 +358,13 @@ void CModHandler::x_SaveMods(TMods&& mods, EHandleExisting handle_existing, TMod
             const auto& canonical_name = mod_entry.first;
             auto& dest_mod_list = dest[canonical_name];
             if (dest_mod_list.empty()) {
-                dest_mod_list = move(mod_entry.second);
+                dest_mod_list = std::move(mod_entry.second);
             }
             else
             if (x_MultipleValuesAllowed(canonical_name)){
                 dest_mod_list.splice(
                         dest_mod_list.end(),
-                        move(mod_entry.second));
+                    std::move(mod_entry.second));
             }
         }
     }
@@ -681,7 +696,7 @@ void CModAdder::x_SetHist(const TModEntry& mod_entry, CSeq_inst& seq_inst)
     transform(id_list.begin(), id_list.end(), back_inserter(secondary_ids),
             [](const string& id_string) { return Ref(new CSeq_id(id_string)); });
 
-    seq_inst.SetHist().SetReplaces().SetIds() = move(secondary_ids);
+    seq_inst.SetHist().SetReplaces().SetIds() = std::move(secondary_ids);
 }
 
 
