@@ -1,4 +1,4 @@
-/*  $Id: wiggle_reader.cpp 632526 2021-06-02 17:25:01Z ivanov $
+/*  $Id: wiggle_reader.cpp 653402 2022-07-27 16:13:42Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -153,12 +153,12 @@ CWiggleReader::xGetData(
 void
 CWiggleReader::xProcessData(
     const TReaderData& readerData,
-    CSeq_annot&)
+    CSeq_annot& annot)
 //  ----------------------------------------------------------------------------
 {
     for (auto curData = readerData.begin(); curData != readerData.end(); curData++) {
         auto line = curData->mData;
-        if (xParseBrowserLine(line)) {
+        if (xParseBrowserLine(line, annot)) {
             continue;
         }
         if (xParseTrackLine(line)) {
@@ -172,6 +172,7 @@ CWiggleReader::xProcessData(
             break;
         }
         xProcessBedData(curData, readerData);
+        break;
     }
 }
 
@@ -461,29 +462,14 @@ CRef<CSeq_table> CWiggleReader::xMakeTable()
         table->SetColumns().push_back(col_val);
         col_val->SetHeader().SetField_name("values");
 
-        if ( 1 ) {
-            AutoPtr< vector<char> > values(new vector<char>());
-            values->reserve(size);
-            ITERATE ( TValues, it, m_Values ) {
-                pos.push_back(it->m_Pos);
-                if ( span_ptr ) {
-                    span_ptr->push_back(it->m_Span);
-                }
-                values->push_back(stat.AsByte(it->m_Value));
-            }
-            col_val->SetData().SetBytes().push_back(values.release());
-        }
-        else {
-            CSeqTable_multi_data::TInt& values = col_val->SetData().SetInt();
-            values.reserve(size);
-
-            ITERATE ( TValues, it, m_Values ) {
-                pos.push_back(it->m_Pos);
-                if ( span_ptr ) {
-                    span_ptr->push_back(it->m_Span);
-                }
-                values.push_back(stat.AsByte(it->m_Value));
+        CSeqTable_multi_data::TInt2& values = col_val->SetData().SetInt2();
+        values.reserve(size);
+        ITERATE ( TValues, it, m_Values ) {
+            pos.push_back(it->m_Pos);
+            if ( span_ptr ) {
+                span_ptr->push_back(it->m_Span);
             }
+            values.push_back(stat.AsByte(it->m_Value));
         }
     }
     else {
@@ -827,7 +813,8 @@ void CWiggleReader::xSetChrom(
 
 //  ----------------------------------------------------------------------------
 bool CWiggleReader::xParseBrowserLine(
-    const string& line)
+    const string& line,
+    CSeq_annot&)
 //  ----------------------------------------------------------------------------
 {
     if (!NStr::StartsWith(line, "browser")) {
