@@ -1,4 +1,4 @@
-/*  $Id: choicestr.cpp 619575 2020-11-06 16:11:34Z gouriano $
+/*  $Id: choicestr.cpp 670437 2023-07-25 12:17:23Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -228,6 +228,15 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
         // convert string member to pointer member
         havePointers = true;
     }
+    bool haveUnionData = false;
+    if ( haveUnion ) {
+        ITERATE ( TVariants, i, m_Variants ) {
+            if ( !x_IsNullType(i)) {
+                haveUnionData = true;
+                break;
+            }
+        }
+    }
 
     string stdNamespace = 
         code.GetNamespace().GetNamespaceRef(CNamespace::KSTDNamespace);
@@ -764,7 +773,7 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
             }
         }
         methods << ")\n{\n";
-        if ( haveUnion || haveObjectPointer ) {
+        if ( ( (haveUnion && haveUnionData) || haveObjectPointer) ) {
             methods <<
                 "    switch ( index ) {\n";
             ITERATE ( TVariants, i, m_Variants ) {
@@ -1247,6 +1256,9 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
                             if (typeStr) {
                                 ITERATE ( TMembers, ir, typeStr->m_Members ) {
                                     if (ir->simple) {
+                                        if (!ir->dataType || !ir->dataType->IsPrimitive()) {
+                                            continue;
+                                        }
                                         string ircType(ir->type->GetCType(
                                             code.GetNamespace()));
                                         if (CClassCode::GetDoxygenComments()) {
@@ -1294,7 +1306,7 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
                 }
             }
         }
-        if ( haveUnion ) {
+        if ( haveUnion  && haveUnionData) {
             code.ClassPrivate() << "    union {\n";
             ITERATE ( TVariants, i, m_Variants ) {
                 if ( i->memberType == eSimpleMember ) {
