@@ -1,4 +1,4 @@
-/*  $Id: dtdparser.cpp 619717 2020-11-09 20:00:26Z gouriano $
+/*  $Id: dtdparser.cpp 626533 2021-03-01 15:16:52Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -706,7 +706,7 @@ DTDEntity* DTDParser::PushEntityLexer(const string& name)
         m_StackLexerName.push_back(fullname);
         lexer_name = fullname;
     } else {
-        in = new CNcbiIstrstream(m_MapEntity[name].GetData().c_str());
+        in = new CNcbiIstrstream(m_MapEntity[name].GetData());
         m_StackPath.push("");
         m_StackLexerName.push_back(name);
         lexer_name = name;
