@@ -1,4 +1,4 @@
-/*  $Id: grpc_integration.cpp 608310 2020-05-14 12:35:38Z ivanov $
+/*  $Id: grpc_integration.cpp 667646 2023-05-18 18:14:23Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -43,7 +43,7 @@
 #  include <grpc/support/alloc.h>
 #  include <grpc/support/log.h>
 #endif
-#if defined(HAVE_LIBCONNEXT)  &&  defined(NCBI_OS_UNIX)
+#ifdef NCBI_OS_UNIX
 #  include <connect/ext/ncbi_ifconf.h>
 #endif
 
@@ -191,7 +191,7 @@ void CGRPCClientContext::AddStandardNCBIMetadata(TGRPCBaseClientContext& cctx)
              }
              return true;
          });
-#  if defined(HAVE_LIBCONNEXT)  &&  defined(NCBI_OS_UNIX)
+#  ifdef NCBI_OS_UNIX
     if ( !rctx.IsSetClientIP() ) {
         char buf[64];
         if (NcbiGetHostIP(buf, sizeof(buf)) != nullptr) {
@@ -372,7 +372,7 @@ void CGRPCServerCallbacks::BeginRequest(TGRPCServerContext* sctx,
                 value = &it.second.begin()->second;
             } else {
                 _ASSERT(it.second.size() == 2);
-                const string *old_name, *new_name;
+                const string *old_name = nullptr, *new_name = nullptr;
                 for (const auto& it2 : it.second) {
                     if (NStr::StartsWith(it2.first, "ncbi_")) {
                         new_name = &it2.first;
@@ -463,7 +463,7 @@ bool CGRPCServerCallbacks::x_IsRealRequest(const TGRPCServerContext* sctx)
 string g_NCBI_GRPC_GetAddress(const char* section,
                               const char* variable,
                               const char* env_var_name,
-                              int* value_source)
+                              CParamBase::EParamSource* value_source)
 {
     auto addr = g_GetConfigString(section, variable, env_var_name, nullptr, value_source);
     if ( addr.empty() ) {
