@@ -1,4 +1,4 @@
-/*  $Id: objistr.cpp 607093 2020-04-30 12:29:36Z grichenk $
+/*  $Id: objistr.cpp 673980 2023-10-16 13:16:36Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -796,10 +796,20 @@ void CObjectIStream::SetStreamPos(CNcbiStreampos pos)
 
 string CObjectIStream::GetPosition(void) const
 {
-    return "byte "+NStr::Int8ToString(NcbiStreamposToInt8(GetStreamPos()));
+    string loc_type;
+    size_t loc_pos;
+    Location(loc_type, loc_pos);
+    return loc_type+" "+NStr::Int8ToString(NcbiStreamposToInt8(loc_pos));
 }
 
-void CObjectIStream::ThrowError1(const CDiagCompileInfo& diag_info, 
+void CObjectIStream::Location(string& loc_type, size_t& loc) const
+{
+    loc_type = "byte";
+    loc = NcbiStreamposToInt8(GetStreamPos());
+}
+
+
+void CObjectIStream::ThrowError1(const CDiagCompileInfo& diag_info,
                                  TFailFlags flags, const char* message)
 {
     ThrowError1(diag_info, flags, string(message));
@@ -888,7 +898,7 @@ void CObjectIStream::EndOfRead(void)
         m_Objects->Clear();
 }
 
-set<TTypeInfo> CObjectIStream::GuessDataType(set<TTypeInfo>& known_types,
+set<TTypeInfo> CObjectIStream::GuessDataType(const set<TTypeInfo>& known_types,
                                              size_t /*max_length*/,
                                              size_t max_bytes)
 {
@@ -1857,8 +1867,12 @@ char ReplaceVisibleChar(char c, EFixNonPrint fix_method,
         }
         switch (fix_method) {
         case eFNP_ReplaceAndWarn:
+#if 0
             CNcbiDiag(eDiag_Error, eDPF_Default)
                 << ErrCode(NCBI_ERRCODE_X, 7) << message << Endm;
+#else
+            ERR_POST_X(7, message);
+#endif
             break;
         case eFNP_Throw:
             NCBI_THROW(CSerialException,eFormatError,message);
