@@ -1,4 +1,4 @@
-/*  $Id: objostr.cpp 604622 2020-03-31 13:31:03Z ivanov $
+/*  $Id: objostr.cpp 607093 2020-04-30 12:29:36Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1198,7 +1198,7 @@ void CObjectOStream::SetFormattingFlags(TSerial_Format_Flags flags)
 #ifdef NCBI_STRICT_GI
 void CObjectOStream::WriteGi(const TGi& obj)
 {
-    WriteInt8(obj);
+    WriteInt8(obj.Get());
 }
 
 void CObjectOStream::CopyGi(CObjectIStream& in)
