@@ -1,4 +1,4 @@
-/*  $Id: typeinfo.cpp 586652 2019-05-20 18:09:55Z gouriano $
+/*  $Id: typeinfo.cpp 667218 2023-05-10 17:54:36Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -334,9 +334,9 @@ bool CTypeInfo::IsParentClassOf(const CClassTypeInfo* /*classInfo*/) const
 
 const CSerialUserOp* CTypeInfo::AsCSerialUserOp(TConstObjectPtr obj) const
 {
-    if (IsCObject() && m_CSerialUserOp != eTriState_False) {
+    if (IsCObject() && m_CSerialUserOp.load(memory_order_relaxed) != eTriState_False) {
         const CSerialUserOp* op = dynamic_cast<const CSerialUserOp*>( static_cast<const CObject*>(obj));
-        m_CSerialUserOp = (op != nullptr) ? eTriState_True : eTriState_False;
+        m_CSerialUserOp.store((op != nullptr) ? eTriState_True : eTriState_False, memory_order_relaxed);
         return op;
     }
     return nullptr;
@@ -344,9 +344,9 @@ const CSerialUserOp* CTypeInfo::AsCSerialUserOp(TConstObjectPtr obj) const
 
 CSerialUserOp* CTypeInfo::AsCSerialUserOp(TObjectPtr obj) const
 {
-    if (IsCObject() && m_CSerialUserOp != eTriState_False) {
+    if (IsCObject() && m_CSerialUserOp.load(memory_order_relaxed) != eTriState_False) {
         CSerialUserOp* op = dynamic_cast<CSerialUserOp*>( static_cast<CObject*>(obj));
-        m_CSerialUserOp = (op != nullptr) ? eTriState_True : eTriState_False;
+        m_CSerialUserOp.store((op != nullptr) ? eTriState_True : eTriState_False, memory_order_relaxed);
         return op;
     }
     return nullptr;
