@@ -1,4 +1,4 @@
-/*  $Id: zlib.cpp 612744 2020-07-27 14:26:09Z ivanov $
+/*  $Id: zlib.cpp 658835 2022-11-09 13:48:43Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -27,14 +27,13 @@
  *           Jean-loup Gailly, Mark Adler
  *           (used a part of zlib library code from: gzio.c, uncompr.c)
  *
- * File Description:  ZLib Compression API
+ * File Description:  ZLib Compression API wrapper
  *
  * NOTE: The zlib documentation can be found here: 
  *           http://zlib.org,  
  *           http://www.gzip.org/zlib/manual.html
  */
 
-
 #include <ncbi_pch.hpp>
 #include <corelib/ncbi_limits.h>
 #include <corelib/ncbifile.hpp>
@@ -45,12 +44,27 @@
 #include <stdio.h>
 
 
+/// Error codes for ERR_COMPRESS and OMPRESS_HANDLE_EXCEPTIONS are really
+/// a subcodes and current maximum  value is defined in 'include/util/error_codes.hpp':
+///     NCBI_DEFINE_ERRCODE_X(Util_Compress, 210, max);
+/// For new values use 'max'+1 and update it there.
+///
 #define NCBI_USE_ERRCODE_X   Util_Compress
 
+
 BEGIN_NCBI_SCOPE
 
 
+// Some of macro below are defined in zutil.h, but not all installations of zlib 
+// have it available :(
+
+
 // Define some macro if not defined
+
+#ifndef DEF_WBITS
+#  define DEF_WBITS MAX_WBITS
+#endif
+
 #ifndef DEF_MEM_LEVEL
 #  if MAX_MEM_LEVEL >= 8
 #    define DEF_MEM_LEVEL 8
@@ -91,13 +105,32 @@ const unsigned int kMaxChunkSize = kMax_UInt;
 // CZipCompression
 //
 
+CZipCompression::CZipCompression(ELevel level)
+    : CCompression(level)
+{
+    // Set advanced compression parameters
+    SetWindowBits ( GetWindowBitsDefault()  );
+    SetMemoryLevel( GetMemoryLevelDefault() );
+    SetStrategy   ( GetStrategyDefault()    );
+
+    // Initialize the compressor stream structure
+    m_Stream = new z_stream;
+    if ( m_Stream ) {
+        memset(m_Stream, 0, sizeof(z_stream));
+    }
+    return;
+}
+
+// @deprecated
 CZipCompression::CZipCompression(ELevel level,  int window_bits,
                                  int mem_level, int strategy)
     : CCompression(level)
 {
-    m_WindowBits = ( window_bits == kZlibDefaultWbits )  ? MAX_WBITS : window_bits;
-    m_MemLevel   = ( mem_level == kZlibDefaultMemLevel ) ? DEF_MEM_LEVEL : mem_level;
-    m_Strategy   = ( strategy == kZlibDefaultStrategy )  ? Z_DEFAULT_STRATEGY : strategy;
+    // Set advanced compression parameters
+    SetWindowBits  (( window_bits == kZlibDefaultWbits )  ? GetWindowBitsDefault()  : window_bits);
+    SetMemoryLevel (( mem_level == kZlibDefaultMemLevel ) ? GetMemoryLevelDefault() : mem_level);
+    SetStrategy    (( strategy == kZlibDefaultStrategy )  ? GetStrategyDefault()    : strategy);
+
     // Initialize the compressor stream structure
     m_Stream = new z_stream;
     if ( m_Stream ) {
@@ -297,7 +330,19 @@ unsigned long s_GetCRC32(const void* buf, size_t len)
 
 CVersionInfo CZipCompression::GetVersion(void) const
 {
-    return CVersionInfo(ZLIB_VERSION, "zlib");
+    return CVersionInfo(ZLIB_VER_MAJOR, ZLIB_VER_MINOR, ZLIB_VER_REVISION, "zlib");
+}
+
+
+bool CZipCompression::HaveSupport(ESupportFeature feature)
+{
+    switch (feature) {
+    case eFeature_NoCompression:
+    case eFeature_Dictionary:
+    case eFeature_EstimateCompressionBufferSize:
+        return true;
+    }
+    return false;
 }
 
 
@@ -306,6 +351,7 @@ bool CZipCompression::CompressBuffer(
                       void*       dst_buf, size_t  dst_size,
                       /* out */   size_t* dst_len)
 {
+    const char* __method_name = "CZipCompression::CompressBuffer";
     *dst_len = 0;
 
     // Check parameters
@@ -314,7 +360,7 @@ bool CZipCompression::CompressBuffer(
     }
     if (!src_buf || !dst_buf || !dst_len) {
         SetError(Z_STREAM_ERROR, "bad argument");
-        ERR_COMPRESS(48, FormatErrorMessage("CZipCompression::CompressBuffer"));
+        ERR_COMPRESS(48, FormatErrorMessage(__method_name));
         return false;
     }
 
@@ -327,8 +373,8 @@ bool CZipCompression::CompressBuffer(
     if ( F_ISSET(fWriteGZipFormat) ) {
         header_len = s_WriteGZipHeader(dst_buf, dst_size);
         if (!header_len) {
-            SetError(Z_STREAM_ERROR, "Cannot write gzip header");
-            ERR_COMPRESS(50, FormatErrorMessage("CZipCompression::CompressBuffer"));
+            SetError(Z_STREAM_ERROR, "cannot write gzip header");
+            ERR_COMPRESS(50, FormatErrorMessage(__method_name));
             return false;
         }
         dst += header_len;
@@ -340,11 +386,20 @@ bool CZipCompression::CompressBuffer(
     STREAM->opaque = NULL;
 
     errcode = deflateInit2_(STREAM, GetLevel(), Z_DEFLATED,
-                            header_len ? -m_WindowBits : m_WindowBits,
-                            m_MemLevel, m_Strategy,
+                            header_len ? -GetWindowBits() : GetWindowBits(),
+                            GetMemoryLevel(), GetStrategy(),
                             ZLIB_VERSION, (int)sizeof(z_stream));
     if (errcode == Z_OK) {
-    
+
+        // Set dictionary
+        if ( m_Dict ) {
+            errcode = deflateSetDictionary(STREAM, (const Bytef*) m_Dict->GetData(), (uInt) m_Dict->GetSize());
+            if (errcode != Z_OK) {
+                SetError(Z_STREAM_ERROR, "cannot set a dictionary");
+                ERR_COMPRESS(121, FormatErrorMessage(__method_name));
+            }
+        }
+        // Compress data
         STREAM->next_in   = (unsigned char*)src_buf;
         STREAM->avail_in  = 0;
         STREAM->next_out  = dst;
@@ -398,19 +453,19 @@ bool CZipCompression::DecompressBuffer(
                       void*       dst_buf, size_t  dst_size,
                       /* out */            size_t* dst_len)
 {
+    const char* __method_name = "CZipCompression::DecompressBuffer";
     *dst_len = 0;
 
     // Check parameters
     if ( !src_len ) {
         if ( F_ISSET(fAllowEmptyData) ) {
-            SetError(Z_OK);
             return true;
         }
         src_buf = NULL;
     }
     if (!src_buf || !dst_buf || !dst_len) {
         SetError(Z_STREAM_ERROR, "bad argument");
-        ERR_COMPRESS(55, FormatErrorMessage("CZipCompression::DecompressBuffer"));
+        ERR_COMPRESS(55, FormatErrorMessage(__method_name));
         return false;
     }
 
@@ -442,18 +497,31 @@ bool CZipCompression::DecompressBuffer(
             gzip_chunk_count++;
         }
 
-        // "window bits" is passed < 0 to tell that there is no zlib header.
+        // "window bits" is passed < 0 to tell that there is a raw deflate data (no gzip header).
         // Note that in this case inflate *requires* an extra "dummy" byte
         // after the compressed stream in order to complete decompression and
         // return Z_STREAM_END. Here the gzip's CRC32 ensures that 4 bytes are
         // present after the compressed stream.
 
-        errcode = inflateInit2_(STREAM, header_len ? -m_WindowBits :m_WindowBits,
+        errcode = inflateInit2_(STREAM, header_len ? -GetWindowBits() : GetWindowBits(),
                                 ZLIB_VERSION, (int)sizeof(z_stream));
         if (errcode != Z_OK) {
             break;
         }
 
+        // Setting dictionary for decompressing is inconsistent for zlib...
+        // For raw deflate we should set it immediately after inflateInit2(), or stream
+        // will produce a data error and never ask for a dictionary using Z_NEED_DICT.
+        // So, set it here forcely, and also later when decompressor ask for it directly.
+        if (header_len  &&  m_Dict) {
+            errcode = inflateSetDictionary(STREAM, (const Bytef*) m_Dict->GetData(), (uInt) m_Dict->GetSize());
+            if (errcode != Z_OK) {
+                break;
+            }
+        }
+
+        // Decompress 
+
         STREAM->next_in   = src;
         STREAM->avail_in  = 0;
         STREAM->next_out  = dst;
@@ -470,9 +538,15 @@ bool CZipCompression::DecompressBuffer(
                 dst_size -= STREAM->avail_out;
             }
             errcode = inflate(STREAM, Z_NO_FLUSH);
+
+            // Set dictionary if asked, and continue.
+            if (errcode == Z_NEED_DICT  &&  m_Dict) {
+                errcode = inflateSetDictionary(STREAM, (const Bytef*) m_Dict->GetData(), (uInt) m_Dict->GetSize());
+                // errcode should be Z_OK to continue with a dictionary
+            }
         } while (errcode == Z_OK);
 
-        // Calculate length of output data.
+        // Calculate length of the output data.
         // Don't use STREAM->total_out for source buffers greater than 4GB,
         // it works if (SIZEOF_LONG > 4) only.
         *dst_len = STREAM->next_out - (unsigned char*)dst_buf;
@@ -511,18 +585,17 @@ bool CZipCompression::DecompressBuffer(
 
     SetError(errcode, zError(errcode));
     if ( errcode != Z_STREAM_END ) {
-        ERR_COMPRESS(59, FormatErrorMessage("CZipCompression::DecompressBuffer",
-                                            STREAM->next_in - (unsigned char*) src_buf));
+        ERR_COMPRESS(59, FormatErrorMessage(__method_name, STREAM->next_in - (unsigned char*) src_buf));
         return false;
     }
     return true;
 }
 
 
-long CZipCompression::EstimateCompressionBufferSize(size_t src_len)
+size_t CZipCompression::EstimateCompressionBufferSize(size_t src_len)
 {
 #if (ZLIB_VERNUM < 0x1200)
-    return -1;
+    return 0;
 #else
     size_t header_len = 0;
     int    errcode    = Z_OK;
@@ -535,26 +608,46 @@ long CZipCompression::EstimateCompressionBufferSize(size_t src_len)
     STREAM->zfree  = NULL;
     STREAM->opaque = NULL;
     errcode = deflateInit2_(STREAM, GetLevel(), Z_DEFLATED,
-                            header_len ? -m_WindowBits : m_WindowBits,
-                            m_MemLevel, m_Strategy,
+                            header_len ? -GetWindowBits() : GetWindowBits(),
+                            GetMemoryLevel(), GetStrategy(),
                             ZLIB_VERSION, (int)sizeof(z_stream));
     if (errcode != Z_OK) {
         SetError(errcode, zError(errcode));
-        return -1;
+        return 0;
     }
-    long n = (long)(deflateBound(STREAM, (unsigned long)src_len) + header_len);
+    size_t n = deflateBound(STREAM, (unsigned long)src_len) + header_len;
     deflateEnd(STREAM);
     return n;
 #endif
 }
 
 
+CCompression::SRecommendedBufferSizes 
+CZipCompression::GetRecommendedBufferSizes(size_t round_up)
+{
+    SRecommendedBufferSizes sizes;
+    sizes.compression_in    = sizes.RoundUp( kCompressionDefaultBufSize, round_up);
+    sizes.compression_out   = sizes.RoundUp( kCompressionDefaultBufSize, round_up);
+    sizes.decompression_in  = sizes.RoundUp( kCompressionDefaultBufSize, round_up);
+    sizes.decompression_out = sizes.RoundUp( kCompressionDefaultBufSize, round_up);
+    return sizes;
+}
+
+
 bool CZipCompression::CompressFile(const string& src_file,
                                    const string& dst_file,
-                                   size_t        buf_size)
+                                   size_t        file_io_bufsize,
+                                   size_t        compression_in_bufsize,
+                                   size_t        compression_out_bufsize)
 {
-    CZipCompressionFile cf(GetLevel(), m_WindowBits, m_MemLevel, m_Strategy);
+    CZipCompressionFile cf(GetLevel());
     cf.SetFlags(cf.GetFlags() | GetFlags());
+    cf.SetWindowBits(GetWindowBits());
+    cf.SetMemoryLevel(GetMemoryLevel());
+    cf.SetStrategy(GetStrategy());
+    if (m_Dict) {
+        cf.SetDictionary(*m_Dict, eNoOwnership);
+    }
 
     // Collect info about compressed file
     CZipCompression::SFileInfo info;
@@ -563,12 +656,13 @@ bool CZipCompression::CompressFile(const string& src_file,
     // write gzip file by default.
     s_CollectFileInfo(src_file, info);
     // Open output file
-    if ( !cf.Open(dst_file, CCompressionFile::eMode_Write, &info) ) {
+    if ( !cf.Open(dst_file, CCompressionFile::eMode_Write, &info,
+                  compression_in_bufsize, compression_out_bufsize) ) {
         SetError(cf.GetErrorCode(), cf.GetErrorDescription());
         return false;
     } 
     // Make compression
-    if ( !CCompression::x_CompressFile(src_file, cf, buf_size) ) {
+    if ( !CCompression::x_CompressFile(src_file, cf, file_io_bufsize) ) {
         if ( cf.GetErrorCode() ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
         }
@@ -584,17 +678,25 @@ bool CZipCompression::CompressFile(const string& src_file,
 
 bool CZipCompression::DecompressFile(const string& src_file,
                                      const string& dst_file,
-                                     size_t        buf_size)
+                                     size_t        file_io_bufsize,
+                                     size_t        decompression_in_bufsize,
+                                     size_t        decompression_out_bufsize)
 {
-    CZipCompressionFile cf(GetLevel(), m_WindowBits, m_MemLevel, m_Strategy);
-    cf.SetFlags(cf.GetFlags() | GetFlags());
+    CZipCompressionFile cf(GetLevel());
+    cf.SetWindowBits(GetWindowBits());
+    cf.SetMemoryLevel(GetMemoryLevel());
+    cf.SetStrategy(GetStrategy());
+    if (m_Dict) {
+        cf.SetDictionary(*m_Dict, eNoOwnership);
+    }
 
     bool need_restore_attr = false;
     SFileInfo info;
 
     // Open compressed file, and define name of the destination file
     if ( F_ISSET(fRestoreFileAttr) ) {
-        if ( !cf.Open(src_file, CCompressionFile::eMode_Read, &info) ) {
+        if ( !cf.Open(src_file, CCompressionFile::eMode_Read, &info,
+                      decompression_in_bufsize, decompression_out_bufsize) ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
             return false;
         } 
@@ -602,23 +704,25 @@ bool CZipCompression::DecompressFile(const string& src_file,
             need_restore_attr = true;
         }
     } else {
-        if ( !cf.Open(src_file, CCompressionFile::eMode_Read, 0) ) {
+        if ( !cf.Open(src_file, CCompressionFile::eMode_Read, NULL,
+                      decompression_in_bufsize, decompression_out_bufsize) ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
             return false;
         } 
     }
     // Decompress file
-    if ( !CCompression::x_DecompressFile(cf, dst_file, buf_size) ) {
+    if ( !CCompression::x_DecompressFile(cf, dst_file, file_io_bufsize) ) {
         if ( cf.GetErrorCode() ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
         }
         cf.Close();
         return false;
     }
-    // Close output file and return result
+    // Close output file
     bool status = cf.Close();
     SetError(cf.GetErrorCode(), cf.GetErrorDescription());
-    // Restore time stamp if needed
+
+    // Restore file timestamp if needed
     if ( status  &&  need_restore_attr ) {
         CFile(dst_file).SetTimeT(&info.mtime);
     }
@@ -628,10 +732,18 @@ bool CZipCompression::DecompressFile(const string& src_file,
 
 bool CZipCompression::DecompressFileIntoDir(const string& src_file,
                                             const string& dst_dir,
-                                            size_t        buf_size)
+                                            size_t        file_io_bufsize,
+                                            size_t        decompression_in_bufsize,
+                                            size_t        decompression_out_bufsize)
 {
-    CZipCompressionFile cf(GetLevel(), m_WindowBits, m_MemLevel, m_Strategy);
+    CZipCompressionFile cf(GetLevel());
     cf.SetFlags(cf.GetFlags() | GetFlags());
+    cf.SetWindowBits(GetWindowBits());
+    cf.SetMemoryLevel(GetMemoryLevel());
+    cf.SetStrategy(GetStrategy());
+    if (m_Dict) {
+        cf.SetDictionary(*m_Dict, eNoOwnership);
+    }
 
     bool need_restore_attr = false;
     SFileInfo info;
@@ -640,7 +752,8 @@ bool CZipCompression::DecompressFileIntoDir(const string& src_file,
 
     // Open compressed file, and define name of the destination file
     if ( F_ISSET(fRestoreFileAttr) ) {
-        if ( !cf.Open(src_file, CCompressionFile::eMode_Read, &info) ) {
+        if ( !cf.Open(src_file, CCompressionFile::eMode_Read, &info,
+                      decompression_in_bufsize, decompression_out_bufsize) ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
             return false;
         } 
@@ -653,7 +766,8 @@ bool CZipCompression::DecompressFileIntoDir(const string& src_file,
             dst_file = CFile::MakePath(dst_dir, info.name);
         }
     } else {
-        if ( !cf.Open(src_file, CCompressionFile::eMode_Read, 0) ) {
+        if ( !cf.Open(src_file, CCompressionFile::eMode_Read, NULL,
+                      decompression_in_bufsize, decompression_out_bufsize) ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
             return false;
         } 
@@ -661,17 +775,18 @@ bool CZipCompression::DecompressFileIntoDir(const string& src_file,
         dst_file = CFile::MakePath(dst_dir, name);
     }
     // Decompress file
-    if ( !CCompression::x_DecompressFile(cf, dst_file, buf_size) ) {
+    if ( !CCompression::x_DecompressFile(cf, dst_file, file_io_bufsize) ) {
         if ( cf.GetErrorCode() ) {
             SetError(cf.GetErrorCode(), cf.GetErrorDescription());
         }
         cf.Close();
         return false;
     }
-    // Close output file and return result
+    // Close output file
     bool status = cf.Close();
     SetError(cf.GetErrorCode(), cf.GetErrorDescription());
-    // Restore time stamp if needed
+
+    // Restore timestamp if needed
     if ( status  &&  need_restore_attr ) {
         CFile(dst_file).SetTimeT(&info.mtime);
     }
@@ -679,6 +794,18 @@ bool CZipCompression::DecompressFileIntoDir(const string& src_file,
 }
 
 
+bool CZipCompression::SetDictionary(CCompressionDictionary& dict, ENcbiOwnership own)
+{
+    if (m_Dict && (m_DictOwn == eTakeOwnership)) {
+        delete m_Dict;
+    }
+    m_Dict = &dict;
+    m_DictOwn = own;
+
+    return true;
+}
+
+
 string CZipCompression::FormatErrorMessage(string where, size_t pos) const
 {
     string str = "[" + where + "]  " + GetErrorDescription();
@@ -691,17 +818,48 @@ string CZipCompression::FormatErrorMessage(string where, size_t pos) const
 }
 
 
+//----------------------------------------------------------------------------
+// 
+// Advanced parameters
+//
+
+int CZipCompression::GetStrategyDefault(void)    { return Z_DEFAULT_STRATEGY; };
+int CZipCompression::GetStrategyMin(void)        { return Z_DEFAULT_STRATEGY; /* 0 */ };
+
+int CZipCompression::GetStrategyMax(void)
+{
+    #if defined(Z_FIXED)        // from v1.2.2.2
+        return Z_FIXED;
+    #endif
+    #if defined(Z_RLE)          // from v1.2.5.2
+        return Z_RLE;
+    #endif
+    #if defined(Z_HUFFMAN_ONLY) // from v0.5
+        return Z_HUFFMAN_ONLY;
+    #endif
+    #if defined(Z_FILTERED)     // from v0.5
+        return Z_FILTERED;
+    #endif
+    return Z_DEFAULT_STRATEGY; 
+};
+
+int CZipCompression::GetWindowBitsDefault(void)  { return MAX_WBITS; };
+int CZipCompression::GetWindowBitsMin(void)      { return DEF_WBITS; };
+int CZipCompression::GetWindowBitsMax(void)      { return MAX_WBITS; };
+
+int CZipCompression::GetMemoryLevelDefault(void) { return DEF_MEM_LEVEL; };
+int CZipCompression::GetMemoryLevelMin(void)     { return 1; };
+int CZipCompression::GetMemoryLevelMax(void)     { return MAX_MEM_LEVEL; };
+
+
 
 //////////////////////////////////////////////////////////////////////////////
 //
 // CZipCompressionFile
 //
 
-
-CZipCompressionFile::CZipCompressionFile(
-    const string& file_name, EMode mode,
-    ELevel level, int window_bits, int mem_level, int strategy)
-    : CZipCompression(level, window_bits, mem_level, strategy),
+CZipCompressionFile::CZipCompressionFile(const string& file_name, EMode mode, ELevel level)
+    : CZipCompression(level),
       m_Mode(eMode_Read), m_File(0), m_Stream(0)
 {
     // For backward compatibility -- use gzip file format by default
@@ -716,10 +874,8 @@ CZipCompressionFile::CZipCompressionFile(
     return;
 }
 
-
-CZipCompressionFile::CZipCompressionFile(
-    ELevel level, int window_bits, int mem_level, int strategy)
-    : CZipCompression(level, window_bits, mem_level, strategy),
+CZipCompressionFile::CZipCompressionFile(ELevel level)
+    : CZipCompression(level),
       m_Mode(eMode_Read), m_File(0), m_Stream(0)
 {
     // For backward compatibility -- use gzip file format by default
@@ -727,6 +883,47 @@ CZipCompressionFile::CZipCompressionFile(
     return;
 }
 
+// @deprecated
+CZipCompressionFile::CZipCompressionFile(
+    const string& file_name, EMode mode,
+    ELevel level, int window_bits, int mem_level, int strategy)
+    : CZipCompression(level),
+      m_Mode(eMode_Read), m_File(0), m_Stream(0)
+{
+    // For backward compatibility -- use gzip file format by default
+    SetFlags(GetFlags() | fGZip);
+
+    // Set advanced compression parameters
+    SetWindowBits  (( window_bits == kZlibDefaultWbits )  ? GetWindowBitsDefault()  : window_bits);
+    SetMemoryLevel (( mem_level == kZlibDefaultMemLevel ) ? GetMemoryLevelDefault() : mem_level);
+    SetStrategy    (( strategy == kZlibDefaultStrategy )  ? GetStrategyDefault()    : strategy);
+
+    if ( !Open(file_name, mode) ) {
+        const string smode = (mode == eMode_Read) ? "reading" : "writing";
+        NCBI_THROW(CCompressionException, eCompressionFile, 
+                   "[CZipCompressionFile]  Cannot open file '" + file_name +
+                   "' for " + smode + ".");
+    }
+    return;
+}
+
+// @deprecated
+CZipCompressionFile::CZipCompressionFile(
+    ELevel level, int window_bits, int mem_level, int strategy)
+    : CZipCompression(level),
+      m_Mode(eMode_Read), m_File(0), m_Stream(0)
+{
+    // For backward compatibility -- use gzip file format by default
+    SetFlags(GetFlags() | fGZip);
+
+    // Set advanced compression parameters
+    SetWindowBits  (( window_bits == kZlibDefaultWbits )  ? GetWindowBitsDefault()  : window_bits);
+    SetMemoryLevel (( mem_level == kZlibDefaultMemLevel ) ? GetMemoryLevelDefault() : mem_level);
+    SetStrategy    (( strategy == kZlibDefaultStrategy )  ? GetStrategyDefault()    : strategy);
+
+    return;
+}
+
 
 CZipCompressionFile::~CZipCompressionFile(void)
 {
@@ -748,13 +945,15 @@ void CZipCompressionFile::GetStreamError(void)
 }
 
 
-bool CZipCompressionFile::Open(const string& file_name, EMode mode)
+bool CZipCompressionFile::Open(const string& file_name, EMode mode,
+                               size_t compression_in_bufsize, size_t compression_out_bufsize)
 {
-    return Open(file_name, mode, 0 /*info*/);
+    return Open(file_name, mode, NULL /*info*/, compression_in_bufsize, compression_out_bufsize);
 }
 
 
-bool CZipCompressionFile::Open(const string& file_name, EMode mode, SFileInfo* info)
+bool CZipCompressionFile::Open(const string& file_name, EMode mode, SFileInfo* info,
+                               size_t compression_in_bufsize, size_t compression_out_bufsize)
 {
     m_Mode = mode;
 
@@ -788,19 +987,33 @@ bool CZipCompressionFile::Open(const string& file_name, EMode mode, SFileInfo* i
 
     // Create compression stream for I/O
     if ( mode == eMode_Read ) {
-        CZipDecompressor* decompressor =  new CZipDecompressor(m_WindowBits, GetFlags());
+        CZipDecompressor* decompressor = new CZipDecompressor(GetFlags());
+
+        decompressor->SetWindowBits(GetWindowBits());
+        decompressor->SetMemoryLevel(GetMemoryLevel());
+        decompressor->SetStrategy(GetStrategy());
+        if (m_Dict) {
+            decompressor->SetDictionary(*m_Dict, eNoOwnership);
+        }
         CCompressionStreamProcessor* processor = 
             new CCompressionStreamProcessor(
                 decompressor, CCompressionStreamProcessor::eDelete,
                 kCompressionDefaultBufSize, kCompressionDefaultBufSize);
         m_Stream = new CCompressionIOStream(*m_File, processor, 0, CCompressionStream::fOwnReader);
+
     } else {
-        CZipCompressor* compressor = new CZipCompressor(GetLevel(), m_WindowBits, m_MemLevel, m_Strategy, GetFlags());
+        CZipCompressor* compressor = new CZipCompressor(GetLevel(), GetFlags());
         if ( F_ISSET(fWriteGZipFormat)  &&  info) {
             // Enable compressor to write info information about
             // compressed file into gzip file header
             compressor->SetFileInfo(*info);
         }
+        compressor->SetWindowBits(GetWindowBits());
+        compressor->SetMemoryLevel(GetMemoryLevel());
+        compressor->SetStrategy(GetStrategy());
+        if (m_Dict) {
+            compressor->SetDictionary(*m_Dict, eNoOwnership);
+        }
         CCompressionStreamProcessor* processor = 
             new CCompressionStreamProcessor(
                 compressor, CCompressionStreamProcessor::eDelete,
@@ -898,14 +1111,26 @@ bool CZipCompressionFile::Close(void)
 //
 
 
-CZipCompressor::CZipCompressor(ELevel level,  int window_bits,
-                               int mem_level, int strategy, TZipFlags flags)
-    : CZipCompression(level, window_bits, mem_level, strategy),
+CZipCompressor::CZipCompressor(ELevel level, TZipFlags flags)
+    : CZipCompression(level),
       m_CRC32(0), m_NeedWriteHeader(true)
 {
     SetFlags(flags);
 }
 
+// @deprecated
+CZipCompressor::CZipCompressor(ELevel level,  int window_bits,
+                               int mem_level, int strategy, TZipFlags flags)
+    : CZipCompression(level),
+      m_CRC32(0), m_NeedWriteHeader(true)
+{
+    SetFlags(flags);
+    // Set advanced compression parameters
+    SetWindowBits  (( window_bits == kZlibDefaultWbits )  ? GetWindowBitsDefault()  : window_bits);
+    SetMemoryLevel (( mem_level == kZlibDefaultMemLevel ) ? GetMemoryLevelDefault() : mem_level);
+    SetStrategy    (( strategy == kZlibDefaultStrategy )  ? GetStrategyDefault()    : strategy);
+}
+
 
 CZipCompressor::~CZipCompressor()
 {
@@ -936,10 +1161,20 @@ CCompressionProcessor::EStatus CZipCompressor::Init(void)
     memset(STREAM, 0, sizeof(z_stream));
     // Create a compressor stream
     int errcode = deflateInit2_(STREAM, GetLevel(), Z_DEFLATED,
-                                F_ISSET(fWriteGZipFormat) ? -m_WindowBits : m_WindowBits,
-                                m_MemLevel, m_Strategy,
+                                F_ISSET(fWriteGZipFormat) ? -GetWindowBits() : GetWindowBits(),
+                                GetMemoryLevel(), GetStrategy(),
                                 ZLIB_VERSION, (int)sizeof(z_stream));
-    SetError(errcode, zError(errcode));
+    if (errcode == Z_OK) {
+        // Set dictionary
+        if (m_Dict) {
+            errcode = deflateSetDictionary(STREAM, (const Bytef*) m_Dict->GetData(), (uInt) m_Dict->GetSize());
+            if (errcode != Z_OK) {
+                SetError(Z_STREAM_ERROR, "cannot set a dictionary");
+            }
+        }
+    } else {
+        SetError(errcode, zError(errcode));
+    }
     if ( errcode == Z_OK ) {
         return eStatus_Success;
     }
@@ -1094,7 +1329,6 @@ CCompressionProcessor::EStatus CZipCompressor::Finish(
             IncreaseOutputSize(footer_len);
             *out_avail += footer_len;
         }
-
         return eStatus_EndOfData;
     }
     ERR_COMPRESS(66, FormatErrorMessage("CZipCompressor::Finish", GetProcessedSize()));
@@ -1126,14 +1360,25 @@ CCompressionProcessor::EStatus CZipCompressor::End(int abandon)
 //
 
 
-CZipDecompressor::CZipDecompressor(int window_bits, TZipFlags flags)
-    : CZipCompression(eLevel_Default, window_bits, 0, 0),
+CZipDecompressor::CZipDecompressor(TZipFlags flags)
+    : CZipCompression(eLevel_Default),
       m_NeedCheckHeader(true), m_IsGZ(false), m_SkipInput(0)
 {
     SetFlags(flags);
 }
 
 
+// @deprecated
+CZipDecompressor::CZipDecompressor(int window_bits, TZipFlags flags)
+    : CZipCompression(eLevel_Default),
+      m_NeedCheckHeader(true), m_IsGZ(false), m_SkipInput(0)
+{
+    SetFlags(flags);
+    // Set advanced compression parameters
+    SetWindowBits(( window_bits == kZlibDefaultWbits )  ? GetWindowBitsDefault()  : window_bits);
+}
+
+
 CZipDecompressor::~CZipDecompressor()
 {
     if ( IsBusy() ) {
@@ -1158,7 +1403,7 @@ CCompressionProcessor::EStatus CZipDecompressor::Init(void)
     memset(STREAM, 0, sizeof(z_stream));
     
     // Create a compressor stream
-    int errcode = inflateInit2_(STREAM, m_WindowBits, ZLIB_VERSION, (int)sizeof(z_stream));
+    int errcode = inflateInit2_(STREAM, GetWindowBits(), ZLIB_VERSION, (int)sizeof(z_stream));
     SetError(errcode, zError(errcode));
     if ( errcode == Z_OK ) {
         return eStatus_Success;
@@ -1262,9 +1507,24 @@ CCompressionProcessor::EStatus CZipDecompressor::Process(
                 // Reinit decompression stream
                 inflateEnd(STREAM);
                 int errcode = inflateInit2_(STREAM,
-                                            m_IsGZ ? -m_WindowBits : m_WindowBits,
+                                            m_IsGZ ? -GetWindowBits() : GetWindowBits(),
                                             ZLIB_VERSION,
                                             (int)sizeof(z_stream));
+                if ( errcode != Z_OK ) {
+                    SetError(errcode, zError(errcode));
+                    return eStatus_Error;
+                }
+                // Setting dictionary for decompressing is inconsistent for zlib...
+                // For raw deflate we should set it immediately after inflateInit2(), or stream
+                // will produce a data error and never ask for a dictionary using Z_NEED_DICT.
+                // So, set it here forcely, and also later when decompressor ask for it directly.
+                if (header_len  &&  m_Dict) {
+                    errcode = inflateSetDictionary(STREAM, (const Bytef*) m_Dict->GetData(), (uInt) m_Dict->GetSize());
+                    if ( errcode != Z_OK ) {
+                        SetError(errcode, zError(errcode));
+                        return eStatus_Error;
+                    }
+                }
                 SetError(errcode, zError(errcode));
                 if ( errcode != Z_OK ) {
                     return eStatus_Error;
@@ -1293,6 +1553,15 @@ CCompressionProcessor::EStatus CZipDecompressor::Process(
         // Try to decompress data
         int errcode = inflate(STREAM, Z_SYNC_FLUSH);
 
+        // Set dictionary if asked, and continue
+        if (errcode == Z_NEED_DICT  &&  m_Dict) {
+            errcode = inflateSetDictionary(STREAM, (const Bytef*) m_Dict->GetData(), (uInt) m_Dict->GetSize());
+            if (errcode == Z_OK) {
+                // Repeat decompression
+                errcode = inflate(STREAM, Z_SYNC_FLUSH);
+            }
+        }
+
         if ( m_DecompressMode == eMode_Unknown ) {
             // The flag fAllowTransparentRead is set
             _ASSERT(F_ISSET(fAllowTransparentRead));
@@ -1441,6 +1710,51 @@ CCompressionProcessor::EStatus CZipDecompressor::End(int abandon)
 }
 
 
+
+//////////////////////////////////////////////////////////////////////////////
+//
+// CZipStreamCompressor /  CZipStreamDecompressor -- deprecated constructors
+//
+
+
+// @deprecated
+CZipStreamCompressor::CZipStreamCompressor(
+        CZipCompression::ELevel    level,
+        streamsize                 in_bufsize,
+        streamsize                 out_bufsize,
+        int                        window_bits,
+        int                        mem_level,
+        int                        strategy ,
+        CZipCompression::TZipFlags flags
+        ) 
+        : CCompressionStreamProcessor(
+              new CZipCompressor(level, flags), eDelete, in_bufsize, out_bufsize)
+{
+    // Set advanced parameters
+    CZipCompressor* c = GetCompressor();
+    c->SetWindowBits (( window_bits == kZlibDefaultWbits )  ? c->GetWindowBitsDefault()  : window_bits);
+    c->SetMemoryLevel(( mem_level == kZlibDefaultMemLevel ) ? c->GetMemoryLevelDefault() : mem_level);
+    c->SetStrategy   (( strategy == kZlibDefaultStrategy )  ? c->GetStrategyDefault()    : strategy);
+
+}
+
+// @deprecated
+CZipStreamDecompressor::CZipStreamDecompressor(
+    streamsize                 in_bufsize,
+    streamsize                 out_bufsize,
+    int                        window_bits,
+    CZipCompression::TZipFlags flags
+    )
+    : CCompressionStreamProcessor( 
+            new CZipDecompressor(flags), eDelete, in_bufsize, out_bufsize)
+{
+    // Set advanced parameters
+    CZipDecompressor* d = GetDecompressor();
+    d->SetWindowBits(( window_bits == kZlibDefaultWbits ) ? d->GetWindowBitsDefault() : window_bits);
+}
+
+
+
 //////////////////////////////////////////////////////////////////////////////
 //
 // Global functions
@@ -1457,7 +1771,6 @@ void g_GZip_ScanForChunks(CNcbiIstream& is, IChunkHandler& handler)
     int      ret = Z_STREAM_END;  // zlib return status, init with Z_STREAM_END to get a (0,0)
     bool     initialized = false;
 
-
     // Default buffer size
     size_t in_size  = kCompressionDefaultBufSize;
     size_t out_size = kCompressionDefaultBufSize * 2;
@@ -1500,7 +1813,7 @@ void g_GZip_ScanForChunks(CNcbiIstream& is, IChunkHandler& handler)
                     strm.zalloc = NULL;
                     strm.zfree  = NULL;
                     strm.opaque = NULL;
-                    ret = inflateInit2(&strm, 15+16 /* max windowbits + automatic gzip header decoding */); /* NCBI_FAKE_WARNING */
+                    ret = inflateInit2(&strm, MAX_WBITS+16 /* max windowbits + automatic gzip header decoding */); /* NCBI_FAKE_WARNING */
                     if (ret != Z_OK) {
                         throw "inflateInit2() failed: " + string(zError(ret));
                     }
